<ul class="nav nav-underline pb-4" hx-boost="true">
  {%
    set pages = [
      ('One', url_for('taskpane', page='one')),
      ('Two', url_for('taskpane', page='two')),
      ('Three', url_for('taskpane', page='three'))
    ]
  %}
  {% for name, url in pages %}
    <li class="nav-item">
      <a class="nav-link {% if page == name|lower %}active{% endif %}" aria-current="page" href="{{ url }}"
        >{{ name }}</a
      >
    </li>
  {% endfor %}
</ul>
