/**
 * These are styles required by core functionality of xlwings Server.
 * For your own CSS, use style.css instead.
*/

/* htmx IndicatorStyles */
.htmx-indicator {
  opacity: 0;
}
.htmx-request .htmx-indicator {
  opacity: 1;
  transition: opacity 200ms ease-in;
}
.htmx-request.htmx-indicator {
  opacity: 1;
  transition: opacity 200ms ease-in;
}

/* Alpine.js */
[x-cloak] {
  /* https://alpinejs.dev/directives/cloak */
  display: none !important;
}

/* Custom */
.z-index-1000 {
  z-index: 1000;
}
