{% extends "base.html" %}

{% block content %}
  <div class="container-fluid pt-3 ps-3">
    <h1>Protected Task Pane</h1>
    <p>Your are logged in as {{ current_user.name }}.</p>
  </div>
{% endblock content %}
