{% extends "base.html" %}

{% block content %}
  <div class="container-fluid ps-3">
    {% include "examples/navigation/_navigation.html" %}
    <h1>This is Page 2</h1>
  </div>
{% endblock content %}
