/*
 * Office JavaScript API library
 *
 * Copyright (c) Microsoft Corporation.  All rights reserved.
 *
 * Your use of this file is governed by the Microsoft Services Agreement http://go.microsoft.com/fwlink/?LinkId=266419.
 *
 * This file also contains the following Promise implementation (with a few small modifications):
 *      * @overview es6-promise - a tiny implementation of Promises/A+.
 *      * @copyright Copyright (c) 2014 Yehuda Katz, Tom Dale, Stefan Penner and contributors (Conversion to ES6 API by Jake Archibald)
 *      * @license   Licensed under MIT license
 *      *            See https://raw.githubusercontent.com/jakearchibald/es6-promise/master/LICENSE
 *      * @version   2.3.0
 */
"undefined"!=typeof OSFPerformance&&(OSFPerformance.hostInitializationStart=OSFPerformance.now())
/*
    Your use of this file is governed by the Microsoft Services Agreement http://go.microsoft.com/fwlink/?LinkId=266419.

    This file also contains the following Promise implementation (with a few small modifications):
        * @overview es6-promise - a tiny implementation of Promises/A+.
        * @copyright Copyright (c) 2014 Yehuda Katz, Tom Dale, Stefan Penner and contributors (Conversion to ES6 API by Jake Archibald)
        * @license   Licensed under MIT license
        *            See https://raw.githubusercontent.com/jakearchibald/es6-promise/master/LICENSE
        * @version   2.3.0
*/;var OfficeExt,__extends=this&&this.__extends||function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}();!function(e){var t=function(){function e(){}return e.prototype.isMsAjaxLoaded=function(){return!!("undefined"!=typeof Sys&&"undefined"!=typeof Type&&Sys.StringBuilder&&"function"==typeof Sys.StringBuilder&&Type.registerNamespace&&"function"==typeof Type.registerNamespace&&Type.registerClass&&"function"==typeof Type.registerClass&&"function"==typeof Function._validateParams&&Sys.Serialization&&Sys.Serialization.JavaScriptSerializer&&"function"==typeof Sys.Serialization.JavaScriptSerializer.serialize)},e.prototype.loadMsAjaxFull=function(e){var t=("https:"===window.location.protocol.toLowerCase()?"https:":"http:")+"//ajax.aspnetcdn.com/ajax/3.5/MicrosoftAjax.js";OSF.OUtil.loadScript(t,e)},Object.defineProperty(e.prototype,"msAjaxError",{get:function(){return null==this._msAjaxError&&this.isMsAjaxLoaded()&&(this._msAjaxError=Error),this._msAjaxError},set:function(e){this._msAjaxError=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"msAjaxString",{get:function(){return null==this._msAjaxString&&this.isMsAjaxLoaded()&&(this._msAjaxString=String),this._msAjaxString},set:function(e){this._msAjaxString=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"msAjaxDebug",{get:function(){return null==this._msAjaxDebug&&this.isMsAjaxLoaded()&&(this._msAjaxDebug=Sys.Debug),this._msAjaxDebug},set:function(e){this._msAjaxDebug=e},enumerable:!0,configurable:!0}),e}();e.MicrosoftAjaxFactory=t}(OfficeExt||(OfficeExt={}));var OsfMsAjaxFactory=new OfficeExt.MicrosoftAjaxFactory,OSF=OSF||{};!function(e){var t=function(){function e(e){this._internalStorage=e}return e.prototype.getItem=function(e){try{return this._internalStorage&&this._internalStorage.getItem(e)}catch(e){return null}},e.prototype.setItem=function(e,t){try{this._internalStorage&&this._internalStorage.setItem(e,t)}catch(e){}},e.prototype.clear=function(){try{this._internalStorage&&this._internalStorage.clear()}catch(e){}},e.prototype.removeItem=function(e){try{this._internalStorage&&this._internalStorage.removeItem(e)}catch(e){}},e.prototype.getKeysWithPrefix=function(e){var t=[];try{for(var n=this._internalStorage&&this._internalStorage.length||0,r=0;r<n;r++){var i=this._internalStorage.key(r);0===i.indexOf(e)&&t.push(i)}}catch(e){}return t},e.prototype.isLocalStorageAvailable=function(){return null!=this._internalStorage},e}();e.SafeStorage=t}(OfficeExt||(OfficeExt={})),OSF.XdmFieldName={ConversationUrl:"ConversationUrl",AppId:"AppId"},OSF.TestFlightStart=1e3,OSF.TestFlightEnd=1009,OSF.FlightNames={UseOriginNotUrl:0,AddinEnforceHttps:2,FirstPartyAnonymousProxyReadyCheckTimeout:6,AddinRibbonIdAllowUnknown:9,ManifestParserDevConsoleLog:15,AddinActionDefinitionHybridMode:18,UseActionIdForUILessCommand:20,RequirementSetRibbonApiOnePointTwo:21,SetFocusToTaskpaneIsEnabled:22,ShortcutInfoArrayInUserPreferenceData:23,OSFTestFlight1000:OSF.TestFlightStart,OSFTestFlight1001:OSF.TestFlightStart+1,OSFTestFlight1002:OSF.TestFlightStart+2,OSFTestFlight1003:OSF.TestFlightStart+3,OSFTestFlight1004:OSF.TestFlightStart+4,OSFTestFlight1005:OSF.TestFlightStart+5,OSFTestFlight1006:OSF.TestFlightStart+6,OSFTestFlight1007:OSF.TestFlightStart+7,OSFTestFlight1008:OSF.TestFlightStart+8,OSFTestFlight1009:OSF.TestFlightEnd},OSF.TrustUXFlightValues={TrustUXControlA:0,TrustUXExperimentB:1,TrustUXExperimentC:2},OSF.FlightTreatmentNames={AddinDialogIFrameContentWindowKillSwitch:"Microsoft.Office.SharedOnline.AddinDialogIFrameContentWindowKillSwitch",AddinTrustUXImprovement:"Microsoft.Office.SharedOnline.AddinTrustUXImprovement",AllowStorageAccessByUserActivationOnIFrameCheck:"Microsoft.Office.SharedOnline.AllowStorageAccessByUserActivationOnIFrameCheck",BlockAutoOpenAddInIfStoreDisabled:"Microsoft.Office.SharedOnline.BlockAutoOpenAddInIfStoreDisabled",CheckProxyIsReadyRetry:"Microsoft.Office.SharedOnline.OEP.CheckProxyIsReadyRetry",InsertionDialogFixesEnabled:"Microsoft.Office.SharedOnline.InsertionDialogFixesEnabled",TeachingUIForPrivateCatelogEnabled:"Microsoft.Office.SharedOnline.TeachingUIForPrivateCatelogEnabled",WopiPreinstalledAddInsEnabled:"Microsoft.Office.SharedOnline.WopiPreinstalledAddInsEnabled",WopiUseNewActivate:"Microsoft.Office.SharedOnline.WopiUseNewActivate",MosManifestEnabled:"Microsoft.Office.SharedOnline.OEP.MosManifest"},OSF.Flights=[],OSF.IntFlights={},OSF.Settings={},OSF.WindowNameItemKeys={BaseFrameName:"baseFrameName",HostInfo:"hostInfo",XdmInfo:"xdmInfo",SerializerVersion:"serializerVersion",AppContext:"appContext",Flights:"flights"},OSF.OUtil=function(){var e=-1,t="&_xdm_Info=",n="&_serializer_version=",r="&_flights=",i="_xdm_",o="#",s="class",a={},l=null,u=null,c=(new Date).getTime();function p(){var e=2147483647*Math.random();return(e^=c^(new Date).getMilliseconds()<<Math.floor(21*Math.random())).toString(16)}function f(){if(!l){try{var e=window.sessionStorage}catch(t){e=null}l=new OfficeExt.SafeStorage(e)}return l}function h(e){var t,n,r=[],i=[],o=e.length;for(t=0;t<o;t++)(n=e[t]).tabIndex?n.tabIndex>0?i.push(n):0===n.tabIndex&&r.push(n):r.push(n);return i=i.sort((function(e,t){var n=e.tabIndex-t.tabIndex;return 0===n&&(n=i.indexOf(e)-i.indexOf(t)),n})),[].concat(i,r)}return{set_entropy:function(e){if("string"==typeof e)for(var t=0;t<e.length;t+=4){for(var n=0,r=0;r<4&&t+r<e.length;r++)n=(n<<8)+e.charCodeAt(t+r);c^=n}else c^="number"==typeof e?e:2147483647*Math.random();c&=2147483647},extend:function(e,t){var n=function(){};n.prototype=t.prototype,e.prototype=new n,e.prototype.constructor=e,e.uber=t.prototype,t.prototype.constructor===Object.prototype.constructor&&(t.prototype.constructor=t)},setNamespace:function(e,t){t&&e&&!t[e]&&(t[e]={})},unsetNamespace:function(e,t){t&&e&&t[e]&&delete t[e]},serializeSettings:function(e){var t={};for(var n in e){var r=e[n];try{r=JSON?JSON.stringify(r,(function(e,t){return OSF.OUtil.isDate(this[e])?OSF.DDA.SettingsManager.DateJSONPrefix+this[e].getTime()+OSF.DDA.SettingsManager.DataJSONSuffix:t})):Sys.Serialization.JavaScriptSerializer.serialize(r),t[n]=r}catch(e){}}return t},deserializeSettings:function(e){var t={};for(var n in e=e||{}){var r=e[n];try{r=JSON?JSON.parse(r,(function(e,t){var n;return"string"==typeof t&&t&&t.length>6&&t.slice(0,5)===OSF.DDA.SettingsManager.DateJSONPrefix&&t.slice(-1)===OSF.DDA.SettingsManager.DataJSONSuffix&&(n=new Date(parseInt(t.slice(5,-1))))?n:t})):Sys.Serialization.JavaScriptSerializer.deserialize(r,!0),t[n]=r}catch(e){}}return t},loadScript:function(e,t,n){if(e&&t){var r=window.document,i=a[e];if(i)i.loaded?t():i.pendingCallbacks.push(t);else{var o=r.createElement("script");o.type="text/javascript",i={loaded:!1,pendingCallbacks:[t],timer:null},a[e]=i;var s=function(){null!=i.timer&&(clearTimeout(i.timer),delete i.timer),i.loaded=!0;for(var e=i.pendingCallbacks.length,t=0;t<e;t++){i.pendingCallbacks.shift()()}},l=function(t){delete a[e],null!=i.timer&&(clearTimeout(i.timer),delete i.timer);for(var n=i.pendingCallbacks.length,r=0;r<n;r++){i.pendingCallbacks.shift()(t)}};o.readyState?o.onreadystatechange=function(){"loaded"!=o.readyState&&"complete"!=o.readyState||(o.onreadystatechange=null,s())}:o.onload=s,o.onerror=l,n=n||3e4,i.timer=setTimeout((function(){window.navigator.userAgent.indexOf("Trident")>0?l(null):l(new Event("Script load timed out"))}),n),o.setAttribute("crossOrigin","anonymous"),o.src=e,r.getElementsByTagName("head")[0].appendChild(o)}}},loadCSS:function(e){if(e){var t=window.document,n=t.createElement("link");n.type="text/css",n.rel="stylesheet",n.href=e,t.getElementsByTagName("head")[0].appendChild(n)}},parseEnum:function(e,t){var n=t[e.trim()];if(void 0===n)throw OsfMsAjaxFactory.msAjaxDebug.trace("invalid enumeration string:"+e),OsfMsAjaxFactory.msAjaxError.argument("str");return n},delayExecutionAndCache:function(){var e={calc:arguments[0]};return function(){return e.calc&&(e.val=e.calc.apply(this,arguments),delete e.calc),e.val}},getUniqueId:function(){return(e+=1).toString()},formatString:function(){var e=arguments,t=e[0];return t.replace(/{(\d+)}/gm,(function(t,n){var r=parseInt(n,10)+1;return void 0===e[r]?"{"+n+"}":e[r]}))},generateConversationId:function(){return[p(),p(),(new Date).getTime().toString()].join("_")},getFrameName:function(e){return i+e+this.generateConversationId()},addXdmInfoAsHash:function(e,n){return OSF.OUtil.addInfoAsHash(e,t,n,!1)},addSerializerVersionAsHash:function(e,t){return OSF.OUtil.addInfoAsHash(e,n,t,!0)},addFlightsAsHash:function(e,t){return OSF.OUtil.addInfoAsHash(e,r,t,!0)},addInfoAsHash:function(e,t,n,r){var i,s=(e=e.trim()||"").split(o),a=s.shift(),l=s.join(o);return i=r?[t,encodeURIComponent(n),l].join(""):[l,t,n].join(""),[a,o,i].join("")},parseHostInfoFromWindowName:function(e,t){return OSF.OUtil.parseInfoFromWindowName(e,t,OSF.WindowNameItemKeys.HostInfo)},parseXdmInfo:function(e){var t=OSF.OUtil.parseXdmInfoWithGivenFragment(e,window.location.hash);return t||(t=OSF.OUtil.parseXdmInfoFromWindowName(e,window.name)),t},parseXdmInfoFromWindowName:function(e,t){return OSF.OUtil.parseInfoFromWindowName(e,t,OSF.WindowNameItemKeys.XdmInfo)},parseXdmInfoWithGivenFragment:function(e,n){return OSF.OUtil.parseInfoWithGivenFragment(t,i,!1,e,n)},parseSerializerVersion:function(e){var t=OSF.OUtil.parseSerializerVersionWithGivenFragment(e,window.location.hash);return isNaN(t)&&(t=OSF.OUtil.parseSerializerVersionFromWindowName(e,window.name)),t},parseSerializerVersionFromWindowName:function(e,t){return parseInt(OSF.OUtil.parseInfoFromWindowName(e,t,OSF.WindowNameItemKeys.SerializerVersion))},parseSerializerVersionWithGivenFragment:function(e,t){return parseInt(OSF.OUtil.parseInfoWithGivenFragment(n,"_serializer_version=",!0,e,t))},parseFlights:function(e){var t=OSF.OUtil.parseFlightsWithGivenFragment(e,window.location.hash);return 0==t.length&&(t=OSF.OUtil.parseFlightsFromWindowName(e,window.name)),t},checkFlight:function(e){return OSF.Flights&&OSF.Flights.indexOf(e)>=0},pushFlight:function(e){return OSF.Flights.indexOf(e)<0&&(OSF.Flights.push(e),!0)},getBooleanSetting:function(e){return OSF.OUtil.getBooleanFromDictionary(OSF.Settings,e)},getBooleanFromDictionary:function(e,t){var n=e&&t&&void 0!==e[t]&&e[t]&&("string"==typeof e[t]&&"TRUE"===e[t].toUpperCase()||"boolean"==typeof e[t]&&e[t]);return void 0!==n&&n},getIntFromDictionary:function(e,t){return e&&t&&void 0!==e[t]&&"string"==typeof e[t]?parseInt(e[t]):NaN},pushIntFlight:function(e,t){return!(e in OSF.IntFlights)&&(OSF.IntFlights[e]=t,!0)},getIntFlight:function(e){return OSF.IntFlights&&e in OSF.IntFlights?OSF.IntFlights[e]:NaN},parseFlightsFromWindowName:function(e,t){return OSF.OUtil.parseArrayWithDefault(OSF.OUtil.parseInfoFromWindowName(e,t,OSF.WindowNameItemKeys.Flights))},parseFlightsWithGivenFragment:function(e,t){return OSF.OUtil.parseArrayWithDefault(OSF.OUtil.parseInfoWithGivenFragment(r,"_flights=",!0,e,t))},parseArrayWithDefault:function(e){var t=[];try{t=JSON.parse(e)}catch(e){}return Array.isArray(t)||(t=[]),t},parseInfoFromWindowName:function(e,t,n){try{var r=JSON.parse(t),i=null!=r?r[n]:null,o=f();if(!e&&o&&null!=r){var s=r[OSF.WindowNameItemKeys.BaseFrameName]+n;i?o.setItem(s,i):i=o.getItem(s)}return i}catch(e){return null}},parseInfoWithGivenFragment:function(e,t,n,r,i){var o=i.split(e),s=o.length>1?o[o.length-1]:null;n&&null!=s&&(s.indexOf("&")>=0&&(s=s.split("&")[0]),s=decodeURIComponent(s));var a=f();if(!r&&a){var l=window.name.indexOf(t);if(l>-1){var u=window.name.indexOf(";",l);-1==u&&(u=window.name.length);var c=window.name.substring(l,u);s?a.setItem(c,s):s=a.getItem(c)}}return s},getConversationId:function(){var e=window.location.search,t=null;if(e){var n=e.indexOf("&");(t=n>0?e.substring(1,n):e.substr(1))&&"="===t.charAt(t.length-1)&&(t=t.substring(0,t.length-1))&&(t=decodeURIComponent(t))}return t},getInfoItems:function(e){var t=e.split("$");return void 0===t[1]&&(t=e.split("|")),void 0===t[1]&&(t=e.split("%7C")),t},getXdmFieldValue:function(e,t){var n="",r=OSF.OUtil.parseXdmInfo(t);if(r){var i=OSF.OUtil.getInfoItems(r);if(null!=i&&i.length>=3)switch(e){case OSF.XdmFieldName.ConversationUrl:n=i[2];break;case OSF.XdmFieldName.AppId:n=i[1]}}return n},validateParamObject:function(e,t,n){var r=Function._validateParams(arguments,[{name:"params",type:Object,mayBeNull:!1},{name:"expectedProperties",type:Object,mayBeNull:!1},{name:"callback",type:Function,mayBeNull:!0}]);if(r)throw r;for(var i in t)if(r=Function._validateParameter(e[i],t[i],i))throw r},writeProfilerMark:function(e){window.msWriteProfilerMark&&(window.msWriteProfilerMark(e),OsfMsAjaxFactory.msAjaxDebug.trace(e))},outputDebug:function(e){void 0!==OsfMsAjaxFactory&&OsfMsAjaxFactory.msAjaxDebug&&OsfMsAjaxFactory.msAjaxDebug.trace&&OsfMsAjaxFactory.msAjaxDebug.trace(e)},defineNondefaultProperty:function(e,t,n,r){for(var i in n=n||{},r){var o=r[i];null==n[o]&&(n[o]=!0)}return Object.defineProperty(e,t,n),e},defineNondefaultProperties:function(e,t,n){for(var r in t=t||{})OSF.OUtil.defineNondefaultProperty(e,r,t[r],n);return e},defineEnumerableProperty:function(e,t,n){return OSF.OUtil.defineNondefaultProperty(e,t,n,["enumerable"])},defineEnumerableProperties:function(e,t){return OSF.OUtil.defineNondefaultProperties(e,t,["enumerable"])},defineMutableProperty:function(e,t,n){return OSF.OUtil.defineNondefaultProperty(e,t,n,["writable","enumerable","configurable"])},defineMutableProperties:function(e,t){return OSF.OUtil.defineNondefaultProperties(e,t,["writable","enumerable","configurable"])},finalizeProperties:function(e,t){t=t||{};for(var n=Object.getOwnPropertyNames(e),r=n.length,i=0;i<r;i++){var o=n[i],s=Object.getOwnPropertyDescriptor(e,o);s.get||s.set||(s.writable=t.writable||!1),s.configurable=t.configurable||!1,s.enumerable=t.enumerable||!0,Object.defineProperty(e,o,s)}return e},mapList:function(e,t){var n=[];if(e)for(var r in e)n.push(t(e[r]));return n},listContainsKey:function(e,t){for(var n in e)if(t==n)return!0;return!1},listContainsValue:function(e,t){for(var n in e)if(t==e[n])return!0;return!1},augmentList:function(e,t){var n=e.push?function(t,n){e.push(n)}:function(t,n){e[t]=n};for(var r in t)n(r,t[r])},redefineList:function(e,t){for(var n in e)delete e[n];for(var r in t)e[r]=t[r]},isArray:function(e){return"[object Array]"===Object.prototype.toString.apply(e)},isFunction:function(e){return"[object Function]"===Object.prototype.toString.apply(e)},isDate:function(e){return"[object Date]"===Object.prototype.toString.apply(e)},addEventListener:function(e,t,n){e.addEventListener?e.addEventListener(t,n,!1):Sys.Browser.agent===Sys.Browser.InternetExplorer&&e.attachEvent?e.attachEvent("on"+t,n):e["on"+t]=n},removeEventListener:function(e,t,n){e.removeEventListener?e.removeEventListener(t,n,!1):Sys.Browser.agent===Sys.Browser.InternetExplorer&&e.detachEvent?e.detachEvent("on"+t,n):e["on"+t]=null},xhrGet:function(e,t,n){var r;try{(r=new XMLHttpRequest).onreadystatechange=function(){4==r.readyState&&(200==r.status?t(r.responseText):n(r.status))},r.open("GET",e,!0),r.send()}catch(e){n(e)}},encodeBase64:function(e){if(!e)return e;var t,n,r,i,o,s,a,l="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",u=[],c=[],p=0,f=e.length;do{t=e.charCodeAt(p++),n=e.charCodeAt(p++),r=e.charCodeAt(p++),a=0,i=255&t,o=t>>8,s=255&n,c[a++]=i>>2,c[a++]=(3&i)<<4|o>>4,c[a++]=(15&o)<<2|s>>6,c[a++]=63&s,isNaN(n)||(i=n>>8,o=255&r,s=r>>8,c[a++]=i>>2,c[a++]=(3&i)<<4|o>>4,c[a++]=(15&o)<<2|s>>6,c[a++]=63&s),isNaN(n)?c[a-1]=64:isNaN(r)&&(c[a-2]=64,c[a-1]=64);for(var h=0;h<a;h++)u.push(l.charAt(c[h]))}while(p<f);return u.join("")},getSessionStorage:function(){return f()},getLocalStorage:function(){if(!u){try{var e=window.localStorage}catch(t){e=null}u=new OfficeExt.SafeStorage(e)}return u},convertIntToCssHexColor:function(e){return"#"+(Number(e)+16777216).toString(16).slice(-6)},attachClickHandler:function(e,t){e.onclick=function(e){t()},e.ontouchend=function(e){t(),e.preventDefault()}},getQueryStringParamValue:function(e,t){var n=Function._validateParams(arguments,[{name:"queryString",type:String,mayBeNull:!1},{name:"paramName",type:String,mayBeNull:!1}]);if(n)return OsfMsAjaxFactory.msAjaxDebug.trace("OSF_Outil_getQueryStringParamValue: Parameters cannot be null."),"";var r=new RegExp("[\\?&]"+t+"=([^&#]*)","i");return r.test(e)?r.exec(e)[1]:(OsfMsAjaxFactory.msAjaxDebug.trace("OSF_Outil_getQueryStringParamValue: The parameter is not found."),"")},getHostnamePortionForLogging:function(e){var t=Function._validateParams(arguments,[{name:"hostname",type:String,mayBeNull:!1}]);if(t)return"";var n=e.split("."),r=n.length;return r>=2?n[r-2]+"."+n[r-1]:1==r?n[0]:void 0},isiOS:function(){return!!window.navigator.userAgent.match(/(iPad|iPhone|iPod)/g)},isChrome:function(){return window.navigator.userAgent.indexOf("Chrome")>0&&!OSF.OUtil.isEdge()},isEdge:function(){return window.navigator.userAgent.indexOf("Edge")>0},isIE:function(){return window.navigator.userAgent.indexOf("Trident")>0},isFirefox:function(){return window.navigator.userAgent.indexOf("Firefox")>0},startsWith:function(e,t,n){return n?e.substr(0,t.length)===t:e.startsWith(t)},containsPort:function(e,t,n,r){return this.startsWith(e,t+"//"+n+":"+r,!0)||this.startsWith(e,n+":"+r,!0)},getRedundandPortString:function(e,t){return e&&t?"https:"==t.protocol&&this.containsPort(e,"https:",t.hostname,"443")?":443":"http:"==t.protocol&&this.containsPort(e,"http:",t.hostname,"80")?":80":"":""},removeChar:function(e,t){return t<e.length-1?e.substring(0,t)+e.substring(t+1):t==e.length-1?e.substring(0,e.length-1):e},cleanUrlOfChar:function(e,t){var n;for(n=0;n<e.length;n++)if(e.charAt(n)===t){if(n+1>=e.length)return this.removeChar(e,n);if("/"===t){if("?"===e.charAt(n+1)||"#"===e.charAt(n+1))return this.removeChar(e,n)}else if("?"===t&&"#"===e.charAt(n+1))return this.removeChar(e,n)}return e},cleanUrl:function(e){if(e=this.cleanUrlOfChar(e,"/"),e=this.cleanUrlOfChar(e,"?"),"https://"==(e=this.cleanUrlOfChar(e,"#")).substr(0,8))-1!=(t=e.indexOf(":443"))&&(t!=e.length-4&&"/"!=e.charAt(t+4)&&"?"!=e.charAt(t+4)&&"#"!=e.charAt(t+4)||(e=e.substring(0,t)+e.substring(t+4)));else if("http://"==e.substr(0,7)){var t;-1!=(t=e.indexOf(":80"))&&(t!=e.length-3&&"/"!=e.charAt(t+3)&&"?"!=e.charAt(t+3)&&"#"!=e.charAt(t+3)||(e=e.substring(0,t)+e.substring(t+3)))}return e},parseUrl:function(e,t){if(void 0===t&&(t=!1),void 0!==e&&e){var n="NotHttps",r=this.isIE(),i={protocol:void 0,hostname:void 0,host:void 0,port:void 0,pathname:void 0,search:void 0,hash:void 0,isPortPartOfUrl:void 0};try{if(r){var o=document.createElement("a");if(o.href=e,!(o&&o.protocol&&o.host&&o.hostname&&o.href&&this.cleanUrl(o.href).toLowerCase()===this.cleanUrl(e).toLowerCase()))throw"InvalidUrl";if(OSF.OUtil.checkFlight(OSF.FlightNames.AddinEnforceHttps)&&t&&"https:"!=o.protocol)throw new Error(n);var s=this.getRedundandPortString(e,o);i.protocol=o.protocol,i.hostname=o.hostname,i.port=""==s?o.port:"",i.host=""!=s?o.hostname:o.host,i.pathname=(r?"/":"")+o.pathname,i.search=o.search,i.hash=o.hash,i.isPortPartOfUrl=this.containsPort(e,o.protocol,o.hostname,o.port)}else{var a=new URL(e);if(a&&a.protocol&&a.host&&a.hostname){if(OSF.OUtil.checkFlight(OSF.FlightNames.AddinEnforceHttps)&&t&&"https:"!=a.protocol)throw new Error(n);i.protocol=a.protocol,i.hostname=a.hostname,i.port=a.port,i.host=a.host,i.pathname=a.pathname,i.search=a.search,i.hash=a.hash,i.isPortPartOfUrl=a.host.lastIndexOf(":"+a.port)==a.host.length-a.port.length-1}}}catch(e){if(e.message===n)throw e}return i}},shallowCopy:function(e){if(null==e)return null;if(e instanceof Object){if(Array.isArray(e)){for(var t=[],n=0;n<e.length;n++)t.push(e[n]);return t}var r=e.constructor();for(var i in e)e.hasOwnProperty(i)&&(r[i]=e[i]);return r}return e},createObject:function(e){var t=null;if(e){t={};for(var n=e.length,r=0;r<n;r++)t[e[r].name]=e[r].value}return t},addClass:function(e,t){if(!OSF.OUtil.hasClass(e,t)){var n=e.getAttribute(s);n?e.setAttribute(s,n+" "+t):e.setAttribute(s,t)}},removeClass:function(e,t){if(OSF.OUtil.hasClass(e,t)){var n=e.getAttribute(s),r=new RegExp("(\\s|^)"+t+"(\\s|$)");n=n.replace(r,""),e.setAttribute(s,n)}},hasClass:function(e,t){var n=e.getAttribute(s);return n&&n.match(new RegExp("(\\s|^)"+t+"(\\s|$)"))},focusToFirstTabbable:function(e,t){var n,r,i=!1,o=function(e){i=!0},s=function(e,t,n){return t<0||t>e||0===t&&n?-1:t!==e-1||n?n?t-1:t+1:-1};if(e=h(e),n=t?e.length-1:0,0===e.length)return null;for(;!i&&n>=0&&n<e.length;)r=e[n],window.focus(),r.addEventListener("focus",o),r.focus(),r.removeEventListener("focus",o),n=s(e.length,n,t),i||r!==document.activeElement||(i=!0);return i?r:null},focusToNextTabbable:function(e,t,n){var r,i,o,s=!1,a=function(e){s=!0},l=function(e,t,n){return t<0||t>e||0===t&&n?-1:t!==e-1||n?n?t-1:t+1:-1};if(r=function(e,t){for(var n=0;n<e.length;n++)if(e[n]===t)return n;return-1}(e=h(e),t),(i=l(e.length,r,n))<0)return null;for(;!s&&i>=0&&i<e.length;)(o=e[i]).addEventListener("focus",a),o.focus(),o.removeEventListener("focus",a),i=l(e.length,i,n),s||o!==document.activeElement||(s=!0);return s?o:null},isNullOrUndefined:function(e){return void 0===e||null===e},stringEndsWith:function(e,t){if(!OSF.OUtil.isNullOrUndefined(e)&&!OSF.OUtil.isNullOrUndefined(t)){if(t.length>e.length)return!1;if(e.substr(e.length-t.length)===t)return!0}return!1},hashCode:function(e){var t=0;if(!OSF.OUtil.isNullOrUndefined(e))for(var n=0,r=e.length;n<r;)t=(t<<5)-t+e.charCodeAt(n++)|0;return t},getValue:function(e,t){return OSF.OUtil.isNullOrUndefined(e)?t:e},externalNativeFunctionExists:function(e){return"unknown"===e||"undefined"!==e}}}(),OSF.OUtil.Guid=function(){var e=["0","1","2","3","4","5","6","7","8","9","a","b","c","d","e","f"];return{generateNewGuid:function(){for(var t="",n=(new Date).getTime(),r=0;r<32&&n>0;r++)8!=r&&12!=r&&16!=r&&20!=r||(t+="-"),t+=e[n%16],n=Math.floor(n/16);for(;r<32;r++)8!=r&&12!=r&&16!=r&&20!=r||(t+="-"),t+=e[Math.floor(16*Math.random())];return t}}}();try{OSF.Flights=OSF.OUtil.parseFlights(!0)}catch(e){}window.OSF=OSF,OSF.OUtil.setNamespace("OSF",window),OSF.MessageIDs={FetchBundleUrl:0,LoadReactBundle:1,LoadBundleSuccess:2,LoadBundleError:3},OSF.AppName={Unsupported:0,Excel:1,Word:2,PowerPoint:4,Outlook:8,ExcelWebApp:16,WordWebApp:32,OutlookWebApp:64,Project:128,AccessWebApp:256,PowerpointWebApp:512,ExcelIOS:1024,Sway:2048,WordIOS:4096,PowerPointIOS:8192,Access:16384,Lync:32768,OutlookIOS:65536,OneNoteWebApp:131072,OneNote:262144,ExcelWinRT:524288,WordWinRT:1048576,PowerpointWinRT:2097152,OutlookAndroid:4194304,OneNoteWinRT:8388608,ExcelAndroid:8388609,VisioWebApp:8388610,OneNoteIOS:8388611,WordAndroid:8388613,PowerpointAndroid:8388614,Visio:8388615,OneNoteAndroid:4194305},OSF.InternalPerfMarker={DataCoercionBegin:"Agave.HostCall.CoerceDataStart",DataCoercionEnd:"Agave.HostCall.CoerceDataEnd"},OSF.HostCallPerfMarker={IssueCall:"Agave.HostCall.IssueCall",ReceiveResponse:"Agave.HostCall.ReceiveResponse",RuntimeExceptionRaised:"Agave.HostCall.RuntimeExecptionRaised"},OSF.AgaveHostAction={Select:0,UnSelect:1,CancelDialog:2,InsertAgave:3,CtrlF6In:4,CtrlF6Exit:5,CtrlF6ExitShift:6,SelectWithError:7,NotifyHostError:8,RefreshAddinCommands:9,PageIsReady:10,TabIn:11,TabInShift:12,TabExit:13,TabExitShift:14,EscExit:15,F2Exit:16,ExitNoFocusable:17,ExitNoFocusableShift:18,MouseEnter:19,MouseLeave:20,UpdateTargetUrl:21,InstallCustomFunctions:22,SendTelemetryEvent:23,UninstallCustomFunctions:24,SendMessage:25,LaunchExtensionComponent:26,StopExtensionComponent:27,RestartExtensionComponent:28,EnableTaskPaneHeaderButton:29,DisableTaskPaneHeaderButton:30,TaskPaneHeaderButtonClicked:31,RemoveAppCommandsAddin:32,RefreshRibbonGallery:33,GetOriginalControlId:34,OfficeJsReady:35,InsertDevManifest:36,InsertDevManifestError:37,SendCustomerContent:38,KeyboardShortcuts:39},OSF.SharedConstants={NotificationConversationIdSuffix:"_ntf"},OSF.DialogMessageType={DialogMessageReceived:0,DialogParentMessageReceived:1,DialogClosed:12006},OSF.OfficeAppContext=function(e,t,n,r,i,o,s,a,l,u,c,p,f,h,d,y,m,g,_,b,O,v,S,C,P,A,F){this._id=e,this._appName=t,this._appVersion=n,this._appUILocale=r,this._dataLocale=i,this._docUrl=o,this._clientMode=s,this._settings=a,this._reason=l,this._osfControlType=u,this._eToken=c,this._correlationId=p,this._appInstanceId=f,this._touchEnabled=h,this._commerceAllowed=d,this._appMinorVersion=y,this._requirementMatrix=m,this._hostCustomMessage=g,this._hostFullVersion=_,this._isDialog=!1,this._clientWindowHeight=b,this._clientWindowWidth=O,this._addinName=v,this._appDomains=S,this._dialogRequirementMatrix=C,this._featureGates=P,this._officeTheme=A,this._initialDisplayMode=F,this.get_id=function(){return this._id},this.get_appName=function(){return this._appName},this.get_appVersion=function(){return this._appVersion},this.get_appUILocale=function(){return this._appUILocale},this.get_dataLocale=function(){return this._dataLocale},this.get_docUrl=function(){return this._docUrl},this.get_clientMode=function(){return this._clientMode},this.get_bindings=function(){return this._bindings},this.get_settings=function(){return this._settings},this.get_reason=function(){return this._reason},this.get_osfControlType=function(){return this._osfControlType},this.get_eToken=function(){return this._eToken},this.get_correlationId=function(){return this._correlationId},this.get_appInstanceId=function(){return this._appInstanceId},this.get_touchEnabled=function(){return this._touchEnabled},this.get_commerceAllowed=function(){return this._commerceAllowed},this.get_appMinorVersion=function(){return this._appMinorVersion},this.get_requirementMatrix=function(){return this._requirementMatrix},this.get_dialogRequirementMatrix=function(){return this._dialogRequirementMatrix},this.get_hostCustomMessage=function(){return this._hostCustomMessage},this.get_hostFullVersion=function(){return this._hostFullVersion},this.get_isDialog=function(){return this._isDialog},this.get_clientWindowHeight=function(){return this._clientWindowHeight},this.get_clientWindowWidth=function(){return this._clientWindowWidth},this.get_addinName=function(){return this._addinName},this.get_appDomains=function(){return this._appDomains},this.get_featureGates=function(){return this._featureGates},this.get_officeTheme=function(){return this._officeTheme},this.get_initialDisplayMode=function(){return this._initialDisplayMode?this._initialDisplayMode:0}},OSF.OsfControlType={DocumentLevel:0,ContainerLevel:1},OSF.ClientMode={ReadOnly:0,ReadWrite:1},OSF.OUtil.setNamespace("Microsoft",window),OSF.OUtil.setNamespace("Office",Microsoft),OSF.OUtil.setNamespace("Client",Microsoft.Office),OSF.OUtil.setNamespace("WebExtension",Microsoft.Office),Microsoft.Office.WebExtension.InitializationReason={Inserted:"inserted",DocumentOpened:"documentOpened",ControlActivation:"controlActivation"},Microsoft.Office.WebExtension.ValueFormat={Unformatted:"unformatted",Formatted:"formatted"},Microsoft.Office.WebExtension.FilterType={All:"all"},Microsoft.Office.WebExtension.Parameters={BindingType:"bindingType",CoercionType:"coercionType",ValueFormat:"valueFormat",FilterType:"filterType",Columns:"columns",SampleData:"sampleData",GoToType:"goToType",SelectionMode:"selectionMode",Id:"id",PromptText:"promptText",ItemName:"itemName",FailOnCollision:"failOnCollision",StartRow:"startRow",StartColumn:"startColumn",RowCount:"rowCount",ColumnCount:"columnCount",Callback:"callback",AsyncContext:"asyncContext",Data:"data",Rows:"rows",OverwriteIfStale:"overwriteIfStale",FileType:"fileType",EventType:"eventType",Handler:"handler",SliceSize:"sliceSize",SliceIndex:"sliceIndex",ActiveView:"activeView",Status:"status",PlatformType:"platformType",HostType:"hostType",ForceConsent:"forceConsent",ForceAddAccount:"forceAddAccount",AuthChallenge:"authChallenge",AllowConsentPrompt:"allowConsentPrompt",ForMSGraphAccess:"forMSGraphAccess",AllowSignInPrompt:"allowSignInPrompt",JsonPayload:"jsonPayload",EnableNewHosts:"enableNewHosts",AccountTypeFilter:"accountTypeFilter",AddinTrustId:"addinTrustId",Reserved:"reserved",Tcid:"tcid",Xml:"xml",Namespace:"namespace",Prefix:"prefix",XPath:"xPath",Text:"text",ImageLeft:"imageLeft",ImageTop:"imageTop",ImageWidth:"imageWidth",ImageHeight:"imageHeight",TaskId:"taskId",FieldId:"fieldId",FieldValue:"fieldValue",ServerUrl:"serverUrl",ListName:"listName",ResourceId:"resourceId",ViewType:"viewType",ViewName:"viewName",GetRawValue:"getRawValue",CellFormat:"cellFormat",TableOptions:"tableOptions",TaskIndex:"taskIndex",ResourceIndex:"resourceIndex",CustomFieldId:"customFieldId",Url:"url",MessageHandler:"messageHandler",Width:"width",Height:"height",RequireHTTPs:"requireHTTPS",MessageToParent:"messageToParent",DisplayInIframe:"displayInIframe",MessageContent:"messageContent",HideTitle:"hideTitle",UseDeviceIndependentPixels:"useDeviceIndependentPixels",PromptBeforeOpen:"promptBeforeOpen",EnforceAppDomain:"enforceAppDomain",UrlNoHostInfo:"urlNoHostInfo",TargetOrigin:"targetOrigin",AppCommandInvocationCompletedData:"appCommandInvocationCompletedData",Base64:"base64",FormId:"formId"},OSF.OUtil.setNamespace("DDA",OSF),OSF.DDA.DocumentMode={ReadOnly:1,ReadWrite:0},OSF.DDA.PropertyDescriptors={AsyncResultStatus:"AsyncResultStatus"},OSF.DDA.EventDescriptors={},OSF.DDA.ListDescriptors={},OSF.DDA.UI={},OSF.DDA.getXdmEventName=function(e,t){return t==Microsoft.Office.WebExtension.EventType.BindingSelectionChanged||t==Microsoft.Office.WebExtension.EventType.BindingDataChanged||t==Microsoft.Office.WebExtension.EventType.DataNodeDeleted||t==Microsoft.Office.WebExtension.EventType.DataNodeInserted||t==Microsoft.Office.WebExtension.EventType.DataNodeReplaced?e+"_"+t:t},OSF.DDA.MethodDispId={dispidMethodMin:64,dispidGetSelectedDataMethod:64,dispidSetSelectedDataMethod:65,dispidAddBindingFromSelectionMethod:66,dispidAddBindingFromPromptMethod:67,dispidGetBindingMethod:68,dispidReleaseBindingMethod:69,dispidGetBindingDataMethod:70,dispidSetBindingDataMethod:71,dispidAddRowsMethod:72,dispidClearAllRowsMethod:73,dispidGetAllBindingsMethod:74,dispidLoadSettingsMethod:75,dispidSaveSettingsMethod:76,dispidGetDocumentCopyMethod:77,dispidAddBindingFromNamedItemMethod:78,dispidAddColumnsMethod:79,dispidGetDocumentCopyChunkMethod:80,dispidReleaseDocumentCopyMethod:81,dispidNavigateToMethod:82,dispidGetActiveViewMethod:83,dispidGetDocumentThemeMethod:84,dispidGetOfficeThemeMethod:85,dispidGetFilePropertiesMethod:86,dispidClearFormatsMethod:87,dispidSetTableOptionsMethod:88,dispidSetFormatsMethod:89,dispidExecuteRichApiRequestMethod:93,dispidAppCommandInvocationCompletedMethod:94,dispidCloseContainerMethod:97,dispidGetAccessTokenMethod:98,dispidGetAuthContextMethod:99,dispidOpenBrowserWindow:102,dispidCreateDocumentMethod:105,dispidInsertFormMethod:106,dispidDisplayRibbonCalloutAsyncMethod:109,dispidGetSelectedTaskMethod:110,dispidGetSelectedResourceMethod:111,dispidGetTaskMethod:112,dispidGetResourceFieldMethod:113,dispidGetWSSUrlMethod:114,dispidGetTaskFieldMethod:115,dispidGetProjectFieldMethod:116,dispidGetSelectedViewMethod:117,dispidGetTaskByIndexMethod:118,dispidGetResourceByIndexMethod:119,dispidSetTaskFieldMethod:120,dispidSetResourceFieldMethod:121,dispidGetMaxTaskIndexMethod:122,dispidGetMaxResourceIndexMethod:123,dispidCreateTaskMethod:124,dispidAddDataPartMethod:128,dispidGetDataPartByIdMethod:129,dispidGetDataPartsByNamespaceMethod:130,dispidGetDataPartXmlMethod:131,dispidGetDataPartNodesMethod:132,dispidDeleteDataPartMethod:133,dispidGetDataNodeValueMethod:134,dispidGetDataNodeXmlMethod:135,dispidGetDataNodesMethod:136,dispidSetDataNodeValueMethod:137,dispidSetDataNodeXmlMethod:138,dispidAddDataNamespaceMethod:139,dispidGetDataUriByPrefixMethod:140,dispidGetDataPrefixByUriMethod:141,dispidGetDataNodeTextMethod:142,dispidSetDataNodeTextMethod:143,dispidMessageParentMethod:144,dispidSendMessageMethod:145,dispidExecuteFeature:146,dispidQueryFeature:147,dispidMethodMax:147},OSF.DDA.EventDispId={dispidEventMin:0,dispidInitializeEvent:0,dispidSettingsChangedEvent:1,dispidDocumentSelectionChangedEvent:2,dispidBindingSelectionChangedEvent:3,dispidBindingDataChangedEvent:4,dispidDocumentOpenEvent:5,dispidDocumentCloseEvent:6,dispidActiveViewChangedEvent:7,dispidDocumentThemeChangedEvent:8,dispidOfficeThemeChangedEvent:9,dispidDialogMessageReceivedEvent:10,dispidDialogNotificationShownInAddinEvent:11,dispidDialogParentMessageReceivedEvent:12,dispidObjectDeletedEvent:13,dispidObjectSelectionChangedEvent:14,dispidObjectDataChangedEvent:15,dispidContentControlAddedEvent:16,dispidActivationStatusChangedEvent:32,dispidRichApiMessageEvent:33,dispidAppCommandInvokedEvent:39,dispidOlkItemSelectedChangedEvent:46,dispidOlkRecipientsChangedEvent:47,dispidOlkAppointmentTimeChangedEvent:48,dispidOlkRecurrenceChangedEvent:49,dispidOlkAttachmentsChangedEvent:50,dispidOlkEnhancedLocationsChangedEvent:51,dispidOlkInfobarClickedEvent:52,dispidTaskSelectionChangedEvent:56,dispidResourceSelectionChangedEvent:57,dispidViewSelectionChangedEvent:58,dispidDataNodeAddedEvent:60,dispidDataNodeReplacedEvent:61,dispidDataNodeDeletedEvent:62,dispidEventMax:63},OSF.DDA.ErrorCodeManager=function(){var e={};return{getErrorArgs:function(t){var n=e[t];return n?(n.name||(n.name=e[this.errorCodes.ooeInternalError].name),n.message||(n.message=e[this.errorCodes.ooeInternalError].message)):n=e[this.errorCodes.ooeInternalError],n},addErrorMessage:function(t,n){e[t]=n},errorCodes:{ooeSuccess:0,ooeChunkResult:1,ooeCoercionTypeNotSupported:1e3,ooeGetSelectionNotMatchDataType:1001,ooeCoercionTypeNotMatchBinding:1002,ooeInvalidGetRowColumnCounts:1003,ooeSelectionNotSupportCoercionType:1004,ooeInvalidGetStartRowColumn:1005,ooeNonUniformPartialGetNotSupported:1006,ooeGetDataIsTooLarge:1008,ooeFileTypeNotSupported:1009,ooeGetDataParametersConflict:1010,ooeInvalidGetColumns:1011,ooeInvalidGetRows:1012,ooeInvalidReadForBlankRow:1013,ooeUnsupportedDataObject:2e3,ooeCannotWriteToSelection:2001,ooeDataNotMatchSelection:2002,ooeOverwriteWorksheetData:2003,ooeDataNotMatchBindingSize:2004,ooeInvalidSetStartRowColumn:2005,ooeInvalidDataFormat:2006,ooeDataNotMatchCoercionType:2007,ooeDataNotMatchBindingType:2008,ooeSetDataIsTooLarge:2009,ooeNonUniformPartialSetNotSupported:2010,ooeInvalidSetColumns:2011,ooeInvalidSetRows:2012,ooeSetDataParametersConflict:2013,ooeCellDataAmountBeyondLimits:2014,ooeSelectionCannotBound:3e3,ooeBindingNotExist:3002,ooeBindingToMultipleSelection:3003,ooeInvalidSelectionForBindingType:3004,ooeOperationNotSupportedOnThisBindingType:3005,ooeNamedItemNotFound:3006,ooeMultipleNamedItemFound:3007,ooeInvalidNamedItemForBindingType:3008,ooeUnknownBindingType:3009,ooeOperationNotSupportedOnMatrixData:3010,ooeInvalidColumnsForBinding:3011,ooeSettingNameNotExist:4e3,ooeSettingsCannotSave:4001,ooeSettingsAreStale:4002,ooeOperationNotSupported:5e3,ooeInternalError:5001,ooeDocumentReadOnly:5002,ooeEventHandlerNotExist:5003,ooeInvalidApiCallInContext:5004,ooeShuttingDown:5005,ooeUnsupportedEnumeration:5007,ooeIndexOutOfRange:5008,ooeBrowserAPINotSupported:5009,ooeInvalidParam:5010,ooeRequestTimeout:5011,ooeInvalidOrTimedOutSession:5012,ooeInvalidApiArguments:5013,ooeOperationCancelled:5014,ooeWorkbookHidden:5015,ooeWriteNotSupportedWhenModalDialogOpen:5016,ooeTooManyIncompleteRequests:5100,ooeRequestTokenUnavailable:5101,ooeActivityLimitReached:5102,ooeRequestPayloadSizeLimitExceeded:5103,ooeResponsePayloadSizeLimitExceeded:5104,ooeCustomXmlNodeNotFound:6e3,ooeCustomXmlError:6100,ooeCustomXmlExceedQuota:6101,ooeCustomXmlOutOfDate:6102,ooeNoCapability:7e3,ooeCannotNavTo:7001,ooeSpecifiedIdNotExist:7002,ooeNavOutOfBound:7004,ooeElementMissing:8e3,ooeProtectedError:8001,ooeInvalidCellsValue:8010,ooeInvalidTableOptionValue:8011,ooeInvalidFormatValue:8012,ooeRowIndexOutOfRange:8020,ooeColIndexOutOfRange:8021,ooeFormatValueOutOfRange:8022,ooeCellFormatAmountBeyondLimits:8023,ooeMemoryFileLimit:11e3,ooeNetworkProblemRetrieveFile:11001,ooeInvalidSliceSize:11002,ooeInvalidCallback:11101,ooeInvalidWidth:12e3,ooeInvalidHeight:12001,ooeNavigationError:12002,ooeInvalidScheme:12003,ooeAppDomains:12004,ooeRequireHTTPS:12005,ooeWebDialogClosed:12006,ooeDialogAlreadyOpened:12007,ooeEndUserAllow:12008,ooeEndUserIgnore:12009,ooeNotUILessDialog:12010,ooeCrossZone:12011,ooeModalDialogOpen:12012,ooeDocumentIsInactive:12013,ooeDialogParentIsMinimized:12014,ooeNotSSOAgave:13e3,ooeSSOUserNotSignedIn:13001,ooeSSOUserAborted:13002,ooeSSOUnsupportedUserIdentity:13003,ooeSSOInvalidResourceUrl:13004,ooeSSOInvalidGrant:13005,ooeSSOClientError:13006,ooeSSOServerError:13007,ooeAddinIsAlreadyRequestingToken:13008,ooeSSOUserConsentNotSupportedByCurrentAddinCategory:13009,ooeSSOConnectionLost:13010,ooeResourceNotAllowed:13011,ooeSSOUnsupportedPlatform:13012,ooeSSOCallThrottled:13013,ooeAccessDenied:13990,ooeGeneralException:13991},initializeErrorMessages:function(t){e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCoercionTypeNotSupported]={name:t.L_InvalidCoercion,message:t.L_CoercionTypeNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeGetSelectionNotMatchDataType]={name:t.L_DataReadError,message:t.L_GetSelectionNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCoercionTypeNotMatchBinding]={name:t.L_InvalidCoercion,message:t.L_CoercionTypeNotMatchBinding},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidGetRowColumnCounts]={name:t.L_DataReadError,message:t.L_InvalidGetRowColumnCounts},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSelectionNotSupportCoercionType]={name:t.L_DataReadError,message:t.L_SelectionNotSupportCoercionType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidGetStartRowColumn]={name:t.L_DataReadError,message:t.L_InvalidGetStartRowColumn},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNonUniformPartialGetNotSupported]={name:t.L_DataReadError,message:t.L_NonUniformPartialGetNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeGetDataIsTooLarge]={name:t.L_DataReadError,message:t.L_GetDataIsTooLarge},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeFileTypeNotSupported]={name:t.L_DataReadError,message:t.L_FileTypeNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeGetDataParametersConflict]={name:t.L_DataReadError,message:t.L_GetDataParametersConflict},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidGetColumns]={name:t.L_DataReadError,message:t.L_InvalidGetColumns},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidGetRows]={name:t.L_DataReadError,message:t.L_InvalidGetRows},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidReadForBlankRow]={name:t.L_DataReadError,message:t.L_InvalidReadForBlankRow},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeUnsupportedDataObject]={name:t.L_DataWriteError,message:t.L_UnsupportedDataObject},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCannotWriteToSelection]={name:t.L_DataWriteError,message:t.L_CannotWriteToSelection},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDataNotMatchSelection]={name:t.L_DataWriteError,message:t.L_DataNotMatchSelection},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeOverwriteWorksheetData]={name:t.L_DataWriteError,message:t.L_OverwriteWorksheetData},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDataNotMatchBindingSize]={name:t.L_DataWriteError,message:t.L_DataNotMatchBindingSize},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSetStartRowColumn]={name:t.L_DataWriteError,message:t.L_InvalidSetStartRowColumn},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidDataFormat]={name:t.L_InvalidFormat,message:t.L_InvalidDataFormat},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDataNotMatchCoercionType]={name:t.L_InvalidDataObject,message:t.L_DataNotMatchCoercionType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDataNotMatchBindingType]={name:t.L_InvalidDataObject,message:t.L_DataNotMatchBindingType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSetDataIsTooLarge]={name:t.L_DataWriteError,message:t.L_SetDataIsTooLarge},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNonUniformPartialSetNotSupported]={name:t.L_DataWriteError,message:t.L_NonUniformPartialSetNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSetColumns]={name:t.L_DataWriteError,message:t.L_InvalidSetColumns},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSetRows]={name:t.L_DataWriteError,message:t.L_InvalidSetRows},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSetDataParametersConflict]={name:t.L_DataWriteError,message:t.L_SetDataParametersConflict},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSelectionCannotBound]={name:t.L_BindingCreationError,message:t.L_SelectionCannotBound},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeBindingNotExist]={name:t.L_InvalidBindingError,message:t.L_BindingNotExist},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeBindingToMultipleSelection]={name:t.L_BindingCreationError,message:t.L_BindingToMultipleSelection},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSelectionForBindingType]={name:t.L_BindingCreationError,message:t.L_InvalidSelectionForBindingType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeOperationNotSupportedOnThisBindingType]={name:t.L_InvalidBindingOperation,message:t.L_OperationNotSupportedOnThisBindingType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNamedItemNotFound]={name:t.L_BindingCreationError,message:t.L_NamedItemNotFound},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeMultipleNamedItemFound]={name:t.L_BindingCreationError,message:t.L_MultipleNamedItemFound},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidNamedItemForBindingType]={name:t.L_BindingCreationError,message:t.L_InvalidNamedItemForBindingType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeUnknownBindingType]={name:t.L_InvalidBinding,message:t.L_UnknownBindingType},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeOperationNotSupportedOnMatrixData]={name:t.L_InvalidBindingOperation,message:t.L_OperationNotSupportedOnMatrixData},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidColumnsForBinding]={name:t.L_InvalidBinding,message:t.L_InvalidColumnsForBinding},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSettingNameNotExist]={name:t.L_ReadSettingsError,message:t.L_SettingNameNotExist},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSettingsCannotSave]={name:t.L_SaveSettingsError,message:t.L_SettingsCannotSave},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSettingsAreStale]={name:t.L_SettingsStaleError,message:t.L_SettingsAreStale},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeOperationNotSupported]={name:t.L_HostError,message:t.L_OperationNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError]={name:t.L_InternalError,message:t.L_InternalErrorDescription},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDocumentReadOnly]={name:t.L_PermissionDenied,message:t.L_DocumentReadOnly},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeEventHandlerNotExist]={name:t.L_EventRegistrationError,message:t.L_EventHandlerNotExist},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidApiCallInContext]={name:t.L_InvalidAPICall,message:t.L_InvalidApiCallInContext},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeShuttingDown]={name:t.L_ShuttingDown,message:t.L_ShuttingDown},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeUnsupportedEnumeration]={name:t.L_UnsupportedEnumeration,message:t.L_UnsupportedEnumerationMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeIndexOutOfRange]={name:t.L_IndexOutOfRange,message:t.L_IndexOutOfRange},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeBrowserAPINotSupported]={name:t.L_APINotSupported,message:t.L_BrowserAPINotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeRequestTimeout]={name:t.L_APICallFailed,message:t.L_RequestTimeout},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidOrTimedOutSession]={name:t.L_InvalidOrTimedOutSession,message:t.L_InvalidOrTimedOutSessionMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidApiArguments]={name:t.L_APICallFailed,message:t.L_InvalidApiArgumentsMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeWorkbookHidden]={name:t.L_APICallFailed,message:t.L_WorkbookHiddenMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeWriteNotSupportedWhenModalDialogOpen]={name:t.L_APICallFailed,message:t.L_WriteNotSupportedWhenModalDialogOpen},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeTooManyIncompleteRequests]={name:t.L_APICallFailed,message:t.L_TooManyIncompleteRequests},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeRequestTokenUnavailable]={name:t.L_APICallFailed,message:t.L_RequestTokenUnavailable},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeActivityLimitReached]={name:t.L_APICallFailed,message:t.L_ActivityLimitReached},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeRequestPayloadSizeLimitExceeded]={name:t.L_APICallFailed,message:t.L_RequestPayloadSizeLimitExceededMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeResponsePayloadSizeLimitExceeded]={name:t.L_APICallFailed,message:t.L_ResponsePayloadSizeLimitExceededMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCustomXmlNodeNotFound]={name:t.L_InvalidNode,message:t.L_CustomXmlNodeNotFound},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCustomXmlError]={name:t.L_CustomXmlError,message:t.L_CustomXmlError},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCustomXmlExceedQuota]={name:t.L_CustomXmlExceedQuotaName,message:t.L_CustomXmlExceedQuotaMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCustomXmlOutOfDate]={name:t.L_CustomXmlOutOfDateName,message:t.L_CustomXmlOutOfDateMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNoCapability]={name:t.L_PermissionDenied,message:t.L_NoCapability},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCannotNavTo]={name:t.L_CannotNavigateTo,message:t.L_CannotNavigateTo},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSpecifiedIdNotExist]={name:t.L_SpecifiedIdNotExist,message:t.L_SpecifiedIdNotExist},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNavOutOfBound]={name:t.L_NavOutOfBound,message:t.L_NavOutOfBound},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCellDataAmountBeyondLimits]={name:t.L_DataWriteReminder,message:t.L_CellDataAmountBeyondLimits},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeElementMissing]={name:t.L_MissingParameter,message:t.L_ElementMissing},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeProtectedError]={name:t.L_PermissionDenied,message:t.L_NoCapability},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidCellsValue]={name:t.L_InvalidValue,message:t.L_InvalidCellsValue},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidTableOptionValue]={name:t.L_InvalidValue,message:t.L_InvalidTableOptionValue},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidFormatValue]={name:t.L_InvalidValue,message:t.L_InvalidFormatValue},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeRowIndexOutOfRange]={name:t.L_OutOfRange,message:t.L_RowIndexOutOfRange},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeColIndexOutOfRange]={name:t.L_OutOfRange,message:t.L_ColIndexOutOfRange},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeFormatValueOutOfRange]={name:t.L_OutOfRange,message:t.L_FormatValueOutOfRange},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCellFormatAmountBeyondLimits]={name:t.L_FormattingReminder,message:t.L_CellFormatAmountBeyondLimits},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeMemoryFileLimit]={name:t.L_MemoryLimit,message:t.L_CloseFileBeforeRetrieve},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNetworkProblemRetrieveFile]={name:t.L_NetworkProblem,message:t.L_NetworkProblemRetrieveFile},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSliceSize]={name:t.L_InvalidValue,message:t.L_SliceSizeNotSupported},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDialogAlreadyOpened]={name:t.L_DisplayDialogError,message:t.L_DialogAlreadyOpened},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidWidth]={name:t.L_IndexOutOfRange,message:t.L_IndexOutOfRange},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidHeight]={name:t.L_IndexOutOfRange,message:t.L_IndexOutOfRange},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNavigationError]={name:t.L_DisplayDialogError,message:t.L_NetworkProblem},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidScheme]={name:t.L_DialogNavigateError,message:t.L_DialogInvalidScheme},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeAppDomains]={name:t.L_DisplayDialogError,message:t.L_DialogAddressNotTrusted},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeRequireHTTPS]={name:t.L_DisplayDialogError,message:t.L_DialogRequireHTTPS},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeEndUserIgnore]={name:t.L_DisplayDialogError,message:t.L_UserClickIgnore},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeCrossZone]={name:t.L_DisplayDialogError,message:t.L_NewWindowCrossZoneErrorString},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeModalDialogOpen]={name:t.L_DisplayDialogError,message:t.L_ModalDialogOpen},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDocumentIsInactive]={name:t.L_DisplayDialogError,message:t.L_DocumentIsInactive},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeDialogParentIsMinimized]={name:t.L_DisplayDialogError,message:t.L_DialogParentIsMinimized},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeNotSSOAgave]={name:t.L_APINotSupported,message:t.L_InvalidSSOAddinMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOUserNotSignedIn]={name:t.L_UserNotSignedIn,message:t.L_UserNotSignedIn},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOUserAborted]={name:t.L_UserAborted,message:t.L_UserAbortedMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOUnsupportedUserIdentity]={name:t.L_UnsupportedUserIdentity,message:t.L_UnsupportedUserIdentityMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOInvalidResourceUrl]={name:t.L_InvalidResourceUrl,message:t.L_InvalidResourceUrlMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOInvalidGrant]={name:t.L_InvalidGrant,message:t.L_InvalidGrantMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOClientError]={name:t.L_SSOClientError,message:t.L_SSOClientErrorMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOServerError]={name:t.L_SSOServerError,message:t.L_SSOServerErrorMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeAddinIsAlreadyRequestingToken]={name:t.L_AddinIsAlreadyRequestingToken,message:t.L_AddinIsAlreadyRequestingTokenMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOUserConsentNotSupportedByCurrentAddinCategory]={name:t.L_SSOUserConsentNotSupportedByCurrentAddinCategory,message:t.L_SSOUserConsentNotSupportedByCurrentAddinCategoryMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOConnectionLost]={name:t.L_SSOConnectionLostError,message:t.L_SSOConnectionLostErrorMessage},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOUnsupportedPlatform]={name:t.L_APINotSupported,message:t.L_SSOUnsupportedPlatform},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeSSOCallThrottled]={name:t.L_APICallFailed,message:t.L_RequestTokenUnavailable},e[OSF.DDA.ErrorCodeManager.errorCodes.ooeOperationCancelled]={name:t.L_OperationCancelledError,message:t.L_OperationCancelledErrorMessage}}}}(),function(e){!function(e){var t=function(){};e.RequirementVersion=t;var n=function(e){this.isSetSupported=function(e,t){if(null==e)return!1;null==t&&(t=0);var n=this._setMap._sets;if(n.hasOwnProperty(e.toLowerCase())){var r=n[e.toLowerCase()];try{var i=this._getVersion(r);t+="";var o=this._getVersion(t);if(i.major>0&&i.major>o.major)return!0;if(i.major>0&&i.minor>=0&&i.major==o.major&&i.minor>=o.minor)return!0}catch(e){return!1}}return!1},this._getVersion=function(e){var t=(e+="").split("."),n=0,r=0;if(t.length<2&&isNaN(Number(e)))throw"version format incorrect";if(n=Number(t[0]),t.length>=2&&(r=Number(t[1])),isNaN(n)||isNaN(r))throw"version format incorrect";return{minor:r,major:n}},this._setMap=e,this.isSetSupported=this.isSetSupported.bind(this)};e.RequirementMatrix=n;var r=function(e){this._addSetMap=function(e){for(var t in e)this._sets[t]=e[t]},this._sets=e};e.DefaultSetRequirement=r;var i=function(e){function t(){return e.call(this,{dialogapi:1.1})||this}return __extends(t,e),t}(r);e.DefaultRequiredDialogSetRequirement=i;var o=function(e){function t(){return e.call(this,{dialogorigin:1.1})||this}return __extends(t,e),t}(r);e.DefaultOptionalDialogSetRequirement=o;var s=function(e){function t(){return e.call(this,{bindingevents:1.1,documentevents:1.1,excelapi:1.1,matrixbindings:1.1,matrixcoercion:1.1,selection:1.1,settings:1.1,tablebindings:1.1,tablecoercion:1.1,textbindings:1.1,textcoercion:1.1})||this}return __extends(t,e),t}(r);e.ExcelClientDefaultSetRequirement=s;var a=function(e){function t(){var t=e.call(this)||this;return t._addSetMap({imagecoercion:1.1}),t}return __extends(t,e),t}(s);e.ExcelClientV1DefaultSetRequirement=a;var l=function(e){function t(){return e.call(this,{mailbox:1.3})||this}return __extends(t,e),t}(r);e.OutlookClientDefaultSetRequirement=l;var u=function(e){function t(){return e.call(this,{bindingevents:1.1,compressedfile:1.1,customxmlparts:1.1,documentevents:1.1,file:1.1,htmlcoercion:1.1,matrixbindings:1.1,matrixcoercion:1.1,ooxmlcoercion:1.1,pdffile:1.1,selection:1.1,settings:1.1,tablebindings:1.1,tablecoercion:1.1,textbindings:1.1,textcoercion:1.1,textfile:1.1,wordapi:1.1})||this}return __extends(t,e),t}(r);e.WordClientDefaultSetRequirement=u;var c=function(e){function t(){var t=e.call(this)||this;return t._addSetMap({customxmlparts:1.2,wordapi:1.2,imagecoercion:1.1}),t}return __extends(t,e),t}(u);e.WordClientV1DefaultSetRequirement=c;var p=function(e){function t(){return e.call(this,{activeview:1.1,compressedfile:1.1,documentevents:1.1,file:1.1,pdffile:1.1,selection:1.1,settings:1.1,textcoercion:1.1})||this}return __extends(t,e),t}(r);e.PowerpointClientDefaultSetRequirement=p;var f=function(e){function t(){var t=e.call(this)||this;return t._addSetMap({imagecoercion:1.1}),t}return __extends(t,e),t}(p);e.PowerpointClientV1DefaultSetRequirement=f;var h=function(e){function t(){return e.call(this,{selection:1.1,textcoercion:1.1})||this}return __extends(t,e),t}(r);e.ProjectClientDefaultSetRequirement=h;var d=function(e){function t(){return e.call(this,{bindingevents:1.1,documentevents:1.1,matrixbindings:1.1,matrixcoercion:1.1,selection:1.1,settings:1.1,tablebindings:1.1,tablecoercion:1.1,textbindings:1.1,textcoercion:1.1,file:1.1})||this}return __extends(t,e),t}(r);e.ExcelWebDefaultSetRequirement=d;var y=function(e){function t(){return e.call(this,{compressedfile:1.1,documentevents:1.1,file:1.1,imagecoercion:1.1,matrixcoercion:1.1,ooxmlcoercion:1.1,pdffile:1.1,selection:1.1,settings:1.1,tablecoercion:1.1,textcoercion:1.1,textfile:1.1})||this}return __extends(t,e),t}(r);e.WordWebDefaultSetRequirement=y;var m=function(e){function t(){return e.call(this,{activeview:1.1,settings:1.1})||this}return __extends(t,e),t}(r);e.PowerpointWebDefaultSetRequirement=m;var g=function(e){function t(){return e.call(this,{mailbox:1.3})||this}return __extends(t,e),t}(r);e.OutlookWebDefaultSetRequirement=g;var _=function(e){function t(){return e.call(this,{activeview:1.1,documentevents:1.1,selection:1.1,settings:1.1,textcoercion:1.1})||this}return __extends(t,e),t}(r);e.SwayWebDefaultSetRequirement=_;var b=function(e){function t(){return e.call(this,{bindingevents:1.1,partialtablebindings:1.1,settings:1.1,tablebindings:1.1,tablecoercion:1.1})||this}return __extends(t,e),t}(r);e.AccessWebDefaultSetRequirement=b;var O=function(e){function t(){return e.call(this,{bindingevents:1.1,documentevents:1.1,matrixbindings:1.1,matrixcoercion:1.1,selection:1.1,settings:1.1,tablebindings:1.1,tablecoercion:1.1,textbindings:1.1,textcoercion:1.1})||this}return __extends(t,e),t}(r);e.ExcelIOSDefaultSetRequirement=O;var v=function(e){function t(){return e.call(this,{bindingevents:1.1,compressedfile:1.1,customxmlparts:1.1,documentevents:1.1,file:1.1,htmlcoercion:1.1,matrixbindings:1.1,matrixcoercion:1.1,ooxmlcoercion:1.1,pdffile:1.1,selection:1.1,settings:1.1,tablebindings:1.1,tablecoercion:1.1,textbindings:1.1,textcoercion:1.1,textfile:1.1})||this}return __extends(t,e),t}(r);e.WordIOSDefaultSetRequirement=v;var S=function(e){function t(){var t=e.call(this)||this;return t._addSetMap({customxmlparts:1.2,wordapi:1.2}),t}return __extends(t,e),t}(v);e.WordIOSV1DefaultSetRequirement=S;var C=function(e){function t(){return e.call(this,{activeview:1.1,compressedfile:1.1,documentevents:1.1,file:1.1,pdffile:1.1,selection:1.1,settings:1.1,textcoercion:1.1})||this}return __extends(t,e),t}(r);e.PowerpointIOSDefaultSetRequirement=C;var P=function(e){function t(){return e.call(this,{mailbox:1.1})||this}return __extends(t,e),t}(r);e.OutlookIOSDefaultSetRequirement=P;var A=function(){function e(){}return e.initializeOsfDda=function(){OSF.OUtil.setNamespace("Requirement",OSF.DDA)},e.getDefaultRequirementMatrix=function(t){this.initializeDefaultSetMatrix();var i=void 0,o=t.get_requirementMatrix();if(null!=o&&o.length>0&&"undefined"!=typeof JSON){var s=JSON.parse(t.get_requirementMatrix().toLowerCase());i=new n(new r(s))}else{var a=e.getClientFullVersionString(t);i=null!=e.DefaultSetArrayMatrix&&null!=e.DefaultSetArrayMatrix[a]?new n(e.DefaultSetArrayMatrix[a]):new n(new r({}))}return i},e.getDefaultDialogRequirementMatrix=function(e){var t=void 0,s=e.get_dialogRequirementMatrix();if(null!=s&&s.length>0&&"undefined"!=typeof JSON){var a=JSON.parse(e.get_requirementMatrix().toLowerCase());t=new r(a)}else{t=new i;var l=e.get_requirementMatrix();if(null!=l&&l.length>0&&"undefined"!=typeof JSON){a=JSON.parse(l.toLowerCase());for(var u in t._sets)a.hasOwnProperty(u)&&(t._sets[u]=a[u]);var c=new o;for(var u in c._sets)a.hasOwnProperty(u)&&(t._sets[u]=a[u])}}return new n(t)},e.getClientFullVersionString=function(e){var t=e.get_appMinorVersion(),n="",r="",i=e.get_appName();return(1024==i||4096==i||8192==i||65536==i)&&1==e.get_appVersion()?r=4096==i&&t>=15?"16.00.01":"16.00":64==e.get_appName()?r=e.get_appVersion():(n=t<10?"0"+t:""+t,r=e.get_appVersion()+"."+n),e.get_appName()+"-"+r},e.initializeDefaultSetMatrix=function(){e.DefaultSetArrayMatrix[e.Excel_RCLIENT_1600]=new s,e.DefaultSetArrayMatrix[e.Word_RCLIENT_1600]=new u,e.DefaultSetArrayMatrix[e.PowerPoint_RCLIENT_1600]=new p,e.DefaultSetArrayMatrix[e.Excel_RCLIENT_1601]=new a,e.DefaultSetArrayMatrix[e.Word_RCLIENT_1601]=new c,e.DefaultSetArrayMatrix[e.PowerPoint_RCLIENT_1601]=new f,e.DefaultSetArrayMatrix[e.Outlook_RCLIENT_1600]=new l,e.DefaultSetArrayMatrix[e.Excel_WAC_1600]=new d,e.DefaultSetArrayMatrix[e.Word_WAC_1600]=new y,e.DefaultSetArrayMatrix[e.Outlook_WAC_1600]=new g,e.DefaultSetArrayMatrix[e.Outlook_WAC_1601]=new g,e.DefaultSetArrayMatrix[e.Project_RCLIENT_1600]=new h,e.DefaultSetArrayMatrix[e.Access_WAC_1600]=new b,e.DefaultSetArrayMatrix[e.PowerPoint_WAC_1600]=new m,e.DefaultSetArrayMatrix[e.Excel_IOS_1600]=new O,e.DefaultSetArrayMatrix[e.SWAY_WAC_1600]=new _,e.DefaultSetArrayMatrix[e.Word_IOS_1600]=new v,e.DefaultSetArrayMatrix[e.Word_IOS_16001]=new S,e.DefaultSetArrayMatrix[e.PowerPoint_IOS_1600]=new C,e.DefaultSetArrayMatrix[e.Outlook_IOS_1600]=new P},e.Excel_RCLIENT_1600="1-16.00",e.Excel_RCLIENT_1601="1-16.01",e.Word_RCLIENT_1600="2-16.00",e.Word_RCLIENT_1601="2-16.01",e.PowerPoint_RCLIENT_1600="4-16.00",e.PowerPoint_RCLIENT_1601="4-16.01",e.Outlook_RCLIENT_1600="8-16.00",e.Excel_WAC_1600="16-16.00",e.Word_WAC_1600="32-16.00",e.Outlook_WAC_1600="64-16.00",e.Outlook_WAC_1601="64-16.01",e.Project_RCLIENT_1600="128-16.00",e.Access_WAC_1600="256-16.00",e.PowerPoint_WAC_1600="512-16.00",e.Excel_IOS_1600="1024-16.00",e.SWAY_WAC_1600="2048-16.00",e.Word_IOS_1600="4096-16.00",e.Word_IOS_16001="4096-16.00.01",e.PowerPoint_IOS_1600="8192-16.00",e.Outlook_IOS_1600="65536-16.00",e.DefaultSetArrayMatrix={},e}();e.RequirementsMatrixFactory=A}(e.Requirement||(e.Requirement={}))}(OfficeExt||(OfficeExt={})),OfficeExt.Requirement.RequirementsMatrixFactory.initializeOsfDda(),Microsoft.Office.WebExtension.ApplicationMode={WebEditor:"webEditor",WebViewer:"webViewer",Client:"client"},Microsoft.Office.WebExtension.DocumentMode={ReadOnly:"readOnly",ReadWrite:"readWrite"},OSF.NamespaceManager=function(){var e,t=!1;return{enableShortcut:function(){t||(window.Office?e=window.Office:OSF.OUtil.setNamespace("Office",window),window.Office=Microsoft.Office.WebExtension,t=!0)},disableShortcut:function(){t&&(e?window.Office=e:OSF.OUtil.unsetNamespace("Office",window),t=!1)}}}(),OSF.NamespaceManager.enableShortcut(),Microsoft.Office.WebExtension.useShortNamespace=function(e){e?OSF.NamespaceManager.enableShortcut():OSF.NamespaceManager.disableShortcut()},Microsoft.Office.WebExtension.select=function(e,t){var n;if(e&&"string"==typeof e){var r=e.indexOf("#");if(-1!=r){var i=e.substring(0,r),o=e.substring(r+1);switch(i){case"binding":case"bindings":o&&(n=new OSF.DDA.BindingPromise(o))}}}if(n)return n.onFail=t,n;if(t){var s=typeof t;if("function"!=s)throw OSF.OUtil.formatString(Strings.OfficeOM.L_CallbackNotAFunction,s);var a={};a[Microsoft.Office.WebExtension.Parameters.Callback]=t,OSF.DDA.issueAsyncResult(a,OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidApiCallInContext,OSF.DDA.ErrorCodeManager.getErrorArgs(OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidApiCallInContext))}},OSF.DDA.Context=function(e,t,n,r,i){if(OSF.OUtil.defineEnumerableProperties(this,{contentLanguage:{value:e.get_dataLocale()},displayLanguage:{value:e.get_appUILocale()},touchEnabled:{value:e.get_touchEnabled()},commerceAllowed:{value:e.get_commerceAllowed()},host:{value:OfficeExt.HostName.Host.getInstance().getHost()},platform:{value:OfficeExt.HostName.Host.getInstance().getPlatform()},isDialog:{value:OSF._OfficeAppFactory.getHostInfo().isDialog},diagnostics:{value:OfficeExt.HostName.Host.getInstance().getDiagnostics(e.get_hostFullVersion())}}),n&&OSF.OUtil.defineEnumerableProperty(this,"license",{value:n}),e.ui&&OSF.OUtil.defineEnumerableProperty(this,"ui",{value:e.ui}),e.auth&&OSF.OUtil.defineEnumerableProperty(this,"auth",{value:e.auth}),e.webAuth&&OSF.OUtil.defineEnumerableProperty(this,"webAuth",{value:e.webAuth}),e.application&&OSF.OUtil.defineEnumerableProperty(this,"application",{value:e.application}),e.extensionLifeCycle&&OSF.OUtil.defineEnumerableProperty(this,"extensionLifeCycle",{value:e.extensionLifeCycle}),e.messaging&&OSF.OUtil.defineEnumerableProperty(this,"messaging",{value:e.messaging}),e.ui&&e.ui.taskPaneAction&&OSF.OUtil.defineEnumerableProperty(this,"taskPaneAction",{value:e.ui.taskPaneAction}),e.ui&&e.ui.ribbonGallery&&OSF.OUtil.defineEnumerableProperty(this,"ribbonGallery",{value:e.ui.ribbonGallery}),e.get_isDialog()){var o=OfficeExt.Requirement.RequirementsMatrixFactory.getDefaultDialogRequirementMatrix(e);OSF.OUtil.defineEnumerableProperty(this,"requirements",{value:o})}else{if(t&&OSF.OUtil.defineEnumerableProperty(this,"document",{value:t}),r){var s=r.displayName||"appOM";delete r.displayName,OSF.OUtil.defineEnumerableProperty(this,s,{value:r})}e.get_officeTheme()?OSF.OUtil.defineEnumerableProperty(this,"officeTheme",{get:function(){return e.get_officeTheme()}}):i&&OSF.OUtil.defineEnumerableProperty(this,"officeTheme",{get:function(){return i()}});o=OfficeExt.Requirement.RequirementsMatrixFactory.getDefaultRequirementMatrix(e);OSF.OUtil.defineEnumerableProperty(this,"requirements",{value:o})}},OSF.DDA.OutlookContext=function(e,t,n,r,i){OSF.DDA.OutlookContext.uber.constructor.call(this,e,null,n,r,i),t&&OSF.OUtil.defineEnumerableProperty(this,"roamingSettings",{value:t})},OSF.OUtil.extend(OSF.DDA.OutlookContext,OSF.DDA.Context),OSF.DDA.OutlookAppOm=function(e,t,n){},OSF.DDA.Application=function(e){},OSF.DDA.Document=function(e,t){var n;switch(e.get_clientMode()){case OSF.ClientMode.ReadOnly:n=Microsoft.Office.WebExtension.DocumentMode.ReadOnly;break;case OSF.ClientMode.ReadWrite:n=Microsoft.Office.WebExtension.DocumentMode.ReadWrite}t&&OSF.OUtil.defineEnumerableProperty(this,"settings",{value:t}),OSF.OUtil.defineMutableProperties(this,{mode:{value:n},url:{value:e.get_docUrl()}})},OSF.DDA.JsomDocument=function(e,t,n){OSF.DDA.JsomDocument.uber.constructor.call(this,e,n),t&&OSF.OUtil.defineEnumerableProperty(this,"bindings",{get:function(){return t}});var r=OSF.DDA.AsyncMethodNames;OSF.DDA.DispIdHost.addAsyncMethods(this,[r.GetSelectedDataAsync,r.SetSelectedDataAsync]),OSF.DDA.DispIdHost.addEventSupport(this,new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.DocumentSelectionChanged]))},OSF.OUtil.extend(OSF.DDA.JsomDocument,OSF.DDA.Document),OSF.OUtil.defineEnumerableProperty(Microsoft.Office.WebExtension,"context",{get:function(){var e;return OSF&&OSF._OfficeAppFactory&&(e=OSF._OfficeAppFactory.getContext()),e}}),OSF.DDA.License=function(e){OSF.OUtil.defineEnumerableProperty(this,"value",{value:e})},OSF.DDA.ApiMethodCall=function(e,t,n,r,i){var o=e.length,s=OSF.OUtil.delayExecutionAndCache((function(){return OSF.OUtil.formatString(Strings.OfficeOM.L_InvalidParameters,i)}));this.verifyArguments=function(e,t){for(var n in e){var r=e[n],i=t[n];if(r.enum)switch(typeof i){case"string":if(OSF.OUtil.listContainsValue(r.enum,i))break;case"undefined":throw OSF.DDA.ErrorCodeManager.errorCodes.ooeUnsupportedEnumeration;default:throw s()}if(r.types&&!OSF.OUtil.listContainsValue(r.types,typeof i))throw s()}},this.extractRequiredArguments=function(t,n,r){if(t.length<o)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_MissingRequiredArguments);var i,a=[];for(i=0;i<o;i++)a.push(t[i]);this.verifyArguments(e,a);var l={};for(i=0;i<o;i++){var u=e[i],c=a[i];if(u.verify)if(!u.verify(c,n,r))throw s();l[u.name]=c}return l},this.fillOptions=function(e,n,r,i){for(var o in e=e||{},t)if(!OSF.OUtil.listContainsKey(e,o)){var s=void 0,a=t[o];a.calculate&&n&&(s=a.calculate(n,r,i)),s||void 0===a.defaultValue||(s=a.defaultValue),e[o]=s}return e},this.constructCallArgs=function(e,t,i,o){var s={};for(var a in e)s[a]=e[a];for(var l in t)s[l]=t[l];for(var u in n)s[u]=n[u](i,o);return r&&(s=r(s,i,o)),s}},OSF.OUtil.setNamespace("AsyncResultEnum",OSF.DDA),OSF.DDA.AsyncResultEnum.Properties={Context:"Context",Value:"Value",Status:"Status",Error:"Error"},Microsoft.Office.WebExtension.AsyncResultStatus={Succeeded:"succeeded",Failed:"failed"},OSF.DDA.AsyncResultEnum.ErrorCode={Success:0,Failed:1},OSF.DDA.AsyncResultEnum.ErrorProperties={Name:"Name",Message:"Message",Code:"Code"},OSF.DDA.AsyncMethodNames={},OSF.DDA.AsyncMethodNames.addNames=function(e){for(var t in e){var n={};OSF.OUtil.defineEnumerableProperties(n,{id:{value:t},displayName:{value:e[t]}}),OSF.DDA.AsyncMethodNames[t]=n}},OSF.DDA.AsyncMethodCall=function(e,t,n,r,i,o,s){var a=e.length,l=new OSF.DDA.ApiMethodCall(e,t,n,o,s);this.verifyAndExtractCall=function(e,n,r){var i=l.extractRequiredArguments(e,n,r),o=function(e,n,r,i){if(e.length>a+2)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyArguments);for(var o,s,u=e.length-1;u>=a;u--){var c=e[u];switch(typeof c){case"object":if(o)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyOptionalObjects);o=c;break;case"function":if(s)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyOptionalFunction);s=c;break;default:throw OsfMsAjaxFactory.msAjaxError.argument(Strings.OfficeOM.L_InValidOptionalArgument)}}if(o=l.fillOptions(o,n,r,i),s){if(o[Microsoft.Office.WebExtension.Parameters.Callback])throw Strings.OfficeOM.L_RedundantCallbackSpecification;o[Microsoft.Office.WebExtension.Parameters.Callback]=s}return l.verifyArguments(t,o),o}(e,i,n,r);return l.constructCallArgs(i,o,n,r)},this.processResponse=function(e,t,n,o){return e==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess?r?r(t,n,o):t:i?i(e,t):OSF.DDA.ErrorCodeManager.getErrorArgs(e)},this.getCallArgs=function(e){for(var t,n,r=e.length-1;r>=a;r--){var i=e[r];switch(typeof i){case"object":t=i;break;case"function":n=i}}return t=t||{},n&&(t[Microsoft.Office.WebExtension.Parameters.Callback]=n),t}},OSF.DDA.AsyncMethodCallFactory={manufacture:function(e){var t=e.supportedOptions?OSF.OUtil.createObject(e.supportedOptions):[],n=e.privateStateCallbacks?OSF.OUtil.createObject(e.privateStateCallbacks):[];return new OSF.DDA.AsyncMethodCall(e.requiredArguments||[],t,n,e.onSucceeded,e.onFailed,e.checkCallArgs,e.method.displayName)}},OSF.DDA.AsyncMethodCalls={},OSF.DDA.AsyncMethodCalls.define=function(e){OSF.DDA.AsyncMethodCalls[e.method.id]=OSF.DDA.AsyncMethodCallFactory.manufacture(e)},OSF.DDA.Error=function(e,t,n){OSF.OUtil.defineEnumerableProperties(this,{name:{value:e},message:{value:t},code:{value:n}})},OSF.DDA.AsyncResult=function(e,t){OSF.OUtil.defineEnumerableProperties(this,{value:{value:e[OSF.DDA.AsyncResultEnum.Properties.Value]},status:{value:t?Microsoft.Office.WebExtension.AsyncResultStatus.Failed:Microsoft.Office.WebExtension.AsyncResultStatus.Succeeded}}),e[OSF.DDA.AsyncResultEnum.Properties.Context]&&OSF.OUtil.defineEnumerableProperty(this,"asyncContext",{value:e[OSF.DDA.AsyncResultEnum.Properties.Context]}),t&&OSF.OUtil.defineEnumerableProperty(this,"error",{value:new OSF.DDA.Error(t[OSF.DDA.AsyncResultEnum.ErrorProperties.Name],t[OSF.DDA.AsyncResultEnum.ErrorProperties.Message],t[OSF.DDA.AsyncResultEnum.ErrorProperties.Code])})},OSF.DDA.issueAsyncResult=function(e,t,n){var r=e[Microsoft.Office.WebExtension.Parameters.Callback];if(r){var i,o={};o[OSF.DDA.AsyncResultEnum.Properties.Context]=e[Microsoft.Office.WebExtension.Parameters.AsyncContext],t==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess?o[OSF.DDA.AsyncResultEnum.Properties.Value]=n:(i={},n=n||OSF.DDA.ErrorCodeManager.getErrorArgs(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError),i[OSF.DDA.AsyncResultEnum.ErrorProperties.Code]=t||OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError,i[OSF.DDA.AsyncResultEnum.ErrorProperties.Name]=n.name||n,i[OSF.DDA.AsyncResultEnum.ErrorProperties.Message]=n.message||n),r(new OSF.DDA.AsyncResult(o,i))}},OSF.DDA.SyncMethodNames={},OSF.DDA.SyncMethodNames.addNames=function(e){for(var t in e){var n={};OSF.OUtil.defineEnumerableProperties(n,{id:{value:t},displayName:{value:e[t]}}),OSF.DDA.SyncMethodNames[t]=n}},OSF.DDA.SyncMethodCall=function(e,t,n,r,i){var o=e.length,s=new OSF.DDA.ApiMethodCall(e,t,n,r,i);this.verifyAndExtractCall=function(e,n,r){var i=s.extractRequiredArguments(e,n,r),a=function(e,n,r,i){if(e.length>o+1)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyArguments);for(var a,l=e.length-1;l>=o;l--){var u=e[l];switch(typeof u){case"object":if(a)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyOptionalObjects);a=u;break;default:throw OsfMsAjaxFactory.msAjaxError.argument(Strings.OfficeOM.L_InValidOptionalArgument)}}return a=s.fillOptions(a,n,r,i),s.verifyArguments(t,a),a}(e,i,n,r);return s.constructCallArgs(i,a,n,r)}},OSF.DDA.SyncMethodCallFactory={manufacture:function(e){var t=e.supportedOptions?OSF.OUtil.createObject(e.supportedOptions):[];return new OSF.DDA.SyncMethodCall(e.requiredArguments||[],t,e.privateStateCallbacks,e.checkCallArgs,e.method.displayName)}},OSF.DDA.SyncMethodCalls={},OSF.DDA.SyncMethodCalls.define=function(e){OSF.DDA.SyncMethodCalls[e.method.id]=OSF.DDA.SyncMethodCallFactory.manufacture(e)},OSF.DDA.ListType=function(){var e={};return{setListType:function(t,n){e[t]=n},isListType:function(t){return OSF.OUtil.listContainsKey(e,t)},getDescriptor:function(t){return e[t]}}}(),OSF.DDA.HostParameterMap=function(e,t){var n="toHost",r="fromHost",i="sourceData",o="self",s={};function a(n,i){var o=n?{}:void 0;for(var l in n){var u,c=n[l];if(OSF.DDA.ListType.isListType(l))for(var p in u=[],c)u.push(a(c[p],i));else if(OSF.OUtil.listContainsKey(s,l))u=s[l][i](c);else if(i==r&&e.preserveNesting(l))u=a(c,i);else{var f=t[l];if(f){var h=f[i];h&&void 0===(u=h[c])&&(u=c)}else u=c}o[l]=u}return o}function l(n,r,s){var l,u=t[n][s];if("toHost"==s)l=function n(r,i){var s;for(var a in i){var l;if(null!=(l=e.isComplexType(a)?n(r,t[a].toHost):r[a])){s||(s={});var u=i[a];u==o&&(u=a),s[u]=e.pack(a,l)}}return s}(a(r,s),u);else if("fromHost"==s){l=a(function n(r,s,a){for(var l in a||(a={}),s){var u,c,p=s[l];if(p==o)u=r;else{if(p==i){a[l]=r.toArray();continue}u=r[p]}if(null==u)a[l]=void 0;else if(u=e.unpack(l,u),e.isComplexType(l))c=t[l].fromHost,e.preserveNesting(l)?a[l]=n(u,c):n(u,c,a);else if(OSF.DDA.ListType.isListType(l)){(c={})[OSF.DDA.ListType.getDescriptor(l)]=o;var f=new Array(u.length);for(var h in u)f[h]=n(u[h],c);a[l]=f}else a[l]=u}return a}(r,u),s)}return l}s[Microsoft.Office.WebExtension.Parameters.Data]={toHost:function(e){if(null!=e&&void 0!==e.rows){var t={};t[OSF.DDA.TableDataProperties.TableRows]=e.rows,t[OSF.DDA.TableDataProperties.TableHeaders]=e.headers,e=t}return e},fromHost:function(e){return e}},s[Microsoft.Office.WebExtension.Parameters.SampleData]=s[Microsoft.Office.WebExtension.Parameters.Data],t||(t={}),this.addMapping=function(e,n){var r,i;if(n.map)for(var s in i={},r=n.map){var a=r[s];a==o&&(a=s),i[a]=s}else r=n.toHost,i=n.fromHost;var l=t[e];if(l){var u=l.toHost;for(var c in u)r[c]=u[c];for(var p in u=l.fromHost)i[p]=u[p]}else l=t[e]={};l.toHost=r,l.fromHost=i},this.toHost=function(e,t){return l(e,t,n)},this.fromHost=function(e,t){return l(e,t,r)},this.self=o,this.sourceData=i,this.addComplexType=function(t){e.addComplexType(t)},this.getDynamicType=function(t){return e.getDynamicType(t)},this.setDynamicType=function(t,n){e.setDynamicType(t,n)},this.dynamicTypes=s,this.doMapValues=function(e,t){return a(e,t)}},OSF.DDA.SpecialProcessor=function(e,t){this.addComplexType=function(t){e.push(t)},this.getDynamicType=function(e){return t[e]},this.setDynamicType=function(e,n){t[e]=n},this.isComplexType=function(t){return OSF.OUtil.listContainsValue(e,t)},this.isDynamicType=function(e){return OSF.OUtil.listContainsKey(t,e)},this.preserveNesting=function(e){var t=[];return OSF.DDA.PropertyDescriptors&&t.push(OSF.DDA.PropertyDescriptors.Subset),OSF.DDA.DataNodeEventProperties&&(t=t.concat([OSF.DDA.DataNodeEventProperties.OldNode,OSF.DDA.DataNodeEventProperties.NewNode,OSF.DDA.DataNodeEventProperties.NextSiblingNode])),OSF.OUtil.listContainsValue(t,e)},this.pack=function(e,n){return this.isDynamicType(e)?t[e].toHost(n):n},this.unpack=function(e,n){return this.isDynamicType(e)?t[e].fromHost(n):n}},OSF.DDA.getDecoratedParameterMap=function(e,t){var n=new OSF.DDA.HostParameterMap(e);n.self;function r(e){var t=null;if(e){t={};for(var n=e.length,r=0;r<n;r++)t[e[r].name]=e[r].value}return t}for(var i in n.define=function(e){var t={},i=r(e.toHost);e.invertible?t.map=i:e.canonical?t.toHost=t.fromHost=i:(t.toHost=i,t.fromHost=r(e.fromHost)),n.addMapping(e.type,t),e.isComplexType&&n.addComplexType(e.type)},t)n.define(t[i]);return n},OSF.OUtil.setNamespace("DispIdHost",OSF.DDA),OSF.DDA.DispIdHost.Methods={InvokeMethod:"invokeMethod",AddEventHandler:"addEventHandler",RemoveEventHandler:"removeEventHandler",OpenDialog:"openDialog",CloseDialog:"closeDialog",MessageParent:"messageParent",SendMessage:"sendMessage"},OSF.DDA.DispIdHost.Delegates={ExecuteAsync:"executeAsync",RegisterEventAsync:"registerEventAsync",UnregisterEventAsync:"unregisterEventAsync",ParameterMap:"parameterMap",OpenDialog:"openDialog",CloseDialog:"closeDialog",MessageParent:"messageParent",SendMessage:"sendMessage"},OSF.DDA.DispIdHost.Facade=function(e,t){var n={},r=OSF.DDA.AsyncMethodNames,i=OSF.DDA.MethodDispId,o={GoToByIdAsync:i.dispidNavigateToMethod,GetSelectedDataAsync:i.dispidGetSelectedDataMethod,SetSelectedDataAsync:i.dispidSetSelectedDataMethod,GetDocumentCopyChunkAsync:i.dispidGetDocumentCopyChunkMethod,ReleaseDocumentCopyAsync:i.dispidReleaseDocumentCopyMethod,GetDocumentCopyAsync:i.dispidGetDocumentCopyMethod,AddFromSelectionAsync:i.dispidAddBindingFromSelectionMethod,AddFromPromptAsync:i.dispidAddBindingFromPromptMethod,AddFromNamedItemAsync:i.dispidAddBindingFromNamedItemMethod,GetAllAsync:i.dispidGetAllBindingsMethod,GetByIdAsync:i.dispidGetBindingMethod,ReleaseByIdAsync:i.dispidReleaseBindingMethod,GetDataAsync:i.dispidGetBindingDataMethod,SetDataAsync:i.dispidSetBindingDataMethod,AddRowsAsync:i.dispidAddRowsMethod,AddColumnsAsync:i.dispidAddColumnsMethod,DeleteAllDataValuesAsync:i.dispidClearAllRowsMethod,RefreshAsync:i.dispidLoadSettingsMethod,SaveAsync:i.dispidSaveSettingsMethod,GetActiveViewAsync:i.dispidGetActiveViewMethod,GetFilePropertiesAsync:i.dispidGetFilePropertiesMethod,GetOfficeThemeAsync:i.dispidGetOfficeThemeMethod,GetDocumentThemeAsync:i.dispidGetDocumentThemeMethod,ClearFormatsAsync:i.dispidClearFormatsMethod,SetTableOptionsAsync:i.dispidSetTableOptionsMethod,SetFormatsAsync:i.dispidSetFormatsMethod,GetAccessTokenAsync:i.dispidGetAccessTokenMethod,GetAuthContextAsync:i.dispidGetAuthContextMethod,ExecuteRichApiRequestAsync:i.dispidExecuteRichApiRequestMethod,AppCommandInvocationCompletedAsync:i.dispidAppCommandInvocationCompletedMethod,CloseContainerAsync:i.dispidCloseContainerMethod,OpenBrowserWindow:i.dispidOpenBrowserWindow,CreateDocumentAsync:i.dispidCreateDocumentMethod,InsertFormAsync:i.dispidInsertFormMethod,ExecuteFeature:i.dispidExecuteFeature,QueryFeature:i.dispidQueryFeature,AddDataPartAsync:i.dispidAddDataPartMethod,GetDataPartByIdAsync:i.dispidGetDataPartByIdMethod,GetDataPartsByNameSpaceAsync:i.dispidGetDataPartsByNamespaceMethod,GetPartXmlAsync:i.dispidGetDataPartXmlMethod,GetPartNodesAsync:i.dispidGetDataPartNodesMethod,DeleteDataPartAsync:i.dispidDeleteDataPartMethod,GetNodeValueAsync:i.dispidGetDataNodeValueMethod,GetNodeXmlAsync:i.dispidGetDataNodeXmlMethod,GetRelativeNodesAsync:i.dispidGetDataNodesMethod,SetNodeValueAsync:i.dispidSetDataNodeValueMethod,SetNodeXmlAsync:i.dispidSetDataNodeXmlMethod,AddDataPartNamespaceAsync:i.dispidAddDataNamespaceMethod,GetDataPartNamespaceAsync:i.dispidGetDataUriByPrefixMethod,GetDataPartPrefixAsync:i.dispidGetDataPrefixByUriMethod,GetNodeTextAsync:i.dispidGetDataNodeTextMethod,SetNodeTextAsync:i.dispidSetDataNodeTextMethod,GetSelectedTask:i.dispidGetSelectedTaskMethod,GetTask:i.dispidGetTaskMethod,GetWSSUrl:i.dispidGetWSSUrlMethod,GetTaskField:i.dispidGetTaskFieldMethod,GetSelectedResource:i.dispidGetSelectedResourceMethod,GetResourceField:i.dispidGetResourceFieldMethod,GetProjectField:i.dispidGetProjectFieldMethod,GetSelectedView:i.dispidGetSelectedViewMethod,GetTaskByIndex:i.dispidGetTaskByIndexMethod,GetResourceByIndex:i.dispidGetResourceByIndexMethod,SetTaskField:i.dispidSetTaskFieldMethod,SetResourceField:i.dispidSetResourceFieldMethod,GetMaxTaskIndex:i.dispidGetMaxTaskIndexMethod,GetMaxResourceIndex:i.dispidGetMaxResourceIndexMethod,CreateTask:i.dispidCreateTaskMethod};for(var s in o)r[s]&&(n[r[s].id]=o[s]);r=OSF.DDA.SyncMethodNames;var a={MessageParent:(i=OSF.DDA.MethodDispId).dispidMessageParentMethod,SendMessage:i.dispidSendMessageMethod};for(var s in a)r[s]&&(n[r[s].id]=a[s]);r=Microsoft.Office.WebExtension.EventType;var l={SettingsChanged:(i=OSF.DDA.EventDispId).dispidSettingsChangedEvent,DocumentSelectionChanged:i.dispidDocumentSelectionChangedEvent,BindingSelectionChanged:i.dispidBindingSelectionChangedEvent,BindingDataChanged:i.dispidBindingDataChangedEvent,ActiveViewChanged:i.dispidActiveViewChangedEvent,OfficeThemeChanged:i.dispidOfficeThemeChangedEvent,DocumentThemeChanged:i.dispidDocumentThemeChangedEvent,AppCommandInvoked:i.dispidAppCommandInvokedEvent,DialogMessageReceived:i.dispidDialogMessageReceivedEvent,DialogParentMessageReceived:i.dispidDialogParentMessageReceivedEvent,ObjectDeleted:i.dispidObjectDeletedEvent,ObjectSelectionChanged:i.dispidObjectSelectionChangedEvent,ObjectDataChanged:i.dispidObjectDataChangedEvent,ContentControlAdded:i.dispidContentControlAddedEvent,RichApiMessage:i.dispidRichApiMessageEvent,ItemChanged:i.dispidOlkItemSelectedChangedEvent,RecipientsChanged:i.dispidOlkRecipientsChangedEvent,AppointmentTimeChanged:i.dispidOlkAppointmentTimeChangedEvent,RecurrenceChanged:i.dispidOlkRecurrenceChangedEvent,AttachmentsChanged:i.dispidOlkAttachmentsChangedEvent,EnhancedLocationsChanged:i.dispidOlkEnhancedLocationsChangedEvent,InfobarClicked:i.dispidOlkInfobarClickedEvent,TaskSelectionChanged:i.dispidTaskSelectionChangedEvent,ResourceSelectionChanged:i.dispidResourceSelectionChangedEvent,ViewSelectionChanged:i.dispidViewSelectionChangedEvent,DataNodeInserted:i.dispidDataNodeAddedEvent,DataNodeReplaced:i.dispidDataNodeReplacedEvent,DataNodeDeleted:i.dispidDataNodeDeletedEvent};for(var u in l)r[u]&&(n[r[u]]=l[u]);function c(e){return e==OSF.DDA.EventDispId.dispidObjectDeletedEvent||e==OSF.DDA.EventDispId.dispidObjectSelectionChangedEvent||e==OSF.DDA.EventDispId.dispidObjectDataChangedEvent||e==OSF.DDA.EventDispId.dispidContentControlAddedEvent}function p(e,t,n,r){if("number"!=typeof e)throw e;r||(r=t.getCallArgs(n)),OSF.DDA.issueAsyncResult(r,e,OSF.DDA.ErrorCodeManager.getErrorArgs(e))}this[OSF.DDA.DispIdHost.Methods.InvokeMethod]=function(r,i,o,s){var a;try{var l=r.id,u=OSF.DDA.AsyncMethodCalls[l];a=u.verifyAndExtractCall(i,o,s);var c=n[l],f=e(l),h=null;if(window.Excel&&window.Office.context.requirements.isSetSupported("RedirectV1Api")&&(window.Excel._RedirectV1APIs=!0),window.Excel&&window.Excel._RedirectV1APIs&&(h=window.Excel._V1APIMap[l])){var d=OSF.OUtil.shallowCopy(a);delete d[Microsoft.Office.WebExtension.Parameters.AsyncContext],h.preprocess&&(d=h.preprocess(d));var y=new window.Excel.RequestContext,m=h.call(y,d);y.sync().then((function(){var e=m.value,t=e.status;delete e.status,delete e["@odata.type"],h.postprocess&&(e=h.postprocess(e,d)),0!=t&&(e=OSF.DDA.ErrorCodeManager.getErrorArgs(t)),OSF.DDA.issueAsyncResult(a,t,e)})).catch((function(e){OSF.DDA.issueAsyncResult(a,OSF.DDA.ErrorCodeManager.errorCodes.ooeFailure,null)}))}else{var g;g=t.toHost?t.toHost(c,a):a;var _=(new Date).getTime();f[OSF.DDA.DispIdHost.Delegates.ExecuteAsync]({dispId:c,hostCallArgs:g,onCalling:function(){},onReceiving:function(){},onComplete:function(e,n){var r;r=e==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&t.fromHost?t.fromHost(c,n):n;var i=u.processResponse(e,r,o,a);OSF.DDA.issueAsyncResult(a,e,i),!OSF.AppTelemetry||OSF.ConstantNames&&OSF.ConstantNames.IsCustomFunctionsRuntime||OSF.AppTelemetry.onMethodDone(c,g,Math.abs((new Date).getTime()-_),e)}})}}catch(e){p(e,u,i,a)}},this[OSF.DDA.DispIdHost.Methods.AddEventHandler]=function(r,i,o,s){var a,l,u,f=!1;function h(e){var t;e==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&((f?i.addObjectEventHandler(l,a[Microsoft.Office.WebExtension.Parameters.Id],u):i.addEventHandler(l,u))||(e=OSF.DDA.ErrorCodeManager.errorCodes.ooeEventHandlerAdditionFailed));e!=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&(t=OSF.DDA.ErrorCodeManager.getErrorArgs(e)),OSF.DDA.issueAsyncResult(a,e,t)}try{var d=OSF.DDA.AsyncMethodCalls[OSF.DDA.AsyncMethodNames.AddHandlerAsync.id];if(a=d.verifyAndExtractCall(r,o,i),l=a[Microsoft.Office.WebExtension.Parameters.EventType],u=a[Microsoft.Office.WebExtension.Parameters.Handler],s)return void h(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess);var y=n[l],m=(f=c(y))?a[Microsoft.Office.WebExtension.Parameters.Id]:o.id||"";if(0==(f?i.getObjectEventHandlerCount(l,m):i.getEventHandlerCount(l)))(0,e(l)[OSF.DDA.DispIdHost.Delegates.RegisterEventAsync])({eventType:l,dispId:y,targetId:m,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)},onComplete:h,onEvent:function(e){var n=t.fromHost(y,e);f?i.fireObjectEvent(m,OSF.DDA.OMFactory.manufactureEventArgs(l,m,n)):i.fireEvent(OSF.DDA.OMFactory.manufactureEventArgs(l,o,n))}});else h(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess)}catch(e){p(e,d,r,a)}},this[OSF.DDA.DispIdHost.Methods.RemoveEventHandler]=function(t,r,i){var o,s,a,l=!1;function u(e){var t;e!=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&(t=OSF.DDA.ErrorCodeManager.getErrorArgs(e)),OSF.DDA.issueAsyncResult(o,e,t)}try{var f=OSF.DDA.AsyncMethodCalls[OSF.DDA.AsyncMethodNames.RemoveHandlerAsync.id];s=(o=f.verifyAndExtractCall(t,i,r))[Microsoft.Office.WebExtension.Parameters.EventType],a=o[Microsoft.Office.WebExtension.Parameters.Handler];var h,d,y=n[s],m=(l=c(y))?o[Microsoft.Office.WebExtension.Parameters.Id]:i.id||"";null===a?(d=l?r.clearObjectEventHandlers(s,m):r.clearEventHandlers(s),h=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess):h=(d=l?r.removeObjectEventHandler(s,m,a):r.removeEventHandler(s,a))?OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess:OSF.DDA.ErrorCodeManager.errorCodes.ooeEventHandlerNotExist;var g=l?r.getObjectEventHandlerCount(s,m):r.getEventHandlerCount(s);if(d&&0==g)(0,e(s)[OSF.DDA.DispIdHost.Delegates.UnregisterEventAsync])({eventType:s,dispId:y,targetId:m,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)},onComplete:u});else u(h)}catch(e){p(e,f,t,o)}},this[OSF.DDA.DispIdHost.Methods.OpenDialog]=function(r,i,o,s){var a,l,u=null,c=Microsoft.Office.WebExtension.EventType.DialogMessageReceived,f=Microsoft.Office.WebExtension.EventType.DialogEventReceived;function h(e){if(e!=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess)n=OSF.DDA.ErrorCodeManager.getErrorArgs(e);else{var t={};t[Microsoft.Office.WebExtension.Parameters.Id]=l,t[Microsoft.Office.WebExtension.Parameters.Data]=i;var n=u.processResponse(e,t,o,a);OSF.DialogShownStatus.hasDialogShown=!0,i.clearEventHandlers(c),i.clearEventHandlers(f)}OSF.DDA.issueAsyncResult(a,e,n)}try{if(null!=c&&null!=f||h(OSF.DDA.ErrorCodeManager.ooeOperationNotSupported),s){if(null==OSF.DDA.AsyncMethodNames.DisplayModalDialogAsync)return void h(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError);u=OSF.DDA.AsyncMethodCalls[OSF.DDA.AsyncMethodNames.DisplayModalDialogAsync.id]}else{if(null==OSF.DDA.AsyncMethodNames.DisplayDialogAsync)return void h(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError);u=OSF.DDA.AsyncMethodCalls[OSF.DDA.AsyncMethodNames.DisplayDialogAsync.id]}a=u.verifyAndExtractCall(r,o,i);var d=n[c],y=e(c),m=null!=y[OSF.DDA.DispIdHost.Delegates.OpenDialog]?y[OSF.DDA.DispIdHost.Delegates.OpenDialog]:y[OSF.DDA.DispIdHost.Delegates.RegisterEventAsync];a.isModal=s,l=JSON.stringify(a),OSF.DialogShownStatus.hasDialogShown||(i.clearQueuedEvent(c),i.clearQueuedEvent(f),i.clearQueuedEvent(Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived)),m({eventType:c,dispId:d,targetId:l,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)},onComplete:h,onEvent:function(e){var n=t.fromHost(d,e),r=OSF.DDA.OMFactory.manufactureEventArgs(c,o,n);if(r.type==f){var s=OSF.DDA.ErrorCodeManager.getErrorArgs(r.error),a={};a[OSF.DDA.AsyncResultEnum.ErrorProperties.Code]=status||OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError,a[OSF.DDA.AsyncResultEnum.ErrorProperties.Name]=s.name||s,a[OSF.DDA.AsyncResultEnum.ErrorProperties.Message]=s.message||s,r.error=new OSF.DDA.Error(a[OSF.DDA.AsyncResultEnum.ErrorProperties.Name],a[OSF.DDA.AsyncResultEnum.ErrorProperties.Message],a[OSF.DDA.AsyncResultEnum.ErrorProperties.Code])}i.fireOrQueueEvent(r),n[OSF.DDA.PropertyDescriptors.MessageType]==OSF.DialogMessageType.DialogClosed&&(i.clearEventHandlers(c),i.clearEventHandlers(f),i.clearEventHandlers(Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived),OSF.DialogShownStatus.hasDialogShown=!1)}})}catch(e){p(e,u,r,a)}},this[OSF.DDA.DispIdHost.Methods.CloseDialog]=function(t,r,i,o){var s,a,l,u=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess;try{var c=OSF.DDA.AsyncMethodCalls[OSF.DDA.AsyncMethodNames.CloseAsync.id];s=c.verifyAndExtractCall(t,o,i),a=Microsoft.Office.WebExtension.EventType.DialogMessageReceived,l=Microsoft.Office.WebExtension.EventType.DialogEventReceived,i.clearEventHandlers(a),i.clearEventHandlers(l);var f=n[a],h=e(a);(null!=h[OSF.DDA.DispIdHost.Delegates.CloseDialog]?h[OSF.DDA.DispIdHost.Delegates.CloseDialog]:h[OSF.DDA.DispIdHost.Delegates.UnregisterEventAsync])({eventType:a,dispId:f,targetId:r,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)},onComplete:function(e){u=e,OSF.DialogShownStatus.hasDialogShown=!1}})}catch(e){p(e,c,t,s)}if(u!=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess)throw OSF.OUtil.formatString(Strings.OfficeOM.L_FunctionCallFailed,OSF.DDA.AsyncMethodNames.CloseAsync.displayName,u)},this[OSF.DDA.DispIdHost.Methods.MessageParent]=function(t,r){var i=OSF.DDA.SyncMethodCalls[OSF.DDA.SyncMethodNames.MessageParent.id].verifyAndExtractCall(t,r,{});return(0,e(OSF.DDA.SyncMethodNames.MessageParent.id)[OSF.DDA.DispIdHost.Delegates.MessageParent])({dispId:n[OSF.DDA.SyncMethodNames.MessageParent.id],hostCallArgs:i,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)}})},this[OSF.DDA.DispIdHost.Methods.SendMessage]=function(t,r,i){var o=OSF.DDA.SyncMethodCalls[OSF.DDA.SyncMethodNames.SendMessage.id].verifyAndExtractCall(t,i,{});return(0,e(OSF.DDA.SyncMethodNames.SendMessage.id)[OSF.DDA.DispIdHost.Delegates.SendMessage])({dispId:n[OSF.DDA.SyncMethodNames.SendMessage.id],hostCallArgs:o,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)}})}},OSF.DDA.DispIdHost.addAsyncMethods=function(e,t,n){for(var r in t){var i=t[r],o=i.displayName;e[o]||OSF.OUtil.defineEnumerableProperty(e,o,{value:function(t){return function(){var r=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.InvokeMethod];r(t,arguments,e,n)}}(i)})}},OSF.DDA.DispIdHost.addEventSupport=function(e,t,n){var r=OSF.DDA.AsyncMethodNames.AddHandlerAsync.displayName,i=OSF.DDA.AsyncMethodNames.RemoveHandlerAsync.displayName;e[r]||OSF.OUtil.defineEnumerableProperty(e,r,{value:function(){var r=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.AddEventHandler];r(arguments,t,e,n)}}),e[i]||OSF.OUtil.defineEnumerableProperty(e,i,{value:function(){var n=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.RemoveEventHandler];n(arguments,t,e)}})},function(e){var t=function(){function e(){}return e.isInstanceOfType=function(e,t){if(null==t)return!1;if(t instanceof e)return!0;var n=t.constructor;return n&&"function"==typeof n&&n.__typeName&&"Object"!==n.__typeName||(n=Object),!(n!==e)||n.__typeName&&e.__typeName&&n.__typeName===e.__typeName},e}();e.MsAjaxTypeHelper=t;var n=function(){function e(){}return e.create=function(e,t){var n=new Error(e);if(n.message=e,t)for(var r in t)n[r]=t[r];return n.popStackFrame(),n},e.parameterCount=function(t){var n="Sys.ParameterCountException: "+(t||"Parameter count mismatch."),r=e.create(n,{name:"Sys.ParameterCountException"});return r.popStackFrame(),r},e.argument=function(t,n){var i="Sys.ArgumentException: "+(n||"Value does not fall within the expected range.");t&&(i+="\n"+r.format("Parameter name: {0}",t));var o=e.create(i,{name:"Sys.ArgumentException",paramName:t});return o.popStackFrame(),o},e.argumentNull=function(t,n){var i="Sys.ArgumentNullException: "+(n||"Value cannot be null.");t&&(i+="\n"+r.format("Parameter name: {0}",t));var o=e.create(i,{name:"Sys.ArgumentNullException",paramName:t});return o.popStackFrame(),o},e.argumentOutOfRange=function(t,n,i){var o="Sys.ArgumentOutOfRangeException: "+(i||"Specified argument was out of the range of valid values.");t&&(o+="\n"+r.format("Parameter name: {0}",t)),null!=n&&(o+="\n"+r.format("Actual value was {0}.",n));var s=e.create(o,{name:"Sys.ArgumentOutOfRangeException",paramName:t,actualValue:n});return s.popStackFrame(),s},e.argumentType=function(t,n,i,o){var s="Sys.ArgumentTypeException: ";s+=o||(n&&i?r.format("Object of type '{0}' cannot be converted to type '{1}'.",n.getName?n.getName():n,i.getName?i.getName():i):"Object cannot be converted to the required type."),t&&(s+="\n"+r.format("Parameter name: {0}",t));var a=e.create(s,{name:"Sys.ArgumentTypeException",paramName:t,actualType:n,expectedType:i});return a.popStackFrame(),a},e.argumentUndefined=function(t,n){var i="Sys.ArgumentUndefinedException: "+(n||"Value cannot be undefined.");t&&(i+="\n"+r.format("Parameter name: {0}",t));var o=e.create(i,{name:"Sys.ArgumentUndefinedException",paramName:t});return o.popStackFrame(),o},e.invalidOperation=function(t){var n="Sys.InvalidOperationException: "+(t||"Operation is not valid due to the current state of the object."),r=e.create(n,{name:"Sys.InvalidOperationException"});return r.popStackFrame(),r},e}();e.MsAjaxError=n;var r=function(){function e(){}return e.format=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];var r=e;return r.replace(/{(\d+)}/gm,(function(e,n){var r=parseInt(n,10);return void 0===t[r]?"{"+n+"}":t[r]}))},e.startsWith=function(e,t){return e.substr(0,t.length)===t},e}();e.MsAjaxString=r;var i=function(){function e(){}return e.trace=function(e){"undefined"!=typeof Debug&&Debug.writeln&&Debug.writeln(e),window.console&&window.console.log&&window.console.log(e),window.opera&&window.opera.postError&&window.opera.postError(e),window.debugService&&window.debugService.trace&&window.debugService.trace(e);var t=document.getElementById("TraceConsole");t&&"TEXTAREA"===t.tagName.toUpperCase()&&(t.innerHTML+=e+"\n")},e}();if(e.MsAjaxDebug=i,!OsfMsAjaxFactory.isMsAjaxLoaded()){var o=function(e,t,n){void 0!==e.__typeName&&null!==e.__typeName||(e.__typeName=t),void 0!==e.__class&&null!==e.__class||(e.__class=n)};o(Function,"Function",!0),o(Error,"Error",!0),o(Object,"Object",!0),o(String,"String",!0),o(Boolean,"Boolean",!0),o(Date,"Date",!0),o(Number,"Number",!0),o(RegExp,"RegExp",!0),o(Array,"Array",!0),Function.createCallback||(Function.createCallback=function(e,t){var n=Function._validateParams(arguments,[{name:"method",type:Function},{name:"context",mayBeNull:!0}]);if(n)throw n;return function(){var n=arguments.length;if(n>0){for(var r=[],i=0;i<n;i++)r[i]=arguments[i];return r[n]=t,e.apply(this,r)}return e.call(this,t)}}),Function.createDelegate||(Function.createDelegate=function(e,t){var n=Function._validateParams(arguments,[{name:"instance",mayBeNull:!0},{name:"method",type:Function}]);if(n)throw n;return function(){return t.apply(e,arguments)}}),Function._validateParams||(Function._validateParams=function(e,t,n){var r,i=t.length;if(n=n||void 0===n,r=Function._validateParameterCount(e,t,n))return r.popStackFrame(),r;for(var o=0,s=e.length;o<s;o++){var a=t[Math.min(o,i-1)],l=a.name;if(a.parameterArray)l+="["+(o-i+1)+"]";else if(!n&&o>=i)break;if(r=Function._validateParameter(e[o],a,l))return r.popStackFrame(),r}return null}),Function._validateParameterCount||(Function._validateParameterCount=function(e,t,r){var i,o,s=t.length,a=e.length;if(a<s){var l=s;for(i=0;i<s;i++){var u=t[i];(u.optional||u.parameterArray)&&l--}a<l&&(o=!0)}else if(r&&a>s)for(o=!0,i=0;i<s;i++)if(t[i].parameterArray){o=!1;break}if(o){var c=n.parameterCount();return c.popStackFrame(),c}return null}),Function._validateParameter||(Function._validateParameter=function(e,t,n){var r,i=t.type,o=!!t.integer,s=!!t.domElement,a=!!t.mayBeNull;if(r=Function._validateParameterType(e,i,o,s,a,n))return r.popStackFrame(),r;var l=t.elementType,u=!!t.elementMayBeNull;if(i===Array&&null!=e&&(l||!u))for(var c=!!t.elementInteger,p=!!t.elementDomElement,f=0;f<e.length;f++){var h=e[f];if(r=Function._validateParameterType(h,l,c,p,u,n+"["+f+"]"))return r.popStackFrame(),r}return null}),Function._validateParameterType||(Function._validateParameterType=function(t,n,r,i,o,s){var a;return void 0===t?o?null:((a=e.MsAjaxError.argumentUndefined(s)).popStackFrame(),a):null===t?o?null:((a=e.MsAjaxError.argumentNull(s)).popStackFrame(),a):n&&!e.MsAjaxTypeHelper.isInstanceOfType(n,t)?((a=e.MsAjaxError.argumentType(s,typeof t,n)).popStackFrame(),a):null}),window.Type||(window.Type=Function),Type.registerNamespace||(Type.registerNamespace=function(e){for(var t=e.split("."),n=window,r=0;r<t.length;r++)n[t[r]]=n[t[r]]||{},n=n[t[r]]}),Type.prototype.registerClass||(Type.prototype.registerClass=function(e){({})}),"undefined"==typeof Sys&&Type.registerNamespace("Sys"),Error.prototype.popStackFrame||(Error.prototype.popStackFrame=function(){if(0!==arguments.length)throw n.parameterCount();if(void 0!==this.stack&&null!==this.stack&&void 0!==this.fileName&&null!==this.fileName&&void 0!==this.lineNumber&&null!==this.lineNumber){for(var e=this.stack.split("\n"),t=e[0],r=this.fileName+":"+this.lineNumber;null!=t&&-1===t.indexOf(r);)e.shift(),t=e[0];var i=e[1];if(null!=i){var o=i.match(/@(.*):(\d+)$/);null!=o&&(this.fileName=o[1],this.lineNumber=parseInt(o[2]),e.shift(),this.stack=e.join("\n"))}}}),OsfMsAjaxFactory.msAjaxError=n,OsfMsAjaxFactory.msAjaxString=r,OsfMsAjaxFactory.msAjaxDebug=i}}(OfficeExt||(OfficeExt={})),OSF.OUtil.setNamespace("SafeArray",OSF.DDA),OSF.DDA.SafeArray.Response={Status:0,Payload:1},OSF.DDA.SafeArray.UniqueArguments={Offset:"offset",Run:"run",BindingSpecificData:"bindingSpecificData",MergedCellGuid:"{66e7831f-81b2-42e2-823c-89e872d541b3}"},OSF.OUtil.setNamespace("Delegate",OSF.DDA.SafeArray),OSF.DDA.SafeArray.Delegate._onException=function(e,t){var n,r=e.number;if(r)switch(r){case-2146828218:n=OSF.DDA.ErrorCodeManager.errorCodes.ooeNoCapability;break;case-2147467259:n=t.dispId==OSF.DDA.EventDispId.dispidDialogMessageReceivedEvent?OSF.DDA.ErrorCodeManager.errorCodes.ooeDialogAlreadyOpened:OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError;break;case-2146828283:case-2147209089:n=OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidParam;break;case-2147208704:n=OSF.DDA.ErrorCodeManager.errorCodes.ooeTooManyIncompleteRequests;break;case-2146827850:default:n=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError}t.onComplete&&t.onComplete(n||OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError)},OSF.DDA.SafeArray.Delegate._onExceptionSyncMethod=function(e,t){var n,r=e.number;if(r)switch(r){case-2146828218:n=OSF.DDA.ErrorCodeManager.errorCodes.ooeNoCapability;break;case-2146827850:default:n=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError}return n||OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError},OSF.DDA.SafeArray.Delegate.SpecialProcessor=function(){function e(e){var t;try{var n=e.ubound(1),r=e.ubound(2);if(e=e.toArray(),1==n&&1==r)t=[e];else{t=[];for(var i=0;i<n;i++){for(var o=[],s=0;s<r;s++){var a=e[i*r+s];a!=OSF.DDA.SafeArray.UniqueArguments.MergedCellGuid&&o.push(a)}o.length>0&&t.push(o)}}}catch(e){}return t}var t={};t[Microsoft.Office.WebExtension.Parameters.Data]={toHost:function(e){if(OSF.DDA.TableDataProperties&&"string"!=typeof e&&void 0!==e[OSF.DDA.TableDataProperties.TableRows]){var t=[];t[0]=e[OSF.DDA.TableDataProperties.TableRows],t[1]=e[OSF.DDA.TableDataProperties.TableHeaders],e=t}return e},fromHost:function(t){var n;if(t.toArray)if(2===t.dimensions())n=e(t);else{var r=t.toArray();2===r.length&&(null!=r[0]&&r[0].toArray||null!=r[1]&&r[1].toArray)?((n={})[OSF.DDA.TableDataProperties.TableRows]=e(r[0]),n[OSF.DDA.TableDataProperties.TableHeaders]=e(r[1])):n=r}else n=t;return n}},OSF.DDA.SafeArray.Delegate.SpecialProcessor.uber.constructor.call(this,[],t),this.unpack=function(e,n){var r;this.isComplexType(e)||OSF.DDA.ListType.isListType(e)?r=void 0!==n&&void 0!==n.toArray?n.toArray():n||{}:r=this.isDynamicType(e)?t[e].fromHost(n):n;return r}},OSF.OUtil.extend(OSF.DDA.SafeArray.Delegate.SpecialProcessor,OSF.DDA.SpecialProcessor),OSF.DDA.SafeArray.Delegate.ParameterMap=OSF.DDA.getDecoratedParameterMap(new OSF.DDA.SafeArray.Delegate.SpecialProcessor,[{type:Microsoft.Office.WebExtension.Parameters.ValueFormat,toHost:[{name:Microsoft.Office.WebExtension.ValueFormat.Unformatted,value:0},{name:Microsoft.Office.WebExtension.ValueFormat.Formatted,value:1}]},{type:Microsoft.Office.WebExtension.Parameters.FilterType,toHost:[{name:Microsoft.Office.WebExtension.FilterType.All,value:0}]}]),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.PropertyDescriptors.AsyncResultStatus,fromHost:[{name:Microsoft.Office.WebExtension.AsyncResultStatus.Succeeded,value:0},{name:Microsoft.Office.WebExtension.AsyncResultStatus.Failed,value:1}]}),OSF.DDA.SafeArray.Delegate.executeAsync=function(e){function t(e){var n=e;if(null!=e&&e.toArray){var r=e.toArray();n=new Array(r.length);for(var i=0;i<r.length;i++)n[i]=t(r[i])}return n}try{e.onCalling&&e.onCalling(),OSF.ClientHostController.execute(e.dispId,function e(t){var n=t;if(OSF.OUtil.isArray(t))for(var r=n.length,i=0;i<r;i++)n[i]=e(n[i]);else if(OSF.OUtil.isDate(t))n=t.getVarDate();else if("object"==typeof t&&!OSF.OUtil.isArray(t))for(var o in n=[],t)OSF.OUtil.isFunction(t[o])||(n[o]=e(t[o]));return n}(e.hostCallArgs),(function(n,r){var i,o;if("number"==typeof n?(i=[],o=n):o=(i=n.toArray())[OSF.DDA.SafeArray.Response.Status],o==OSF.DDA.ErrorCodeManager.errorCodes.ooeChunkResult)return null!=(s=t(s=i[OSF.DDA.SafeArray.Response.Payload]))&&(e._chunkResultData||(e._chunkResultData=new Array),e._chunkResultData[s[0]]=s[1]),!1;if(e.onReceiving&&e.onReceiving(),e.onComplete){var s;if(o==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess){if(i.length>2){s=[];for(var a=1;a<i.length;a++)s[a-1]=i[a]}else s=i[OSF.DDA.SafeArray.Response.Payload];if(e._chunkResultData&&null!=(s=t(s))){var l=s[s.length-1];e._chunkResultData.length==l?s[s.length-1]=e._chunkResultData:o=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError}}else s=i[OSF.DDA.SafeArray.Response.Payload];e.onComplete(o,s)}return!0}))}catch(t){OSF.DDA.SafeArray.Delegate._onException(t,e)}},OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent=function(e,t){var n=(new Date).getTime();return function(r){t.onReceiving&&t.onReceiving();var i=r.toArray?r.toArray()[OSF.DDA.SafeArray.Response.Status]:r;t.onComplete&&t.onComplete(i),OSF.AppTelemetry&&OSF.AppTelemetry.onRegisterDone(e,t.dispId,Math.abs((new Date).getTime()-n),i)}},OSF.DDA.SafeArray.Delegate.registerEventAsync=function(e){e.onCalling&&e.onCalling();var t=OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent(!0,e);try{OSF.ClientHostController.registerEvent(e.dispId,e.targetId,(function(t,n){e.onEvent&&e.onEvent(n),OSF.AppTelemetry&&OSF.AppTelemetry.onEventDone(e.dispId)}),t)}catch(t){OSF.DDA.SafeArray.Delegate._onException(t,e)}},OSF.DDA.SafeArray.Delegate.unregisterEventAsync=function(e){e.onCalling&&e.onCalling();var t=OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent(!1,e);try{OSF.ClientHostController.unregisterEvent(e.dispId,e.targetId,t)}catch(t){OSF.DDA.SafeArray.Delegate._onException(t,e)}},OSF.ClientMode={ReadWrite:0,ReadOnly:1},OSF.DDA.RichInitializationReason={1:Microsoft.Office.WebExtension.InitializationReason.Inserted,2:Microsoft.Office.WebExtension.InitializationReason.DocumentOpened},OSF.InitializationHelper=function(e,t,n,r,i){this._hostInfo=e,this._webAppState=t,this._context=n,this._settings=r,this._hostFacade=i,this._initializeSettings=this.initializeSettings},OSF.InitializationHelper.prototype.deserializeSettings=function(e,t){var n=OSF.OUtil.getSessionStorage();if(n){var r=n.getItem(OSF._OfficeAppFactory.getCachedSessionSettingsKey());r?e=JSON.parse(r):(r=JSON.stringify(e),n.setItem(OSF._OfficeAppFactory.getCachedSessionSettingsKey(),r))}var i=OSF.DDA.SettingsManager.deserializeSettings(e);return t?new OSF.DDA.RefreshableSettings(i):new OSF.DDA.Settings(i)},OSF.InitializationHelper.prototype.saveAndSetDialogInfo=function(e){},OSF.InitializationHelper.prototype.setAgaveHostCommunication=function(){},OSF.InitializationHelper.prototype.prepareRightBeforeWebExtensionInitialize=function(e){this.prepareApiSurface(e),Microsoft.Office.WebExtension.initialize(this.getInitializationReason(e))},OSF.InitializationHelper.prototype.prepareApiSurface=function(e){var t,n,r=new OSF.DDA.License(e.get_eToken()),i=OSF.DDA.OfficeTheme&&OSF.DDA.OfficeTheme.getOfficeTheme?OSF.DDA.OfficeTheme.getOfficeTheme:null;e.get_isDialog()?OSF.DDA.UI.ChildUI&&(e.ui=new OSF.DDA.UI.ChildUI):OSF.DDA.UI.ParentUI&&(e.ui=new OSF.DDA.UI.ParentUI,OfficeExt.Container&&OSF.DDA.DispIdHost.addAsyncMethods(e.ui,[OSF.DDA.AsyncMethodNames.CloseContainerAsync])),OSF.DDA.OpenBrowser&&OSF.DDA.DispIdHost.addAsyncMethods(e.ui,[OSF.DDA.AsyncMethodNames.OpenBrowserWindow]),OSF.DDA.ExecuteFeature&&OSF.DDA.DispIdHost.addAsyncMethods(e.ui,[OSF.DDA.AsyncMethodNames.ExecuteFeature]),OSF.DDA.QueryFeature&&OSF.DDA.DispIdHost.addAsyncMethods(e.ui,[OSF.DDA.AsyncMethodNames.QueryFeature]),OSF.DDA.Auth&&(e.auth=new OSF.DDA.Auth,OSF.DDA.DispIdHost.addAsyncMethods(e.auth,[OSF.DDA.AsyncMethodNames.GetAccessTokenAsync])),OSF._OfficeAppFactory.setContext(new OSF.DDA.Context(e,e.doc,r,null,i)),t=OSF.DDA.DispIdHost.getClientDelegateMethods,n=OSF.DDA.SafeArray.Delegate.ParameterMap,OSF._OfficeAppFactory.setHostFacade(new OSF.DDA.DispIdHost.Facade(t,n))},OSF.InitializationHelper.prototype.getInitializationReason=function(e){return OSF.DDA.RichInitializationReason[e.get_reason()]},OSF.DDA.DispIdHost.getClientDelegateMethods=function(e){var t={};if(t[OSF.DDA.DispIdHost.Delegates.ExecuteAsync]=OSF.DDA.SafeArray.Delegate.executeAsync,t[OSF.DDA.DispIdHost.Delegates.RegisterEventAsync]=OSF.DDA.SafeArray.Delegate.registerEventAsync,t[OSF.DDA.DispIdHost.Delegates.UnregisterEventAsync]=OSF.DDA.SafeArray.Delegate.unregisterEventAsync,t[OSF.DDA.DispIdHost.Delegates.OpenDialog]=OSF.DDA.SafeArray.Delegate.openDialog,t[OSF.DDA.DispIdHost.Delegates.CloseDialog]=OSF.DDA.SafeArray.Delegate.closeDialog,t[OSF.DDA.DispIdHost.Delegates.MessageParent]=OSF.DDA.SafeArray.Delegate.messageParent,t[OSF.DDA.DispIdHost.Delegates.SendMessage]=OSF.DDA.SafeArray.Delegate.sendMessage,OSF.DDA.AsyncMethodNames.RefreshAsync&&e==OSF.DDA.AsyncMethodNames.RefreshAsync.id){t[OSF.DDA.DispIdHost.Delegates.ExecuteAsync]=OSF.DDA.ClientSettingsManager.getSettingsExecuteMethod((function(e,t,n){return"function"==typeof OSF.DDA.ClientSettingsManager.refresh?OSF.DDA.ClientSettingsManager.refresh(t,n):OSF.DDA.ClientSettingsManager.read(t,n)}))}if(OSF.DDA.AsyncMethodNames.SaveAsync&&e==OSF.DDA.AsyncMethodNames.SaveAsync.id){t[OSF.DDA.DispIdHost.Delegates.ExecuteAsync]=OSF.DDA.ClientSettingsManager.getSettingsExecuteMethod((function(e,t,n){return OSF.DDA.ClientSettingsManager.write(e[OSF.DDA.SettingsManager.SerializedSettings],e[Microsoft.Office.WebExtension.Parameters.OverwriteIfStale],t,n)}))}return t},function(e){var t=function(){function e(){}return e.prototype.execute=function(e,t,n){setTimeout((function(){window.external.Execute(e,t,n)}),0)},e.prototype.registerEvent=function(e,t,n,r){setTimeout((function(){window.external.RegisterEvent(e,t,n,r)}),0)},e.prototype.unregisterEvent=function(e,t,n){setTimeout((function(){window.external.UnregisterEvent(e,t,n)}),0)},e.prototype.openDialog=function(t,n,r,i){if(!e.popup||e.popup.closed){window.dialogAPIErrorCode=void 0;var o=JSON.parse(n),s=o.url;if(s){var a=s.split("#"),l="?";a[0].indexOf("?")>-1&&(l="&");var u="width="+screen.width*o.width/100+", height="+screen.height*o.height/100;a[0]=a[0].concat(l).concat("action=displayDialog");var c=a.join("#");if(e.popup=window.open(c,"",u),e.DialogEventListener=function(n){if(n.source==e.popup)try{var i=JSON.parse(n.data);i.dialogMessage&&r(t,[OSF.DialogMessageType.DialogMessageReceived,i.dialogMessage.messageContent])}catch(e){OsfMsAjaxFactory.msAjaxDebug.trace("messages received cannot be handlered. Message:"+n.data)}},null!=e.popup&&null==window.dialogAPIErrorCode)window.addEventListener("message",e.DialogEventListener),e.interval=window.setInterval((function(){try{(null==e.popup||e.popup.closed)&&(window.clearInterval(e.interval),window.removeEventListener("message",e.DialogEventListener),e.NotifyError=null,r(t,[OSF.DialogMessageType.DialogClosed]))}catch(e){OsfMsAjaxFactory.msAjaxDebug.trace("Error happened when popup window closed.")}}),500),e.NotifyError=function(e){r(t,[e])},i(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess);else{var p=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError;window.dialogAPIErrorCode&&(p=window.dialogAPIErrorCode),i(p)}}else i(OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidScheme)}else i(OSF.DDA.ErrorCodeManager.errorCodes.ooeDialogAlreadyOpened)},e.prototype.messageParent=function(e){var t=e[Microsoft.Office.WebExtension.Parameters.MessageToParent],n={dialogMessage:{messageType:OSF.DialogMessageType.DialogMessageReceived,messageContent:t}};window.opener.postMessage(JSON.stringify(n),window.location.origin)},e.prototype.closeDialog=function(t,n,r){e.popup?(e.interval&&window.clearInterval(e.interval),e.popup.close(),e.popup=null,window.removeEventListener("message",e.DialogEventListener),e.NotifyError=null,r(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess)):r(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError)},e.prototype.sendMessage=function(e){},e}();e.MacRichClientHostController=t}(OfficeExt||(OfficeExt={}));var OSFWebkit,OSFLog,Logger,OSFAriaLogger,OSFAppTelemetry,OSFPerfUtil;OSF=OSF||{};!function(e){var t=function(){function e(e){this.data=e,this.safeArrayFlag=this.isSafeArray(e)}return e.prototype.dimensions=function(){var e=0;return this.safeArrayFlag?e=this.data[0][0]:this.isArray()&&(e=2),e},e.prototype.getItem=function(){var e=null;e=this.safeArrayFlag?this.toArray():this.data;for(var t=0;t<arguments.length;t++)e=e[arguments[t]];return e},e.prototype.lbound=function(e){return 0},e.prototype.ubound=function(e){var t=0;if(this.safeArrayFlag)t=this.data[0][e];else if(this.isArray()){if(1==e)return this.data.length;if(2==e){if(OSF.OUtil.isArray(this.data[0]))return this.data[0].length;if(null!=this.data[0])return 1}}return t},e.prototype.toArray=function(){if(0==this.isArray())return this.data;for(var t=[],n=this.safeArrayFlag?1:0;n<this.data.length;n++){var r=this.data[n];this.isSafeArray(r)?t.push(new e(r)):t.push(r)}return t},e.prototype.isArray=function(){return OSF.OUtil.isArray(this.data)},e.prototype.isSafeArray=function(e){var t=!1;if(OSF.OUtil.isArray(e)&&OSF.OUtil.isArray(e[0])){var n=e[0],r=n[0];if(n.length!=r+1)return!1;for(var i=1,o=1;o<n.length;o++){var s=n[o];if(0==isFinite(s))return!1;i*=s}t=++i==e.length}return t},e}();e.WebkitSafeArray=t}(OSFWebkit||(OSFWebkit={})),function(e){!function(e){var t=null;e.agaveHostCallback=function(e,n){t.agaveHostCallback(e,n)},e.agaveHostEventCallback=function(e,n){t.agaveHostEventCallback(e,n)},e.GetScriptMessenger=function(){return null==t&&(t=r.IsSupportedByHost()?new r:new i("OSF.ScriptMessaging.agaveHostCallback","OSF.ScriptMessaging.agaveHostEventCallback")),t};var n=function(e,t,n){this.id=e,this.targetId=t,this.handler=n},r=function(){function e(){this.eventHandlers=[]}return e.IsSupportedByHost=function(){return!!window._osfHostAPI},e.prototype.invokeMethod=function(e,t,n,r){window._osfHostAPI(e,this.constructMessagingArgs(t,n),(function(e){var t=r(e);return void 0===t||!0===t}))},e.prototype.registerEvent=function(e,t,n,r,i,o){var s={id:n,targetId:r},a=window._osfHostRegisterEvent(e,this.constructMessagingArgs(t,s),(function(e){return i(e),!1}),(function(e){return o(e),!0}));this.eventHandlers.push({targetId:r,dispId:n,callbackId:a.eventCallbackId})},e.prototype.unregisterEvent=function(e,t,n,r,i){var o={id:n,targetId:r};window._osfHostAPI(e,this.constructMessagingArgs(t,o),(function(e){return i(e),!0}));for(var s=this.eventHandlers.length-1;s>=0;s--){var a=this.eventHandlers[s];a.dispId===n&&a.targetId===r&&(this.eventHandlers.splice(s,1),window._osfHostUnregisterCallback(a.callbackId))}},e.prototype.agaveHostCallback=function(e,t){},e.prototype.agaveHostEventCallback=function(e,t){},e.prototype.constructMessagingArgs=function(e,t){return{params:t,methodId:e}},e}(),i=function(){function e(t,n){this.callingIndex=0,this.callbackList={},this.eventHandlerList={},this.asyncMethodCallbackFunctionName=t,this.eventCallbackFunctionName=n,this.conversationId=e.getCurrentTimeMS().toString()}return e.prototype.invokeMethod=function(e,t,n,r){this.postWebkitMessage({},e,t,n,r)},e.prototype.registerEvent=function(e,t,r,i,o,s){var a={eventCallbackFunction:this.eventCallbackFunctionName},l={id:r,targetId:i},u=this.postWebkitMessage(a,e,t,l,s);this.eventHandlerList[u]=new n(r,i,o)},e.prototype.unregisterEvent=function(e,t,n,r,i){var o={id:n,targetId:r};for(var s in this.eventHandlerList)if(this.eventHandlerList.hasOwnProperty(s)){var a=this.eventHandlerList[s];a.id==n&&a.targetId==r&&delete this.eventHandlerList[s]}this.invokeMethod(e,t,o,i)},e.prototype.agaveHostCallback=function(e,t){var n=this.callbackList[e];if(n){var r=n(t);void 0!==r&&!0!==r||delete this.callbackList[e]}},e.prototype.agaveHostEventCallback=function(e,t){var n=this.eventHandlerList[e];n&&n.handler(t)},e.prototype.postWebkitMessage=function(t,n,r,i,o){t.methodId=r,t.params=i;var s="";o&&(s=this.generateCorrelationId(),this.callbackList[s]=o,t.callbackId=s,t.callbackFunction=this.asyncMethodCallbackFunctionName);var a=function(){window.webkit.messageHandlers[n].postMessage(JSON.stringify(t))},l=e.getCurrentTimeMS();return null==this.lastMessageTimestamp||l-this.lastMessageTimestamp>=e.MESSAGE_TIME_DELTA?(a(),this.lastMessageTimestamp=l):(this.lastMessageTimestamp+=e.MESSAGE_TIME_DELTA,setTimeout((function(){a()}),this.lastMessageTimestamp-l)),s},e.prototype.generateCorrelationId=function(){return++this.callingIndex,this.conversationId+this.callingIndex},e.getCurrentTimeMS=function(){return(new Date).getTime()},e.MESSAGE_TIME_DELTA=10,e}()}(e.ScriptMessaging||(e.ScriptMessaging={}))}(OSFWebkit||(OSFWebkit={})),OSF.ScriptMessaging=OSFWebkit.ScriptMessaging,function(e){e.MessageHandlerName="Agave",e.PopupMessageHandlerName="WefPopupHandler",function(e){e[e.Settings=0]="Settings",e[e.SolutionReferenceId=1]="SolutionReferenceId",e[e.AppType=2]="AppType",e[e.MajorVersion=3]="MajorVersion",e[e.MinorVersion=4]="MinorVersion",e[e.RevisionVersion=5]="RevisionVersion",e[e.APIVersionSequence=6]="APIVersionSequence",e[e.AppCapabilities=7]="AppCapabilities",e[e.APPUILocale=8]="APPUILocale",e[e.AppDataLocale=9]="AppDataLocale",e[e.BindingCount=10]="BindingCount",e[e.DocumentUrl=11]="DocumentUrl",e[e.ActivationMode=12]="ActivationMode",e[e.ControlIntegrationLevel=13]="ControlIntegrationLevel",e[e.SolutionToken=14]="SolutionToken",e[e.APISetVersion=15]="APISetVersion",e[e.CorrelationId=16]="CorrelationId",e[e.InstanceId=17]="InstanceId",e[e.TouchEnabled=18]="TouchEnabled",e[e.CommerceAllowed=19]="CommerceAllowed",e[e.RequirementMatrix=20]="RequirementMatrix",e[e.HostCustomMessage=21]="HostCustomMessage",e[e.HostFullVersion=22]="HostFullVersion",e[e.InitialDisplayMode=23]="InitialDisplayMode"}(e.AppContextProperties||(e.AppContextProperties={})),function(e){e[e.Execute=1]="Execute",e[e.RegisterEvent=2]="RegisterEvent",e[e.UnregisterEvent=3]="UnregisterEvent",e[e.WriteSettings=4]="WriteSettings",e[e.GetContext=5]="GetContext",e[e.SendMessage=6]="SendMessage",e[e.MessageParent=7]="MessageParent",e[e.SendMessage2=8]="SendMessage2",e[e.MessageParent2=9]="MessageParent2"}(e.MethodId||(e.MethodId={}));var t=function(){function t(e){this.hostScriptProxy=e,this.useFullDialogAPI=!!window._enableFullDialogAPI,this.osfSupportsDialogOrigin=!!window._osfEnableDialogOrigin}return t.prototype.execute=function(t,n,r){var i={id:t,apiArgs:n};this.hostScriptProxy.invokeMethod(OSF.Webkit.MessageHandlerName,OSF.Webkit.MethodId.Execute,i,(function(t){if(r){var n=[];if(OSF.OUtil.isArray(t))for(var i=0;i<t.length;i++){var o=t[i];OSF.OUtil.isArray(o)&&(o=new e.WebkitSafeArray(o)),n.unshift(o)}return r.apply(null,n)}}))},t.prototype.registerEvent=function(t,n,r,i){this.hostScriptProxy.registerEvent(OSF.Webkit.MessageHandlerName,OSF.Webkit.MethodId.RegisterEvent,t,n,(function(t){var n=t,i=0;OSF.OUtil.isArray(t)&&t.length>=2&&(n=t[0],i=t[1]),r&&r(i,new e.WebkitSafeArray(n))}),(function(t){if(i)return i(new e.WebkitSafeArray(t))}))},t.prototype.unregisterEvent=function(t,n,r){this.hostScriptProxy.unregisterEvent(OSF.Webkit.MessageHandlerName,OSF.Webkit.MethodId.UnregisterEvent,t,n,(function(t){return r(new e.WebkitSafeArray(t))}))},t.prototype.messageParent=function(e){if(this.useFullDialogAPI&&e){var t=e[Microsoft.Office.WebExtension.Parameters.MessageToParent];"boolean"==typeof t&&(!0===t?e[Microsoft.Office.WebExtension.Parameters.MessageToParent]="true":!1===t&&(e[Microsoft.Office.WebExtension.Parameters.MessageToParent]=""))}var n,r;if(this.osfSupportsDialogOrigin&&e[Microsoft.Office.WebExtension.Parameters.TargetOrigin]?(n=this.useFullDialogAPI?JSON.stringify(e):e[Microsoft.Office.WebExtension.Parameters.MessageToParent],r=OSF.Webkit.MethodId.MessageParent2):(n=e[Microsoft.Office.WebExtension.Parameters.MessageToParent],r=OSF.Webkit.MethodId.MessageParent),this.useFullDialogAPI)this.hostScriptProxy.invokeMethod(OSF.Webkit.MessageHandlerName,r,n,null);else{var i={dialogMessage:{messageType:OSF.DialogMessageType.DialogMessageReceived,messageContent:n}},o=r==OSF.Webkit.MethodId.MessageParent2?e[Microsoft.Office.WebExtension.Parameters.TargetOrigin]:window.location.origin;window.opener.postMessage(JSON.stringify(i),o)}},t.prototype.openDialog=function(e,n,r,i){if(this.useFullDialogAPI)this.registerEvent(e,n,r,i);else if(!t.popup||t.popup.closed){t.OpenDialogCallback=void 0;var o=JSON.parse(n),s=o.url;if(s){var a=s.split("#"),l="?";a[0].indexOf("?")>-1&&(l="&");var u="width="+screen.width*o.width/100+", height="+screen.height*o.height/100;a[0]=a[0].concat(l).concat("action=displayDialog");var c=a.join("#");t.popup=window.open(c,"",u),t.DialogEventListener=function(t){if(t.origin==window.location.origin)try{var n=JSON.parse(t.data);n.dialogMessage&&r(e,[OSF.DialogMessageType.DialogMessageReceived,n.dialogMessage.messageContent,t.origin])}catch(e){OsfMsAjaxFactory.msAjaxDebug.trace("messages received cannot be handlered. Message:"+t.data)}},t.OpenDialogCallback=function(e){e==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&(window.addEventListener("message",t.DialogEventListener),t.interval=window.setInterval(p,1e3),t.NotifyError=f),i(e)}}else i(OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidScheme)}else i(OSF.DDA.ErrorCodeManager.errorCodes.ooeDialogAlreadyOpened);function p(){try{(null==t.popup||t.popup.closed)&&(window.clearInterval(t.interval),window.removeEventListener("message",t.DialogEventListener),t.NotifyError=null,t.popup=null,r(e,[OSF.DialogMessageType.DialogClosed]))}catch(e){OsfMsAjaxFactory.msAjaxDebug.trace("Error happened when popup window closed.")}}function f(t){r(e,[t])}},t.prototype.closeDialog=function(e,n,r){this.useFullDialogAPI?this.unregisterEvent(e,n,r):t.popup?(t.interval&&window.clearInterval(t.interval),t.popup.close(),t.popup=null,window.removeEventListener("message",t.DialogEventListener),t.NotifyError=null,r(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess)):r(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError)},t.prototype.sendMessage=function(e){var t,n;this.osfSupportsDialogOrigin&&e[Microsoft.Office.WebExtension.Parameters.TargetOrigin]?(t=JSON.stringify(e),n=OSF.Webkit.MethodId.SendMessage2):(t=e[Microsoft.Office.WebExtension.Parameters.MessageContent],n=OSF.Webkit.MethodId.SendMessage,!isNaN(parseFloat(t))&&isFinite(t)&&(t=t.toString())),this.hostScriptProxy.invokeMethod(OSF.Webkit.MessageHandlerName,n,t,null)},t}();e.WebkitHostController=t}(OSFWebkit||(OSFWebkit={})),OSF.Webkit=OSFWebkit,OSF.initializeWebkitCommon=function(){OSF.DDA.ClientSettingsManager={getSettingsExecuteMethod:function(e){return function(t){var n=function(e,n){t.onReceiving&&t.onReceiving(),t.onComplete&&t.onComplete(e,n)};try{e(t.hostCallArgs,t.onCalling,n)}catch(e){n(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError,{name:Strings.OfficeOM.L_InternalError,message:e})}}},read:function(e,t){e&&e();OSF._OfficeAppFactory.getInitializationHelper().getAppContext(null,(function(e){t&&t(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess,e.get_settings())}))},write:function(e,t,n,r){var i={},o=[],s=[];for(var a in e)o.push(a),s.push(e[a]);i.keys=o,i.values=s,n&&n();OSF.ScriptMessaging.GetScriptMessenger().invokeMethod(OSF.Webkit.MessageHandlerName,OSF.Webkit.MethodId.WriteSettings,i,(function(e){r&&r(e[0],null)}))}},OSF.InitializationHelper.prototype.initializeSettings=function(e,t){var n=e.get_settings();return this.deserializeSettings(n,t)},OSF.InitializationHelper.prototype.getAppContext=function(e,t){var n;n=this._hostInfo.isDialog&&window.webkit.messageHandlers[OSF.Webkit.PopupMessageHandlerName]?OSF.Webkit.PopupMessageHandlerName:OSF.Webkit.MessageHandlerName,OSF.ScriptMessaging.GetScriptMessenger().invokeMethod(n,OSF.Webkit.MethodId.GetContext,[],(function(e){for(var n,r=OSF.Webkit.AppContextProperties,i=e[r.AppType],o=e[r.Settings],s={},a=o[0],l=o[1],u=0;u<a.length;u++)s[a[u]]=l[u];var c=e[r.SolutionReferenceId],p=e[r.MajorVersion],f=e[r.MinorVersion],h=e[r.AppCapabilities],d=e[r.APPUILocale],y=e[r.AppDataLocale],m=e[r.DocumentUrl],g=e[r.ActivationMode],_=e[r.ControlIntegrationLevel],b=e[r.SolutionToken];b=b?b.toString():"";var O=e[r.CorrelationId],v=e[r.InstanceId],S=e[r.TouchEnabled],C=e[r.CommerceAllowed],P=e[r.RequirementMatrix],A=e[r.HostCustomMessage],F=e[r.HostFullVersion],D=e[r.InitialDisplayMode];n=new OSF.OfficeAppContext(c,i,p,d,y,m,h,s,g,_,b,O,v,S,C,f,P,A,F,void 0,void 0,void 0,void 0,void 0,void 0,void 0,D),OSF.AppTelemetry&&OSF.AppTelemetry.initialize(n),t(n)}))}},function(e){!function(t){var n=function(){function e(){this._osfOfficeTheme=null,this._osfOfficeThemeTimeStamp=null}return e.prototype.getOfficeTheme=function(){if(OSF.DDA._OsfControlContext){if(this._osfOfficeTheme&&this._osfOfficeThemeTimeStamp&&(new Date).getTime()-this._osfOfficeThemeTimeStamp<e._osfOfficeThemeCacheValidPeriod)OSF.AppTelemetry&&OSF.AppTelemetry.onPropertyDone("GetOfficeThemeInfo",0);else{var t=(new Date).getTime(),n=OSF.DDA._OsfControlContext.GetOfficeThemeInfo(),r=(new Date).getTime();for(var i in OSF.AppTelemetry&&OSF.AppTelemetry.onPropertyDone("GetOfficeThemeInfo",Math.abs(r-t)),this._osfOfficeTheme=JSON.parse(n),this._osfOfficeTheme)this._osfOfficeTheme[i]=OSF.OUtil.convertIntToCssHexColor(this._osfOfficeTheme[i]);this._osfOfficeThemeTimeStamp=r}return this._osfOfficeTheme}},e.instance=function(){return null==e._instance&&(e._instance=new e),e._instance},e._osfOfficeThemeCacheValidPeriod=5e3,e._instance=null,e}();t.OfficeThemeManager=n,OSF.OUtil.setNamespace("OfficeTheme",OSF.DDA),OSF.DDA.OfficeTheme.getOfficeTheme=e.OfficeTheme.OfficeThemeManager.instance().getOfficeTheme}(e.OfficeTheme||(e.OfficeTheme={}))}(OfficeExt||(OfficeExt={})),OSF.initializeRichCommon=function(){OSF.DDA.ClientSettingsManager={getSettingsExecuteMethod:function(e){return function(t){var n;try{n=e(t.hostCallArgs,t.onCalling,(function(e,n){t.onReceiving&&t.onReceiving(),t.onComplete&&t.onComplete(e,n)}))}catch(e){var r=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError;n={name:Strings.OfficeOM.L_InternalError,message:e},t.onComplete&&t.onComplete(r,n)}}},read:function(e,t){var n=[],r=[];e&&e(),"undefined"!=typeof OsfOMToken&&OsfOMToken?OSF.DDA._OsfControlContext.GetSettings(OsfOMToken).Read(n,r):OSF.DDA._OsfControlContext.GetSettings().Read(n,r);for(var i={},o=0;o<n.length;o++)i[n[o]]=r[o];return t&&t(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess,i),i},write:function(e,t,n,r){var i,o=[],s=[];for(var a in e)o.push(a),s.push(e[a]);n&&n(),void 0!==(i="undefined"!=typeof OsfOMToken&&OsfOMToken?OSF.DDA._OsfControlContext.GetSettings(OsfOMToken):OSF.DDA._OsfControlContext.GetSettings()).WriteAsync?i.WriteAsync(o,s,r):(i.Write(o,s),r&&r(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess))},refresh:function(e,t){var n,r=[],i=[];e&&e(),n="undefined"!=typeof OsfOMToken&&OsfOMToken?OSF.DDA._OsfControlContext.GetSettings(OsfOMToken):OSF.DDA._OsfControlContext.GetSettings();var o=function(){n.Read(r,i);for(var e={},o=0;o<r.length;o++)e[r[o]]=i[o];t&&t(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess,e)};n.RefreshAsync?n.RefreshAsync((function(){o()})):o()}},OSF.InitializationHelper.prototype.initializeSettings=function(e){var t=OSF.DDA.ClientSettingsManager.read();return this.deserializeSettings(t,e)},OSF.InitializationHelper.prototype.getAppContext=function(e,t){var n,r,i="Warning: Office.js is loaded outside of Office client";try{if(!window.external||!OSF.OUtil.externalNativeFunctionExists(typeof window.external.GetContext))return void OsfMsAjaxFactory.msAjaxDebug.trace(i);r=OSF.DDA._OsfControlContext=window.external.GetContext()}catch(e){return void OsfMsAjaxFactory.msAjaxDebug.trace(i)}var o,s,a,l,u,c,p,f,h,d,y=r.GetAppType(),m=r.GetSolutionRef(),g=r.GetAppVersionMajor(),_=r.GetAppVersionMinor(),b=r.GetAppUILocale(),O=r.GetAppDataLocale(),v=r.GetDocUrl(),S=r.GetAppCapabilities(),C=r.GetActivationMode(),P=r.GetControlIntegrationLevel();try{o=r.GetSolutionToken()}catch(e){}if(OSF.OUtil.externalNativeFunctionExists(typeof r.GetCorrelationId)&&(s=r.GetCorrelationId()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetInstanceId)&&(a=r.GetInstanceId()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetTouchEnabled)&&(l=r.GetTouchEnabled()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetCommerceAllowed)&&(u=r.GetCommerceAllowed()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetSupportedMatrix)&&(c=r.GetSupportedMatrix()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetHostCustomMessage)&&(p=r.GetHostCustomMessage()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetHostFullVersion)&&(f=r.GetHostFullVersion()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetDialogRequirementMatrix)&&(h=r.GetDialogRequirementMatrix()),OSF.OUtil.externalNativeFunctionExists(typeof r.GetFeaturesForSolution))try{var A=r.GetFeaturesForSolution();A&&(d=JSON.parse(A))}catch(e){OsfMsAjaxFactory.msAjaxDebug.trace("Exception while creating the SDX FeatureGates object. Details: "+e)}var F=0;OSF.OUtil.externalNativeFunctionExists(typeof r.GetInitialDisplayMode)&&(F=r.GetInitialDisplayMode()),o=o?o.toString():"",n=new OSF.OfficeAppContext(m,y,g,b,O,v,S,[],C,P,o,s,a,l,u,_,c,p,f,void 0,void 0,void 0,void 0,h,d,void 0,F),OSF.AppTelemetry&&OSF.AppTelemetry.initialize(n),t(n)}},window.webkit&&window.webkit.messageHandlers&&window.webkit.messageHandlers.Agave?(OSF.ClientHostController=new OSFWebkit.WebkitHostController(OSF.ScriptMessaging.GetScriptMessenger()),OSF.initializeWebkitCommon(),OSF.InitializationHelper.prototype.initializeMacSettings=function(e,t){return this._initializeSettings(e,t)}):(OSF.ClientHostController=new OfficeExt.MacRichClientHostController,OSF.initializeRichCommon(),OSF.InitializationHelper.prototype.initializeMacSettings=function(e,t){return this._initializeSettings(t)}),function(e){var t=function(){function e(e){this._table=e,this._fields={}}return Object.defineProperty(e.prototype,"Fields",{get:function(){return this._fields},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"Table",{get:function(){return this._table},enumerable:!0,configurable:!0}),e.prototype.SerializeFields=function(){},e.prototype.SetSerializedField=function(e,t){null!=t&&(this._serializedFields[e]=t.toString())},e.prototype.SerializeRow=function(){return this._serializedFields={},this.SetSerializedField("Table",this._table),this.SerializeFields(),JSON.stringify(this._serializedFields)},e}();e.BaseUsageData=t;var n=function(e){function t(){return e.call(this,"AppActivated")||this}return __extends(t,e),Object.defineProperty(t.prototype,"CorrelationId",{get:function(){return this.Fields.CorrelationId},set:function(e){this.Fields.CorrelationId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"SessionId",{get:function(){return this.Fields.SessionId},set:function(e){this.Fields.SessionId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppId",{get:function(){return this.Fields.AppId},set:function(e){this.Fields.AppId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppInstanceId",{get:function(){return this.Fields.AppInstanceId},set:function(e){this.Fields.AppInstanceId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppURL",{get:function(){return this.Fields.AppURL},set:function(e){this.Fields.AppURL=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AssetId",{get:function(){return this.Fields.AssetId},set:function(e){this.Fields.AssetId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"Browser",{get:function(){return this.Fields.Browser},set:function(e){this.Fields.Browser=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"UserId",{get:function(){return this.Fields.UserId},set:function(e){this.Fields.UserId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"Host",{get:function(){return this.Fields.Host},set:function(e){this.Fields.Host=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"HostVersion",{get:function(){return this.Fields.HostVersion},set:function(e){this.Fields.HostVersion=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"ClientId",{get:function(){return this.Fields.ClientId},set:function(e){this.Fields.ClientId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppSizeWidth",{get:function(){return this.Fields.AppSizeWidth},set:function(e){this.Fields.AppSizeWidth=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppSizeHeight",{get:function(){return this.Fields.AppSizeHeight},set:function(e){this.Fields.AppSizeHeight=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"Message",{get:function(){return this.Fields.Message},set:function(e){this.Fields.Message=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"DocUrl",{get:function(){return this.Fields.DocUrl},set:function(e){this.Fields.DocUrl=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"OfficeJSVersion",{get:function(){return this.Fields.OfficeJSVersion},set:function(e){this.Fields.OfficeJSVersion=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"HostJSVersion",{get:function(){return this.Fields.HostJSVersion},set:function(e){this.Fields.HostJSVersion=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"WacHostEnvironment",{get:function(){return this.Fields.WacHostEnvironment},set:function(e){this.Fields.WacHostEnvironment=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"IsFromWacAutomation",{get:function(){return this.Fields.IsFromWacAutomation},set:function(e){this.Fields.IsFromWacAutomation=e},enumerable:!0,configurable:!0}),t.prototype.SerializeFields=function(){this.SetSerializedField("CorrelationId",this.CorrelationId),this.SetSerializedField("SessionId",this.SessionId),this.SetSerializedField("AppId",this.AppId),this.SetSerializedField("AppInstanceId",this.AppInstanceId),this.SetSerializedField("AppURL",this.AppURL),this.SetSerializedField("AssetId",this.AssetId),this.SetSerializedField("Browser",this.Browser),this.SetSerializedField("UserId",this.UserId),this.SetSerializedField("Host",this.Host),this.SetSerializedField("HostVersion",this.HostVersion),this.SetSerializedField("ClientId",this.ClientId),this.SetSerializedField("AppSizeWidth",this.AppSizeWidth),this.SetSerializedField("AppSizeHeight",this.AppSizeHeight),this.SetSerializedField("Message",this.Message),this.SetSerializedField("DocUrl",this.DocUrl),this.SetSerializedField("OfficeJSVersion",this.OfficeJSVersion),this.SetSerializedField("HostJSVersion",this.HostJSVersion),this.SetSerializedField("WacHostEnvironment",this.WacHostEnvironment),this.SetSerializedField("IsFromWacAutomation",this.IsFromWacAutomation)},t}(t);e.AppActivatedUsageData=n;var r=function(e){function t(){return e.call(this,"ScriptLoad")||this}return __extends(t,e),Object.defineProperty(t.prototype,"CorrelationId",{get:function(){return this.Fields.CorrelationId},set:function(e){this.Fields.CorrelationId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"SessionId",{get:function(){return this.Fields.SessionId},set:function(e){this.Fields.SessionId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"ScriptId",{get:function(){return this.Fields.ScriptId},set:function(e){this.Fields.ScriptId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"StartTime",{get:function(){return this.Fields.StartTime},set:function(e){this.Fields.StartTime=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"ResponseTime",{get:function(){return this.Fields.ResponseTime},set:function(e){this.Fields.ResponseTime=e},enumerable:!0,configurable:!0}),t.prototype.SerializeFields=function(){this.SetSerializedField("CorrelationId",this.CorrelationId),this.SetSerializedField("SessionId",this.SessionId),this.SetSerializedField("ScriptId",this.ScriptId),this.SetSerializedField("StartTime",this.StartTime),this.SetSerializedField("ResponseTime",this.ResponseTime)},t}(t);e.ScriptLoadUsageData=r;var i=function(e){function t(){return e.call(this,"AppClosed")||this}return __extends(t,e),Object.defineProperty(t.prototype,"CorrelationId",{get:function(){return this.Fields.CorrelationId},set:function(e){this.Fields.CorrelationId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"SessionId",{get:function(){return this.Fields.SessionId},set:function(e){this.Fields.SessionId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"FocusTime",{get:function(){return this.Fields.FocusTime},set:function(e){this.Fields.FocusTime=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppSizeFinalWidth",{get:function(){return this.Fields.AppSizeFinalWidth},set:function(e){this.Fields.AppSizeFinalWidth=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"AppSizeFinalHeight",{get:function(){return this.Fields.AppSizeFinalHeight},set:function(e){this.Fields.AppSizeFinalHeight=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"OpenTime",{get:function(){return this.Fields.OpenTime},set:function(e){this.Fields.OpenTime=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"CloseMethod",{get:function(){return this.Fields.CloseMethod},set:function(e){this.Fields.CloseMethod=e},enumerable:!0,configurable:!0}),t.prototype.SerializeFields=function(){this.SetSerializedField("CorrelationId",this.CorrelationId),this.SetSerializedField("SessionId",this.SessionId),this.SetSerializedField("FocusTime",this.FocusTime),this.SetSerializedField("AppSizeFinalWidth",this.AppSizeFinalWidth),this.SetSerializedField("AppSizeFinalHeight",this.AppSizeFinalHeight),this.SetSerializedField("OpenTime",this.OpenTime),this.SetSerializedField("CloseMethod",this.CloseMethod)},t}(t);e.AppClosedUsageData=i;var o=function(e){function t(){return e.call(this,"APIUsage")||this}return __extends(t,e),Object.defineProperty(t.prototype,"CorrelationId",{get:function(){return this.Fields.CorrelationId},set:function(e){this.Fields.CorrelationId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"SessionId",{get:function(){return this.Fields.SessionId},set:function(e){this.Fields.SessionId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"APIType",{get:function(){return this.Fields.APIType},set:function(e){this.Fields.APIType=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"APIID",{get:function(){return this.Fields.APIID},set:function(e){this.Fields.APIID=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"Parameters",{get:function(){return this.Fields.Parameters},set:function(e){this.Fields.Parameters=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"ResponseTime",{get:function(){return this.Fields.ResponseTime},set:function(e){this.Fields.ResponseTime=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"ErrorType",{get:function(){return this.Fields.ErrorType},set:function(e){this.Fields.ErrorType=e},enumerable:!0,configurable:!0}),t.prototype.SerializeFields=function(){this.SetSerializedField("CorrelationId",this.CorrelationId),this.SetSerializedField("SessionId",this.SessionId),this.SetSerializedField("APIType",this.APIType),this.SetSerializedField("APIID",this.APIID),this.SetSerializedField("Parameters",this.Parameters),this.SetSerializedField("ResponseTime",this.ResponseTime),this.SetSerializedField("ErrorType",this.ErrorType)},t}(t);e.APIUsageUsageData=o;var s=function(e){function t(){return e.call(this,"AppInitialization")||this}return __extends(t,e),Object.defineProperty(t.prototype,"CorrelationId",{get:function(){return this.Fields.CorrelationId},set:function(e){this.Fields.CorrelationId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"SessionId",{get:function(){return this.Fields.SessionId},set:function(e){this.Fields.SessionId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"SuccessCode",{get:function(){return this.Fields.SuccessCode},set:function(e){this.Fields.SuccessCode=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"Message",{get:function(){return this.Fields.Message},set:function(e){this.Fields.Message=e},enumerable:!0,configurable:!0}),t.prototype.SerializeFields=function(){this.SetSerializedField("CorrelationId",this.CorrelationId),this.SetSerializedField("SessionId",this.SessionId),this.SetSerializedField("SuccessCode",this.SuccessCode),this.SetSerializedField("Message",this.Message)},t}(t);e.AppInitializationUsageData=s;var a=function(e){function t(){return e.call(this,"CheckWACHost")||this}return __extends(t,e),Object.defineProperty(t.prototype,"isWacKnownHost",{get:function(){return this.Fields.isWacKnownHost},set:function(e){this.Fields.isWacKnownHost=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"instanceId",{get:function(){return this.Fields.instanceId},set:function(e){this.Fields.instanceId=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"hostType",{get:function(){return this.Fields.hostType},set:function(e){this.Fields.hostType=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"hostPlatform",{get:function(){return this.Fields.hostPlatform},set:function(e){this.Fields.hostPlatform=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"wacDomain",{get:function(){return this.Fields.wacDomain},set:function(e){this.Fields.wacDomain=e},enumerable:!0,configurable:!0}),t.prototype.SerializeFields=function(){this.SetSerializedField("isWacKnownHost",this.isWacKnownHost),this.SetSerializedField("instanceId",this.instanceId),this.SetSerializedField("hostType",this.hostType),this.SetSerializedField("hostPlatform",this.hostPlatform),this.SetSerializedField("wacDomain",this.wacDomain)},t}(t);e.CheckWACHostUsageData=a}(OSFLog||(OSFLog={})),function(e){"use strict";!function(e){e[e.info=0]="info",e[e.warning=1]="warning",e[e.error=2]="error"}(e.TraceLevel||(e.TraceLevel={})),function(e){e[e.none=0]="none",e[e.flush=1]="flush"}(e.SendFlag||(e.SendFlag={})),e.allowUploadingData=function(){},e.sendLog=function(e,t,n){};var t=function(){function e(){}return e.prototype.writeLog=function(e){},e.prototype.loadProxyFrame=function(){},e}();OSF.Logger||(OSF.Logger=e),e.ulsEndpoint=function(){try{return new t}catch(e){return null}}()}(Logger||(Logger={})),function(e){var t=[{name:"AppActivated",enabled:!0,critical:!0,points:[{name:"Browser",type:"string"},{name:"Message",type:"string"},{name:"Host",type:"string"},{name:"AppSizeWidth",type:"int64"},{name:"AppSizeHeight",type:"int64"},{name:"IsFromWacAutomation",type:"string"}]},{name:"ScriptLoad",enabled:!0,critical:!1,points:[{name:"ScriptId",type:"string"},{name:"StartTime",type:"double"},{name:"ResponseTime",type:"double"}]},{name:"APIUsage",enabled:function(){if(!OSF._OfficeAppFactory||!OSF._OfficeAppFactory.getHostInfo)return!1;var e=OSF._OfficeAppFactory.getHostInfo();if(!e)return!1;switch(e.hostType){case"outlook":switch(e.hostPlatform){case"mac":case"web":return!0;default:return!1}default:return!1}}(),critical:!1,points:[{name:"APIType",type:"string"},{name:"APIID",type:"int64"},{name:"Parameters",type:"string"},{name:"ResponseTime",type:"int64"},{name:"ErrorType",type:"int64"}]},{name:"AppInitialization",enabled:!0,critical:!1,points:[{name:"SuccessCode",type:"int64"},{name:"Message",type:"string"}]},{name:"AppClosed",enabled:!0,critical:!1,points:[{name:"FocusTime",type:"int64"},{name:"AppSizeFinalWidth",type:"int64"},{name:"AppSizeFinalHeight",type:"int64"},{name:"OpenTime",type:"int64"}]},{name:"CheckWACHost",enabled:!0,critical:!1,points:[{name:"isWacKnownHost",type:"int64"},{name:"solutionId",type:"string"},{name:"hostType",type:"string"},{name:"hostPlatform",type:"string"},{name:"correlationId",type:"string"}]}];function n(e,t){var n=void 0===t.rename?t.name:t.rename,r=void 0;switch(t.type){case"string":r=oteljs.makeStringDataField(n,e);break;case"double":"string"==typeof e&&(e=parseFloat(e)),r=oteljs.makeDoubleDataField(n,e);break;case"int64":"string"==typeof e&&(e=parseInt(e)),r=oteljs.makeInt64DataField(n,e);break;case"boolean":"string"==typeof e&&(e="true"===e),r=oteljs.makeBooleanDataField(n,e)}return r}function r(e){for(var n=0,r=t;n<r.length;n++){var i=r[n];if(i.name===e)return i}}function i(e,t){(function(e){var t=r(e);return void 0!==t&&t.enabled})(e)&&"undefined"!=typeof OTel&&OTel.OTelLogger.onTelemetryLoaded((function(){var i=function(e,t){var i=r(e);if(void 0!==i){for(var o=[],s=0,a=i.points;s<a.length;s++){var l=a[s],u=t[l.name];if(void 0!==u){var c=n(u,l);void 0!==c&&o.push(c)}}var p={dataCategories:oteljs.DataCategories.ProductServiceUsage};return i.critical&&(p.samplingPolicy=oteljs.SamplingPolicy.CriticalBusinessImpact),p.diagnosticLevel=oteljs.DiagnosticLevel.NecessaryServiceDataEvent,{eventName:"Office.Extensibility.OfficeJs."+e+"X",dataFields:o,eventFlags:p}}}(e,t);void 0!==i&&Microsoft.Office.WebExtension.sendTelemetryEvent(i)}))}var o=function(){function e(){}return e.prototype.getAriaCDNLocation=function(){return OSF._OfficeAppFactory.getLoadScriptHelper().getOfficeJsBasePath()+"ariatelemetry/aria-web-telemetry.js"},e.getInstance=function(){return void 0===e.AriaLoggerObj&&(e.AriaLoggerObj=new e),e.AriaLoggerObj},e.prototype.isIUsageData=function(e){return void 0!==e.Fields},e.prototype.shouldSendDirectToAria=function(e,t){var n;if(!e)return!1;if("win32"===e.toLowerCase())n=[16,0,11601];else{if("mac"!==e.toLowerCase())return!0;n=[16,28]}if(!t)return!1;for(var r=t.split("."),i=0;i<n.length&&i<r.length;i++){var o=parseInt(r[i],10);if(isNaN(o))return!1;if(o<n[i])return!0;if(o>n[i])return!1}return!1},e.prototype.isDirectToAriaEnabled=function(){if(void 0===this.EnableDirectToAria||null===this.EnableDirectToAria){var e=void 0,t=void 0;OSF._OfficeAppFactory&&OSF._OfficeAppFactory.getHostInfo&&(e=OSF._OfficeAppFactory.getHostInfo().hostPlatform),window.external&&void 0!==window.external.GetContext&&void 0!==window.external.GetContext().GetHostFullVersion&&(t=window.external.GetContext().GetHostFullVersion()),this.EnableDirectToAria=this.shouldSendDirectToAria(e,t)}return this.EnableDirectToAria},e.prototype.sendTelemetry=function(t,n){e.EnableSendingTelemetryWithLegacyAria&&this.isDirectToAriaEnabled()&&OSF.OUtil.loadScript(this.getAriaCDNLocation(),(function(){try{if(!this.ALogger){this.ALogger=AWTLogManager.initialize("db334b301e7b474db5e0f02f07c51a47-a1b5bc36-1bbe-482f-a64a-c2d9cb606706-7439")}var e=new AWTEventProperties;for(var r in e.setName("Office.Extensibility.OfficeJS."+t),n)"table"!==r.toLowerCase()&&e.setProperty(r,n[r]);var i=new Date;e.setProperty("Date",i.toISOString()),this.ALogger.logEvent(e)}catch(e){}}),1e3),e.EnableSendingTelemetryWithOTel&&i(t,n)},e.prototype.logData=function(e){this.isIUsageData(e)?this.sendTelemetry(e.Table,e.Fields):this.sendTelemetry(e.Table,e)},e.EnableSendingTelemetryWithOTel=!0,e.EnableSendingTelemetryWithLegacyAria=!1,e}();e.AriaLogger=o}(OSFAriaLogger||(OSFAriaLogger={})),function(e){"use strict";var t,n=OSF.OUtil.Guid.generateNewGuid(),r="",i=new RegExp("^https?://store\\.office(ppe|-int)?\\.com/","i"),o="PRIVATE";e.enableTelemetry=!0;var s=function(){};e.AppInfo=s;var a=function(e,t){this.name=e,this.handler=t},l=function(){function e(){this.clientIDKey="Office API client",this.logIdSetKey="Office App Log Id Set"}return e.prototype.getClientId=function(){var e=this.getValue(this.clientIDKey);return(!e||e.length<=0||e.length>40)&&(e=OSF.OUtil.Guid.generateNewGuid(),this.setValue(this.clientIDKey,e)),e},e.prototype.saveLog=function(e,t){var n=this.getValue(this.logIdSetKey);n=(n&&n.length>0?n+";":"")+e,this.setValue(this.logIdSetKey,n),this.setValue(e,t)},e.prototype.enumerateLog=function(e,t){var n=this.getValue(this.logIdSetKey);if(n){var r=n.split(";");for(var i in r){var o=r[i],s=this.getValue(o);s&&(e&&e(o,s),t&&this.remove(o))}t&&this.remove(this.logIdSetKey)}},e.prototype.getValue=function(e){var t=OSF.OUtil.getLocalStorage(),n="";return t&&(n=t.getItem(e)),n},e.prototype.setValue=function(e,t){var n=OSF.OUtil.getLocalStorage();n&&n.setItem(e,t)},e.prototype.remove=function(e){var t=OSF.OUtil.getLocalStorage();if(t)try{t.removeItem(e)}catch(e){}},e}(),u=function(){function t(){}return t.prototype.LogData=function(t){if(e.enableTelemetry)try{OSFAriaLogger.AriaLogger.getInstance().logData(t)}catch(e){}},t.prototype.LogRawData=function(t){if(e.enableTelemetry)try{OSFAriaLogger.AriaLogger.getInstance().logData(JSON.parse(t))}catch(e){}},t}();function c(e){return e&&(e=e.replace(/[{}]/g,"").toLowerCase()),e||""}function p(e,t){var i=new OSFLog.AppInitializationUsageData;i.CorrelationId=c(r),i.SessionId=n,i.SuccessCode=e?1:0,i.Message=t,(new u).LogData(i)}function f(){var e=0!=(OSF._OfficeAppFactory.getHostInfo().flags&OSF.HostInfoFlags.PublicAddin);return e||!!t&&d(OSF._OfficeAppFactory.getHostInfo().hostPlatform,t.hostVersion)}function h(e,t){return f()||t!==e?t:o}function d(e,t){var n=!1,r=/^(\d+)\.(\d+)\.(\d+)\.(\d+)$/.exec(t);if(r){var i=parseInt(r[1]),o=parseInt(r[2]),s=parseInt(r[3]);"win32"==e?(i<16||16==i&&s<14225)&&(n=!0):"mac"==e&&(i<16||16==i&&(o<52||52==o&&s<808))&&(n=!0)}return n}e.initialize=function(r){if(e.enableTelemetry&&!t){t=new s,r.get_hostFullVersion()?t.hostVersion=r.get_hostFullVersion():t.hostVersion=r.get_appVersion(),t.appId=f()?r.get_id():o,t.marketplaceType=r._marketplaceType,t.browser=window.navigator.userAgent,t.correlationId=c(r.get_correlationId()),t.clientId=(new l).getClientId(),t.appInstanceId=r.get_appInstanceId(),t.appInstanceId&&(t.appInstanceId=c(t.appInstanceId),t.appInstanceId=h(r.get_id(),t.appInstanceId)),t.message=r.get_hostCustomMessage(),t.officeJSVersion=OSF.ConstantNames.FileVersion,t.hostJSVersion="16.0.15407.10000",r._wacHostEnvironment&&(t.wacHostEnvironment=r._wacHostEnvironment),void 0!==r._isFromWacAutomation&&null!==r._isFromWacAutomation&&(t.isFromWacAutomation=r._isFromWacAutomation.toString().toLowerCase());var u=r.get_docUrl();t.docUrl=i.test(u)?u:"";var p=location.href;p&&(p=p.split("?")[0].split("#")[0]),t.appURL="",function(e,t){var n,r;t.assetId="",t.userId="";try{n=decodeURIComponent(e);var i=(r=(new DOMParser).parseFromString(n,"text/xml")).getElementsByTagName("t")[0].attributes.getNamedItem("cid"),o=r.getElementsByTagName("t")[0].attributes.getNamedItem("oid");i&&i.nodeValue?t.userId=i.nodeValue:o&&o.nodeValue&&(t.userId=o.nodeValue),t.assetId=r.getElementsByTagName("t")[0].attributes.getNamedItem("aid").nodeValue}catch(e){}finally{n=null,r=null,null}}(r.get_eToken(),t),t.sessionId=n,"undefined"!=typeof OTel&&OTel.OTelLogger.initialize(t),function(){var t=new Date,n=null,r=0,i=!1,o=function(){document.hasFocus()?null==n&&(n=new Date):n&&(r+=Math.abs((new Date).getTime()-n.getTime()),n=null)},s=[];s.push(new a("focus",o)),s.push(new a("blur",o)),s.push(new a("focusout",o)),s.push(new a("focusin",o));var l=function(){for(var o=0;o<s.length;o++)OSF.OUtil.removeEventListener(window,s[o].name,s[o].handler);s.length=0,i||(document.hasFocus()&&n&&(r+=Math.abs((new Date).getTime()-n.getTime()),n=null),e.onAppClosed(Math.abs((new Date).getTime()-t.getTime()),r),i=!0)};s.push(new a("beforeunload",l)),s.push(new a("unload",l));for(var u=0;u<s.length;u++)OSF.OUtil.addEventListener(window,s[u].name,s[u].handler);o()}(),e.onAppActivated()}},e.onAppActivated=function(){if(t){(new l).enumerateLog((function(e,t){return(new u).LogRawData(t)}),!0);var e=new OSFLog.AppActivatedUsageData;e.SessionId=n,e.AppId=t.appId,e.AssetId=t.assetId,e.AppURL="",e.UserId="",e.ClientId=t.clientId,e.Browser=t.browser,e.HostVersion=t.hostVersion,e.CorrelationId=c(t.correlationId),e.AppSizeWidth=window.innerWidth,e.AppSizeHeight=window.innerHeight,e.AppInstanceId=t.appInstanceId,e.Message=t.message,e.DocUrl=t.docUrl,e.OfficeJSVersion=t.officeJSVersion,e.HostJSVersion=t.hostJSVersion,t.wacHostEnvironment&&(e.WacHostEnvironment=t.wacHostEnvironment),void 0!==t.isFromWacAutomation&&null!==t.isFromWacAutomation&&(e.IsFromWacAutomation=t.isFromWacAutomation),(new u).LogData(e)}},e.onScriptDone=function(e,t,r,i){var o=new OSFLog.ScriptLoadUsageData;o.CorrelationId=c(i),o.SessionId=n,o.ScriptId=e,o.StartTime=t,o.ResponseTime=r,(new u).LogData(o)},e.onCallDone=function(e,i,o,s,a){if(t&&function(){if(!OSF._OfficeAppFactory||!OSF._OfficeAppFactory.getHostInfo)return!1;var e=OSF._OfficeAppFactory.getHostInfo();if(!e)return!1;switch(e.hostType){case"outlook":switch(e.hostPlatform){case"mac":case"web":return!0;default:return!1}default:return!1}}()&&function(e,t){if("method"===t)switch(e){case 3:case 4:case 38:case 37:case 10:case 12:return!0;default:return!1}return!1}(i,e)){var l=new OSFLog.APIUsageUsageData;l.CorrelationId=c(r),l.SessionId=n,l.APIType=e,l.APIID=i,l.Parameters=o,l.ResponseTime=s,l.ErrorType=a,(new u).LogData(l)}},e.onMethodDone=function(e,t,n,r){var i=null;if(t)if("number"==typeof t)i=String(t);else if("object"==typeof t)for(var o in t)null!==i?i+=",":i="","number"==typeof t[o]&&(i+=String(t[o]));else i="";OSF.AppTelemetry.onCallDone("method",e,i,n,r)},e.onPropertyDone=function(e,t){OSF.AppTelemetry.onCallDone("property",-1,e,t)},e.onCheckWACHost=function(e,t,n,r,i){var o=new OSFLog.CheckWACHostUsageData;o.isWacKnownHost=e,o.instanceId=t,o.hostType=n,o.hostPlatform=r,o.wacDomain="",(new u).LogData(o)},e.onEventDone=function(e,t){OSF.AppTelemetry.onCallDone("event",e,null,0,t)},e.onRegisterDone=function(e,t,n,r){OSF.AppTelemetry.onCallDone(e?"registerevent":"unregisterevent",t,null,n,r)},e.onAppClosed=function(e,i){if(t){var o=new OSFLog.AppClosedUsageData;o.CorrelationId=c(r),o.SessionId=n,o.FocusTime=i,o.OpenTime=e,o.AppSizeFinalWidth=window.innerWidth,o.AppSizeFinalHeight=window.innerHeight,(new l).saveLog(n,o.SerializeRow())}},e.setOsfControlAppCorrelationId=function(e){r=c(e)},e.doAppInitializationLogging=p,e.logAppCommonMessage=function(e){p(!1,e)},e.logAppException=function(e){p(!0,e)},e.canSendAddinId=f,e.getCompliantAppInstanceId=h,e._isComplianceExceptedHost=d,OSF.AppTelemetry=e}(OSFAppTelemetry||(OSFAppTelemetry={})),function(e){function t(e,t){if(t+="_Resource",void 0!==oteljs)return[oteljs.makeDoubleDataField(t+"_responseEnd",e.responseEnd),oteljs.makeDoubleDataField(t+"_responseStart",e.responseStart),oteljs.makeDoubleDataField(t+"_startTime",e.startTime),oteljs.makeDoubleDataField(t+"_transferSize",e.transferSize)]}e.sendPerformanceTelemetry=function(){var e,n;"undefined"!=typeof OTel&&OSF.AppTelemetry.enableTelemetry&&"undefined"!=typeof OSFPerformance&&"undefined"!=typeof performance&&performance.getEntriesByType&&(performance.getEntriesByType("resource").forEach((function(t){var r=t.name.toLowerCase();OSF.OUtil.stringEndsWith(r,OSFPerformance.hostSpecificFileName)?e=t:(OSF.OUtil.stringEndsWith(r,OSF.ConstantNames.OfficeDebugJS)||OSF.OUtil.stringEndsWith(r,OSF.ConstantNames.OfficeJS))&&(n=t)})),OTel.OTelLogger.onTelemetryLoaded((function(){var r=t(e,"HostJs");r=(r=r.concat(t(n,"OfficeJs"))).concat([oteljs.makeDoubleDataField("officeExecuteStartDate",OSFPerformance.officeExecuteStartDate),oteljs.makeDoubleDataField("officeExecuteStart",OSFPerformance.officeExecuteStart),oteljs.makeDoubleDataField("officeExecuteEnd",OSFPerformance.officeExecuteEnd),oteljs.makeDoubleDataField("hostInitializationStart",OSFPerformance.hostInitializationStart),oteljs.makeDoubleDataField("hostInitializationEnd",OSFPerformance.hostInitializationEnd),oteljs.makeDoubleDataField("totalJSHeapSize",OSFPerformance.totalJSHeapSize),oteljs.makeDoubleDataField("usedJSHeapSize",OSFPerformance.usedJSHeapSize),oteljs.makeDoubleDataField("jsHeapSizeLimit",OSFPerformance.jsHeapSizeLimit),oteljs.makeDoubleDataField("getAppContextStart",OSFPerformance.getAppContextStart),oteljs.makeDoubleDataField("getAppContextEnd",OSFPerformance.getAppContextEnd),oteljs.makeDoubleDataField("createOMEnd",OSFPerformance.createOMEnd),oteljs.makeDoubleDataField("officeOnReady",OSFPerformance.officeOnReady),oteljs.makeBooleanDataField("isSharedRuntime",0!=(OSF._OfficeAppFactory.getHostInfo().flags&OSF.HostInfoFlags.SharedApp))]),Microsoft.Office.WebExtension.sendTelemetryEvent({eventName:"Office.Extensibility.OfficeJs.JSPerformanceTelemetryV06",dataFields:r,eventFlags:{dataCategories:oteljs.DataCategories.ProductServiceUsage,diagnosticLevel:oteljs.DiagnosticLevel.NecessaryServiceDataEvent}})})))}}(OSFPerfUtil||(OSFPerfUtil={})),Microsoft.Office.WebExtension.EventType={},OSF.EventDispatch=function(e){if(this._eventHandlers={},this._objectEventHandlers={},this._queuedEventsArgs={},null!=e)for(var t=0;t<e.length;t++){var n=e[t];"objectDeleted"==n||"objectSelectionChanged"==n||"objectDataChanged"==n||"contentControlAdded"==n?this._objectEventHandlers[n]={}:this._eventHandlers[n]=[],this._queuedEventsArgs[n]=[]}},OSF.EventDispatch.prototype={getSupportedEvents:function(){var e=[];for(var t in this._eventHandlers)e.push(t);for(var t in this._objectEventHandlers)e.push(t);return e},supportsEvent:function(e){for(var t in this._eventHandlers)if(e==t)return!0;for(var t in this._objectEventHandlers)if(e==t)return!0;return!1},hasEventHandler:function(e,t){var n=this._eventHandlers[e];if(n&&n.length>0)for(var r=0;r<n.length;r++)if(n[r]===t)return!0;return!1},hasObjectEventHandler:function(e,t,n){var r=this._objectEventHandlers[e];if(null!=r)for(var i=r[t],o=0;null!=i&&o<i.length;o++)if(i[o]===n)return!0;return!1},addEventHandler:function(e,t){if("function"!=typeof t)return!1;var n=this._eventHandlers[e];return!(!n||this.hasEventHandler(e,t))&&(n.push(t),!0)},addObjectEventHandler:function(e,t,n){if("function"!=typeof n)return!1;var r=this._objectEventHandlers[e];return!(!r||this.hasObjectEventHandler(e,t,n))&&(null==r[t]&&(r[t]=[]),r[t].push(n),!0)},addEventHandlerAndFireQueuedEvent:function(e,t){var n=0==this._eventHandlers[e].length,r=this.addEventHandler(e,t);return n&&r&&this.fireQueuedEvent(e),r},removeEventHandler:function(e,t){var n=this._eventHandlers[e];if(n&&n.length>0)for(var r=0;r<n.length;r++)if(n[r]===t)return n.splice(r,1),!0;return!1},removeObjectEventHandler:function(e,t,n){var r=this._objectEventHandlers[e];if(null!=r)for(var i=r[t],o=0;null!=i&&o<i.length;o++)if(i[o]===n)return i.splice(o,1),!0;return!1},clearEventHandlers:function(e){return void 0!==this._eventHandlers[e]&&this._eventHandlers[e].length>0&&(this._eventHandlers[e]=[],!0)},clearObjectEventHandlers:function(e,t){return null!=this._objectEventHandlers[e]&&null!=this._objectEventHandlers[e][t]&&(this._objectEventHandlers[e][t]=[],!0)},getEventHandlerCount:function(e){return null!=this._eventHandlers[e]?this._eventHandlers[e].length:-1},getObjectEventHandlerCount:function(e,t){return null==this._objectEventHandlers[e]||null==this._objectEventHandlers[e][t]?0:this._objectEventHandlers[e][t].length},fireEvent:function(e){if(null==e.type)return!1;var t=e.type;if(t&&this._eventHandlers[t]){for(var n=this._eventHandlers[t],r=0;r<n.length;r++)n[r](e);return!0}return!1},fireObjectEvent:function(e,t){if(null==t.type)return!1;var n=t.type;if(n&&this._objectEventHandlers[n]){var r=this._objectEventHandlers[n][e];if(null!=r){for(var i=0;i<r.length;i++)r[i](t);return!0}}return!1},fireOrQueueEvent:function(e){var t=e.type;if(t&&this._eventHandlers[t]){var n=this._eventHandlers[t],r=this._queuedEventsArgs[t];return 0==n.length?r.push(e):this.fireEvent(e),!0}return!1},fireQueuedEvent:function(e){if(e&&this._eventHandlers[e]){var t=this._eventHandlers[e],n=this._queuedEventsArgs[e];if(t.length>0){for(var r=t[0];n.length>0;){r(n.shift())}return!0}}return!1},clearQueuedEvent:function(e){e&&this._eventHandlers[e]&&(this._queuedEventsArgs[e]&&(this._queuedEventsArgs[e]=[]))}},OSF.DDA.OMFactory=OSF.DDA.OMFactory||{},OSF.DDA.OMFactory.manufactureEventArgs=function(e,t,n){var r;switch(e){case Microsoft.Office.WebExtension.EventType.DocumentSelectionChanged:r=new OSF.DDA.DocumentSelectionChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.BindingSelectionChanged:r=new OSF.DDA.BindingSelectionChangedEventArgs(this.manufactureBinding(n,t.document),n[OSF.DDA.PropertyDescriptors.Subset]);break;case Microsoft.Office.WebExtension.EventType.BindingDataChanged:r=new OSF.DDA.BindingDataChangedEventArgs(this.manufactureBinding(n,t.document));break;case Microsoft.Office.WebExtension.EventType.SettingsChanged:r=new OSF.DDA.SettingsChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.ActiveViewChanged:r=new OSF.DDA.ActiveViewChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.OfficeThemeChanged:r=new OSF.DDA.Theming.OfficeThemeChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.DocumentThemeChanged:r=new OSF.DDA.Theming.DocumentThemeChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.AppCommandInvoked:r=OSF.DDA.AppCommand.AppCommandInvokedEventArgs.create(n),"outlook"==OSF._OfficeAppFactory.getHostInfo().hostType&&"mac"==OSF._OfficeAppFactory.getHostInfo().hostPlatform&&OSF.DDA.convertOlkAppointmentTimeToDateFormat(r);break;case Microsoft.Office.WebExtension.EventType.ObjectDeleted:case Microsoft.Office.WebExtension.EventType.ObjectSelectionChanged:case Microsoft.Office.WebExtension.EventType.ObjectDataChanged:case Microsoft.Office.WebExtension.EventType.ContentControlAdded:r=new OSF.DDA.ObjectEventArgs(e,n[Microsoft.Office.WebExtension.Parameters.Id]);break;case Microsoft.Office.WebExtension.EventType.RichApiMessage:r=new OSF.DDA.RichApiMessageEventArgs(e,n);break;case Microsoft.Office.WebExtension.EventType.DataNodeInserted:r=new OSF.DDA.NodeInsertedEventArgs(this.manufactureDataNode(n[OSF.DDA.DataNodeEventProperties.NewNode]),n[OSF.DDA.DataNodeEventProperties.InUndoRedo]);break;case Microsoft.Office.WebExtension.EventType.DataNodeReplaced:r=new OSF.DDA.NodeReplacedEventArgs(this.manufactureDataNode(n[OSF.DDA.DataNodeEventProperties.OldNode]),this.manufactureDataNode(n[OSF.DDA.DataNodeEventProperties.NewNode]),n[OSF.DDA.DataNodeEventProperties.InUndoRedo]);break;case Microsoft.Office.WebExtension.EventType.DataNodeDeleted:r=new OSF.DDA.NodeDeletedEventArgs(this.manufactureDataNode(n[OSF.DDA.DataNodeEventProperties.OldNode]),this.manufactureDataNode(n[OSF.DDA.DataNodeEventProperties.NextSiblingNode]),n[OSF.DDA.DataNodeEventProperties.InUndoRedo]);break;case Microsoft.Office.WebExtension.EventType.TaskSelectionChanged:r=new OSF.DDA.TaskSelectionChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.ResourceSelectionChanged:r=new OSF.DDA.ResourceSelectionChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.ViewSelectionChanged:r=new OSF.DDA.ViewSelectionChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.DialogMessageReceived:r=new OSF.DDA.DialogEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived:r=new OSF.DDA.DialogParentEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.ItemChanged:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkItemSelectedChangedEventArgs(n),t.initialize(r.initialData),"win32"!=OSF._OfficeAppFactory.getHostInfo().hostPlatform&&"mac"!=OSF._OfficeAppFactory.getHostInfo().hostPlatform||t.setCurrentItemNumber(r.itemNumber.itemNumber);break;case Microsoft.Office.WebExtension.EventType.RecipientsChanged:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkRecipientsChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.AppointmentTimeChanged:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkAppointmentTimeChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.RecurrenceChanged:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkRecurrenceChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.AttachmentsChanged:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkAttachmentsChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.EnhancedLocationsChanged:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkEnhancedLocationsChangedEventArgs(n);break;case Microsoft.Office.WebExtension.EventType.InfobarClicked:if("outlook"!=OSF._OfficeAppFactory.getHostInfo().hostType)throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e));r=new OSF.DDA.OlkInfobarClickedEventArgs(n);break;default:throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,e))}return r},OSF.DDA.AsyncMethodNames.addNames({AddHandlerAsync:"addHandlerAsync",RemoveHandlerAsync:"removeHandlerAsync"}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddHandlerAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.EventType,enum:Microsoft.Office.WebExtension.EventType,verify:function(e,t,n){return n.supportsEvent(e)}},{name:Microsoft.Office.WebExtension.Parameters.Handler,types:["function"]}],supportedOptions:[],privateStateCallbacks:[]}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.RemoveHandlerAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.EventType,enum:Microsoft.Office.WebExtension.EventType,verify:function(e,t,n){return n.supportsEvent(e)}}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.Handler,value:{types:["function","object"],defaultValue:null}}],privateStateCallbacks:[]}),OSF.DialogShownStatus={hasDialogShown:!1,isWindowDialog:!1},OSF.OUtil.augmentList(OSF.DDA.EventDescriptors,{DialogMessageReceivedEvent:"DialogMessageReceivedEvent"}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{DialogMessageReceived:"dialogMessageReceived",DialogEventReceived:"dialogEventReceived"}),OSF.OUtil.augmentList(OSF.DDA.PropertyDescriptors,{MessageType:"messageType",MessageContent:"messageContent",MessageOrigin:"messageOrigin"}),OSF.DDA.DialogEventType={},OSF.OUtil.augmentList(OSF.DDA.DialogEventType,{DialogClosed:"dialogClosed",NavigationFailed:"naviationFailed"}),OSF.DDA.AsyncMethodNames.addNames({DisplayDialogAsync:"displayDialogAsync",DisplayModalDialogAsync:"displayModalDialogAsync",CloseAsync:"close"}),OSF.DDA.SyncMethodNames.addNames({MessageParent:"messageParent",MessageChild:"messageChild",SendMessage:"sendMessage",AddMessageHandler:"addEventHandler"}),OSF.DDA.UI.ParentUI=function(){var e;if(e=null!=Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived?new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.DialogMessageReceived,Microsoft.Office.WebExtension.EventType.DialogEventReceived,Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived]):new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.DialogMessageReceived,Microsoft.Office.WebExtension.EventType.DialogEventReceived]),Microsoft.Office.WebExtension.FeatureGates&&Microsoft.Office.WebExtension.FeatureGates.OfficeJsDialogRefactor){var t=this,n=function(n,r){t[n]||OSF.OUtil.defineEnumerableProperty(t,n,{value:function(){var n=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.OpenDialog];n(arguments,e,t,r)}})};n(OSF.DDA.AsyncMethodNames.DisplayDialogAsync.displayName,!1),Microsoft.Office.WebExtension.FeatureGates.ModalWebDialogAPI&&n(OSF.DDA.AsyncMethodNames.DisplayModalDialogAsync.displayName,!0)}else{var r=OSF.DDA.AsyncMethodNames.DisplayDialogAsync.displayName,i=this;i[r]||OSF.OUtil.defineEnumerableProperty(i,r,{value:function(){var t=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.OpenDialog];t(arguments,e,i,!1)}})}OSF.OUtil.finalizeProperties(this)},OSF.DDA.UI.ChildUI=function(e){var t=OSF.DDA.SyncMethodNames.MessageParent.displayName,n=this;n[t]||OSF.OUtil.defineEnumerableProperty(n,t,{value:function(){var e=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.MessageParent];return e(arguments,n)}});var r=OSF.DDA.SyncMethodNames.AddMessageHandler.displayName;n[r]||void 0===OSF.DialogParentMessageEventDispatch||OSF.DDA.DispIdHost.addEventSupport(n,OSF.DialogParentMessageEventDispatch,e),OSF.OUtil.finalizeProperties(this)},OSF.DialogHandler=function(){},OSF.DDA.DialogEventArgs=function(e){e[OSF.DDA.PropertyDescriptors.MessageType]==OSF.DialogMessageType.DialogMessageReceived?OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.DialogMessageReceived},message:{value:e[OSF.DDA.PropertyDescriptors.MessageContent]},origin:{value:e[OSF.DDA.PropertyDescriptors.MessageOrigin]}}):OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.DialogEventReceived},error:{value:e[OSF.DDA.PropertyDescriptors.MessageType]}})},OSF.DDA.DialogParentEventArgs=function(e){OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived},message:{value:e[OSF.DDA.PropertyDescriptors.MessageContent]},origin:{value:e[OSF.DDA.PropertyDescriptors.MessageOrigin]}})};var DialogApiManager=function(){function e(){}return e.defineApi=function(e,t){OSF.DDA.AsyncMethodCalls.define({method:e,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Url,types:["string"]}],supportedOptions:t,privateStateCallbacks:[],onSucceeded:function(e,t,n){var r=e[Microsoft.Office.WebExtension.Parameters.Id],i=e[Microsoft.Office.WebExtension.Parameters.Data],o=new OSF.DialogHandler,s=OSF.DDA.AsyncMethodNames.CloseAsync.displayName;OSF.OUtil.defineEnumerableProperty(o,s,{value:function(){var e=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.CloseDialog];e(arguments,r,i,o)}});var a=OSF.DDA.SyncMethodNames.AddMessageHandler.displayName;if(OSF.OUtil.defineEnumerableProperty(o,a,{value:function(){var e=OSF.DDA.SyncMethodCalls[OSF.DDA.SyncMethodNames.AddMessageHandler.id],t=e.verifyAndExtractCall(arguments,o,i),n=t[Microsoft.Office.WebExtension.Parameters.EventType],r=t[Microsoft.Office.WebExtension.Parameters.Handler];return i.addEventHandlerAndFireQueuedEvent(n,r)}}),!0===OSF.DDA.UI.EnableSendMessageDialogAPI){var l=OSF.DDA.SyncMethodNames.SendMessage.displayName;OSF.OUtil.defineEnumerableProperty(o,l,{value:function(){var e=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.SendMessage];return e(arguments,i,o)}})}if(!0===OSF.DDA.UI.EnableMessageChildDialogAPI){var u=OSF.DDA.SyncMethodNames.MessageChild.displayName;OSF.OUtil.defineEnumerableProperty(o,u,{value:function(){var e=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.SendMessage];return e(arguments,i,o)}})}return o},checkCallArgs:function(e,t,n){return e[Microsoft.Office.WebExtension.Parameters.Width]<=0&&(e[Microsoft.Office.WebExtension.Parameters.Width]=1),!e[Microsoft.Office.WebExtension.Parameters.UseDeviceIndependentPixels]&&e[Microsoft.Office.WebExtension.Parameters.Width]>100&&(e[Microsoft.Office.WebExtension.Parameters.Width]=99),e[Microsoft.Office.WebExtension.Parameters.Height]<=0&&(e[Microsoft.Office.WebExtension.Parameters.Height]=1),!e[Microsoft.Office.WebExtension.Parameters.UseDeviceIndependentPixels]&&e[Microsoft.Office.WebExtension.Parameters.Height]>100&&(e[Microsoft.Office.WebExtension.Parameters.Height]=99),e[Microsoft.Office.WebExtension.Parameters.RequireHTTPs]||(e[Microsoft.Office.WebExtension.Parameters.RequireHTTPs]=!0),e}})},e.messageChildRichApiBridge=function(){if(!0===OSF.DDA.UI.EnableMessageChildDialogAPI){var e=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.SendMessage];return e(arguments,null,null)}},e.initOnce=function(){e.defineApi(OSF.DDA.AsyncMethodNames.DisplayDialogAsync,e.displayDialogAsyncApiSupportedOptions),e.defineApi(OSF.DDA.AsyncMethodNames.DisplayModalDialogAsync,e.displayModalDialogAsyncApiSupportedOptions)},e.displayDialogAsyncApiSupportedOptions=[{name:Microsoft.Office.WebExtension.Parameters.Width,value:{types:["number"],defaultValue:99}},{name:Microsoft.Office.WebExtension.Parameters.Height,value:{types:["number"],defaultValue:99}},{name:Microsoft.Office.WebExtension.Parameters.RequireHTTPs,value:{types:["boolean"],defaultValue:!0}},{name:Microsoft.Office.WebExtension.Parameters.DisplayInIframe,value:{types:["boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.HideTitle,value:{types:["boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.UseDeviceIndependentPixels,value:{types:["boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.PromptBeforeOpen,value:{types:["boolean"],defaultValue:!0}},{name:Microsoft.Office.WebExtension.Parameters.EnforceAppDomain,value:{types:["boolean"],defaultValue:!0}},{name:Microsoft.Office.WebExtension.Parameters.UrlNoHostInfo,value:{types:["boolean"],defaultValue:!1}}],e.displayModalDialogAsyncApiSupportedOptions=e.displayDialogAsyncApiSupportedOptions.concat([{name:"abortWhenParentIsMinimized",value:{types:["boolean"],defaultValue:!1}},{name:"abortWhenDocIsInactive",value:{types:["boolean"],defaultValue:!1}}]),e}();DialogApiManager.initOnce(),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.CloseAsync,requiredArguments:[],supportedOptions:[],privateStateCallbacks:[]}),OSF.DDA.SyncMethodCalls.define({method:OSF.DDA.SyncMethodNames.MessageParent,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.MessageToParent,types:["string","number","boolean"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.TargetOrigin,value:{types:["string"],defaultValue:""}}]}),OSF.DDA.SyncMethodCalls.define({method:OSF.DDA.SyncMethodNames.AddMessageHandler,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.EventType,enum:Microsoft.Office.WebExtension.EventType,verify:function(e,t,n){return n.supportsEvent(e)}},{name:Microsoft.Office.WebExtension.Parameters.Handler,types:["function"]}],supportedOptions:[]}),OSF.DDA.SyncMethodCalls.define({method:OSF.DDA.SyncMethodNames.SendMessage,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.MessageContent,types:["string"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.TargetOrigin,value:{types:["string"],defaultValue:""}}],privateStateCallbacks:[]}),OSF.DDA.SafeArray.Delegate.openDialog=function(e){try{e.onCalling&&e.onCalling();var t=OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent(!0,e);OSF.ClientHostController.openDialog(e.dispId,e.targetId,(function(t,n){e.onEvent&&e.onEvent(n),OSF.AppTelemetry&&OSF.AppTelemetry.onEventDone(e.dispId)}),t)}catch(t){OSF.DDA.SafeArray.Delegate._onException(t,e)}},OSF.DDA.SafeArray.Delegate.closeDialog=function(e){e.onCalling&&e.onCalling();var t=OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent(!1,e);try{OSF.ClientHostController.closeDialog(e.dispId,e.targetId,t)}catch(t){OSF.DDA.SafeArray.Delegate._onException(t,e)}},OSF.DDA.SafeArray.Delegate.messageParent=function(e){try{e.onCalling&&e.onCalling();var t=(new Date).getTime(),n=OSF.ClientHostController.messageParent(e.hostCallArgs);return e.onReceiving&&e.onReceiving(),OSF.AppTelemetry&&OSF.AppTelemetry.onMethodDone(e.dispId,e.hostCallArgs,Math.abs((new Date).getTime()-t),n),n}catch(e){return OSF.DDA.SafeArray.Delegate._onExceptionSyncMethod(e)}},OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidDialogMessageReceivedEvent,fromHost:[{name:OSF.DDA.EventDescriptors.DialogMessageReceivedEvent,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDescriptors.DialogMessageReceivedEvent,fromHost:[{name:OSF.DDA.PropertyDescriptors.MessageType,value:0},{name:OSF.DDA.PropertyDescriptors.MessageContent,value:1},{name:OSF.DDA.PropertyDescriptors.MessageOrigin,value:2}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.sendMessage=function(e){try{e.onCalling&&e.onCalling();(new Date).getTime();var t=OSF.ClientHostController.sendMessage(e.hostCallArgs);return e.onReceiving&&e.onReceiving(),t}catch(e){return OSF.DDA.SafeArray.Delegate._onExceptionSyncMethod(e)}},OSF.OUtil.augmentList(OSF.DDA.EventDescriptors,{DialogParentMessageReceivedEvent:"DialogParentMessageReceivedEvent"}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{DialogParentMessageReceived:"dialogParentMessageReceived",DialogParentEventReceived:"dialogParentEventReceived"}),OSF.DialogParentMessageEventDispatch=new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.DialogParentMessageReceived,Microsoft.Office.WebExtension.EventType.DialogParentEventReceived]),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidDialogParentMessageReceivedEvent,fromHost:[{name:OSF.DDA.EventDescriptors.DialogParentMessageReceivedEvent,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDescriptors.DialogParentMessageReceivedEvent,fromHost:[{name:OSF.DDA.PropertyDescriptors.MessageType,value:0},{name:OSF.DDA.PropertyDescriptors.MessageContent,value:1},{name:OSF.DDA.PropertyDescriptors.MessageOrigin,value:2}],isComplexType:!0}),OSF.DDA.UI.EnableMessageChildDialogAPI=!0,Microsoft.Office.WebExtension.TableData=function(e,t){function n(e){if(null==e||null==e)return null;try{for(var t=OSF.DDA.DataCoercion.findArrayDimensionality(e,2);t<2;t++)e=[e];return e}catch(e){}}OSF.OUtil.defineEnumerableProperties(this,{headers:{get:function(){return t},set:function(e){t=n(e)}},rows:{get:function(){return e},set:function(t){e=null==t||OSF.OUtil.isArray(t)&&0==t.length?[]:n(t)}}}),this.headers=t,this.rows=e},OSF.DDA.OMFactory=OSF.DDA.OMFactory||{},OSF.DDA.OMFactory.manufactureTableData=function(e){return new Microsoft.Office.WebExtension.TableData(e[OSF.DDA.TableDataProperties.TableRows],e[OSF.DDA.TableDataProperties.TableHeaders])},Microsoft.Office.WebExtension.CoercionType={Text:"text",Matrix:"matrix",Table:"table"},OSF.DDA.DataCoercion={findArrayDimensionality:function(e){if(OSF.OUtil.isArray(e)){for(var t=0,n=0;n<e.length;n++)t=Math.max(t,OSF.DDA.DataCoercion.findArrayDimensionality(e[n]));return t+1}return 0},getCoercionDefaultForBinding:function(e){switch(e){case Microsoft.Office.WebExtension.BindingType.Matrix:return Microsoft.Office.WebExtension.CoercionType.Matrix;case Microsoft.Office.WebExtension.BindingType.Table:return Microsoft.Office.WebExtension.CoercionType.Table;case Microsoft.Office.WebExtension.BindingType.Text:default:return Microsoft.Office.WebExtension.CoercionType.Text}},getBindingDefaultForCoercion:function(e){switch(e){case Microsoft.Office.WebExtension.CoercionType.Matrix:return Microsoft.Office.WebExtension.BindingType.Matrix;case Microsoft.Office.WebExtension.CoercionType.Table:return Microsoft.Office.WebExtension.BindingType.Table;case Microsoft.Office.WebExtension.CoercionType.Text:case Microsoft.Office.WebExtension.CoercionType.Html:case Microsoft.Office.WebExtension.CoercionType.Ooxml:default:return Microsoft.Office.WebExtension.BindingType.Text}},determineCoercionType:function(e){if(null==e||null==e)return null;var t=null,n=typeof e;if(void 0!==e.rows)t=Microsoft.Office.WebExtension.CoercionType.Table;else if(OSF.OUtil.isArray(e))t=Microsoft.Office.WebExtension.CoercionType.Matrix;else{if("string"!=n&&"number"!=n&&"boolean"!=n&&!OSF.OUtil.isDate(e))throw OSF.DDA.ErrorCodeManager.errorCodes.ooeUnsupportedDataObject;t=Microsoft.Office.WebExtension.CoercionType.Text}return t},coerceData:function(e,t,n){return(n=n||OSF.DDA.DataCoercion.determineCoercionType(e))&&n!=t&&(OSF.OUtil.writeProfilerMark(OSF.InternalPerfMarker.DataCoercionBegin),e=OSF.DDA.DataCoercion._coerceDataFromTable(t,OSF.DDA.DataCoercion._coerceDataToTable(e,n)),OSF.OUtil.writeProfilerMark(OSF.InternalPerfMarker.DataCoercionEnd)),e},_matrixToText:function(e){if(1==e.length&&1==e[0].length)return""+e[0][0];for(var t="",n=0;n<e.length;n++)t+=e[n].join("\t")+"\n";return t.substring(0,t.length-1)},_textToMatrix:function(e){for(var t=e.split("\n"),n=0;n<t.length;n++)t[n]=t[n].split("\t");return t},_tableToText:function(e){var t="";null!=e.headers&&(t=OSF.DDA.DataCoercion._matrixToText([e.headers])+"\n");var n=OSF.DDA.DataCoercion._matrixToText(e.rows);return""==n&&(t=t.substring(0,t.length-1)),t+n},_tableToMatrix:function(e){var t=e.rows;return null!=e.headers&&t.unshift(e.headers),t},_coerceDataFromTable:function(e,t){var n;switch(e){case Microsoft.Office.WebExtension.CoercionType.Table:n=t;break;case Microsoft.Office.WebExtension.CoercionType.Matrix:n=OSF.DDA.DataCoercion._tableToMatrix(t);break;case Microsoft.Office.WebExtension.CoercionType.SlideRange:n=null,OSF.DDA.OMFactory.manufactureSlideRange&&(n=OSF.DDA.OMFactory.manufactureSlideRange(OSF.DDA.DataCoercion._tableToText(t))),null==n&&(n=OSF.DDA.DataCoercion._tableToText(t));break;case Microsoft.Office.WebExtension.CoercionType.Text:case Microsoft.Office.WebExtension.CoercionType.Html:case Microsoft.Office.WebExtension.CoercionType.Ooxml:default:n=OSF.DDA.DataCoercion._tableToText(t)}return n},_coerceDataToTable:function(e,t){var n;switch(null==t&&(t=OSF.DDA.DataCoercion.determineCoercionType(e)),t){case Microsoft.Office.WebExtension.CoercionType.Table:n=e;break;case Microsoft.Office.WebExtension.CoercionType.Matrix:n=new Microsoft.Office.WebExtension.TableData(e);break;case Microsoft.Office.WebExtension.CoercionType.Text:case Microsoft.Office.WebExtension.CoercionType.Html:case Microsoft.Office.WebExtension.CoercionType.Ooxml:default:n=new Microsoft.Office.WebExtension.TableData(OSF.DDA.DataCoercion._textToMatrix(e))}return n}},OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:Microsoft.Office.WebExtension.Parameters.CoercionType,toHost:[{name:Microsoft.Office.WebExtension.CoercionType.Text,value:0},{name:Microsoft.Office.WebExtension.CoercionType.Matrix,value:1},{name:Microsoft.Office.WebExtension.CoercionType.Table,value:2}]}),OSF.DDA.AsyncMethodNames.addNames({GetSelectedDataAsync:"getSelectedDataAsync",SetSelectedDataAsync:"setSelectedDataAsync"}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetSelectedDataAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,enum:Microsoft.Office.WebExtension.CoercionType}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.ValueFormat,value:{enum:Microsoft.Office.WebExtension.ValueFormat,defaultValue:Microsoft.Office.WebExtension.ValueFormat.Unformatted}},{name:Microsoft.Office.WebExtension.Parameters.FilterType,value:{enum:Microsoft.Office.WebExtension.FilterType,defaultValue:Microsoft.Office.WebExtension.FilterType.All}}],privateStateCallbacks:[],onSucceeded:function(e,t,n){var r=e[Microsoft.Office.WebExtension.Parameters.Data];return OSF.DDA.TableDataProperties&&r&&(null!=r[OSF.DDA.TableDataProperties.TableRows]||null!=r[OSF.DDA.TableDataProperties.TableHeaders])&&(r=OSF.DDA.OMFactory.manufactureTableData(r)),null==(r=OSF.DDA.DataCoercion.coerceData(r,n[Microsoft.Office.WebExtension.Parameters.CoercionType]))?null:r}}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetSelectedDataAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["string","object","number","boolean"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:{enum:Microsoft.Office.WebExtension.CoercionType,calculate:function(e){return OSF.DDA.DataCoercion.determineCoercionType(e[Microsoft.Office.WebExtension.Parameters.Data])}}},{name:Microsoft.Office.WebExtension.Parameters.ImageLeft,value:{types:["number","boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.ImageTop,value:{types:["number","boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.ImageWidth,value:{types:["number","boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.ImageHeight,value:{types:["number","boolean"],defaultValue:!1}}],privateStateCallbacks:[]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetSelectedDataMethod,fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:0},{name:Microsoft.Office.WebExtension.Parameters.ValueFormat,value:1},{name:Microsoft.Office.WebExtension.Parameters.FilterType,value:2}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSetSelectedDataMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:0},{name:Microsoft.Office.WebExtension.Parameters.Data,value:1},{name:Microsoft.Office.WebExtension.Parameters.ImageLeft,value:2},{name:Microsoft.Office.WebExtension.Parameters.ImageTop,value:3},{name:Microsoft.Office.WebExtension.Parameters.ImageWidth,value:4},{name:Microsoft.Office.WebExtension.Parameters.ImageHeight,value:5}]}),OSF.DDA.SettingsManager={SerializedSettings:"serializedSettings",RefreshingSettings:"refreshingSettings",DateJSONPrefix:"Date(",DataJSONSuffix:")",serializeSettings:function(e){return OSF.OUtil.serializeSettings(e)},deserializeSettings:function(e){return OSF.OUtil.deserializeSettings(e)}},OSF.DDA.Settings=function(e){e=e||{};var t=function(e){var t=OSF.OUtil.getSessionStorage();if(t){var n=OSF.DDA.SettingsManager.serializeSettings(e),r=JSON?JSON.stringify(n):Sys.Serialization.JavaScriptSerializer.serialize(n);t.setItem(OSF._OfficeAppFactory.getCachedSessionSettingsKey(),r)}};OSF.OUtil.defineEnumerableProperties(this,{get:{value:function(t){var n=Function._validateParams(arguments,[{name:"name",type:String,mayBeNull:!1}]);if(n)throw n;var r=e[t];return void 0===r?null:r}},set:{value:function(n,r){var i=Function._validateParams(arguments,[{name:"name",type:String,mayBeNull:!1},{name:"value",mayBeNull:!0}]);if(i)throw i;e[n]=r,t(e)}},remove:{value:function(n){var r=Function._validateParams(arguments,[{name:"name",type:String,mayBeNull:!1}]);if(r)throw r;delete e[n],t(e)}}}),OSF.DDA.DispIdHost.addAsyncMethods(this,[OSF.DDA.AsyncMethodNames.SaveAsync],e)},OSF.DDA.RefreshableSettings=function(e){OSF.DDA.RefreshableSettings.uber.constructor.call(this,e),OSF.DDA.DispIdHost.addAsyncMethods(this,[OSF.DDA.AsyncMethodNames.RefreshAsync],e),OSF.DDA.DispIdHost.addEventSupport(this,new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.SettingsChanged]))},OSF.OUtil.extend(OSF.DDA.RefreshableSettings,OSF.DDA.Settings),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{SettingsChanged:"settingsChanged"}),OSF.DDA.SettingsChangedEventArgs=function(e){OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.SettingsChanged},settings:{value:e}})},OSF.DDA.AsyncMethodNames.addNames({RefreshAsync:"refreshAsync",SaveAsync:"saveAsync"}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.RefreshAsync,requiredArguments:[],supportedOptions:[],privateStateCallbacks:[{name:OSF.DDA.SettingsManager.RefreshingSettings,value:function(e,t){return t}}],onSucceeded:function(e,t,n){var r=e[OSF.DDA.SettingsManager.SerializedSettings],i=OSF.DDA.SettingsManager.deserializeSettings(r),o=n[OSF.DDA.SettingsManager.RefreshingSettings];for(var s in o)t.remove(s);for(var s in i)t.set(s,i[s]);return t}}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SaveAsync,requiredArguments:[],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.OverwriteIfStale,value:{types:["boolean"],defaultValue:!0}}],privateStateCallbacks:[{name:OSF.DDA.SettingsManager.SerializedSettings,value:function(e,t){return OSF.DDA.SettingsManager.serializeSettings(t)}}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidLoadSettingsMethod,fromHost:[{name:OSF.DDA.SettingsManager.SerializedSettings,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSaveSettingsMethod,toHost:[{name:OSF.DDA.SettingsManager.SerializedSettings,value:OSF.DDA.SettingsManager.SerializedSettings},{name:Microsoft.Office.WebExtension.Parameters.OverwriteIfStale,value:Microsoft.Office.WebExtension.Parameters.OverwriteIfStale}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidSettingsChangedEvent}),Microsoft.Office.WebExtension.BindingType={Table:"table",Text:"text",Matrix:"matrix"},OSF.DDA.BindingProperties={Id:"BindingId",Type:Microsoft.Office.WebExtension.Parameters.BindingType},OSF.OUtil.augmentList(OSF.DDA.ListDescriptors,{BindingList:"BindingList"}),OSF.OUtil.augmentList(OSF.DDA.PropertyDescriptors,{Subset:"subset",BindingProperties:"BindingProperties"}),OSF.DDA.ListType.setListType(OSF.DDA.ListDescriptors.BindingList,OSF.DDA.PropertyDescriptors.BindingProperties),OSF.DDA.BindingPromise=function(e,t){this._id=e,OSF.OUtil.defineEnumerableProperty(this,"onFail",{get:function(){return t},set:function(e){var n=typeof e;if("undefined"!=n&&"function"!=n)throw OSF.OUtil.formatString(Strings.OfficeOM.L_CallbackNotAFunction,n);t=e}})},OSF.DDA.BindingPromise.prototype={_fetch:function(e){if(this.binding)e&&e(this.binding);else if(!this._binding){var t=this;Microsoft.Office.WebExtension.context.document.bindings.getByIdAsync(this._id,(function(n){n.status==Microsoft.Office.WebExtension.AsyncResultStatus.Succeeded?(OSF.OUtil.defineEnumerableProperty(t,"binding",{value:n.value}),e&&e(t.binding)):t.onFail&&t.onFail(n)}))}return this},getDataAsync:function(){var e=arguments;return this._fetch((function(t){t.getDataAsync.apply(t,e)})),this},setDataAsync:function(){var e=arguments;return this._fetch((function(t){t.setDataAsync.apply(t,e)})),this},addHandlerAsync:function(){var e=arguments;return this._fetch((function(t){t.addHandlerAsync.apply(t,e)})),this},removeHandlerAsync:function(){var e=arguments;return this._fetch((function(t){t.removeHandlerAsync.apply(t,e)})),this}},OSF.DDA.BindingFacade=function(e){this._eventDispatches=[],OSF.OUtil.defineEnumerableProperty(this,"document",{value:e});var t=OSF.DDA.AsyncMethodNames;OSF.DDA.DispIdHost.addAsyncMethods(this,[t.AddFromSelectionAsync,t.AddFromNamedItemAsync,t.GetAllAsync,t.GetByIdAsync,t.ReleaseByIdAsync])},OSF.DDA.UnknownBinding=function(e,t){OSF.OUtil.defineEnumerableProperties(this,{document:{value:t},id:{value:e}})},OSF.DDA.Binding=function(e,t){OSF.OUtil.defineEnumerableProperties(this,{document:{value:t},id:{value:e}});var n=OSF.DDA.AsyncMethodNames;OSF.DDA.DispIdHost.addAsyncMethods(this,[n.GetDataAsync,n.SetDataAsync]);var r=Microsoft.Office.WebExtension.EventType,i=t.bindings._eventDispatches;i[e]||(i[e]=new OSF.EventDispatch([r.BindingSelectionChanged,r.BindingDataChanged]));var o=i[e];OSF.DDA.DispIdHost.addEventSupport(this,o)},OSF.DDA.generateBindingId=function(){return"UnnamedBinding_"+OSF.OUtil.getUniqueId()+"_"+(new Date).getTime()},OSF.DDA.OMFactory=OSF.DDA.OMFactory||{},OSF.DDA.OMFactory.manufactureBinding=function(e,t){var n,r=e[OSF.DDA.BindingProperties.Id],i=e[OSF.DDA.BindingProperties.RowCount],o=e[OSF.DDA.BindingProperties.ColumnCount],s=e[OSF.DDA.BindingProperties.HasHeaders];switch(e[OSF.DDA.BindingProperties.Type]){case Microsoft.Office.WebExtension.BindingType.Text:n=new OSF.DDA.TextBinding(r,t);break;case Microsoft.Office.WebExtension.BindingType.Matrix:n=new OSF.DDA.MatrixBinding(r,t,i,o);break;case Microsoft.Office.WebExtension.BindingType.Table:n=new(OSF.DDA.ExcelDocument&&Microsoft.Office.WebExtension.context.document&&Microsoft.Office.WebExtension.context.document instanceof OSF.DDA.ExcelDocument&&OSF.DDA.ExcelTableBinding?OSF.DDA.ExcelTableBinding:OSF.DDA.TableBinding)(r,t,i,o,s);break;default:n=new OSF.DDA.UnknownBinding(r,t)}return n},OSF.DDA.AsyncMethodNames.addNames({AddFromSelectionAsync:"addFromSelectionAsync",AddFromNamedItemAsync:"addFromNamedItemAsync",GetAllAsync:"getAllAsync",GetByIdAsync:"getByIdAsync",ReleaseByIdAsync:"releaseByIdAsync",GetDataAsync:"getDataAsync",SetDataAsync:"setDataAsync"}),function(){function e(e){return OSF.DDA.OMFactory.manufactureBinding(e,Microsoft.Office.WebExtension.context.document)}function t(e){return e.id}OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddFromSelectionAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.BindingType,enum:Microsoft.Office.WebExtension.BindingType}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:{types:["string"],calculate:OSF.DDA.generateBindingId}},{name:Microsoft.Office.WebExtension.Parameters.Columns,value:{types:["object"],defaultValue:null}}],privateStateCallbacks:[],onSucceeded:e}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddFromNamedItemAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.ItemName,types:["string"]},{name:Microsoft.Office.WebExtension.Parameters.BindingType,enum:Microsoft.Office.WebExtension.BindingType}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:{types:["string"],calculate:OSF.DDA.generateBindingId}},{name:Microsoft.Office.WebExtension.Parameters.Columns,value:{types:["object"],defaultValue:null}}],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.FailOnCollision,value:function(){return!0}}],onSucceeded:e}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetAllAsync,requiredArguments:[],supportedOptions:[],privateStateCallbacks:[],onSucceeded:function(t){return OSF.OUtil.mapList(t[OSF.DDA.ListDescriptors.BindingList],e)}}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetByIdAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Id,types:["string"]}],supportedOptions:[],privateStateCallbacks:[],onSucceeded:e}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.ReleaseByIdAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Id,types:["string"]}],supportedOptions:[],privateStateCallbacks:[],onSucceeded:function(e,t,n){var r=n[Microsoft.Office.WebExtension.Parameters.Id];delete t._eventDispatches[r]}}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetDataAsync,requiredArguments:[],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:{enum:Microsoft.Office.WebExtension.CoercionType,calculate:function(e,t){return OSF.DDA.DataCoercion.getCoercionDefaultForBinding(t.type)}}},{name:Microsoft.Office.WebExtension.Parameters.ValueFormat,value:{enum:Microsoft.Office.WebExtension.ValueFormat,defaultValue:Microsoft.Office.WebExtension.ValueFormat.Unformatted}},{name:Microsoft.Office.WebExtension.Parameters.FilterType,value:{enum:Microsoft.Office.WebExtension.FilterType,defaultValue:Microsoft.Office.WebExtension.FilterType.All}},{name:Microsoft.Office.WebExtension.Parameters.Rows,value:{types:["object","string"],defaultValue:null}},{name:Microsoft.Office.WebExtension.Parameters.Columns,value:{types:["object"],defaultValue:null}},{name:Microsoft.Office.WebExtension.Parameters.StartRow,value:{types:["number"],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.StartColumn,value:{types:["number"],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.RowCount,value:{types:["number"],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.ColumnCount,value:{types:["number"],defaultValue:0}}],checkCallArgs:function(e,t,n){if(0==e[Microsoft.Office.WebExtension.Parameters.StartRow]&&0==e[Microsoft.Office.WebExtension.Parameters.StartColumn]&&0==e[Microsoft.Office.WebExtension.Parameters.RowCount]&&0==e[Microsoft.Office.WebExtension.Parameters.ColumnCount]&&(delete e[Microsoft.Office.WebExtension.Parameters.StartRow],delete e[Microsoft.Office.WebExtension.Parameters.StartColumn],delete e[Microsoft.Office.WebExtension.Parameters.RowCount],delete e[Microsoft.Office.WebExtension.Parameters.ColumnCount]),e[Microsoft.Office.WebExtension.Parameters.CoercionType]!=OSF.DDA.DataCoercion.getCoercionDefaultForBinding(t.type)&&(e[Microsoft.Office.WebExtension.Parameters.StartRow]||e[Microsoft.Office.WebExtension.Parameters.StartColumn]||e[Microsoft.Office.WebExtension.Parameters.RowCount]||e[Microsoft.Office.WebExtension.Parameters.ColumnCount]))throw OSF.DDA.ErrorCodeManager.errorCodes.ooeCoercionTypeNotMatchBinding;return e},privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:t}],onSucceeded:function(e,t,n){var r=e[Microsoft.Office.WebExtension.Parameters.Data];return OSF.DDA.TableDataProperties&&r&&(null!=r[OSF.DDA.TableDataProperties.TableRows]||null!=r[OSF.DDA.TableDataProperties.TableHeaders])&&(r=OSF.DDA.OMFactory.manufactureTableData(r)),null==(r=OSF.DDA.DataCoercion.coerceData(r,n[Microsoft.Office.WebExtension.Parameters.CoercionType]))?null:r}}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetDataAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["string","object","number","boolean"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:{enum:Microsoft.Office.WebExtension.CoercionType,calculate:function(e){return OSF.DDA.DataCoercion.determineCoercionType(e[Microsoft.Office.WebExtension.Parameters.Data])}}},{name:Microsoft.Office.WebExtension.Parameters.Rows,value:{types:["object","string"],defaultValue:null}},{name:Microsoft.Office.WebExtension.Parameters.Columns,value:{types:["object"],defaultValue:null}},{name:Microsoft.Office.WebExtension.Parameters.StartRow,value:{types:["number"],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.StartColumn,value:{types:["number"],defaultValue:0}}],checkCallArgs:function(e,t,n){if(0==e[Microsoft.Office.WebExtension.Parameters.StartRow]&&0==e[Microsoft.Office.WebExtension.Parameters.StartColumn]&&(delete e[Microsoft.Office.WebExtension.Parameters.StartRow],delete e[Microsoft.Office.WebExtension.Parameters.StartColumn]),e[Microsoft.Office.WebExtension.Parameters.CoercionType]!=OSF.DDA.DataCoercion.getCoercionDefaultForBinding(t.type)&&(e[Microsoft.Office.WebExtension.Parameters.StartRow]||e[Microsoft.Office.WebExtension.Parameters.StartColumn]))throw OSF.DDA.ErrorCodeManager.errorCodes.ooeCoercionTypeNotMatchBinding;return e},privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:t}]})}(),OSF.OUtil.augmentList(OSF.DDA.BindingProperties,{RowCount:"BindingRowCount",ColumnCount:"BindingColumnCount",HasHeaders:"HasHeaders"}),OSF.DDA.MatrixBinding=function(e,t,n,r){OSF.DDA.MatrixBinding.uber.constructor.call(this,e,t),OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.BindingType.Matrix},rowCount:{value:n||0},columnCount:{value:r||0}})},OSF.OUtil.extend(OSF.DDA.MatrixBinding,OSF.DDA.Binding),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.PropertyDescriptors.BindingProperties,fromHost:[{name:OSF.DDA.BindingProperties.Id,value:0},{name:OSF.DDA.BindingProperties.Type,value:1},{name:OSF.DDA.SafeArray.UniqueArguments.BindingSpecificData,value:2}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:Microsoft.Office.WebExtension.Parameters.BindingType,toHost:[{name:Microsoft.Office.WebExtension.BindingType.Text,value:0},{name:Microsoft.Office.WebExtension.BindingType.Matrix,value:1},{name:Microsoft.Office.WebExtension.BindingType.Table,value:2}],invertible:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidAddBindingFromSelectionMethod,fromHost:[{name:OSF.DDA.PropertyDescriptors.BindingProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.BindingType,value:1}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidAddBindingFromNamedItemMethod,fromHost:[{name:OSF.DDA.PropertyDescriptors.BindingProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.ItemName,value:0},{name:Microsoft.Office.WebExtension.Parameters.Id,value:1},{name:Microsoft.Office.WebExtension.Parameters.BindingType,value:2},{name:Microsoft.Office.WebExtension.Parameters.FailOnCollision,value:3}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidReleaseBindingMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetBindingMethod,fromHost:[{name:OSF.DDA.PropertyDescriptors.BindingProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetAllBindingsMethod,fromHost:[{name:OSF.DDA.ListDescriptors.BindingList,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetBindingDataMethod,fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:1},{name:Microsoft.Office.WebExtension.Parameters.ValueFormat,value:2},{name:Microsoft.Office.WebExtension.Parameters.FilterType,value:3},{name:OSF.DDA.PropertyDescriptors.Subset,value:4}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSetBindingDataMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:1},{name:Microsoft.Office.WebExtension.Parameters.Data,value:2},{name:OSF.DDA.SafeArray.UniqueArguments.Offset,value:3}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.SafeArray.UniqueArguments.BindingSpecificData,fromHost:[{name:OSF.DDA.BindingProperties.RowCount,value:0},{name:OSF.DDA.BindingProperties.ColumnCount,value:1},{name:OSF.DDA.BindingProperties.HasHeaders,value:2}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.PropertyDescriptors.Subset,toHost:[{name:OSF.DDA.SafeArray.UniqueArguments.Offset,value:0},{name:OSF.DDA.SafeArray.UniqueArguments.Run,value:1}],canonical:!0,isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.SafeArray.UniqueArguments.Offset,toHost:[{name:Microsoft.Office.WebExtension.Parameters.StartRow,value:0},{name:Microsoft.Office.WebExtension.Parameters.StartColumn,value:1}],canonical:!0,isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.SafeArray.UniqueArguments.Run,toHost:[{name:Microsoft.Office.WebExtension.Parameters.RowCount,value:0},{name:Microsoft.Office.WebExtension.Parameters.ColumnCount,value:1}],canonical:!0,isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidAddRowsMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.Data,value:1}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidAddColumnsMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.Data,value:1}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidClearAllRowsMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0}]}),OSF.OUtil.augmentList(OSF.DDA.PropertyDescriptors,{TableDataProperties:"TableDataProperties"}),OSF.OUtil.augmentList(OSF.DDA.BindingProperties,{RowCount:"BindingRowCount",ColumnCount:"BindingColumnCount",HasHeaders:"HasHeaders"}),OSF.DDA.TableDataProperties={TableRows:"TableRows",TableHeaders:"TableHeaders"},OSF.DDA.TableBinding=function(e,t,n,r,i){OSF.DDA.TableBinding.uber.constructor.call(this,e,t),OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.BindingType.Table},rowCount:{value:n||0},columnCount:{value:r||0},hasHeaders:{value:i||!1}});var o=OSF.DDA.AsyncMethodNames;OSF.DDA.DispIdHost.addAsyncMethods(this,[o.AddRowsAsync,o.AddColumnsAsync,o.DeleteAllDataValuesAsync])},OSF.OUtil.extend(OSF.DDA.TableBinding,OSF.DDA.Binding),OSF.DDA.AsyncMethodNames.addNames({AddRowsAsync:"addRowsAsync",AddColumnsAsync:"addColumnsAsync",DeleteAllDataValuesAsync:"deleteAllDataValuesAsync"}),function(){function e(e){return e.id}OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddRowsAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["object"]}],supportedOptions:[],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:e}]}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddColumnsAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["object"]}],supportedOptions:[],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:e}]}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.DeleteAllDataValuesAsync,requiredArguments:[],supportedOptions:[],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:e}]})}(),OSF.DDA.TextBinding=function(e,t){OSF.DDA.TextBinding.uber.constructor.call(this,e,t),OSF.OUtil.defineEnumerableProperty(this,"type",{value:Microsoft.Office.WebExtension.BindingType.Text})},OSF.OUtil.extend(OSF.DDA.TextBinding,OSF.DDA.Binding),OSF.DDA.AsyncMethodNames.addNames({AddFromPromptAsync:"addFromPromptAsync"}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddFromPromptAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.BindingType,enum:Microsoft.Office.WebExtension.BindingType}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:{types:["string"],calculate:OSF.DDA.generateBindingId}},{name:Microsoft.Office.WebExtension.Parameters.PromptText,value:{types:["string"],calculate:function(){return Strings.OfficeOM.L_AddBindingFromPromptDefaultText}}},{name:Microsoft.Office.WebExtension.Parameters.SampleData,value:{types:["object"],defaultValue:null}}],privateStateCallbacks:[],onSucceeded:function(e){return OSF.DDA.OMFactory.manufactureBinding(e,Microsoft.Office.WebExtension.context.document)}}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidAddBindingFromPromptMethod,fromHost:[{name:OSF.DDA.PropertyDescriptors.BindingProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.BindingType,value:1},{name:Microsoft.Office.WebExtension.Parameters.PromptText,value:2}]}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{DocumentSelectionChanged:"documentSelectionChanged"}),OSF.DDA.DocumentSelectionChangedEventArgs=function(e){OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.DocumentSelectionChanged},document:{value:e}})},OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{ObjectDeleted:"objectDeleted"}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{ObjectSelectionChanged:"objectSelectionChanged"}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{ObjectDataChanged:"objectDataChanged"}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{ContentControlAdded:"contentControlAdded"}),OSF.DDA.ObjectEventArgs=function(e,t){OSF.OUtil.defineEnumerableProperties(this,{type:{value:e},object:{value:t}})},OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidDocumentSelectionChangedEvent}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidObjectDeletedEvent,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0}],fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:OSF.DDA.SafeArray.Delegate.ParameterMap.sourceData}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidObjectSelectionChangedEvent,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0}],fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:OSF.DDA.SafeArray.Delegate.ParameterMap.sourceData}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidObjectDataChangedEvent,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0}],fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:OSF.DDA.SafeArray.Delegate.ParameterMap.sourceData}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidContentControlAddedEvent,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0}],fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:OSF.DDA.SafeArray.Delegate.ParameterMap.sourceData}]}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{BindingSelectionChanged:"bindingSelectionChanged",BindingDataChanged:"bindingDataChanged"}),OSF.OUtil.augmentList(OSF.DDA.EventDescriptors,{BindingSelectionChangedEvent:"BindingSelectionChangedEvent"}),OSF.DDA.BindingSelectionChangedEventArgs=function(e,t){for(var n in OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.BindingSelectionChanged},binding:{value:e}}),t)OSF.OUtil.defineEnumerableProperty(this,n,{value:t[n]})},OSF.DDA.BindingDataChangedEventArgs=function(e){OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.BindingDataChanged},binding:{value:e}})},OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDescriptors.BindingSelectionChangedEvent,fromHost:[{name:OSF.DDA.PropertyDescriptors.BindingProperties,value:0},{name:OSF.DDA.PropertyDescriptors.Subset,value:1}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidBindingSelectionChangedEvent,fromHost:[{name:OSF.DDA.EventDescriptors.BindingSelectionChangedEvent,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidBindingDataChangedEvent,fromHost:[{name:OSF.DDA.PropertyDescriptors.BindingProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.FilterType,{OnlyVisible:"onlyVisible"}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:Microsoft.Office.WebExtension.Parameters.FilterType,toHost:[{name:Microsoft.Office.WebExtension.FilterType.OnlyVisible,value:1}]}),Microsoft.Office.WebExtension.GoToType={Binding:"binding",NamedItem:"namedItem",Slide:"slide",Index:"index"},Microsoft.Office.WebExtension.SelectionMode={Default:"default",Selected:"selected",None:"none"},Microsoft.Office.WebExtension.Index={First:"first",Last:"last",Next:"next",Previous:"previous"},OSF.DDA.AsyncMethodNames.addNames({GoToByIdAsync:"goToByIdAsync"}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GoToByIdAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Id,types:["string","number"]},{name:Microsoft.Office.WebExtension.Parameters.GoToType,enum:Microsoft.Office.WebExtension.GoToType}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.SelectionMode,value:{enum:Microsoft.Office.WebExtension.SelectionMode,defaultValue:Microsoft.Office.WebExtension.SelectionMode.Default}}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:Microsoft.Office.WebExtension.Parameters.GoToType,toHost:[{name:Microsoft.Office.WebExtension.GoToType.Binding,value:0},{name:Microsoft.Office.WebExtension.GoToType.NamedItem,value:1},{name:Microsoft.Office.WebExtension.GoToType.Slide,value:2},{name:Microsoft.Office.WebExtension.GoToType.Index,value:3}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:Microsoft.Office.WebExtension.Parameters.SelectionMode,toHost:[{name:Microsoft.Office.WebExtension.SelectionMode.Default,value:0},{name:Microsoft.Office.WebExtension.SelectionMode.Selected,value:1},{name:Microsoft.Office.WebExtension.SelectionMode.None,value:2}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidNavigateToMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.GoToType,value:1},{name:Microsoft.Office.WebExtension.Parameters.SelectionMode,value:2}]}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{RichApiMessage:"richApiMessage"}),OSF.DDA.RichApiMessageEventArgs=function(e,t){var n=t[Microsoft.Office.WebExtension.Parameters.Data],r=[];if(n)for(var i=0;i<n.length;i++){var o=n[i];o.toArray&&(o=o.toArray()),r.push({messageCategory:o[0],messageType:o[1],targetId:o[2],message:o[3],id:o[4],isRemoteOverride:o[5]})}OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.RichApiMessage},entries:{value:r}})},function(e){var t=function(){function e(){this._eventDispatch=null,this._registerHandlers=[],this._eventDispatch=new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.RichApiMessage]),OSF.DDA.DispIdHost.addEventSupport(this,this._eventDispatch)}return e.prototype.register=function(e){var t=this;return this._registerWithHostPromise||(this._registerWithHostPromise=new Office.Promise((function(e,n){t.addHandlerAsync(Microsoft.Office.WebExtension.EventType.RichApiMessage,(function(e){t._registerHandlers.forEach((function(t){t&&t(e)}))}),(function(t){"failed"==t.status?n(t.error):e()}))}))),this._registerWithHostPromise.then((function(){t._registerHandlers.push(e)}))},e}();e.RichApiMessageManager=t}(OfficeExt||(OfficeExt={})),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidRichApiMessageEvent,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:0}],fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:OSF.DDA.SafeArray.Delegate.ParameterMap.sourceData}]}),OSF.DDA.AsyncMethodNames.addNames({ExecuteRichApiRequestAsync:"executeRichApiRequestAsync"}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.ExecuteRichApiRequestAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["object"]}],supportedOptions:[]}),OSF.OUtil.setNamespace("RichApi",OSF.DDA),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidExecuteRichApiRequestMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:0}],fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]}),Microsoft.Office.WebExtension.FileType={Text:"text",Compressed:"compressed",Pdf:"pdf"},OSF.OUtil.augmentList(OSF.DDA.PropertyDescriptors,{FileProperties:"FileProperties",FileSliceProperties:"FileSliceProperties"}),OSF.DDA.FileProperties={Handle:"FileHandle",FileSize:"FileSize",SliceSize:Microsoft.Office.WebExtension.Parameters.SliceSize},OSF.DDA.File=function(e,t,n){OSF.OUtil.defineEnumerableProperties(this,{size:{value:t},sliceCount:{value:Math.ceil(t/n)}});var r={};r[OSF.DDA.FileProperties.Handle]=e,r[OSF.DDA.FileProperties.SliceSize]=n;var i=OSF.DDA.AsyncMethodNames;OSF.DDA.DispIdHost.addAsyncMethods(this,[i.GetDocumentCopyChunkAsync,i.ReleaseDocumentCopyAsync],r)},OSF.DDA.FileSliceOffset="fileSliceoffset",OSF.DDA.AsyncMethodNames.addNames({GetDocumentCopyAsync:"getFileAsync",GetDocumentCopyChunkAsync:"getSliceAsync",ReleaseDocumentCopyAsync:"closeAsync"}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetDocumentCopyAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.FileType,enum:Microsoft.Office.WebExtension.FileType}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.SliceSize,value:{types:["number"],defaultValue:4194304}}],checkCallArgs:function(e,t,n){var r=e[Microsoft.Office.WebExtension.Parameters.SliceSize];if(r<=0||r>4194304)throw OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSliceSize;return e},onSucceeded:function(e,t,n){return new OSF.DDA.File(e[OSF.DDA.FileProperties.Handle],e[OSF.DDA.FileProperties.FileSize],n[Microsoft.Office.WebExtension.Parameters.SliceSize])}}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetDocumentCopyChunkAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.SliceIndex,types:["number"]}],privateStateCallbacks:[{name:OSF.DDA.FileProperties.Handle,value:function(e,t){return t[OSF.DDA.FileProperties.Handle]}},{name:OSF.DDA.FileProperties.SliceSize,value:function(e,t){return t[OSF.DDA.FileProperties.SliceSize]}}],checkCallArgs:function(e,t,n){var r=e[Microsoft.Office.WebExtension.Parameters.SliceIndex];if(r<0||r>=t.sliceCount)throw OSF.DDA.ErrorCodeManager.errorCodes.ooeIndexOutOfRange;return e[OSF.DDA.FileSliceOffset]=parseInt((r*n[OSF.DDA.FileProperties.SliceSize]).toString()),e},onSucceeded:function(e,t,n){var r={};return OSF.OUtil.defineEnumerableProperties(r,{data:{value:OSF.OUtil.shallowCopy(e[Microsoft.Office.WebExtension.Parameters.Data])},index:{value:n[Microsoft.Office.WebExtension.Parameters.SliceIndex]},size:{value:e[OSF.DDA.FileProperties.SliceSize]}}),r}}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.ReleaseDocumentCopyAsync,privateStateCallbacks:[{name:OSF.DDA.FileProperties.Handle,value:function(e,t){return t[OSF.DDA.FileProperties.Handle]}}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.PropertyDescriptors.FileProperties,fromHost:[{name:OSF.DDA.FileProperties.Handle,value:0},{name:OSF.DDA.FileProperties.FileSize,value:1}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.PropertyDescriptors.FileSliceProperties,fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:0},{name:OSF.DDA.FileProperties.SliceSize,value:1}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:Microsoft.Office.WebExtension.Parameters.FileType,toHost:[{name:Microsoft.Office.WebExtension.FileType.Text,value:0},{name:Microsoft.Office.WebExtension.FileType.Compressed,value:5},{name:Microsoft.Office.WebExtension.FileType.Pdf,value:6}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetDocumentCopyMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.FileType,value:0}],fromHost:[{name:OSF.DDA.PropertyDescriptors.FileProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetDocumentCopyChunkMethod,toHost:[{name:OSF.DDA.FileProperties.Handle,value:0},{name:OSF.DDA.FileSliceOffset,value:1},{name:OSF.DDA.FileProperties.SliceSize,value:2}],fromHost:[{name:OSF.DDA.PropertyDescriptors.FileSliceProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidReleaseDocumentCopyMethod,toHost:[{name:OSF.DDA.FileProperties.Handle,value:0}]}),OSF.DDA.FilePropertiesDescriptor={Url:"Url"},OSF.OUtil.augmentList(OSF.DDA.PropertyDescriptors,{FilePropertiesDescriptor:"FilePropertiesDescriptor"}),Microsoft.Office.WebExtension.FileProperties=function(e){OSF.OUtil.defineEnumerableProperties(this,{url:{value:e[OSF.DDA.FilePropertiesDescriptor.Url]}})},OSF.DDA.AsyncMethodNames.addNames({GetFilePropertiesAsync:"getFilePropertiesAsync"}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetFilePropertiesAsync,fromHost:[{name:OSF.DDA.PropertyDescriptors.FilePropertiesDescriptor,value:0}],requiredArguments:[],supportedOptions:[],onSucceeded:function(e,t,n){return new Microsoft.Office.WebExtension.FileProperties(e)}}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.PropertyDescriptors.FilePropertiesDescriptor,fromHost:[{name:OSF.DDA.FilePropertiesDescriptor.Url,value:0}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetFilePropertiesMethod,fromHost:[{name:OSF.DDA.PropertyDescriptors.FilePropertiesDescriptor,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]}),OSF.DDA.ExcelTableBinding=function(e,t,n,r,i){var o=OSF.DDA.AsyncMethodNames;OSF.DDA.DispIdHost.addAsyncMethods(this,[o.ClearFormatsAsync,o.SetTableOptionsAsync,o.SetFormatsAsync]),OSF.DDA.ExcelTableBinding.uber.constructor.call(this,e,t,n,r,i),OSF.OUtil.finalizeProperties(this)},OSF.OUtil.extend(OSF.DDA.ExcelTableBinding,OSF.DDA.TableBinding),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetSelectedDataAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["string","object","number","boolean"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:{enum:Microsoft.Office.WebExtension.CoercionType,calculate:function(e){return OSF.DDA.DataCoercion.determineCoercionType(e[Microsoft.Office.WebExtension.Parameters.Data])}}},{name:Microsoft.Office.WebExtension.Parameters.CellFormat,value:{types:["object"],defaultValue:[]}},{name:Microsoft.Office.WebExtension.Parameters.TableOptions,value:{types:["object"],defaultValue:[]}}],privateStateCallbacks:[]}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetDataAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["string","object","number","boolean"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:{enum:Microsoft.Office.WebExtension.CoercionType,calculate:function(e){return OSF.DDA.DataCoercion.determineCoercionType(e[Microsoft.Office.WebExtension.Parameters.Data])}}},{name:Microsoft.Office.WebExtension.Parameters.Rows,value:{types:["object","string"],defaultValue:null}},{name:Microsoft.Office.WebExtension.Parameters.Columns,value:{types:["object"],defaultValue:null}},{name:Microsoft.Office.WebExtension.Parameters.StartRow,value:{types:["number"],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.StartColumn,value:{types:["number"],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.CellFormat,value:{types:["object"],defaultValue:[]}},{name:Microsoft.Office.WebExtension.Parameters.TableOptions,value:{types:["object"],defaultValue:[]}}],checkCallArgs:function(e,t,n){var r=Microsoft.Office.WebExtension.Parameters;if(0==e[r.StartRow]&&0==e[r.StartColumn]&&OSF.OUtil.isArray(e[r.CellFormat])&&0===e[r.CellFormat].length&&OSF.OUtil.isArray(e[r.TableOptions])&&0===e[r.TableOptions].length&&(delete e[r.StartRow],delete e[r.StartColumn],delete e[r.CellFormat],delete e[r.TableOptions]),e[r.CoercionType]!=OSF.DDA.DataCoercion.getCoercionDefaultForBinding(t.type)&&(e[r.StartRow]&&0!=e[r.StartRow]||e[r.StartColumn]&&0!=e[r.StartColumn]||e[r.CellFormat]||e[r.TableOptions]))throw OSF.DDA.ErrorCodeManager.errorCodes.ooeCoercionTypeNotMatchBinding;return e},privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:function(e){return e.id}}]}),OSF.DDA.BindingPromise.prototype.setTableOptionsAsync=function(){var e=arguments;return this._fetch((function(t){t.setTableOptionsAsync.apply(t,e)})),this},OSF.DDA.BindingPromise.prototype.setFormatsAsync=function(){var e=arguments;return this._fetch((function(t){t.setFormatsAsync.apply(t,e)})),this},OSF.DDA.BindingPromise.prototype.clearFormatsAsync=function(){var e=arguments;return this._fetch((function(t){t.clearFormatsAsync.apply(t,e)})),this},function(){function e(e){return e.id}OSF.DDA.AsyncMethodNames.addNames({ClearFormatsAsync:"clearFormatsAsync",SetTableOptionsAsync:"setTableOptionsAsync",SetFormatsAsync:"setFormatsAsync"}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.ClearFormatsAsync,requiredArguments:[],supportedOptions:[],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:e}]}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetTableOptionsAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.TableOptions,defaultValue:[]}],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:e}]}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetFormatsAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.CellFormat,defaultValue:[]}],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:e}]})}(),Microsoft.Office.WebExtension.Table={All:0,Data:1,Headers:2},function(){OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidClearFormatsMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSetTableOptionsMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.TableOptions,value:1}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSetFormatsMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.CellFormat,value:1}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSetSelectedDataMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:0},{name:Microsoft.Office.WebExtension.Parameters.Data,value:1},{name:Microsoft.Office.WebExtension.Parameters.CellFormat,value:2},{name:Microsoft.Office.WebExtension.Parameters.TableOptions,value:3}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSetBindingDataMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:1},{name:Microsoft.Office.WebExtension.Parameters.Data,value:2},{name:OSF.DDA.SafeArray.UniqueArguments.Offset,value:3},{name:Microsoft.Office.WebExtension.Parameters.CellFormat,value:4},{name:Microsoft.Office.WebExtension.Parameters.TableOptions,value:5}]});var e={headerRow:0,bandedRows:1,firstColumn:2,lastColumn:3,bandedColumns:4,filterButton:5,style:6,totalRow:7},t={row:0,column:1},n={alignHorizontal:{text:"alignHorizontal",type:1},alignVertical:{text:"alignVertical",type:2},backgroundColor:{text:"backgroundColor",type:101},borderStyle:{text:"borderStyle",type:201},borderColor:{text:"borderColor",type:202},borderTopStyle:{text:"borderTopStyle",type:203},borderTopColor:{text:"borderTopColor",type:204},borderBottomStyle:{text:"borderBottomStyle",type:205},borderBottomColor:{text:"borderBottomColor",type:206},borderLeftStyle:{text:"borderLeftStyle",type:207},borderLeftColor:{text:"borderLeftColor",type:208},borderRightStyle:{text:"borderRightStyle",type:209},borderRightColor:{text:"borderRightColor",type:210},borderOutlineStyle:{text:"borderOutlineStyle",type:211},borderOutlineColor:{text:"borderOutlineColor",type:212},borderInlineStyle:{text:"borderInlineStyle",type:213},borderInlineColor:{text:"borderInlineColor",type:214},fontFamily:{text:"fontFamily",type:301},fontStyle:{text:"fontStyle",type:302},fontSize:{text:"fontSize",type:303},fontUnderlineStyle:{text:"fontUnderlineStyle",type:304},fontColor:{text:"fontColor",type:305},fontDirection:{text:"fontDirection",type:306},fontStrikethrough:{text:"fontStrikethrough",type:307},fontSuperscript:{text:"fontSuperscript",type:308},fontSubscript:{text:"fontSubscript",type:309},fontNormal:{text:"fontNormal",type:310},indentLeft:{text:"indentLeft",type:401},indentRight:{text:"indentRight",type:402},numberFormat:{text:"numberFormat",type:501},width:{text:"width",type:701},height:{text:"height",type:702},wrapping:{text:"wrapping",type:703}},r=[{name:"none",value:0},{name:"thin",value:1},{name:"medium",value:2},{name:"dashed",value:3},{name:"dotted",value:4},{name:"thick",value:5},{name:"double",value:6},{name:"hair",value:7},{name:"medium dashed",value:8},{name:"dash dot",value:9},{name:"medium dash dot",value:10},{name:"dash dot dot",value:11},{name:"medium dash dot dot",value:12},{name:"slant dash dot",value:13}],i=[{name:"none",value:0},{name:"black",value:1},{name:"blue",value:2},{name:"gray",value:3},{name:"green",value:4},{name:"orange",value:5},{name:"pink",value:6},{name:"purple",value:7},{name:"red",value:8},{name:"teal",value:9},{name:"turquoise",value:10},{name:"violet",value:11},{name:"white",value:12},{name:"yellow",value:13},{name:"automatic",value:14}],o=OSF.DDA.SafeArray.Delegate.ParameterMap;o.define({type:n.alignHorizontal.text,toHost:[{name:"general",value:0},{name:"left",value:1},{name:"center",value:2},{name:"right",value:3},{name:"fill",value:4},{name:"justify",value:5},{name:"center across selection",value:6},{name:"distributed",value:7}]}),o.define({type:n.alignVertical.text,toHost:[{name:"top",value:0},{name:"center",value:1},{name:"bottom",value:2},{name:"justify",value:3},{name:"distributed",value:4}]}),o.define({type:n.backgroundColor.text,toHost:i}),o.define({type:n.borderStyle.text,toHost:r}),o.define({type:n.borderColor.text,toHost:i}),o.define({type:n.borderTopStyle.text,toHost:r}),o.define({type:n.borderTopColor.text,toHost:i}),o.define({type:n.borderBottomStyle.text,toHost:r}),o.define({type:n.borderBottomColor.text,toHost:i}),o.define({type:n.borderLeftStyle.text,toHost:r}),o.define({type:n.borderLeftColor.text,toHost:i}),o.define({type:n.borderRightStyle.text,toHost:r}),o.define({type:n.borderRightColor.text,toHost:i}),o.define({type:n.borderOutlineStyle.text,toHost:r}),o.define({type:n.borderOutlineColor.text,toHost:i}),o.define({type:n.borderInlineStyle.text,toHost:r}),o.define({type:n.borderInlineColor.text,toHost:i}),o.define({type:n.fontStyle.text,toHost:[{name:"regular",value:0},{name:"italic",value:1},{name:"bold",value:2},{name:"bold italic",value:3}]}),o.define({type:n.fontUnderlineStyle.text,toHost:[{name:"none",value:0},{name:"single",value:1},{name:"double",value:2},{name:"single accounting",value:3},{name:"double accounting",value:4}]}),o.define({type:n.fontColor.text,toHost:i}),o.define({type:n.fontDirection.text,toHost:[{name:"context",value:0},{name:"left-to-right",value:1},{name:"right-to-left",value:2}]}),o.define({type:n.width.text,toHost:[{name:"auto fit",value:-1}]}),o.define({type:n.height.text,toHost:[{name:"auto fit",value:-1}]}),o.define({type:Microsoft.Office.WebExtension.Parameters.TableOptions,toHost:[{name:"headerRow",value:0},{name:"bandedRows",value:1},{name:"firstColumn",value:2},{name:"lastColumn",value:3},{name:"bandedColumns",value:4},{name:"filterButton",value:5},{name:"style",value:6},{name:"totalRow",value:7}]}),o.dynamicTypes[Microsoft.Office.WebExtension.Parameters.CellFormat]={toHost:function(e){for(var t in e)e[t].format&&(e[t].format=o.doMapValues(e[t].format,"toHost"));return e},fromHost:function(e){return e}},o.setDynamicType(Microsoft.Office.WebExtension.Parameters.CellFormat,{toHost:function(e){var r=[];for(var i in e){var o=e[i],s=[];if(void 0!==o.cells){var a,l=o.cells;if("object"==typeof o.cells)for(var u in a=[],l)void 0!==t[u]&&(a[t[u]]=l[u]);else a=l;s[0]=a}if(o.format){var c=o.format,p=[];for(var f in c)void 0!==n[f]&&p.push([n[f].type,c[f]]);s[1]=p}r[i]=s}return r},fromHost:function(e){return e}}),o.setDynamicType(Microsoft.Office.WebExtension.Parameters.TableOptions,{toHost:function(t){var n=[];for(var r in t)void 0!==e[r]&&(n[e[r]]=t[r]);return n},fromHost:function(e){return e}})}(),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.CoercionType,{Image:"image"}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.CoercionType,{XmlSvg:"xmlSvg"}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:Microsoft.Office.WebExtension.Parameters.CoercionType,toHost:[{name:Microsoft.Office.WebExtension.CoercionType.Image,value:8},{name:Microsoft.Office.WebExtension.CoercionType.XmlSvg,value:9}]}),function(e){!function(t){var n=function(){function n(){var e=this;this._pseudoDocument=null,this._eventDispatch=null,this._useAssociatedActionsOnly=null,this._processAppCommandInvocation=function(t){var n=e._verifyManifestCallback(t.callbackName);if(n.errorCode==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess){var r=e._constructEventObjectForCallback(t);r?window.setTimeout((function(){n.callback(r)}),0):e._invokeAppCommandCompletedMethod(t.appCommandId,OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError,"")}else e._invokeAppCommandCompletedMethod(t.appCommandId,n.errorCode,"")}}return n.initializeOsfDda=function(){OSF.DDA.AsyncMethodNames.addNames({AppCommandInvocationCompletedAsync:"appCommandInvocationCompletedAsync"}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AppCommandInvocationCompletedAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Id,types:["string"]},{name:Microsoft.Office.WebExtension.Parameters.Status,types:["number"]},{name:Microsoft.Office.WebExtension.Parameters.AppCommandInvocationCompletedData,types:["string"]}]}),OSF.OUtil.augmentList(OSF.DDA.EventDescriptors,{AppCommandInvokedEvent:"AppCommandInvokedEvent"}),OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{AppCommandInvoked:"appCommandInvoked"}),OSF.OUtil.setNamespace("AppCommand",OSF.DDA),OSF.DDA.AppCommand.AppCommandInvokedEventArgs=e.AppCommand.AppCommandInvokedEventArgs},n.prototype.initializeAndChangeOnce=function(e){t.registerDdaFacade(),this._pseudoDocument={},OSF.DDA.DispIdHost.addAsyncMethods(this._pseudoDocument,[OSF.DDA.AsyncMethodNames.AppCommandInvocationCompletedAsync]),this._eventDispatch=new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.AppCommandInvoked]);OSF.DDA.DispIdHost.addEventSupport(this._pseudoDocument,this._eventDispatch),this._pseudoDocument.addHandlerAsync(Microsoft.Office.WebExtension.EventType.AppCommandInvoked,this._processAppCommandInvocation,(function(t){e&&("succeeded"==t.status?e(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess):e(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError))}))},n.prototype._verifyManifestCallback=function(e){var t={callback:null,errorCode:OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidCallback};e=e.trim();try{var n=this._getCallbackFunc(e);if("function"!=typeof n)return t}catch(e){return t}return{callback:n,errorCode:OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess}},n.prototype._getUseAssociatedActionsOnly=function(){var e=this;if(null==this._useAssociatedActionsOnly){this._useAssociatedActionsOnly=!1;try{!0===window.useAssociatedActionsOnly?this._useAssociatedActionsOnly=!0:this._useAssociatedActionsOnly=OSF._OfficeAppFactory.getLoadScriptHelper().getUseAssociatedActionsOnlyDefined(),OTel&&oteljs&&Microsoft.Office.WebExtension.sendTelemetryEvent&&OTel.OTelLogger.onTelemetryLoaded((function(){var t=[oteljs.makeBooleanDataField("UseAction",!0===e._useAssociatedActionsOnly)];Microsoft.Office.WebExtension.sendTelemetryEvent({eventName:"Office.Extensibility.OfficeJs.AppCommandDefinition",dataFields:t,eventFlags:{dataCategories:oteljs.DataCategories.ProductServiceUsage,diagnosticLevel:oteljs.DiagnosticLevel.NecessaryServiceDataEvent}})}))}catch(e){}}return this._useAssociatedActionsOnly},n.prototype._getCallbackFuncFromWindow=function(e){for(var t=e.split("."),n=window,r=0;r<t.length-1;r++){if(!n[t[r]]||"object"!=typeof n[t[r]]&&"function"!=typeof n[t[r]])return null;n=n[t[r]]}return n[t[t.length-1]]},n.prototype._getCallbackFuncFromActionAssociateTable=function(e){var t=e.toUpperCase();return Office.actions._association.mappings[t]},n.prototype._getCallbackFunc=function(e){var t=null;return this._getUseAssociatedActionsOnly()||(t=this._getCallbackFuncFromWindow(e)),t||(t=this._getCallbackFuncFromActionAssociateTable(e)),t},n.prototype._invokeAppCommandCompletedMethod=function(e,t,n){this._pseudoDocument.appCommandInvocationCompletedAsync(e,t,n)},n.prototype._constructEventObjectForCallback=function(e){var t=this,n=new i;try{var r=JSON.parse(e.eventObjStr);this._translateEventObjectInternal(r,n),Object.defineProperty(n,"completed",{value:function(r){n.completedContext=r;var i=JSON.stringify(n);t._invokeAppCommandCompletedMethod(e.appCommandId,OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess,i)},enumerable:!0})}catch(e){n=null}return n},n.prototype._translateEventObjectInternal=function(e,t){for(var n in e)if(e.hasOwnProperty(n)){var r=e[n];"object"==typeof r&&null!=r?(OSF.OUtil.defineEnumerableProperty(t,n,{value:{}}),this._translateEventObjectInternal(r,t[n])):Object.defineProperty(t,n,{value:r,enumerable:!0,writable:!0})}},n.prototype._constructObjectByTemplate=function(e,t){var n={};if(!e||!t)return n;for(var r in e)if(e.hasOwnProperty(r)&&(n[r]=null,null!=t[r])){var i=e[r],o=t[r],s=typeof o;"object"==typeof i&&null!=i?n[r]=this._constructObjectByTemplate(i,o):"number"!=s&&"string"!=s&&"boolean"!=s||(n[r]=o)}return n},n.instance=function(){return null==n._instance&&(n._instance=new n),n._instance},n._instance=null,n}();t.AppCommandManager=n;var r=function(){function e(e,t,n){this.type=Microsoft.Office.WebExtension.EventType.AppCommandInvoked,this.appCommandId=e,this.callbackName=t,this.eventObjStr=n}return e.create=function(n){return new e(n[t.AppCommandInvokedEventEnums.AppCommandId],n[t.AppCommandInvokedEventEnums.CallbackName],n[t.AppCommandInvokedEventEnums.EventObjStr])},e}();t.AppCommandInvokedEventArgs=r;var i=function(){};t.AppCommandCallbackEventArgs=i,t.AppCommandInvokedEventEnums={AppCommandId:"appCommandId",CallbackName:"callbackName",EventObjStr:"eventObjStr"}}(e.AppCommand||(e.AppCommand={}))}(OfficeExt||(OfficeExt={})),OfficeExt.AppCommand.AppCommandManager.initializeOsfDda(),function(e){!function(t){t.registerDdaFacade=function(){if(OSF.DDA.SafeArray){var t=OSF.DDA.SafeArray.Delegate.ParameterMap;t.define({type:OSF.DDA.MethodDispId.dispidAppCommandInvocationCompletedMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.Status,value:1},{name:Microsoft.Office.WebExtension.Parameters.AppCommandInvocationCompletedData,value:2}]}),t.define({type:OSF.DDA.EventDispId.dispidAppCommandInvokedEvent,fromHost:[{name:OSF.DDA.EventDescriptors.AppCommandInvokedEvent,value:t.self}],isComplexType:!0}),t.define({type:OSF.DDA.EventDescriptors.AppCommandInvokedEvent,fromHost:[{name:e.AppCommand.AppCommandInvokedEventEnums.AppCommandId,value:0},{name:e.AppCommand.AppCommandInvokedEventEnums.CallbackName,value:1},{name:e.AppCommand.AppCommandInvokedEventEnums.EventObjStr,value:2}],isComplexType:!0})}}}(e.AppCommand||(e.AppCommand={}))}(OfficeExt||(OfficeExt={})),Microsoft.Office.WebExtension.AccountTypeFilter={NoFilter:"noFilter",AAD:"aad",MSA:"msa"},OSF.DDA.AsyncMethodNames.addNames({GetAccessTokenAsync:"getAccessTokenAsync"}),OSF.DDA.Auth=function(){},OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetAccessTokenAsync,requiredArguments:[],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.ForceConsent,value:{types:["boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.ForceAddAccount,value:{types:["boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.AuthChallenge,value:{types:["string"],defaultValue:""}},{name:Microsoft.Office.WebExtension.Parameters.AllowConsentPrompt,value:{types:["boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.ForMSGraphAccess,value:{types:["boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.AllowSignInPrompt,value:{types:["boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.EnableNewHosts,value:{types:["number"],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.AccountTypeFilter,value:{enum:Microsoft.Office.WebExtension.AccountTypeFilter,defaultValue:Microsoft.Office.WebExtension.AccountTypeFilter.NoFilter}}],checkCallArgs:function(e,t,n){var r,i=OSF._OfficeAppFactory.getInitializationHelper()._appContext;if(i&&i._wopiHostOriginForSingleSignOn){var o=OSF.OUtil.Guid.generateNewGuid();window.parent.parent.postMessage('{"MessageId":"AddinTrustedOrigin","AddinTrustId":"'+o+'"}',i._wopiHostOriginForSingleSignOn),e[Microsoft.Office.WebExtension.Parameters.AddinTrustId]=o}if(window.Office.context.requirements.isSetSupported("JsonPayloadSSO")){for(var s=((r={})[Microsoft.Office.WebExtension.Parameters.ForceConsent]=!1,r[Microsoft.Office.WebExtension.Parameters.ForceAddAccount]=!1,r[Microsoft.Office.WebExtension.Parameters.AuthChallenge]=!0,r[Microsoft.Office.WebExtension.Parameters.AllowConsentPrompt]=!0,r[Microsoft.Office.WebExtension.Parameters.ForMSGraphAccess]=!0,r[Microsoft.Office.WebExtension.Parameters.AllowSignInPrompt]=!0,r[Microsoft.Office.WebExtension.Parameters.EnableNewHosts]=!0,r[Microsoft.Office.WebExtension.Parameters.AccountTypeFilter]=!0,r),a={},l=0,u=Object.keys(s);l<u.length;l++){var c=u[l];s[c]&&(a[c]=e[c]),delete e[c]}e[Microsoft.Office.WebExtension.Parameters.JsonPayload]=JSON.stringify(a)}return e},onSucceeded:function(e,t,n){return e[Microsoft.Office.WebExtension.Parameters.Data]}}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetAccessTokenMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.JsonPayload,value:0},{name:Microsoft.Office.WebExtension.Parameters.ForceConsent,value:0},{name:Microsoft.Office.WebExtension.Parameters.ForceAddAccount,value:1},{name:Microsoft.Office.WebExtension.Parameters.AuthChallenge,value:2},{name:Microsoft.Office.WebExtension.Parameters.AllowConsentPrompt,value:3},{name:Microsoft.Office.WebExtension.Parameters.ForMSGraphAccess,value:4},{name:Microsoft.Office.WebExtension.Parameters.AllowSignInPrompt,value:5}],fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]}),OSF.DDA.AsyncMethodNames.addNames({OpenBrowserWindow:"openBrowserWindow"}),OSF.DDA.OpenBrowser=function(){},OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.OpenBrowserWindow,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Url,types:["string"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.Reserved,value:{types:["number"],defaultValue:0}}],privateStateCallbacks:[]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidOpenBrowserWindow,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Reserved,value:0},{name:Microsoft.Office.WebExtension.Parameters.Url,value:1}]}),OSF.DDA.AsyncMethodNames.addNames({ExecuteFeature:"executeFeatureAsync",QueryFeature:"queryFeatureAsync"}),OSF.OUtil.augmentList(OSF.DDA.PropertyDescriptors,{FeatureProperties:"FeatureProperties",TcidEnabled:"TcidEnabled",TcidVisible:"TcidVisible"}),OSF.DDA.ExecuteFeature=function(){},OSF.DDA.QueryFeature=function(){},OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.ExecuteFeature,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Tcid,types:["number"]}],privateStateCallbacks:[]}),OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.QueryFeature,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Tcid,types:["number"]}],privateStateCallbacks:[]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.PropertyDescriptors.FeatureProperties,fromHost:[{name:OSF.DDA.PropertyDescriptors.TcidEnabled,value:0},{name:OSF.DDA.PropertyDescriptors.TcidVisible,value:1}],isComplexType:!0}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidExecuteFeature,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Tcid,value:0}]}),OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidQueryFeature,fromHost:[{name:OSF.DDA.PropertyDescriptors.FeatureProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.Tcid,value:0}]}),OSF.DDA.ExcelDocument=function(e,t){var n=new OSF.DDA.BindingFacade(this);OSF.DDA.DispIdHost.addAsyncMethods(n,[OSF.DDA.AsyncMethodNames.AddFromPromptAsync]),OSF.DDA.DispIdHost.addAsyncMethods(this,[OSF.DDA.AsyncMethodNames.GoToByIdAsync]),OSF.DDA.DispIdHost.addAsyncMethods(this,[OSF.DDA.AsyncMethodNames.GetDocumentCopyAsync]),OSF.DDA.DispIdHost.addAsyncMethods(this,[OSF.DDA.AsyncMethodNames.GetFilePropertiesAsync]),OSF.DDA.ExcelDocument.uber.constructor.call(this,e,n,t),OSF.OUtil.finalizeProperties(this)},OSF.OUtil.extend(OSF.DDA.ExcelDocument,OSF.DDA.JsomDocument),OSF.InitializationHelper.prototype.prepareRightAfterWebExtensionInitialize=function(e){e&&e.get_isDialog()||OfficeExt.AppCommand.AppCommandManager.instance().initializeAndChangeOnce()},OSF.InitializationHelper.prototype.loadAppSpecificScriptAndCreateOM=function(e,t,n){OSF.DDA.ErrorCodeManager.initializeErrorMessages(Strings.OfficeOM),e.doc=new OSF.DDA.ExcelDocument(e,this.initializeMacSettings(e,!0)),OSF.DDA.DispIdHost.addAsyncMethods(OSF.DDA.RichApi,[OSF.DDA.AsyncMethodNames.ExecuteRichApiRequestAsync]),e.get_isDialog()||(OSF.DDA.RichApi.richApiMessageManager=new OfficeExt.RichApiMessageManager),t()},OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetSelectedDataAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["string","object","number","boolean"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:{enum:Microsoft.Office.WebExtension.CoercionType,calculate:function(e){return OSF.DDA.DataCoercion.determineCoercionType(e[Microsoft.Office.WebExtension.Parameters.Data])}}},{name:Microsoft.Office.WebExtension.Parameters.CellFormat,value:{types:["number","object"],defaultValue:[]}},{name:Microsoft.Office.WebExtension.Parameters.TableOptions,value:{types:["number","object"],defaultValue:[]}},{name:Microsoft.Office.WebExtension.Parameters.ImageWidth,value:{types:["number","boolean"],defaultValue:!1}},{name:Microsoft.Office.WebExtension.Parameters.ImageHeight,value:{types:["number","boolean"],defaultValue:!1}}],privateStateCallbacks:[]});var OfficeExtension;__extends=this&&this.__extends||function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}();!function(e){var t;!function(e){e.OfficeRequire=null}(e._Internal||(e._Internal={})),function(e){!function(t){t.Init=function(){return function(){"use strict";function t(e){return"function"==typeof e}var n,r=Array.isArray?Array.isArray:function(e){return"[object Array]"===Object.prototype.toString.call(e)},i=0,o=function(e,t){h[i]=e,h[i+1]=t,2===(i+=2)&&(n?n(d):u())};var s=("undefined"!=typeof window?window:void 0)||{},a=(s.MutationObserver||s.WebKitMutationObserver,"undefined"!=typeof process&&"[object process]"==={}.toString.call(process)),l="undefined"!=typeof Uint8ClampedArray&&"undefined"!=typeof importScripts&&"undefined"!=typeof MessageChannel;var u,c,p,f,h=new Array(1e3);function d(){for(var e=0;e<i;e+=2){(0,h[e])(h[e+1]),h[e]=void 0,h[e+1]=void 0}i=0}function y(){}a?(p=process.nextTick,f=process.versions.node.match(/^(?:(\d+)\.)?(?:(\d+)\.)?(\*|\d+)$/),Array.isArray(f)&&"0"===f[1]&&"10"===f[2]&&(p=window.setImmediate),u=function(){p(d)}):l?((c=new MessageChannel).port1.onmessage=d,u=function(){c.port2.postMessage(0)}):u=function(){setTimeout(d,1)};var m=void 0,g=new A;function _(e,n){if(n.constructor===e.constructor)!function(e,t){1===t._state?v(e,t._result):2===t._state?S(e,t._result):C(t,void 0,(function(t){b(e,t)}),(function(t){S(e,t)}))}(e,n);else{var r=function(e){try{return e.then}catch(e){return g.error=e,g}}(n);r===g?S(e,g.error):void 0===r?v(e,n):t(r)?function(e,t,n){o((function(e){var r=!1,i=function(e,t,n,r){try{e.call(t,n,r)}catch(e){return e}}(n,t,(function(n){r||(r=!0,t!==n?b(e,n):v(e,n))}),(function(t){r||(r=!0,S(e,t))}),e._label);!r&&i&&(r=!0,S(e,i))}),e)}(e,n,r):v(e,n)}}function b(e,t){var n;e===t?S(e,new TypeError("You cannot resolve a promise with itself")):"function"==typeof(n=t)||"object"==typeof n&&null!==n?_(e,t):v(e,t)}function O(e){e._onerror&&e._onerror(e._result),P(e)}function v(e,t){e._state===m&&(e._result=t,e._state=1,0!==e._subscribers.length&&o(P,e))}function S(e,t){e._state===m&&(e._state=2,e._result=t,o(O,e))}function C(e,t,n,r){var i=e._subscribers,s=i.length;e._onerror=null,i[s]=t,i[s+1]=n,i[s+2]=r,0===s&&e._state&&o(P,e)}function P(e){var t=e._subscribers,n=e._state;if(0!==t.length){for(var r,i,o=e._result,s=0;s<t.length;s+=3)r=t[s],i=t[s+n],r?D(n,r,i,o):i(o);e._subscribers.length=0}}function A(){this.error=null}var F=new A;function D(e,n,r,i){var o,s,a,l,u=t(r);if(u){if((o=function(e,t){try{return e(t)}catch(e){return F.error=e,F}}(r,i))===F?(l=!0,s=o.error,o=null):a=!0,n===o)return void S(n,new TypeError("A promises callback cannot return that same promise."))}else o=i,a=!0;n._state!==m||(u&&a?b(n,o):l?S(n,s):1===e?v(n,o):2===e&&S(n,o))}function E(e,t){var n=this;n._instanceConstructor=e,n.promise=new e(y),n._validateInput(t)?(n._input=t,n.length=t.length,n._remaining=t.length,n._init(),0===n.length?v(n.promise,n._result):(n.length=n.length||0,n._enumerate(),0===n._remaining&&v(n.promise,n._result))):S(n.promise,n._validationError())}E.prototype._validateInput=function(e){return r(e)},E.prototype._validationError=function(){return new e.Error("Array Methods must be provided an Array")},E.prototype._init=function(){this._result=new Array(this.length)};var R=E;E.prototype._enumerate=function(){for(var e=this,t=e.length,n=e.promise,r=e._input,i=0;n._state===m&&i<t;i++)e._eachEntry(r[i],i)},E.prototype._eachEntry=function(e,t){var n,r=this,i=r._instanceConstructor;"object"==typeof(n=e)&&null!==n?e.constructor===i&&e._state!==m?(e._onerror=null,r._settledAt(e._state,t,e._result)):r._willSettleAt(i.resolve(e),t):(r._remaining--,r._result[t]=e)},E.prototype._settledAt=function(e,t,n){var r=this,i=r.promise;i._state===m&&(r._remaining--,2===e?S(i,n):r._result[t]=n),0===r._remaining&&v(i,r._result)},E.prototype._willSettleAt=function(e,t){var n=this;C(e,void 0,(function(e){n._settledAt(1,t,e)}),(function(e){n._settledAt(2,t,e)}))};var I=function(e){return new R(this,e).promise};var x=function(e){var t=new this(y);if(!r(e))return S(t,new TypeError("You must pass an array to race.")),t;var n=e.length;function i(e){b(t,e)}function o(e){S(t,e)}for(var s=0;t._state===m&&s<n;s++)C(this.resolve(e[s]),void 0,i,o);return t};var N=function(e){if(e&&"object"==typeof e&&e.constructor===this)return e;var t=new this(y);return b(t,e),t};var T=function(e){var t=new this(y);return S(t,e),t},w=0;var j=M;function M(e){this._id=w++,this._state=void 0,this._result=void 0,this._subscribers=[],y!==e&&(t(e)||function(){throw new TypeError("You must pass a resolver function as the first argument to the promise constructor")}(),this instanceof M||function(){throw new TypeError("Failed to construct 'Promise': Please use the 'new' operator, this object constructor cannot be called as a function.")}(),function(e,t){try{t((function(t){b(e,t)}),(function(t){S(e,t)}))}catch(t){S(e,t)}}(this,e))}return M.all=I,M.race=x,M.resolve=N,M.reject=T,M._setScheduler=function(e){n=e},M._setAsap=function(e){o=e},M._asap=o,M.prototype={constructor:M,then:function(e,t){var n=this,r=n._state;if(1===r&&!e||2===r&&!t)return this;var i=new this.constructor(y),s=n._result;if(r){var a=arguments[r-1];o((function(){D(r,i,a,s)}))}else C(n,i,e,t);return i},catch:function(e){return this.then(null,e)}},j}.call(this)}}(e.PromiseImpl||(e.PromiseImpl={}))}(e._Internal||(e._Internal={})),function(e){var t,n;e.OfficePromise="undefined"==typeof window&&"function"==typeof Promise?Promise:"undefined"!=typeof window&&window.Promise?(t=window.navigator.userAgent,(n=t.indexOf("Edge/"))>=0&&(t=t.substring(n+5,t.length))<"14.14393"?e.PromiseImpl.Init():window.Promise):e.PromiseImpl.Init()}(t=e._Internal||(e._Internal={}));var n=t.OfficePromise;e.Promise=n}(OfficeExtension||(OfficeExtension={})),function(e){var t=function(){function e(){}return e.prototype._resolveRequestUrlAndHeaderInfo=function(){return u._createPromiseFromResult(null)},e.prototype._createRequestExecutorOrNull=function(){return null},e.prototype.getEventRegistration=function(e){return null},e}();e.SessionBase=t;var n=function(){function e(){}return e.setCustomSendRequestFunc=function(t){e.s_customSendRequestFunc=t},e.xhrSendRequestFunc=function(e){return u.createPromise((function(t,n){var i=new XMLHttpRequest;if(i.open(e.method,e.url),i.onload=function(){var e={statusCode:i.status,headers:u._parseHttpResponseHeaders(i.getAllResponseHeaders()),body:i.responseText};t(e)},i.onerror=function(){n(new r.RuntimeError({code:o.connectionFailure,httpStatusCode:i.status,message:u._getResourceString(s.connectionFailureWithStatus,i.statusText)}))},e.headers)for(var a in e.headers)i.setRequestHeader(a,e.headers[a]);i.send(u._getRequestBodyText(e))}))},e.fetchSendRequestFunc=function(e){var t=u._getRequestBodyText(e);return""===t&&(t=void 0),fetch(e.url,{method:e.method,headers:e.headers,body:t}).then((function(e){return e.text().then((function(t){var n=e.status,r={};return e.headers.forEach((function(e,t){r[t]=e})),{statusCode:n,headers:r,body:t}}))}))},e.sendRequest=function(t){e.validateAndNormalizeRequest(t);var n=e.s_customSendRequestFunc;return n||(n="undefined"!=typeof fetch?e.fetchSendRequestFunc:e.xhrSendRequestFunc),n(t)},e.setCustomSendLocalDocumentRequestFunc=function(t){e.s_customSendLocalDocumentRequestFunc=t},e.sendLocalDocumentRequest=function(t){return e.validateAndNormalizeRequest(t),(e.s_customSendLocalDocumentRequestFunc||e.officeJsSendLocalDocumentRequestFunc)(t)},e.officeJsSendLocalDocumentRequestFunc=function(e){e=u._validateLocalDocumentRequest(e);var t=u._buildRequestMessageSafeArray(e);return u.createPromise((function(e,n){OSF.DDA.RichApi.executeRichApiRequestAsync(t,(function(t){var n;n="succeeded"==t.status?{statusCode:l.getResponseStatusCode(t),headers:l.getResponseHeaders(t),body:l.getResponseBody(t)}:l.buildHttpResponseFromOfficeJsError(t.error.code,t.error.message),u.log("Response:"),u.log(JSON.stringify(n)),e(n)}))}))},e.validateAndNormalizeRequest=function(e){if(u.isNullOrUndefined(e))throw r.RuntimeError._createInvalidArgError({argumentName:"request"});u.isNullOrEmptyString(e.method)&&(e.method="GET"),e.method=e.method.toUpperCase();var t=!1;if("object"==typeof e.headers&&e.headers[a.testRequestNameHeader]&&(t=!0),!t){var n=c._getCurrentTestNameWithSequenceId();n&&(e.headers||(e.headers={}),e.headers[a.testRequestNameHeader]=n)}},e.logRequest=function(t){if(u._logEnabled){if(u.log("---HTTP Request---"),u.log(t.method+" "+t.url),t.headers)for(var n in t.headers)u.log(n+": "+t.headers[n]);e._logBodyEnabled&&u.log(u._getRequestBodyText(t))}},e.logResponse=function(t){if(u._logEnabled){if(u.log("---HTTP Response---"),u.log(""+t.statusCode),t.headers)for(var n in t.headers)u.log(n+": "+t.headers[n]);e._logBodyEnabled&&u.log(t.body)}},e._logBodyEnabled=!1,e}();e.HttpUtility=n;var r,i=function(){function e(e){var t=this;this.m_bridge=e,this.m_promiseResolver={},this.m_handlers=[],this.m_bridge.onMessageFromHost=function(e){var n=JSON.parse(e);if(3==n.type){var r=n.message;if(r&&r.entries)for(var i=0;i<r.entries.length;i++){var o=r.entries[i];if(Array.isArray(o)){var s={messageCategory:o[0],messageType:o[1],targetId:o[2],message:o[3],id:o[4]};r.entries[i]=s}}}t.dispatchMessage(n)}}return e.init=function(t){if("object"==typeof t&&t){var r=new e(t);e.s_instance=r,n.setCustomSendLocalDocumentRequestFunc((function(t){t=u._validateLocalDocumentRequest(t);var n=0;u.isReadonlyRestRequest(t.method)||(n=1);var i=t.url.indexOf("?");if(i>=0){var o=t.url.substr(i+1),s=u._parseRequestFlagsAndCustomDataFromQueryStringIfAny(o);s.flags>=0&&(n=s.flags)}"string"==typeof t.body&&(t.body=JSON.parse(t.body));var a={id:e.nextId(),type:1,flags:n,message:t};return r.sendMessageToHostAndExpectResponse(a).then((function(e){return e.message}))}));for(var i=0;i<e.s_onInitedHandlers.length;i++)e.s_onInitedHandlers[i](r)}},Object.defineProperty(e,"instance",{get:function(){return e.s_instance},enumerable:!0,configurable:!0}),e.prototype.sendMessageToHost=function(e){this.m_bridge.sendMessageToHost(JSON.stringify(e))},e.prototype.sendMessageToHostAndExpectResponse=function(e){var t=this,n=u.createPromise((function(n,r){t.m_promiseResolver[e.id]=n}));return this.m_bridge.sendMessageToHost(JSON.stringify(e)),n},e.prototype.addHostMessageHandler=function(e){this.m_handlers.push(e)},e.prototype.removeHostMessageHandler=function(e){var t=this.m_handlers.indexOf(e);t>=0&&this.m_handlers.splice(t,1)},e.onInited=function(t){e.s_onInitedHandlers.push(t),e.s_instance&&t(e.s_instance)},e.prototype.dispatchMessage=function(e){if("number"==typeof e.id){var t=this.m_promiseResolver[e.id];if(t)return t(e),void delete this.m_promiseResolver[e.id]}for(var n=0;n<this.m_handlers.length;n++)this.m_handlers[n](e)},e.nextId=function(){return e.s_nextId++},e.s_onInitedHandlers=[],e.s_nextId=1,e}();e.HostBridge=i,"object"==typeof _richApiNativeBridge&&_richApiNativeBridge&&i.init(_richApiNativeBridge),function(e){var t=function(t){function n(e){var r=t.call(this,"string"==typeof e?e:e.message)||this;if(Object.setPrototypeOf(r,n.prototype),r.name="RichApi.Error","string"==typeof e?r.message=e:(r.code=e.code,r.message=e.message,r.traceMessages=e.traceMessages||[],r.innerError=e.innerError||null,r.debugInfo=r._createDebugInfo(e.debugInfo||{}),r.httpStatusCode=e.httpStatusCode,r.data=e.data),u.isNullOrUndefined(r.httpStatusCode)||200===r.httpStatusCode){var i={};i[o.accessDenied]=401,i[o.connectionFailure]=500,i[o.generalException]=500,i[o.invalidArgument]=400,i[o.invalidObjectPath]=400,i[o.invalidOrTimedOutSession]=408,i[o.invalidRequestContext]=400,i[o.timeout]=408,i[o.valueNotLoaded]=400,r.httpStatusCode=i[r.code]}return u.isNullOrUndefined(r.httpStatusCode)&&(r.httpStatusCode=500),r}return __extends(n,t),n.prototype.toString=function(){return this.code+": "+this.message},n.prototype._createDebugInfo=function(t){var n={code:this.code,message:this.message,toString:function(){return JSON.stringify(this)}};for(var r in t)n[r]=t[r];return this.innerError&&(this.innerError instanceof e.RuntimeError?n.innerError=this.innerError.debugInfo:n.innerError=this.innerError),n},n._createInvalidArgError=function(t){return new e.RuntimeError({code:o.invalidArgument,httpStatusCode:400,message:u.isNullOrEmptyString(t.argumentName)?u._getResourceString(s.invalidArgumentGeneric):u._getResourceString(s.invalidArgument,t.argumentName),debugInfo:t.errorLocation?{errorLocation:t.errorLocation}:{},innerError:t.innerError})},n}(Error);e.RuntimeError=t}(r=e._Internal||(e._Internal={})),e.Error=r.RuntimeError;var o=function(){function e(){}return e.apiNotFound="ApiNotFound",e.accessDenied="AccessDenied",e.generalException="GeneralException",e.activityLimitReached="ActivityLimitReached",e.invalidArgument="InvalidArgument",e.connectionFailure="ConnectionFailure",e.timeout="Timeout",e.invalidOrTimedOutSession="InvalidOrTimedOutSession",e.invalidObjectPath="InvalidObjectPath",e.invalidRequestContext="InvalidRequestContext",e.valueNotLoaded="ValueNotLoaded",e.requestPayloadSizeLimitExceeded="RequestPayloadSizeLimitExceeded",e.responsePayloadSizeLimitExceeded="ResponsePayloadSizeLimitExceeded",e.writeNotSupportedWhenModalDialogOpen="WriteNotSupportedWhenModalDialogOpen",e}();e.CoreErrorCodes=o;var s=function(){function e(){}return e.apiNotFoundDetails="ApiNotFoundDetails",e.connectionFailureWithStatus="ConnectionFailureWithStatus",e.connectionFailureWithDetails="ConnectionFailureWithDetails",e.invalidArgument="InvalidArgument",e.invalidArgumentGeneric="InvalidArgumentGeneric",e.timeout="Timeout",e.invalidOrTimedOutSessionMessage="InvalidOrTimedOutSessionMessage",e.invalidSheetName="InvalidSheetName",e.invalidObjectPath="InvalidObjectPath",e.invalidRequestContext="InvalidRequestContext",e.valueNotLoaded="ValueNotLoaded",e}();e.CoreResourceStrings=s;var a=function(){function e(){}return e.flags="flags",e.sourceLibHeader="SdkVersion",e.processQuery="ProcessQuery",e.localDocument="http://document.localhost/",e.localDocumentApiPrefix="http://document.localhost/_api/",e.customData="customdata",e.testRequestNameHeader="x-test-request-name",e}();e.CoreConstants=a;var l=function(){function e(){}return e.buildMessageArrayForIRequestExecutor=function(t,n,r,i){var o=JSON.stringify(r.Body);u.log("Request:"),u.log(o);var s={};return u._copyHeaders(r.Headers,s),s[a.sourceLibHeader]=i,e.buildRequestMessageSafeArray(t,n,"POST",a.processQuery,s,o)},e.buildResponseOnSuccess=function(e,t){var n={HttpStatusCode:200,ErrorCode:"",ErrorMessage:"",Headers:null,Body:null};return n.Body=JSON.parse(e),n.Headers=t,n},e.buildResponseOnError=function(t,n){var r={HttpStatusCode:500,ErrorCode:"",ErrorMessage:"",Headers:null,Body:null};return r.ErrorCode=o.generalException,r.ErrorMessage=n,t==e.OfficeJsErrorCode_ooeNoCapability?(r.ErrorCode=o.accessDenied,r.HttpStatusCode=401):t==e.OfficeJsErrorCode_ooeActivityLimitReached?(r.ErrorCode=o.activityLimitReached,r.HttpStatusCode=429):t==e.OfficeJsErrorCode_ooeInvalidOrTimedOutSession?(r.ErrorCode=o.invalidOrTimedOutSession,r.HttpStatusCode=408,r.ErrorMessage=u._getResourceString(s.invalidOrTimedOutSessionMessage)):t==e.OfficeJsErrorCode_ooeRequestPayloadSizeLimitExceeded?(r.ErrorCode=o.requestPayloadSizeLimitExceeded,r.HttpStatusCode=400):t==e.OfficeJsErrorCode_ooeResponsePayloadSizeLimitExceeded?(r.ErrorCode=o.responsePayloadSizeLimitExceeded,r.HttpStatusCode=400):t==e.OfficeJsErrorCode_ooeWriteNotSupportedWhenModalDialogOpen?(r.ErrorCode=o.writeNotSupportedWhenModalDialogOpen,r.HttpStatusCode=400):t==e.OfficeJsErrorCode_ooeInvalidSheetName&&(r.ErrorCode=o.invalidRequestContext,r.HttpStatusCode=400,r.ErrorMessage=u._getResourceString(s.invalidSheetName)),r},e.buildHttpResponseFromOfficeJsError=function(t,n){var r=500,i={error:{}};return i.error.code=o.generalException,i.error.message=n,t===e.OfficeJsErrorCode_ooeNoCapability?(r=403,i.error.code=o.accessDenied):t===e.OfficeJsErrorCode_ooeActivityLimitReached&&(r=429,i.error.code=o.activityLimitReached),{statusCode:r,headers:{},body:JSON.stringify(i)}},e.buildRequestMessageSafeArray=function(e,t,n,r,i,o){var s=[];if(i)for(var a in i)s.push(a),s.push(i[a]);return[e,n,r,s,o,0,t,"","","","","",""]},e.getResponseBody=function(t){return e.getResponseBodyFromSafeArray(t.value.data)},e.getResponseHeaders=function(t){return e.getResponseHeadersFromSafeArray(t.value.data)},e.getResponseBodyFromSafeArray=function(e){var t=e[2];return"string"==typeof t?t:t.join("")},e.getResponseHeadersFromSafeArray=function(e){var t=e[1];if(!t)return null;for(var n={},r=0;r<t.length-1;r+=2)n[t[r]]=t[r+1];return n},e.getResponseStatusCode=function(t){return e.getResponseStatusCodeFromSafeArray(t.value.data)},e.getResponseStatusCodeFromSafeArray=function(e){return e[0]},e.OfficeJsErrorCode_ooeInvalidOrTimedOutSession=5012,e.OfficeJsErrorCode_ooeActivityLimitReached=5102,e.OfficeJsErrorCode_ooeNoCapability=7e3,e.OfficeJsErrorCode_ooeRequestPayloadSizeLimitExceeded=5103,e.OfficeJsErrorCode_ooeResponsePayloadSizeLimitExceeded=5104,e.OfficeJsErrorCode_ooeWriteNotSupportedWhenModalDialogOpen=5016,e.OfficeJsErrorCode_ooeInvalidSheetName=1014,e}();e.RichApiMessageUtility=l,function(e){e.getPromiseType=function(){if("undefined"!=typeof Promise)return Promise;if(void 0!==Office&&Office.Promise)return Office.Promise;if(void 0!==OfficeExtension&&OfficeExtension.Promise)return OfficeExtension.Promise;throw new e.Error("No Promise implementation found")}}(r=e._Internal||(e._Internal={}));var u=function(){function e(){}return e.log=function(t){e._logEnabled&&"undefined"!=typeof console&&console.log&&console.log(t)},e.checkArgumentNull=function(t,n){if(e.isNullOrUndefined(t))throw r.RuntimeError._createInvalidArgError({argumentName:n})},e.isNullOrUndefined=function(e){return null===e||void 0===e},e.isUndefined=function(e){return void 0===e},e.isNullOrEmptyString=function(e){return null===e||(void 0===e||0==e.length)},e.isPlainJsonObject=function(t){if(e.isNullOrUndefined(t))return!1;if("object"!=typeof t)return!1;if("[object Object]"!==Object.prototype.toString.apply(t))return!1;if(t.constructor&&!Object.prototype.hasOwnProperty.call(t,"constructor")&&!Object.prototype.hasOwnProperty.call(t.constructor.prototype,"hasOwnProperty"))return!1;for(var n in t)if(!Object.prototype.hasOwnProperty.call(t,n))return!1;return!0},e.trim=function(e){return e.replace(new RegExp("^\\s+|\\s+$","g"),"")},e.caseInsensitiveCompareString=function(t,n){return e.isNullOrUndefined(t)?e.isNullOrUndefined(n):!e.isNullOrUndefined(n)&&t.toUpperCase()==n.toUpperCase()},e.isReadonlyRestRequest=function(t){return e.caseInsensitiveCompareString(t,"GET")},e._getResourceString=function(t,n){var r;if("undefined"!=typeof window&&window.Strings&&window.Strings.OfficeOM){var i="L_"+t,o=window.Strings.OfficeOM[i];o&&(r=o)}if(r||(r=e.s_resourceStringValues[t]),r||(r=t),!e.isNullOrUndefined(n))if(Array.isArray(n)){var s=n;r=e._formatString(r,s)}else r=r.replace("{0}",n);return r},e._formatString=function(e,t){return e.replace(/\{\d\}/g,(function(e){var n=parseInt(e.substr(1,e.length-2));if(n<t.length)return t[n];throw r.RuntimeError._createInvalidArgError({argumentName:"format"})}))},Object.defineProperty(e,"Promise",{get:function(){return r.getPromiseType()},enumerable:!0,configurable:!0}),e.createPromise=function(t){return new e.Promise(t)},e._createPromiseFromResult=function(t){return e.createPromise((function(e,n){e(t)}))},e._createPromiseFromException=function(t){return e.createPromise((function(e,n){n(t)}))},e._createTimeoutPromise=function(t){return e.createPromise((function(e,n){setTimeout((function(){e(null)}),t)}))},e._createInvalidArgError=function(e){return r.RuntimeError._createInvalidArgError(e)},e._isLocalDocumentUrl=function(t){return e._getLocalDocumentUrlPrefixLength(t)>0},e._getLocalDocumentUrlPrefixLength=function(e){for(var t=["http://document.localhost","https://document.localhost","//document.localhost"],n=e.toLowerCase().trim(),r=0;r<t.length;r++){if(n===t[r])return t[r].length;if(n.substr(0,t[r].length+1)===t[r]+"/")return t[r].length+1}return 0},e._validateLocalDocumentRequest=function(t){var n=e._getLocalDocumentUrlPrefixLength(t.url);if(n<=0)throw r.RuntimeError._createInvalidArgError({argumentName:"request"});var i=t.url.substr(n),o=i.toLowerCase();return"_api"===o?i="":"_api/"===o.substr(0,"_api/".length)&&(i=i.substr("_api/".length)),{method:t.method,url:i,headers:t.headers,body:t.body}},e._parseRequestFlagsAndCustomDataFromQueryStringIfAny=function(e){for(var t={flags:-1,customData:""},n=e.split("&"),r=0;r<n.length;r++){var i=n[r].split("=");if(i[0].toLowerCase()===a.flags){var o=parseInt(i[1]);o&=8191,t.flags=o}else i[0].toLowerCase()===a.customData&&(t.customData=decodeURIComponent(i[1]))}return t},e._getRequestBodyText=function(e){var t="";return"string"==typeof e.body?t=e.body:e.body&&"object"==typeof e.body&&(t=JSON.stringify(e.body)),t},e._parseResponseBody=function(t){if("string"==typeof t.body){var n=e.trim(t.body);return JSON.parse(n)}return t.body},e._buildRequestMessageSafeArray=function(t){var n=0;e.isReadonlyRestRequest(t.method)||(n=1);var r="";if(t.url.substr(0,a.processQuery.length).toLowerCase()===a.processQuery.toLowerCase()){var i=t.url.indexOf("?");if(i>0){var o=t.url.substr(i+1),s=e._parseRequestFlagsAndCustomDataFromQueryStringIfAny(o);s.flags>=0&&(n=s.flags),r=s.customData}}return l.buildRequestMessageSafeArray(r,n,t.method,t.url,t.headers,e._getRequestBodyText(t))},e._parseHttpResponseHeaders=function(t){var n={};if(!e.isNullOrEmptyString(t))for(var r=new RegExp("\r?\n"),i=t.split(r),o=0;o<i.length;o++){var s=i[o];if(null!=s){var a=s.indexOf(":");if(a>0){var l=s.substr(0,a),u=s.substr(a+1);l=e.trim(l),u=e.trim(u),n[l.toUpperCase()]=u}}}return n},e._parseErrorResponse=function(t){var n=e._parseErrorResponseBody(t),r=t.statusCode.toString();if(e.isNullOrUndefined(n)||"object"!=typeof n||!n.error)return e._createDefaultErrorResponse(r);var i=n.error,o=i.innerError;return o&&o.code?e._createErrorResponse(o.code,r,o.message):i.code?e._createErrorResponse(i.code,r,i.message):e._createDefaultErrorResponse(r)},e._parseErrorResponseBody=function(t){if(e.isPlainJsonObject(t.body))return t.body;if(!e.isNullOrEmptyString(t.body)){var n=e.trim(t.body);try{return JSON.parse(n)}catch(t){e.log("Error when parse "+n)}}},e._createDefaultErrorResponse=function(t){return{errorCode:o.connectionFailure,errorMessage:e._getResourceString(s.connectionFailureWithStatus,t)}},e._createErrorResponse=function(t,n,r){return{errorCode:t,errorMessage:e._getResourceString(s.connectionFailureWithDetails,[n,t,r])}},e._copyHeaders=function(e,t){if(e&&t)for(var n in e)t[n]=e[n]},e.addResourceStringValues=function(t){for(var n in t)e.s_resourceStringValues[n]=t[n]},e._logEnabled=!1,e.s_resourceStringValues={ApiNotFoundDetails:"The method or property {0} is part of the {1} requirement set, which is not available in your version of {2}.",ConnectionFailureWithStatus:"The request failed with status code of {0}.",ConnectionFailureWithDetails:"The request failed with status code of {0}, error code {1} and the following error message: {2}",InvalidArgument:"The argument '{0}' doesn't work for this situation, is missing, or isn't in the right format.",InvalidObjectPath:'The object path \'{0}\' isn\'t working for what you\'re trying to do. If you\'re using the object across multiple "context.sync" calls and outside the sequential execution of a ".run" batch, please use the "context.trackedObjects.add()" and "context.trackedObjects.remove()" methods to manage the object\'s lifetime.',InvalidRequestContext:"Cannot use the object across different request contexts.",Timeout:"The operation has timed out.",ValueNotLoaded:'The value of the result object has not been loaded yet. Before reading the value property, call "context.sync()" on the associated request context.'},e}();e.CoreUtility=u;var c=function(){function e(){}return e.setMock=function(t){e.s_isMock=t},e.isMock=function(){return e.s_isMock},e._setCurrentTestName=function(t){e.s_currentTestName=t,e.s_currentTestSequenceId=0},e._getCurrentTestNameWithSequenceId=function(){return e.s_currentTestName?(e.s_currentTestSequenceId++,e.s_currentTestName+"."+e.s_currentTestSequenceId):null},e}();e.TestUtility=c,e._internalConfig={showDisposeInfoInDebugInfo:!1,showInternalApiInDebugInfo:!1,enableEarlyDispose:!0,alwaysPolyfillClientObjectUpdateMethod:!1,alwaysPolyfillClientObjectRetrieveMethod:!1,enableConcurrentFlag:!0,enableUndoableFlag:!0,appendTypeNameToObjectPathInfo:!1,enablePreviewExecution:!1},e.config={extendedErrorLogging:!1};var p=function(){function t(){}return t.createSetPropertyAction=function(t,n,r,i,o){S.validateObjectPath(n);var s={Id:t._nextId(),ActionType:4,Name:r,ObjectPathId:n._objectPath.objectPathInfo.Id,ArgumentInfo:{}},a=[i],l=S.setMethodArguments(t,s.ArgumentInfo,a);S.validateReferencedObjectPaths(l);var u=new h(s,0,o);if(u.referencedObjectPath=n._objectPath,u.referencedArgumentObjectPaths=l,e._internalConfig.enablePreviewExecution&&0!=(16&o)){var c={Id:t._nextId(),ActionType:4,Name:r,ObjectId:"",ObjectType:"",Arguments:[i]};n._addPreviewExecutionAction(c)}return n._addAction(u)},t.createQueryAction=function(e,t,n,r){S.validateObjectPath(t);var i={Id:e._nextId(),ActionType:2,Name:"",ObjectPathId:t._objectPath.objectPathInfo.Id,QueryInfo:n},o=new h(i,1,4);return o.referencedObjectPath=t._objectPath,t._addAction(o,r)},t.createQueryAsJsonAction=function(e,t,n,r){S.validateObjectPath(t);var i={Id:e._nextId(),ActionType:7,Name:"",ObjectPathId:t._objectPath.objectPathInfo.Id,QueryInfo:n},o=new h(i,1,4);return o.referencedObjectPath=t._objectPath,t._addAction(o,r)},t.createUpdateAction=function(e,t,n){S.validateObjectPath(t);var r={Id:e._nextId(),ActionType:9,Name:"",ObjectPathId:t._objectPath.objectPathInfo.Id,ObjectState:n},i=new h(r,0,0);return i.referencedObjectPath=t._objectPath,t._addAction(i)},t}();e.CommonActionFactory=p;var f=function(){function t(e,t){this.m_contextBase=e,this.m_objectPath=t}return Object.defineProperty(t.prototype,"_objectPath",{get:function(){return this.m_objectPath},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_context",{get:function(){return this.m_contextBase},enumerable:!0,configurable:!0}),t.prototype._addAction=function(e,t){var n=this;return void 0===t&&(t=null),u.createPromise((function(r,i){n._context._addServiceApiAction(e,t,r,i)}))},t.prototype._addPreviewExecutionAction=function(e){},t.prototype._retrieve=function(t,n){var r=e._internalConfig.alwaysPolyfillClientObjectRetrieveMethod;r||(r=!S.isSetSupported("RichApiRuntime","1.1"));var i=y._parseQueryOption(t);return r?p.createQueryAction(this._context,this,i,n):p.createQueryAsJsonAction(this._context,this,i,n)},t.prototype._recursivelyUpdate=function(t){var n=e._internalConfig.alwaysPolyfillClientObjectUpdateMethod;n||(n=!S.isSetSupported("RichApiRuntime","1.2"));try{var i=this[v.scalarPropertyNames];i||(i=[]);var a=this[v.scalarPropertyUpdateable];if(!a){a=[];for(var l=0;l<i.length;l++)a.push(!1)}var c=this[v.navigationPropertyNames];c||(c=[]);var f={},h={},d=0;for(var y in t){var m=i.indexOf(y);if(m>=0){if(!a[m])throw new r.RuntimeError({code:o.invalidArgument,httpStatusCode:400,message:u._getResourceString(C.attemptingToSetReadOnlyProperty,y),debugInfo:{errorLocation:y}});f[y]=t[y],++d}else{if(!(c.indexOf(y)>=0))throw new r.RuntimeError({code:o.invalidArgument,httpStatusCode:400,message:u._getResourceString(C.propertyDoesNotExist,y),debugInfo:{errorLocation:y}});h[y]=t[y]}}if(d>0)if(n)for(l=0;l<i.length;l++){var g=f[y=i[l]];S.isUndefined(g)||p.createSetPropertyAction(this._context,this,y,g)}else p.createUpdateAction(this._context,this,f);for(var y in h){var _=this[y],b=h[y];_._recursivelyUpdate(b)}}catch(e){throw new r.RuntimeError({code:o.invalidArgument,httpStatusCode:400,message:u._getResourceString(s.invalidArgument,"properties"),debugInfo:{errorLocation:this._className+".update"},innerError:e})}},t}();e.ClientObjectBase=f;var h=function(){function e(e,t,n){this.m_actionInfo=e,this.m_operationType=t,this.m_flags=n}return Object.defineProperty(e.prototype,"actionInfo",{get:function(){return this.m_actionInfo},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"operationType",{get:function(){return this.m_operationType},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"flags",{get:function(){return this.m_flags},enumerable:!0,configurable:!0}),e}();e.Action=h;var d=function(){function t(e,t,n,r,i,o){this.m_objectPathInfo=e,this.m_parentObjectPath=t,this.m_isCollection=n,this.m_isInvalidAfterRequest=r,this.m_isValid=!0,this.m_operationType=i,this.m_flags=o}return Object.defineProperty(t.prototype,"id",{get:function(){var e=this.m_objectPathInfo.ArgumentInfo;if(e){var t=e.Arguments;if(t)return t[0]}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"parent",{get:function(){var e=this.m_parentObjectPath;if(e)return e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"parentId",{get:function(){return this.parent?this.parent.id:void 0},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"objectPathInfo",{get:function(){return this.m_objectPathInfo},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"operationType",{get:function(){return this.m_operationType},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"flags",{get:function(){return this.m_flags},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isCollection",{get:function(){return this.m_isCollection},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isInvalidAfterRequest",{get:function(){return this.m_isInvalidAfterRequest},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"parentObjectPath",{get:function(){return this.m_parentObjectPath},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"argumentObjectPaths",{get:function(){return this.m_argumentObjectPaths},set:function(e){this.m_argumentObjectPaths=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isValid",{get:function(){return this.m_isValid},set:function(e){this.m_isValid=e,!e&&6===this.m_objectPathInfo.ObjectPathType&&this.m_savedObjectPathInfo&&(t.copyObjectPathInfo(this.m_savedObjectPathInfo.pathInfo,this.m_objectPathInfo),this.m_parentObjectPath=this.m_savedObjectPathInfo.parent,this.m_isValid=!0,this.m_savedObjectPathInfo=null)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"originalObjectPathInfo",{get:function(){return this.m_originalObjectPathInfo},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"getByIdMethodName",{get:function(){return this.m_getByIdMethodName},set:function(e){this.m_getByIdMethodName=e},enumerable:!0,configurable:!0}),t.prototype._updateAsNullObject=function(){this.resetForUpdateUsingObjectData(),this.m_objectPathInfo.ObjectPathType=7,this.m_objectPathInfo.Name="",this.m_parentObjectPath=null},t.prototype.saveOriginalObjectPathInfo=function(){e.config.extendedErrorLogging&&!this.m_originalObjectPathInfo&&(this.m_originalObjectPathInfo={},t.copyObjectPathInfo(this.m_objectPathInfo,this.m_originalObjectPathInfo))},t.prototype.updateUsingObjectData=function(e,n){var r=e[v.referenceId];if(!u.isNullOrEmptyString(r)){if(!this.m_savedObjectPathInfo&&!this.isInvalidAfterRequest&&t.isRestorableObjectPath(this.m_objectPathInfo.ObjectPathType)){var i={};t.copyObjectPathInfo(this.m_objectPathInfo,i),this.m_savedObjectPathInfo={pathInfo:i,parent:this.m_parentObjectPath}}return this.saveOriginalObjectPathInfo(),this.resetForUpdateUsingObjectData(),this.m_objectPathInfo.ObjectPathType=6,this.m_objectPathInfo.Name=r,delete this.m_objectPathInfo.ParentObjectPathId,void(this.m_parentObjectPath=null)}if(n){var o=n[v.collectionPropertyPath];if(!u.isNullOrEmptyString(o)&&n.context){var s=S.tryGetObjectIdFromLoadOrRetrieveResult(e);if(!u.isNullOrUndefined(s)){for(var a=o.split("."),l=n.context[a[0]],c=1;c<a.length;c++)l=l[a[c]];return this.saveOriginalObjectPathInfo(),this.resetForUpdateUsingObjectData(),this.m_parentObjectPath=l._objectPath,this.m_objectPathInfo.ParentObjectPathId=this.m_parentObjectPath.objectPathInfo.Id,this.m_objectPathInfo.ObjectPathType=5,this.m_objectPathInfo.Name="",void(this.m_objectPathInfo.ArgumentInfo.Arguments=[s])}}}var p=this.parentObjectPath&&this.parentObjectPath.isCollection,f=this.getByIdMethodName;if(p||!u.isNullOrEmptyString(f)){s=S.tryGetObjectIdFromLoadOrRetrieveResult(e);if(!u.isNullOrUndefined(s))return this.saveOriginalObjectPathInfo(),this.resetForUpdateUsingObjectData(),u.isNullOrEmptyString(f)?(this.m_objectPathInfo.ObjectPathType=5,this.m_objectPathInfo.Name=""):(this.m_objectPathInfo.ObjectPathType=3,this.m_objectPathInfo.Name=f),void(this.m_objectPathInfo.ArgumentInfo.Arguments=[s])}},t.prototype.resetForUpdateUsingObjectData=function(){this.m_isInvalidAfterRequest=!1,this.m_isValid=!0,this.m_operationType=1,this.m_flags=4,this.m_objectPathInfo.ArgumentInfo={},this.m_argumentObjectPaths=null,this.m_getByIdMethodName=null},t.isRestorableObjectPath=function(e){return 1===e||5===e||3===e||4===e},t.copyObjectPathInfo=function(e,t){t.Id=e.Id,t.ArgumentInfo=e.ArgumentInfo,t.Name=e.Name,t.ObjectPathType=e.ObjectPathType,t.ParentObjectPathId=e.ParentObjectPathId},t}();e.ObjectPath=d;var y=function(){function e(){this.m_nextId=0}return e.prototype._nextId=function(){return++this.m_nextId},e.prototype._addServiceApiAction=function(e,t,n,r){this.m_serviceApiQueue||(this.m_serviceApiQueue=new b(this)),this.m_serviceApiQueue.add(e,t,n,r)},e._parseQueryOption=function(t){var n={};if("string"==typeof t){var i=t;n.Select=S._parseSelectExpand(i)}else if(Array.isArray(t))n.Select=t;else if("object"==typeof t){var o=t;if(e.isLoadOption(o)){if("string"==typeof o.select)n.Select=S._parseSelectExpand(o.select);else if(Array.isArray(o.select))n.Select=o.select;else if(!S.isNullOrUndefined(o.select))throw r.RuntimeError._createInvalidArgError({argumentName:"option.select"});if("string"==typeof o.expand)n.Expand=S._parseSelectExpand(o.expand);else if(Array.isArray(o.expand))n.Expand=o.expand;else if(!S.isNullOrUndefined(o.expand))throw r.RuntimeError._createInvalidArgError({argumentName:"option.expand"});if("number"==typeof o.top)n.Top=o.top;else if(!S.isNullOrUndefined(o.top))throw r.RuntimeError._createInvalidArgError({argumentName:"option.top"});if("number"==typeof o.skip)n.Skip=o.skip;else if(!S.isNullOrUndefined(o.skip))throw r.RuntimeError._createInvalidArgError({argumentName:"option.skip"})}else n=e.parseStrictLoadOption(t)}else if(!S.isNullOrUndefined(t))throw r.RuntimeError._createInvalidArgError({argumentName:"option"});return n},e.isLoadOption=function(e){if(!S.isUndefined(e.select)&&("string"==typeof e.select||Array.isArray(e.select)))return!0;if(!S.isUndefined(e.expand)&&("string"==typeof e.expand||Array.isArray(e.expand)))return!0;if(!S.isUndefined(e.top)&&"number"==typeof e.top)return!0;if(!S.isUndefined(e.skip)&&"number"==typeof e.skip)return!0;for(var t in e)return!1;return!0},e.parseStrictLoadOption=function(t){var n={Select:[]};return e.parseStrictLoadOptionHelper(n,"","option",t),n},e.combineQueryPath=function(e,t,n){return 0===e.length?t:e+n+t},e.parseStrictLoadOptionHelper=function(t,n,i,o){for(var s in o){var a=o[s];if("$all"===s){if("boolean"!=typeof a)throw r.RuntimeError._createInvalidArgError({argumentName:e.combineQueryPath(i,s,".")});a&&t.Select.push(e.combineQueryPath(n,"*","/"))}else if("$top"===s){if("number"!=typeof a||n.length>0)throw r.RuntimeError._createInvalidArgError({argumentName:e.combineQueryPath(i,s,".")});t.Top=a}else if("$skip"===s){if("number"!=typeof a||n.length>0)throw r.RuntimeError._createInvalidArgError({argumentName:e.combineQueryPath(i,s,".")});t.Skip=a}else if("boolean"==typeof a)a&&t.Select.push(e.combineQueryPath(n,s,"/"));else{if("object"!=typeof a)throw r.RuntimeError._createInvalidArgError({argumentName:e.combineQueryPath(i,s,".")});e.parseStrictLoadOptionHelper(t,e.combineQueryPath(n,s,"/"),e.combineQueryPath(i,s,"."),a)}}},e}();e.ClientRequestContextBase=y;var m=function(){function e(e){this.m_objectPath=e}return e.prototype._handleResult=function(e){u.isNullOrUndefined(e)?this.m_objectPath._updateAsNullObject():this.m_objectPath.updateUsingObjectData(e,null)},e}(),g=function(){function t(e){this.m_contextBase=e,this.m_actions=[],this.m_actionResultHandler={},this.m_referencedObjectPaths={},this.m_instantiatedObjectPaths={},this.m_preSyncPromises=[],this.m_previewExecutionActions=[]}return t.prototype.addAction=function(e){this.m_actions.push(e),1==e.actionInfo.ActionType&&(this.m_instantiatedObjectPaths[e.actionInfo.ObjectPathId]=e)},t.prototype.addPreviewExecutionAction=function(e){this.m_previewExecutionActions.push(e)},Object.defineProperty(t.prototype,"hasActions",{get:function(){return this.m_actions.length>0},enumerable:!0,configurable:!0}),t.prototype._getLastAction=function(){return this.m_actions[this.m_actions.length-1]},t.prototype.ensureInstantiateObjectPath=function(e){if(e){if(this.m_instantiatedObjectPaths[e.objectPathInfo.Id])return;if(this.ensureInstantiateObjectPath(e.parentObjectPath),this.ensureInstantiateObjectPaths(e.argumentObjectPaths),!this.m_instantiatedObjectPaths[e.objectPathInfo.Id]){var t={Id:this.m_contextBase._nextId(),ActionType:1,Name:"",ObjectPathId:e.objectPathInfo.Id},n=new h(t,1,4);n.referencedObjectPath=e,this.addReferencedObjectPath(e),this.addAction(n);var r=new m(e);this.addActionResultHandler(n,r)}}},t.prototype.ensureInstantiateObjectPaths=function(e){if(e)for(var t=0;t<e.length;t++)this.ensureInstantiateObjectPath(e[t])},t.prototype.addReferencedObjectPath=function(e){if(e&&!this.m_referencedObjectPaths[e.objectPathInfo.Id]){if(!e.isValid)throw new r.RuntimeError({code:o.invalidObjectPath,httpStatusCode:400,message:u._getResourceString(s.invalidObjectPath,S.getObjectPathExpression(e)),debugInfo:{errorLocation:S.getObjectPathExpression(e)}});for(;e;)this.m_referencedObjectPaths[e.objectPathInfo.Id]=e,3==e.objectPathInfo.ObjectPathType&&this.addReferencedObjectPaths(e.argumentObjectPaths),e=e.parentObjectPath}},t.prototype.addReferencedObjectPaths=function(e){if(e)for(var t=0;t<e.length;t++)this.addReferencedObjectPath(e[t])},t.prototype.addActionResultHandler=function(e,t){this.m_actionResultHandler[e.actionInfo.Id]=t},t.prototype.aggregrateRequestFlags=function(e,t,n){return 0===t&&(e|=1,0==(2&n)&&(e&=-17),0==(8&n)&&(e&=-257),e&=-5),1&n&&(e|=2),0==(4&n)&&(e&=-5),e},t.prototype.finallyNormalizeFlags=function(t){return 0==(1&t)&&(t&=-17,t&=-257),e._internalConfig.enableConcurrentFlag||(t&=-5),e._internalConfig.enableUndoableFlag||(t&=-17),S.isSetSupported("RichApiRuntimeFlag","1.1")||(t&=-5,t&=-17),S.isSetSupported("RichApiRuntimeFlag","1.2")||(t&=-257),"number"==typeof this.m_flagsForTesting&&(t=this.m_flagsForTesting),t},t.prototype.buildRequestMessageBodyAndRequestFlags=function(){e._internalConfig.enableEarlyDispose&&t._calculateLastUsedObjectPathIds(this.m_actions);var n=276,r={};for(var i in this.m_referencedObjectPaths)n=this.aggregrateRequestFlags(n,this.m_referencedObjectPaths[i].operationType,this.m_referencedObjectPaths[i].flags),r[i]=this.m_referencedObjectPaths[i].objectPathInfo;for(var o=[],s=!1,a=0;a<this.m_actions.length;a++){var l=this.m_actions[a];3===l.actionInfo.ActionType&&l.actionInfo.Name===v.keepReference&&(s=!0),n=this.aggregrateRequestFlags(n,l.operationType,l.flags),o.push(l.actionInfo)}n=this.finallyNormalizeFlags(n);var u={AutoKeepReference:this.m_contextBase._autoCleanup&&s,Actions:o,ObjectPaths:r};return this.m_previewExecutionActions.length>0&&(u.PreviewExecutionActions=this.m_previewExecutionActions,n|=4096),{body:u,flags:n}},t.prototype.processResponse=function(e){if(e)for(var t=0;t<e.length;t++){var n=e[t],r=this.m_actionResultHandler[n.ActionId];r&&r._handleResult(n.Value)}},t.prototype.invalidatePendingInvalidObjectPaths=function(){for(var e in this.m_referencedObjectPaths)this.m_referencedObjectPaths[e].isInvalidAfterRequest&&(this.m_referencedObjectPaths[e].isValid=!1)},t.prototype._addPreSyncPromise=function(e){this.m_preSyncPromises.push(e)},Object.defineProperty(t.prototype,"_preSyncPromises",{get:function(){return this.m_preSyncPromises},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_actions",{get:function(){return this.m_actions},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_objectPaths",{get:function(){return this.m_referencedObjectPaths},enumerable:!0,configurable:!0}),t.prototype._removeKeepReferenceAction=function(e){for(var t=this.m_actions.length-1;t>=0;t--){var n=this.m_actions[t].actionInfo;if(n.ObjectPathId===e&&3===n.ActionType&&n.Name===v.keepReference){this.m_actions.splice(t,1);break}}},t._updateLastUsedActionIdOfObjectPathId=function(e,n,r){for(;n;){if(e[n.objectPathInfo.Id])return;e[n.objectPathInfo.Id]=r;var i=n.argumentObjectPaths;if(i)for(var o=i.length,s=0;s<o;s++)t._updateLastUsedActionIdOfObjectPathId(e,i[s],r);n=n.parentObjectPath}},t._calculateLastUsedObjectPathIds=function(e){for(var n={},r=e.length,i=r-1;i>=0;--i){var o=(f=e[i]).actionInfo.Id;f.referencedObjectPath&&t._updateLastUsedActionIdOfObjectPathId(n,f.referencedObjectPath,o);var s=f.referencedArgumentObjectPaths;if(s)for(var a=s.length,l=0;l<a;l++)t._updateLastUsedActionIdOfObjectPathId(n,s[l],o)}var u={};for(var c in n){var p=u[o=n[c]];p||(p=[],u[o]=p),p.push(parseInt(c))}for(i=0;i<r;i++){var f,h=u[(f=e[i]).actionInfo.Id];h&&h.length>0?f.actionInfo.L=h:f.actionInfo.L&&delete f.actionInfo.L}},t}();e.ClientRequestBase=g;var _=function(){function e(e){this.m_type=e}return Object.defineProperty(e.prototype,"value",{get:function(){if(!this.m_isLoaded)throw new r.RuntimeError({code:o.valueNotLoaded,httpStatusCode:400,message:u._getResourceString(s.valueNotLoaded),debugInfo:{errorLocation:"clientResult.value"}});return this.m_value},enumerable:!0,configurable:!0}),e.prototype._handleResult=function(e){this.m_isLoaded=!0,"object"==typeof e&&e&&e._IsNull||(1===this.m_type?this.m_value=S.adjustToDateTime(e):this.m_value=e)},e}();e.ClientResult=_;var b=function(){function e(e){this.m_context=e,this.m_actions=[]}return e.prototype.add=function(e,t,n,r){var i=this;this.m_actions.push({action:e,resultHandler:t,resolve:n,reject:r}),1===this.m_actions.length&&setTimeout((function(){return i.processActions()}),0)},e.prototype.processActions=function(){var e=this;if(0!==this.m_actions.length){var t=this.m_actions;this.m_actions=[];for(var n=new g(this.m_context),r=0;r<t.length;r++){var i=t[r];n.ensureInstantiateObjectPath(i.action.referencedObjectPath),n.ensureInstantiateObjectPaths(i.action.referencedArgumentObjectPaths),n.addAction(i.action),n.addReferencedObjectPath(i.action.referencedObjectPath),n.addReferencedObjectPaths(i.action.referencedArgumentObjectPaths)}var o=n.buildRequestMessageBodyAndRequestFlags(),s=o.body,l=o.flags,c={Url:a.localDocumentApiPrefix,Headers:null,Body:s};u.log("Request:"),u.log(JSON.stringify(s)),(new O).executeAsync(this.m_context._customData,l,c).then((function(r){e.processResponse(n,t,r)})).catch((function(e){for(var n=0;n<t.length;n++){t[n].reject(e)}}))}},e.prototype.processResponse=function(e,t,n){var r=this.getErrorFromResponse(n),i=null;n.Body.Results?i=n.Body.Results:n.Body.ProcessedResults&&n.Body.ProcessedResults.Results&&(i=n.Body.ProcessedResults.Results),i||(i=[]),this.processActionResults(e,t,i,r)},e.prototype.getErrorFromResponse=function(e){return u.isNullOrEmptyString(e.ErrorCode)?e.Body&&e.Body.Error?new r.RuntimeError({code:e.Body.Error.Code,httpStatusCode:e.Body.Error.HttpStatusCode,message:e.Body.Error.Message}):null:new r.RuntimeError({code:e.ErrorCode,httpStatusCode:e.HttpStatusCode,message:e.ErrorMessage})},e.prototype.processActionResults=function(e,t,n,r){e.processResponse(n);for(var i=0;i<t.length;i++){for(var o=t[i],s=o.action.actionInfo.Id,a=!1,l=0;l<n.length;l++)if(s==n[l].ActionId){var u=n[l].Value;o.resultHandler&&(o.resultHandler._handleResult(u),u=o.resultHandler.value),o.resolve&&o.resolve(u),a=!0;break}!a&&o.reject&&(r?o.reject(r):o.reject("No response for the action."))}},e}(),O=function(){function e(){}return e.prototype.getRequestUrl=function(e,t){return"/"!=e.charAt(e.length-1)&&(e+="/"),e=(e+=a.processQuery)+"?"+a.flags+"="+t.toString()},e.prototype.executeAsync=function(t,r,i){var o={method:"POST",url:this.getRequestUrl(i.Url,r),headers:{},body:i.Body};if(o.headers[a.sourceLibHeader]=e.SourceLibHeaderValue,o.headers["CONTENT-TYPE"]="application/json",i.Headers)for(var s in i.Headers)o.headers[s]=i.Headers[s];return(u._isLocalDocumentUrl(o.url)?n.sendLocalDocumentRequest:n.sendRequest)(o).then((function(e){var t;if(200===e.statusCode)t={HttpStatusCode:e.statusCode,ErrorCode:null,ErrorMessage:null,Headers:e.headers,Body:u._parseResponseBody(e)};else{u.log("Error Response:"+e.body);var n=u._parseErrorResponse(e);t={HttpStatusCode:e.statusCode,ErrorCode:n.errorCode,ErrorMessage:n.errorMessage,Headers:e.headers,Body:null,RawErrorResponseBody:u._parseErrorResponseBody(e)}}return t}))},e.SourceLibHeaderValue="officejs-rest",e}();e.HttpRequestExecutor=O;var v=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),t.collectionPropertyPath="_collectionPropertyPath",t.id="Id",t.idLowerCase="id",t.idPrivate="_Id",t.keepReference="_KeepReference",t.objectPathIdPrivate="_ObjectPathId",t.referenceId="_ReferenceId",t.items="_Items",t.itemsLowerCase="items",t.scalarPropertyNames="_scalarPropertyNames",t.scalarPropertyOriginalNames="_scalarPropertyOriginalNames",t.navigationPropertyNames="_navigationPropertyNames",t.scalarPropertyUpdateable="_scalarPropertyUpdateable",t.previewExecutionObjectId="_previewExecutionObjectId",t}(a);e.CommonConstants=v;var S=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),t.validateObjectPath=function(e){for(var n=e._objectPath;n;){if(!n.isValid)throw new r.RuntimeError({code:o.invalidObjectPath,httpStatusCode:400,message:u._getResourceString(s.invalidObjectPath,t.getObjectPathExpression(n)),debugInfo:{errorLocation:t.getObjectPathExpression(n)}});n=n.parentObjectPath}},t.validateReferencedObjectPaths=function(e){if(e)for(var n=0;n<e.length;n++)for(var i=e[n];i;){if(!i.isValid)throw new r.RuntimeError({code:o.invalidObjectPath,httpStatusCode:400,message:u._getResourceString(s.invalidObjectPath,t.getObjectPathExpression(i))});i=i.parentObjectPath}},t._toCamelLowerCase=function(e){if(u.isNullOrEmptyString(e))return e;for(var t=0;t<e.length&&e.charCodeAt(t)>=65&&e.charCodeAt(t)<=90;)t++;return t<e.length?e.substr(0,t).toLowerCase()+e.substr(t):e.toLowerCase()},t.adjustToDateTime=function(e){if(u.isNullOrUndefined(e))return null;if("string"==typeof e)return new Date(e);if(Array.isArray(e)){for(var n=e,r=0;r<n.length;r++)n[r]=t.adjustToDateTime(n[r]);return n}throw u._createInvalidArgError({argumentName:"date"})},t.tryGetObjectIdFromLoadOrRetrieveResult=function(e){var t=e[v.id];return u.isNullOrUndefined(t)&&(t=e[v.idLowerCase]),u.isNullOrUndefined(t)&&(t=e[v.idPrivate]),t},t.getObjectPathExpression=function(e){for(var n="";e;){switch(e.objectPathInfo.ObjectPathType){case 1:n=n;break;case 2:n="new()"+(n.length>0?".":"")+n;break;case 3:n=t.normalizeName(e.objectPathInfo.Name)+"()"+(n.length>0?".":"")+n;break;case 4:n=t.normalizeName(e.objectPathInfo.Name)+(n.length>0?".":"")+n;break;case 5:n="getItem()"+(n.length>0?".":"")+n;break;case 6:n="_reference()"+(n.length>0?".":"")+n}e=e.parentObjectPath}return n},t.setMethodArguments=function(e,n,r){if(u.isNullOrUndefined(r))return null;var i=new Array,o=new Array,s=t.collectObjectPathInfos(e,r,i,o);return n.Arguments=r,s&&(n.ReferencedObjectPathIds=o),i},t.validateContext=function(e,t){if(e&&t&&t._context!==e)throw new r.RuntimeError({code:o.invalidRequestContext,httpStatusCode:400,message:u._getResourceString(s.invalidRequestContext)})},t.isSetSupported=function(e,t){return!("undefined"!=typeof window&&window.Office&&window.Office.context&&window.Office.context.requirements)||window.Office.context.requirements.isSetSupported(e,t)},t.throwIfApiNotSupported=function(e,n,i,a){if(t._doApiNotSupportedCheck&&!t.isSetSupported(n,i)){var l=u._getResourceString(s.apiNotFoundDetails,[e,n+" "+i,a]);throw new r.RuntimeError({code:o.apiNotFound,httpStatusCode:404,message:l,debugInfo:{errorLocation:e}})}},t.calculateApiFlags=function(e,n,r){return t.isSetSupported(n,r)||(e&=-3),e},t._parseSelectExpand=function(e){var t=[];if(!u.isNullOrEmptyString(e))for(var n=e.split(","),r=0;r<n.length;r++){var i=n[r];(i=o(i.trim())).length>0&&t.push(i)}return t;function o(e){var t=e.toLowerCase();if("items"===t||"items/"===t)return"*";return("items/"===t.substr(0,6)||"items."===t.substr(0,6))&&(e=e.substr(6)),e.replace(new RegExp("[/.]items[/.]","gi"),"/")}},t.changePropertyNameToCamelLowerCase=function(e){if(Array.isArray(e)){for(var n=[],r=0;r<e.length;r++)n.push(this.changePropertyNameToCamelLowerCase(e[r]));return n}if("object"==typeof e&&null!==e){n={};for(var i in e){var o=e[i];if(i===v.items){(n={})[v.itemsLowerCase]=this.changePropertyNameToCamelLowerCase(o);break}n[t._toCamelLowerCase(i)]=this.changePropertyNameToCamelLowerCase(o)}return n}return e},t.purifyJson=function(e){if(Array.isArray(e)){for(var t=[],n=0;n<e.length;n++)t.push(this.purifyJson(e[n]));return t}if("object"==typeof e&&null!==e){t={};for(var r in e)if(95!==r.charCodeAt(0)){var i=e[r];"object"==typeof i&&null!==i&&Array.isArray(i.items)&&(i=i.items),t[r]=this.purifyJson(i)}return t}return e},t.collectObjectPathInfos=function(e,n,r,i){for(var o=!1,s=0;s<n.length;s++)if(n[s]instanceof f){var a=n[s];t.validateContext(e,a),n[s]=a._objectPath.objectPathInfo.Id,i.push(a._objectPath.objectPathInfo.Id),r.push(a._objectPath),o=!0}else if(Array.isArray(n[s])){var l=new Array;t.collectObjectPathInfos(e,n[s],r,l)?(i.push(l),o=!0):i.push(0)}else u.isPlainJsonObject(n[s])?(i.push(0),t.replaceClientObjectPropertiesWithObjectPathIds(n[s],r)):i.push(0);return o},t.replaceClientObjectPropertiesWithObjectPathIds=function(e,n){var r,i;for(var o in e){var s=e[o];if(s instanceof f)n.push(s._objectPath),e[o]=((r={})[v.objectPathIdPrivate]=s._objectPath.objectPathInfo.Id,r);else if(Array.isArray(s))for(var a=0;a<s.length;a++)if(s[a]instanceof f){var l=s[a];n.push(l._objectPath),s[a]=((i={})[v.objectPathIdPrivate]=l._objectPath.objectPathInfo.Id,i)}else u.isPlainJsonObject(s[a])&&t.replaceClientObjectPropertiesWithObjectPathIds(s[a],n);else u.isPlainJsonObject(s)&&t.replaceClientObjectPropertiesWithObjectPathIds(s,n)}},t.normalizeName=function(e){return e.substr(0,1).toLowerCase()+e.substr(1)},t._doApiNotSupportedCheck=!1,t}(u);e.CommonUtility=S;var C=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),t.propertyDoesNotExist="PropertyDoesNotExist",t.attemptingToSetReadOnlyProperty="AttemptingToSetReadOnlyProperty",t}(s);e.CommonResourceStrings=C;var P=function(e){function t(t){var n=e.call(this)||this;return n.m_shouldPolyfill=t,n}return __extends(t,e),t.prototype._handleResult=function(t){e.prototype._handleResult.call(this,t),this.m_shouldPolyfill&&(this.m_value=S.changePropertyNameToCamelLowerCase(this.m_value)),this.m_value=this.removeItemNodes(this.m_value)},t.prototype.removeItemNodes=function(e){return"object"==typeof e&&null!==e&&e[v.itemsLowerCase]&&(e=e[v.itemsLowerCase]),S.purifyJson(e)},t}(_);e.ClientRetrieveResult=P;var A=function(){function e(e){this.callback=e}return e.prototype._handleResult=function(e){this.callback&&this.callback()},e}(),F=function(e){function t(t){var n=e.call(this)||this;return n.callback=t,n}return __extends(t,e),t.prototype._handleResult=function(t){e.prototype._handleResult.call(this,t),this.callback()},t}(_);e.ClientResultCallback=F;var D=function(){function t(){}return t.invokeMethod=function(e,t,n,r,i,o){return void 0===n&&(n=0),void 0===r&&(r=[]),void 0===i&&(i=0),void 0===o&&(o=0),u.createPromise((function(o,s){var a=new _,l={Id:e._context._nextId(),ActionType:3,Name:t,ObjectPathId:e._objectPath.objectPathInfo.Id,ArgumentInfo:{}},u=S.setMethodArguments(e._context,l.ArgumentInfo,r),c=new h(l,n,i);c.referencedObjectPath=e._objectPath,c.referencedArgumentObjectPaths=u,e._context._addServiceApiAction(c,a,o,s)}))},t.invokeMethodWithClientResultCallback=function(e,t,n){var r=[];return u.createPromise((function(i,o){var s=new F(e),a={Id:t._context._nextId(),ActionType:3,Name:n,ObjectPathId:t._objectPath.objectPathInfo.Id,ArgumentInfo:{}},l=S.setMethodArguments(t._context,a.ArgumentInfo,r),u=new h(a,0,0);u.referencedObjectPath=t._objectPath,u.referencedArgumentObjectPaths=l,t._context._addServiceApiAction(u,s,i,o)}))},t.invokeRetrieve=function(n,r){var i,o=e._internalConfig.alwaysPolyfillClientObjectRetrieveMethod;if(o||(o=!S.isSetSupported("RichApiRuntime","1.1")),"object"==typeof r[0]&&r[0].hasOwnProperty("$all")){if(!r[0].$all)throw e.Error._createInvalidArgError({});i=r[0]}else i=t._parseSelectOption(r);return n._retrieve(i,new P(o))},t._parseSelectOption=function(n){if(!n||!n[0])throw e.Error._createInvalidArgError({});var r=n[0]&&"string"!=typeof n[0]?n[0]:n;return Array.isArray(r)?r:t.parseRecursiveSelect(r)},t.parseRecursiveSelect=function(e){var t=function(e){return Object.keys(e).reduce((function(n,r){var i=e[r];return"object"==typeof i?n.concat(t(i).map((function(e){return r+"/"+e}))):i?n.concat(r):n}),[])};return t(e)},t.invokeRecursiveUpdate=function(e,t){return u.createPromise((function(n,r){e._recursivelyUpdate(t);var i={Id:e._context._nextId(),ActionType:5,Name:"Trace",ObjectPathId:0},o=new h(i,1,4);e._context._addServiceApiAction(o,null,n,r)}))},t.createRootServiceObject=function(e,t){var n={Id:t._nextId(),ObjectPathType:1,Name:""};return new e(t,new d(n,null,!1,!1,1,4))},t.createTopLevelServiceObject=function(e,t,n,r,i){var o={Id:t._nextId(),ObjectPathType:2,Name:n};return new e(t,new d(o,null,r,!1,1,4|i))},t.createPropertyObject=function(e,t,n,r,i){var o={Id:t._context._nextId(),ObjectPathType:4,Name:n,ParentObjectPathId:t._objectPath.objectPathInfo.Id},s=new d(o,t._objectPath,r,!1,1,4|i);return new e(t._context,s)},t.createIndexerObject=function(e,t,n){var r={Id:t._context._nextId(),ObjectPathType:5,Name:"",ParentObjectPathId:t._objectPath.objectPathInfo.Id,ArgumentInfo:{}};r.ArgumentInfo.Arguments=n;var i=new d(r,t._objectPath,!1,!1,1,4);return new e(t._context,i)},t.createMethodObject=function(e,t,n,r,i,o,s,a,l){var u={Id:t._context._nextId(),ObjectPathType:3,Name:n,ParentObjectPathId:t._objectPath.objectPathInfo.Id,ArgumentInfo:{}},c=S.setMethodArguments(t._context,u.ArgumentInfo,i),p=new d(u,t._objectPath,o,s,r,l);return p.argumentObjectPaths=c,p.getByIdMethodName=a,new e(t._context,p)},t.createAndInstantiateMethodObject=function(e,t,n,r,i,o,s,a,l){return u.createPromise((function(u,c){var p={Id:t._context._nextId(),ObjectPathType:3,Name:n,ParentObjectPathId:t._objectPath.objectPathInfo.Id,ArgumentInfo:{}},f=S.setMethodArguments(t._context,p.ArgumentInfo,i),y=new d(p,t._objectPath,o,s,r,l);y.argumentObjectPaths=f,y.getByIdMethodName=a;var m=new _,g={Id:t._context._nextId(),ActionType:1,Name:"",ObjectPathId:y.objectPathInfo.Id,QueryInfo:{}},b=new h(g,1,4);b.referencedObjectPath=y,t._context._addServiceApiAction(b,m,(function(){return u(new e(t._context,y))}),c)}))},t.createTraceAction=function(e,t){return u.createPromise((function(n,r){var i={Id:e._nextId(),ActionType:5,Name:"Trace",ObjectPathId:0},o=new h(i,1,4),s=new A(t);e._addServiceApiAction(o,s,n,r)}))},t.localDocumentContext=new y,t}();e.OperationalApiHelper=D;var E=function(){function e(e,t,n){this.eventId=e,this.targetId=t,this.eventArgumentTransform=n,this.registeredCallbacks=[]}return e.prototype.add=function(e){this.hasZero()&&z.getGenericEventRegistration("").register(this.eventId,this.targetId,this.registerCallback),this.registeredCallbacks.push(e)},e.prototype.remove=function(e){var t=this.registeredCallbacks.lastIndexOf(e);-1!==t&&this.registeredCallbacks.splice(t,1)},e.prototype.removeAll=function(){this.registeredCallbacks=[],z.getGenericEventRegistration("").unregister(this.eventId,this.targetId,this.registerCallback)},e.prototype.hasZero=function(){return 0===this.registeredCallbacks.length},Object.defineProperty(e.prototype,"registerCallback",{get:function(){var e=this;return this.outsideCallback||(this.outsideCallback=function(t){e.call(t)}),this.outsideCallback},enumerable:!0,configurable:!0}),e.prototype.call=function(t){var n=this;this.eventArgumentTransform(t).then((function(t){var r=n.registeredCallbacks.map((function(n){return e.callCallback(n,t)}));u.Promise.all(r)}))},e.callCallback=function(t,n){return u._createPromiseFromResult(null).then(e.wrapCallbackInFunction(t,n)).catch((function(e){u.log("Error when invoke handler: "+JSON.stringify(e))}))},e.wrapCallbackInFunction=function(e,t){return function(){return e(t)}},e}();e.GenericEventRegistryOperational=E;var R=function(){function e(){this.eventToTargetToHandlerMap={}}return Object.defineProperty(e,"globalEventRegistry",{get:function(){return e.singleton||(e.singleton=new e),e.singleton},enumerable:!0,configurable:!0}),e.getGlobalEventRegistry=function(t,n,r){var i=e.globalEventRegistry.eventToTargetToHandlerMap;i.hasOwnProperty(t)||(i[t]={});var o=i[t];return o.hasOwnProperty(n)||(o[n]=new E(t,n,r)),o[n]},e.singleton=void 0,e}();e.GlobalEventRegistryOperational=R;var I=function(){function e(e){this.genericEventInfo=e}return e.prototype.add=function(e){var t=this,n=void 0,r=u.createPromise((function(e){n=e}));return this.register(),this.createTrace((function(){var r=t.genericEventInfo.eventType,i=t.genericEventInfo.getTargetIdFunc();R.getGlobalEventRegistry(r,i,t.genericEventInfo.eventArgsTransformFunc).add(e),n()})),r},e.prototype.remove=function(e){var t=this;this.register(),this.createTrace((function(){var n=t.genericEventInfo.eventType,r=t.genericEventInfo.getTargetIdFunc();R.getGlobalEventRegistry(n,r,t.genericEventInfo.eventArgsTransformFunc).remove(e)}))},e.prototype.removeAll=function(){var e=this;this.unregister(),this.createTrace((function(){var t=e.genericEventInfo.eventType,n=e.genericEventInfo.getTargetIdFunc();R.getGlobalEventRegistry(t,n,e.genericEventInfo.eventArgsTransformFunc).removeAll()}))},e.prototype.createTrace=function(e){D.createTraceAction(this.genericEventInfo.object._context,e)},e.prototype.register=function(){D.invokeMethod(this.genericEventInfo.object,this.genericEventInfo.register,0,[],0),z.getGenericEventRegistration("").isReady||z.getGenericEventRegistration("").ready()},e.prototype.unregister=function(){D.invokeMethod(this.genericEventInfo.object,this.genericEventInfo.unregister)},e}();e.GenericEventHandlerOperational=I;var x=function(){function e(){}return e.invokeOn=function(e,t,n){var r=void 0,i=u.createPromise((function(e,t){r=e}));return e.add(t).then((function(){r({})})),i},e.invokeOff=function(t,n,r,i){if(!r&&!i){var o=Object.keys(t).map((function(e){return t[e]}));return e.invokeAllOff(o)}return r?(i?n.remove(i):n.removeAll(),u.createPromise((function(e,t){return e()}))):u._createPromiseFromException(r+" must be supplied if handler is supplied.")},e.invokeAllOff=function(e){return e.forEach((function(e){e.removeAll()})),u.createPromise((function(e,t){return e()}))},e}();e.EventHelper=x;var N=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),t.propertyNotLoaded="PropertyNotLoaded",t.runMustReturnPromise="RunMustReturnPromise",t.cannotRegisterEvent="CannotRegisterEvent",t.invalidOrTimedOutSession="InvalidOrTimedOutSession",t.cannotUpdateReadOnlyProperty="CannotUpdateReadOnlyProperty",t}(o);e.ErrorCodes=N;var T=function(){function e(e){this.m_callback=e}return e.prototype._handleResult=function(e){this.m_callback&&this.m_callback()},e}(),w=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),n.createMethodAction=function(t,n,r,i,o,s){$.validateObjectPath(n);var a={Id:t._nextId(),ActionType:3,Name:r,ObjectPathId:n._objectPath.objectPathInfo.Id,ArgumentInfo:{}},l=$.setMethodArguments(t,a.ArgumentInfo,o);$.validateReferencedObjectPaths(l);var u=$._fixupApiFlags(s),c=new h(a,i,u);if(c.referencedObjectPath=n._objectPath,c.referencedArgumentObjectPaths=l,n._addAction(c),e._internalConfig.enablePreviewExecution&&0!=(16&u)){var p={Id:t._nextId(),ActionType:3,Name:r,Arguments:o,ObjectId:"",ObjectType:""};n._addPreviewExecutionAction(p)}return c},n.createRecursiveQueryAction=function(e,t,n){$.validateObjectPath(t);var r={Id:e._nextId(),ActionType:6,Name:"",ObjectPathId:t._objectPath.objectPathInfo.Id,RecursiveQueryInfo:n},i=new h(r,1,4);return i.referencedObjectPath=t._objectPath,t._addAction(i),i},n.createEnsureUnchangedAction=function(e,t,n){$.validateObjectPath(t);var r={Id:e._nextId(),ActionType:8,Name:"",ObjectPathId:t._objectPath.objectPathInfo.Id,ObjectState:n},i=new h(r,1,4);return i.referencedObjectPath=t._objectPath,t._addAction(i),i},n.createInstantiateAction=function(e,t){$.validateObjectPath(t),e._pendingRequest.ensureInstantiateObjectPath(t._objectPath.parentObjectPath),e._pendingRequest.ensureInstantiateObjectPaths(t._objectPath.argumentObjectPaths);var n={Id:e._nextId(),ActionType:1,Name:"",ObjectPathId:t._objectPath.objectPathInfo.Id},r=new h(n,1,4);return r.referencedObjectPath=t._objectPath,t._addAction(r,new J(t),!0),r},n.createTraceAction=function(e,t,n){var r={Id:e._nextId(),ActionType:5,Name:"Trace",ObjectPathId:0},i=new h(r,1,4);return e._pendingRequest.addAction(i),n&&e._pendingRequest.addTrace(r.Id,t),i},n.createTraceMarkerForCallback=function(e,t){var r=n.createTraceAction(e,null,!1);e._pendingRequest.addActionResultHandler(r,new T(t))},n}(p);e.ActionFactory=w;var j=function(t){function n(n,r){var i=t.call(this,n,r)||this;return $.checkArgumentNull(n,"context"),i.m_context=n,i._objectPath&&(!n._processingResult&&n._pendingRequest&&(w.createInstantiateAction(n,i),n._autoCleanup&&i._KeepReference&&n.trackedObjects._autoAdd(i)),e._internalConfig.appendTypeNameToObjectPathInfo&&i._objectPath.objectPathInfo&&i._className&&(i._objectPath.objectPathInfo.T=i._className)),i}return __extends(n,t),Object.defineProperty(n.prototype,"context",{get:function(){return this.m_context},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isNull",{get:function(){return(void 0!==this.m_isNull||!c.isMock())&&($.throwIfNotLoaded("isNull",this._isNull,null,this._isNull),this._isNull)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isNullObject",{get:function(){return(void 0!==this.m_isNull||!c.isMock())&&($.throwIfNotLoaded("isNullObject",this._isNull,null,this._isNull),this._isNull)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isNull",{get:function(){return this.m_isNull},set:function(e){this.m_isNull=e,e&&this._objectPath&&this._objectPath._updateAsNullObject()},enumerable:!0,configurable:!0}),n.prototype._addAction=function(e,t,n){return void 0===t&&(t=null),n||(this.context._pendingRequest.ensureInstantiateObjectPath(this._objectPath),this.context._pendingRequest.ensureInstantiateObjectPaths(e.referencedArgumentObjectPaths)),this.context._pendingRequest.addAction(e),this.context._pendingRequest.addReferencedObjectPath(this._objectPath),this.context._pendingRequest.addReferencedObjectPaths(e.referencedArgumentObjectPaths),this.context._pendingRequest.addActionResultHandler(e,t),u._createPromiseFromResult(null)},n.prototype._addPreviewExecutionAction=function(e){if(!$.isUndefined(this._className)){e.ObjectType=this._className;var t=$._getPropertyValueWithoutCheckLoaded(this,B.idLowerCase);$.isUndefined(t)&&(t=$._getPropertyValueWithoutCheckLoaded(this,B.idPrivate)),$.isUndefined(t)&&(t=$._getPropertyValueWithoutCheckLoaded(this,B.previewExecutionObjectId)),e.ObjectId=t,this.context._pendingRequest.addPreviewExecutionAction(e)}},n.prototype._handleResult=function(e){this._isNull=$.isNullOrUndefined(e),this.context.trackedObjects._autoTrackIfNecessaryWhenHandleObjectResultValue(this,e)},n.prototype._handleIdResult=function(e){this._isNull=$.isNullOrUndefined(e),$.fixObjectPathIfNecessary(this,e),this.context.trackedObjects._autoTrackIfNecessaryWhenHandleObjectResultValue(this,e)},n.prototype._handleRetrieveResult=function(e,t){this._handleIdResult(e)},n.prototype._recursivelySet=function(e,t,i,a,l){var c=e instanceof n,p=e;if(c){if(Object.getPrototypeOf(this)!==Object.getPrototypeOf(e))throw r.RuntimeError._createInvalidArgError({argumentName:"properties",errorLocation:this._className+".set"});e=JSON.parse(JSON.stringify(e))}try{for(var f,h=0;h<i.length;h++)f=i[h],e.hasOwnProperty(f)&&void 0!==e[f]&&(this[f]=e[f]);for(h=0;h<a.length;h++)if(f=a[h],e.hasOwnProperty(f)&&void 0!==e[f]){var d=c?p[f]:e[f];this[f].set(d,t)}var y=!c;t&&!$.isNullOrUndefined(y)&&(y=t.throwOnReadOnly);for(h=0;h<l.length;h++)if(f=l[h],e.hasOwnProperty(f)&&void 0!==e[f]&&y)throw new r.RuntimeError({code:o.invalidArgument,httpStatusCode:400,message:u._getResourceString(Z.cannotApplyPropertyThroughSetMethod,f),debugInfo:{errorLocation:f}});for(f in e)if(i.indexOf(f)<0&&a.indexOf(f)<0){var m=Object.getOwnPropertyDescriptor(Object.getPrototypeOf(this),f);if(!m)throw new r.RuntimeError({code:o.invalidArgument,httpStatusCode:400,message:u._getResourceString(C.propertyDoesNotExist,f),debugInfo:{errorLocation:f}});if(y&&!m.set)throw new r.RuntimeError({code:o.invalidArgument,httpStatusCode:400,message:u._getResourceString(C.attemptingToSetReadOnlyProperty,f),debugInfo:{errorLocation:f}})}}catch(e){throw new r.RuntimeError({code:o.invalidArgument,httpStatusCode:400,message:u._getResourceString(s.invalidArgument,"properties"),debugInfo:{errorLocation:this._className+".set"},innerError:e})}},n}(f);e.ClientObject=j;var M=function(){function e(e){this.m_session=e}return e.prototype.executeAsync=function(e,t,n){var r={url:a.processQuery,method:"POST",headers:n.Headers,body:n.Body},o="";n.Headers&&(o=n.Headers[B.officeControlId]);var s={id:i.nextId(),type:1,flags:t,controlId:o,message:r};return u.log(JSON.stringify(s)),this.m_session.sendMessageToHost(s).then((function(e){u.log("Received response: "+JSON.stringify(e));var t,n=e.message;if(200===n.statusCode)t={HttpStatusCode:n.statusCode,ErrorCode:null,ErrorMessage:null,Headers:n.headers,Body:u._parseResponseBody(n)};else{u.log("Error Response:"+n.body);var r=u._parseErrorResponse(n);t={HttpStatusCode:n.statusCode,ErrorCode:r.errorCode,ErrorMessage:r.errorMessage,Headers:n.headers,Body:null}}return t}))},e}(),k=function(e){function t(t){var n=e.call(this)||this;return n.m_bridge=t,n.m_bridge.addHostMessageHandler((function(e){if(3===e.type){var t=e.controlId;if(u.isNullOrEmptyString(t))z.getGenericEventRegistration(t)._handleRichApiMessage(e.message);else{var n=z.peekGenericEventRegistrationOrNull(t);n&&n._handleRichApiMessage(e.message),(n=z.peekGenericEventRegistrationOrNull(""))&&n._handleRichApiMessage(e.message)}}})),n}return __extends(t,e),t.getInstanceIfHostBridgeInited=function(){return i.instance?((u.isNullOrUndefined(t.s_instance)||t.s_instance.m_bridge!==i.instance)&&(t.s_instance=new t(i.instance)),t.s_instance):null},t.prototype._resolveRequestUrlAndHeaderInfo=function(){return u._createPromiseFromResult(null)},t.prototype._createRequestExecutorOrNull=function(){return u.log("NativeBridgeSession::CreateRequestExecutor"),new M(this)},t.prototype.getEventRegistration=function(e){return z.getGenericEventRegistration(e)},t.prototype.sendMessageToHost=function(e){return this.m_bridge.sendMessageToHostAndExpectResponse(e)},t}(t);e.HostBridgeSession=k;var U=function(n){function i(e){var o=n.call(this)||this;if(o.m_customRequestHeaders={},o.m_batchMode=0,o._onRunFinishedNotifiers=[],t._overrideSession)o.m_requestUrlAndHeaderInfoResolver=t._overrideSession;else if(($.isNullOrUndefined(e)||"string"==typeof e&&0===e.length)&&((e=i.defaultRequestUrlAndHeaders)||(e={url:a.localDocument,headers:{}})),"string"==typeof e)o.m_requestUrlAndHeaderInfo={url:e,headers:{}};else if(i.isRequestUrlAndHeaderInfoResolver(e))o.m_requestUrlAndHeaderInfoResolver=e;else{if(!i.isRequestUrlAndHeaderInfo(e))throw r.RuntimeError._createInvalidArgError({argumentName:"url"});var s=e;o.m_requestUrlAndHeaderInfo={url:s.url,headers:{}},u._copyHeaders(s.headers,o.m_requestUrlAndHeaderInfo.headers)}return!o.m_requestUrlAndHeaderInfoResolver&&o.m_requestUrlAndHeaderInfo&&u._isLocalDocumentUrl(o.m_requestUrlAndHeaderInfo.url)&&k.getInstanceIfHostBridgeInited()&&(o.m_requestUrlAndHeaderInfo=null,o.m_requestUrlAndHeaderInfoResolver=k.getInstanceIfHostBridgeInited()),o.m_requestUrlAndHeaderInfoResolver instanceof t&&(o.m_session=o.m_requestUrlAndHeaderInfoResolver),o._processingResult=!1,o._customData=B.iterativeExecutor,o.sync=o.sync.bind(o),o}return __extends(i,n),Object.defineProperty(i.prototype,"session",{get:function(){return this.m_session},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"eventRegistration",{get:function(){return this.m_session?this.m_session.getEventRegistration(this._controlId):r.officeJsEventRegistration},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"_url",{get:function(){return this.m_requestUrlAndHeaderInfo?this.m_requestUrlAndHeaderInfo.url:null},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"_pendingRequest",{get:function(){return null==this.m_pendingRequest&&(this.m_pendingRequest=new H(this)),this.m_pendingRequest},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"_controlId",{get:function(){var e=this.m_customRequestHeaders[B.officeControlId];return u.isNullOrUndefined(e)&&(e=""),e},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"debugInfo",{get:function(){return{pendingStatements:new Y(this._rootObjectPropertyName,this._pendingRequest._objectPaths,this._pendingRequest._actions,e._internalConfig.showDisposeInfoInDebugInfo).process()}},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"trackedObjects",{get:function(){return this.m_trackedObjects||(this.m_trackedObjects=new X(this)),this.m_trackedObjects},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"requestHeaders",{get:function(){return this.m_customRequestHeaders},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"batchMode",{get:function(){return this.m_batchMode},enumerable:!0,configurable:!0}),i.prototype.ensureInProgressBatchIfBatchMode=function(){if(1===this.m_batchMode&&!this.m_explicitBatchInProgress)throw $.createRuntimeError(o.generalException,u._getResourceString(Z.notInsideBatch),null)},i.prototype.load=function(e,t){$.validateContext(this,e);var n=i._parseQueryOption(t);p.createQueryAction(this,e,n,e)},i.prototype.loadRecursive=function(e,t,n){if(!$.isPlainJsonObject(t))throw r.RuntimeError._createInvalidArgError({argumentName:"options"});var o={};for(var s in t)o[s]=i._parseQueryOption(t[s]);var a=w.createRecursiveQueryAction(this,e,{Queries:o,MaxDepth:n});this._pendingRequest.addActionResultHandler(a,e)},i.prototype.trace=function(e){w.createTraceAction(this,e,!0)},i.prototype._processOfficeJsErrorResponse=function(e,t){},i.prototype.ensureRequestUrlAndHeaderInfo=function(){var e=this;return $._createPromiseFromResult(null).then((function(){if(!e.m_requestUrlAndHeaderInfo)return e.m_requestUrlAndHeaderInfoResolver._resolveRequestUrlAndHeaderInfo().then((function(t){if(e.m_requestUrlAndHeaderInfo=t,e.m_requestUrlAndHeaderInfo||(e.m_requestUrlAndHeaderInfo={url:a.localDocument,headers:{}}),$.isNullOrEmptyString(e.m_requestUrlAndHeaderInfo.url)&&(e.m_requestUrlAndHeaderInfo.url=a.localDocument),e.m_requestUrlAndHeaderInfo.headers||(e.m_requestUrlAndHeaderInfo.headers={}),"function"==typeof e.m_requestUrlAndHeaderInfoResolver._createRequestExecutorOrNull){var n=e.m_requestUrlAndHeaderInfoResolver._createRequestExecutorOrNull();n&&(e._requestExecutor=n)}}))}))},i.prototype.syncPrivateMain=function(){var e=this;return this.ensureRequestUrlAndHeaderInfo().then((function(){var t=e._pendingRequest;return e.m_pendingRequest=null,e.processPreSyncPromises(t).then((function(){return e.syncPrivate(t)}))}))},i.prototype.syncPrivate=function(t){var n=this;if(c.isMock())return u._createPromiseFromResult(null);if(!t.hasActions)return this.processPendingEventHandlers(t);var i=t.buildRequestMessageBodyAndRequestFlags(),o=i.body,s=i.flags;this._requestFlagModifier&&(s|=this._requestFlagModifier),this._requestExecutor||(u._isLocalDocumentUrl(this.m_requestUrlAndHeaderInfo.url)?this._requestExecutor=new Q(this):this._requestExecutor=new O);var l=this._requestExecutor,p={};u._copyHeaders(this.m_requestUrlAndHeaderInfo.headers,p),u._copyHeaders(this.m_customRequestHeaders,p),delete this.m_customRequestHeaders[B.officeScriptEventId];var f=c._getCurrentTestNameWithSequenceId();f&&(p[a.testRequestNameHeader]=f);var h={Url:this.m_requestUrlAndHeaderInfo.url,Headers:p,Body:o};t.invalidatePendingInvalidObjectPaths();var d=null,y=null;return this._lastSyncStart="undefined"==typeof performance?Date.now():performance.now(),this._lastRequestFlags=s,l.executeAsync(this._customData,s,h).then((function(r){return n._lastSyncEnd="undefined"==typeof performance?Date.now():performance.now(),e.config.executePerfLogFunc&&e.config.executePerfLogFunc({syncStart:n._lastSyncStart,syncEnd:n._lastSyncEnd}),d=n.processRequestExecutorResponseMessage(t,r),n.processPendingEventHandlers(t).catch((function(e){u.log("Error in processPendingEventHandlers"),u.log(JSON.stringify(e)),y=e}))})).then((function(){if(d)throw u.log("Throw error from response: "+JSON.stringify(d)),d;if(y){u.log("Throw error from ProcessEventHandler: "+JSON.stringify(y));var e=null;if(y instanceof r.RuntimeError)(e=y).traceMessages=t._responseTraceMessages;else{var n=null;n="string"==typeof y?y:y.message,$.isNullOrEmptyString(n)&&(n=u._getResourceString(Z.cannotRegisterEvent)),e=new r.RuntimeError({code:N.cannotRegisterEvent,httpStatusCode:400,message:n,traceMessages:t._responseTraceMessages})}throw e}}))},i.prototype.processRequestExecutorResponseMessage=function(t,n){n.Body&&n.Body.TraceIds&&t._setResponseTraceIds(n.Body.TraceIds);var i=t._responseTraceMessages,o=null;if(n.Body){if(n.Body.Error&&n.Body.Error.ActionIndex>=0){var s=new Y(this._rootObjectPropertyName,t._objectPaths,t._actions,!1,!0),a=s.processForDebugStatementInfo(n.Body.Error.ActionIndex);o={statement:a.statement,surroundingStatements:a.surroundingStatements,fullStatements:["Please enable config.extendedErrorLogging to see full statements."]},e.config.extendedErrorLogging&&(s=new Y(this._rootObjectPropertyName,t._objectPaths,t._actions,!1,!1),o.fullStatements=s.process())}var l=null;if(n.Body.Results?l=n.Body.Results:n.Body.ProcessedResults&&n.Body.ProcessedResults.Results&&(l=n.Body.ProcessedResults.Results),l){this._processingResult=!0;try{t.processResponse(l)}finally{this._processingResult=!1}}}if(!$.isNullOrEmptyString(n.ErrorCode))return new r.RuntimeError({code:n.ErrorCode,httpStatusCode:n.HttpStatusCode,message:n.ErrorMessage,traceMessages:i,data:{responseBody:n.RawErrorResponseBody}});if(n.Body&&n.Body.Error){var u={errorLocation:n.Body.Error.Location};return o&&(u.statement=o.statement,u.surroundingStatements=o.surroundingStatements,u.fullStatements=o.fullStatements),new r.RuntimeError({code:n.Body.Error.Code,httpStatusCode:n.Body.Error.HttpStatusCode,message:n.Body.Error.Message,traceMessages:i,debugInfo:u})}return null},i.prototype.processPendingEventHandlers=function(e){for(var t=$._createPromiseFromResult(null),n=0;n<e._pendingProcessEventHandlers.length;n++){var r=e._pendingProcessEventHandlers[n];t=t.then(this.createProcessOneEventHandlersFunc(r,e))}return t},i.prototype.createProcessOneEventHandlersFunc=function(e,t){return function(){return e._processRegistration(t)}},i.prototype.processPreSyncPromises=function(e){for(var t=$._createPromiseFromResult(null),n=0;n<e._preSyncPromises.length;n++){var r=e._preSyncPromises[n];t=t.then(this.createProcessOneProSyncFunc(r))}return t},i.prototype.createProcessOneProSyncFunc=function(e){return function(){return e}},i.prototype.sync=function(e){return c.isMock()?u._createPromiseFromResult(e):this.syncPrivateMain().then((function(){return e}))},i.prototype.batch=function(e){var t=this;if(1!==this.m_batchMode)return u._createPromiseFromException($.createRuntimeError(o.generalException,null,null));if(this.m_explicitBatchInProgress)return u._createPromiseFromException($.createRuntimeError(o.generalException,u._getResourceString(Z.pendingBatchInProgress),null));if($.isNullOrUndefined(e))return $._createPromiseFromResult(null);this.m_explicitBatchInProgress=!0;var n,r,i,s=this.m_pendingRequest;this.m_pendingRequest=new H(this);try{n=e(this._rootObject,this)}catch(e){return this.m_explicitBatchInProgress=!1,this.m_pendingRequest=s,u._createPromiseFromException(e)}return"object"==typeof n&&n&&"function"==typeof n.then?i=$._createPromiseFromResult(null).then((function(){return n})).then((function(e){return t.m_explicitBatchInProgress=!1,r=t.m_pendingRequest,t.m_pendingRequest=s,e})).catch((function(e){return t.m_explicitBatchInProgress=!1,r=t.m_pendingRequest,t.m_pendingRequest=s,u._createPromiseFromException(e)})):(this.m_explicitBatchInProgress=!1,r=this.m_pendingRequest,this.m_pendingRequest=s,i=$._createPromiseFromResult(n)),i.then((function(e){return t.ensureRequestUrlAndHeaderInfo().then((function(){return t.syncPrivate(r)})).then((function(){return e}))}))},i._run=function(e,t,n,r,o,s){return void 0===n&&(n=3),void 0===r&&(r=5e3),i._runCommon("run",null,e,0,t,n,r,null,o,s)},i.isValidRequestInfo=function(e){return"string"==typeof e||i.isRequestUrlAndHeaderInfo(e)||i.isRequestUrlAndHeaderInfoResolver(e)},i.isRequestUrlAndHeaderInfo=function(e){return"object"==typeof e&&null!==e&&Object.getPrototypeOf(e)===Object.getPrototypeOf({})&&!$.isNullOrUndefined(e.url)},i.isRequestUrlAndHeaderInfoResolver=function(e){return"object"==typeof e&&null!==e&&"function"==typeof e._resolveRequestUrlAndHeaderInfo},i._runBatch=function(e,t,n,r,o,s,a,l){return void 0===o&&(o=3),void 0===s&&(s=5e3),i._runBatchCommon(0,e,t,n,o,s,r,a,l)},i._runExplicitBatch=function(e,t,n,r,o,s,a,l){return void 0===o&&(o=3),void 0===s&&(s=5e3),i._runBatchCommon(1,e,t,n,o,s,r,a,l)},i._runBatchCommon=function(e,t,n,r,o,s,a,l,u){var c,p;void 0===o&&(o=3),void 0===s&&(s=5e3);var f=null,h=null,d=0,y=null;if(n.length>0)if(i.isValidRequestInfo(n[0]))f=n[0],d=1;else if($.isPlainJsonObject(n[0])){if(null!=(f=(y=n[0]).session)&&!i.isValidRequestInfo(f))return i.createErrorPromise(t);h=y.previousObjects,d=1}if(n.length==d+1)p=n[d+0];else{if(null!=y||n.length!=d+2)return i.createErrorPromise(t);h=n[d+0],p=n[d+1]}if(null!=h)if(h instanceof j)c=function(){return h.context};else if(h instanceof i)c=function(){return h};else{if(!Array.isArray(h))return i.createErrorPromise(t);var m=h;if(0==m.length)return i.createErrorPromise(t);for(var g=0;g<m.length;g++){if(!(m[g]instanceof j))return i.createErrorPromise(t);if(m[g].context!=m[0].context)return i.createErrorPromise(t,Z.invalidRequestContext)}c=function(){return m[0].context}}else c=r;var _=null;return a&&(_=function(e){return a(y||{},e)}),i._runCommon(t,f,c,e,p,o,s,_,l,u)},i.createErrorPromise=function(e,t){return void 0===t&&(t=s.invalidArgument),u._createPromiseFromException($.createRuntimeError(t,u._getResourceString(t),e))},i._runCommon=function(n,r,o,s,a,l,c,p,f,h){t._overrideSession&&(r=t._overrideSession);var d,y,m,g=u.createPromise((function(e,t){e()})),_=!1;return g.then((function(){if((d=o(r))._autoCleanup)return new e.Promise((function(e,t){d._onRunFinishedNotifiers.push((function(){d._autoCleanup=!0,e()}))}));d._autoCleanup=!0})).then((function(){return"function"!=typeof a?i.createErrorPromise(n):(m=d.m_batchMode,d.m_batchMode=s,p&&p(d),e=a(1==s?d.batch.bind(d):d),($.isNullOrUndefined(e)||"function"!=typeof e.then)&&$.throwError(Z.runMustReturnPromise),e);var e})).then((function(e){return 1===s?e:d.sync(e)})).then((function(e){_=!0,y=e})).catch((function(e){y=e})).then((function(){var e=d.trackedObjects._retrieveAndClearAutoCleanupList();for(var t in d._autoCleanup=!1,d.m_batchMode=m,e)e[t]._objectPath.isValid=!1;var n=0;if($._synchronousCleanup||i.isRequestUrlAndHeaderInfoResolver(r))return o();function o(){n++;var t=d.m_pendingRequest,r=d.m_batchMode,i=new H(d);d.m_pendingRequest=i,d.m_batchMode=0;try{for(var s in e)d.trackedObjects.remove(e[s])}finally{d.m_batchMode=r,d.m_pendingRequest=t}return d.syncPrivate(i).then((function(){f&&f(n)})).catch((function(){h&&h(n),n<l&&setTimeout((function(){o()}),c)}))}o()})).then((function(){d._onRunFinishedNotifiers&&d._onRunFinishedNotifiers.length>0&&d._onRunFinishedNotifiers.shift()();if(_)return y;throw y}))},i}(y);e.ClientRequestContext=U;var L=function(){function e(e,t){this.m_proxy=e,this.m_shouldPolyfill=t;var n=e[B.scalarPropertyNames],r=e[B.navigationPropertyNames],i=e[B.className],o=e[B.isCollection];if(n)for(var s=0;s<n.length;s++)$.definePropertyThrowUnloadedException(this,i,n[s]);if(r)for(s=0;s<r.length;s++)$.definePropertyThrowUnloadedException(this,i,r[s]);o&&$.definePropertyThrowUnloadedException(this,i,B.itemsLowerCase)}return Object.defineProperty(e.prototype,"$proxy",{get:function(){return this.m_proxy},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"$isNullObject",{get:function(){if(!this.m_isLoaded)throw new r.RuntimeError({code:N.valueNotLoaded,httpStatusCode:400,message:u._getResourceString(Z.valueNotLoaded),debugInfo:{errorLocation:"retrieveResult.$isNullObject"}});return this.m_isNullObject},enumerable:!0,configurable:!0}),e.prototype.toJSON=function(){if(this.m_isLoaded)return this.m_isNullObject?null:($.isUndefined(this.m_json)&&(this.m_json=$.purifyJson(this.m_value)),this.m_json)},e.prototype.toString=function(){return JSON.stringify(this.toJSON())},e.prototype._handleResult=function(e){this.m_isLoaded=!0,null===e||"object"==typeof e&&e&&e._IsNull?(this.m_isNullObject=!0,e=null):this.m_isNullObject=!1,this.m_shouldPolyfill&&(e=$.changePropertyNameToCamelLowerCase(e)),this.m_value=e,this.m_proxy._handleRetrieveResult(e,this)},e}(),B=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),t.getItemAt="GetItemAt",t.index="_Index",t.iterativeExecutor="IterativeExecutor",t.isTracked="_IsTracked",t.eventMessageCategory=65536,t.eventWorkbookId="Workbook",t.eventSourceRemote="Remote",t.proxy="$proxy",t.className="_className",t.isCollection="_isCollection",t.collectionPropertyPath="_collectionPropertyPath",t.objectPathInfoDoNotKeepReferenceFieldName="D",t.officeScriptEventId="X-OfficeScriptEventId",t.officeScriptFireRecordingEvent="X-OfficeScriptFireRecordingEvent",t.officeControlId="X-OfficeControlId",t}(v);e.Constants=B;var H=function(e){function t(t){var n=e.call(this,t)||this;return n.m_context=t,n.m_pendingProcessEventHandlers=[],n.m_pendingEventHandlerActions={},n.m_traceInfos={},n.m_responseTraceIds={},n.m_responseTraceMessages=[],n}return __extends(t,e),Object.defineProperty(t.prototype,"traceInfos",{get:function(){return this.m_traceInfos},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_responseTraceMessages",{get:function(){return this.m_responseTraceMessages},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_responseTraceIds",{get:function(){return this.m_responseTraceIds},enumerable:!0,configurable:!0}),t.prototype._setResponseTraceIds=function(e){if(e)for(var t=0;t<e.length;t++){var n=e[t];this.m_responseTraceIds[n]=n;var r=this.m_traceInfos[n];u.isNullOrUndefined(r)||this.m_responseTraceMessages.push(r)}},t.prototype.addTrace=function(e,t){this.m_traceInfos[e]=t},t.prototype._addPendingEventHandlerAction=function(e,t){this.m_pendingEventHandlerActions[e._id]||(this.m_pendingEventHandlerActions[e._id]=[],this.m_pendingProcessEventHandlers.push(e)),this.m_pendingEventHandlerActions[e._id].push(t)},Object.defineProperty(t.prototype,"_pendingProcessEventHandlers",{get:function(){return this.m_pendingProcessEventHandlers},enumerable:!0,configurable:!0}),t.prototype._getPendingEventHandlerActions=function(e){return this.m_pendingEventHandlerActions[e._id]},t}(g);e.ClientRequest=H;var W=function(){function e(e,t,n,r){var i=this;this.m_id=e._nextId(),this.m_context=e,this.m_name=n,this.m_handlers=[],this.m_registered=!1,this.m_eventInfo=r,this.m_callback=function(e){i.m_eventInfo.eventArgsTransformFunc(e).then((function(e){return i.fireEvent(e)}))}}return Object.defineProperty(e.prototype,"_registered",{get:function(){return this.m_registered},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"_id",{get:function(){return this.m_id},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"_handlers",{get:function(){return this.m_handlers},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"_context",{get:function(){return this.m_context},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"_callback",{get:function(){return this.m_callback},enumerable:!0,configurable:!0}),e.prototype.add=function(e){var t=w.createTraceAction(this.m_context,null,!1);return this.m_context._pendingRequest._addPendingEventHandlerAction(this,{id:t.actionInfo.Id,handler:e,operation:0}),new V(this.m_context,this,e)},e.prototype.remove=function(e){var t=w.createTraceAction(this.m_context,null,!1);this.m_context._pendingRequest._addPendingEventHandlerAction(this,{id:t.actionInfo.Id,handler:e,operation:1})},e.prototype.removeAll=function(){var e=w.createTraceAction(this.m_context,null,!1);this.m_context._pendingRequest._addPendingEventHandlerAction(this,{id:e.actionInfo.Id,handler:null,operation:2})},e.prototype._processRegistration=function(e){var t=this,n=u._createPromiseFromResult(null),r=e._getPendingEventHandlerActions(this);if(!r)return n;for(var i=[],o=0;o<this.m_handlers.length;o++)i.push(this.m_handlers[o]);var s=!1;for(o=0;o<r.length;o++)if(e._responseTraceIds[r[o].id])switch(s=!0,r[o].operation){case 0:i.push(r[o].handler);break;case 1:for(var a=i.length-1;a>=0;a--)if(i[a]===r[o].handler){i.splice(a,1);break}break;case 2:i=[]}return s&&(!this.m_registered&&i.length>0?n=n.then((function(){return t.m_eventInfo.registerFunc(t.m_callback)})).then((function(){return t.m_registered=!0})):this.m_registered&&0==i.length&&(n=n.then((function(){return t.m_eventInfo.unregisterFunc(t.m_callback)})).catch((function(e){u.log("Error when unregister event: "+JSON.stringify(e))})).then((function(){return t.m_registered=!1}))),n=n.then((function(){return t.m_handlers=i}))),n},e.prototype.fireEvent=function(e){for(var t=[],n=0;n<this.m_handlers.length;n++){var r=this.m_handlers[n],i=u._createPromiseFromResult(null).then(this.createFireOneEventHandlerFunc(r,e)).catch((function(e){u.log("Error when invoke handler: "+JSON.stringify(e))}));t.push(i)}u.Promise.all(t)},e.prototype.createFireOneEventHandlerFunc=function(e,t){return function(){return e(t)}},e}();e.EventHandlers=W;var V=function(){function e(e,t,n){this.m_context=e,this.m_allHandlers=t,this.m_handler=n}return Object.defineProperty(e.prototype,"context",{get:function(){return this.m_context},enumerable:!0,configurable:!0}),e.prototype.remove=function(){this.m_allHandlers&&this.m_handler&&(this.m_allHandlers.remove(this.m_handler),this.m_allHandlers=null,this.m_handler=null)},e}();e.EventHandlerResult=V,function(e){var t=function(){function t(){}return t.prototype.register=function(t,n,r){switch(t){case 4:return $.promisify((function(e){return Office.context.document.bindings.getByIdAsync(n,e)})).then((function(e){return $.promisify((function(t){return e.addHandlerAsync(Office.EventType.BindingDataChanged,r,t)}))}));case 3:return $.promisify((function(e){return Office.context.document.bindings.getByIdAsync(n,e)})).then((function(e){return $.promisify((function(t){return e.addHandlerAsync(Office.EventType.BindingSelectionChanged,r,t)}))}));case 2:return $.promisify((function(e){return Office.context.document.addHandlerAsync(Office.EventType.DocumentSelectionChanged,r,e)}));case 1:return $.promisify((function(e){return Office.context.document.settings.addHandlerAsync(Office.EventType.SettingsChanged,r,e)}));case 5:return OSF.DDA.RichApi.richApiMessageManager.register(r);case 13:return $.promisify((function(e){return Office.context.document.addHandlerAsync(Office.EventType.ObjectDeleted,r,{id:n},e)}));case 14:return $.promisify((function(e){return Office.context.document.addHandlerAsync(Office.EventType.ObjectSelectionChanged,r,{id:n},e)}));case 15:return $.promisify((function(e){return Office.context.document.addHandlerAsync(Office.EventType.ObjectDataChanged,r,{id:n},e)}));case 16:return $.promisify((function(e){return Office.context.document.addHandlerAsync(Office.EventType.ContentControlAdded,r,{id:n},e)}));default:throw e.RuntimeError._createInvalidArgError({argumentName:"eventId"})}},t.prototype.unregister=function(t,n,r){switch(t){case 4:return $.promisify((function(e){return Office.context.document.bindings.getByIdAsync(n,e)})).then((function(e){return $.promisify((function(t){return e.removeHandlerAsync(Office.EventType.BindingDataChanged,{handler:r},t)}))}));case 3:return $.promisify((function(e){return Office.context.document.bindings.getByIdAsync(n,e)})).then((function(e){return $.promisify((function(t){return e.removeHandlerAsync(Office.EventType.BindingSelectionChanged,{handler:r},t)}))}));case 2:return $.promisify((function(e){return Office.context.document.removeHandlerAsync(Office.EventType.DocumentSelectionChanged,{handler:r},e)}));case 1:return $.promisify((function(e){return Office.context.document.settings.removeHandlerAsync(Office.EventType.SettingsChanged,{handler:r},e)}));case 5:return $.promisify((function(e){return OSF.DDA.RichApi.richApiMessageManager.removeHandlerAsync("richApiMessage",{handler:r},e)}));case 13:return $.promisify((function(e){return Office.context.document.removeHandlerAsync(Office.EventType.ObjectDeleted,{id:n,handler:r},e)}));case 14:return $.promisify((function(e){return Office.context.document.removeHandlerAsync(Office.EventType.ObjectSelectionChanged,{id:n,handler:r},e)}));case 15:return $.promisify((function(e){return Office.context.document.removeHandlerAsync(Office.EventType.ObjectDataChanged,{id:n,handler:r},e)}));case 16:return $.promisify((function(e){return Office.context.document.removeHandlerAsync(Office.EventType.ContentControlAdded,{id:n,handler:r},e)}));default:throw e.RuntimeError._createInvalidArgError({argumentName:"eventId"})}},t}();e.officeJsEventRegistration=new t}(r=e._Internal||(e._Internal={}));var G=function(){function e(e,t){this.m_handlersByEventByTarget={},this.m_registerEventImpl=e,this.m_unregisterEventImpl=t}return e.getTargetIdOrDefault=function(e){return $.isNullOrUndefined(e)?"":e},e.prototype.getHandlers=function(t,n){n=e.getTargetIdOrDefault(n);var r=this.m_handlersByEventByTarget[t];r||(r={},this.m_handlersByEventByTarget[t]=r);var i=r[n];return i||(i=[],r[n]=i),i},e.prototype.callHandlers=function(e,t,n){for(var r=this.getHandlers(e,t),i=0;i<r.length;i++)r[i](n)},e.prototype.hasHandlers=function(e,t){return this.getHandlers(e,t).length>0},e.prototype.register=function(e,t,n){if(!n)throw r.RuntimeError._createInvalidArgError({argumentName:"handler"});var i=this.getHandlers(e,t);return i.push(n),1===i.length?this.m_registerEventImpl(e,t):$._createPromiseFromResult(null)},e.prototype.unregister=function(e,t,n){if(!n)throw r.RuntimeError._createInvalidArgError({argumentName:"handler"});for(var i=this.getHandlers(e,t),o=i.length-1;o>=0;o--)if(i[o]===n){i.splice(o,1);break}return 0===i.length?this.m_unregisterEventImpl(e,t):$._createPromiseFromResult(null)},e}();e.EventRegistration=G;var z=function(){function e(){this.m_eventRegistration=new G(this._registerEventImpl.bind(this),this._unregisterEventImpl.bind(this)),this.m_richApiMessageHandler=this._handleRichApiMessage.bind(this)}return e.prototype.ready=function(){var t=this;return this.m_ready||(e._testReadyImpl?this.m_ready=e._testReadyImpl().then((function(){t.m_isReady=!0})):i.instance?this.m_ready=$._createPromiseFromResult(null).then((function(){t.m_isReady=!0})):this.m_ready=r.officeJsEventRegistration.register(5,"",this.m_richApiMessageHandler).then((function(){t.m_isReady=!0}))),this.m_ready},Object.defineProperty(e.prototype,"isReady",{get:function(){return this.m_isReady},enumerable:!0,configurable:!0}),e.prototype.register=function(e,t,n){var r=this;return this.ready().then((function(){return r.m_eventRegistration.register(e,t,n)}))},e.prototype.unregister=function(e,t,n){var r=this;return this.ready().then((function(){return r.m_eventRegistration.unregister(e,t,n)}))},e.prototype._registerEventImpl=function(e,t){return $._createPromiseFromResult(null)},e.prototype._unregisterEventImpl=function(e,t){return $._createPromiseFromResult(null)},e.prototype._handleRichApiMessage=function(e){if(e&&e.entries)for(var t=0;t<e.entries.length;t++){var n=e.entries[t];if(n.messageCategory==B.eventMessageCategory){u._logEnabled&&u.log(JSON.stringify(n));var r=n.messageType,i=n.targetId;if(this.m_eventRegistration.hasHandlers(r,i)){var o=JSON.parse(n.message);n.isRemoteOverride&&(o.source=B.eventSourceRemote),this.m_eventRegistration.callHandlers(r,i,o)}}}},e.getGenericEventRegistration=function(t){u.isNullOrUndefined(t)&&(t="");var n=e.s_genericEventRegistrations[t];return n||(n=new e,e.s_genericEventRegistrations[t]=n),n},e.peekGenericEventRegistrationOrNull=function(t){return u.isNullOrUndefined(t)&&(t=""),e.s_genericEventRegistrations[t]},e.richApiMessageEventCategory=65536,e.s_genericEventRegistrations={},e}();e.GenericEventRegistration=z,e._testSetRichApiMessageReadyImpl=function(e){z._testReadyImpl=e},e._testTriggerRichApiMessageEvent=function(e){z.getGenericEventRegistration("")._handleRichApiMessage(e)};var q=function(e){function t(t,n,r,i){var o=e.call(this,t,n,r,i)||this;return o.m_genericEventInfo=i,o}return __extends(t,e),t.prototype.add=function(e){var t=this;0==this._handlers.length&&this.m_genericEventInfo.registerFunc&&this.m_genericEventInfo.registerFunc();var n=this._context._controlId;return z.getGenericEventRegistration(n).isReady||this._context._pendingRequest._addPreSyncPromise(z.getGenericEventRegistration(n).ready()),w.createTraceMarkerForCallback(this._context,(function(){t._handlers.push(e),1==t._handlers.length&&z.getGenericEventRegistration(n).register(t.m_genericEventInfo.eventType,t.m_genericEventInfo.getTargetIdFunc(),t._callback)})),new V(this._context,this,e)},t.prototype.remove=function(e){var t=this;1==this._handlers.length&&this.m_genericEventInfo.unregisterFunc&&this.m_genericEventInfo.unregisterFunc();var n=this._context._controlId;w.createTraceMarkerForCallback(this._context,(function(){for(var r=t._handlers,i=r.length-1;i>=0;i--)if(r[i]===e){r.splice(i,1);break}0==r.length&&z.getGenericEventRegistration(n).unregister(t.m_genericEventInfo.eventType,t.m_genericEventInfo.getTargetIdFunc(),t._callback)}))},t.prototype.removeAll=function(){},t}(W);e.GenericEventHandlers=q;var J=function(){function e(e){this.m_clientObject=e}return e.prototype._handleResult=function(e){this.m_clientObject._handleIdResult(e)},e}(),K=function(){function e(){}return e.createGlobalObjectObjectPath=function(e){var t={Id:e._nextId(),ObjectPathType:1,Name:""};return new d(t,null,!1,!1,1,4)},e.createNewObjectObjectPath=function(e,t,n,r){var i={Id:e._nextId(),ObjectPathType:2,Name:t};return new d(i,null,n,!1,1,$._fixupApiFlags(r))},e.createPropertyObjectPath=function(e,t,n,r,i,o){var s={Id:e._nextId(),ObjectPathType:4,Name:n,ParentObjectPathId:t._objectPath.objectPathInfo.Id};return new d(s,t._objectPath,r,i,1,$._fixupApiFlags(o))},e.createIndexerObjectPath=function(e,t,n){var r={Id:e._nextId(),ObjectPathType:5,Name:"",ParentObjectPathId:t._objectPath.objectPathInfo.Id,ArgumentInfo:{}};return r.ArgumentInfo.Arguments=n,new d(r,t._objectPath,!1,!1,1,4)},e.createIndexerObjectPathUsingParentPath=function(e,t,n){var r={Id:e._nextId(),ObjectPathType:5,Name:"",ParentObjectPathId:t.objectPathInfo.Id,ArgumentInfo:{}};return r.ArgumentInfo.Arguments=n,new d(r,t,!1,!1,1,4)},e.createMethodObjectPath=function(e,t,n,r,i,o,s,a,l){var u={Id:e._nextId(),ObjectPathType:3,Name:n,ParentObjectPathId:t._objectPath.objectPathInfo.Id,ArgumentInfo:{}},c=$.setMethodArguments(e,u.ArgumentInfo,i),p=new d(u,t._objectPath,o,s,r,$._fixupApiFlags(l));return p.argumentObjectPaths=c,p.getByIdMethodName=a,p},e.createReferenceIdObjectPath=function(e,t){var n={Id:e._nextId(),ObjectPathType:6,Name:t,ArgumentInfo:{}};return new d(n,null,!1,!1,1,4)},e.createChildItemObjectPathUsingIndexerOrGetItemAt=function(t,n,r,i,o){var s=$.tryGetObjectIdFromLoadOrRetrieveResult(i);return t&&!$.isNullOrUndefined(s)?e.createChildItemObjectPathUsingIndexer(n,r,i):e.createChildItemObjectPathUsingGetItemAt(n,r,i,o)},e.createChildItemObjectPathUsingIndexer=function(e,t,n){var r=$.tryGetObjectIdFromLoadOrRetrieveResult(n),i=i={Id:e._nextId(),ObjectPathType:5,Name:"",ParentObjectPathId:t._objectPath.objectPathInfo.Id,ArgumentInfo:{}};return i.ArgumentInfo.Arguments=[r],new d(i,t._objectPath,!1,!1,1,4)},e.createChildItemObjectPathUsingGetItemAt=function(e,t,n,r){var i=n[B.index];i&&(r=i);var o={Id:e._nextId(),ObjectPathType:3,Name:B.getItemAt,ParentObjectPathId:t._objectPath.objectPathInfo.Id,ArgumentInfo:{}};return o.ArgumentInfo.Arguments=[r],new d(o,t._objectPath,!1,!1,1,4)},e}();e.ObjectPathFactory=K;var Q=function(){function t(e){this.m_context=e}return t.prototype.executeAsync=function(n,r,i){var o=this,s=l.buildMessageArrayForIRequestExecutor(n,r,i,t.SourceLibHeaderValue);return new e.Promise((function(e,t){OSF.DDA.RichApi.executeRichApiRequestAsync(s,(function(t){var n;u.log("Response:"),u.log(JSON.stringify(t)),"succeeded"==t.status?n=l.buildResponseOnSuccess(l.getResponseBody(t),l.getResponseHeaders(t)):(n=l.buildResponseOnError(t.error.code,t.error.message),o.m_context._processOfficeJsErrorResponse(t.error.code,n)),e(n)}))}))},t.SourceLibHeaderValue="officejs",t}(),X=function(){function e(e){this._autoCleanupList={},this.m_context=e}return e.prototype.add=function(e){var t=this;Array.isArray(e)?e.forEach((function(e){return t._addCommon(e,!0)})):this._addCommon(e,!0)},e.prototype._autoAdd=function(e){this._addCommon(e,!1),this._autoCleanupList[e._objectPath.objectPathInfo.Id]=e},e.prototype._autoTrackIfNecessaryWhenHandleObjectResultValue=function(e,t){this.m_context._autoCleanup&&!e[B.isTracked]&&e!==this.m_context._rootObject&&t&&!$.isNullOrEmptyString(t[B.referenceId])&&(this._autoCleanupList[e._objectPath.objectPathInfo.Id]=e,e[B.isTracked]=!0)},e.prototype._addCommon=function(e,t){if(e[B.isTracked])t&&this.m_context._autoCleanup&&delete this._autoCleanupList[e._objectPath.objectPathInfo.Id];else{var n=e[B.referenceId];if(e._objectPath.objectPathInfo[B.objectPathInfoDoNotKeepReferenceFieldName])throw $.createRuntimeError(o.generalException,u._getResourceString(Z.objectIsUntracked),null);$.isNullOrEmptyString(n)&&e._KeepReference&&(e._KeepReference(),w.createInstantiateAction(this.m_context,e),t&&this.m_context._autoCleanup&&delete this._autoCleanupList[e._objectPath.objectPathInfo.Id],e[B.isTracked]=!0)}},e.prototype.remove=function(e){var t=this;Array.isArray(e)?e.forEach((function(e){return t._removeCommon(e)})):this._removeCommon(e)},e.prototype._removeCommon=function(e){e._objectPath.objectPathInfo[B.objectPathInfoDoNotKeepReferenceFieldName]=!0,e.context._pendingRequest._removeKeepReferenceAction(e._objectPath.objectPathInfo.Id);var t=e[B.referenceId];if(!$.isNullOrEmptyString(t)){var n=this.m_context._rootObject;n._RemoveReference&&n._RemoveReference(t)}delete e[B.isTracked]},e.prototype._retrieveAndClearAutoCleanupList=function(){var e=this._autoCleanupList;return this._autoCleanupList={},e},e}();e.TrackedObjects=X;var Y=function(){function t(e,t,n,r,i){e||(e="root"),this.m_globalObjName=e,this.m_referencedObjectPaths=t,this.m_actions=n,this.m_statements=[],this.m_variableNameForObjectPathMap={},this.m_variableNameToObjectPathMap={},this.m_declaredObjectPathMap={},this.m_showDispose=r,this.m_removePII=i}return t.prototype.process=function(){this.m_showDispose&&H._calculateLastUsedObjectPathIds(this.m_actions);for(var e=0;e<this.m_actions.length;e++)this.processOneAction(this.m_actions[e]);return this.m_statements},t.prototype.processForDebugStatementInfo=function(e){this.m_showDispose&&H._calculateLastUsedObjectPathIds(this.m_actions);this.m_statements=[];for(var t=-1,n=0;n<this.m_actions.length&&(this.processOneAction(this.m_actions[n]),e==n&&(t=this.m_statements.length-1),!(t>=0&&this.m_statements.length>t+5+1));n++);if(t<0)return null;var r=t-5;r<0&&(r=0);var i=t+1+5;i>this.m_statements.length&&(i=this.m_statements.length);var o=[];0!=r&&o.push("...");for(var s=r;s<t;s++)o.push(this.m_statements[s]);o.push("// >>>>>"),o.push(this.m_statements[t]),o.push("// <<<<<");for(var a=t+1;a<i;a++)o.push(this.m_statements[a]);return i<this.m_statements.length&&o.push("..."),{statement:this.m_statements[t],surroundingStatements:o}},t.prototype.processOneAction=function(e){switch(e.actionInfo.ActionType){case 1:this.processInstantiateAction(e);break;case 3:this.processMethodAction(e);break;case 2:this.processQueryAction(e);break;case 7:this.processQueryAsJsonAction(e);break;case 6:this.processRecursiveQueryAction(e);break;case 4:this.processSetPropertyAction(e);break;case 5:this.processTraceAction(e);break;case 8:this.processEnsureUnchangedAction(e);break;case 9:this.processUpdateAction(e)}},t.prototype.processInstantiateAction=function(e){var t=e.actionInfo.ObjectPathId,n=this.m_referencedObjectPaths[t],r=this.getObjVarName(t);if(this.m_declaredObjectPathMap[t]){i="// Instantiate {"+r+"}";i=this.appendDisposeCommentIfRelevant(i,e),this.m_statements.push(i)}else{var i="var "+r+" = "+this.buildObjectPathExpressionWithParent(n)+";";i=this.appendDisposeCommentIfRelevant(i,e),this.m_statements.push(i),this.m_declaredObjectPathMap[t]=r}},t.prototype.processMethodAction=function(t){var n=t.actionInfo.Name;if("_KeepReference"===n){if(!e._internalConfig.showInternalApiInDebugInfo)return;n="track"}var r=this.getObjVarName(t.actionInfo.ObjectPathId)+"."+$._toCamelLowerCase(n)+"("+this.buildArgumentsExpression(t.actionInfo.ArgumentInfo)+");";r=this.appendDisposeCommentIfRelevant(r,t),this.m_statements.push(r)},t.prototype.processQueryAction=function(e){var t=this.buildQueryExpression(e),n=this.getObjVarName(e.actionInfo.ObjectPathId)+".load("+t+");";n=this.appendDisposeCommentIfRelevant(n,e),this.m_statements.push(n)},t.prototype.processQueryAsJsonAction=function(e){var t=this.buildQueryExpression(e),n=this.getObjVarName(e.actionInfo.ObjectPathId)+".retrieve("+t+");";n=this.appendDisposeCommentIfRelevant(n,e),this.m_statements.push(n)},t.prototype.processRecursiveQueryAction=function(e){var t="";e.actionInfo.RecursiveQueryInfo&&(t=JSON.stringify(e.actionInfo.RecursiveQueryInfo));var n=this.getObjVarName(e.actionInfo.ObjectPathId)+".loadRecursive("+t+");";n=this.appendDisposeCommentIfRelevant(n,e),this.m_statements.push(n)},t.prototype.processSetPropertyAction=function(e){var t=this.getObjVarName(e.actionInfo.ObjectPathId)+"."+$._toCamelLowerCase(e.actionInfo.Name)+" = "+this.buildArgumentsExpression(e.actionInfo.ArgumentInfo)+";";t=this.appendDisposeCommentIfRelevant(t,e),this.m_statements.push(t)},t.prototype.processTraceAction=function(e){var t="context.trace();";t=this.appendDisposeCommentIfRelevant(t,e),this.m_statements.push(t)},t.prototype.processEnsureUnchangedAction=function(e){var t=this.getObjVarName(e.actionInfo.ObjectPathId)+".ensureUnchanged("+JSON.stringify(e.actionInfo.ObjectState)+");";t=this.appendDisposeCommentIfRelevant(t,e),this.m_statements.push(t)},t.prototype.processUpdateAction=function(e){var t=this.getObjVarName(e.actionInfo.ObjectPathId)+".update("+JSON.stringify(e.actionInfo.ObjectState)+");";t=this.appendDisposeCommentIfRelevant(t,e),this.m_statements.push(t)},t.prototype.appendDisposeCommentIfRelevant=function(e,t){var n=this;if(this.m_showDispose){var r=t.actionInfo.L;if(r&&r.length>0)return e+" // And then dispose {"+r.map((function(e){return n.getObjVarName(e)})).join(", ")+"}"}return e},t.prototype.buildQueryExpression=function(e){if(e.actionInfo.QueryInfo){var t={};return t.select=e.actionInfo.QueryInfo.Select,t.expand=e.actionInfo.QueryInfo.Expand,t.skip=e.actionInfo.QueryInfo.Skip,t.top=e.actionInfo.QueryInfo.Top,void 0===t.top&&void 0===t.skip&&void 0===t.expand?void 0===t.select?"":JSON.stringify(t.select):JSON.stringify(t)}return""},t.prototype.buildObjectPathExpressionWithParent=function(e){return(5==e.objectPathInfo.ObjectPathType||3==e.objectPathInfo.ObjectPathType||4==e.objectPathInfo.ObjectPathType)&&e.objectPathInfo.ParentObjectPathId?this.getObjVarName(e.objectPathInfo.ParentObjectPathId)+"."+this.buildObjectPathExpression(e):this.buildObjectPathExpression(e)},t.prototype.buildObjectPathExpression=function(e){var t=this.buildObjectPathInfoExpression(e.objectPathInfo),n=e.originalObjectPathInfo;return n&&(t=t+" /* originally "+this.buildObjectPathInfoExpression(n)+" */"),t},t.prototype.buildObjectPathInfoExpression=function(e){switch(e.ObjectPathType){case 1:return"context."+this.m_globalObjName;case 5:return"getItem("+this.buildArgumentsExpression(e.ArgumentInfo)+")";case 3:return $._toCamelLowerCase(e.Name)+"("+this.buildArgumentsExpression(e.ArgumentInfo)+")";case 2:return e.Name+".newObject()";case 7:return"null";case 4:return $._toCamelLowerCase(e.Name);case 6:return"context."+this.m_globalObjName+"._getObjectByReferenceId("+JSON.stringify(e.Name)+")"}},t.prototype.buildArgumentsExpression=function(e){var t="";if(!e.Arguments||0===e.Arguments.length)return t;if(this.m_removePII)return void 0===e.Arguments[0]?t:"...";for(var n=0;n<e.Arguments.length;n++)n>0&&(t+=", "),t+=this.buildArgumentLiteral(e.Arguments[n],e.ReferencedObjectPathIds?e.ReferencedObjectPathIds[n]:null);return"undefined"===t&&(t=""),t},t.prototype.buildArgumentLiteral=function(e,t){return"number"==typeof e&&e===t?this.getObjVarName(t):JSON.stringify(e)},t.prototype.getObjVarNameBase=function(e){var t="v",n=this.m_referencedObjectPaths[e];if(n)switch(n.objectPathInfo.ObjectPathType){case 1:t=this.m_globalObjName;break;case 4:t=$._toCamelLowerCase(n.objectPathInfo.Name);break;case 3:var r=n.objectPathInfo.Name;r.length>3&&"Get"===r.substr(0,3)&&(r=r.substr(3)),t=$._toCamelLowerCase(r);break;case 5:var i=this.getObjVarNameBase(n.objectPathInfo.ParentObjectPathId);t="s"===i.charAt(i.length-1)?i.substr(0,i.length-1):i+"Item"}return t},t.prototype.getObjVarName=function(e){if(this.m_variableNameForObjectPathMap[e])return this.m_variableNameForObjectPathMap[e];var t=this.getObjVarNameBase(e);if(!this.m_variableNameToObjectPathMap[t])return this.m_variableNameForObjectPathMap[e]=t,this.m_variableNameToObjectPathMap[t]=e,t;for(var n=1;this.m_variableNameToObjectPathMap[t+n.toString()];)n++;return t+=n.toString(),this.m_variableNameForObjectPathMap[e]=t,this.m_variableNameToObjectPathMap[t]=e,t},t}(),Z=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),t.cannotRegisterEvent="CannotRegisterEvent",t.connectionFailureWithStatus="ConnectionFailureWithStatus",t.connectionFailureWithDetails="ConnectionFailureWithDetails",t.propertyNotLoaded="PropertyNotLoaded",t.runMustReturnPromise="RunMustReturnPromise",t.moreInfoInnerError="MoreInfoInnerError",t.cannotApplyPropertyThroughSetMethod="CannotApplyPropertyThroughSetMethod",t.invalidOperationInCellEditMode="InvalidOperationInCellEditMode",t.objectIsUntracked="ObjectIsUntracked",t.customFunctionDefintionMissing="CustomFunctionDefintionMissing",t.customFunctionImplementationMissing="CustomFunctionImplementationMissing",t.customFunctionNameContainsBadChars="CustomFunctionNameContainsBadChars",t.customFunctionNameCannotSplit="CustomFunctionNameCannotSplit",t.customFunctionUnexpectedNumberOfEntriesInResultBatch="CustomFunctionUnexpectedNumberOfEntriesInResultBatch",t.customFunctionCancellationHandlerMissing="CustomFunctionCancellationHandlerMissing",t.customFunctionInvalidFunction="CustomFunctionInvalidFunction",t.customFunctionInvalidFunctionMapping="CustomFunctionInvalidFunctionMapping",t.customFunctionWindowMissing="CustomFunctionWindowMissing",t.customFunctionDefintionMissingOnWindow="CustomFunctionDefintionMissingOnWindow",t.pendingBatchInProgress="PendingBatchInProgress",t.notInsideBatch="NotInsideBatch",t.cannotUpdateReadOnlyProperty="CannotUpdateReadOnlyProperty",t}(C);e.ResourceStrings=Z,u.addResourceStringValues({CannotRegisterEvent:"The event handler cannot be registered.",PropertyNotLoaded:"The property '{0}' is not available. Before reading the property's value, call the load method on the containing object and call \"context.sync()\" on the associated request context.",RunMustReturnPromise:'The batch function passed to the ".run" method didn\'t return a promise. The function must return a promise, so that any automatically-tracked objects can be released at the completion of the batch operation. Typically, you return a promise by returning the response from "context.sync()".',InvalidOrTimedOutSessionMessage:"Your Office Online session has expired or is invalid. To continue, refresh the page.",InvalidOperationInCellEditMode:"Excel is in cell-editing mode. Please exit the edit mode by pressing ENTER or TAB or selecting another cell, and then try again.",InvalidSheetName:"The request cannot be processed because the specified worksheet cannot be found. Please try again.",CustomFunctionDefintionMissing:"A property with the name '{0}' that represents the function's definition must exist on Excel.Script.CustomFunctions.",CustomFunctionDefintionMissingOnWindow:"A property with the name '{0}' that represents the function's definition must exist on the window object.",CustomFunctionImplementationMissing:"The property with the name '{0}' on Excel.Script.CustomFunctions that represents the function's definition must contain a 'call' property that implements the function.",CustomFunctionNameContainsBadChars:"The function name may only contain letters, digits, underscores, and periods.",CustomFunctionNameCannotSplit:"The function name must contain a non-empty namespace and a non-empty short name.",CustomFunctionUnexpectedNumberOfEntriesInResultBatch:"The batching function returned a number of results that doesn't match the number of parameter value sets that were passed into it.",CustomFunctionCancellationHandlerMissing:"The cancellation handler onCanceled is missing in the function. The handler must be present as the function is defined as cancelable.",CustomFunctionInvalidFunction:"The property with the name '{0}' that represents the function's definition is not a valid function.",CustomFunctionInvalidFunctionMapping:"The property with the name '{0}' on CustomFunctionMappings that represents the function's definition is not a valid function.",CustomFunctionWindowMissing:"The window object was not found.",PendingBatchInProgress:"There is a pending batch in progress. The batch method may not be called inside another batch, or simultaneously with another batch.",NotInsideBatch:"Operations may not be invoked outside of a batch method.",CannotUpdateReadOnlyProperty:"The property '{0}' is read-only and it cannot be updated.",ObjectIsUntracked:"The object is untracked."});var $=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),n.fixObjectPathIfNecessary=function(e,t){e&&e._objectPath&&t&&e._objectPath.updateUsingObjectData(t,e)},n.load=function(e,t){return e.context.load(e,t),e},n.loadAndSync=function(e,t){return e.context.load(e,t),e.context.sync().then((function(){return e}))},n.retrieve=function(t,r){var i=e._internalConfig.alwaysPolyfillClientObjectRetrieveMethod;i||(i=!n.isSetSupported("RichApiRuntime","1.1"));var o=new L(t,i);return t._retrieve(r,o),o},n.retrieveAndSync=function(e,t){var r=n.retrieve(e,t);return e.context.sync().then((function(){return r}))},n.toJson=function(e,t,r,i){var o={};for(var s in t){void 0!==(a=t[s])&&(o[s]=a)}for(var s in r){var a;void 0!==(a=r[s])&&(a[n.fieldName_isCollection]&&void 0!==a[n.fieldName_m__items]?o[s]=a.toJSON().items:o[s]=a.toJSON())}return i&&(o.items=i.map((function(e){return e.toJSON()}))),o},n.throwError=function(e,t,n){throw new r.RuntimeError({code:e,httpStatusCode:400,message:u._getResourceString(e,t),debugInfo:n?{errorLocation:n}:void 0})},n.createRuntimeError=function(e,t,n,i,o){return new r.RuntimeError({code:e,httpStatusCode:i,message:t,debugInfo:{errorLocation:n},data:o})},n.throwIfNotLoaded=function(e,t,r,i){if(!i&&u.isUndefined(t)&&e.charCodeAt(0)!=n.s_underscoreCharCode&&!n.s_suppressPropertyNotLoadedException)throw n.createPropertyNotLoadedException(r,e)},n._getPropertyValueWithoutCheckLoaded=function(e,t){n.s_suppressPropertyNotLoadedException=!0;try{return e[t]}finally{n.s_suppressPropertyNotLoadedException=!1}},n.createPropertyNotLoadedException=function(e,t){return new r.RuntimeError({code:N.propertyNotLoaded,httpStatusCode:400,message:u._getResourceString(Z.propertyNotLoaded,t),debugInfo:e?{errorLocation:e+"."+t}:void 0})},n.createCannotUpdateReadOnlyPropertyException=function(e,t){return new r.RuntimeError({code:N.cannotUpdateReadOnlyProperty,httpStatusCode:400,message:u._getResourceString(Z.cannotUpdateReadOnlyProperty,t),debugInfo:e?{errorLocation:e+"."+t}:void 0})},n.promisify=function(t){return new e.Promise((function(e,n){t((function(t){"failed"==t.status?n(t.error):e(t.value)}))}))},n._addActionResultHandler=function(e,t,n){e.context._pendingRequest.addActionResultHandler(t,n)},n._handleNavigationPropertyResults=function(e,t,n){for(var r=0;r<n.length-1;r+=2)u.isUndefined(t[n[r+1]])||e[n[r]]._handleResult(t[n[r+1]])},n._fixupApiFlags=function(e){return"boolean"==typeof e&&(e=e?1:0),e},n.definePropertyThrowUnloadedException=function(e,t,r){Object.defineProperty(e,r,{configurable:!0,enumerable:!0,get:function(){throw n.createPropertyNotLoadedException(t,r)},set:function(){throw n.createCannotUpdateReadOnlyPropertyException(t,r)}})},n.defineReadOnlyPropertyWithValue=function(e,t,r){Object.defineProperty(e,t,{configurable:!0,enumerable:!0,get:function(){return r},set:function(){throw n.createCannotUpdateReadOnlyPropertyException(null,t)}})},n.processRetrieveResult=function(e,t,r,i){if(!u.isNullOrUndefined(t))if(i){var o=t[B.itemsLowerCase];if(Array.isArray(o)){for(var s=[],a=0;a<o.length;a++){var l=i(o[a],a),c={};c[B.proxy]=l,l._handleRetrieveResult(o[a],c),s.push(c)}n.defineReadOnlyPropertyWithValue(r,B.itemsLowerCase,s)}}else{var p=e[B.scalarPropertyNames],f=e[B.navigationPropertyNames],h=e[B.className];if(p)for(a=0;a<p.length;a++){var d=t[y=p[a]];u.isUndefined(d)?n.definePropertyThrowUnloadedException(r,h,y):n.defineReadOnlyPropertyWithValue(r,y,d)}if(f)for(a=0;a<f.length;a++){var y;d=t[y=f[a]];if(u.isUndefined(d))n.definePropertyThrowUnloadedException(r,h,y);else{var m=e[y],g={};m._handleRetrieveResult(d,g),g[B.proxy]=m,Array.isArray(g[B.itemsLowerCase])&&(g=g[B.itemsLowerCase]),n.defineReadOnlyPropertyWithValue(r,y,g)}}}},n.setMockData=function(e,t,r,i){if(u.isNullOrUndefined(t))e._handleResult(t);else{if(e[B.scalarPropertyOriginalNames]){for(var o={},s=e[B.scalarPropertyOriginalNames],a=e[B.scalarPropertyNames],l=0;l<a.length;l++)void 0!==t[a[l]]&&(o[s[l]]=t[a[l]]);e._handleResult(o)}if(e[B.navigationPropertyNames]){var c=e[B.navigationPropertyNames];for(l=0;l<c.length;l++)if(void 0!==t[c[l]]){var p=e[c[l]];p.setMockData&&p.setMockData(t[c[l]])}}if(e[B.isCollection]&&r){var f=Array.isArray(t)?t:t[B.itemsLowerCase];if(Array.isArray(f)){var h=[];for(l=0;l<f.length;l++){var d=r(f,l);n.setMockData(d,f[l]),h.push(d)}i(h)}}}},n.applyMixin=function(e,t){Object.getOwnPropertyNames(t.prototype).forEach((function(n){"constructor"!==n&&Object.defineProperty(e.prototype,n,Object.getOwnPropertyDescriptor(t.prototype,n))}))},n.ensureTypeInitialized=function(e){var t=new U;new e(t,K.createNewObjectObjectPath(t,"Temp",!1,0))},n.fieldName_m__items="m__items",n.fieldName_isCollection="_isCollection",n._synchronousCleanup=!1,n.s_underscoreCharCode="_".charCodeAt(0),n.s_suppressPropertyNotLoadedException=!1,n}(S);e.Utility=$;var ee=function(){function e(){}return e.invokeMethod=function(e,t,n,r,i,o){var s=w.createMethodAction(e.context,e,t,n,r,i),a=new _(o);return $._addActionResultHandler(e,s,a),a},e.invokeEnsureUnchanged=function(e,t){w.createEnsureUnchangedAction(e.context,e,t)},e.invokeSetProperty=function(e,t,n,r){w.createSetPropertyAction(e.context,e,t,n,r)},e.createRootServiceObject=function(e,t){return new e(t,K.createGlobalObjectObjectPath(t))},e.createObjectFromReferenceId=function(e,t,n){return new e(t,K.createReferenceIdObjectPath(t,n))},e.createTopLevelServiceObject=function(e,t,n,r,i){return new e(t,K.createNewObjectObjectPath(t,n,r,i))},e.createPropertyObject=function(e,t,n,r,i){var o=K.createPropertyObjectPath(t.context,t,n,r,!1,i);return new e(t.context,o)},e.createIndexerObject=function(e,t,n){var r=K.createIndexerObjectPath(t.context,t,n);return new e(t.context,r)},e.createMethodObject=function(e,t,n,r,i,o,s,a,l){var u=K.createMethodObjectPath(t.context,t,n,r,i,o,s,a,l);return new e(t.context,u)},e.createChildItemObject=function(e,t,n,r,i){var o=K.createChildItemObjectPathUsingIndexerOrGetItemAt(t,n.context,n,r,i);return new e(n.context,o)},e}();e.BatchApiHelper=ee;var te=function(){function t(e){if(this.m_namespaceMap={},this.m_namespace=e.metadata.name,this.m_targetNamespaceObject=e.targetNamespaceObject,this.m_namespaceMap[this.m_namespace]=e.targetNamespaceObject,e.namespaceMap)for(var t in e.namespaceMap)this.m_namespaceMap[t]=e.namespaceMap[t];this.m_defaultApiSetName=e.metadata.defaultApiSetName,this.m_hostName=e.metadata.hostName;var n=e.metadata;if(n.enumTypes)for(var r=0;r<n.enumTypes.length;r++)this.buildEnumType(n.enumTypes[r]);if(n.apiSets){for(r=0;r<n.apiSets.length;r++){var i=n.apiSets[r];Array.isArray(i)&&(n.apiSets[r]={version:i[0],name:i[1]||this.m_defaultApiSetName})}this.m_apiSets=n.apiSets}if(this.m_strings=n.strings,n.clientObjectTypes)for(r=0;r<n.clientObjectTypes.length;r++){i=n.clientObjectTypes[r];Array.isArray(i)&&(this.ensureArraySize(i,11),n.clientObjectTypes[r]={name:this.getString(i[0]),behaviorFlags:i[1],collectionPropertyPath:this.getString(i[6]),newObjectServerTypeFullName:this.getString(i[9]),newObjectApiFlags:i[10],childItemTypeFullName:this.getString(i[7]),scalarProperties:i[2],navigationProperties:i[3],scalarMethods:i[4],navigationMethods:i[5],events:i[8]}),this.buildClientObjectType(n.clientObjectTypes[r],e.fullyInitialize)}}return t.prototype.ensureArraySize=function(e,t){for(var n=t-e.length;n>0;)e.push(0),n--},t.prototype.getString=function(e){return"number"==typeof e?e>0?this.m_strings[e-1]:null:e},t.prototype.buildEnumType=function(e){var t;if(Array.isArray(e)){(t={name:e[0],fields:e[2]}).fields||(t.fields={});var n=e[1];if(Array.isArray(n))for(var r=0;r<n.length;r++)t.fields[n[r]]=this.toSimpleCamelUpperCase(n[r])}else t=e;this.m_targetNamespaceObject[t.name]=t.fields},t.prototype.buildClientObjectType=function(n,r){var i=this,o=function(t,r){j.apply(this,arguments),i.m_targetNamespaceObject[n.name]._typeInited||(i.buildPrototype(i.m_targetNamespaceObject[n.name],n),i.m_targetNamespaceObject[n.name]._typeInited=!0),e._internalConfig.appendTypeNameToObjectPathInfo&&this._objectPath&&this._objectPath.objectPathInfo&&this._className&&(this._objectPath.objectPathInfo.T=this._className)};(this.m_targetNamespaceObject[n.name]=o,this.extendsType(o,j),this.buildNewObject(o,n),0!=(2&n.behaviorFlags)&&(o.prototype._KeepReference=function(){ee.invokeMethod(this,"_KeepReference",1,[],0,0)}),0!=(32&n.behaviorFlags))&&this.getFunction(t.CustomizationCodeNamespace+"."+n.name+"_StaticCustomize").call(null,o);r&&(this.buildPrototype(o,n),o._typeInited=!0)},t.prototype.extendsType=function(e,t){function n(){this.constructor=e}e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)},t.prototype.findObjectUnderPath=function(t,n,r){for(var i=t,o=r;o<n.length;o++){if("object"!=typeof i)throw new e.Error("Cannot find "+n.join("."));i=i[n[o]]}return i},t.prototype.getFunction=function(t){var n=this.resolveObjectByFullName(t);if("function"!=typeof n)throw new e.Error("Cannot find function or type: "+t);return n},t.prototype.resolveObjectByFullName=function(e){var t=e.split(".");if(1===t.length)return this.m_targetNamespaceObject[t[0]];var n=t[0];return n===this.m_namespace?this.findObjectUnderPath(this.m_targetNamespaceObject,t,1):this.m_namespaceMap[n]?this.findObjectUnderPath(this.m_namespaceMap[n],t,1):this.findObjectUnderPath(this.m_targetNamespaceObject,t,0)},t.prototype.evaluateSimpleExpression=function(t,n){if($.isNullOrUndefined(t))return null;var r=t.split(".");if(3===r.length&&"OfficeExtension"===r[0]&&"Constants"===r[1])return B[r[2]];if("this"===r[0]){for(var i=n,o=1;o<r.length;o++)i="toString()"==r[o]?i.toString():"()"===r[o].substr(r[o].length-2)?i[r[o].substr(0,r[o].length-2)]():i[r[o]];return i}throw new e.Error("Cannot evaluate: "+t)},t.prototype.evaluateEventTargetId=function(e,t){return $.isNullOrEmptyString(e)?"":this.evaluateSimpleExpression(e,t)},t.prototype.isAllDigits=function(e){for(var t="0".charCodeAt(0),n="9".charCodeAt(0),r=0;r<e.length;r++)if(e.charCodeAt(r)<t||e.charCodeAt(r)>n)return!1;return!0},t.prototype.evaluateEventType=function(t){if($.isNullOrEmptyString(t))return 0;if(this.isAllDigits(t))return parseInt(t);var n=this.resolveObjectByFullName(t);if("number"!=typeof n)throw new e.Error("Invalid event type: "+t);return n},t.prototype.buildPrototype=function(e,t){this.buildScalarProperties(e,t),this.buildNavigationProperties(e,t),this.buildScalarMethods(e,t),this.buildNavigationMethods(e,t),this.buildEvents(e,t),this.buildHandleResult(e,t),this.buildHandleIdResult(e,t),this.buildHandleRetrieveResult(e,t),this.buildLoad(e,t),this.buildRetrieve(e,t),this.buildSetMockData(e,t),this.buildEnsureUnchanged(e,t),this.buildUpdate(e,t),this.buildSet(e,t),this.buildToJSON(e,t),this.buildItems(e,t),this.buildTypeMetadataInfo(e,t),this.buildTrackUntrack(e,t),this.buildMixin(e,t)},t.prototype.toSimpleCamelUpperCase=function(e){return e.substr(0,1).toUpperCase()+e.substr(1)},t.prototype.ensureOriginalName=function(e){null===e.originalName&&(e.originalName=this.toSimpleCamelUpperCase(e.name))},t.prototype.getFieldName=function(e){return"m_"+e.name},t.prototype.throwIfApiNotSupported=function(e,t){if(this.m_apiSets&&t.apiSetInfoOrdinal>0){var n=this.m_apiSets[t.apiSetInfoOrdinal-1];n&&$.throwIfApiNotSupported(e.name+"."+t.name,n.name,n.version,this.m_hostName)}},t.prototype.buildScalarProperties=function(e,t){if(Array.isArray(t.scalarProperties))for(var n=0;n<t.scalarProperties.length;n++){var r=t.scalarProperties[n];Array.isArray(r)&&(this.ensureArraySize(r,6),t.scalarProperties[n]={name:this.getString(r[0]),behaviorFlags:r[1],apiSetInfoOrdinal:r[2],originalName:this.getString(r[3]),setMethodApiFlags:r[4],undoableApiSetInfoOrdinal:r[5]}),this.buildScalarProperty(e,t,t.scalarProperties[n])}},t.prototype.calculateApiFlags=function(e,t){if(t>0){var n=this.m_apiSets[t-1];n&&(e=S.calculateApiFlags(e,n.name,n.version))}return e},t.prototype.buildScalarProperty=function(e,n,r){this.ensureOriginalName(r);var i=this,o=this.getFieldName(r),s={get:function(){return $.throwIfNotLoaded(r.name,this[o],n.name,this._isNull),i.throwIfApiNotSupported(n,r),this[o]},enumerable:!0,configurable:!0};0==(2&r.behaviorFlags)&&(s.set=function(e){if(4&r.behaviorFlags&&i.getFunction(t.CustomizationCodeNamespace+"."+n.name+"_"+r.originalName+"_Set").call(this,this,e).handled)return;this[o]=e;var s=i.calculateApiFlags(r.setMethodApiFlags,r.undoableApiSetInfoOrdinal);ee.invokeSetProperty(this,r.originalName,e,s)}),Object.defineProperty(e.prototype,r.name,s)},t.prototype.buildNavigationProperties=function(e,t){if(Array.isArray(t.navigationProperties))for(var n=0;n<t.navigationProperties.length;n++){var r=t.navigationProperties[n];Array.isArray(r)&&(this.ensureArraySize(r,8),t.navigationProperties[n]={name:this.getString(r[0]),behaviorFlags:r[2],apiSetInfoOrdinal:r[3],originalName:this.getString(r[4]),getMethodApiFlags:r[5],setMethodApiFlags:r[6],propertyTypeFullName:this.getString(r[1]),undoableApiSetInfoOrdinal:r[7]}),this.buildNavigationProperty(e,t,t.navigationProperties[n])}},t.prototype.buildNavigationProperty=function(e,n,r){this.ensureOriginalName(r);var i=this,o=this.getFieldName(r),s={get:function(){(this[i.getFieldName(r)]||(i.throwIfApiNotSupported(n,r),this[o]=ee.createPropertyObject(i.getFunction(r.propertyTypeFullName),this,r.originalName,0!=(16&r.behaviorFlags),r.getMethodApiFlags)),64&r.behaviorFlags)&&i.getFunction(t.CustomizationCodeNamespace+"."+n.name+"_"+r.originalName+"_Get").call(this,this,this[o]);return this[o]},enumerable:!0,configurable:!0};0==(2&r.behaviorFlags)&&(s.set=function(e){if(4&r.behaviorFlags&&i.getFunction(t.CustomizationCodeNamespace+"."+n.name+"_"+r.originalName+"_Set").call(this,this,e).handled)return;this[o]=e;var s=i.calculateApiFlags(r.setMethodApiFlags,r.undoableApiSetInfoOrdinal);ee.invokeSetProperty(this,r.originalName,e,s)}),Object.defineProperty(e.prototype,r.name,s)},t.prototype.buildScalarMethods=function(e,t){if(Array.isArray(t.scalarMethods))for(var n=0;n<t.scalarMethods.length;n++){var r=t.scalarMethods[n];Array.isArray(r)&&(this.ensureArraySize(r,7),t.scalarMethods[n]={name:this.getString(r[0]),behaviorFlags:r[2],apiSetInfoOrdinal:r[3],originalName:this.getString(r[5]),apiFlags:r[4],parameterCount:r[1],undoableApiSetInfoOrdinal:r[6]}),this.buildScalarMethod(e,t,t.scalarMethods[n])}},t.prototype.buildScalarMethod=function(e,n,r){this.ensureOriginalName(r);var i=this;e.prototype[r.name]=function(){var e=[];if(64&r.behaviorFlags&&r.parameterCount>0){for(var o=0;o<r.parameterCount-1;o++)e.push(arguments[o]);var s=[];for(o=r.parameterCount-1;o<arguments.length;o++)s.push(arguments[o]);e.push(s)}else for(o=0;o<arguments.length;o++)e.push(arguments[o]);if(1&r.behaviorFlags){var a=i.getFunction(t.CustomizationCodeNamespace+"."+n.name+"_"+r.originalName),l=[this];for(o=0;o<e.length;o++)l.push(e[o]);var u=a.apply(this,l),c=u.handled,p=u.result;if(c)return p}i.throwIfApiNotSupported(n,r);var f=0;32&r.behaviorFlags&&(f=1);var h=0;2&r.behaviorFlags&&(h=1);var d=i.calculateApiFlags(r.apiFlags,r.undoableApiSetInfoOrdinal);return ee.invokeMethod(this,r.originalName,h,e,d,f)}},t.prototype.buildNavigationMethods=function(e,t){if(Array.isArray(t.navigationMethods))for(var n=0;n<t.navigationMethods.length;n++){var r=t.navigationMethods[n];Array.isArray(r)&&(this.ensureArraySize(r,9),t.navigationMethods[n]={name:this.getString(r[0]),behaviorFlags:r[3],apiSetInfoOrdinal:r[4],originalName:this.getString(r[6]),apiFlags:r[5],parameterCount:r[2],returnTypeFullName:this.getString(r[1]),returnObjectGetByIdMethodName:this.getString(r[7]),undoableApiSetInfoOrdinal:r[8]}),this.buildNavigationMethod(e,t,t.navigationMethods[n])}},t.prototype.buildNavigationMethod=function(e,n,r){this.ensureOriginalName(r);var i=this;e.prototype[r.name]=function(){var e=[];if(64&r.behaviorFlags&&r.parameterCount>0){for(var o=0;o<r.parameterCount-1;o++)e.push(arguments[o]);var s=[];for(o=r.parameterCount-1;o<arguments.length;o++)s.push(arguments[o]);e.push(s)}else for(o=0;o<arguments.length;o++)e.push(arguments[o]);if(1&r.behaviorFlags){var a=i.getFunction(t.CustomizationCodeNamespace+"."+n.name+"_"+r.originalName),l=[this];for(o=0;o<e.length;o++)l.push(e[o]);var u=a.apply(this,l),c=u.handled,p=u.result;if(c)return p}if(i.throwIfApiNotSupported(n,r),0!=(16&r.behaviorFlags))return ee.createIndexerObject(i.getFunction(r.returnTypeFullName),this,e);var f=0;2&r.behaviorFlags&&(f=1);var h=i.calculateApiFlags(r.apiFlags,r.undoableApiSetInfoOrdinal);return ee.createMethodObject(i.getFunction(r.returnTypeFullName),this,r.originalName,f,e,0!=(4&r.behaviorFlags),0!=(8&r.behaviorFlags),r.returnObjectGetByIdMethodName,h)}},t.prototype.buildHandleResult=function(e,n){var r=this;e.prototype._handleResult=function(e){if(j.prototype._handleResult.call(this,e),!$.isNullOrUndefined(e)){if($.fixObjectPathIfNecessary(this,e),8&n.behaviorFlags)r.getFunction(t.CustomizationCodeNamespace+"."+n.name+"_HandleResult").call(this,this,e);if(n.scalarProperties)for(var i=0;i<n.scalarProperties.length;i++)$.isUndefined(e[n.scalarProperties[i].originalName])||(0!=(8&n.scalarProperties[i].behaviorFlags)?this[r.getFieldName(n.scalarProperties[i])]=$.adjustToDateTime(e[n.scalarProperties[i].originalName]):this[r.getFieldName(n.scalarProperties[i])]=e[n.scalarProperties[i].originalName]);if(n.navigationProperties){for(var o=[],s=0;s<n.navigationProperties.length;s++)o.push(n.navigationProperties[s].name),o.push(n.navigationProperties[s].originalName);$._handleNavigationPropertyResults(this,e,o)}if(0!=(1&n.behaviorFlags)){var a=r.hasIndexMethod(n);if(!$.isNullOrUndefined(e[B.items])){this.m__items=[];for(var l=e[B.items],u=r.getFunction(n.childItemTypeFullName),c=0;c<l.length;c++){var p=ee.createChildItemObject(u,a,this,l[c],c);p._handleResult(l[c]),this.m__items.push(p)}}}}}},t.prototype.buildHandleRetrieveResult=function(e,t){var n=this;e.prototype._handleRetrieveResult=function(e,r){if(j.prototype._handleRetrieveResult.call(this,e,r),!$.isNullOrUndefined(e)){if(t.scalarProperties)for(var i=0;i<t.scalarProperties.length;i++)8&t.scalarProperties[i].behaviorFlags&&($.isNullOrUndefined(e[t.scalarProperties[i].name])||(e[t.scalarProperties[i].name]=$.adjustToDateTime(e[t.scalarProperties[i].name])));if(1&t.behaviorFlags){var o=n.hasIndexMethod(t),s=n.getFunction(t.childItemTypeFullName),a=this;$.processRetrieveResult(a,e,r,(function(e,t){return ee.createChildItemObject(s,o,a,e,t)}))}else $.processRetrieveResult(this,e,r)}}},t.prototype.buildHandleIdResult=function(e,n){var r=this;e.prototype._handleIdResult=function(e){if(j.prototype._handleIdResult.call(this,e),!$.isNullOrUndefined(e)){if(16&n.behaviorFlags)r.getFunction(t.CustomizationCodeNamespace+"."+n.name+"_HandleIdResult").call(this,this,e);if(n.scalarProperties)for(var i=0;i<n.scalarProperties.length;i++){var o=n.scalarProperties[i].originalName;"Id"!==o&&"_Id"!==o&&"_ReferenceId"!==o||$.isNullOrUndefined(e[n.scalarProperties[i].originalName])||(this[r.getFieldName(n.scalarProperties[i])]=e[n.scalarProperties[i].originalName])}}}},t.prototype.buildLoad=function(e,t){e.prototype.load=function(e){return $.load(this,e)}},t.prototype.buildRetrieve=function(e,t){e.prototype.retrieve=function(e){return $.retrieve(this,e)}},t.prototype.buildNewObject=function(e,t){$.isNullOrEmptyString(t.newObjectServerTypeFullName)||(e.newObject=function(n){return ee.createTopLevelServiceObject(e,n,t.newObjectServerTypeFullName,0!=(1&t.behaviorFlags),t.newObjectApiFlags)})},t.prototype.buildSetMockData=function(e,t){var n=this;if(1&t.behaviorFlags){var r=n.hasIndexMethod(t);e.prototype.setMockData=function(e){var i=this;$.setMockData(i,e,(function(e,o){return ee.createChildItemObject(n.getFunction(t.childItemTypeFullName),r,i,e,o)}),(function(e){i.m__items=e}))}}else e.prototype.setMockData=function(e){$.setMockData(this,e)}},t.prototype.buildEnsureUnchanged=function(e,t){e.prototype.ensureUnchanged=function(e){ee.invokeEnsureUnchanged(this,e)}},t.prototype.buildUpdate=function(e,t){e.prototype.update=function(e){this._recursivelyUpdate(e)}},t.prototype.buildSet=function(e,t){if(0==(1&t.behaviorFlags)){var n=[],r=[];if(t.scalarProperties)for(var i=0;i<t.scalarProperties.length;i++)0==(2&t.scalarProperties[i].behaviorFlags)&&0!=(1&t.scalarProperties[i].behaviorFlags)?r.push(t.scalarProperties[i].name):n.push(t.scalarProperties[i].name);var o=[];if(t.navigationProperties)for(i=0;i<t.navigationProperties.length;i++)0!=(16&t.navigationProperties[i].behaviorFlags)||0==(1&t.navigationProperties[i].behaviorFlags)||0==(32&t.navigationProperties[i].behaviorFlags)?n.push(t.navigationProperties[i].name):o.push(t.navigationProperties[i].name);0===o.length&&0===r.length||(e.prototype.set=function(e,t){this._recursivelySet(e,t,r,o,n)})}},t.prototype.buildItems=function(e,t){0!=(1&t.behaviorFlags)&&Object.defineProperty(e.prototype,"items",{get:function(){return $.throwIfNotLoaded("items",this.m__items,t.name,this._isNull),this.m__items},enumerable:!0,configurable:!0})},t.prototype.buildToJSON=function(e,t){var n=this;0==(1&t.behaviorFlags)?e.prototype.toJSON=function(){var e={};if(t.scalarProperties)for(var r=0;r<t.scalarProperties.length;r++)0!=(1&t.scalarProperties[r].behaviorFlags)&&(e[t.scalarProperties[r].name]=this[n.getFieldName(t.scalarProperties[r])]);var i={};if(t.navigationProperties)for(r=0;r<t.navigationProperties.length;r++)0!=(1&t.navigationProperties[r].behaviorFlags)&&(i[t.navigationProperties[r].name]=this[n.getFieldName(t.navigationProperties[r])]);return $.toJson(this,e,i)}:e.prototype.toJSON=function(){return $.toJson(this,{},{},this.m__items)}},t.prototype.buildTypeMetadataInfo=function(e,t){Object.defineProperty(e.prototype,"_className",{get:function(){return t.name},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"_isCollection",{get:function(){return 0!=(1&t.behaviorFlags)},enumerable:!0,configurable:!0}),$.isNullOrEmptyString(t.collectionPropertyPath)||Object.defineProperty(e.prototype,"_collectionPropertyPath",{get:function(){return t.collectionPropertyPath},enumerable:!0,configurable:!0}),t.scalarProperties&&t.scalarProperties.length>0&&(Object.defineProperty(e.prototype,"_scalarPropertyNames",{get:function(){return this.m__scalarPropertyNames||(this.m__scalarPropertyNames=t.scalarProperties.map((function(e){return e.name}))),this.m__scalarPropertyNames},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"_scalarPropertyOriginalNames",{get:function(){return this.m__scalarPropertyOriginalNames||(this.m__scalarPropertyOriginalNames=t.scalarProperties.map((function(e){return e.originalName}))),this.m__scalarPropertyOriginalNames},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"_scalarPropertyUpdateable",{get:function(){return this.m__scalarPropertyUpdateable||(this.m__scalarPropertyUpdateable=t.scalarProperties.map((function(e){return 0==(2&e.behaviorFlags)}))),this.m__scalarPropertyUpdateable},enumerable:!0,configurable:!0})),t.navigationProperties&&t.navigationProperties.length>0&&Object.defineProperty(e.prototype,"_navigationPropertyNames",{get:function(){return this.m__navigationPropertyNames||(this.m__navigationPropertyNames=t.navigationProperties.map((function(e){return e.name}))),this.m__navigationPropertyNames},enumerable:!0,configurable:!0})},t.prototype.buildTrackUntrack=function(e,t){2&t.behaviorFlags&&(e.prototype.track=function(){return this.context.trackedObjects.add(this),this},e.prototype.untrack=function(){return this.context.trackedObjects.remove(this),this})},t.prototype.buildMixin=function(e,t){if(4&t.behaviorFlags){var n=this.getFunction(t.name+"Custom");$.applyMixin(e,n)}},t.prototype.getOnEventName=function(e){return"_"===e[0]?"_on"+e.substr(1):"on"+e},t.prototype.buildEvents=function(e,t){if(t.events)for(var n=0;n<t.events.length;n++){var r=t.events[n];Array.isArray(r)&&(this.ensureArraySize(r,7),t.events[n]={name:this.getString(r[0]),behaviorFlags:r[1],apiSetInfoOrdinal:r[2],typeExpression:this.getString(r[3]),targetIdExpression:this.getString(r[4]),register:this.getString(r[5]),unregister:this.getString(r[6])}),this.buildEvent(e,t,t.events[n])}},t.prototype.buildEvent=function(e,t,n){1&n.behaviorFlags?this.buildV0Event(e,t,n):this.buildV2Event(e,t,n)},t.prototype.buildV2Event=function(e,n,r){var i=this,o=this.getOnEventName(r.name),s=this.getFieldName(r);Object.defineProperty(e.prototype,o,{get:function(){if(!this[s]){i.throwIfApiNotSupported(n,r);var e=this,o=null;"null"!==r.register&&(o=this[r.register].bind(this));var a=null;"null"!==r.unregister&&(a=this[r.unregister].bind(this));var l=null;2&r.behaviorFlags&&(l=i.getFunction(t.CustomizationCodeNamespace+"."+n.name+"_"+r.name+"_EventArgsTransform"));var u=i.evaluateEventType(r.typeExpression);this[s]=new q(this.context,this,r.name,{eventType:u,getTargetIdFunc:function(){return i.evaluateEventTargetId(r.targetIdExpression,e)},registerFunc:o,unregisterFunc:a,eventArgsTransformFunc:function(t){return l&&(t=l.call(e,e,t)),$._createPromiseFromResult(t)}})}return this[s]},enumerable:!0,configurable:!0})},t.prototype.buildV0Event=function(e,n,r){var i=this,o=this.getOnEventName(r.name),s=this.getFieldName(r);Object.defineProperty(e.prototype,o,{get:function(){if(!this[s]){i.throwIfApiNotSupported(n,r);var e=this,o=null;if($.isNullOrEmptyString(r.register)){var a=i.evaluateEventType(r.typeExpression);o=function(t){var n=i.evaluateEventTargetId(r.targetIdExpression,e);return e.context.eventRegistration.register(a,n,t)}}else if("null"!==r.register){var l=i.getFunction(r.register);o=function(t){return l.call(e,e,t)}}var u=null;if($.isNullOrEmptyString(r.unregister)){var c=i.evaluateEventType(r.typeExpression);u=function(t){var n=i.evaluateEventTargetId(r.targetIdExpression,e);return e.context.eventRegistration.unregister(c,n,t)}}else if("null"!==r.unregister){var p=i.getFunction(r.unregister);u=function(t){return p.call(e,e,t)}}var f=null;2&r.behaviorFlags&&(f=i.getFunction(t.CustomizationCodeNamespace+"."+n.name+"_"+r.name+"_EventArgsTransform"));this[s]=new W(this.context,this,r.name,{registerFunc:o,unregisterFunc:u,eventArgsTransformFunc:function(t){return f&&(t=f.call(e,e,t)),$._createPromiseFromResult(t)}})}return this[s]},enumerable:!0,configurable:!0})},t.prototype.hasIndexMethod=function(e){var t=!1;if(e.navigationMethods)for(var n=0;n<e.navigationMethods.length;n++)if(0!=(16&e.navigationMethods[n].behaviorFlags)){t=!0;break}return t},t.CustomizationCodeNamespace="_CC",t}();e.LibraryBuilder=te;var ne,re=1,ie=function(e){return e.DdaMethod.Version=re,e},oe=function(e){return re=e.Version,e.Error&&(e.error={},e.error.Code=e.Error),e};!function(e){e.SendingId="sId",e.RespondingId="rId",e.CommandKey="command",e.SessionInfoKey="sessionInfo",e.ParamsKey="params",e.ApiReadyCommand="apiready",e.ExecuteMethodCommand="executeMethod",e.GetAppContextCommand="getAppContext",e.RegisterEventCommand="registerEvent",e.UnregisterEventCommand="unregisterEvent",e.FireEventCommand="fireEvent"}(ne||(ne={}));var se=function(){function e(){}return e.sessionContext="sc",e.embeddingPageOrigin="EmbeddingPageOrigin",e.embeddingPageSessionInfo="EmbeddingPageSessionInfo",e}();e.EmbeddedConstants=se;var ae=function(e){function t(t,n){var r=e.call(this)||this;return r.m_chosenWindow=null,r.m_chosenOrigin=null,r.m_enabled=!0,r.m_onMessageHandler=r._onMessage.bind(r),r.m_callbackList={},r.m_id=0,r.m_timeoutId=-1,r.m_appContext=null,r.m_url=t,r.m_options=n,r.m_options||(r.m_options={sessionKey:Math.random().toString()}),r.m_options.sessionKey||(r.m_options.sessionKey=Math.random().toString()),r.m_options.container||(r.m_options.container=document.body),r.m_options.timeoutInMilliseconds||(r.m_options.timeoutInMilliseconds=6e4),r.m_options.height||(r.m_options.height="400px"),r.m_options.width||(r.m_options.width="100%"),r.m_options.webApplication&&r.m_options.webApplication.accessToken&&r.m_options.webApplication.accessTokenTtl||(r.m_options.webApplication=null),r}return __extends(t,e),t.prototype._getIFrameSrc=function(){var e=window.location.protocol+"//"+window.location.host,t=se.embeddingPageOrigin+"="+encodeURIComponent(e)+"&"+se.embeddingPageSessionInfo+"="+encodeURIComponent(this.m_options.sessionKey),n=!1;(this.m_url.toLowerCase().indexOf("/_layouts/preauth.aspx")>0||this.m_url.toLowerCase().indexOf("/_layouts/15/preauth.aspx")>0)&&(n=!0);var r=document.createElement("a");if(r.href=this.m_url,this.m_options.webApplication){var i=se.embeddingPageOrigin+"="+e+"&"+se.embeddingPageSessionInfo+"="+this.m_options.sessionKey;0===r.search.length||"?"===r.search?r.search="?"+se.sessionContext+"="+encodeURIComponent(i):r.search=r.search+"&"+se.sessionContext+"="+encodeURIComponent(i)}else n?0===r.hash.length||"#"===r.hash?r.hash="#"+t:r.hash=r.hash+"&"+t:0===r.search.length||"?"===r.search?r.search="?"+t:r.search=r.search+"&"+t;return r.href},t.prototype.init=function(){var e=this;window.addEventListener("message",this.m_onMessageHandler);var t=this._getIFrameSrc();return u.createPromise((function(n,r){var i=document.createElement("iframe");if(e.m_options.id&&(i.id=e.m_options.id,i.name=e.m_options.id),i.style.height=e.m_options.height,i.style.width=e.m_options.width,e.m_options.webApplication){var a=document.createElement("form");a.setAttribute("action",t),a.setAttribute("method","post"),a.setAttribute("target",i.name),e.m_options.container.appendChild(a);var l=document.createElement("input");l.setAttribute("type","hidden"),l.setAttribute("name","access_token"),l.setAttribute("value",e.m_options.webApplication.accessToken),a.appendChild(l);var c=document.createElement("input");c.setAttribute("type","hidden"),c.setAttribute("name","access_token_ttl"),c.setAttribute("value",e.m_options.webApplication.accessTokenTtl),a.appendChild(c),e.m_options.container.appendChild(i),a.submit()}else i.src=t,e.m_options.container.appendChild(i);e.m_timeoutId=window.setTimeout((function(){e.close();var t=$.createRuntimeError(o.timeout,u._getResourceString(s.timeout),"EmbeddedSession.init");r(t)}),e.m_options.timeoutInMilliseconds),e.m_promiseResolver=n}))},t.prototype._invoke=function(e,t,n){this.m_enabled?(ie&&(n=ie(n)),this._sendMessageWithCallback(this.m_id++,e,n,(function(e){oe&&(e=oe(e));var n=e.Error;delete e.Error,t(n||0,e)}))):t(5001,null)},t.prototype.close=function(){window.removeEventListener("message",this.m_onMessageHandler),window.clearTimeout(this.m_timeoutId),this.m_enabled=!1},t.prototype.getEventRegistration=function(e){return this.m_sessionEventManager||(this.m_sessionEventManager=new G(this._registerEventImpl.bind(this),this._unregisterEventImpl.bind(this))),this.m_sessionEventManager},t.prototype._createRequestExecutorOrNull=function(){return new le(this)},t.prototype._resolveRequestUrlAndHeaderInfo=function(){return u._createPromiseFromResult(null)},t.prototype._registerEventImpl=function(e,t){var n=this;return u.createPromise((function(r,i){n._sendMessageWithCallback(n.m_id++,ne.RegisterEventCommand,{EventId:e,TargetId:t},(function(){r(null)}))}))},t.prototype._unregisterEventImpl=function(e,t){var n=this;return u.createPromise((function(r,i){n._sendMessageWithCallback(n.m_id++,ne.UnregisterEventCommand,{EventId:e,TargetId:t},(function(){r()}))}))},t.prototype._onMessage=function(e){var t=this;if(this.m_enabled&&(!this.m_chosenWindow||this.m_chosenWindow===e.source&&this.m_chosenOrigin===e.origin)){var n=e.data;if(n&&n[ne.CommandKey]===ne.ApiReadyCommand)!this.m_chosenWindow&&this._isValidDescendant(e.source)&&n[ne.SessionInfoKey]===this.m_options.sessionKey&&(this.m_chosenWindow=e.source,this.m_chosenOrigin=e.origin,this._sendMessageWithCallback(this.m_id++,ne.GetAppContextCommand,null,(function(e){t._setupContext(e),window.clearTimeout(t.m_timeoutId),t.m_promiseResolver()})));else if(n&&n[ne.CommandKey]===ne.FireEventCommand){var r=n[ne.ParamsKey],i=r.EventId,o=r.TargetId,s=r.Data;if(this.m_sessionEventManager)for(var a=this.m_sessionEventManager.getHandlers(i,o),l=0;l<a.length;l++)a[l](s)}else if(n&&n.hasOwnProperty(ne.RespondingId)){var u=n[ne.RespondingId],c=this.m_callbackList[u];"function"==typeof c&&c(n[ne.ParamsKey]),delete this.m_callbackList[u]}}},t.prototype._sendMessageWithCallback=function(e,t,n,r){this.m_callbackList[e]=r;var i={};i[ne.SendingId]=e,i[ne.CommandKey]=t,i[ne.ParamsKey]=n,this.m_chosenWindow.postMessage(JSON.stringify(i),this.m_chosenOrigin)},t.prototype._isValidDescendant=function(e){function t(n){if(n===e)return!0;for(var r=0,i=n.frames.length;r<i;r++)if(t(n.frames[r]))return!0;return!1}for(var n=(this.m_options.container||document.body).getElementsByTagName("iframe"),r=0,i=n.length;r<i;r++)if(t(n[r].contentWindow))return!0;return!1},t.prototype._setupContext=function(e){this.m_appContext=e},t}(t);e.EmbeddedSession=ae;var le=function(){function e(e){this.m_session=e}return e.prototype.executeAsync=function(t,n,r){var i=this,o=l.buildMessageArrayForIRequestExecutor(t,n,r,e.SourceLibHeaderValue);return u.createPromise((function(t,n){i.m_session._invoke(ne.ExecuteMethodCommand,(function(e,n){var r;u.log("Response:"),u.log(JSON.stringify(n)),r=0==e?l.buildResponseOnSuccess(l.getResponseBodyFromSafeArray(n.Data),l.getResponseHeadersFromSafeArray(n.Data)):l.buildResponseOnError(n.error.Code,n.error.Message),t(r)}),e._transformMessageArrayIntoParams(o))}))},e._transformMessageArrayIntoParams=function(t){return{ArrayData:t,DdaMethod:{DispatchId:e.DispidExecuteRichApiRequestMethod}}},e.DispidExecuteRichApiRequestMethod=93,e.SourceLibHeaderValue="Embedded",e}()}(OfficeExtension||(OfficeExtension={}));__extends=this&&this.__extends||function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}();var OfficeFirstPartyDialog,OfficeFirstPartyAuth,OfficeCore,Office,__awaiter=this&&this.__awaiter||function(e,t,n,r){return new(n||(n=Promise))((function(i,o){function s(e){try{l(r.next(e))}catch(e){o(e)}}function a(e){try{l(r.throw(e))}catch(e){o(e)}}function l(e){e.done?i(e.value):new n((function(t){t(e.value)})).then(s,a)}l((r=r.apply(e,t||[])).next())}))},__generator=this&&this.__generator||function(e,t){var n,r,i,o,s={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return o={next:a(0),throw:a(1),return:a(2)},"function"==typeof Symbol&&(o[Symbol.iterator]=function(){return this}),o;function a(o){return function(a){return function(o){if(n)throw new TypeError("Generator is already executing.");for(;s;)try{if(n=1,r&&(i=2&o[0]?r.return:o[0]?r.throw||((i=r.return)&&i.call(r),0):r.next)&&!(i=i.call(r,o[1])).done)return i;switch(r=0,i&&(o=[2&o[0],i.value]),o[0]){case 0:case 1:i=o;break;case 4:return s.label++,{value:o[1],done:!1};case 5:s.label++,r=o[1],o=[0];continue;case 7:o=s.ops.pop(),s.trys.pop();continue;default:if(!(i=s.trys,(i=i.length>0&&i[i.length-1])||6!==o[0]&&2!==o[0])){s=0;continue}if(3===o[0]&&(!i||o[1]>i[0]&&o[1]<i[3])){s.label=o[1];break}if(6===o[0]&&s.label<i[1]){s.label=i[1],i=o;break}if(i&&s.label<i[2]){s.label=i[2],s.ops.push(o);break}i[2]&&s.ops.pop(),s.trys.pop();continue}o=t.call(e,s)}catch(e){o=[6,e],r=0}finally{n=i=0}if(5&o[0])throw o[1];return{value:o[0]?o[1]:void 0,done:!0}}([o,a])}}};!function(e){OfficeExtension.BatchApiHelper.createPropertyObject,OfficeExtension.BatchApiHelper.createMethodObject,OfficeExtension.BatchApiHelper.createIndexerObject,OfficeExtension.BatchApiHelper.createRootServiceObject;var t=OfficeExtension.BatchApiHelper.createTopLevelServiceObject,n=(OfficeExtension.BatchApiHelper.createChildItemObject,OfficeExtension.BatchApiHelper.invokeMethod),r=(OfficeExtension.BatchApiHelper.invokeEnsureUnchanged,OfficeExtension.BatchApiHelper.invokeSetProperty,OfficeExtension.Utility.isNullOrUndefined),i=(OfficeExtension.Utility.isUndefined,OfficeExtension.Utility.throwIfNotLoaded,OfficeExtension.Utility.throwIfApiNotSupported,OfficeExtension.Utility.load,OfficeExtension.Utility.retrieve,OfficeExtension.Utility.toJson),o=OfficeExtension.Utility.fixObjectPathIfNecessary,s=(OfficeExtension.Utility._handleNavigationPropertyResults,OfficeExtension.Utility.adjustToDateTime,OfficeExtension.Utility.processRetrieveResult),a=function(a){function l(){return null!==a&&a.apply(this,arguments)||this}return __extends(l,a),Object.defineProperty(l.prototype,"_className",{get:function(){return"BiShim"},enumerable:!0,configurable:!0}),l.prototype.initialize=function(e){n(this,"Initialize",0,[e],0,0)},l.prototype.getData=function(){return n(this,"getData",1,[],4,0)},l.prototype.setVisualObjects=function(e){n(this,"setVisualObjects",0,[e],2,0)},l.prototype.setVisualObjectsToPersist=function(e){n(this,"setVisualObjectsToPersist",0,[e],2,0)},l.prototype._handleResult=function(e){(a.prototype._handleResult.call(this,e),r(e))||o(this,e)},l.prototype._handleRetrieveResult=function(e,t){a.prototype._handleRetrieveResult.call(this,e,t),s(this,e,t)},l.newObject=function(n){return t(e.BiShim,n,"Microsoft.AgaveVisual.BiShim",!1,4)},l.prototype.toJSON=function(){return i(this,{},{})},l}(OfficeExtension.ClientObject);e.BiShim=a,function(e){e.generalException1="GeneralException"}(e.AgaveVisualErrorCodes||(e.AgaveVisualErrorCodes={}))}(OfficeCore||(OfficeCore={})),function(e){OfficeExtension.BatchApiHelper.createPropertyObject;var t=OfficeExtension.BatchApiHelper.createMethodObject,n=(OfficeExtension.BatchApiHelper.createIndexerObject,OfficeExtension.BatchApiHelper.createRootServiceObject,OfficeExtension.BatchApiHelper.createTopLevelServiceObject),r=(OfficeExtension.BatchApiHelper.createChildItemObject,OfficeExtension.BatchApiHelper.invokeMethod),i=OfficeExtension.BatchApiHelper.invokeEnsureUnchanged,o=(OfficeExtension.BatchApiHelper.invokeSetProperty,OfficeExtension.Utility.isNullOrUndefined),s=OfficeExtension.Utility.isUndefined,a=OfficeExtension.Utility.throwIfNotLoaded,l=(OfficeExtension.Utility.throwIfApiNotSupported,OfficeExtension.Utility.load),u=OfficeExtension.Utility.retrieve,c=OfficeExtension.Utility.toJson,p=OfficeExtension.Utility.fixObjectPathIfNecessary,f=(OfficeExtension.Utility._handleNavigationPropertyResults,OfficeExtension.Utility.adjustToDateTime,OfficeExtension.Utility.processRetrieveResult),h=function(i){function s(){return null!==i&&i.apply(this,arguments)||this}return __extends(s,i),Object.defineProperty(s.prototype,"_className",{get:function(){return"FlightingService"},enumerable:!0,configurable:!0}),s.prototype.getClientSessionId=function(){return r(this,"GetClientSessionId",1,[],4,0)},s.prototype.getDeferredFlights=function(){return r(this,"GetDeferredFlights",1,[],4,0)},s.prototype.getFeature=function(n,r,i,o){return t(e.ABType,this,"GetFeature",1,[n,r,i,o],!1,!1,null,4)},s.prototype.getFeatureGate=function(n,r){return t(e.ABType,this,"GetFeatureGate",1,[n,r],!1,!1,null,4)},s.prototype.resetOverride=function(e){r(this,"ResetOverride",0,[e],0,0)},s.prototype.setOverride=function(e,t,n){r(this,"SetOverride",0,[e,t,n],0,0)},s.prototype._handleResult=function(e){(i.prototype._handleResult.call(this,e),o(e))||p(this,e)},s.prototype._handleRetrieveResult=function(e,t){i.prototype._handleRetrieveResult.call(this,e,t),f(this,e,t)},s.newObject=function(t){return n(e.FlightingService,t,"Microsoft.Experiment.FlightingService",!1,4)},s.prototype.toJSON=function(){return c(this,{},{})},s}(OfficeExtension.ClientObject);e.FlightingService=h;var d=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ABType"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["value"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"value",{get:function(){return a("value",this._V,"ABType",this._isNull),this._V},enumerable:!0,configurable:!0}),t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!o(t)){var n=t;p(this,n),s(n.Value)||(this._V=n.Value)}},t.prototype.load=function(e){return l(this,e)},t.prototype.retrieve=function(e){return u(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),f(this,t,n)},t.prototype.toJSON=function(){return c(this,{value:this._V},{})},t.prototype.ensureUnchanged=function(e){i(this,e)},t}(OfficeExtension.ClientObject);e.ABType=d,function(e){e.boolean="Boolean",e.integer="Integer",e.string="String"}(e.FeatureType||(e.FeatureType={})),function(e){e.generalException="GeneralException"}(e.ExperimentErrorCodes||(e.ExperimentErrorCodes={}))}(OfficeCore||(OfficeCore={})),function(e){var t,n=function(){function e(e){this._dialogService=e}return e.prototype.close=function(){return this._dialogService.close(),this._dialogService.context.sync()},e.prototype.messageChild=function(e,t){DialogApiManager&&DialogApiManager.messageChildRichApiBridge&&DialogApiManager.messageChildRichApiBridge(e,t)},e}();e.Dialog=n,e.displayWebDialog=function(t,n){return void 0===n&&(n={}),new OfficeExtension.CoreUtility.Promise((function(i,o){if(n.width&&n.height&&(!r(n.width)||!r(n.height)))throw new OfficeExtension.Error({code:"InvalidArgument",message:'Dimensions must be "number%" or number.'});var s=new OfficeExtension.ClientRequestContext,a=OfficeCore.FirstPartyDialogService.newObject(s),l=new e.Dialog(a),u=a.onDialogMessage.add((function(e){OfficeExtension.Utility.log("dialogMessageHandler:"+JSON.stringify(e));var t,r,a,c=JSON.parse(e.message),p=c.errorCode?new OfficeExtension.Error((t=c.errorCode,(r={})[12002]={code:"InvalidUrl",message:"Cannot load URL, no such page or bad URL syntax."},r[12003]={code:"InvalidUrl",message:"HTTPS is required."},r[12004]={code:"Untrusted",message:"Domain is not trusted."},r[12005]={code:"InvalidUrl",message:"HTTPS is required."},r[12007]={code:"FailedToOpen",message:"Another dialog is already opened."},(a=r)[t]?a[t]:{code:"Unknown",message:"An unknown error has occured with code: "+t})):null,f=c.type,h=c.origin,d=c.message;switch(f){case 17:p?o(p):i(l);break;case 12:n.onMessage&&n.onMessage(d,l,h);break;case 10:default:12006===c.errorCode?(u&&(u.remove(),s.sync()),n.onClose&&n.onClose()):n.onRuntimeError&&(n.onRuntimeError(p,l),o(p))}return OfficeExtension.CoreUtility.Promise.resolve()}));return s.sync().then((function(){var e={width:n.width?parseInt(n.width):50,height:n.height?parseInt(n.height):50,displayInIFrame:n.displayInIFrame,dialogTitle:n.dialogTitle};return a.displayDialog(t,e),s.sync()})).catch((function(e){o(e)}))}));function r(e){return/^(\-|\+)?([0-9]+)%?$/.test(e)}},function(e){e[e.dialogMessageReceived=0]="dialogMessageReceived",e[e.dialogEventReceived=1]="dialogEventReceived"}(t||(t={}))}(OfficeFirstPartyDialog||(OfficeFirstPartyDialog={})),function(e){e.OfficeOnlineDomainList=["*.dod.online.office365.us","*.gov.online.office365.us","*.officeapps-df.live.com","*.officeapps.live.com","*.online.office.de","*.partner.officewebapps.cn"],e.isHostOriginTrusted=function(){if(void 0===window.external||void 0===window.external.GetContext){var t=OSF.getClientEndPoint()._targetUrl,n=t.split("/")[2].split(":")[0].split("?")[0];return 0!=t.indexOf("https:")||e.OfficeOnlineDomainList.forEach((function(e){if(0==e.indexOf("*.")&&(e=e.substring(2)),n.indexOf(e)==n.length-e.length)return!0})),!1}return!0}}(OfficeCore||(OfficeCore={})),function(e){var t=function(){function t(e){this.context=e}return Object.defineProperty(t.prototype,"roamingSettings",{get:function(){return this.m_roamingSettings||(this.m_roamingSettings=e.AuthenticationService.newObject(this.context).roamingSettings),this.m_roamingSettings},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tap",{get:function(){return this.m_tap||(this.m_tap=e.Tap.newObject(this.context)),this.m_tap},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"skill",{get:function(){return this.m_skill||(this.m_skill=e.Skill.newObject(this.context)),this.m_skill},enumerable:!0,configurable:!0}),t}();e.FirstPartyApis=t;var n=function(n){function r(e){return n.call(this,e)||this}return __extends(r,n),Object.defineProperty(r.prototype,"firstParty",{get:function(){return this.m_firstPartyApis||(this.m_firstPartyApis=new t(this)),this.m_firstPartyApis},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"flighting",{get:function(){return this.flightingService},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"telemetry",{get:function(){return this.m_telemetry||(this.m_telemetry=e.TelemetryService.newObject(this)),this.m_telemetry},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"ribbon",{get:function(){return this.m_ribbon||(this.m_ribbon=e.DynamicRibbon.newObject(this)),this.m_ribbon},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"bi",{get:function(){return this.m_biShim||(this.m_biShim=e.BiShim.newObject(this)),this.m_biShim},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"flightingService",{get:function(){return this.m_flightingService||(this.m_flightingService=e.FlightingService.newObject(this)),this.m_flightingService},enumerable:!0,configurable:!0}),r}(OfficeExtension.ClientRequestContext);e.RequestContext=n,e.run=function(t,n){return OfficeExtension.ClientRequestContext._runBatch("OfficeCore.run",arguments,(function(t){return new e.RequestContext(t)}))}}(OfficeCore||(OfficeCore={})),function(e){!function(e){function t(){var e=new OfficeCore.RequestContext;return"web"==OSF._OfficeAppFactory.getHostInfo().hostPlatform&&(e._customData="WacPartition"),e}e.isFeatureEnabled=function(e,n){return __awaiter(this,void 0,void 0,(function(){var r,i,o;return __generator(this,(function(s){switch(s.label){case 0:return r=t(),i=OfficeCore.License.newObject(r),o=i.isFeatureEnabled(e,n),[4,r.sync()];case 1:return s.sent(),[2,o.value]}}))}))},e.getFeatureTier=function(e,n){return __awaiter(this,void 0,void 0,(function(){var r,i,o;return __generator(this,(function(s){switch(s.label){case 0:return r=t(),i=OfficeCore.License.newObject(r),o=i.getFeatureTier(e,n),[4,r.sync()];case 1:return s.sent(),[2,o.value]}}))}))},e.isFreemiumUpsellEnabled=function(){return __awaiter(this,void 0,void 0,(function(){var e,n,r;return __generator(this,(function(i){switch(i.label){case 0:return e=t(),n=OfficeCore.License.newObject(e),r=n.isFreemiumUpsellEnabled(),[4,e.sync()];case 1:return i.sent(),[2,r.value]}}))}))},e.launchUpsellExperience=function(e){return __awaiter(this,void 0,void 0,(function(){var n;return __generator(this,(function(r){switch(r.label){case 0:return n=t(),OfficeCore.License.newObject(n).launchUpsellExperience(e),[4,n.sync()];case 1:return r.sent(),[2]}}))}))},e.onFeatureStateChanged=function(e,n){return __awaiter(this,void 0,void 0,(function(){var r,i,o,s;return __generator(this,(function(a){switch(a.label){case 0:return r=t(),i=OfficeCore.License.newObject(r),(o=i.getLicenseFeature(e)).onStateChanged.add(n),s=function(){return o.onStateChanged.remove(n),null},[4,r.sync()];case 1:return a.sent(),[2,s]}}))}))},e.getMsaDeviceTicket=function(e,n,r){return __awaiter(this,void 0,void 0,(function(){var i,o,s;return __generator(this,(function(a){switch(a.label){case 0:return i=t(),o=OfficeCore.License.newObject(i),s=o.getMsaDeviceTicket(e,n,r),[4,i.sync()];case 1:return a.sent(),[2,s.value]}}))}))}}(e.license||(e.license={}))}(Office||(Office={})),function(e){OfficeExtension.BatchApiHelper.createPropertyObject,OfficeExtension.BatchApiHelper.createMethodObject,OfficeExtension.BatchApiHelper.createIndexerObject,OfficeExtension.BatchApiHelper.createRootServiceObject;var t,n=OfficeExtension.BatchApiHelper.createTopLevelServiceObject,r=(OfficeExtension.BatchApiHelper.createChildItemObject,OfficeExtension.BatchApiHelper.invokeMethod),i=(OfficeExtension.BatchApiHelper.invokeEnsureUnchanged,OfficeExtension.BatchApiHelper.invokeSetProperty,OfficeExtension.Utility.isNullOrUndefined),o=(OfficeExtension.Utility.isUndefined,OfficeExtension.Utility.throwIfNotLoaded,OfficeExtension.Utility.throwIfApiNotSupported,OfficeExtension.Utility.load,OfficeExtension.Utility.retrieve,OfficeExtension.Utility.toJson),s=OfficeExtension.Utility.fixObjectPathIfNecessary,a=(OfficeExtension.Utility._handleNavigationPropertyResults,OfficeExtension.Utility.adjustToDateTime,OfficeExtension.Utility.processRetrieveResult),l=(OfficeExtension.Utility.setMockData,OfficeExtension.CommonUtility.calculateApiFlags,function(l){function u(){return null!==l&&l.apply(this,arguments)||this}return __extends(u,l),Object.defineProperty(u.prototype,"_className",{get:function(){return"Skill"},enumerable:!0,configurable:!0}),u.prototype.executeAction=function(e,t,n){return r(this,"ExecuteAction",1,[e,t,n],5,0)},u.prototype.notifyPaneEvent=function(e,t){r(this,"NotifyPaneEvent",1,[e,t],5,0)},u.prototype.registerHostSkillEvent=function(){r(this,"RegisterHostSkillEvent",0,[],1,0)},u.prototype.testFireEvent=function(){r(this,"TestFireEvent",0,[],1,0)},u.prototype.unregisterHostSkillEvent=function(){r(this,"UnregisterHostSkillEvent",0,[],1,0)},u.prototype._handleResult=function(e){(l.prototype._handleResult.call(this,e),i(e))||s(this,e)},u.prototype._handleRetrieveResult=function(e,t){l.prototype._handleRetrieveResult.call(this,e,t),a(this,e,t)},u.newObject=function(t){return n(e.Skill,t,"Microsoft.SkillApi.Skill",!1,4)},Object.defineProperty(u.prototype,"onHostSkillEvent",{get:function(){var e=this;return this.m_hostSkillEvent||(this.m_hostSkillEvent=new OfficeExtension.GenericEventHandlers(this.context,this,"HostSkillEvent",{eventType:65538,registerFunc:function(){return e.registerHostSkillEvent()},unregisterFunc:function(){return e.unregisterHostSkillEvent()},getTargetIdFunc:function(){return""},eventArgsTransformFunc:function(n){var r=t.Skill_HostSkillEvent_EventArgsTransform(e,n);return OfficeExtension.Utility._createPromiseFromResult(r)}})),this.m_hostSkillEvent},enumerable:!0,configurable:!0}),u.prototype.toJSON=function(){return o(this,{},{})},u}(OfficeExtension.ClientObject));e.Skill=l,function(e){e.Skill_HostSkillEvent_EventArgsTransform=function(e,t){return{type:t.type,data:t.data}}}(t=e._CC||(e._CC={})),function(e){e.generalException="GeneralException"}(e.SkillErrorCodes||(e.SkillErrorCodes={}))}(OfficeCore||(OfficeCore={})),function(e){OfficeExtension.BatchApiHelper.createPropertyObject,OfficeExtension.BatchApiHelper.createMethodObject,OfficeExtension.BatchApiHelper.createIndexerObject,OfficeExtension.BatchApiHelper.createRootServiceObject;var t=OfficeExtension.BatchApiHelper.createTopLevelServiceObject,n=(OfficeExtension.BatchApiHelper.createChildItemObject,OfficeExtension.BatchApiHelper.invokeMethod),r=(OfficeExtension.BatchApiHelper.invokeEnsureUnchanged,OfficeExtension.BatchApiHelper.invokeSetProperty,OfficeExtension.Utility.isNullOrUndefined),i=(OfficeExtension.Utility.isUndefined,OfficeExtension.Utility.throwIfNotLoaded,OfficeExtension.Utility.throwIfApiNotSupported),o=(OfficeExtension.Utility.load,OfficeExtension.Utility.retrieve,OfficeExtension.Utility.toJson),s=OfficeExtension.Utility.fixObjectPathIfNecessary,a=(OfficeExtension.Utility._handleNavigationPropertyResults,OfficeExtension.Utility.adjustToDateTime,OfficeExtension.Utility.processRetrieveResult),l=(OfficeExtension.Utility.setMockData,OfficeExtension.CommonUtility.calculateApiFlags,function(l){function u(){return null!==l&&l.apply(this,arguments)||this}return __extends(u,l),Object.defineProperty(u.prototype,"_className",{get:function(){return"TelemetryService"},enumerable:!0,configurable:!0}),u.prototype.sendCustomerContent=function(e,t,r,o,s){i("TelemetryService.sendCustomerContent","Telemetry","1.3","OfficeCore"),n(this,"SendCustomerContent",1,[e,t,r,o,s],4,0)},u.prototype.sendTelemetryEvent=function(e,t,r,i,o){n(this,"SendTelemetryEvent",1,[e,t,r,i,o],4,0)},u.prototype._handleResult=function(e){(l.prototype._handleResult.call(this,e),r(e))||s(this,e)},u.prototype._handleRetrieveResult=function(e,t){l.prototype._handleRetrieveResult.call(this,e,t),a(this,e,t)},u.newObject=function(n){return t(e.TelemetryService,n,"Microsoft.Telemetry.TelemetryService",!1,4)},u.prototype.toJSON=function(){return o(this,{},{})},u}(OfficeExtension.ClientObject));e.TelemetryService=l,function(e){e.unset="Unset",e.string="String",e.boolean="Boolean",e.int64="Int64",e.double="Double"}(e.DataFieldType||(e.DataFieldType={})),function(e){e.generalException="GeneralException"}(e.TelemetryErrorCodes||(e.TelemetryErrorCodes={}))}(OfficeCore||(OfficeCore={})),function(e){var t="officeWebAuthReplyUrls",n=!1;function r(r){return new OfficeExtension.CoreUtility.Promise((function(o,s){if(OSF.WebAuth&&"web"==OSF._OfficeAppFactory.getHostInfo().hostPlatform){var a=!1;try{if(!Office||!Office.context||!Office.context.webAuth)return void s({code:"GetAuthContextAsyncMissing",message:"Office:["+!Office+"],Office.context:["+!Office.context+"],Office.context.webAuth:["+!Office.context.webAuth+"]"});Office.context.webAuth.getAuthContextAsync((function(l){if("succeeded"===l.status){a=!0;var u=l.value;if(!u||u.isAnonymous)return void s({code:"CannotGetAuthContext",message:u.Error});var c="msa"===u.authorityType.toLowerCase();if(OSF.WebAuth.config={authFlow:e.authFlow,authVersion:e.authVersion?e.authVersion:null,msal:"flight"===e.authVersion?e.msal:null,delayWebAuth:e.delay,delayMsal:e.delayMsal,debugging:e.debugging,authority:e.authorityOverride?e.authorityOverride:u.authority,idp:u.authorityType.toLowerCase(),appIds:[c&&u.msaAppId?u.msaAppId:u.appId],redirectUri:r||null,upn:u.upn,telemetryInstance:"otel",autoPopup:e.autoPopup,enableUpnCheck:e.upnCheck,enableConsoleLogging:e.debugging},OSF.WebAuth.load().then((function(e){i(e,n=!0),o()})).catch((function(e){i(e,n),s({code:"PackageNotLoaded",message:e instanceof Event?e.type:e})})),"implicit"===e.authFlow){var p=r||window.location.href.split("?")[0],f=sessionStorage.getItem(t);f||""===f?f=p:f+=", "+p,f&&sessionStorage.setItem(t,f)}}else OSF.WebAuth.config=null,s({code:"CannotGetAuthContext",message:l.status})}))}catch(e){OSF.WebAuth.config=null,OSF.WebAuth.load().then((function(){o()})).catch((function(){s({code:a?"CannotGetAuthContext":"FailedToLoad",message:e})}))}}else o()}))}function i(e,t){"undefined"!=typeof OTel&&OTel.OTelLogger.onTelemetryLoaded((function(){var n=[oteljs.makeStringDataField("IdentityProvider",OSF.WebAuth.config.idp),oteljs.makeStringDataField("AppId",OSF.WebAuth.config.appIds[0]),oteljs.makeStringDataField("Target",OSF.WebAuth.config.authFlow),oteljs.makeBooleanDataField("Result",t),oteljs.makeStringDataField("Error",e instanceof Event?e.type:"")];if(e&&!(e instanceof Event)&&e.Telemetry)for(var r in e.Telemetry)if(e.Telemetry[r])switch(r){case"succeeded":n.push(oteljs.makeBooleanDataField(r,e.Telemetry[r]));break;case"loadedApplicationCount":case"timeToLoad":n.push(oteljs.makeInt64DataField(r,e.Telemetry[r]));break;default:n.push(oteljs.makeStringDataField(r,e.Telemetry[r]))}OTel.OTelLogger.sendTelemetryEvent({eventName:"Office.Extensibility.OfficeJs.OfficeFirstPartyAuth.Load",dataFields:n,eventFlags:{dataCategories:oteljs.DataCategories.ProductServiceUsage}})}))}function o(e,t,n){"undefined"!=typeof OTel&&OTel.OTelLogger.onTelemetryLoaded((function(){var r=[oteljs.makeStringDataField("IdentityProvider",OSF.WebAuth.config.idp),oteljs.makeStringDataField("AppId",OSF.WebAuth.config.appIds[0]),oteljs.makeStringDataField("Target",OSF.WebAuth.config.authFlow),oteljs.makeBooleanDataField("Result",t),oteljs.makeStringDataField("Error",e instanceof Event?e.type:e.ErrorCode),oteljs.makeBooleanDataField("Popup","boolean"==typeof n&&n)];if(e&&!(e instanceof Event)&&e.Telemetry)for(var i in e.Telemetry)if(e.Telemetry[i])switch(i){case"succeeded":r.push(oteljs.makeBooleanDataField(i,e.Telemetry[i]));break;case"timeToGetToken":r.push(oteljs.makeInt64DataField(i,e.Telemetry[i]));break;default:r.push(oteljs.makeStringDataField(i,e.Telemetry[i]))}OTel.OTelLogger.sendTelemetryEvent({eventName:"Office.Extensibility.OfficeJs.OfficeFirstPartyAuth.GetAccessToken",dataFields:r,eventFlags:{dataCategories:oteljs.DataCategories.ProductServiceUsage}})}))}e.authFlow="authcode",e.autoPopup=!1,e.upnCheck=!0,e.msal="https://alcdn.msauth.net/browser/2.21.0/js/msal-browser.min.js",e.debugging=!1,e.delay=0,e.delayMsal=0,e.load=r,e.getAccessToken=function(e,t){return new OfficeExtension.CoreUtility.Promise((function(r,i){if("web"==OSF._OfficeAppFactory.getHostInfo().hostPlatform)Office.context.webAuth.getAuthContextAsync((function(s){var a,l,u=!1;"succeeded"===s.status&&(s.value.supportsAuthToken&&(u=!0));if(u){var c=new OfficeCore.RequestContext,p=OfficeCore.AuthenticationService.newObject(c);c._customData="WacPartition";var f=p.getAccessToken(e,null);c.sync().then((function(){r(f.value)}))}else OSF.WebAuth&&n?OSF.WebAuth.config.appIds[0]&&OSF.WebAuth.getToken(e.resource,OSF.WebAuth.config.appIds[0],OSF._OfficeAppFactory.getHostInfo().osfControlAppCorrelationId,t&&t.popup?t.popup:null).then((function(e){o(e,!0,!(!t||!t.popup)&&t.popup),r({accessToken:e.Token,tokenIdenityType:"msa"==OSF.WebAuth.config.idp.toLowerCase()?OfficeCore.IdentityType.microsoftAccount:OfficeCore.IdentityType.organizationAccount})})).catch((function(e){o(e,!1,!(!t||!t.popup)&&t.popup),i({code:e.ErrorCode,message:e instanceof Event?e.type:e.ErrorMessage})})):(a=n,l=OSF.WebAuth.loadAttempts,"undefined"!=typeof OTel&&OTel.OTelLogger.onTelemetryLoaded((function(){var e=[oteljs.makeBooleanDataField("Loaded",a),oteljs.makeInt64DataField("LoadAttempts","number"==typeof l?l:0)];OTel.OTelLogger.sendTelemetryEvent({eventName:"Office.Extensibility.OfficeJs.OfficeFirstPartyAuth.UnexpectedAcquire",dataFields:e,eventFlags:{dataCategories:oteljs.DataCategories.ProductServiceUsage}})})))}));else{var s=new OfficeCore.RequestContext,a=OfficeCore.AuthenticationService.newObject(s),l=a.onTokenReceived.add((function(e){if(!OfficeExtension.CoreUtility.isNullOrUndefined(e))if(l.remove(),s.sync().catch((function(){})),0==e.code)r(e.tokenValue);else if(OfficeExtension.CoreUtility.isNullOrUndefined(e.errorInfo))i({code:e.code});else try{i(JSON.parse(e.errorInfo))}catch(t){i({code:e.code,message:e.errorInfo})}return null}));s.sync().then((function(){var t=a.getAccessToken(e,a._targetId);return s.sync().then((function(){if(OfficeExtension.CoreUtility.isNullOrUndefined(t.value))return null;var e=t.value.accessToken;OfficeExtension.CoreUtility.isNullOrUndefined(e)||r(t.value)}))})).catch((function(e){i(e)}))}}))},e.getPrimaryIdentityInfo=function(){var e=new OfficeCore.RequestContext,t=OfficeCore.AuthenticationService.newObject(e);e._customData="WacPartition";var n=t.getPrimaryIdentityInfo();return e.sync().then((function(){return n.value}))},e.getIdentities=function(){var e=new OfficeCore.RequestContext,t=OfficeCore.AuthenticationService.newObject(e).getIdentities();return e.sync().then((function(){return t.value}))},"undefined"!=typeof window&&window.OSF&&function(){try{if("undefined"==typeof window||!window.sessionStorage)return;var e=sessionStorage.getItem(t);null!==e&&-1!==e.indexOf(window.location.origin+window.location.pathname)&&r()}catch(e){console.error(e)}}()}(OfficeFirstPartyAuth||(OfficeFirstPartyAuth={})),function(e){var t="Office",n=OfficeExtension.BatchApiHelper.createPropertyObject,r=OfficeExtension.BatchApiHelper.createMethodObject,i=OfficeExtension.BatchApiHelper.createIndexerObject,o=(OfficeExtension.BatchApiHelper.createRootServiceObject,OfficeExtension.BatchApiHelper.createTopLevelServiceObject),s=OfficeExtension.BatchApiHelper.createChildItemObject,a=OfficeExtension.BatchApiHelper.invokeMethod,l=OfficeExtension.BatchApiHelper.invokeEnsureUnchanged,u=OfficeExtension.BatchApiHelper.invokeSetProperty,c=OfficeExtension.Utility.isNullOrUndefined,p=OfficeExtension.Utility.isUndefined,f=OfficeExtension.Utility.throwIfNotLoaded,h=OfficeExtension.Utility.throwIfApiNotSupported,d=OfficeExtension.Utility.load,y=OfficeExtension.Utility.retrieve,m=OfficeExtension.Utility.toJson,g=OfficeExtension.Utility.fixObjectPathIfNecessary,_=OfficeExtension.Utility._handleNavigationPropertyResults,b=OfficeExtension.Utility.adjustToDateTime,O=OfficeExtension.Utility.processRetrieveResult,v=OfficeExtension.Utility.setMockData,S=OfficeExtension.CommonUtility.calculateApiFlags;!function(e){e.organizationAccount="OrganizationAccount",e.microsoftAccount="MicrosoftAccount",e.unsupported="Unsupported"}(e.IdentityType||(e.IdentityType={}));var C=function(r){function i(){return null!==r&&r.apply(this,arguments)||this}return __extends(i,r),Object.defineProperty(i.prototype,"_className",{get:function(){return"AuthenticationService"},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"_navigationPropertyNames",{get:function(){return["roamingSettings"]},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"roamingSettings",{get:function(){return this._R||(this._R=n(e.RoamingSettingCollection,this,"RoamingSettings",!1,4)),this._R},enumerable:!0,configurable:!0}),i.prototype.getAccessToken=function(e,t){return a(this,"GetAccessToken",1,[e,t],5,0)},i.prototype.getIdentities=function(){return h("AuthenticationService.getIdentities","FirstPartyAuthentication","1.3",t),a(this,"GetIdentities",1,[],5,0)},i.prototype.getPrimaryIdentityInfo=function(){return h("AuthenticationService.getPrimaryIdentityInfo","FirstPartyAuthentication","1.2",t),a(this,"GetPrimaryIdentityInfo",1,[],5,0)},i.prototype._handleResult=function(e){if(r.prototype._handleResult.call(this,e),!c(e)){var t=e;g(this,t),_(this,t,["roamingSettings","RoamingSettings"])}},i.prototype.load=function(e){return d(this,e)},i.prototype.retrieve=function(e){return y(this,e)},i.prototype._handleRetrieveResult=function(e,t){r.prototype._handleRetrieveResult.call(this,e,t),O(this,e,t)},i.newObject=function(t){return o(e.AuthenticationService,t,"Microsoft.Authentication.AuthenticationService",!1,4)},Object.defineProperty(i.prototype,"onTokenReceived",{get:function(){var e=this;return h("AuthenticationService.onTokenReceived","FirstPartyAuthentication","1.2",t),this.m_tokenReceived||(this.m_tokenReceived=new OfficeExtension.GenericEventHandlers(this.context,this,"TokenReceived",{eventType:3001,registerFunc:function(){return OfficeExtension.Utility._createPromiseFromResult(null)},unregisterFunc:function(){return OfficeExtension.Utility._createPromiseFromResult(null)},getTargetIdFunc:function(){return e._targetId},eventArgsTransformFunc:function(t){var n=P.AuthenticationService_TokenReceived_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_tokenReceived},enumerable:!0,configurable:!0}),i.prototype.toJSON=function(){return m(this,{},{})},i}(OfficeExtension.ClientObject);e.AuthenticationService=C;var P,A=function(){function e(){}return Object.defineProperty(e.prototype,"_targetId",{get:function(){return null==this.m_targetId&&(void 0!==OSF&&OSF.OUtil?this.m_targetId=OSF.OUtil.Guid.generateNewGuid():this.m_targetId=""+this.context._nextId()),this.m_targetId},enumerable:!0,configurable:!0}),e}();e.AuthenticationServiceCustom=A,OfficeExtension.Utility.applyMixin(C,A),function(e){e.AuthenticationService_TokenReceived_EventArgsTransform=function(e,t){var n=t;return{tokenValue:n.tokenValue,code:n.code,errorInfo:n.errorInfo}}}(P=e._CC||(e._CC={}));var F="RoamingSetting",D=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"RoamingSetting"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["id","value"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Value"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"id",{get:function(){return f("id",this._I,F,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"value",{get:function(){return f("value",this._V,F,this._isNull),this._V},set:function(e){this._V=e,u(this,"Value",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["value"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!c(t)){var n=t;g(this,n),p(n.Id)||(this._I=n.Id),p(n.Value)||(this._V=n.Value)}},t.prototype.load=function(e){return d(this,e)},t.prototype.retrieve=function(e){return y(this,e)},t.prototype._handleIdResult=function(t){e.prototype._handleIdResult.call(this,t),c(t)||p(t.Id)||(this._I=t.Id)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),O(this,t,n)},t.prototype.toJSON=function(){return m(this,{id:this._I,value:this._V},{})},t.prototype.setMockData=function(e){v(this,e)},t.prototype.ensureUnchanged=function(e){l(this,e)},t}(OfficeExtension.ClientObject);e.RoamingSetting=D;var E=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"RoamingSettingCollection"},enumerable:!0,configurable:!0}),n.prototype.getItem=function(t){return r(e.RoamingSetting,this,"GetItem",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return r(e.RoamingSetting,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.RoamingSettingCollection=E;var R=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"BeforeDocumentCloseNotification"},enumerable:!0,configurable:!0}),n.prototype.disable=function(){a(this,"Disable",1,[],4,0)},n.prototype.enable=function(){a(this,"Enable",1,[],4,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.BeforeDocumentCloseNotification,t,"Microsoft.BeforeDocumentCloseNotification.BeforeDocumentCloseNotification",!1,4)},Object.defineProperty(n.prototype,"onBeforeCloseNotificationCancelled",{get:function(){return this.m_beforeCloseNotificationCancelled||(this.m_beforeCloseNotificationCancelled=new OfficeExtension.GenericEventHandlers(this.context,this,"BeforeCloseNotificationCancelled",{eventType:65540,registerFunc:function(){return OfficeExtension.Utility._createPromiseFromResult(null)},unregisterFunc:function(){return OfficeExtension.Utility._createPromiseFromResult(null)},getTargetIdFunc:function(){return""},eventArgsTransformFunc:function(e){return OfficeExtension.Utility._createPromiseFromResult({})}})),this.m_beforeCloseNotificationCancelled},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.BeforeDocumentCloseNotification=R,function(e){e.ariaBrowserPipeUrl="AriaBrowserPipeUrl",e.ariaUploadUrl="AriaUploadUrl",e.ariaVNextUploadUrl="AriaVNextUploadUrl",e.lokiAutoDiscoverUrl="LokiAutoDiscoverUrl"}(e.ServiceProvider||(e.ServiceProvider={}));var I=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ServiceUrlProvider"},enumerable:!0,configurable:!0}),n.prototype.getServiceUrl=function(e,t){return a(this,"GetServiceUrl",1,[e,t],4,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.ServiceUrlProvider,t,"Microsoft.DesktopCompliance.ServiceUrlProvider",!1,4)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.ServiceUrlProvider=I;var x=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"LinkedIn"},enumerable:!0,configurable:!0}),n.prototype.isEnabledForOffice=function(){return a(this,"IsEnabledForOffice",1,[],4,0)},n.prototype.recordLinkedInSettingsCompliance=function(e,t){a(this,"RecordLinkedInSettingsCompliance",0,[e,t],0,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.LinkedIn,t,"Microsoft.DesktopCompliance.LinkedIn",!1,4)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.LinkedIn=x;var N=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"NetworkUsage"},enumerable:!0,configurable:!0}),n.prototype.isInDisconnectedMode=function(){return a(this,"IsInDisconnectedMode",1,[],4,0)},n.prototype.isInOnlineMode=function(){return a(this,"IsInOnlineMode",1,[],4,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.NetworkUsage,t,"Microsoft.DesktopCompliance.NetworkUsage",!1,4)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.NetworkUsage=N;var T=function(i){function s(){return null!==i&&i.apply(this,arguments)||this}return __extends(s,i),Object.defineProperty(s.prototype,"_className",{get:function(){return"DynamicRibbon"},enumerable:!0,configurable:!0}),Object.defineProperty(s.prototype,"_navigationPropertyNames",{get:function(){return["buttons"]},enumerable:!0,configurable:!0}),Object.defineProperty(s.prototype,"buttons",{get:function(){return this._B||(this._B=n(e.RibbonButtonCollection,this,"Buttons",!0,4)),this._B},enumerable:!0,configurable:!0}),s.prototype.executeRequestCreate=function(e){h("DynamicRibbon.executeRequestCreate","DynamicRibbon","1.2",t),a(this,"ExecuteRequestCreate",1,[e],4,0)},s.prototype.executeRequestUpdate=function(e){a(this,"ExecuteRequestUpdate",1,[e],4,0)},s.prototype.getButton=function(t){return r(e.RibbonButton,this,"GetButton",1,[t],!1,!1,null,4)},s.prototype.getTab=function(t){return r(e.RibbonTab,this,"GetTab",1,[t],!1,!1,null,4)},s.prototype._handleResult=function(e){if(i.prototype._handleResult.call(this,e),!c(e)){var t=e;g(this,t),_(this,t,["buttons","Buttons"])}},s.prototype.load=function(e){return d(this,e)},s.prototype.retrieve=function(e){return y(this,e)},s.prototype._handleRetrieveResult=function(e,t){i.prototype._handleRetrieveResult.call(this,e,t),O(this,e,t)},s.newObject=function(t){return o(e.DynamicRibbon,t,"Microsoft.DynamicRibbon.DynamicRibbon",!1,4)},s.prototype.toJSON=function(){return m(this,{},{buttons:this._B})},s}(OfficeExtension.ClientObject);e.DynamicRibbon=T;var w=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"RibbonTab"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["id"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"id",{get:function(){return f("id",this._I,"RibbonTab",this._isNull),this._I},enumerable:!0,configurable:!0}),t.prototype.setVisibility=function(e){a(this,"SetVisibility",0,[e],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!c(t)){var n=t;g(this,n),p(n.Id)||(this._I=n.Id)}},t.prototype.load=function(e){return d(this,e)},t.prototype.retrieve=function(e){return y(this,e)},t.prototype._handleIdResult=function(t){e.prototype._handleIdResult.call(this,t),c(t)||p(t.Id)||(this._I=t.Id)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),O(this,t,n)},t.prototype.toJSON=function(){return m(this,{id:this._I},{})},t.prototype.setMockData=function(e){v(this,e)},t.prototype.ensureUnchanged=function(e){l(this,e)},t}(OfficeExtension.ClientObject);e.RibbonTab=w;var j="RibbonButton",M=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"RibbonButton"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["id","enabled","label"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Enabled","Label"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"enabled",{get:function(){return f("enabled",this._E,j,this._isNull),this._E},set:function(e){this._E=e,u(this,"Enabled",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"id",{get:function(){return f("id",this._I,j,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"label",{get:function(){return f("label",this._L,j,this._isNull),this._L},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["enabled"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype.setEnabled=function(e){a(this,"SetEnabled",0,[e],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!c(t)){var n=t;g(this,n),p(n.Enabled)||(this._E=n.Enabled),p(n.Id)||(this._I=n.Id),p(n.Label)||(this._L=n.Label)}},t.prototype.load=function(e){return d(this,e)},t.prototype.retrieve=function(e){return y(this,e)},t.prototype._handleIdResult=function(t){e.prototype._handleIdResult.call(this,t),c(t)||p(t.Id)||(this._I=t.Id)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),O(this,t,n)},t.prototype.toJSON=function(){return m(this,{enabled:this._E,id:this._I,label:this._L},{})},t.prototype.setMockData=function(e){v(this,e)},t.prototype.ensureUnchanged=function(e){l(this,e)},t}(OfficeExtension.ClientObject);e.RibbonButton=M;var k=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"RibbonButtonCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return f("items",this.m__items,"RibbonButtonCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return a(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return i(e.RibbonButton,this,[t])},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!c(n)){var r=n;if(g(this,r),!c(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var a=s(e.RibbonButton,!0,this,i[o],o);a._handleResult(i[o]),this.m__items.push(a)}}}},n.prototype.load=function(e){return d(this,e)},n.prototype.retrieve=function(e){return y(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),O(this,n,r,(function(t,n){return s(e.RibbonButton,!0,i,t,n)}))},n.prototype.toJSON=function(){return m(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;v(this,t,(function(t,r){return s(e.RibbonButton,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.RibbonButtonCollection=k;var U=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"FirstPartyDialogService"},enumerable:!0,configurable:!0}),n.prototype.close=function(){a(this,"Close",1,[],5,0)},n.prototype.displayDialog=function(e,t){a(this,"DisplayDialog",1,[e,t],5,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.FirstPartyDialogService,t,"Microsoft.FirstPartyDialog.FirstPartyDialogService",!1,4)},Object.defineProperty(n.prototype,"onDialogMessage",{get:function(){var e=this;return this.m_dialogMessage||(this.m_dialogMessage=new OfficeExtension.GenericEventHandlers(this.context,this,"DialogMessage",{eventType:65536,registerFunc:function(){return OfficeExtension.Utility._createPromiseFromResult(null)},unregisterFunc:function(){return OfficeExtension.Utility._createPromiseFromResult(null)},getTargetIdFunc:function(){return""},eventArgsTransformFunc:function(t){var n=P.FirstPartyDialogService_DialogMessage_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_dialogMessage},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.FirstPartyDialogService=U,function(e){e.FirstPartyDialogService_DialogMessage_EventArgsTransform=function(e,t){return{message:t.message}}}(P=e._CC||(e._CC={})),function(e){e.shortTime="ShortTime",e.longTime="LongTime",e.shortDate="ShortDate",e.longDate="LongDate"}(e.TimeStringFormat||(e.TimeStringFormat={}));var L=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"LocaleApi"},enumerable:!0,configurable:!0}),n.prototype.formatDateTimeString=function(e,t,n){return a(this,"FormatDateTimeString",1,[e,t,n],4,0)},n.prototype.getLocaleDateTimeFormattingInfo=function(e){return a(this,"GetLocaleDateTimeFormattingInfo",1,[e],4,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.LocaleApi,t,"Microsoft.LocaleApi.LocaleApi",!1,4)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.LocaleApi=L;var B="Comment",H=function(t){function i(){return null!==t&&t.apply(this,arguments)||this}return __extends(i,t),Object.defineProperty(i.prototype,"_className",{get:function(){return"Comment"},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"_scalarPropertyNames",{get:function(){return["id","text","created","level","resolved","author","mentions"]},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Text","Created","Level","Resolved","Author","Mentions"]},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!1,!1,!0,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"_navigationPropertyNames",{get:function(){return["parent","parentOrNullObject","replies"]},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"parent",{get:function(){return this._P||(this._P=n(e.Comment,this,"Parent",!1,4)),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"parentOrNullObject",{get:function(){return this._Pa||(this._Pa=n(e.Comment,this,"ParentOrNullObject",!1,4)),this._Pa},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"replies",{get:function(){return this._R||(this._R=n(e.CommentCollection,this,"Replies",!0,4)),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"author",{get:function(){return f("author",this._A,B,this._isNull),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"created",{get:function(){return f("created",this._C,B,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"id",{get:function(){return f("id",this._I,B,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"level",{get:function(){return f("level",this._L,B,this._isNull),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"mentions",{get:function(){return f("mentions",this._M,B,this._isNull),this._M},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"resolved",{get:function(){return f("resolved",this._Re,B,this._isNull),this._Re},set:function(e){this._Re=e,u(this,"Resolved",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"text",{get:function(){return f("text",this._T,B,this._isNull),this._T},set:function(e){this._T=e,u(this,"Text",e,0)},enumerable:!0,configurable:!0}),i.prototype.set=function(e,t){this._recursivelySet(e,t,["text","resolved"],[],["parent","parentOrNullObject","replies"])},i.prototype.update=function(e){this._recursivelyUpdate(e)},i.prototype.delete=function(){a(this,"Delete",0,[],0,0)},i.prototype.getParentOrSelf=function(){return r(e.Comment,this,"GetParentOrSelf",1,[],!1,!1,null,4)},i.prototype.getRichText=function(e){return a(this,"GetRichText",1,[e],4,0)},i.prototype.reply=function(t,n){return r(e.Comment,this,"Reply",0,[t,n],!1,!1,null,0)},i.prototype.setRichText=function(e,t){return a(this,"SetRichText",0,[e,t],0,0)},i.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!c(e)){var n=e;g(this,n),p(n.Author)||(this._A=n.Author),p(n.Created)||(this._C=b(n.Created)),p(n.Id)||(this._I=n.Id),p(n.Level)||(this._L=n.Level),p(n.Mentions)||(this._M=n.Mentions),p(n.Resolved)||(this._Re=n.Resolved),p(n.Text)||(this._T=n.Text),_(this,n,["parent","Parent","parentOrNullObject","ParentOrNullObject","replies","Replies"])}},i.prototype.load=function(e){return d(this,e)},i.prototype.retrieve=function(e){return y(this,e)},i.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),c(e)||p(e.Id)||(this._I=e.Id)},i.prototype._handleRetrieveResult=function(e,n){if(t.prototype._handleRetrieveResult.call(this,e,n),!c(e)){var r=e;p(r.Created)||(r.created=b(r.created)),O(this,e,n)}},i.prototype.toJSON=function(){return m(this,{author:this._A,created:this._C,id:this._I,level:this._L,mentions:this._M,resolved:this._Re,text:this._T},{replies:this._R})},i.prototype.setMockData=function(e){v(this,e)},i.prototype.ensureUnchanged=function(e){l(this,e)},i}(OfficeExtension.ClientObject);e.Comment=H;var W=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"CommentCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return f("items",this.m__items,"CommentCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return a(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return i(e.Comment,this,[t])},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!c(n)){var r=n;if(g(this,r),!c(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var a=s(e.Comment,!0,this,i[o],o);a._handleResult(i[o]),this.m__items.push(a)}}}},n.prototype.load=function(e){return d(this,e)},n.prototype.retrieve=function(e){return y(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),O(this,n,r,(function(t,n){return s(e.Comment,!0,i,t,n)}))},n.prototype.toJSON=function(){return m(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;v(this,t,(function(t,r){return s(e.Comment,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.CommentCollection=W;var V=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ExtensionLifeCycle"},enumerable:!0,configurable:!0}),n.prototype.launchExtensionComponent=function(e,t,n){a(this,"LaunchExtensionComponent",1,[e,t,n],5,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.ExtensionLifeCycle,t,"Microsoft.OfficeSharedApi.ExtensionLifeCycle",!1,4)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.ExtensionLifeCycle=V,function(e){e.plain="Plain",e.markdown="Markdown",e.delta="Delta"}(e.CommentTextFormat||(e.CommentTextFormat={})),function(e){e.placeHolderRendered="PlaceHolderRendered",e.initialCardRendered="InitialCardRendered"}(e.PersonaCardPerfPoint||(e.PersonaCardPerfPoint={})),function(e){e.notSet="NotSet",e.free="Free",e.idle="Idle",e.busy="Busy",e.idleBusy="IdleBusy",e.doNotDisturb="DoNotDisturb",e.unalertable="Unalertable",e.unavailable="Unavailable"}(e.UnifiedCommunicationAvailability||(e.UnifiedCommunicationAvailability={})),function(e){e.online="Online",e.notOnline="NotOnline",e.away="Away",e.busy="Busy",e.beRightBack="BeRightBack",e.onThePhone="OnThePhone",e.outToLunch="OutToLunch",e.inAMeeting="InAMeeting",e.outOfOffice="OutOfOffice",e.doNotDisturb="DoNotDisturb",e.inAConference="InAConference",e.getting="Getting",e.notABuddy="NotABuddy",e.disconnected="Disconnected",e.notInstalled="NotInstalled",e.urgentInterruptionsOnly="UrgentInterruptionsOnly",e.mayBeAvailable="MayBeAvailable",e.idle="Idle",e.inPresentation="InPresentation"}(e.UnifiedCommunicationStatus||(e.UnifiedCommunicationStatus={})),function(e){e.free="Free",e.busy="Busy",e.idle="Idle",e.doNotDistrub="DoNotDistrub",e.blocked="Blocked",e.notSet="NotSet",e.outOfOffice="OutOfOffice"}(e.UnifiedCommunicationPresence||(e.UnifiedCommunicationPresence={})),function(e){e.unknown="Unknown",e.free="Free",e.busy="Busy",e.elsewhere="Elsewhere",e.tentative="Tentative",e.outOfOffice="OutOfOffice"}(e.FreeBusyCalendarState||(e.FreeBusyCalendarState={})),function(e){e.unknown="Unknown",e.enterprise="Enterprise",e.contact="Contact",e.bot="Bot",e.phoneOnly="PhoneOnly",e.oneOff="OneOff",e.distributionList="DistributionList",e.personalDistributionList="PersonalDistributionList",e.anonymous="Anonymous",e.unifiedGroup="UnifiedGroup"}(e.PersonaType||(e.PersonaType={})),function(e){e.workPhone="WorkPhone",e.homePhone="HomePhone",e.mobilePhone="MobilePhone",e.businessFax="BusinessFax",e.otherPhone="OtherPhone"}(e.PhoneType||(e.PhoneType={})),function(e){e.workAddress="WorkAddress",e.homeAddress="HomeAddress",e.otherAddress="OtherAddress"}(e.AddressType||(e.AddressType={})),function(e){e.unknown="Unknown",e.individual="Individual",e.group="Group"}(e.MemberType||(e.MemberType={}));var G,z="MemberInfoList",q=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"MemberInfoList"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["isWarmedUp","isWarmingUp"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["IsWarmedUp","IsWarmingUp"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isWarmedUp",{get:function(){return f("isWarmedUp",this._I,z,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isWarmingUp",{get:function(){return f("isWarmingUp",this._Is,z,this._isNull),this._Is},enumerable:!0,configurable:!0}),n.prototype.getPersonaForMember=function(t){return r(e.Persona,this,"GetPersonaForMember",1,[t],!1,!1,null,4)},n.prototype.items=function(){return a(this,"Items",1,[],4,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!c(e)){var n=e;g(this,n),p(n.IsWarmedUp)||(this._I=n.IsWarmedUp),p(n.IsWarmingUp)||(this._Is=n.IsWarmingUp)}},n.prototype.load=function(e){return d(this,e)},n.prototype.retrieve=function(e){return y(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.prototype.toJSON=function(){return m(this,{isWarmedUp:this._I,isWarmingUp:this._Is},{})},n.prototype.setMockData=function(e){v(this,e)},n.prototype.ensureUnchanged=function(e){l(this,e)},n}(OfficeExtension.ClientObject);e.MemberInfoList=q,function(e){e.hostId="HostId",e.type="Type",e.photo="Photo",e.personaInfo="PersonaInfo",e.unifiedCommunicationInfo="UnifiedCommunicationInfo",e.organization="Organization",e.unifiedGroupInfo="UnifiedGroupInfo",e.members="Members",e.membership="Membership",e.capabilities="Capabilities",e.customizations="Customizations",e.viewableSources="ViewableSources",e.placeholder="Placeholder"}(G=e.PersonaDataUpdated||(e.PersonaDataUpdated={}));var J=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"PersonaActions"},enumerable:!0,configurable:!0}),t.prototype.addContact=function(){a(this,"AddContact",0,[],0,0)},t.prototype.callPhoneNumber=function(e){a(this,"CallPhoneNumber",0,[e],0,0)},t.prototype.composeEmail=function(e){a(this,"ComposeEmail",0,[e],0,0)},t.prototype.composeInstantMessage=function(e){a(this,"ComposeInstantMessage",0,[e],0,0)},t.prototype.editContact=function(){a(this,"EditContact",0,[],0,0)},t.prototype.editContactByIdentifier=function(e){a(this,"EditContactByIdentifier",0,[e],0,0)},t.prototype.editUnifiedGroup=function(){a(this,"EditUnifiedGroup",0,[],0,0)},t.prototype.getChangePhotoUrlAndOpenInBrowser=function(){a(this,"GetChangePhotoUrlAndOpenInBrowser",0,[],0,0)},t.prototype.hideHoverCardForPersona=function(){a(this,"HideHoverCardForPersona",0,[],0,0)},t.prototype.joinUnifiedGroup=function(){a(this,"JoinUnifiedGroup",0,[],0,0)},t.prototype.leaveUnifiedGroup=function(){a(this,"LeaveUnifiedGroup",0,[],0,0)},t.prototype.openGroupCalendar=function(){a(this,"OpenGroupCalendar",0,[],0,0)},t.prototype.openLinkContactUx=function(){a(this,"OpenLinkContactUx",0,[],0,0)},t.prototype.openOutlookProperties=function(){a(this,"OpenOutlookProperties",0,[],0,0)},t.prototype.pinPersonaToQuickContacts=function(){a(this,"PinPersonaToQuickContacts",0,[],0,0)},t.prototype.scheduleMeeting=function(){a(this,"ScheduleMeeting",0,[],0,0)},t.prototype.showContactCard=function(e,t,n,r,i,o){a(this,"ShowContactCard",0,[e,t,n,r,i,o],0,0)},t.prototype.showContextMenu=function(e,t,n,r,i,o){a(this,"ShowContextMenu",0,[e,t,n,r,i,o],0,0)},t.prototype.showExpandedCard=function(e,t,n,r,i,o){a(this,"ShowExpandedCard",0,[e,t,n,r,i,o],0,0)},t.prototype.showHoverCardForPersona=function(e,t,n,r,i,o){a(this,"ShowHoverCardForPersona",0,[e,t,n,r,i,o],0,0)},t.prototype.startAudioCall=function(){a(this,"StartAudioCall",0,[],0,0)},t.prototype.startVideoCall=function(){a(this,"StartVideoCall",0,[],0,0)},t.prototype.subscribeToGroup=function(){a(this,"SubscribeToGroup",0,[],0,0)},t.prototype.toggleTagForAlerts=function(){a(this,"ToggleTagForAlerts",0,[],0,0)},t.prototype.unsubscribeFromGroup=function(){a(this,"UnsubscribeFromGroup",0,[],0,0)},t.prototype._handleResult=function(t){(e.prototype._handleResult.call(this,t),c(t))||g(this,t)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),O(this,t,n)},t.prototype.toJSON=function(){return m(this,{},{})},t}(OfficeExtension.ClientObject);e.PersonaActions=J;var K="PersonaInfoSource",Q=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"PersonaInfoSource"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["displayName","email","emailAddresses","sipAddresses","birthday","birthdays","title","jobInfoDepartment","companyName","office","linkedTitles","linkedDepartments","linkedCompanyNames","linkedOffices","phones","addresses","webSites","notes"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["DisplayName","Email","EmailAddresses","SipAddresses","Birthday","Birthdays","Title","JobInfoDepartment","CompanyName","Office","LinkedTitles","LinkedDepartments","LinkedCompanyNames","LinkedOffices","Phones","Addresses","WebSites","Notes"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"addresses",{get:function(){return f("addresses",this._A,K,this._isNull),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"birthday",{get:function(){return f("birthday",this._B,K,this._isNull),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"birthdays",{get:function(){return f("birthdays",this._Bi,K,this._isNull),this._Bi},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"companyName",{get:function(){return f("companyName",this._C,K,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"displayName",{get:function(){return f("displayName",this._D,K,this._isNull),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"email",{get:function(){return f("email",this._E,K,this._isNull),this._E},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"emailAddresses",{get:function(){return f("emailAddresses",this._Em,K,this._isNull),this._Em},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"jobInfoDepartment",{get:function(){return f("jobInfoDepartment",this._J,K,this._isNull),this._J},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"linkedCompanyNames",{get:function(){return f("linkedCompanyNames",this._L,K,this._isNull),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"linkedDepartments",{get:function(){return f("linkedDepartments",this._Li,K,this._isNull),this._Li},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"linkedOffices",{get:function(){return f("linkedOffices",this._Lin,K,this._isNull),this._Lin},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"linkedTitles",{get:function(){return f("linkedTitles",this._Link,K,this._isNull),this._Link},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"notes",{get:function(){return f("notes",this._N,K,this._isNull),this._N},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"office",{get:function(){return f("office",this._O,K,this._isNull),this._O},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"phones",{get:function(){return f("phones",this._P,K,this._isNull),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"sipAddresses",{get:function(){return f("sipAddresses",this._S,K,this._isNull),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"title",{get:function(){return f("title",this._T,K,this._isNull),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"webSites",{get:function(){return f("webSites",this._W,K,this._isNull),this._W},enumerable:!0,configurable:!0}),t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!c(t)){var n=t;g(this,n),p(n.Addresses)||(this._A=n.Addresses),p(n.Birthday)||(this._B=n.Birthday),p(n.Birthdays)||(this._Bi=n.Birthdays),p(n.CompanyName)||(this._C=n.CompanyName),p(n.DisplayName)||(this._D=n.DisplayName),p(n.Email)||(this._E=n.Email),p(n.EmailAddresses)||(this._Em=n.EmailAddresses),p(n.JobInfoDepartment)||(this._J=n.JobInfoDepartment),p(n.LinkedCompanyNames)||(this._L=n.LinkedCompanyNames),p(n.LinkedDepartments)||(this._Li=n.LinkedDepartments),p(n.LinkedOffices)||(this._Lin=n.LinkedOffices),p(n.LinkedTitles)||(this._Link=n.LinkedTitles),p(n.Notes)||(this._N=n.Notes),p(n.Office)||(this._O=n.Office),p(n.Phones)||(this._P=n.Phones),p(n.SipAddresses)||(this._S=n.SipAddresses),p(n.Title)||(this._T=n.Title),p(n.WebSites)||(this._W=n.WebSites)}},t.prototype.load=function(e){return d(this,e)},t.prototype.retrieve=function(e){return y(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),O(this,t,n)},t.prototype.toJSON=function(){return m(this,{addresses:this._A,birthday:this._B,birthdays:this._Bi,companyName:this._C,displayName:this._D,email:this._E,emailAddresses:this._Em,jobInfoDepartment:this._J,linkedCompanyNames:this._L,linkedDepartments:this._Li,linkedOffices:this._Lin,linkedTitles:this._Link,notes:this._N,office:this._O,phones:this._P,sipAddresses:this._S,title:this._T,webSites:this._W},{})},t.prototype.setMockData=function(e){v(this,e)},t.prototype.ensureUnchanged=function(e){l(this,e)},t}(OfficeExtension.ClientObject);e.PersonaInfoSource=Q;var X="PersonaInfo",Y=function(t){function r(){return null!==t&&t.apply(this,arguments)||this}return __extends(r,t),Object.defineProperty(r.prototype,"_className",{get:function(){return"PersonaInfo"},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"_scalarPropertyNames",{get:function(){return["displayName","email","emailAddresses","sipAddresses","birthday","birthdays","title","jobInfoDepartment","companyName","office","linkedTitles","linkedDepartments","linkedCompanyNames","linkedOffices","webSites","notes","isPersonResolved"]},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"_scalarPropertyOriginalNames",{get:function(){return["DisplayName","Email","EmailAddresses","SipAddresses","Birthday","Birthdays","Title","JobInfoDepartment","CompanyName","Office","LinkedTitles","LinkedDepartments","LinkedCompanyNames","LinkedOffices","WebSites","Notes","IsPersonResolved"]},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"_navigationPropertyNames",{get:function(){return["sources"]},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"sources",{get:function(){return this._So||(this._So=n(e.PersonaInfoSource,this,"Sources",!1,4)),this._So},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"birthday",{get:function(){return f("birthday",this._B,X,this._isNull),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"birthdays",{get:function(){return f("birthdays",this._Bi,X,this._isNull),this._Bi},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"companyName",{get:function(){return f("companyName",this._C,X,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"displayName",{get:function(){return f("displayName",this._D,X,this._isNull),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"email",{get:function(){return f("email",this._E,X,this._isNull),this._E},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"emailAddresses",{get:function(){return f("emailAddresses",this._Em,X,this._isNull),this._Em},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"isPersonResolved",{get:function(){return f("isPersonResolved",this._I,X,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"jobInfoDepartment",{get:function(){return f("jobInfoDepartment",this._J,X,this._isNull),this._J},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"linkedCompanyNames",{get:function(){return f("linkedCompanyNames",this._L,X,this._isNull),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"linkedDepartments",{get:function(){return f("linkedDepartments",this._Li,X,this._isNull),this._Li},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"linkedOffices",{get:function(){return f("linkedOffices",this._Lin,X,this._isNull),this._Lin},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"linkedTitles",{get:function(){return f("linkedTitles",this._Link,X,this._isNull),this._Link},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"notes",{get:function(){return f("notes",this._N,X,this._isNull),this._N},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"office",{get:function(){return f("office",this._O,X,this._isNull),this._O},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"sipAddresses",{get:function(){return f("sipAddresses",this._S,X,this._isNull),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"title",{get:function(){return f("title",this._T,X,this._isNull),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"webSites",{get:function(){return f("webSites",this._W,X,this._isNull),this._W},enumerable:!0,configurable:!0}),r.prototype.getAddresses=function(){return a(this,"GetAddresses",1,[],4,0)},r.prototype.getPhones=function(){return a(this,"GetPhones",1,[],4,0)},r.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!c(e)){var n=e;g(this,n),p(n.Birthday)||(this._B=b(n.Birthday)),p(n.Birthdays)||(this._Bi=b(n.Birthdays)),p(n.CompanyName)||(this._C=n.CompanyName),p(n.DisplayName)||(this._D=n.DisplayName),p(n.Email)||(this._E=n.Email),p(n.EmailAddresses)||(this._Em=n.EmailAddresses),p(n.IsPersonResolved)||(this._I=n.IsPersonResolved),p(n.JobInfoDepartment)||(this._J=n.JobInfoDepartment),p(n.LinkedCompanyNames)||(this._L=n.LinkedCompanyNames),p(n.LinkedDepartments)||(this._Li=n.LinkedDepartments),p(n.LinkedOffices)||(this._Lin=n.LinkedOffices),p(n.LinkedTitles)||(this._Link=n.LinkedTitles),p(n.Notes)||(this._N=n.Notes),p(n.Office)||(this._O=n.Office),p(n.SipAddresses)||(this._S=n.SipAddresses),p(n.Title)||(this._T=n.Title),p(n.WebSites)||(this._W=n.WebSites),_(this,n,["sources","Sources"])}},r.prototype.load=function(e){return d(this,e)},r.prototype.retrieve=function(e){return y(this,e)},r.prototype._handleRetrieveResult=function(e,n){if(t.prototype._handleRetrieveResult.call(this,e,n),!c(e)){var r=e;p(r.Birthday)||(r.birthday=b(r.birthday)),p(r.Birthdays)||(r.birthdays=b(r.birthdays)),O(this,e,n)}},r.prototype.toJSON=function(){return m(this,{birthday:this._B,birthdays:this._Bi,companyName:this._C,displayName:this._D,email:this._E,emailAddresses:this._Em,isPersonResolved:this._I,jobInfoDepartment:this._J,linkedCompanyNames:this._L,linkedDepartments:this._Li,linkedOffices:this._Lin,linkedTitles:this._Link,notes:this._N,office:this._O,sipAddresses:this._S,title:this._T,webSites:this._W},{sources:this._So})},r.prototype.setMockData=function(e){v(this,e)},r.prototype.ensureUnchanged=function(e){l(this,e)},r}(OfficeExtension.ClientObject);e.PersonaInfo=Y;var Z="PersonaUnifiedCommunicationInfo",$=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"PersonaUnifiedCommunicationInfo"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["availability","status","isSelf","isTagged","customStatusString","isBlocked","presenceTooltip","isOutOfOffice","outOfOfficeNote","timezone","meetingLocation","meetingSubject","timezoneBias","idleStartTime","overallCapability","isOnBuddyList","presenceNote","voiceMailUri","availabilityText","availabilityTooltip","isDurationInAvailabilityText","freeBusyStatus","calendarState","presence"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Availability","Status","IsSelf","IsTagged","CustomStatusString","IsBlocked","PresenceTooltip","IsOutOfOffice","OutOfOfficeNote","Timezone","MeetingLocation","MeetingSubject","TimezoneBias","IdleStartTime","OverallCapability","IsOnBuddyList","PresenceNote","VoiceMailUri","AvailabilityText","AvailabilityTooltip","IsDurationInAvailabilityText","FreeBusyStatus","CalendarState","Presence"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"availability",{get:function(){return f("availability",this._A,Z,this._isNull),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"availabilityText",{get:function(){return f("availabilityText",this._Av,Z,this._isNull),this._Av},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"availabilityTooltip",{get:function(){return f("availabilityTooltip",this._Ava,Z,this._isNull),this._Ava},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"calendarState",{get:function(){return f("calendarState",this._C,Z,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"customStatusString",{get:function(){return f("customStatusString",this._Cu,Z,this._isNull),this._Cu},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"freeBusyStatus",{get:function(){return f("freeBusyStatus",this._F,Z,this._isNull),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"idleStartTime",{get:function(){return f("idleStartTime",this._I,Z,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isBlocked",{get:function(){return f("isBlocked",this._Is,Z,this._isNull),this._Is},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isDurationInAvailabilityText",{get:function(){return f("isDurationInAvailabilityText",this._IsD,Z,this._isNull),this._IsD},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isOnBuddyList",{get:function(){return f("isOnBuddyList",this._IsO,Z,this._isNull),this._IsO},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isOutOfOffice",{get:function(){return f("isOutOfOffice",this._IsOu,Z,this._isNull),this._IsOu},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isSelf",{get:function(){return f("isSelf",this._IsS,Z,this._isNull),this._IsS},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isTagged",{get:function(){return f("isTagged",this._IsT,Z,this._isNull),this._IsT},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"meetingLocation",{get:function(){return f("meetingLocation",this._M,Z,this._isNull),this._M},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"meetingSubject",{get:function(){return f("meetingSubject",this._Me,Z,this._isNull),this._Me},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"outOfOfficeNote",{get:function(){return f("outOfOfficeNote",this._O,Z,this._isNull),this._O},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"overallCapability",{get:function(){return f("overallCapability",this._Ov,Z,this._isNull),this._Ov},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"presence",{get:function(){return f("presence",this._P,Z,this._isNull),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"presenceNote",{get:function(){return f("presenceNote",this._Pr,Z,this._isNull),this._Pr},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"presenceTooltip",{get:function(){return f("presenceTooltip",this._Pre,Z,this._isNull),this._Pre},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"status",{get:function(){return f("status",this._S,Z,this._isNull),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"timezone",{get:function(){return f("timezone",this._T,Z,this._isNull),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"timezoneBias",{get:function(){return f("timezoneBias",this._Ti,Z,this._isNull),this._Ti},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"voiceMailUri",{get:function(){return f("voiceMailUri",this._V,Z,this._isNull),this._V},enumerable:!0,configurable:!0}),t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!c(t)){var n=t;g(this,n),p(n.Availability)||(this._A=n.Availability),p(n.AvailabilityText)||(this._Av=n.AvailabilityText),p(n.AvailabilityTooltip)||(this._Ava=n.AvailabilityTooltip),p(n.CalendarState)||(this._C=n.CalendarState),p(n.CustomStatusString)||(this._Cu=n.CustomStatusString),p(n.FreeBusyStatus)||(this._F=n.FreeBusyStatus),p(n.IdleStartTime)||(this._I=b(n.IdleStartTime)),p(n.IsBlocked)||(this._Is=n.IsBlocked),p(n.IsDurationInAvailabilityText)||(this._IsD=n.IsDurationInAvailabilityText),p(n.IsOnBuddyList)||(this._IsO=n.IsOnBuddyList),p(n.IsOutOfOffice)||(this._IsOu=n.IsOutOfOffice),p(n.IsSelf)||(this._IsS=n.IsSelf),p(n.IsTagged)||(this._IsT=n.IsTagged),p(n.MeetingLocation)||(this._M=n.MeetingLocation),p(n.MeetingSubject)||(this._Me=n.MeetingSubject),p(n.OutOfOfficeNote)||(this._O=n.OutOfOfficeNote),p(n.OverallCapability)||(this._Ov=n.OverallCapability),p(n.Presence)||(this._P=n.Presence),p(n.PresenceNote)||(this._Pr=n.PresenceNote),p(n.PresenceTooltip)||(this._Pre=n.PresenceTooltip),p(n.Status)||(this._S=n.Status),p(n.Timezone)||(this._T=n.Timezone),p(n.TimezoneBias)||(this._Ti=n.TimezoneBias),p(n.VoiceMailUri)||(this._V=n.VoiceMailUri)}},t.prototype.load=function(e){return d(this,e)},t.prototype.retrieve=function(e){return y(this,e)},t.prototype._handleRetrieveResult=function(t,n){if(e.prototype._handleRetrieveResult.call(this,t,n),!c(t)){var r=t;p(r.IdleStartTime)||(r.idleStartTime=b(r.idleStartTime)),O(this,t,n)}},t.prototype.toJSON=function(){return m(this,{availability:this._A,availabilityText:this._Av,availabilityTooltip:this._Ava,calendarState:this._C,customStatusString:this._Cu,freeBusyStatus:this._F,idleStartTime:this._I,isBlocked:this._Is,isDurationInAvailabilityText:this._IsD,isOnBuddyList:this._IsO,isOutOfOffice:this._IsOu,isSelf:this._IsS,isTagged:this._IsT,meetingLocation:this._M,meetingSubject:this._Me,outOfOfficeNote:this._O,overallCapability:this._Ov,presence:this._P,presenceNote:this._Pr,presenceTooltip:this._Pre,status:this._S,timezone:this._T,timezoneBias:this._Ti,voiceMailUri:this._V},{})},t.prototype.setMockData=function(e){v(this,e)},t.prototype.ensureUnchanged=function(e){l(this,e)},t}(OfficeExtension.ClientObject);e.PersonaUnifiedCommunicationInfo=$;var ee=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"PersonaPhotoInfo"},enumerable:!0,configurable:!0}),t.prototype.getImageUri=function(e){return a(this,"getImageUri",1,[e],4,0)},t.prototype.getImageUriWithMetadata=function(e){return a(this,"getImageUriWithMetadata",1,[e],4,0)},t.prototype.getPlaceholderUri=function(e){return a(this,"getPlaceholderUri",1,[e],4,0)},t.prototype.setPlaceholderColor=function(e){a(this,"setPlaceholderColor",0,[e],0,0)},t.prototype._handleResult=function(t){(e.prototype._handleResult.call(this,t),c(t))||g(this,t)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),O(this,t,n)},t.prototype.toJSON=function(){return m(this,{},{})},t}(OfficeExtension.ClientObject);e.PersonaPhotoInfo=ee;var te=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PersonaCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return f("items",this.m__items,"PersonaCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return a(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return i(e.Persona,this,[t])},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!c(n)){var r=n;if(g(this,r),!c(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var a=s(e.Persona,!0,this,i[o],o);a._handleResult(i[o]),this.m__items.push(a)}}}},n.prototype.load=function(e){return d(this,e)},n.prototype.retrieve=function(e){return y(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),O(this,n,r,(function(t,n){return s(e.Persona,!0,i,t,n)}))},n.prototype.toJSON=function(){return m(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;v(this,t,(function(t,r){return s(e.Persona,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.PersonaCollection=te;var ne="PersonaOrganizationInfo",re=function(t){function r(){return null!==t&&t.apply(this,arguments)||this}return __extends(r,t),Object.defineProperty(r.prototype,"_className",{get:function(){return"PersonaOrganizationInfo"},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"_scalarPropertyNames",{get:function(){return["isWarmedUp","isWarmingUp"]},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"_scalarPropertyOriginalNames",{get:function(){return["IsWarmedUp","IsWarmingUp"]},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"_navigationPropertyNames",{get:function(){return["hierarchy","manager","directReports"]},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"directReports",{get:function(){return this._D||(this._D=n(e.PersonaCollection,this,"DirectReports",!0,4)),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"hierarchy",{get:function(){return this._H||(this._H=n(e.PersonaCollection,this,"Hierarchy",!0,4)),this._H},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"manager",{get:function(){return this._M||(this._M=n(e.Persona,this,"Manager",!1,4)),this._M},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"isWarmedUp",{get:function(){return f("isWarmedUp",this._I,ne,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(r.prototype,"isWarmingUp",{get:function(){return f("isWarmingUp",this._Is,ne,this._isNull),this._Is},enumerable:!0,configurable:!0}),r.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!c(e)){var n=e;g(this,n),p(n.IsWarmedUp)||(this._I=n.IsWarmedUp),p(n.IsWarmingUp)||(this._Is=n.IsWarmingUp),_(this,n,["directReports","DirectReports","hierarchy","Hierarchy","manager","Manager"])}},r.prototype.load=function(e){return d(this,e)},r.prototype.retrieve=function(e){return y(this,e)},r.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},r.prototype.toJSON=function(){return m(this,{isWarmedUp:this._I,isWarmingUp:this._Is},{})},r.prototype.setMockData=function(e){v(this,e)},r.prototype.ensureUnchanged=function(e){l(this,e)},r}(OfficeExtension.ClientObject);e.PersonaOrganizationInfo=re,function(e){e.email="Email",e.workPhone="WorkPhone",e.workPhone2="WorkPhone2",e.workFax="WorkFax",e.mobilePhone="MobilePhone",e.homePhone="HomePhone",e.homePhone2="HomePhone2",e.otherPhone="OtherPhone",e.sipAddress="SipAddress",e.profile="Profile",e.office="Office",e.company="Company",e.workAddress="WorkAddress",e.homeAddress="HomeAddress",e.otherAddress="OtherAddress",e.birthday="Birthday"}(e.CustomizedData||(e.CustomizedData={}));var ie="UnifiedGroupInfo",oe=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"UnifiedGroupInfo"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["description","oneDrive","oneNote","isPublic","amIOwner","amIMember","amISubscribed","memberCount","ownerCount","hasGuests","site","planner","classification","subscriptionEnabled"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Description","OneDrive","OneNote","IsPublic","AmIOwner","AmIMember","AmISubscribed","MemberCount","OwnerCount","HasGuests","Site","Planner","Classification","SubscriptionEnabled"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"amIMember",{get:function(){return f("amIMember",this._A,ie,this._isNull),this._A},set:function(e){this._A=e,u(this,"AmIMember",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"amIOwner",{get:function(){return f("amIOwner",this._Am,ie,this._isNull),this._Am},set:function(e){this._Am=e,u(this,"AmIOwner",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"amISubscribed",{get:function(){return f("amISubscribed",this._AmI,ie,this._isNull),this._AmI},set:function(e){this._AmI=e,u(this,"AmISubscribed",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"classification",{get:function(){return f("classification",this._C,ie,this._isNull),this._C},set:function(e){this._C=e,u(this,"Classification",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"description",{get:function(){return f("description",this._D,ie,this._isNull),this._D},set:function(e){this._D=e,u(this,"Description",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"hasGuests",{get:function(){return f("hasGuests",this._H,ie,this._isNull),this._H},set:function(e){this._H=e,u(this,"HasGuests",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isPublic",{get:function(){return f("isPublic",this._I,ie,this._isNull),this._I},set:function(e){this._I=e,u(this,"IsPublic",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"memberCount",{get:function(){return f("memberCount",this._M,ie,this._isNull),this._M},set:function(e){this._M=e,u(this,"MemberCount",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"oneDrive",{get:function(){return f("oneDrive",this._O,ie,this._isNull),this._O},set:function(e){this._O=e,u(this,"OneDrive",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"oneNote",{get:function(){return f("oneNote",this._On,ie,this._isNull),this._On},set:function(e){this._On=e,u(this,"OneNote",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"ownerCount",{get:function(){return f("ownerCount",this._Ow,ie,this._isNull),this._Ow},set:function(e){this._Ow=e,u(this,"OwnerCount",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"planner",{get:function(){return f("planner",this._P,ie,this._isNull),this._P},set:function(e){this._P=e,u(this,"Planner",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"site",{get:function(){return f("site",this._S,ie,this._isNull),this._S},set:function(e){this._S=e,u(this,"Site",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"subscriptionEnabled",{get:function(){return f("subscriptionEnabled",this._Su,ie,this._isNull),this._Su},set:function(e){this._Su=e,u(this,"SubscriptionEnabled",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["description","oneDrive","oneNote","isPublic","amIOwner","amIMember","amISubscribed","memberCount","ownerCount","hasGuests","site","planner","classification","subscriptionEnabled"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!c(t)){var n=t;g(this,n),p(n.AmIMember)||(this._A=n.AmIMember),p(n.AmIOwner)||(this._Am=n.AmIOwner),p(n.AmISubscribed)||(this._AmI=n.AmISubscribed),p(n.Classification)||(this._C=n.Classification),p(n.Description)||(this._D=n.Description),p(n.HasGuests)||(this._H=n.HasGuests),p(n.IsPublic)||(this._I=n.IsPublic),p(n.MemberCount)||(this._M=n.MemberCount),p(n.OneDrive)||(this._O=n.OneDrive),p(n.OneNote)||(this._On=n.OneNote),p(n.OwnerCount)||(this._Ow=n.OwnerCount),p(n.Planner)||(this._P=n.Planner),p(n.Site)||(this._S=n.Site),p(n.SubscriptionEnabled)||(this._Su=n.SubscriptionEnabled)}},t.prototype.load=function(e){return d(this,e)},t.prototype.retrieve=function(e){return y(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),O(this,t,n)},t.prototype.toJSON=function(){return m(this,{amIMember:this._A,amIOwner:this._Am,amISubscribed:this._AmI,classification:this._C,description:this._D,hasGuests:this._H,isPublic:this._I,memberCount:this._M,oneDrive:this._O,oneNote:this._On,ownerCount:this._Ow,planner:this._P,site:this._S,subscriptionEnabled:this._Su},{})},t.prototype.setMockData=function(e){v(this,e)},t.prototype.ensureUnchanged=function(e){l(this,e)},t}(OfficeExtension.ClientObject);e.UnifiedGroupInfo=oe;var se,ae="Persona";!function(e){e[e.immediate=0]="immediate",e[e.load=3]="load"}(se=e.PersonaPromiseType||(e.PersonaPromiseType={}));var le=function(){};e.PersonaInfoAndSource=le;var ue=function(t){function i(){return null!==t&&t.apply(this,arguments)||this}return __extends(i,t),Object.defineProperty(i.prototype,"_className",{get:function(){return"Persona"},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"_scalarPropertyNames",{get:function(){return["hostId","type","capabilities","diagnosticId","instanceId"]},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"_scalarPropertyOriginalNames",{get:function(){return["HostId","Type","Capabilities","DiagnosticId","InstanceId"]},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"_navigationPropertyNames",{get:function(){return["photo","personaInfo","unifiedCommunicationInfo","organization","unifiedGroupInfo","actions"]},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"actions",{get:function(){return this._A||(this._A=n(e.PersonaActions,this,"Actions",!1,4)),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"organization",{get:function(){return this._O||(this._O=n(e.PersonaOrganizationInfo,this,"Organization",!1,4)),this._O},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"personaInfo",{get:function(){return this._P||(this._P=n(e.PersonaInfo,this,"PersonaInfo",!1,4)),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"photo",{get:function(){return this._Ph||(this._Ph=n(e.PersonaPhotoInfo,this,"Photo",!1,4)),this._Ph},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"unifiedCommunicationInfo",{get:function(){return this._U||(this._U=n(e.PersonaUnifiedCommunicationInfo,this,"UnifiedCommunicationInfo",!1,4)),this._U},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"unifiedGroupInfo",{get:function(){return this._Un||(this._Un=n(e.UnifiedGroupInfo,this,"UnifiedGroupInfo",!1,4)),this._Un},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"capabilities",{get:function(){return f("capabilities",this._C,ae,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"diagnosticId",{get:function(){return f("diagnosticId",this._D,ae,this._isNull),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"hostId",{get:function(){return f("hostId",this._H,ae,this._isNull),this._H},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"instanceId",{get:function(){return f("instanceId",this._I,ae,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"type",{get:function(){return f("type",this._T,ae,this._isNull),this._T},enumerable:!0,configurable:!0}),i.prototype.set=function(e,t){this._recursivelySet(e,t,[],["unifiedGroupInfo"],["actions","organization","personaInfo","photo","unifiedCommunicationInfo"])},i.prototype.update=function(e){this._recursivelyUpdate(e)},i.prototype.dispose=function(){a(this,"Dispose",0,[],0,0)},i.prototype.getCustomizations=function(){return a(this,"GetCustomizations",1,[],4,0)},i.prototype.getMembers=function(){return r(e.MemberInfoList,this,"GetMembers",1,[],!1,!1,null,4)},i.prototype.getMembership=function(){return r(e.MemberInfoList,this,"GetMembership",1,[],!1,!1,null,4)},i.prototype.getViewableSources=function(){return a(this,"GetViewableSources",1,[],4,0)},i.prototype.reportTimeForRender=function(e,t){a(this,"ReportTimeForRender",0,[e,t],0,0)},i.prototype.warmup=function(e){a(this,"Warmup",0,[e],0,0)},i.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!c(e)){var n=e;g(this,n),p(n.Capabilities)||(this._C=n.Capabilities),p(n.DiagnosticId)||(this._D=n.DiagnosticId),p(n.HostId)||(this._H=n.HostId),p(n.InstanceId)||(this._I=n.InstanceId),p(n.Type)||(this._T=n.Type),_(this,n,["actions","Actions","organization","Organization","personaInfo","PersonaInfo","photo","Photo","unifiedCommunicationInfo","UnifiedCommunicationInfo","unifiedGroupInfo","UnifiedGroupInfo"])}},i.prototype.load=function(e){return d(this,e)},i.prototype.retrieve=function(e){return y(this,e)},i.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},i.prototype.toJSON=function(){return m(this,{capabilities:this._C,diagnosticId:this._D,hostId:this._H,instanceId:this._I,type:this._T},{organization:this._O,personaInfo:this._P,unifiedCommunicationInfo:this._U,unifiedGroupInfo:this._Un})},i.prototype.setMockData=function(e){v(this,e)},i.prototype.ensureUnchanged=function(e){l(this,e)},i}(OfficeExtension.ClientObject);e.Persona=ue;var ce=function(){function t(){}return t.prototype.performAsyncOperation=function(e,t,n,r){var i=this;e!=se.immediate?r().then((function(e){if(e)n();else{var o=i;o.load("hostId"),o.context.sync().then((function(){var e=o.hostId;i.getPersonaLifetime().then((function(i){var s=function(a){return new OfficeExtension.CoreUtility.Promise((function(l,u){if(a.sendingPersonaHostId==e)for(var c=0;c<a.dataUpdated.length;++c){var p=a.dataUpdated[c];if(t==p)return void r().then((function(e){e&&(n(),i.onPersonaUpdated.remove(s),o.context.sync()),l(e)}))}l(!1)}))};i.onPersonaUpdated.add(s),o.context.sync()}))}))}})):n()},t.prototype.getOrganizationAsync=function(e){var t=this;return new OfficeExtension.CoreUtility.Promise((function(n,r){var i=t;t.performAsyncOperation(e,G.organization,(function(){var e=i.organization;e.load("*"),i.context.sync().then((function(){n(e)}))}),(function(){return new OfficeExtension.CoreUtility.Promise((function(e,t){var n=i.organization;n.load("isWarmedUp"),i.context.sync().then((function(){e(n.isWarmedUp)}))}))}))}))},t.prototype.getIsPersonaInfoResolvedCheck=function(){var e=this;return new OfficeExtension.CoreUtility.Promise((function(t,n){var r=e.personaInfo;r.load("isPersonResolved"),e.context.sync().then((function(){t(r.isPersonResolved)}))}))},t.prototype.getPersonaInfoAsync=function(e){var t=this;return new OfficeExtension.CoreUtility.Promise((function(n,r){var i=t;t.performAsyncOperation(e,G.personaInfo,(function(){var e=i.personaInfo;e.load(),i.context.sync().then((function(){n(e)}))}),(function(){return t.getIsPersonaInfoResolvedCheck()}))}))},t.prototype.getPersonaInfoWithSourceAsync=function(e){var t=this;return new OfficeExtension.CoreUtility.Promise((function(n,r){var i=t;t.performAsyncOperation(e,G.personaInfo,(function(){var e=new le;e.info=i.personaInfo,e.info.load(),e.source=i.personaInfo.sources,e.source.load(),i.context.sync().then((function(){n(e)}))}),(function(){return t.getIsPersonaInfoResolvedCheck()}))}))},t.prototype.getUnifiedCommunicationInfo=function(e){var t=this;return new OfficeExtension.CoreUtility.Promise((function(n,r){var i=t;t.performAsyncOperation(e,G.personaInfo,(function(){var e=i.unifiedCommunicationInfo;e.load("*"),i.context.sync().then((function(){n(e)}))}),(function(){return t.getIsPersonaInfoResolvedCheck()}))}))},t.prototype.getUnifiedGroupInfoAsync=function(e){var t=this;return new OfficeExtension.CoreUtility.Promise((function(n,r){var i=t;t.performAsyncOperation(e,G.personaInfo,(function(){var e=i.unifiedGroupInfo;e.load("*"),i.context.sync().then((function(){n(e)}))}),(function(){return t.getIsPersonaInfoResolvedCheck()}))}))},t.prototype.getTypeAsync=function(t){var n=this;return new OfficeExtension.CoreUtility.Promise((function(r,i){var o=n;n.performAsyncOperation(t,G.personaInfo,(function(){o.load("type"),o.context.sync().then((function(){r(e.PersonaType[o.type.valueOf()])}))}),(function(){return n.getIsPersonaInfoResolvedCheck()}))}))},t.prototype.getCustomizationsAsync=function(e){var t=this;return new OfficeExtension.CoreUtility.Promise((function(n,r){var i=t;t.performAsyncOperation(e,G.personaInfo,(function(){var e=i.getCustomizations();i.context.sync().then((function(){n(e.value)}))}),(function(){return t.getIsPersonaInfoResolvedCheck()}))}))},t.prototype.getMembersAsync=function(e){var t=this;return new OfficeExtension.CoreUtility.Promise((function(n,r){var i=t;t.performAsyncOperation(e,G.members,(function(){var e=i.getMembers();e.load("isWarmedUp"),i.context.sync().then((function(){n(e)}))}),(function(){return new OfficeExtension.CoreUtility.Promise((function(e,t){var n=i.getMembers();n.load("isWarmedUp"),i.context.sync().then((function(){e(n.isWarmedUp)}))}))}))}))},t.prototype.getMembershipAsync=function(e){var t=this;return new OfficeExtension.CoreUtility.Promise((function(n,r){var i=t;t.performAsyncOperation(e,G.membership,(function(){var e=i.getMembership();e.load("*"),i.context.sync().then((function(){n(e)}))}),(function(){return new OfficeExtension.CoreUtility.Promise((function(e){var t=i.getMembership();t.load("isWarmedUp"),i.context.sync().then((function(){e(t.isWarmedUp)}))}))}))}))},t.prototype.getPersonaLifetime=function(){var e=this;return new OfficeExtension.CoreUtility.Promise((function(t,n){var r=e;r.load("instanceId"),r.context.sync().then((function(){new ye(r.context,r.instanceId).getPersonaLifetime().then((function(e){t(e)}))}))}))},t}();e.PersonaCustom=ce,OfficeExtension.Utility.applyMixin(ue,ce);var pe=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PersonaLifetime"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["instanceId"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["InstanceId"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"instanceId",{get:function(){return f("instanceId",this._I,"PersonaLifetime",this._isNull),this._I},enumerable:!0,configurable:!0}),n.prototype.getPersona=function(t){return r(e.Persona,this,"GetPersona",1,[t],!1,!1,null,4)},n.prototype.getPersonaForOrgByEntryId=function(t,n,i,o){return r(e.Persona,this,"GetPersonaForOrgByEntryId",1,[t,n,i,o],!1,!1,null,4)},n.prototype.getPersonaForOrgEntry=function(t,n,i,o){return r(e.Persona,this,"GetPersonaForOrgEntry",1,[t,n,i,o],!1,!1,null,4)},n.prototype.getPolicies=function(){return a(this,"GetPolicies",1,[],4,0)},n.prototype.getTextScaleFactor=function(){return a(this,"GetTextScaleFactor",1,[],4,0)},n.prototype._RegisterPersonaUpdatedEvent=function(){a(this,"_RegisterPersonaUpdatedEvent",0,[],0,0)},n.prototype._UnregisterPersonaUpdatedEvent=function(){a(this,"_UnregisterPersonaUpdatedEvent",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!c(e)){var n=e;g(this,n),p(n.InstanceId)||(this._I=n.InstanceId)}},n.prototype.load=function(e){return d(this,e)},n.prototype.retrieve=function(e){return y(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},Object.defineProperty(n.prototype,"onPersonaUpdated",{get:function(){var e=this;return this.m_personaUpdated||(this.m_personaUpdated=new OfficeExtension.GenericEventHandlers(this.context,this,"PersonaUpdated",{eventType:3502,registerFunc:function(){return e._RegisterPersonaUpdatedEvent()},unregisterFunc:function(){return e._UnregisterPersonaUpdatedEvent()},getTargetIdFunc:function(){return e.instanceId},eventArgsTransformFunc:function(e){var t={dataUpdated:e.dataUpdated,sendingPersonaHostId:e.sendingPersonaHostId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_personaUpdated},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return m(this,{instanceId:this._I},{})},n.prototype.setMockData=function(e){v(this,e)},n.prototype.ensureUnchanged=function(e){l(this,e)},n}(OfficeExtension.ClientObject);e.PersonaLifetime=pe;var fe="LokiTokenProvider",he=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"LokiTokenProvider"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["emailOrUpn","instanceId"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["EmailOrUpn","InstanceId"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"emailOrUpn",{get:function(){return f("emailOrUpn",this._E,fe,this._isNull),this._E},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"instanceId",{get:function(){return f("instanceId",this._I,fe,this._isNull),this._I},enumerable:!0,configurable:!0}),t.prototype.requestClientAccessToken=function(){a(this,"RequestClientAccessToken",0,[],0,0)},t.prototype.requestIdentityUniqueId=function(){a(this,"RequestIdentityUniqueId",0,[],0,0)},t.prototype.requestToken=function(){a(this,"RequestToken",0,[],0,0)},t.prototype._RegisterClientAccessTokenAvailableEvent=function(){a(this,"_RegisterClientAccessTokenAvailableEvent",0,[],0,0)},t.prototype._RegisterIdentityUniqueIdAvailableEvent=function(){a(this,"_RegisterIdentityUniqueIdAvailableEvent",0,[],0,0)},t.prototype._RegisterLokiTokenAvailableEvent=function(){a(this,"_RegisterLokiTokenAvailableEvent",0,[],0,0)},t.prototype._UnregisterClientAccessTokenAvailableEvent=function(){a(this,"_UnregisterClientAccessTokenAvailableEvent",0,[],0,0)},t.prototype._UnregisterIdentityUniqueIdAvailableEvent=function(){a(this,"_UnregisterIdentityUniqueIdAvailableEvent",0,[],0,0)},t.prototype._UnregisterLokiTokenAvailableEvent=function(){a(this,"_UnregisterLokiTokenAvailableEvent",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!c(t)){var n=t;g(this,n),p(n.EmailOrUpn)||(this._E=n.EmailOrUpn),p(n.InstanceId)||(this._I=n.InstanceId)}},t.prototype.load=function(e){return d(this,e)},t.prototype.retrieve=function(e){return y(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),O(this,t,n)},Object.defineProperty(t.prototype,"onClientAccessTokenAvailable",{get:function(){var e=this;return this.m_clientAccessTokenAvailable||(this.m_clientAccessTokenAvailable=new OfficeExtension.GenericEventHandlers(this.context,this,"ClientAccessTokenAvailable",{eventType:3505,registerFunc:function(){return e._RegisterClientAccessTokenAvailableEvent()},unregisterFunc:function(){return e._UnregisterClientAccessTokenAvailableEvent()},getTargetIdFunc:function(){return e.instanceId},eventArgsTransformFunc:function(e){var t={clientAccessToken:e.clientAccessToken,isAvailable:e.isAvailable,tokenTTLInSeconds:e.tokenTTLInSeconds};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_clientAccessTokenAvailable},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"onIdentityUniqueIdAvailable",{get:function(){var e=this;return this.m_identityUniqueIdAvailable||(this.m_identityUniqueIdAvailable=new OfficeExtension.GenericEventHandlers(this.context,this,"IdentityUniqueIdAvailable",{eventType:3504,registerFunc:function(){return e._RegisterIdentityUniqueIdAvailableEvent()},unregisterFunc:function(){return e._UnregisterIdentityUniqueIdAvailableEvent()},getTargetIdFunc:function(){return e.instanceId},eventArgsTransformFunc:function(e){var t={isAvailable:e.isAvailable,uniqueId:e.uniqueId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_identityUniqueIdAvailable},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"onLokiTokenAvailable",{get:function(){var e=this;return this.m_lokiTokenAvailable||(this.m_lokiTokenAvailable=new OfficeExtension.GenericEventHandlers(this.context,this,"LokiTokenAvailable",{eventType:3503,registerFunc:function(){return e._RegisterLokiTokenAvailableEvent()},unregisterFunc:function(){return e._UnregisterLokiTokenAvailableEvent()},getTargetIdFunc:function(){return e.instanceId},eventArgsTransformFunc:function(e){var t={isAvailable:e.isAvailable,lokiAutoDiscoverUrl:e.lokiAutoDiscoverUrl,lokiToken:e.lokiToken};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_lokiTokenAvailable},enumerable:!0,configurable:!0}),t.prototype.toJSON=function(){return m(this,{emailOrUpn:this._E,instanceId:this._I},{})},t.prototype.setMockData=function(e){v(this,e)},t.prototype.ensureUnchanged=function(e){l(this,e)},t}(OfficeExtension.ClientObject);e.LokiTokenProvider=he;var de=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"LokiTokenProviderFactory"},enumerable:!0,configurable:!0}),n.prototype.getLokiTokenProvider=function(t){return r(e.LokiTokenProvider,this,"GetLokiTokenProvider",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.LokiTokenProviderFactory,t,"Microsoft.People.LokiTokenProviderFactory",!1,4)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.LokiTokenProviderFactory=de;var ye=function(){function t(e,t){this.context=e,this.instanceId=t}return Object.defineProperty(t.prototype,"serviceContext",{get:function(){return this.m_serviceConext||(this.m_serviceConext=e.ServiceContext.newObject(this.context)),this.m_serviceConext},enumerable:!0,configurable:!0}),t.prototype.getPersonaLifetime=function(){var e=this;return new OfficeExtension.CoreUtility.Promise((function(t,n){var r=e.serviceContext.getPersonaLifetime(e.instanceId);e.context.sync().then((function(){r.load("instanceId"),e.context.sync().then((function(){t(r)}))}))}))},t.prototype.getInitialPersona=function(){var e=this;return new OfficeExtension.CoreUtility.Promise((function(t,n){var r=e.serviceContext.getInitialPersona(e.instanceId);e.context.sync().then((function(){t(r)}))}))},t.prototype.getLokiTokenProvider=function(){var e=this;return new OfficeExtension.CoreUtility.Promise((function(t,n){var r=e.serviceContext.getLokiTokenProvider(e.instanceId);e.context.sync().then((function(){r.load("instanceId"),e.context.sync().then((function(){t(r)}))}))}))},t}();e.PeopleApiContext=ye;var me=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ServiceContext"},enumerable:!0,configurable:!0}),n.prototype.accountEmailOrUpn=function(e){return a(this,"AccountEmailOrUpn",1,[e],4,0)},n.prototype.dispose=function(e){a(this,"Dispose",0,[e],0,0)},n.prototype.getInitialPersona=function(t){return r(e.Persona,this,"GetInitialPersona",1,[t],!1,!1,null,4)},n.prototype.getLokiTokenProvider=function(t){return r(e.LokiTokenProvider,this,"GetLokiTokenProvider",1,[t],!1,!1,null,4)},n.prototype.getPersonaLifetime=function(t){return r(e.PersonaLifetime,this,"GetPersonaLifetime",1,[t],!1,!1,null,4)},n.prototype.getPersonaPolicies=function(){return a(this,"GetPersonaPolicies",1,[],4,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.ServiceContext,t,"Microsoft.People.ServiceContext",!1,4)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.ServiceContext=me;var ge=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"RichapiPcxFeatureChecks"},enumerable:!0,configurable:!0}),n.prototype.isAddChangePhotoLinkOnLpcPersonaImageFlightEnabled=function(){return a(this,"IsAddChangePhotoLinkOnLpcPersonaImageFlightEnabled",1,[],4,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.RichapiPcxFeatureChecks,t,"Microsoft.People.RichapiPcxFeatureChecks",!1,4)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.RichapiPcxFeatureChecks=ge;var _e=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Tap"},enumerable:!0,configurable:!0}),n.prototype.getEnterpriseUserInfo=function(){return a(this,"GetEnterpriseUserInfo",1,[],5,0)},n.prototype.getMruFriendlyPath=function(e){return a(this,"GetMruFriendlyPath",1,[e],5,0)},n.prototype.launchFileUrlInOfficeApp=function(e,t){return a(this,"LaunchFileUrlInOfficeApp",1,[e,t],5,0)},n.prototype.performLocalSearch=function(e,t,n,r){return a(this,"PerformLocalSearch",1,[e,t,n,r],5,0)},n.prototype.readSearchCache=function(e,t,n){return a(this,"ReadSearchCache",1,[e,t,n],5,0)},n.prototype.writeSearchCache=function(e,t,n){return a(this,"WriteSearchCache",1,[e,t,n],5,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.Tap,t,"Microsoft.TapRichApi.Tap",!1,4)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.Tap=_e,function(e){e.unknown="Unknown",e.chart="Chart",e.smartArt="SmartArt",e.table="Table",e.image="Image",e.slide="Slide",e.ole="OLE",e.text="Text"}(e.ObjectType||(e.ObjectType={}));var be=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"AppRuntimePersistenceService"},enumerable:!0,configurable:!0}),n.prototype.getAppRuntimeStartState=function(){return a(this,"GetAppRuntimeStartState",1,[],4,0)},n.prototype.setAppRuntimeStartState=function(e){a(this,"SetAppRuntimeStartState",0,[e],S(2,"SharedRuntimeInternal","1.2"),0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.AppRuntimePersistenceService,t,"Microsoft.AppRuntime.AppRuntimePersistenceService",!1,4)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.AppRuntimePersistenceService=be;var Oe,ve=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"AppRuntimeService"},enumerable:!0,configurable:!0}),n.prototype.getAppRuntimeState=function(){return a(this,"GetAppRuntimeState",1,[],4,0)},n.prototype.setAppRuntimeState=function(e){a(this,"SetAppRuntimeState",0,[e],S(2,"SharedRuntimeInternal","1.2"),0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.AppRuntimeService,t,"Microsoft.AppRuntime.AppRuntimeService",!1,4)},Object.defineProperty(n.prototype,"onVisibilityChanged",{get:function(){return this.m_visibilityChanged||(this.m_visibilityChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"VisibilityChanged",{eventType:65539,registerFunc:function(){return OfficeExtension.Utility._createPromiseFromResult(null)},unregisterFunc:function(){return OfficeExtension.Utility._createPromiseFromResult(null)},getTargetIdFunc:function(){return""},eventArgsTransformFunc:function(e){var t={visibility:e.visibility};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_visibilityChanged},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.AppRuntimeService=ve,function(e){e.inactive="Inactive",e.background="Background",e.visible="Visible"}(e.AppRuntimeState||(e.AppRuntimeState={})),function(e){e.hidden="Hidden",e.visible="Visible"}(e.Visibility||(e.Visibility={})),function(e){e.unknown="Unknown",e.basic="Basic",e.premium="Premium"}(Oe=e.LicenseFeatureTier||(e.LicenseFeatureTier={}));var Se=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"License"},enumerable:!0,configurable:!0}),n.prototype.getFeatureTier=function(e,t){return a(this,"GetFeatureTier",1,[e,t],4,0)},n.prototype.getLicenseFeature=function(t){return r(e.LicenseFeature,this,"GetLicenseFeature",1,[t],!1,!1,null,4)},n.prototype.getMsaDeviceTicket=function(e,t,n){return a(this,"GetMsaDeviceTicket",1,[e,t,n],5,0)},n.prototype.isFeatureEnabled=function(e,t){return a(this,"IsFeatureEnabled",1,[e,t],4,0)},n.prototype.isFreemiumUpsellEnabled=function(){return a(this,"IsFreemiumUpsellEnabled",1,[],4,0)},n.prototype.launchUpsellExperience=function(e){a(this,"LaunchUpsellExperience",1,[e],4,0)},n.prototype._TestFireStateChangedEvent=function(e){a(this,"_TestFireStateChangedEvent",0,[e],1,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.License,t,"Microsoft.Office.Licensing.License",!1,4)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.License=Se;var Ce=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"LicenseFeature"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["id"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"id",{get:function(){return f("id",this._I,"LicenseFeature",this._isNull),this._I},enumerable:!0,configurable:!0}),t.prototype._RegisterStateChange=function(){a(this,"_RegisterStateChange",1,[],4,0)},t.prototype._UnregisterStateChange=function(){a(this,"_UnregisterStateChange",1,[],4,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!c(t)){var n=t;g(this,n),p(n.Id)||(this._I=n.Id)}},t.prototype.load=function(e){return d(this,e)},t.prototype.retrieve=function(e){return y(this,e)},t.prototype._handleIdResult=function(t){e.prototype._handleIdResult.call(this,t),c(t)||p(t.Id)||(this._I=t.Id)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),O(this,t,n)},Object.defineProperty(t.prototype,"onStateChanged",{get:function(){var e=this;return this.m_stateChanged||(this.m_stateChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"StateChanged",{eventType:1,registerFunc:function(){return e._RegisterStateChange()},unregisterFunc:function(){return e._UnregisterStateChange()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(t){var n=P.LicenseFeature_StateChanged_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_stateChanged},enumerable:!0,configurable:!0}),t.prototype.toJSON=function(){return m(this,{id:this._I},{})},t.prototype.setMockData=function(e){v(this,e)},t.prototype.ensureUnchanged=function(e){l(this,e)},t}(OfficeExtension.ClientObject);e.LicenseFeature=Ce,function(e){e.LicenseFeature_StateChanged_EventArgsTransform=function(e,t){var n={feature:t.featureName,isEnabled:t.isEnabled,tier:t.tierName};return t.tierName&&(n.tier=0==t.tierName?Oe.unknown:1==t.tierName?Oe.basic:2==t.tierName?Oe.premium:t.tierName),n}}(P=e._CC||(e._CC={}));var Pe=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"MsaDeviceTicketOptions"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["scopes"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Scopes"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"scopes",{get:function(){return f("scopes",this._S,"MsaDeviceTicketOptions",this._isNull),this._S},set:function(e){this._S=e,u(this,"Scopes",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["scopes"],[],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!c(e)){var n=e;g(this,n),p(n.Scopes)||(this._S=n.Scopes)}},n.prototype.load=function(e){return d(this,e)},n.prototype.retrieve=function(e){return y(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.MsaDeviceTicketOptions,t,"Microsoft.Office.Licensing.MsaDeviceTicketOptions",!1,4)},n.prototype.toJSON=function(){return m(this,{scopes:this._S},{})},n.prototype.setMockData=function(e){v(this,e)},n.prototype.ensureUnchanged=function(e){l(this,e)},n}(OfficeExtension.ClientObject);e.MsaDeviceTicketOptions=Pe;var Ae=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"DialogPage"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_Id",{get:function(){return f("_Id",this.__I,"DialogPage",this._isNull),this.__I},enumerable:!0,configurable:!0}),n.prototype.close=function(){a(this,"Close",1,[],4,0)},n.prototype.readyToShow=function(){a(this,"ReadyToShow",1,[],4,0)},n.prototype.registerOnShow=function(){a(this,"RegisterOnShow",1,[],4,0)},n.prototype.sendMessageToHost=function(e){a(this,"SendMessageToHost",1,[e],4,0)},n.prototype.unregisterOnShow=function(){a(this,"UnregisterOnShow",1,[],4,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!c(e)){var n=e;g(this,n),p(n._Id)||(this.__I=n._Id)}},n.prototype.load=function(e){return d(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),c(e)||p(e._Id)||(this.__I=e._Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.DialogPage,t,"Microsoft.Office.DialogPage.DialogPage",!1,4)},Object.defineProperty(n.prototype,"onOnShowEvent",{get:function(){var e=this;return this.m_onShowEvent||(this.m_onShowEvent=new OfficeExtension.GenericEventHandlers(this.context,this,"OnShowEvent",{eventType:1,registerFunc:function(){return e.registerOnShow()},unregisterFunc:function(){return e.unregisterOnShow()},getTargetIdFunc:function(){return e._Id},eventArgsTransformFunc:function(e){return OfficeExtension.Utility._createPromiseFromResult({})}})),this.m_onShowEvent},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.DialogPage=Ae;var Fe=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ActionService"},enumerable:!0,configurable:!0}),n.prototype.areShortcutsInUse=function(e){return a(this,"AreShortcutsInUse",0,[e],0,0)},n.prototype.getShortcuts=function(){return a(this,"GetShortcuts",1,[],4,0)},n.prototype.replaceShortcuts=function(e){a(this,"ReplaceShortcuts",0,[e],0,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),c(e))||g(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),O(this,e,n)},n.newObject=function(t){return o(e.ActionService,t,"Microsoft.Office.ActionService",!1,4)},n.prototype.toJSON=function(){return m(this,{},{})},n}(OfficeExtension.ClientObject);e.ActionService=Fe,function(e){e.apiNotAvailable="ApiNotAvailable",e.clientError="ClientError",e.controlIdNotFound="ControlIdNotFound",e.entryIdRequired="EntryIdRequired",e.generalException="GeneralException",e.hostRestartNeeded="HostRestartNeeded",e.instanceNotFound="InstanceNotFound",e.interactiveFlowAborted="InteractiveFlowAborted",e.invalidArgument="InvalidArgument",e.invalidGrant="InvalidGrant",e.invalidResourceUrl="InvalidResourceUrl",e.invalidRibbonDefinition="InvalidRibbonDefinition",e.objectNotFound="ObjectNotFound",e.resourceNotSupported="ResourceNotSupported",e.serverError="ServerError",e.serviceUrlNotFound="ServiceUrlNotFound",e.sharedRuntimeNotAvailable="SharedRuntimeNotAvailable",e.ticketInvalidParams="TicketInvalidParams",e.ticketNetworkError="TicketNetworkError",e.ticketUnauthorized="TicketUnauthorized",e.ticketUninitialized="TicketUninitialized",e.ticketUnknownError="TicketUnknownError",e.unexpectedError="UnexpectedError",e.unsupportedUserIdentity="UnsupportedUserIdentity",e.userNotSignedIn="UserNotSignedIn"}(e.ErrorCodes||(e.ErrorCodes={})),e.Interfaces||(e.Interfaces={})}(OfficeCore||(OfficeCore={})),function(e){var t,n;!function(e){e.hidden="Hidden",e.taskpane="Taskpane"}(t=e.VisibilityMode||(e.VisibilityMode={})),function(e){e.none="None",e.load="Load"}(n=e.StartupBehavior||(e.StartupBehavior={})),function(e){function r(e){var t=new OfficeCore.RequestContext;return t._requestFlagModifier|=64,e&&(t._customData="WacPartition"),t}function i(e){return __awaiter(this,void 0,void 0,(function(){var t;return __generator(this,(function(n){switch(n.label){case 0:return t=r(!0),OfficeCore.AppRuntimeService.newObject(t).setAppRuntimeState(e),[4,t.sync()];case 1:return n.sent(),[2]}}))}))}var o;e.setStartupBehavior=function(e){return __awaiter(this,void 0,void 0,(function(){var t,i;return __generator(this,(function(o){switch(o.label){case 0:if(e!==n.load&&e!==n.none)throw OfficeExtension.Utility.createRuntimeError(OfficeExtension.ErrorCodes.invalidArgument,null,null);return t=e==n.load?OfficeCore.AppRuntimeState.background:OfficeCore.AppRuntimeState.inactive,i=r(!1),OfficeCore.AppRuntimePersistenceService.newObject(i).setAppRuntimeStartState(t),[4,i.sync()];case 1:return o.sent(),[2]}}))}))},e.getStartupBehavior=function(){return __awaiter(this,void 0,void 0,(function(){var e,t,i,o;return __generator(this,(function(s){switch(s.label){case 0:return e=r(!1),t=OfficeCore.AppRuntimePersistenceService.newObject(e),i=t.getAppRuntimeStartState(),[4,e.sync()];case 1:return s.sent(),o=i.value,[2,o==OfficeCore.AppRuntimeState.inactive?n.none:n.load]}}))}))},e._getState=function(){return __awaiter(this,void 0,void 0,(function(){var e,t,n;return __generator(this,(function(i){switch(i.label){case 0:return e=r(!0),t=OfficeCore.AppRuntimeService.newObject(e),n=t.getAppRuntimeState(),[4,e.sync()];case 1:return i.sent(),[2,n.value]}}))}))},e.showAsTaskpane=function(){return i(OfficeCore.AppRuntimeState.visible)},e.hide=function(){return i(OfficeCore.AppRuntimeState.background)},e.onVisibilityModeChanged=function(e){return __awaiter(this,void 0,void 0,(function(){var n,i,s=this;return __generator(this,(function(a){switch(a.label){case 0:return n=function(){if(!o){var e=r(!0);o=OfficeCore.AppRuntimeService.newObject(e)}return o}(),i=n.onVisibilityChanged.add((function(n){if(e){var r={visibilityMode:(i=n.visibility,i===OfficeCore.Visibility.visible?t.taskpane:t.hidden)};e(r)}var i;return null})),[4,n.context.sync()];case 1:return a.sent(),[2,function(){return __awaiter(s,void 0,void 0,(function(){return __generator(this,(function(e){switch(e.label){case 0:return i.remove(),[4,n.context.sync()];case 1:return e.sent(),[2]}}))}))}]}}))}))},function(e){function t(e){var t=new OfficeCore.RequestContext;return t._requestFlagModifier|=64,e&&(t._customData="WacPartition"),t}e.enable=function(){return __awaiter(this,void 0,void 0,(function(){var e,n;return __generator(this,(function(r){switch(r.label){case 0:return e=!1,"web"==OSF._OfficeAppFactory.getHostInfo().hostPlatform&&(e=!0),n=t(e),OfficeCore.BeforeDocumentCloseNotification.newObject(n).enable(),[4,n.sync()];case 1:return r.sent(),[2]}}))}))},e.disable=function(){return __awaiter(this,void 0,void 0,(function(){var e,n;return __generator(this,(function(r){switch(r.label){case 0:return e=!1,"web"==OSF._OfficeAppFactory.getHostInfo().hostPlatform&&(e=!0),n=t(e),OfficeCore.BeforeDocumentCloseNotification.newObject(n).disable(),[4,n.sync()];case 1:return r.sent(),[2]}}))}))},e.onCloseActionCancelled=function(e){return __awaiter(this,void 0,void 0,(function(){var n,r,i,o,s=this;return __generator(this,(function(a){switch(a.label){case 0:return n=!1,"web"==OSF._OfficeAppFactory.getHostInfo().hostPlatform&&(n=!0),r=t(n),i=OfficeCore.BeforeDocumentCloseNotification.newObject(r),o=i.onBeforeCloseNotificationCancelled.add((function(t){return e&&e(),null})),[4,r.sync()];case 1:return a.sent(),[2,function(){return __awaiter(s,void 0,void 0,(function(){return __generator(this,(function(e){switch(e.label){case 0:return o.remove(),[4,r.sync()];case 1:return e.sent(),[2]}}))}))}]}}))}))}}(e.beforeDocumentCloseNotification||(e.beforeDocumentCloseNotification={}))}(e.addin||(e.addin={}))}(Office||(Office={})),function(e){var t;!function(e){e.taskpane="Taskpane"}(t||(t={})),function(e){e.launchTaskpane=function(e){return __awaiter(this,void 0,void 0,(function(){var n,r,i;return __generator(this,(function(o){switch(o.label){case 0:return n=new OfficeCore.RequestContext,r=OfficeCore.ExtensionLifeCycle.newObject(n),null!=(i=e.settings)&&(e.settings=OSF.OUtil.serializeSettings(i)),r.launchExtensionComponent("",t.taskpane,e),[4,n.sync()];case 1:return o.sent(),[2]}}))}))}}(e.extensionLifeCycle||(e.extensionLifeCycle={}))}(Office||(Office={})),function(e){!function(e){function t(){var e=new OfficeCore.RequestContext;return"web"==OSF._OfficeAppFactory.getHostInfo().hostPlatform&&(e._customData="WacPartition"),e}e.requestUpdate=function(e){var n=t(),r=n.ribbon;function i(e){void 0!==e.controls&&void 0!==e.controls.length&&e.controls.length&&e.controls.filter((function(e){return!!e.id})).forEach((function(e){var t=r.getButton(e.id);void 0!==e.enabled&&null!==e.enabled&&(t.enabled=e.enabled)}))}return e.tabs.filter((function(e){return!!e.id})).forEach((function(e){var t=r.getTab(e.id);void 0!==e.visible&&null!==e.visible&&t.setVisibility(e.visible),e.groups&&e.groups.length?e.groups.filter((function(e){return!!e.id})).forEach((function(e){i(e)})):i(e)})),n.sync()},e.requestCreateControls=function(e){var n,r=t();return r.ribbon.executeRequestCreate(JSON.stringify(e)),(n=250,new Promise((function(e,t){return setTimeout((function(){return e()}),n)}))).then((function(){return r.sync()}))}}(e.ribbon||(e.ribbon={}))}(Office||(Office={})),function(e){var t;OfficeExtension.BatchApiHelper.createPropertyObject,OfficeExtension.BatchApiHelper.createMethodObject,OfficeExtension.BatchApiHelper.createIndexerObject,OfficeExtension.BatchApiHelper.createRootServiceObject,OfficeExtension.BatchApiHelper.createTopLevelServiceObject,OfficeExtension.BatchApiHelper.createChildItemObject,OfficeExtension.BatchApiHelper.invokeMethod,OfficeExtension.BatchApiHelper.invokeEnsureUnchanged,OfficeExtension.BatchApiHelper.invokeSetProperty,OfficeExtension.Utility.isNullOrUndefined,OfficeExtension.Utility.isUndefined,OfficeExtension.Utility.throwIfNotLoaded,OfficeExtension.Utility.throwIfApiNotSupported,OfficeExtension.Utility.load,OfficeExtension.Utility.retrieve,OfficeExtension.Utility.toJson,OfficeExtension.Utility.fixObjectPathIfNecessary,OfficeExtension.Utility._handleNavigationPropertyResults,OfficeExtension.Utility.adjustToDateTime,OfficeExtension.Utility.processRetrieveResult,OfficeExtension.Utility.setMockData,OfficeExtension.CommonUtility.calculateApiFlags;!function(e){e.generalException="GeneralException"}(t||(t={}));new OfficeExtension.LibraryBuilder({metadata:{version:"1.0.0",name:"OfficeCore",defaultApiSetName:"OfficeSharedApi",hostName:"Office",apiSets:[],strings:["AddinInternalService"],enumTypes:[],clientObjectTypes:[[1,0,0,0,[["notifyActionHandlerReady",0,2,0,4]],0,0,0,0,"Microsoft.InternalService.AddinInternalService",4]]},targetNamespaceObject:e})}(OfficeCore||(OfficeCore={})),function(e){var t;!function(t){var n,r,i=OfficeExtension.Utility.isNullOrUndefined;function o(e){if(e){var t=e.toUpperCase(),r=n.mappings[t];if(!i(r)&&"function"==typeof r)return r}throw OfficeExtension.Utility.createRuntimeError("invalidOperation","sourceData","ActionProxy._getFunction")}function s(e){try{OfficeExtension.Utility.log("ActionProxy._handleMessage"),OfficeExtension.Utility.checkArgumentNull(e,"args");for(var t=e.entries,n=0;n<t.length;n++)if(2===t[n].messageCategory)if("string"==typeof t[n].message&&(t[n].message=JSON.parse(t[n].message)),1e3===t[n].messageType){var r=null,i=o(t[n].message[0]);if(t[n].message.length>=2){var s=t[n].message[1];s&&(r=a(s)?JSON.parse(s):s)}void 0!==OSF&&OSF.AppTelemetry&&OSF.AppTelemetry.CallOnAppActivatedIfPending&&OSF.AppTelemetry.CallOnAppActivatedIfPending(),i.apply(null,[r])}else OfficeExtension.Utility.log("ActionProxy._handleMessage unknown message type "+t[n].messageType)}catch(e){throw function(e){var t=function(e){var t="Unknown Error";if(e)try{e.toString&&(t=e.toString()),t=t+" "+JSON.stringify(e)}catch(e){t="Unexpected Error"}return t}(e);OfficeExtension.Utility.log(t)}(e),e}return OfficeExtension.Utility._createPromiseFromResult(null)}function a(e){return"string"==typeof e&&"{"===e[0]}function l(){try{Microsoft.Office.WebExtension.onReadyInternal().then((function(){return function(){if(void 0!==OSF&&OSF.DDA&&OSF.DDA.RichApi&&OSF.DDA.RichApi.richApiMessageManager)return(new OfficeExtension.ClientRequestContext).eventRegistration.register(5,"",s)}()})).then((function(){var e,t,n=OSF._OfficeAppFactory.getHostInfo();return!0===n.isDialog||"web"===n.hostPlatform&&"word"!==n.hostType&&"excel"!==n.hostType?void 0:(e=new OfficeExtension.ClientRequestContext,t=OfficeCore.AddinInternalService.newObject(e),e._customData="WacPartition",t.notifyActionHandlerReady(),e.sync())}))}catch(e){}}OfficeExtension.Utility.log("ActionProxy.initOnce"),void 0!==e.actions&&(r=e.actions._association,n=r),"undefined"!=typeof document&&(document.readyState&&"loading"!==document.readyState?(OfficeExtension.Utility.log("ActionProxy.initOnce: document.readyState is not loading state"),l()):document.addEventListener&&document.addEventListener("DOMContentLoaded",(function(){OfficeExtension.Utility.log("ActionProxy.initOnce: DOMContentLoaded event triggered"),l()}))),OfficeExtension.HostBridge.onInited((function(e){!function(e){e.addHostMessageHandler((function(e){3===e.type&&s(e.message)}))}(e)}))}(t||(t={}))}(Office||(Office={})),function(e){!function(e){function t(){var e=new OfficeCore.RequestContext;return"web"==OSF._OfficeAppFactory.getHostInfo().hostPlatform&&(e._customData="WacPartition"),e}e.areShortcutsInUse=function(e){return __awaiter(this,void 0,void 0,(function(){var n,r,i,o,s;return __generator(this,(function(a){switch(a.label){case 0:return n=t(),r=OfficeCore.ActionService.newObject(n),i=r.areShortcutsInUse(e),[4,n.sync()];case 1:for(a.sent(),o=[],s=0;s<e.length;s++)o.push({shortcut:e[s],inUse:i.value[s]});return[2,o]}}))}))},e.replaceShortcuts=function(e){return __awaiter(this,void 0,void 0,(function(){var n;return __generator(this,(function(r){switch(r.label){case 0:return n=t(),OfficeCore.ActionService.newObject(n).replaceShortcuts(e),[4,n.sync()];case 1:return r.sent(),[2]}}))}))},e.getShortcuts=function(){return __awaiter(this,void 0,void 0,(function(){var e,n,r;return __generator(this,(function(i){switch(i.label){case 0:return e=t(),n=OfficeCore.ActionService.newObject(e),r=n.getShortcuts(),[4,e.sync()];case 1:return i.sent(),[2,r.value]}}))}))}}(e.actions||(e.actions={}))}(Office||(Office={})),function(e){var t;(t=e.dialogPage||(e.dialogPage={})).close=function(){return __awaiter(this,void 0,void 0,(function(){var e;return __generator(this,(function(t){switch(t.label){case 0:return e=new OfficeCore.RequestContext,OfficeCore.DialogPage.newObject(e).close(),[4,e.sync()];case 1:return t.sent(),[2]}}))}))},t.readyToShow=function(){return __awaiter(this,void 0,void 0,(function(){var e;return __generator(this,(function(t){switch(t.label){case 0:return e=new OfficeCore.RequestContext,OfficeCore.DialogPage.newObject(e).readyToShow(),[4,e.sync()];case 1:return t.sent(),[2]}}))}))},t.onShow=function(e){return __awaiter(this,void 0,void 0,(function(){var t,n,r;return __generator(this,(function(i){switch(i.label){case 0:return t=new OfficeCore.RequestContext,(n=OfficeCore.DialogPage.newObject(t)).onOnShowEvent.add(e),r=function(){return n.onOnShowEvent.remove(e),null},[4,t.sync()];case 1:return i.sent(),[2,r]}}))}))},t.sendMessageToHost=function(e){return __awaiter(this,void 0,void 0,(function(){var t;return __generator(this,(function(n){switch(n.label){case 0:return t=new OfficeCore.RequestContext,OfficeCore.DialogPage.newObject(t).sendMessageToHost(e),[4,t.sync()];case 1:return n.sent(),[2]}}))}))}}(Office||(Office={}));__extends=this&&this.__extends||function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}();var Excel,__assign=this&&this.__assign||function(){return(__assign=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},_BeginExcel="_BeginExcel";!function(e){function t(e,t){var r=f("OfficeOnline")?new n:new d;return y(e,r),OfficeExtension.CoreUtility.Promise.resolve().then((function(){return t(r)})).then(r.sync)}var n=function(e){function t(t){var n=e.call(this,t)||this;return n._customData="WacPartition",n.m_wacWorkbook=A(r,n),n._rootObject=n.m_wacWorkbook,n._rootObjectPropertyName="wacWorkbook",n}return __extends(t,e),Object.defineProperty(t.prototype,"wacWorkbook",{get:function(){return this.m_wacWorkbook},enumerable:!0,configurable:!0}),t}(OfficeCore.RequestContext),r=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),t.prototype.createAndOpenWorkbook=function(e){E(this,"CreateAndOpenWorkbook",0,[e],0,0)},t}(OfficeExtension.ClientObject);function i(e){return e[0].toLowerCase()+e.slice(1)}e.createWorkbook=function(e){return w("Excel.createWorkbook",v,"1.8",O),t({delayForCellEdit:!0},(function(t){t instanceof n?t.wacWorkbook.createAndOpenWorkbook(e):t.workbook.application.createWorkbook(e).open()}))};var o,s,a,l=[["RedDownArrow","YellowSideArrow","GreenUpArrow"],["GrayDownArrow","GraySideArrow","GrayUpArrow"],["RedFlag","YellowFlag","GreenFlag"],["RedCircleWithBorder","YellowCircle","GreenCircle"],["RedTrafficLight","YellowTrafficLight","GreenTrafficLight"],["RedDiamond","YellowTriangle","GreenCircle"],["RedCrossSymbol","YellowExclamationSymbol","GreenCheckSymbol"],["RedCross","YellowExclamation","GreenCheck"],["RedDownArrow","YellowDownInclineArrow","YellowUpInclineArrow","GreenUpArrow"],["GrayDownArrow","GrayDownInclineArrow","GrayUpInclineArrow","GrayUpArrow"],["BlackCircle","GrayCircle","PinkCircle","RedCircle"],["OneBar","TwoBars","ThreeBars","FourBars"],["BlackCircleWithBorder","RedCircleWithBorder","YellowCircle","GreenCircle"],["RedDownArrow","YellowDownInclineArrow","YellowSideArrow","YellowUpInclineArrow","GreenUpArrow"],["GrayDownArrow","GrayDownInclineArrow","GraySideArrow","GrayUpInclineArrow","GrayUpArrow"],["NoBars","OneBar","TwoBars","ThreeBars","FourBars"],["WhiteCircleAllWhiteQuarters","CircleWithThreeWhiteQuarters","CircleWithTwoWhiteQuarters","CircleWithOneWhiteQuarter","BlackCircle"],["SilverStar","HalfGoldStar","GoldStar"],["RedDownTriangle","YellowDash","GreenUpTriangle"],["NoFilledBoxes","OneFilledBox","TwoFilledBoxes","ThreeFilledBoxes","FourFilledBoxes"]];function u(e,t,n,r){if(Array.isArray(n)&&n.length>0&&Array.isArray(n[0])&&n.length*n[0].length>1500&&p()){var i=Math.max(1,Math.round(1500/n[0].length));e._ValidateArraySize(n.length,n[0].length);for(var o=0;o<n.length;o+=i){var s=i;o+s>n.length&&(s=n.length-o);var a=e.getRow(o).untrack().getBoundingRect(e.getRow(o+s-1).untrack()).untrack(),l=n.slice(o,o+s);I(a,t,l,null!=r?r:0)}return!0}return!1}function c(e){return OfficeExtension.Utility.isSetSupported("ExcelApi",e.toString())}function p(){return c(1.3)}function f(e){return!("undefined"==typeof window||!window.Office||!window.Office.context)&&window.Office.context.platform===e}e.icons={},["ThreeArrows","ThreeArrowsGray","ThreeFlags","ThreeTrafficLights1","ThreeTrafficLights2","ThreeSigns","ThreeSymbols","ThreeSymbols2","FourArrows","FourArrowsGray","FourRedToBlack","FourRating","FourTrafficLights","FiveArrows","FiveArrowsGray","FiveRating","FiveQuarters","ThreeStars","ThreeTriangles","FiveBoxes"].map((function(t,n){var r=i(t);e.icons[r]=[],l[n].map((function(n,o){n=i(n);var s={set:t,index:o};e.icons[r].push(s),e.icons[r][n]=s}))})),function(e){e.NotStarted="notStarted",e.Running="running",e.Succeeded="succeeded",e.Failed="failed"}(o||(o={})),function(e){e[e.OK=200]="OK",e[e.Created=201]="Created",e[e.Accepted=202]="Accepted",e[e.NoContent=204]="NoContent",e[e.HighestSuccessCode=299]="HighestSuccessCode",e[e.TooManyRequests=429]="TooManyRequests",e[e.InternalServerError=500]="InternalServerError",e[e.ServiceUnavailable=503]="ServiceUnavailable",e[e.GatewayTimeout=504]="GatewayTimeout"}(s||(s={})),function(e){e.Close="Session.close",e.CommitChanges="Session.commitChanges",e.Create="Session.resolveRequestUrlAndHeaderInfo",e.Refresh="Session.refreshSession"}(a=e.SessionOperation||(e.SessionOperation={}));var h=function(){function e(e,t,n){var r=void 0===n?{}:n,i=r.persistChanges,o=void 0===i||i,s=r.commitExplicitly,a=void 0===s||s;this.m_requestId="",this.m_workbookUrl=e?this.ensureUrlFormatEndWithSlash(e):"",this.m_requestHeaders=t||{},this.m_persistChanges=o,this.m_commitExplicitly=a}return Object.defineProperty(e.prototype,"requestId",{get:function(){return this.m_requestId},enumerable:!0,configurable:!0}),e.prototype.close=function(){var t=this;if(this.m_requestUrlAndHeaderInfo&&!OfficeExtension.Utility._isLocalDocumentUrl(this.m_requestUrlAndHeaderInfo.url)){var n={method:"POST",url:this.ensureUrlFormatEndWithSlash(this.m_requestUrlAndHeaderInfo.url)+e.CLOSE_SESSION,headers:this.m_requestUrlAndHeaderInfo.headers,body:""};return OfficeExtension.HttpUtility.sendRequest(n).then((function(n){if(n.statusCode!==s.NoContent)throw t.createErrorFromResponseInfo(n,a.Close);for(var r in t.m_requestUrlAndHeaderInfo=null,t.m_requestHeaders)if(r.toLowerCase()===e.WorkbookSessionIdHeaderNameLower){delete t.m_requestHeaders[r];break}}))}return OfficeExtension.Utility._createPromiseFromResult(null)},e.prototype.commitChanges=function(t){var n=this;if(void 0===t&&(t={}),!this.m_commitExplicitly)throw this.createError(s.InternalServerError,"Can not call commitChanges() if commitExplicitly is not set.",a.CommitChanges);if(!this.m_workbookUrl||OfficeExtension.Utility._isLocalDocumentUrl(this.m_workbookUrl))throw this.createError(s.InternalServerError,"Not supported for local documents.",a.CommitChanges);if(!this.m_requestUrlAndHeaderInfo)throw this.createError(s.InternalServerError,"Need to call this._resolveRequestUrlAndHeaderInfo() to get the session id first.",a.CommitChanges);var r=this.createCommitChangesRequestInfo(t);return OfficeExtension.HttpUtility.sendRequest(r).then((function(r){var i=r.statusCode;if(i===s.Accepted)return n.delay(e.POLL_DELAY).then((function(i){return n.pollResourceLocation(n.getHeaderIgnoreCase(r.headers,e.LOCATION_HEADER),a.CommitChanges,t)})).then((function(e){if(JSON.parse(e.body).status===o.Failed)throw n.createErrorFromResponseInfo(e,a.CommitChanges);return n.parseCooldownTime(e)}));if(i>=s.OK&&i<=s.HighestSuccessCode)return n.parseCooldownTime(r);throw n.createErrorFromResponseInfo(r,a.CommitChanges)}))},e.prototype._resolveRequestUrlAndHeaderInfo=function(t){var n=this;if(void 0===t&&(t={}),this.m_requestUrlAndHeaderInfo)return OfficeExtension.Utility._createPromiseFromResult(this.m_requestUrlAndHeaderInfo);if(!this.m_workbookUrl||OfficeExtension.Utility._isLocalDocumentUrl(this.m_workbookUrl))return this.m_requestUrlAndHeaderInfo={url:this.m_workbookUrl,headers:this.m_requestHeaders},OfficeExtension.Utility._createPromiseFromResult(this.m_requestUrlAndHeaderInfo);if(this.getHeaderIgnoreCase(this.m_requestHeaders,e.WorkbookSessionIdHeaderNameLower))return this.m_requestUrlAndHeaderInfo={url:this.m_workbookUrl,headers:this.m_requestHeaders},OfficeExtension.Utility._createPromiseFromResult(this.m_requestUrlAndHeaderInfo);var r=this.createAsyncGraphSessionRequestInfo(t);return OfficeExtension.HttpUtility.sendRequest(r).then((function(r){if(n.m_requestId=n.getHeaderIgnoreCase(r.headers,e.REQUEST_ID_HEADER),r.statusCode!==s.Accepted&&r.statusCode!==s.Created)throw n.createErrorFromResponseInfo(r,a.Create);return r.statusCode===s.Created?(n.formatRequestUrlAndHeaderInfo(r),n.m_requestUrlAndHeaderInfo):n.delay(e.POLL_DELAY).then((function(i){return n.pollResourceLocation(n.getHeaderIgnoreCase(r.headers,e.LOCATION_HEADER),a.Create,t)})).then((function(r){var i=JSON.parse(r.body);if(i.status===o.Failed)throw n.createErrorFromResponseInfo(r,a.Create);var s={method:"GET",url:i.resourceLocation,headers:__assign({},t,{Authorization:n.getHeaderIgnoreCase(n.m_requestHeaders,e.AUTHORIZATION_HEADER)}),body:void 0};return OfficeExtension.HttpUtility.sendRequest(s).then((function(e){return n.formatRequestUrlAndHeaderInfo(e),n.m_requestUrlAndHeaderInfo}))}))}))},e.prototype.refreshSession=function(){var e=this;if(!this.m_workbookUrl||OfficeExtension.Utility._isLocalDocumentUrl(this.m_workbookUrl))throw this.createError(s.InternalServerError,"Not supported for local documents.",a.Refresh);if(!this.m_requestUrlAndHeaderInfo)throw this.createError(s.InternalServerError,"Need to call this._resolveRequestUrlAndHeaderInfo() to get the session id first.",a.Refresh);var t=this.createRefreshSessionRequestInfo();return OfficeExtension.HttpUtility.sendRequest(t).then((function(t){if(t.statusCode!=s.NoContent)throw e.createErrorFromResponseInfo(t,a.Refresh);return OfficeExtension.Utility._createPromiseFromResult(null)}))},e.prototype.getHeaderIgnoreCase=function(e,t){var n=Object.keys(e).filter((function(e){return e.toLowerCase()===t.toLowerCase()}));return n.length>0?e[n[0]]:void 0},e.prototype.createCommitChangesRequestInfo=function(t){void 0===t&&(t={});var n=this.getCorrectGraphVersionUrl()+e.COMMIT_CHANGES,r={};return OfficeExtension.Utility._copyHeaders(this.m_requestUrlAndHeaderInfo.headers,r),OfficeExtension.Utility._copyHeaders(t,r),r[e.PREFER_HEADER]=e.PREFER_HEADER_VAL,{url:n,method:"POST",headers:r,body:{}}},e.prototype.createAsyncGraphSessionRequestInfo=function(t){void 0===t&&(t={});var n=this.getCorrectGraphVersionUrl()+e.CREATE_SESSION,r={};return OfficeExtension.Utility._copyHeaders(this.m_requestHeaders,r),OfficeExtension.Utility._copyHeaders(t,r),r[e.CONTENT_TYPE_HEADER]=e.CONTENT_TYPE_HEADER_VAL,r[e.PREFER_HEADER]=e.PREFER_HEADER_VAL,{url:n,method:"POST",headers:r,body:{persistChanges:this.m_persistChanges,commitExplicitly:this.m_commitExplicitly}}},e.prototype.createRefreshSessionRequestInfo=function(){var t=this.getCorrectGraphVersionUrl()+e.REFRESH_SESSION,n={};return OfficeExtension.Utility._copyHeaders(this.m_requestUrlAndHeaderInfo.headers,n),n[e.CONTENT_TYPE_HEADER]=e.CONTENT_TYPE_HEADER_VAL,{url:t,method:"POST",headers:n,body:{}}},e.prototype.getCorrectGraphVersionUrl=function(){return this.m_workbookUrl.replace(new RegExp("graph.microsoft.com/.*?/"),"graph.microsoft.com/"+e.ASYNC_API_GRAPH_VERSION+"/")},e.prototype.pollResourceLocation=function(t,n,r,i){var a=this;if(void 0===r&&(r={}),void 0===i&&(i=0),i>=e.MAX_POLL_ATTEMPTS)throw this.createError(s.InternalServerError,"Timout while polling for the resource location.",n);var l={method:"GET",url:t,headers:__assign({},r,{Authorization:this.getHeaderIgnoreCase(this.m_requestHeaders,e.AUTHORIZATION_HEADER)}),body:void 0};return OfficeExtension.HttpUtility.sendRequest(l).then((function(l){if(l.statusCode!==s.OK)return a.pollResourceLocation(t,n,r,i+1);switch(JSON.parse(l.body).status){case o.Succeeded:case o.Failed:return l;case o.NotStarted:case o.Running:return a.delay(e.POLL_DELAY).then((function(e){return a.pollResourceLocation(t,n,r,i+1)}));default:throw a.createErrorFromResponseInfo(l,n)}}))},e.prototype.parseCooldownTime=function(t){var n=this.getHeaderIgnoreCase(t.headers,e.RETRY_AFTER_HEADER);return n?1e3*parseInt(n):e.DEFAULT_COMMIT_CHANGES_RETRY_AFTER},e.prototype.formatRequestUrlAndHeaderInfo=function(t){if(t.statusCode!==s.OK&&t.statusCode!==s.Created)throw this.createErrorFromResponseInfo(t,a.Create);var n=JSON.parse(t.body).id,r={};OfficeExtension.Utility._copyHeaders(this.m_requestHeaders,r),r[e.WorkbookSessionIdHeaderName]=n,this.m_requestUrlAndHeaderInfo={url:this.getCorrectGraphVersionUrl(),headers:r}},e.prototype.ensureUrlFormatEndWithSlash=function(e){return"/"!==e.charAt(e.length-1)&&(e+="/"),e},e.prototype.delay=function(e){return new OfficeExtension.CoreUtility.Promise((function(t,n){return setTimeout((function(){return t()}),e)}))},e.prototype.createErrorFromResponseInfo=function(t,n){var r=OfficeExtension.Utility._parseErrorResponse(t),i={retryAfter:this.getHeaderIgnoreCase(t.headers,e.RETRY_AFTER_HEADER),responseBody:OfficeExtension.Utility._parseErrorResponseBody(t)};return OfficeExtension.Utility.createRuntimeError(r.errorCode,r.errorMessage,n,t.statusCode,i)},e.prototype.createError=function(e,t,n){return OfficeExtension.Utility.createRuntimeError(""+e,t,n)},e.WorkbookSessionIdHeaderName="Workbook-Session-Id",e.WorkbookSessionIdHeaderNameLower="workbook-session-id",e.ASYNC_API_GRAPH_VERSION="beta",e.POLL_DELAY=1e4,e.MAX_POLL_ATTEMPTS=10,e.DEFAULT_COMMIT_CHANGES_RETRY_AFTER=1e4,e.LOCATION_HEADER="location",e.AUTHORIZATION_HEADER="authorization",e.REQUEST_ID_HEADER="request-id",e.RETRY_AFTER_HEADER="retry-after",e.PREFER_HEADER="Prefer",e.PREFER_HEADER_VAL="respond-async",e.CONTENT_TYPE_HEADER="Content-Type",e.CONTENT_TYPE_HEADER_VAL="application/json",e.CLOSE_SESSION="closeSession",e.COMMIT_CHANGES="commitChanges",e.CREATE_SESSION="createSession",e.REFRESH_SESSION="refreshSession",e}();e.Session=h;var d=function(t){function n(n){var r=t.call(this,n)||this;return r.m_workbook=A(e.Workbook,r),r._rootObject=r.m_workbook,r._rootObjectPropertyName="workbook",r}return __extends(n,t),n.prototype._processOfficeJsErrorResponse=function(e,t){5004===e&&(t.ErrorCode=Fo.invalidOperationInCellEditMode,t.HttpStatusCode=400,t.ErrorMessage=OfficeExtension.Utility._getResourceString(OfficeExtension.ResourceStrings.invalidOperationInCellEditMode))},Object.defineProperty(n.prototype,"workbook",{get:function(){return this.m_workbook},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"application",{get:function(){return this.workbook.application},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"runtime",{get:function(){return this.workbook._Runtime},enumerable:!0,configurable:!0}),n}(OfficeCore.RequestContext);function y(e,t){var n=e;n.delayForCellEdit&&OfficeExtension.CommonUtility.isSetSupported("DelayForCellEdit")?t._requestFlagModifier|=64:t._requestFlagModifier&=-65,n._makerSafe&&(t._requestFlagModifier|=1024)}function m(e){var t={BindingColumnCount:e.bindingColumnCount,BindingId:e.bindingId,BindingRowCount:e.bindingRowCount,bindingType:e.bindingType,HasHeaders:e.hasHeaders};return window.OSF.DDA.OMFactory.manufactureBinding(t,window.Microsoft.Office.WebExtension.context.document)}function g(e,t){var n;return n=null==e.headers?e.rows:e,void 0===(n=window.OSF.DDA.DataCoercion.coerceData(n,t[window.Microsoft.Office.WebExtension.Parameters.CoercionType]))?null:n}function _(e){if(!("undefined"!=typeof window&&window.Office&&window.Office.context&&window.Office.context.diagnostics&&window.Office.context.diagnostics.version))return!1;var t=window.Office.context.diagnostics.platform.toLowerCase();if("officeonline"===t)return!1;var n=e[t];null==n&&(n=e.general);var r=window.Office.context.diagnostics.version,i=/^(\d+)\.(\d+)\.(\d+)\.(\d+)$/.exec(r);if(i){var o=parseInt(i[1]),s=parseInt(i[2]),a=parseInt(i[3]);if(o<n.desiredMajor)return!0;if(o===n.desiredMajor&&s<n.desiredMinor)return!0;if(o===n.desiredMajor&&s===n.desiredMinor&&a<n.desiredBuild){var l=i[4];if(/^3\d\d\d+$/.exec(l))return!0}}return!1}e.RequestContext=d,e.run=function(t,n){return OfficeExtension.ClientRequestContext._runBatch("Excel.run",arguments,(function(t){return new e.RequestContext(t)}),y)},e.runBatch=function(t,n){return OfficeExtension.ClientRequestContext._runExplicitBatch("Excel.runBatch",arguments,(function(t){return new e.RequestContext(t)}),y)},e._RedirectV1APIs=!1,e._V1APIMap={GetDataAsync:{call:function(e,t){return e.workbook._V1Api.bindingGetData(t)},postprocess:g},GetSelectedDataAsync:{call:function(e,t){return e.workbook._V1Api.getSelectedData(t)},postprocess:g},GoToByIdAsync:{call:function(e,t){return e.workbook._V1Api.gotoById(t)}},AddColumnsAsync:{call:function(e,t){return e.workbook._V1Api.bindingAddColumns(t)}},AddFromSelectionAsync:{call:function(e,t){return e.workbook._V1Api.bindingAddFromSelection(t)},postprocess:m},AddFromNamedItemAsync:{call:function(e,t){return e.workbook._V1Api.bindingAddFromNamedItem(t)},postprocess:m},AddFromPromptAsync:{call:function(e,t){return _({ios:{desiredMajor:2,desiredMinor:20,desiredBuild:0}})&&OfficeExtension.CommonUtility.isSetSupported("DelayForCellEdit")&&(e._requestFlagModifier|=64),e.workbook._V1Api.bindingAddFromPrompt(t)},postprocess:m},AddRowsAsync:{call:function(e,t){return e.workbook._V1Api.bindingAddRows(t)}},GetByIdAsync:{call:function(e,t){return e.workbook._V1Api.bindingGetById(t)},postprocess:m},ReleaseByIdAsync:{call:function(e,t){return e.workbook._V1Api.bindingReleaseById(t)}},GetAllAsync:{call:function(e){return e.workbook._V1Api.bindingGetAll()},postprocess:function(e){return e.bindings.map((function(e){return m(e)}))}},DeleteAllDataValuesAsync:{call:function(e,t){return e.workbook._V1Api.bindingDeleteAllDataValues(t)}},SetSelectedDataAsync:{preprocess:function(e){var t=e.cellFormat;return"undefined"!=typeof window&&window.OSF.DDA.SafeArray?window.OSF.OUtil.listContainsKey(window.OSF.DDA.SafeArray.Delegate.ParameterMap.dynamicTypes,"cellFormat")&&(e.cellFormat=window.OSF.DDA.SafeArray.Delegate.ParameterMap.dynamicTypes.cellFormat.toHost(t)):"undefined"!=typeof window&&window.OSF.DDA.WAC&&window.OSF.OUtil.listContainsKey(window.OSF.DDA.WAC.Delegate.ParameterMap.dynamicTypes,"cellFormat")&&(e.cellFormat=window.OSF.DDA.WAC.Delegate.ParameterMap.dynamicTypes.cellFormat.toHost(t)),e},call:function(e,t){return e.workbook._V1Api.setSelectedData(t)}},SetDataAsync:{preprocess:function(e){var t=e.cellFormat;return"undefined"!=typeof window&&window.OSF.DDA.SafeArray?window.OSF.OUtil.listContainsKey(window.OSF.DDA.SafeArray.Delegate.ParameterMap.dynamicTypes,"cellFormat")&&(e.cellFormat=window.OSF.DDA.SafeArray.Delegate.ParameterMap.dynamicTypes.cellFormat.toHost(t)):"undefined"!=typeof window&&window.OSF.DDA.WAC&&window.OSF.OUtil.listContainsKey(window.OSF.DDA.WAC.Delegate.ParameterMap.dynamicTypes,"cellFormat")&&(e.cellFormat=window.OSF.DDA.WAC.Delegate.ParameterMap.dynamicTypes.cellFormat.toHost(t)),e},call:function(e,t){return e.workbook._V1Api.bindingSetData(t)}},SetFormatsAsync:{preprocess:function(e){var t=e.cellFormat;return"undefined"!=typeof window&&window.OSF.DDA.SafeArray?window.OSF.OUtil.listContainsKey(window.OSF.DDA.SafeArray.Delegate.ParameterMap.dynamicTypes,"cellFormat")&&(e.cellFormat=window.OSF.DDA.SafeArray.Delegate.ParameterMap.dynamicTypes.cellFormat.toHost(t)):"undefined"!=typeof window&&window.OSF.DDA.WAC&&window.OSF.OUtil.listContainsKey(window.OSF.DDA.WAC.Delegate.ParameterMap.dynamicTypes,"cellFormat")&&(e.cellFormat=window.OSF.DDA.WAC.Delegate.ParameterMap.dynamicTypes.cellFormat.toHost(t)),e},call:function(e,t){return e.workbook._V1Api.bindingSetFormats(t)}},SetTableOptionsAsync:{call:function(e,t){return e.workbook._V1Api.bindingSetTableOptions(t)}},ClearFormatsAsync:{call:function(e,t){return e.workbook._V1Api.bindingClearFormats(t)}},GetFilePropertiesAsync:{call:function(e){return e.workbook._V1Api.getFilePropertiesAsync()}}};var b;!function(e){e.office10EventIdBindingSelectionChangedEvent=3,e.office10EventIdBindingDataChangedEvent=4,e.office10EventIdDocumentSelectionChangedEvent=2,e.office10EventIdRichApiMessageEvent=5,e.office10EventIdSettingsChangedEvent=1}(b=e._CC||(e._CC={}));var O="Excel",v="ExcelApi",S=OfficeExtension.BatchApiHelper.createPropertyObject,C=OfficeExtension.BatchApiHelper.createMethodObject,P=OfficeExtension.BatchApiHelper.createIndexerObject,A=OfficeExtension.BatchApiHelper.createRootServiceObject,F=OfficeExtension.BatchApiHelper.createTopLevelServiceObject,D=OfficeExtension.BatchApiHelper.createChildItemObject,E=OfficeExtension.BatchApiHelper.invokeMethod,R=OfficeExtension.BatchApiHelper.invokeEnsureUnchanged,I=OfficeExtension.BatchApiHelper.invokeSetProperty,x=OfficeExtension.Utility.isNullOrUndefined,N=OfficeExtension.Utility.isUndefined,T=OfficeExtension.Utility.throwIfNotLoaded,w=OfficeExtension.Utility.throwIfApiNotSupported,j=OfficeExtension.Utility.load,M=OfficeExtension.Utility.retrieve,k=OfficeExtension.Utility.toJson,U=OfficeExtension.Utility.fixObjectPathIfNecessary,L=OfficeExtension.Utility._handleNavigationPropertyResults,B=OfficeExtension.Utility.adjustToDateTime,H=OfficeExtension.Utility.processRetrieveResult,W=OfficeExtension.Utility.setMockData,V=OfficeExtension.CommonUtility.calculateApiFlags;!function(e){e.connectionOnly="ConnectionOnly",e.table="Table",e.pivotTable="PivotTable",e.pivotChart="PivotChart"}(e.LoadToType||(e.LoadToType={}));var G="Query",z=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"Query"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["loadedTo","loadedToDataModel","name","refreshDate","rowsLoadedCount","error"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["LoadedTo","LoadedToDataModel","Name","RefreshDate","RowsLoadedCount","Error"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"error",{get:function(){return T("error",this._E,G,this._isNull),this._E},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"loadedTo",{get:function(){return T("loadedTo",this._L,G,this._isNull),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"loadedToDataModel",{get:function(){return T("loadedToDataModel",this._Lo,G,this._isNull),this._Lo},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"name",{get:function(){return T("name",this._N,G,this._isNull),this._N},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"refreshDate",{get:function(){return T("refreshDate",this._R,G,this._isNull),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"rowsLoadedCount",{get:function(){return T("rowsLoadedCount",this._Ro,G,this._isNull),this._Ro},enumerable:!0,configurable:!0}),t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Error)||(this._E=n.Error),N(n.LoadedTo)||(this._L=n.LoadedTo),N(n.LoadedToDataModel)||(this._Lo=n.LoadedToDataModel),N(n.Name)||(this._N=n.Name),N(n.RefreshDate)||(this._R=B(n.RefreshDate)),N(n.RowsLoadedCount)||(this._Ro=n.RowsLoadedCount)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){if(e.prototype._handleRetrieveResult.call(this,t,n),!x(t)){var r=t;N(r.RefreshDate)||(r.refreshDate=B(r.refreshDate)),H(this,t,n)}},t.prototype.toJSON=function(){return k(this,{},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.Query=z;var q=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"QueryCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"QueryCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.Query,this,[t])},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Query,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Query,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Query,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.QueryCollection=q,function(e){e.unknown="Unknown",e.none="None",e.failedLoadToWorksheet="FailedLoadToWorksheet",e.failedLoadToDataModel="FailedLoadToDataModel",e.failedDownload="FailedDownload",e.failedToCompleteDownload="FailedToCompleteDownload"}(e.QueryError||(e.QueryError={}));var J="NumberFormatProperty",K=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"NumberFormatProperty"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["key","currency","dateTime","dateTimeHasDayOfWeek","dateTimeHasMonth","dateTimeHasYear","numeric","percent","text"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Key","Currency","DateTime","DateTimeHasDayOfWeek","DateTimeHasMonth","DateTimeHasYear","Numeric","Percent","Text"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"currency",{get:function(){return T("currency",this._C,J,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dateTime",{get:function(){return T("dateTime",this._D,J,this._isNull),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dateTimeHasDayOfWeek",{get:function(){return T("dateTimeHasDayOfWeek",this._Da,J,this._isNull),this._Da},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dateTimeHasMonth",{get:function(){return T("dateTimeHasMonth",this._Dat,J,this._isNull),this._Dat},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dateTimeHasYear",{get:function(){return T("dateTimeHasYear",this._Date,J,this._isNull),this._Date},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"key",{get:function(){return T("key",this._K,J,this._isNull),this._K},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"numeric",{get:function(){return T("numeric",this._N,J,this._isNull),this._N},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"percent",{get:function(){return T("percent",this._P,J,this._isNull),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"text",{get:function(){return T("text",this._T,J,this._isNull),this._T},enumerable:!0,configurable:!0}),t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Currency)||(this._C=n.Currency),N(n.DateTime)||(this._D=n.DateTime),N(n.DateTimeHasDayOfWeek)||(this._Da=n.DateTimeHasDayOfWeek),N(n.DateTimeHasMonth)||(this._Dat=n.DateTimeHasMonth),N(n.DateTimeHasYear)||(this._Date=n.DateTimeHasYear),N(n.Key)||(this._K=n.Key),N(n.Numeric)||(this._N=n.Numeric),N(n.Percent)||(this._P=n.Percent),N(n.Text)||(this._T=n.Text)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{currency:this._C,dateTime:this._D,dateTimeHasDayOfWeek:this._Da,dateTimeHasMonth:this._Dat,dateTimeHasYear:this._Date,key:this._K,numeric:this._N,percent:this._P,text:this._T},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.NumberFormatProperty=K;var Q=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"NumberFormatPropertyCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"NumberFormatPropertyCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getItemAt=function(t){return C(e.NumberFormatProperty,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.NumberFormatProperty,!1,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.NumberFormatProperty,!1,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.NumberFormatProperty,!1,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.NumberFormatPropertyCollection=Q;var X=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"LinkedWorkbook"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["id"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"id",{get:function(){return T("id",this._I,"LinkedWorkbook",this._isNull),this._I},enumerable:!0,configurable:!0}),t.prototype.breakLinks=function(){E(this,"BreakLinks",0,[],0,0)},t.prototype.refresh=function(){E(this,"Refresh",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Id)||(this._I=n.Id)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleIdResult=function(t){e.prototype._handleIdResult.call(this,t),x(t)||N(t.Id)||(this._I=t.Id)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.LinkedWorkbook=X;var Y="LinkedWorkbookCollection",Z=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"LinkedWorkbookCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["workbookLinksRefreshMode"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["WorkbookLinksRefreshMode"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,Y,this._isNull),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"workbookLinksRefreshMode",{get:function(){return T("workbookLinksRefreshMode",this._W,Y,this._isNull),this._W},set:function(e){this._W=e,I(this,"WorkbookLinksRefreshMode",e,0)},enumerable:!0,configurable:!0}),n.prototype.breakAllLinks=function(){E(this,"BreakAllLinks",0,[],0,0)},n.prototype.getItem=function(t){return P(e.LinkedWorkbook,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.LinkedWorkbook,this,"GetItemOrNullObject",0,[t],!1,!1,null,0)},n.prototype.refreshAll=function(){E(this,"RefreshAll",0,[],0,0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),N(r.WorkbookLinksRefreshMode)||(this._W=r.WorkbookLinksRefreshMode),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.LinkedWorkbook,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.LinkedWorkbook,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.LinkedWorkbook,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.LinkedWorkbookCollection=Z,function(e){e.manual="Manual",e.automatic="Automatic"}(e.WorkbookLinksRefreshMode||(e.WorkbookLinksRefreshMode={})),function(e){e.unknown="Unknown",e.cube="Cube",e.localRange="LocalRange",e.localTable="LocalTable",e.oledbDefault="OledbDefault"}(e.DataSourceType||(e.DataSourceType={})),function(e){e.unknown="Unknown",e.equals="Equals",e.before="Before",e.beforeOrEqualTo="BeforeOrEqualTo",e.after="After",e.afterOrEqualTo="AfterOrEqualTo",e.between="Between",e.tomorrow="Tomorrow",e.today="Today",e.yesterday="Yesterday",e.nextWeek="NextWeek",e.thisWeek="ThisWeek",e.lastWeek="LastWeek",e.nextMonth="NextMonth",e.thisMonth="ThisMonth",e.lastMonth="LastMonth",e.nextQuarter="NextQuarter",e.thisQuarter="ThisQuarter",e.lastQuarter="LastQuarter",e.nextYear="NextYear",e.thisYear="ThisYear",e.lastYear="LastYear",e.yearToDate="YearToDate",e.allDatesInPeriodQuarter1="AllDatesInPeriodQuarter1",e.allDatesInPeriodQuarter2="AllDatesInPeriodQuarter2",e.allDatesInPeriodQuarter3="AllDatesInPeriodQuarter3",e.allDatesInPeriodQuarter4="AllDatesInPeriodQuarter4",e.allDatesInPeriodJanuary="AllDatesInPeriodJanuary",e.allDatesInPeriodFebruary="AllDatesInPeriodFebruary",e.allDatesInPeriodMarch="AllDatesInPeriodMarch",e.allDatesInPeriodApril="AllDatesInPeriodApril",e.allDatesInPeriodMay="AllDatesInPeriodMay",e.allDatesInPeriodJune="AllDatesInPeriodJune",e.allDatesInPeriodJuly="AllDatesInPeriodJuly",e.allDatesInPeriodAugust="AllDatesInPeriodAugust",e.allDatesInPeriodSeptember="AllDatesInPeriodSeptember",e.allDatesInPeriodOctober="AllDatesInPeriodOctober",e.allDatesInPeriodNovember="AllDatesInPeriodNovember",e.allDatesInPeriodDecember="AllDatesInPeriodDecember"}(e.DateFilterCondition||(e.DateFilterCondition={})),function(e){e.unknown="Unknown",e.equals="Equals",e.beginsWith="BeginsWith",e.endsWith="EndsWith",e.contains="Contains",e.greaterThan="GreaterThan",e.greaterThanOrEqualTo="GreaterThanOrEqualTo",e.lessThan="LessThan",e.lessThanOrEqualTo="LessThanOrEqualTo",e.between="Between"}(e.LabelFilterCondition||(e.LabelFilterCondition={})),function(e){e.unknown="Unknown",e.value="Value",e.manual="Manual",e.label="Label",e.date="Date"}(e.PivotFilterType||(e.PivotFilterType={})),function(e){e.items="Items",e.percent="Percent",e.sum="Sum"}(e.TopBottomSelectionType||(e.TopBottomSelectionType={})),function(e){e.unknown="Unknown",e.equals="Equals",e.greaterThan="GreaterThan",e.greaterThanOrEqualTo="GreaterThanOrEqualTo",e.lessThan="LessThan",e.lessThanOrEqualTo="LessThanOrEqualTo",e.between="Between",e.topN="TopN",e.bottomN="BottomN"}(e.ValueFilterCondition||(e.ValueFilterCondition={})),function(e){e.categories="Categories",e.values="Values",e.xvalues="XValues",e.yvalues="YValues",e.bubbleSizes="BubbleSizes"}(e.ChartSeriesDimension||(e.ChartSeriesDimension={}));var $=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"Runtime"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["enableEvents"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["EnableEvents"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"enableEvents",{get:function(){return T("enableEvents",this._E,"Runtime",this._isNull),w("Runtime.enableEvents",v,"1.8",O),this._E},set:function(e){this._E=e,I(this,"EnableEvents",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["enableEvents"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.EnableEvents)||(this._E=n.EnableEvents)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{enableEvents:this._E},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.Runtime=$;var ee="Application",te=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Application"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["calculationMode","calculationEngineVersion","calculationState","decimalSeparator","thousandsSeparator","useSystemSeparators"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["CalculationMode","CalculationEngineVersion","CalculationState","DecimalSeparator","ThousandsSeparator","UseSystemSeparators"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!1,!1,!1,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["iterativeCalculation","ribbon","cultureInfo"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"cultureInfo",{get:function(){return w("Application.cultureInfo",v,"1.11",O),this._Cu||(this._Cu=S(e.CultureInfo,this,"CultureInfo",!1,4)),this._Cu},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"iterativeCalculation",{get:function(){return w("Application.iterativeCalculation",v,"1.9",O),this._I||(this._I=S(e.IterativeCalculation,this,"IterativeCalculation",!1,4)),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"ribbon",{get:function(){return w("Application.ribbon",v,"1.9",O),this._R||(this._R=S(e.Ribbon,this,"Ribbon",!1,4)),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"calculationEngineVersion",{get:function(){return T("calculationEngineVersion",this._C,ee,this._isNull),w("Application.calculationEngineVersion",v,"1.9",O),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"calculationMode",{get:function(){return T("calculationMode",this._Ca,ee,this._isNull),this._Ca},set:function(e){this._Ca=e,I(this,"CalculationMode",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"calculationState",{get:function(){return T("calculationState",this._Cal,ee,this._isNull),w("Application.calculationState",v,"1.9",O),this._Cal},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"decimalSeparator",{get:function(){return T("decimalSeparator",this._D,ee,this._isNull),w("Application.decimalSeparator",v,"1.11",O),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"thousandsSeparator",{get:function(){return T("thousandsSeparator",this._T,ee,this._isNull),w("Application.thousandsSeparator",v,"1.11",O),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"useSystemSeparators",{get:function(){return T("useSystemSeparators",this._U,ee,this._isNull),w("Application.useSystemSeparators",v,"1.11",O),this._U},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["calculationMode"],["iterativeCalculation","ribbon"],["cultureInfo"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.calculate=function(e){E(this,"Calculate",0,[e],0,0)},n.prototype.createWorkbook=function(t){return w("Application.createWorkbook",v,"1.8",O),C(e.WorkbookCreated,this,"CreateWorkbook",1,[t],!1,!0,"_GetWorkbookCreatedById",0)},n.prototype.suspendApiCalculationUntilNextSync=function(){b.Application_SuspendApiCalculationUntilNextSync(this).handled||(w("Application.suspendApiCalculationUntilNextSync",v,"1.6",O),E(this,"SuspendApiCalculationUntilNextSync",0,[],V(2,"ExcelApiUndo","1.6"),0))},n.prototype.suspendScreenUpdatingUntilNextSync=function(){w("Application.suspendScreenUpdatingUntilNextSync",v,"1.9",O),E(this,"SuspendScreenUpdatingUntilNextSync",0,[],V(2,"ExcelApiUndo","1.6"),0)},n.prototype._GetWorkbookCreatedById=function(t){return w("Application._GetWorkbookCreatedById",v,"1.8",O),C(e.WorkbookCreated,this,"_GetWorkbookCreatedById",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.CalculationEngineVersion)||(this._C=n.CalculationEngineVersion),N(n.CalculationMode)||(this._Ca=n.CalculationMode),N(n.CalculationState)||(this._Cal=n.CalculationState),N(n.DecimalSeparator)||(this._D=n.DecimalSeparator),N(n.ThousandsSeparator)||(this._T=n.ThousandsSeparator),N(n.UseSystemSeparators)||(this._U=n.UseSystemSeparators),L(this,n,["cultureInfo","CultureInfo","iterativeCalculation","IterativeCalculation","ribbon","Ribbon"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{calculationEngineVersion:this._C,calculationMode:this._Ca,calculationState:this._Cal,decimalSeparator:this._D,thousandsSeparator:this._T,useSystemSeparators:this._U},{cultureInfo:this._Cu,iterativeCalculation:this._I,ribbon:this._R})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Application=te,function(e){e.Application_SuspendApiCalculationUntilNextSync=function(e){return f("Mac")&&c(1.6)&&!c(1.7)?{handled:!0}:{handled:!1}}}(b=e._CC||(e._CC={}));var ne="IterativeCalculation",re=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"IterativeCalculation"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["enabled","maxIteration","maxChange"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Enabled","MaxIteration","MaxChange"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"enabled",{get:function(){return T("enabled",this._E,ne,this._isNull),this._E},set:function(e){this._E=e,I(this,"Enabled",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"maxChange",{get:function(){return T("maxChange",this._M,ne,this._isNull),this._M},set:function(e){this._M=e,I(this,"MaxChange",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"maxIteration",{get:function(){return T("maxIteration",this._Ma,ne,this._isNull),this._Ma},set:function(e){this._Ma=e,I(this,"MaxIteration",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["enabled","maxIteration","maxChange"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Enabled)||(this._E=n.Enabled),N(n.MaxChange)||(this._M=n.MaxChange),N(n.MaxIteration)||(this._Ma=n.MaxIteration)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{enabled:this._E,maxChange:this._M,maxIteration:this._Ma},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.IterativeCalculation=re;var ie="Workbook",oe=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Workbook"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["name","readOnly","isDirty","chartDataPointTrack","usePrecisionAsDisplayed","calculationEngineVersion","autoSave","previouslySaved"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name","ReadOnly","IsDirty","ChartDataPointTrack","UsePrecisionAsDisplayed","CalculationEngineVersion","AutoSave","PreviouslySaved"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!1,!0,!0,!0,!1,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["worksheets","names","tables","application","bindings","functions","_V1Api","pivotTables","settings","customXmlParts","internalTest","properties","styles","protection","dataConnections","_Runtime","comments","slicers","tableStyles","pivotTableStyles","slicerStyles","timelineStyles","queries","linkedWorkbooks"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"application",{get:function(){return this._A||(this._A=S(e.Application,this,"Application",!1,4)),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"bindings",{get:function(){return this._B||(this._B=S(e.BindingCollection,this,"Bindings",!0,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"comments",{get:function(){return w("Workbook.comments",v,"1.10",O),this._Co||(this._Co=S(e.CommentCollection,this,"Comments",!0,4)),this._Co},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"customXmlParts",{get:function(){return w("Workbook.customXmlParts",v,"1.5",O),this._Cus||(this._Cus=S(e.CustomXmlPartCollection,this,"CustomXmlParts",!0,4)),this._Cus},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"dataConnections",{get:function(){return w("Workbook.dataConnections",v,"1.7",O),this._D||(this._D=S(e.DataConnectionCollection,this,"DataConnections",!1,4)),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"functions",{get:function(){return w("Workbook.functions",v,"1.2",O),this._F||(this._F=S(e.Functions,this,"Functions",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"internalTest",{get:function(){return w("Workbook.internalTest",v,"1.6",O),this._I||(this._I=S(e.InternalTest,this,"InternalTest",!1,4)),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"linkedWorkbooks",{get:function(){return w("Workbook.linkedWorkbooks","ExcelApiOnline","1.1",O),this._L||(this._L=S(e.LinkedWorkbookCollection,this,"LinkedWorkbooks",!0,4)),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"names",{get:function(){return this._Na||(this._Na=S(e.NamedItemCollection,this,"Names",!0,4)),this._Na},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"pivotTableStyles",{get:function(){return w("Workbook.pivotTableStyles",v,"1.10",O),this._Pi||(this._Pi=S(e.PivotTableStyleCollection,this,"PivotTableStyles",!0,4)),this._Pi},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"pivotTables",{get:function(){return w("Workbook.pivotTables",v,"1.3",O),this._P||(this._P=S(e.PivotTableCollection,this,"PivotTables",!0,4)),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"properties",{get:function(){return w("Workbook.properties",v,"1.7",O),this._Pro||(this._Pro=S(e.DocumentProperties,this,"Properties",!1,4)),this._Pro},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"protection",{get:function(){return w("Workbook.protection",v,"1.7",O),this._Prot||(this._Prot=S(e.WorkbookProtection,this,"Protection",!1,4)),this._Prot},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"queries",{get:function(){return w("Workbook.queries",v,"1.14",O),this._Q||(this._Q=S(e.QueryCollection,this,"Queries",!0,4)),this._Q},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"settings",{get:function(){return w("Workbook.settings",v,"1.4",O),this._S||(this._S=S(e.SettingCollection,this,"Settings",!0,4)),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"slicerStyles",{get:function(){return w("Workbook.slicerStyles",v,"1.10",O),this._Sli||(this._Sli=S(e.SlicerStyleCollection,this,"SlicerStyles",!0,4)),this._Sli},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"slicers",{get:function(){return w("Workbook.slicers",v,"1.10",O),this._Sl||(this._Sl=S(e.SlicerCollection,this,"Slicers",!0,4)),this._Sl},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"styles",{get:function(){return w("Workbook.styles",v,"1.7",O),this._St||(this._St=S(e.StyleCollection,this,"Styles",!0,4)),this._St},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"tableStyles",{get:function(){return w("Workbook.tableStyles",v,"1.10",O),this._Ta||(this._Ta=S(e.TableStyleCollection,this,"TableStyles",!0,4)),this._Ta},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"tables",{get:function(){return this._T||(this._T=S(e.TableCollection,this,"Tables",!0,4)),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"timelineStyles",{get:function(){return w("Workbook.timelineStyles",v,"1.10",O),this._Ti||(this._Ti=S(e.TimelineStyleCollection,this,"TimelineStyles",!0,4)),this._Ti},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"worksheets",{get:function(){return this._W||(this._W=S(e.WorksheetCollection,this,"Worksheets",!0,4)),this._W},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_Runtime",{get:function(){return w("Workbook._Runtime",v,"1.5",O),this.__R||(this.__R=S(e.Runtime,this,"_Runtime",!1,4)),this.__R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_V1Api",{get:function(){return w("Workbook._V1Api",v,"1.3",O),this.__V||(this.__V=S(e._V1Api,this,"_V1Api",!1,4)),this.__V},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"autoSave",{get:function(){return T("autoSave",this._Au,ie,this._isNull),w("Workbook.autoSave",v,"1.9",O),this._Au},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"calculationEngineVersion",{get:function(){return T("calculationEngineVersion",this._C,ie,this._isNull),w("Workbook.calculationEngineVersion",v,"1.9",O),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"chartDataPointTrack",{get:function(){return T("chartDataPointTrack",this._Ch,ie,this._isNull),w("Workbook.chartDataPointTrack",v,"1.9",O),this._Ch},set:function(e){this._Ch=e,I(this,"ChartDataPointTrack",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isDirty",{get:function(){return T("isDirty",this._Is,ie,this._isNull),w("Workbook.isDirty",v,"1.9",O),this._Is},set:function(e){this._Is=e,I(this,"IsDirty",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,ie,this._isNull),w("Workbook.name",v,"1.7",O),this._N},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"previouslySaved",{get:function(){return T("previouslySaved",this._Pr,ie,this._isNull),w("Workbook.previouslySaved",v,"1.9",O),this._Pr},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"readOnly",{get:function(){return T("readOnly",this._R,ie,this._isNull),w("Workbook.readOnly",v,"1.8",O),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"usePrecisionAsDisplayed",{get:function(){return T("usePrecisionAsDisplayed",this._U,ie,this._isNull),w("Workbook.usePrecisionAsDisplayed",v,"1.9",O),this._U},set:function(e){this._U=e,I(this,"UsePrecisionAsDisplayed",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["isDirty","chartDataPointTrack","usePrecisionAsDisplayed"],["properties"],["_Runtime","_V1Api","application","bindings","comments","customXmlParts","dataConnections","functions","internalTest","linkedWorkbooks","names","pivotTables","pivotTableStyles","protection","queries","settings","slicers","slicerStyles","styles","tables","tableStyles","timelineStyles","worksheets"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.close=function(e){w("Workbook.close",v,"1.11",O),E(this,"Close",0,[e],0,0)},n.prototype.enableOfficeScriptRecording=function(e){b.Workbook_EnableOfficeScriptRecording(this,e).handled||(w("Workbook.enableOfficeScriptRecording","ExcelApiOnline","1.1",O),E(this,"EnableOfficeScriptRecording",0,[e],0,0))},n.prototype.getActiveCell=function(){return w("Workbook.getActiveCell",v,"1.7",O),C(e.Range,this,"GetActiveCell",1,[],!1,!0,null,4)},n.prototype.getActiveChart=function(){return w("Workbook.getActiveChart",v,"1.9",O),C(e.Chart,this,"GetActiveChart",1,[],!1,!1,null,4)},n.prototype.getActiveChartOrNullObject=function(){return w("Workbook.getActiveChartOrNullObject",v,"1.9",O),C(e.Chart,this,"GetActiveChartOrNullObject",1,[],!1,!1,null,4)},n.prototype.getActiveSlicer=function(){return w("Workbook.getActiveSlicer",v,"1.10",O),C(e.Slicer,this,"GetActiveSlicer",1,[],!1,!1,null,4)},n.prototype.getActiveSlicerOrNullObject=function(){return w("Workbook.getActiveSlicerOrNullObject",v,"1.10",O),C(e.Slicer,this,"GetActiveSlicerOrNullObject",1,[],!1,!1,null,4)},n.prototype.getIsActiveCollabSession=function(){return w("Workbook.getIsActiveCollabSession",v,"1.9",O),E(this,"GetIsActiveCollabSession",0,[],0,0)},n.prototype.getSelectedRange=function(){return C(e.Range,this,"GetSelectedRange",1,[],!1,!0,null,4)},n.prototype.getSelectedRanges=function(){return w("Workbook.getSelectedRanges",v,"1.9",O),C(e.RangeAreas,this,"GetSelectedRanges",1,[],!1,!0,null,4)},n.prototype.insertWorksheetsFromBase64=function(e,t){return w("Workbook.insertWorksheetsFromBase64",v,"1.13",O),E(this,"InsertWorksheetsFromBase64",0,[e,t],0,0)},n.prototype.recordAction=function(e){b.Workbook_RecordAction(this,e).handled||E(this,"RecordAction",1,[e],4,0)},n.prototype.registerCustomFunctions=function(e,t,n,r,i,o,s){w("Workbook.registerCustomFunctions","CustomFunctions","1.1",O),E(this,"RegisterCustomFunctions",0,[e,t,n,r,i,o,s],8,0)},n.prototype.save=function(e){w("Workbook.save",v,"1.11",O),E(this,"Save",0,[e],0,0)},n.prototype._GetObjectByReferenceId=function(e){return E(this,"_GetObjectByReferenceId",1,[e],4,0)},n.prototype._GetObjectTypeNameByReferenceId=function(e){return E(this,"_GetObjectTypeNameByReferenceId",1,[e],4,0)},n.prototype._GetRangeForEventByReferenceId=function(t){return C(e.Range,this,"_GetRangeForEventByReferenceId",1,[t],!1,!1,null,4)},n.prototype._GetRangeOrNullObjectForEventByReferenceId=function(t){return C(e.Range,this,"_GetRangeOrNullObjectForEventByReferenceId",1,[t],!1,!1,null,4)},n.prototype._GetRangesForEventByReferenceId=function(t){return w("Workbook._GetRangesForEventByReferenceId",v,"1.9",O),C(e.RangeAreas,this,"_GetRangesForEventByReferenceId",1,[t],!1,!1,null,4)},n.prototype._GetRangesOrNullObjectForEventByReferenceId=function(t){return w("Workbook._GetRangesOrNullObjectForEventByReferenceId",v,"1.9",O),C(e.RangeAreas,this,"_GetRangesOrNullObjectForEventByReferenceId",1,[t],!1,!1,null,4)},n.prototype._GetReferenceCount=function(){return E(this,"_GetReferenceCount",1,[],4,0)},n.prototype._RegisterActivatedEvent=function(){w("Workbook._RegisterActivatedEvent",v,"1.13",O),E(this,"_RegisterActivatedEvent",1,[],0,0)},n.prototype._RegisterAutoSaveSettingChangedEvent=function(){w("Workbook._RegisterAutoSaveSettingChangedEvent",v,"1.9",O),E(this,"_RegisterAutoSaveSettingChangedEvent",0,[],0,0)},n.prototype._RemoveAllReferences=function(){E(this,"_RemoveAllReferences",1,[],0,0)},n.prototype._RemoveReference=function(e){E(this,"_RemoveReference",1,[e],0,0)},n.prototype._SetOsfControlContainerReadyForCustomFunctions=function(){w("Workbook._SetOsfControlContainerReadyForCustomFunctions","CustomFunctions","1.1",O),E(this,"_SetOsfControlContainerReadyForCustomFunctions",0,[],0,0)},n.prototype._UnregisterActivatedEvent=function(){w("Workbook._UnregisterActivatedEvent",v,"1.13",O),E(this,"_UnregisterActivatedEvent",1,[],0,0)},n.prototype._UnregisterAutoSaveSettingChangedEvent=function(){w("Workbook._UnregisterAutoSaveSettingChangedEvent",v,"1.9",O),E(this,"_UnregisterAutoSaveSettingChangedEvent",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AutoSave)||(this._Au=n.AutoSave),N(n.CalculationEngineVersion)||(this._C=n.CalculationEngineVersion),N(n.ChartDataPointTrack)||(this._Ch=n.ChartDataPointTrack),N(n.IsDirty)||(this._Is=n.IsDirty),N(n.Name)||(this._N=n.Name),N(n.PreviouslySaved)||(this._Pr=n.PreviouslySaved),N(n.ReadOnly)||(this._R=n.ReadOnly),N(n.UsePrecisionAsDisplayed)||(this._U=n.UsePrecisionAsDisplayed),L(this,n,["application","Application","bindings","Bindings","comments","Comments","customXmlParts","CustomXmlParts","dataConnections","DataConnections","functions","Functions","internalTest","InternalTest","linkedWorkbooks","LinkedWorkbooks","names","Names","pivotTableStyles","PivotTableStyles","pivotTables","PivotTables","properties","Properties","protection","Protection","queries","Queries","settings","Settings","slicerStyles","SlicerStyles","slicers","Slicers","styles","Styles","tableStyles","TableStyles","tables","Tables","timelineStyles","TimelineStyles","worksheets","Worksheets","_Runtime","_Runtime","_V1Api","_V1Api"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},Object.defineProperty(n.prototype,"onActivated",{get:function(){var e=this;return w("Workbook.onActivated",v,"1.13",O),this.m_activated||(this.m_activated=new OfficeExtension.GenericEventHandlers(this.context,this,"Activated",{eventType:2203,registerFunc:function(){return e._RegisterActivatedEvent()},unregisterFunc:function(){return e._UnregisterActivatedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.workbookActivated};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_activated},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onAutoSaveSettingChanged",{get:function(){var e=this;return w("Workbook.onAutoSaveSettingChanged",v,"1.9",O),this.m_autoSaveSettingChanged||(this.m_autoSaveSettingChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"AutoSaveSettingChanged",{eventType:2200,registerFunc:function(){return e._RegisterAutoSaveSettingChangedEvent()},unregisterFunc:function(){return e._UnregisterAutoSaveSettingChangedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.workbookAutoSaveSettingChanged};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_autoSaveSettingChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onSelectionChanged",{get:function(){var e=this;return w("Workbook.onSelectionChanged",v,"1.3",O),this.m_selectionChanged||(this.m_selectionChanged=new OfficeExtension.EventHandlers(this.context,this,"SelectionChanged",{registerFunc:function(t){return e.context.eventRegistration.register(b.office10EventIdDocumentSelectionChangedEvent,"",t)},unregisterFunc:function(t){return e.context.eventRegistration.unregister(b.office10EventIdDocumentSelectionChangedEvent,"",t)},eventArgsTransformFunc:function(t){var n=b.Workbook_SelectionChanged_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_selectionChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_onMessage",{get:function(){var e=this;return w("Workbook._onMessage",v,"1.7",O),this.m__Message||(this.m__Message=new OfficeExtension.EventHandlers(this.context,this,"_Message",{registerFunc:function(t){return e.context.eventRegistration.register(b.office10EventIdRichApiMessageEvent,"",t)},unregisterFunc:function(t){return e.context.eventRegistration.unregister(b.office10EventIdRichApiMessageEvent,"",t)},eventArgsTransformFunc:function(t){var n=b.Workbook__Message_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m__Message},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{autoSave:this._Au,calculationEngineVersion:this._C,chartDataPointTrack:this._Ch,isDirty:this._Is,name:this._N,previouslySaved:this._Pr,readOnly:this._R,usePrecisionAsDisplayed:this._U},{bindings:this._B,comments:this._Co,customXmlParts:this._Cus,names:this._Na,pivotTables:this._P,pivotTableStyles:this._Pi,properties:this._Pro,protection:this._Prot,settings:this._S,slicers:this._Sl,slicerStyles:this._Sli,styles:this._St,tables:this._T,tableStyles:this._Ta,timelineStyles:this._Ti,worksheets:this._W})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Workbook=oe,function(e){e.Workbook_EnableOfficeScriptRecording=function(e,t){return this.context.eventId=t,{handled:!1}},e.Workbook_RecordAction=function(e,t){return this.context.sdxPayload=t,{handled:!1}},e.Workbook_SelectionChanged_EventArgsTransform=function(e,t){return{workbook:e}},e.Workbook__Message_EventArgsTransform=function(e,t){return{entries:t.entries,workbook:e}}}(b=e._CC||(e._CC={}));var se=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"WorkbookProtection"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["protected"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Protected"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"protected",{get:function(){return T("protected",this._P,"WorkbookProtection",this._isNull),this._P},enumerable:!0,configurable:!0}),t.prototype.protect=function(e){E(this,"Protect",0,[e],0,0)},t.prototype.unprotect=function(e){E(this,"Unprotect",0,[e],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Protected)||(this._P=n.Protected)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{protected:this._P},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.WorkbookProtection=se;var ae=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"WorkbookCreated"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["id"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"id",{get:function(){return T("id",this._I,"WorkbookCreated",this._isNull),this._I},enumerable:!0,configurable:!0}),t.prototype.open=function(){E(this,"Open",1,[],4,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Id)||(this._I=n.Id)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleIdResult=function(t){e.prototype._handleIdResult.call(this,t),x(t)||N(t.Id)||(this._I=t.Id)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{id:this._I},{})},t}(OfficeExtension.ClientObject);e.WorkbookCreated=ae;var le="Worksheet",ue=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Worksheet"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_collectionPropertyPath",{get:function(){return"workbook.worksheets"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["name","id","position","visibility","tabColor","standardWidth","standardHeight","showGridlines","showHeadings","enableCalculation","tabId"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name","Id","Position","Visibility","TabColor","StandardWidth","StandardHeight","ShowGridlines","ShowHeadings","EnableCalculation","TabId"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!1,!0,!0,!0,!0,!1,!0,!0,!0,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["charts","tables","protection","pivotTables","names","freezePanes","pageLayout","visuals","shapes","horizontalPageBreaks","verticalPageBreaks","autoFilter","slicers","comments","customProperties","namedSheetViews"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"autoFilter",{get:function(){return w("Worksheet.autoFilter",v,"1.9",O),this._A||(this._A=S(e.AutoFilter,this,"AutoFilter",!1,4)),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"charts",{get:function(){return this.m_charts||(this.m_charts=S(e.ChartCollection,this,"Charts",!0,4)),b.Worksheet_Charts_Get(this,this.m_charts),this.m_charts},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"comments",{get:function(){return w("Worksheet.comments",v,"1.10",O),this.m_comments||(this.m_comments=S(e.CommentCollection,this,"Comments",!0,4)),b.Worksheet_Comments_Get(this,this.m_comments),this.m_comments},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"customProperties",{get:function(){return w("Worksheet.customProperties",v,"1.12",O),this._C||(this._C=S(e.WorksheetCustomPropertyCollection,this,"CustomProperties",!0,4)),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"freezePanes",{get:function(){return w("Worksheet.freezePanes",v,"1.7",O),this._F||(this._F=S(e.WorksheetFreezePanes,this,"FreezePanes",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"horizontalPageBreaks",{get:function(){return w("Worksheet.horizontalPageBreaks",v,"1.9",O),this._Ho||(this._Ho=S(e.PageBreakCollection,this,"HorizontalPageBreaks",!0,4)),this._Ho},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"namedSheetViews",{get:function(){return w("Worksheet.namedSheetViews","ExcelApiOnline","1.1",O),this._Na||(this._Na=S(e.NamedSheetViewCollection,this,"NamedSheetViews",!0,4)),this._Na},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"names",{get:function(){return w("Worksheet.names",v,"1.4",O),this.m_names||(this.m_names=S(e.NamedItemCollection,this,"Names",!0,4)),b.Worksheet_Names_Get(this,this.m_names),this.m_names},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"pageLayout",{get:function(){return w("Worksheet.pageLayout",v,"1.9",O),this._P||(this._P=S(e.PageLayout,this,"PageLayout",!1,4)),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"pivotTables",{get:function(){return w("Worksheet.pivotTables",v,"1.3",O),this.m_pivotTables||(this.m_pivotTables=S(e.PivotTableCollection,this,"PivotTables",!0,4)),b.Worksheet_PivotTables_Get(this,this.m_pivotTables),this.m_pivotTables},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"protection",{get:function(){return w("Worksheet.protection",v,"1.2",O),this._Pr||(this._Pr=S(e.WorksheetProtection,this,"Protection",!1,4)),this._Pr},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"shapes",{get:function(){return w("Worksheet.shapes",v,"1.9",O),this.m_shapes||(this.m_shapes=S(e.ShapeCollection,this,"Shapes",!0,4)),b.Worksheet_Shapes_Get(this,this.m_shapes),this.m_shapes},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"slicers",{get:function(){return w("Worksheet.slicers",v,"1.10",O),this._S||(this._S=S(e.SlicerCollection,this,"Slicers",!0,4)),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"tables",{get:function(){return this.m_tables||(this.m_tables=S(e.TableCollection,this,"Tables",!0,4)),b.Worksheet_Tables_Get(this,this.m_tables),this.m_tables},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"verticalPageBreaks",{get:function(){return w("Worksheet.verticalPageBreaks",v,"1.9",O),this._V||(this._V=S(e.PageBreakCollection,this,"VerticalPageBreaks",!0,4)),this._V},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"visuals",{get:function(){return w("Worksheet.visuals",v,"1.10",O),this.m_visuals||(this.m_visuals=S(e.VisualCollection,this,"Visuals",!0,4)),b.Worksheet_Visuals_Get(this,this.m_visuals),this.m_visuals},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"enableCalculation",{get:function(){return T("enableCalculation",this._E,le,this._isNull),w("Worksheet.enableCalculation",v,"1.9",O),this._E},set:function(e){this._E=e,I(this,"EnableCalculation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,le,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,le,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"position",{get:function(){return T("position",this._Po,le,this._isNull),this._Po},set:function(e){this._Po=e,I(this,"Position",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showGridlines",{get:function(){return T("showGridlines",this.m_showGridlines,le,this._isNull),w("Worksheet.showGridlines",v,"1.8",O),this.m_showGridlines},set:function(e){b.Worksheet_ShowGridlines_Set(this,e).handled||(this.m_showGridlines=e,I(this,"ShowGridlines",e,0))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showHeadings",{get:function(){return T("showHeadings",this.m_showHeadings,le,this._isNull),w("Worksheet.showHeadings",v,"1.8",O),this.m_showHeadings},set:function(e){b.Worksheet_ShowHeadings_Set(this,e).handled||(this.m_showHeadings=e,I(this,"ShowHeadings",e,0))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"standardHeight",{get:function(){return T("standardHeight",this._St,le,this._isNull),w("Worksheet.standardHeight",v,"1.7",O),this._St},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"standardWidth",{get:function(){return T("standardWidth",this._Sta,le,this._isNull),w("Worksheet.standardWidth",v,"1.7",O),this._Sta},set:function(e){this._Sta=e,I(this,"StandardWidth",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"tabColor",{get:function(){return T("tabColor",this._T,le,this._isNull),w("Worksheet.tabColor",v,"1.7",O),this._T},set:function(e){this._T=e,I(this,"TabColor",e,V(2,"ExcelApiUndo","1.8"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"tabId",{get:function(){return T("tabId",this._Ta,le,this._isNull),w("Worksheet.tabId",v,"1.14",O),this._Ta},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"visibility",{get:function(){return T("visibility",this._Vi,le,this._isNull),this._Vi},set:function(e){this._Vi=e,I(this,"Visibility",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name","position","visibility","tabColor","standardWidth","showGridlines","showHeadings","enableCalculation"],["pageLayout"],["autoFilter","charts","comments","customProperties","freezePanes","horizontalPageBreaks","namedSheetViews","names","pivotTables","protection","shapes","slicers","tables","verticalPageBreaks","visuals"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.activate=function(){E(this,"Activate",1,[],0,0)},n.prototype.calculate=function(e){w("Worksheet.calculate",v,"1.6",O),E(this,"Calculate",0,[e],0,0)},n.prototype.copy=function(t,n){return w("Worksheet.copy",v,"1.7",O),C(e.Worksheet,this,"Copy",0,[t,n],!1,!1,"_GetAnotherWorksheetById",0)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.findAll=function(t,n){return w("Worksheet.findAll",v,"1.9",O),C(e.RangeAreas,this,"FindAll",1,[t,n],!1,!0,null,4)},n.prototype.findAllOrNullObject=function(t,n){return w("Worksheet.findAllOrNullObject",v,"1.9",O),C(e.RangeAreas,this,"FindAllOrNullObject",1,[t,n],!1,!0,null,4)},n.prototype.getCell=function(t,n){return C(e.Range,this,"GetCell",1,[t,n],!1,!0,null,4)},n.prototype.getNext=function(t){return w("Worksheet.getNext",v,"1.5",O),C(e.Worksheet,this,"GetNext",1,[t],!1,!0,"_GetSheetById",4)},n.prototype.getNextOrNullObject=function(t){return w("Worksheet.getNextOrNullObject",v,"1.5",O),C(e.Worksheet,this,"GetNextOrNullObject",1,[t],!1,!0,"_GetSheetById",4)},n.prototype.getPrevious=function(t){return w("Worksheet.getPrevious",v,"1.5",O),C(e.Worksheet,this,"GetPrevious",1,[t],!1,!0,"_GetSheetById",4)},n.prototype.getPreviousOrNullObject=function(t){return w("Worksheet.getPreviousOrNullObject",v,"1.5",O),C(e.Worksheet,this,"GetPreviousOrNullObject",1,[t],!1,!0,"_GetSheetById",4)},n.prototype.getRange=function(t){return C(e.Range,this,"GetRange",1,[t],!1,!0,null,4)},n.prototype.getRangeByIndexes=function(t,n,r,i){return w("Worksheet.getRangeByIndexes",v,"1.7",O),C(e.Range,this,"GetRangeByIndexes",1,[t,n,r,i],!1,!0,null,4)},n.prototype.getRanges=function(t){return w("Worksheet.getRanges",v,"1.9",O),C(e.RangeAreas,this,"GetRanges",1,[t],!1,!0,null,4)},n.prototype.getUsedRange=function(t){return C(e.Range,this,"GetUsedRange",1,[t],!1,!0,null,4)},n.prototype.getUsedRangeOrNullObject=function(t){return w("Worksheet.getUsedRangeOrNullObject",v,"1.4",O),C(e.Range,this,"GetUsedRangeOrNullObject",1,[t],!1,!0,null,4)},n.prototype.replaceAll=function(e,t,n){return w("Worksheet.replaceAll",v,"1.9",O),E(this,"ReplaceAll",0,[e,t,n],0,0)},n.prototype.showOutlineLevels=function(e,t){w("Worksheet.showOutlineLevels",v,"1.10",O),E(this,"ShowOutlineLevels",0,[e,t],0,0)},n.prototype._GetAnotherWorksheetById=function(t){return w("Worksheet._GetAnotherWorksheetById",v,"1.7",O),C(e.Worksheet,this,"_GetAnotherWorksheetById",0,[t],!1,!1,null,0)},n.prototype._GetSheetById=function(t){return w("Worksheet._GetSheetById",v,"1.7",O),C(e.Worksheet,this,"_GetSheetById",1,[t],!1,!1,null,4)},n.prototype._RegisterActivatedEvent=function(){w("Worksheet._RegisterActivatedEvent",v,"1.7",O),E(this,"_RegisterActivatedEvent",0,[],0,0)},n.prototype._RegisterCalculatedEvent=function(){w("Worksheet._RegisterCalculatedEvent",v,"1.8",O),E(this,"_RegisterCalculatedEvent",0,[],0,0)},n.prototype._RegisterColumnSortedEvent=function(){w("Worksheet._RegisterColumnSortedEvent",v,"1.10",O),E(this,"_RegisterColumnSortedEvent",0,[],0,0)},n.prototype._RegisterDataChangedEvent=function(){w("Worksheet._RegisterDataChangedEvent",v,"1.7",O),E(this,"_RegisterDataChangedEvent",0,[],V(2,"ExcelApiUndo","1.6"),0)},n.prototype._RegisterDeactivatedEvent=function(){w("Worksheet._RegisterDeactivatedEvent",v,"1.7",O),E(this,"_RegisterDeactivatedEvent",0,[],0,0)},n.prototype._RegisterEventNameChanged=function(){w("Worksheet._RegisterEventNameChanged","ExcelApiOnline","1.1",O),E(this,"_RegisterEventNameChanged",0,[],0,0)},n.prototype._RegisterEventVisibilityChanged=function(){w("Worksheet._RegisterEventVisibilityChanged","ExcelApiOnline","1.1",O),E(this,"_RegisterEventVisibilityChanged",0,[],0,0)},n.prototype._RegisterFormatChangedEvent=function(){w("Worksheet._RegisterFormatChangedEvent",v,"1.9",O),E(this,"_RegisterFormatChangedEvent",0,[],0,0)},n.prototype._RegisterFormulaChangedEvent=function(){w("Worksheet._RegisterFormulaChangedEvent",v,"1.13",O),E(this,"_RegisterFormulaChangedEvent",1,[],0,0)},n.prototype._RegisterProtectionChangedEvent=function(){w("Worksheet._RegisterProtectionChangedEvent",v,"1.14",O),E(this,"_RegisterProtectionChangedEvent",1,[],0,0)},n.prototype._RegisterRowHiddenChangedEvent=function(){w("Worksheet._RegisterRowHiddenChangedEvent",v,"1.11",O),E(this,"_RegisterRowHiddenChangedEvent",0,[],0,0)},n.prototype._RegisterRowSortedEvent=function(){w("Worksheet._RegisterRowSortedEvent",v,"1.10",O),E(this,"_RegisterRowSortedEvent",0,[],0,0)},n.prototype._RegisterSelectionChangedEvent=function(){w("Worksheet._RegisterSelectionChangedEvent",v,"1.7",O),E(this,"_RegisterSelectionChangedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._RegisterSingleClickedEvent=function(){w("Worksheet._RegisterSingleClickedEvent",v,"1.10",O),E(this,"_RegisterSingleClickedEvent",0,[],0,0)},n.prototype._UnregisterActivatedEvent=function(){w("Worksheet._UnregisterActivatedEvent",v,"1.7",O),E(this,"_UnregisterActivatedEvent",0,[],0,0)},n.prototype._UnregisterCalculatedEvent=function(){w("Worksheet._UnregisterCalculatedEvent",v,"1.8",O),E(this,"_UnregisterCalculatedEvent",0,[],0,0)},n.prototype._UnregisterColumnSortedEvent=function(){w("Worksheet._UnregisterColumnSortedEvent",v,"1.10",O),E(this,"_UnregisterColumnSortedEvent",0,[],0,0)},n.prototype._UnregisterDataChangedEvent=function(){w("Worksheet._UnregisterDataChangedEvent",v,"1.7",O),E(this,"_UnregisterDataChangedEvent",0,[],V(2,"ExcelApiUndo","1.6"),0)},n.prototype._UnregisterDeactivatedEvent=function(){w("Worksheet._UnregisterDeactivatedEvent",v,"1.7",O),E(this,"_UnregisterDeactivatedEvent",0,[],0,0)},n.prototype._UnregisterEventNameChanged=function(){w("Worksheet._UnregisterEventNameChanged","ExcelApiOnline","1.1",O),E(this,"_UnregisterEventNameChanged",0,[],0,0)},n.prototype._UnregisterEventVisibilityChanged=function(){w("Worksheet._UnregisterEventVisibilityChanged","ExcelApiOnline","1.1",O),E(this,"_UnregisterEventVisibilityChanged",0,[],0,0)},n.prototype._UnregisterFormatChangedEvent=function(){w("Worksheet._UnregisterFormatChangedEvent",v,"1.9",O),E(this,"_UnregisterFormatChangedEvent",0,[],0,0)},n.prototype._UnregisterFormulaChangedEvent=function(){w("Worksheet._UnregisterFormulaChangedEvent",v,"1.13",O),E(this,"_UnregisterFormulaChangedEvent",1,[],0,0)},n.prototype._UnregisterProtectionChangedEvent=function(){w("Worksheet._UnregisterProtectionChangedEvent",v,"1.14",O),E(this,"_UnregisterProtectionChangedEvent",1,[],0,0)},n.prototype._UnregisterRowHiddenChangedEvent=function(){w("Worksheet._UnregisterRowHiddenChangedEvent",v,"1.11",O),E(this,"_UnregisterRowHiddenChangedEvent",0,[],0,0)},n.prototype._UnregisterRowSortedEvent=function(){w("Worksheet._UnregisterRowSortedEvent",v,"1.10",O),E(this,"_UnregisterRowSortedEvent",0,[],0,0)},n.prototype._UnregisterSelectionChangedEvent=function(){w("Worksheet._UnregisterSelectionChangedEvent",v,"1.7",O),E(this,"_UnregisterSelectionChangedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._UnregisterSingleClickedEvent=function(){w("Worksheet._UnregisterSingleClickedEvent",v,"1.10",O),E(this,"_UnregisterSingleClickedEvent",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.EnableCalculation)||(this._E=n.EnableCalculation),N(n.Id)||(this._I=n.Id),N(n.Name)||(this._N=n.Name),N(n.Position)||(this._Po=n.Position),N(n.ShowGridlines)||(this.m_showGridlines=n.ShowGridlines),N(n.ShowHeadings)||(this.m_showHeadings=n.ShowHeadings),N(n.StandardHeight)||(this._St=n.StandardHeight),N(n.StandardWidth)||(this._Sta=n.StandardWidth),N(n.TabColor)||(this._T=n.TabColor),N(n.TabId)||(this._Ta=n.TabId),N(n.Visibility)||(this._Vi=n.Visibility),L(this,n,["autoFilter","AutoFilter","charts","Charts","comments","Comments","customProperties","CustomProperties","freezePanes","FreezePanes","horizontalPageBreaks","HorizontalPageBreaks","namedSheetViews","NamedSheetViews","names","Names","pageLayout","PageLayout","pivotTables","PivotTables","protection","Protection","shapes","Shapes","slicers","Slicers","tables","Tables","verticalPageBreaks","VerticalPageBreaks","visuals","Visuals"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},Object.defineProperty(n.prototype,"onActivated",{get:function(){var e=this;return w("Worksheet.onActivated",v,"1.7",O),this.m_activated||(this.m_activated=new OfficeExtension.GenericEventHandlers(this.context,this,"Activated",{eventType:11,registerFunc:function(){return e._RegisterActivatedEvent()},unregisterFunc:function(){return e._UnregisterActivatedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetActivated,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_activated},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onCalculated",{get:function(){var e=this;return w("Worksheet.onCalculated",v,"1.8",O),this.m_calculated||(this.m_calculated=new OfficeExtension.GenericEventHandlers(this.context,this,"Calculated",{eventType:16,registerFunc:function(){return e._RegisterCalculatedEvent()},unregisterFunc:function(){return e._UnregisterCalculatedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetCalculated,address:e.address,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_calculated},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onChanged",{get:function(){var e=this;return w("Worksheet.onChanged",v,"1.7",O),this.m_changed||(this.m_changed=new OfficeExtension.GenericEventHandlers(this.context,this,"Changed",{eventType:10,registerFunc:function(){return e._RegisterDataChangedEvent()},unregisterFunc:function(){return e._UnregisterDataChangedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(t){var n=b.Worksheet_Changed_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_changed},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onColumnSorted",{get:function(){var e=this;return w("Worksheet.onColumnSorted",v,"1.10",O),this.m_columnSorted||(this.m_columnSorted=new OfficeExtension.GenericEventHandlers(this.context,this,"ColumnSorted",{eventType:20,registerFunc:function(){return e._RegisterColumnSortedEvent()},unregisterFunc:function(){return e._UnregisterColumnSortedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetColumnSorted,address:e.address,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_columnSorted},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onDeactivated",{get:function(){var e=this;return w("Worksheet.onDeactivated",v,"1.7",O),this.m_deactivated||(this.m_deactivated=new OfficeExtension.GenericEventHandlers(this.context,this,"Deactivated",{eventType:12,registerFunc:function(){return e._RegisterDeactivatedEvent()},unregisterFunc:function(){return e._UnregisterDeactivatedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetDeactivated,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_deactivated},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onFormatChanged",{get:function(){var e=this;return w("Worksheet.onFormatChanged",v,"1.9",O),this.m_formatChanged||(this.m_formatChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"FormatChanged",{eventType:18,registerFunc:function(){return e._RegisterFormatChangedEvent()},unregisterFunc:function(){return e._UnregisterFormatChangedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(t){var n=b.Worksheet_FormatChanged_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_formatChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onFormulaChanged",{get:function(){var e=this;return w("Worksheet.onFormulaChanged",v,"1.13",O),this.m_formulaChanged||(this.m_formulaChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"FormulaChanged",{eventType:23,registerFunc:function(){return e._RegisterFormulaChangedEvent()},unregisterFunc:function(){return e._UnregisterFormulaChangedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(t){var n=b.Worksheet_FormulaChanged_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_formulaChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onNameChanged",{get:function(){var e=this;return w("Worksheet.onNameChanged","ExcelApiOnline","1.1",O),this.m_nameChanged||(this.m_nameChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"NameChanged",{eventType:25,registerFunc:function(){return e._RegisterEventNameChanged()},unregisterFunc:function(){return e._UnregisterEventNameChanged()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetNameChanged,nameAfter:e.nameAfter,nameBefore:e.nameBefore,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_nameChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onProtectionChanged",{get:function(){var e=this;return w("Worksheet.onProtectionChanged",v,"1.14",O),this.m_protectionChanged||(this.m_protectionChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"ProtectionChanged",{eventType:24,registerFunc:function(){return e._RegisterProtectionChangedEvent()},unregisterFunc:function(){return e._UnregisterProtectionChangedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetProtectionChanged,isProtected:e.isProtected,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_protectionChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onRowHiddenChanged",{get:function(){var e=this;return w("Worksheet.onRowHiddenChanged",v,"1.11",O),this.m_rowHiddenChanged||(this.m_rowHiddenChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"RowHiddenChanged",{eventType:22,registerFunc:function(){return e._RegisterRowHiddenChangedEvent()},unregisterFunc:function(){return e._UnregisterRowHiddenChangedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetRowHiddenChanged,address:e.address,changeType:e.changeType,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_rowHiddenChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onRowSorted",{get:function(){var e=this;return w("Worksheet.onRowSorted",v,"1.10",O),this.m_rowSorted||(this.m_rowSorted=new OfficeExtension.GenericEventHandlers(this.context,this,"RowSorted",{eventType:19,registerFunc:function(){return e._RegisterRowSortedEvent()},unregisterFunc:function(){return e._UnregisterRowSortedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetRowSorted,address:e.address,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_rowSorted},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onSelectionChanged",{get:function(){var e=this;return w("Worksheet.onSelectionChanged",v,"1.7",O),this.m_selectionChanged||(this.m_selectionChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"SelectionChanged",{eventType:14,registerFunc:function(){return e._RegisterSelectionChangedEvent()},unregisterFunc:function(){return e._UnregisterSelectionChangedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetSelectionChanged,address:e.address,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_selectionChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onSingleClicked",{get:function(){var e=this;return w("Worksheet.onSingleClicked",v,"1.10",O),this.m_singleClicked||(this.m_singleClicked=new OfficeExtension.GenericEventHandlers(this.context,this,"SingleClicked",{eventType:21,registerFunc:function(){return e._RegisterSingleClickedEvent()},unregisterFunc:function(){return e._UnregisterSingleClickedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetSingleClicked,address:e.address,offsetX:e.offsetX,offsetY:e.offsetY,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_singleClicked},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onVisibilityChanged",{get:function(){var e=this;return w("Worksheet.onVisibilityChanged","ExcelApiOnline","1.1",O),this.m_visibilityChanged||(this.m_visibilityChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"VisibilityChanged",{eventType:26,registerFunc:function(){return e._RegisterEventVisibilityChanged()},unregisterFunc:function(){return e._UnregisterEventVisibilityChanged()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetVisibilityChanged,source:e.source,visibilityAfter:e.visibilityAfter,visibilityBefore:e.visibilityBefore,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_visibilityChanged},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{enableCalculation:this._E,id:this._I,name:this._N,position:this._Po,showGridlines:this.m_showGridlines,showHeadings:this.m_showHeadings,standardHeight:this._St,standardWidth:this._Sta,tabColor:this._T,tabId:this._Ta,visibility:this._Vi},{autoFilter:this._A,charts:this.m_charts,comments:this.m_comments,customProperties:this._C,horizontalPageBreaks:this._Ho,names:this.m_names,pageLayout:this._P,pivotTables:this.m_pivotTables,protection:this._Pr,shapes:this.m_shapes,slicers:this._S,tables:this.m_tables,verticalPageBreaks:this._V,visuals:this.m_visuals})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Worksheet=ue,function(t){t.Worksheet_Charts_Get=function(e,t){t._ParentObject=e},t.Worksheet_Comments_Get=function(e,t){t._ParentObject=e},t.Worksheet_Names_Get=function(e,t){t._ParentObject=e},t.Worksheet_PivotTables_Get=function(e,t){t._ParentObject=e},t.Worksheet_Shapes_Get=function(e,t){t._ParentObject=e},t.Worksheet_ShowGridlines_Set=function(e,t){return e.m_showGridlines=t,I(e,"Gridlines",t,0),{handled:!0}},t.Worksheet_ShowHeadings_Set=function(e,t){return e.m_showHeadings=t,I(e,"Headings",t,0),{handled:!0}},t.Worksheet_Tables_Get=function(e,t){t._ParentObject=e},t.Worksheet_Visuals_Get=function(e,t){t._ParentObject=e},t.Worksheet_Changed_EventArgsTransform=function(t,n){var r,i,o,s,a=n;return null==a.valueBefore&&null==a.valueAfter||(r={valueBefore:a.valueBefore,valueAfter:a.valueAfter,valueTypeBefore:a.valueTypeBefore,valueTypeAfter:a.valueTypeAfter}),a.shiftDirection==e.InsertDeleteCellsShiftDirection.shiftCellLeft?i=e.DeleteShiftDirection.left:a.shiftDirection==e.InsertDeleteCellsShiftDirection.shiftCellUp?i=e.DeleteShiftDirection.up:a.shiftDirection==e.InsertDeleteCellsShiftDirection.shiftCellRight?o=e.InsertShiftDirection.right:a.shiftDirection==e.InsertDeleteCellsShiftDirection.shiftCellDown&&(o=e.InsertShiftDirection.down),a.shiftDirection!=e.InsertDeleteCellsShiftDirection.none&&(s={deleteShiftDirection:i,insertShiftDirection:o}),{type:e.EventType.worksheetChanged,changeType:a.changeType,source:a.source,worksheetId:t.id,address:a.address,getRange:function(e){return w("WorksheetChangedEventArgs.getRange",v,"1.8",O),e.workbook._GetRangeForEventByReferenceId(a.referenceId)},getRangeOrNullObject:function(e){return w("WorksheetChangedEventArgs.getRangeOrNullObject",v,"1.8",O),e.workbook._GetRangeOrNullObjectForEventByReferenceId(a.referenceId)},details:r,triggerSource:a.triggerSource,changeDirectionState:s}},t.Worksheet_FormatChanged_EventArgsTransform=function(t,n){var r=n;return{type:e.EventType.worksheetFormatChanged,source:r.source,worksheetId:t.id,address:r.address,getRange:function(e){return w("WorksheetFormatChangedEventArgs.getRange",v,"1.9",O),e.workbook._GetRangeForEventByReferenceId(r.referenceId)},getRangeOrNullObject:function(e){return w("WorksheetFormatChangedEventArgs.getRangeOrNullObject",v,"1.9",O),e.workbook._GetRangeOrNullObjectForEventByReferenceId(r.referenceId)}}},t.Worksheet_FormulaChanged_EventArgsTransform=function(t,n){var r,i=n;return null!=i.formulaDetails&&(r=i.formulaDetails),{type:e.EventType.worksheetFormulaChanged,source:i.source,worksheetId:i.worksheetId,formulaDetails:r}}}(b=e._CC||(e._CC={}));var ce=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"WorksheetCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"WorksheetCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t){return C(e.Worksheet,this,"Add",0,[t],!1,!0,null,V(2,"ExcelApiUndo","1.2"))},n.prototype.getActiveWorksheet=function(){return C(e.Worksheet,this,"GetActiveWorksheet",1,[],!1,!1,null,4)},n.prototype.getCount=function(e){return w("WorksheetCollection.getCount",v,"1.4",O),E(this,"GetCount",1,[e],4,0)},n.prototype.getFirst=function(t){return w("WorksheetCollection.getFirst",v,"1.5",O),C(e.Worksheet,this,"GetFirst",1,[t],!1,!0,null,4)},n.prototype.getItem=function(t){return P(e.Worksheet,this,[t])},n.prototype.getItemOrNullObject=function(t){return w("WorksheetCollection.getItemOrNullObject",v,"1.4",O),C(e.Worksheet,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype.getLast=function(t){return w("WorksheetCollection.getLast",v,"1.5",O),C(e.Worksheet,this,"GetLast",1,[t],!1,!0,null,4)},n.prototype._RegisterActivatedEvent=function(){b.WorksheetCollection__RegisterActivatedEvent(this).handled||(w("WorksheetCollection._RegisterActivatedEvent",v,"1.7",O),E(this,"_RegisterActivatedEvent",1,[],0,0))},n.prototype._RegisterAddedEvent=function(){b.WorksheetCollection__RegisterAddedEvent(this).handled||(w("WorksheetCollection._RegisterAddedEvent",v,"1.7",O),E(this,"_RegisterAddedEvent",1,[],0,0))},n.prototype._RegisterCalculatedEvent=function(){w("WorksheetCollection._RegisterCalculatedEvent",v,"1.8",O),E(this,"_RegisterCalculatedEvent",0,[],0,0)},n.prototype._RegisterColumnSortedEvent=function(){b.WorksheetCollection__RegisterColumnSortedEvent(this).handled||(w("WorksheetCollection._RegisterColumnSortedEvent",v,"1.10",O),E(this,"_RegisterColumnSortedEvent",1,[],0,0))},n.prototype._RegisterDataChangedEvent=function(){b.WorksheetCollection__RegisterDataChangedEvent(this).handled||(w("WorksheetCollection._RegisterDataChangedEvent",v,"1.9",O),E(this,"_RegisterDataChangedEvent",1,[],0,0))},n.prototype._RegisterDeactivatedEvent=function(){w("WorksheetCollection._RegisterDeactivatedEvent",v,"1.7",O),E(this,"_RegisterDeactivatedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._RegisterDeletedEvent=function(){b.WorksheetCollection__RegisterDeletedEvent(this).handled||(w("WorksheetCollection._RegisterDeletedEvent",v,"1.7",O),E(this,"_RegisterDeletedEvent",1,[],0,0))},n.prototype._RegisterEventMoved=function(){w("WorksheetCollection._RegisterEventMoved","ExcelApiOnline","1.1",O),E(this,"_RegisterEventMoved",0,[],0,0)},n.prototype._RegisterEventNameChanged=function(){w("WorksheetCollection._RegisterEventNameChanged","ExcelApiOnline","1.1",O),E(this,"_RegisterEventNameChanged",1,[],4,0)},n.prototype._RegisterEventVisibilityChanged=function(){w("WorksheetCollection._RegisterEventVisibilityChanged","ExcelApiOnline","1.1",O),E(this,"_RegisterEventVisibilityChanged",1,[],4,0)},n.prototype._RegisterFormatChangedEvent=function(){w("WorksheetCollection._RegisterFormatChangedEvent",v,"1.9",O),E(this,"_RegisterFormatChangedEvent",0,[],0,0)},n.prototype._RegisterFormulaChangedEvent=function(){w("WorksheetCollection._RegisterFormulaChangedEvent",v,"1.13",O),E(this,"_RegisterFormulaChangedEvent",1,[],0,0)},n.prototype._RegisterProtectionChangedEvent=function(){w("WorksheetCollection._RegisterProtectionChangedEvent",v,"1.14",O),E(this,"_RegisterProtectionChangedEvent",1,[],0,0)},n.prototype._RegisterRowHiddenChangedEvent=function(){w("WorksheetCollection._RegisterRowHiddenChangedEvent",v,"1.11",O),E(this,"_RegisterRowHiddenChangedEvent",0,[],0,0)},n.prototype._RegisterRowSortedEvent=function(){b.WorksheetCollection__RegisterRowSortedEvent(this).handled||(w("WorksheetCollection._RegisterRowSortedEvent",v,"1.10",O),E(this,"_RegisterRowSortedEvent",1,[],0,0))},n.prototype._RegisterSelectionChangedEvent=function(){w("WorksheetCollection._RegisterSelectionChangedEvent",v,"1.9",O),E(this,"_RegisterSelectionChangedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._RegisterSingleClickedEvent=function(){w("WorksheetCollection._RegisterSingleClickedEvent",v,"1.10",O),E(this,"_RegisterSingleClickedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._UnregisterActivatedEvent=function(){w("WorksheetCollection._UnregisterActivatedEvent",v,"1.7",O),E(this,"_UnregisterActivatedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._UnregisterAddedEvent=function(){b.WorksheetCollection__UnregisterAddedEvent(this).handled||(w("WorksheetCollection._UnregisterAddedEvent",v,"1.7",O),E(this,"_UnregisterAddedEvent",1,[],0,0))},n.prototype._UnregisterCalculatedEvent=function(){w("WorksheetCollection._UnregisterCalculatedEvent",v,"1.8",O),E(this,"_UnregisterCalculatedEvent",0,[],0,0)},n.prototype._UnregisterColumnSortedEvent=function(){b.WorksheetCollection__UnregisterColumnSortedEvent(this).handled||(w("WorksheetCollection._UnregisterColumnSortedEvent",v,"1.10",O),E(this,"_UnregisterColumnSortedEvent",1,[],0,0))},n.prototype._UnregisterDataChangedEvent=function(){b.WorksheetCollection__UnregisterDataChangedEvent(this).handled||(w("WorksheetCollection._UnregisterDataChangedEvent",v,"1.9",O),E(this,"_UnregisterDataChangedEvent",1,[],0,0))},n.prototype._UnregisterDeactivatedEvent=function(){w("WorksheetCollection._UnregisterDeactivatedEvent",v,"1.7",O),E(this,"_UnregisterDeactivatedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._UnregisterDeletedEvent=function(){b.WorksheetCollection__UnregisterDeletedEvent(this).handled||(w("WorksheetCollection._UnregisterDeletedEvent",v,"1.7",O),E(this,"_UnregisterDeletedEvent",1,[],0,0))},n.prototype._UnregisterEventMoved=function(){w("WorksheetCollection._UnregisterEventMoved","ExcelApiOnline","1.1",O),E(this,"_UnregisterEventMoved",0,[],0,0)},n.prototype._UnregisterEventNameChanged=function(){w("WorksheetCollection._UnregisterEventNameChanged","ExcelApiOnline","1.1",O),E(this,"_UnregisterEventNameChanged",1,[],4,0)},n.prototype._UnregisterEventVisibilityChanged=function(){w("WorksheetCollection._UnregisterEventVisibilityChanged","ExcelApiOnline","1.1",O),E(this,"_UnregisterEventVisibilityChanged",1,[],4,0)},n.prototype._UnregisterFormatChangedEvent=function(){w("WorksheetCollection._UnregisterFormatChangedEvent",v,"1.9",O),E(this,"_UnregisterFormatChangedEvent",0,[],0,0)},n.prototype._UnregisterFormulaChangedEvent=function(){w("WorksheetCollection._UnregisterFormulaChangedEvent",v,"1.13",O),E(this,"_UnregisterFormulaChangedEvent",1,[],0,0)},n.prototype._UnregisterProtectionChangedEvent=function(){w("WorksheetCollection._UnregisterProtectionChangedEvent",v,"1.14",O),E(this,"_UnregisterProtectionChangedEvent",1,[],0,0)},n.prototype._UnregisterRowHiddenChangedEvent=function(){w("WorksheetCollection._UnregisterRowHiddenChangedEvent",v,"1.11",O),E(this,"_UnregisterRowHiddenChangedEvent",0,[],0,0)},n.prototype._UnregisterRowSortedEvent=function(){b.WorksheetCollection__UnregisterRowSortedEvent(this).handled||(w("WorksheetCollection._UnregisterRowSortedEvent",v,"1.10",O),E(this,"_UnregisterRowSortedEvent",1,[],0,0))},n.prototype._UnregisterSelectionChangedEvent=function(){w("WorksheetCollection._UnregisterSelectionChangedEvent",v,"1.9",O),E(this,"_UnregisterSelectionChangedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._UnregisterSingleClickedEvent=function(){w("WorksheetCollection._UnregisterSingleClickedEvent",v,"1.10",O),E(this,"_UnregisterSingleClickedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Worksheet,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Worksheet,!0,i,t,n)}))},Object.defineProperty(n.prototype,"onActivated",{get:function(){var e=this;return w("WorksheetCollection.onActivated",v,"1.7",O),this.m_activated||(this.m_activated=new OfficeExtension.GenericEventHandlers(this.context,this,"Activated",{eventType:11,registerFunc:function(){return e._RegisterActivatedEvent()},unregisterFunc:function(){return e._UnregisterActivatedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetActivated,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_activated},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onAdded",{get:function(){var e=this;return w("WorksheetCollection.onAdded",v,"1.7",O),this.m_added||(this.m_added=new OfficeExtension.GenericEventHandlers(this.context,this,"Added",{eventType:13,registerFunc:function(){return e._RegisterAddedEvent()},unregisterFunc:function(){return e._UnregisterAddedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetAdded,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_added},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onCalculated",{get:function(){var e=this;return w("WorksheetCollection.onCalculated",v,"1.8",O),this.m_calculated||(this.m_calculated=new OfficeExtension.GenericEventHandlers(this.context,this,"Calculated",{eventType:16,registerFunc:function(){return e._RegisterCalculatedEvent()},unregisterFunc:function(){return e._UnregisterCalculatedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetCalculated,address:e.address,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_calculated},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onChanged",{get:function(){var e=this;return w("WorksheetCollection.onChanged",v,"1.9",O),this.m_changed||(this.m_changed=new OfficeExtension.GenericEventHandlers(this.context,this,"Changed",{eventType:10,registerFunc:function(){return e._RegisterDataChangedEvent()},unregisterFunc:function(){return e._UnregisterDataChangedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(t){var n=b.WorksheetCollection_Changed_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_changed},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onColumnSorted",{get:function(){var e=this;return w("WorksheetCollection.onColumnSorted",v,"1.10",O),this.m_columnSorted||(this.m_columnSorted=new OfficeExtension.GenericEventHandlers(this.context,this,"ColumnSorted",{eventType:20,registerFunc:function(){return e._RegisterColumnSortedEvent()},unregisterFunc:function(){return e._UnregisterColumnSortedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetColumnSorted,address:e.address,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_columnSorted},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onDeactivated",{get:function(){var e=this;return w("WorksheetCollection.onDeactivated",v,"1.7",O),this.m_deactivated||(this.m_deactivated=new OfficeExtension.GenericEventHandlers(this.context,this,"Deactivated",{eventType:12,registerFunc:function(){return e._RegisterDeactivatedEvent()},unregisterFunc:function(){return e._UnregisterDeactivatedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetDeactivated,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_deactivated},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onDeleted",{get:function(){var e=this;return w("WorksheetCollection.onDeleted",v,"1.7",O),this.m_deleted||(this.m_deleted=new OfficeExtension.GenericEventHandlers(this.context,this,"Deleted",{eventType:15,registerFunc:function(){return e._RegisterDeletedEvent()},unregisterFunc:function(){return e._UnregisterDeletedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetDeleted,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_deleted},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onFormatChanged",{get:function(){var e=this;return w("WorksheetCollection.onFormatChanged",v,"1.9",O),this.m_formatChanged||(this.m_formatChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"FormatChanged",{eventType:18,registerFunc:function(){return e._RegisterFormatChangedEvent()},unregisterFunc:function(){return e._UnregisterFormatChangedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(t){var n=b.WorksheetCollection_FormatChanged_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_formatChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onFormulaChanged",{get:function(){var e=this;return w("WorksheetCollection.onFormulaChanged",v,"1.13",O),this.m_formulaChanged||(this.m_formulaChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"FormulaChanged",{eventType:23,registerFunc:function(){return e._RegisterFormulaChangedEvent()},unregisterFunc:function(){return e._UnregisterFormulaChangedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(t){var n=b.WorksheetCollection_FormulaChanged_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_formulaChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onMoved",{get:function(){var e=this;return w("WorksheetCollection.onMoved","ExcelApiOnline","1.1",O),this.m_moved||(this.m_moved=new OfficeExtension.GenericEventHandlers(this.context,this,"Moved",{eventType:27,registerFunc:function(){return e._RegisterEventMoved()},unregisterFunc:function(){return e._UnregisterEventMoved()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetMoved,positionAfter:e.positionAfter,positionBefore:e.positionBefore,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_moved},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onNameChanged",{get:function(){var e=this;return w("WorksheetCollection.onNameChanged","ExcelApiOnline","1.1",O),this.m_nameChanged||(this.m_nameChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"NameChanged",{eventType:25,registerFunc:function(){return e._RegisterEventNameChanged()},unregisterFunc:function(){return e._UnregisterEventNameChanged()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetNameChanged,nameAfter:e.nameAfter,nameBefore:e.nameBefore,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_nameChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onProtectionChanged",{get:function(){var e=this;return w("WorksheetCollection.onProtectionChanged",v,"1.14",O),this.m_protectionChanged||(this.m_protectionChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"ProtectionChanged",{eventType:24,registerFunc:function(){return e._RegisterProtectionChangedEvent()},unregisterFunc:function(){return e._UnregisterProtectionChangedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetProtectionChanged,isProtected:e.isProtected,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_protectionChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onRowHiddenChanged",{get:function(){var e=this;return w("WorksheetCollection.onRowHiddenChanged",v,"1.11",O),this.m_rowHiddenChanged||(this.m_rowHiddenChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"RowHiddenChanged",{eventType:22,registerFunc:function(){return e._RegisterRowHiddenChangedEvent()},unregisterFunc:function(){return e._UnregisterRowHiddenChangedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetRowHiddenChanged,address:e.address,changeType:e.changeType,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_rowHiddenChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onRowSorted",{get:function(){var e=this;return w("WorksheetCollection.onRowSorted",v,"1.10",O),this.m_rowSorted||(this.m_rowSorted=new OfficeExtension.GenericEventHandlers(this.context,this,"RowSorted",{eventType:19,registerFunc:function(){return e._RegisterRowSortedEvent()},unregisterFunc:function(){return e._UnregisterRowSortedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetRowSorted,address:e.address,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_rowSorted},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onSelectionChanged",{get:function(){var e=this;return w("WorksheetCollection.onSelectionChanged",v,"1.9",O),this.m_selectionChanged||(this.m_selectionChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"SelectionChanged",{eventType:14,registerFunc:function(){return e._RegisterSelectionChangedEvent()},unregisterFunc:function(){return e._UnregisterSelectionChangedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetSelectionChanged,address:e.address,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_selectionChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onSingleClicked",{get:function(){var e=this;return w("WorksheetCollection.onSingleClicked",v,"1.10",O),this.m_singleClicked||(this.m_singleClicked=new OfficeExtension.GenericEventHandlers(this.context,this,"SingleClicked",{eventType:21,registerFunc:function(){return e._RegisterSingleClickedEvent()},unregisterFunc:function(){return e._UnregisterSingleClickedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetSingleClicked,address:e.address,offsetX:e.offsetX,offsetY:e.offsetY,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_singleClicked},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onVisibilityChanged",{get:function(){var e=this;return w("WorksheetCollection.onVisibilityChanged","ExcelApiOnline","1.1",O),this.m_visibilityChanged||(this.m_visibilityChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"VisibilityChanged",{eventType:26,registerFunc:function(){return e._RegisterEventVisibilityChanged()},unregisterFunc:function(){return e._UnregisterEventVisibilityChanged()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.worksheetVisibilityChanged,source:e.source,visibilityAfter:e.visibilityAfter,visibilityBefore:e.visibilityBefore,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_visibilityChanged},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Worksheet,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.WorksheetCollection=ce,function(t){function n(e,t,n){return!f("OfficeOnline")&&!c(1.12)||f("OfficeOnline")&&!c(1.11)?(w("WorksheetCollection."+t,v,n,O),E(e,t,0,[],0,0),{handled:!0}):{handled:!1}}t.WorksheetCollection__RegisterActivatedEvent=function(e){return c(1.9)?{handled:!1}:(w("WorksheetCollection._RegisterActivatedEvent",v,"1.7",O),E(e,"_RegisterActivatedEvent",0,[],0,0),{handled:!0})},t.WorksheetCollection__RegisterAddedEvent=function(e){return n(e,"_RegisterAddedEvent","1.7")},t.WorksheetCollection__RegisterColumnSortedEvent=function(e){return n(e,"_RegisterColumnSortedEvent","1.10")},t.WorksheetCollection__RegisterDataChangedEvent=function(e){return n(e,"_RegisterDataChangedEvent","1.9")},t.WorksheetCollection__RegisterDeletedEvent=function(e){return n(e,"_RegisterDeletedEvent","1.7")},t.WorksheetCollection__RegisterRowSortedEvent=function(e){return n(e,"_RegisterRowSortedEvent","1.10")},t.WorksheetCollection__UnregisterAddedEvent=function(e){return n(e,"_UnregisterAddedEvent","1.7")},t.WorksheetCollection__UnregisterColumnSortedEvent=function(e){return n(e,"_UnregisterColumnSortedEvent","1.10")},t.WorksheetCollection__UnregisterDataChangedEvent=function(e){return n(e,"_UnregisterDataChangedEvent","1.9")},t.WorksheetCollection__UnregisterDeletedEvent=function(e){return n(e,"_UnregisterDeletedEvent","1.7")},t.WorksheetCollection__UnregisterRowSortedEvent=function(e){return n(e,"_UnregisterRowSortedEvent","1.10")},t.WorksheetCollection_Changed_EventArgsTransform=function(t,n){var r,i,o,s,a=n;return null==a.valueBefore&&null==a.valueAfter||(r={valueBefore:a.valueBefore,valueAfter:a.valueAfter,valueTypeBefore:a.valueTypeBefore,valueTypeAfter:a.valueTypeAfter}),a.shiftDirection==e.InsertDeleteCellsShiftDirection.shiftCellLeft?i=e.DeleteShiftDirection.left:a.shiftDirection==e.InsertDeleteCellsShiftDirection.shiftCellUp?i=e.DeleteShiftDirection.up:a.shiftDirection==e.InsertDeleteCellsShiftDirection.shiftCellRight?o=e.InsertShiftDirection.right:a.shiftDirection==e.InsertDeleteCellsShiftDirection.shiftCellDown&&(o=e.InsertShiftDirection.down),a.shiftDirection!=e.InsertDeleteCellsShiftDirection.none&&(s={deleteShiftDirection:i,insertShiftDirection:o}),{type:e.EventType.worksheetChanged,changeType:a.changeType,source:a.source,worksheetId:a.worksheetId,address:a.address,getRange:function(e){return w("WorksheetChangedEventArgs.getRange",v,"1.9",O),e.workbook._GetRangeForEventByReferenceId(a.referenceId)},getRangeOrNullObject:function(e){return w("WorksheetChangedEventArgs.getRangeOrNullObject",v,"1.9",O),e.workbook._GetRangeOrNullObjectForEventByReferenceId(a.referenceId)},details:r,triggerSource:a.triggerSource,changeDirectionState:s}},t.WorksheetCollection_FormatChanged_EventArgsTransform=function(t,n){var r=n;return{type:e.EventType.worksheetFormatChanged,source:r.source,worksheetId:r.worksheetId,address:r.address,getRange:function(e){return w("WorksheetFormatChangedEventArgs.getRange",v,"1.9",O),e.workbook._GetRangeForEventByReferenceId(r.referenceId)},getRangeOrNullObject:function(e){return w("WorksheetFormatChangedEventArgs.getRangeOrNullObject",v,"1.9",O),e.workbook._GetRangeOrNullObjectForEventByReferenceId(r.referenceId)}}},t.WorksheetCollection_FormulaChanged_EventArgsTransform=function(t,n){var r,i=n;return null!=i.formulaDetails&&(r=i.formulaDetails),{type:e.EventType.worksheetFormulaChanged,source:i.source,worksheetId:i.worksheetId,formulaDetails:r}}}(b=e._CC||(e._CC={}));var pe="WorksheetProtection",fe=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"WorksheetProtection"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["protected","options"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Protected","Options"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"options",{get:function(){return T("options",this._O,pe,this._isNull),this._O},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"protected",{get:function(){return T("protected",this._P,pe,this._isNull),this._P},enumerable:!0,configurable:!0}),t.prototype.protect=function(e,t){b.WorksheetProtection_Protect(this,e,t).handled||E(this,"Protect",0,[e,t],0,0)},t.prototype.unprotect=function(e){E(this,"Unprotect",0,[e],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Options)||(this._O=n.Options),N(n.Protected)||(this._P=n.Protected)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{options:this._O,protected:this._P},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.WorksheetProtection=fe,function(e){e.WorksheetProtection_Protect=function(e,t,n){return _({mac:{desiredMajor:16,desiredMinor:7,desiredBuild:17101600},ios:{desiredMajor:2,desiredMinor:7,desiredBuild:1016},general:{desiredMajor:16,desiredMinor:0,desiredBuild:8716}})?(E(e,"Protect",0,[t],0,0),{handled:!0}):{handled:!1}}}(b=e._CC||(e._CC={}));var he=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"WorksheetFreezePanes"},enumerable:!0,configurable:!0}),n.prototype.freezeAt=function(e){E(this,"FreezeAt",0,[e],0,0)},n.prototype.freezeColumns=function(e){E(this,"FreezeColumns",0,[e],0,0)},n.prototype.freezeRows=function(e){E(this,"FreezeRows",0,[e],0,0)},n.prototype.getLocation=function(){return C(e.Range,this,"GetLocation",1,[],!1,!0,null,4)},n.prototype.getLocationOrNullObject=function(){return C(e.Range,this,"GetLocationOrNullObject",1,[],!1,!0,null,4)},n.prototype.unfreeze=function(){E(this,"Unfreeze",0,[],0,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),x(e))||U(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{})},n}(OfficeExtension.ClientObject);e.WorksheetFreezePanes=he;var de="Range",ye=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Range"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["numberFormat","numberFormatLocal","values","text","formulas","formulasLocal","rowIndex","columnIndex","rowCount","columnCount","address","addressLocal","cellCount","_ReferenceId","valueTypes","formulasR1C1","hidden","rowHidden","columnHidden","isEntireColumn","isEntireRow","hyperlink","style","linkedDataTypeState","hasSpill","top","left","height","width","savedAsArray","numberFormatCategories"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["NumberFormat","NumberFormatLocal","Values","Text","Formulas","FormulasLocal","RowIndex","ColumnIndex","RowCount","ColumnCount","Address","AddressLocal","CellCount","_ReferenceId","ValueTypes","FormulasR1C1","Hidden","RowHidden","ColumnHidden","IsEntireColumn","IsEntireRow","Hyperlink","Style","LinkedDataTypeState","HasSpill","Top","Left","Height","Width","SavedAsArray","NumberFormatCategories"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!1,!0,!0,!1,!1,!1,!1,!1,!1,!1,!1,!1,!0,!1,!0,!0,!1,!1,!0,!0,!1,!1,!1,!1,!1,!1,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format","worksheet","sort","conditionalFormats","dataValidation"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"conditionalFormats",{get:function(){return w("Range.conditionalFormats",v,"1.6",O),this._Con||(this._Con=S(e.ConditionalFormatCollection,this,"ConditionalFormats",!0,4)),this._Con},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"dataValidation",{get:function(){return w("Range.dataValidation",v,"1.8",O),this._D||(this._D=S(e.DataValidation,this,"DataValidation",!1,4)),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.RangeFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"sort",{get:function(){return w("Range.sort",v,"1.2",O),this._So||(this._So=S(e.RangeSort,this,"Sort",!1,4)),this._So},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"worksheet",{get:function(){return this._Wo||(this._Wo=S(e.Worksheet,this,"Worksheet",!1,4)),this._Wo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"address",{get:function(){return T("address",this._A,de,this._isNull),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"addressLocal",{get:function(){return T("addressLocal",this._Ad,de,this._isNull),this._Ad},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"cellCount",{get:function(){return T("cellCount",this._C,de,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"columnCount",{get:function(){return T("columnCount",this._Co,de,this._isNull),this._Co},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"columnHidden",{get:function(){return T("columnHidden",this._Col,de,this._isNull),w("Range.columnHidden",v,"1.2",O),this._Col},set:function(e){this._Col=e,I(this,"ColumnHidden",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"columnIndex",{get:function(){return T("columnIndex",this._Colu,de,this._isNull),this._Colu},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"formulas",{get:function(){return T("formulas",this.m_formulas,de,this._isNull),this.m_formulas},set:function(e){b.Range_Formulas_Set(this,e).handled||(this.m_formulas=e,I(this,"Formulas",e,0))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"formulasLocal",{get:function(){return T("formulasLocal",this.m_formulasLocal,de,this._isNull),this.m_formulasLocal},set:function(e){b.Range_FormulasLocal_Set(this,e).handled||(this.m_formulasLocal=e,I(this,"FormulasLocal",e,0))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"formulasR1C1",{get:function(){return T("formulasR1C1",this.m_formulasR1C1,de,this._isNull),w("Range.formulasR1C1",v,"1.2",O),this.m_formulasR1C1},set:function(e){b.Range_FormulasR1C1_Set(this,e).handled||(this.m_formulasR1C1=e,I(this,"FormulasR1C1",e,0))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"hasSpill",{get:function(){return T("hasSpill",this._H,de,this._isNull),w("Range.hasSpill",v,"1.12",O),this._H},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"height",{get:function(){return T("height",this._He,de,this._isNull),w("Range.height",v,"1.10",O),this._He},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"hidden",{get:function(){return T("hidden",this._Hi,de,this._isNull),w("Range.hidden",v,"1.2",O),this._Hi},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"hyperlink",{get:function(){return T("hyperlink",this._Hy,de,this._isNull),w("Range.hyperlink",v,"1.7",O),this._Hy},set:function(e){this._Hy=e,I(this,"Hyperlink",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isEntireColumn",{get:function(){return T("isEntireColumn",this.m_isEntireColumn,de,this._isNull),w("Range.isEntireColumn",v,"1.7",O),this.m_isEntireColumn},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isEntireRow",{get:function(){return T("isEntireRow",this.m_isEntireRow,de,this._isNull),w("Range.isEntireRow",v,"1.7",O),this.m_isEntireRow},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"left",{get:function(){return T("left",this._L,de,this._isNull),w("Range.left",v,"1.10",O),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"linkedDataTypeState",{get:function(){return T("linkedDataTypeState",this._Li,de,this._isNull),w("Range.linkedDataTypeState",v,"1.9",O),this._Li},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormat",{get:function(){return T("numberFormat",this.m_numberFormat,de,this._isNull),this.m_numberFormat},set:function(e){b.Range_NumberFormat_Set(this,e).handled||(this.m_numberFormat=e,I(this,"NumberFormat",e,V(2,"ExcelApiUndo","1.7")))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormatCategories",{get:function(){return T("numberFormatCategories",this._N,de,this._isNull),w("Range.numberFormatCategories",v,"1.12",O),this._N},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormatLocal",{get:function(){return T("numberFormatLocal",this._Nu,de,this._isNull),w("Range.numberFormatLocal",v,"1.7",O),this._Nu},set:function(e){this._Nu=e,I(this,"NumberFormatLocal",e,V(2,"ExcelApiUndo","1.7"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rowCount",{get:function(){return T("rowCount",this._R,de,this._isNull),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rowHidden",{get:function(){return T("rowHidden",this._Ro,de,this._isNull),w("Range.rowHidden",v,"1.2",O),this._Ro},set:function(e){this._Ro=e,I(this,"RowHidden",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rowIndex",{get:function(){return T("rowIndex",this._Row,de,this._isNull),this._Row},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"savedAsArray",{get:function(){return T("savedAsArray",this._S,de,this._isNull),w("Range.savedAsArray",v,"1.12",O),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"style",{get:function(){return T("style",this._St,de,this._isNull),w("Range.style",v,"1.7",O),this._St},set:function(e){this._St=e,I(this,"Style",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"text",{get:function(){return T("text",this._T,de,this._isNull),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"top",{get:function(){return T("top",this._To,de,this._isNull),w("Range.top",v,"1.10",O),this._To},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"valueTypes",{get:function(){return T("valueTypes",this._V,de,this._isNull),this._V},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"values",{get:function(){return T("values",this.m_values,de,this._isNull),this.m_values},set:function(e){b.Range_Values_Set(this,e).handled||(this.m_values=e,I(this,"Values",e,V(2,"ExcelApiUndo","1.7")))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"width",{get:function(){return T("width",this._W,de,this._isNull),w("Range.width",v,"1.10",O),this._W},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_ReferenceId",{get:function(){return T("_ReferenceId",this.__R,de,this._isNull),this.__R},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["numberFormat","numberFormatLocal","values","formulas","formulasLocal","formulasR1C1","rowHidden","columnHidden","hyperlink","style"],["format","dataValidation"],["conditionalFormats","sort","worksheet"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.autoFill=function(e,t){w("Range.autoFill",v,"1.9",O),E(this,"AutoFill",0,[e,t],0,0)},n.prototype.calculate=function(){w("Range.calculate",v,"1.6",O),E(this,"Calculate",0,[],0,0)},n.prototype.clear=function(e){E(this,"Clear",0,[e],0,0)},n.prototype.convertDataTypeToText=function(){w("Range.convertDataTypeToText",v,"1.9",O),E(this,"ConvertDataTypeToText",0,[],0,0)},n.prototype.convertToLinkedDataType=function(e,t){w("Range.convertToLinkedDataType",v,"1.9",O),E(this,"ConvertToLinkedDataType",0,[e,t],0,0)},n.prototype.copyFrom=function(e,t,n,r){w("Range.copyFrom",v,"1.9",O),E(this,"CopyFrom",0,[e,t,n,r],0,0)},n.prototype.delete=function(e){E(this,"Delete",0,[e],0,0)},n.prototype.find=function(t,n){return w("Range.find",v,"1.9",O),C(e.Range,this,"Find",1,[t,n],!1,!0,null,4)},n.prototype.findOrNullObject=function(t,n){return w("Range.findOrNullObject",v,"1.9",O),C(e.Range,this,"FindOrNullObject",1,[t,n],!1,!0,null,4)},n.prototype.flashFill=function(){w("Range.flashFill",v,"1.9",O),E(this,"FlashFill",0,[],0,0)},n.prototype.getAbsoluteResizedRange=function(t,n){return w("Range.getAbsoluteResizedRange",v,"1.7",O),C(e.Range,this,"GetAbsoluteResizedRange",1,[t,n],!1,!0,null,4)},n.prototype.getBoundingRect=function(t){return C(e.Range,this,"GetBoundingRect",1,[t],!1,!0,null,4)},n.prototype.getCell=function(t,n){return C(e.Range,this,"GetCell",1,[t,n],!1,!0,null,4)},n.prototype.getCellProperties=function(e){return w("Range.getCellProperties",v,"1.9",O),E(this,"GetCellProperties",0,[e],0,0)},n.prototype.getColumn=function(t){return C(e.Range,this,"GetColumn",1,[t],!1,!0,null,4)},n.prototype.getColumnProperties=function(e){return w("Range.getColumnProperties",v,"1.9",O),E(this,"GetColumnProperties",0,[e],0,0)},n.prototype.getColumnsAfter=function(t){var n=b.Range_GetColumnsAfter(this,t),r=n.handled,i=n.result;return r?i:(w("Range.getColumnsAfter",v,"1.3",O),C(e.Range,this,"GetColumnsAfter",1,[t],!1,!0,null,4))},n.prototype.getColumnsBefore=function(t){var n=b.Range_GetColumnsBefore(this,t),r=n.handled,i=n.result;return r?i:(w("Range.getColumnsBefore",v,"1.3",O),C(e.Range,this,"GetColumnsBefore",1,[t],!1,!0,null,4))},n.prototype.getDataClassificationIds=function(){return w("Range.getDataClassificationIds","ExcelApiOnline","1.1",O),E(this,"GetDataClassificationIds",1,[],4,0)},n.prototype.getDependents=function(){return w("Range.getDependents",v,"1.15",O),C(e.WorkbookRangeAreas,this,"GetDependents",1,[],!1,!0,null,4)},n.prototype.getDirectDependents=function(){return w("Range.getDirectDependents",v,"1.13",O),C(e.WorkbookRangeAreas,this,"GetDirectDependents",1,[],!1,!0,null,4)},n.prototype.getDirectPrecedents=function(){return w("Range.getDirectPrecedents",v,"1.12",O),C(e.WorkbookRangeAreas,this,"GetDirectPrecedents",1,[],!1,!0,null,4)},n.prototype.getEntireColumn=function(){return C(e.Range,this,"GetEntireColumn",1,[],!1,!0,null,4)},n.prototype.getEntireRow=function(){return C(e.Range,this,"GetEntireRow",1,[],!1,!0,null,4)},n.prototype.getExtendedRange=function(t,n){return w("Range.getExtendedRange",v,"1.13",O),C(e.Range,this,"GetExtendedRange",1,[t,n],!1,!0,null,4)},n.prototype.getImage=function(){return w("Range.getImage",v,"1.7",O),E(this,"GetImage",1,[],4,0)},n.prototype.getIntersection=function(t){return C(e.Range,this,"GetIntersection",1,[t],!1,!0,null,4)},n.prototype.getIntersectionOrNullObject=function(t){return w("Range.getIntersectionOrNullObject",v,"1.4",O),C(e.Range,this,"GetIntersectionOrNullObject",1,[t],!1,!0,null,4)},n.prototype.getLastCell=function(){return C(e.Range,this,"GetLastCell",1,[],!1,!0,null,4)},n.prototype.getLastColumn=function(){return C(e.Range,this,"GetLastColumn",1,[],!1,!0,null,4)},n.prototype.getLastRow=function(){return C(e.Range,this,"GetLastRow",1,[],!1,!0,null,4)},n.prototype.getMergedAreas=function(){return w("Range.getMergedAreas","ExcelApiOnline","1.1",O),C(e.RangeAreas,this,"GetMergedAreas",1,[],!1,!0,null,4)},n.prototype.getMergedAreasOrNullObject=function(){return w("Range.getMergedAreasOrNullObject",v,"1.13",O),C(e.RangeAreas,this,"GetMergedAreasOrNullObject",0,[],!1,!1,null,V(2,"ExcelApiUndo","1.7"))},n.prototype.getNumberFormatProperties=function(){return w("Range.getNumberFormatProperties",v,"1.15",O),C(e.NumberFormatPropertyCollection,this,"GetNumberFormatProperties",1,[],!0,!1,null,4)},n.prototype.getOffsetRange=function(t,n){return C(e.Range,this,"GetOffsetRange",1,[t,n],!1,!0,null,4)},n.prototype.getPivotTables=function(t){return w("Range.getPivotTables",v,"1.12",O),C(e.PivotTableScopedCollection,this,"GetPivotTables",1,[t],!0,!1,null,4)},n.prototype.getPrecedents=function(){return w("Range.getPrecedents",v,"1.14",O),C(e.WorkbookRangeAreas,this,"GetPrecedents",1,[],!1,!0,null,4)},n.prototype.getRangeEdge=function(t,n){return w("Range.getRangeEdge",v,"1.13",O),C(e.Range,this,"GetRangeEdge",1,[t,n],!1,!0,null,4)},n.prototype.getResizedRange=function(t,n){var r=b.Range_GetResizedRange(this,t,n),i=r.handled,o=r.result;return i?o:(w("Range.getResizedRange",v,"1.3",O),C(e.Range,this,"GetResizedRange",1,[t,n],!1,!0,null,4))},n.prototype.getRow=function(t){return C(e.Range,this,"GetRow",1,[t],!1,!0,null,4)},n.prototype.getRowProperties=function(e){return w("Range.getRowProperties",v,"1.9",O),E(this,"GetRowProperties",0,[e],0,0)},n.prototype.getRowsAbove=function(t){var n=b.Range_GetRowsAbove(this,t),r=n.handled,i=n.result;return r?i:(w("Range.getRowsAbove",v,"1.3",O),C(e.Range,this,"GetRowsAbove",1,[t],!1,!0,null,4))},n.prototype.getRowsBelow=function(t){var n=b.Range_GetRowsBelow(this,t),r=n.handled,i=n.result;return r?i:(w("Range.getRowsBelow",v,"1.3",O),C(e.Range,this,"GetRowsBelow",1,[t],!1,!0,null,4))},n.prototype.getSpecialCells=function(t,n){return w("Range.getSpecialCells",v,"1.9",O),C(e.RangeAreas,this,"GetSpecialCells",1,[t,n],!1,!0,null,4)},n.prototype.getSpecialCellsOrNullObject=function(t,n){return w("Range.getSpecialCellsOrNullObject",v,"1.9",O),C(e.RangeAreas,this,"GetSpecialCellsOrNullObject",1,[t,n],!1,!0,null,4)},n.prototype.getSpillParent=function(){return w("Range.getSpillParent",v,"1.12",O),C(e.Range,this,"GetSpillParent",1,[],!1,!0,null,4)},n.prototype.getSpillParentOrNullObject=function(){return w("Range.getSpillParentOrNullObject",v,"1.12",O),C(e.Range,this,"GetSpillParentOrNullObject",1,[],!1,!0,null,4)},n.prototype.getSpillingToRange=function(){return w("Range.getSpillingToRange",v,"1.12",O),C(e.Range,this,"GetSpillingToRange",1,[],!1,!0,null,4)},n.prototype.getSpillingToRangeOrNullObject=function(){return w("Range.getSpillingToRangeOrNullObject",v,"1.12",O),C(e.Range,this,"GetSpillingToRangeOrNullObject",1,[],!1,!0,null,4)},n.prototype.getSurroundingDataRegion=function(){return w("Range.getSurroundingDataRegion",v,"1.15",O),C(e.Range,this,"GetSurroundingDataRegion",1,[],!1,!1,null,4)},n.prototype.getSurroundingRegion=function(){return w("Range.getSurroundingRegion",v,"1.7",O),C(e.Range,this,"GetSurroundingRegion",1,[],!1,!0,null,4)},n.prototype.getTables=function(t){return w("Range.getTables",v,"1.9",O),C(e.TableScopedCollection,this,"GetTables",1,[t],!0,!1,null,4)},n.prototype.getUsedRange=function(t){return C(e.Range,this,"GetUsedRange",1,[t],!1,!0,null,4)},n.prototype.getUsedRangeOrNullObject=function(t){return w("Range.getUsedRangeOrNullObject",v,"1.4",O),C(e.Range,this,"GetUsedRangeOrNullObject",1,[t],!1,!0,null,4)},n.prototype.getVisibleView=function(){return w("Range.getVisibleView",v,"1.3",O),C(e.RangeView,this,"GetVisibleView",1,[],!1,!1,null,4)},n.prototype.group=function(e){w("Range.group",v,"1.10",O),E(this,"Group",0,[e],0,0)},n.prototype.hideGroupDetails=function(e){w("Range.hideGroupDetails",v,"1.10",O),E(this,"HideGroupDetails",0,[e],0,0)},n.prototype.insert=function(t){return C(e.Range,this,"Insert",0,[t],!1,!0,null,0)},n.prototype.merge=function(e){w("Range.merge",v,"1.2",O),E(this,"Merge",0,[e],0,0)},n.prototype.moveTo=function(e){w("Range.moveTo",v,"1.11",O),E(this,"MoveTo",0,[e],0,0)},n.prototype.removeDuplicates=function(t,n){return w("Range.removeDuplicates",v,"1.9",O),C(e.RemoveDuplicatesResult,this,"RemoveDuplicates",0,[t,n],!1,!0,null,0)},n.prototype.replaceAll=function(e,t,n){return w("Range.replaceAll",v,"1.9",O),E(this,"ReplaceAll",0,[e,t,n],0,0)},n.prototype.select=function(){E(this,"Select",1,[],0,0)},n.prototype.setCellProperties=function(e){w("Range.setCellProperties",v,"1.9",O),E(this,"SetCellProperties",0,[e],0,0)},n.prototype.setColumnProperties=function(e){w("Range.setColumnProperties",v,"1.9",O),E(this,"SetColumnProperties",0,[e],0,0)},n.prototype.setDirty=function(){w("Range.setDirty",v,"1.9",O),E(this,"SetDirty",0,[],0,0)},n.prototype.setRowProperties=function(e){w("Range.setRowProperties",v,"1.9",O),E(this,"SetRowProperties",0,[e],0,0)},n.prototype.showCard=function(){w("Range.showCard",v,"1.7",O),E(this,"ShowCard",0,[],0,0)},n.prototype.showGroupDetails=function(e){w("Range.showGroupDetails",v,"1.10",O),E(this,"ShowGroupDetails",0,[e],0,0)},n.prototype.showTeachingCallout=function(e,t){w("Range.showTeachingCallout",v,"1.9",O),E(this,"ShowTeachingCallout",0,[e,t],0,0)},n.prototype.ungroup=function(e){w("Range.ungroup",v,"1.10",O),E(this,"Ungroup",0,[e],0,0)},n.prototype.unmerge=function(){w("Range.unmerge",v,"1.2",O),E(this,"Unmerge",0,[],0,0)},n.prototype._KeepReference=function(){E(this,"_KeepReference",1,[],0,0)},n.prototype._ValidateArraySize=function(e,t){w("Range._ValidateArraySize",v,"1.3",O),E(this,"_ValidateArraySize",1,[e,t],4,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),b.Range_HandleResult(this,n),N(n.Address)||(this._A=n.Address),N(n.AddressLocal)||(this._Ad=n.AddressLocal),N(n.CellCount)||(this._C=n.CellCount),N(n.ColumnCount)||(this._Co=n.ColumnCount),N(n.ColumnHidden)||(this._Col=n.ColumnHidden),N(n.ColumnIndex)||(this._Colu=n.ColumnIndex),N(n.Formulas)||(this.m_formulas=n.Formulas),N(n.FormulasLocal)||(this.m_formulasLocal=n.FormulasLocal),N(n.FormulasR1C1)||(this.m_formulasR1C1=n.FormulasR1C1),N(n.HasSpill)||(this._H=n.HasSpill),N(n.Height)||(this._He=n.Height),N(n.Hidden)||(this._Hi=n.Hidden),N(n.Hyperlink)||(this._Hy=n.Hyperlink),N(n.IsEntireColumn)||(this.m_isEntireColumn=n.IsEntireColumn),N(n.IsEntireRow)||(this.m_isEntireRow=n.IsEntireRow),N(n.Left)||(this._L=n.Left),N(n.LinkedDataTypeState)||(this._Li=n.LinkedDataTypeState),N(n.NumberFormat)||(this.m_numberFormat=n.NumberFormat),N(n.NumberFormatCategories)||(this._N=n.NumberFormatCategories),N(n.NumberFormatLocal)||(this._Nu=n.NumberFormatLocal),N(n.RowCount)||(this._R=n.RowCount),N(n.RowHidden)||(this._Ro=n.RowHidden),N(n.RowIndex)||(this._Row=n.RowIndex),N(n.SavedAsArray)||(this._S=n.SavedAsArray),N(n.Style)||(this._St=n.Style),N(n.Text)||(this._T=n.Text),N(n.Top)||(this._To=n.Top),N(n.ValueTypes)||(this._V=n.ValueTypes),N(n.Values)||(this.m_values=n.Values),N(n.Width)||(this._W=n.Width),N(n._ReferenceId)||(this.__R=n._ReferenceId),L(this,n,["conditionalFormats","ConditionalFormats","dataValidation","DataValidation","format","Format","sort","Sort","worksheet","Worksheet"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e._ReferenceId)||(this.__R=e._ReferenceId)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.track=function(){return this.context.trackedObjects.add(this),this},n.prototype.untrack=function(){return this.context.trackedObjects.remove(this),this},n.prototype.toJSON=function(){return k(this,{address:this._A,addressLocal:this._Ad,cellCount:this._C,columnCount:this._Co,columnHidden:this._Col,columnIndex:this._Colu,formulas:this.m_formulas,formulasLocal:this.m_formulasLocal,formulasR1C1:this.m_formulasR1C1,hasSpill:this._H,height:this._He,hidden:this._Hi,hyperlink:this._Hy,isEntireColumn:this.m_isEntireColumn,isEntireRow:this.m_isEntireRow,left:this._L,linkedDataTypeState:this._Li,numberFormat:this.m_numberFormat,numberFormatCategories:this._N,numberFormatLocal:this._Nu,rowCount:this._R,rowHidden:this._Ro,rowIndex:this._Row,savedAsArray:this._S,style:this._St,text:this._T,top:this._To,values:this.m_values,valueTypes:this._V,width:this._W},{conditionalFormats:this._Con,dataValidation:this._D,format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Range=ye;var me=function(){function t(){}return t.prototype._ensureInteger=function(t,n){"number"==typeof t&&isFinite(t)&&Math.floor(t)===t||OfficeExtension.Utility.throwError(e.ErrorCodes.invalidArgument,t,n)},t.prototype._getAdjacentRange=function(e,t,n,r,i){var o;null==t&&(t=1),this._ensureInteger(t,e);var s=0,a=0;return t>0?o=n.getOffsetRange(r,i):(o=n,s=r,a=i),1===Math.abs(t)?o:o.getBoundingRect(n.getOffsetRange(r*t+s,i*t+a))},t}();e.RangeCustom=me,OfficeExtension.Utility.applyMixin(ye,me),function(t){t.Range_HandleResult=function(e,t){N(t.isEntireColumn)||(e.m_isEntireColumn=t.isEntireColumn),N(t.isEntireRow)||(e.m_isEntireRow=t.isEntireRow)},t.Range_GetColumnsAfter=function(t,n){return p()?{handled:!1,result:null}:(null==n&&(n=1),t._ensureInteger(n,"RowsAbove"),0===n&&OfficeExtension.Utility.throwError(e.ErrorCodes.invalidArgument,"count","RowsAbove"),{handled:!0,result:t._getAdjacentRange("getColumnsAfter",n,t.getLastColumn(),0,1)})},t.Range_GetColumnsBefore=function(t,n){return p()?{handled:!1,result:null}:(null==n&&(n=1),t._ensureInteger(n,"RowsAbove"),0===n&&OfficeExtension.Utility.throwError(e.ErrorCodes.invalidArgument,"count","RowsAbove"),{handled:!0,result:t._getAdjacentRange("getColumnsBefore",n,t.getColumn(0),0,-1)})},t.Range_GetResizedRange=function(e,t,n){return p()?{handled:!1,result:null}:(e._ensureInteger(t,"getResizedRange"),e._ensureInteger(n,"getResizedRange"),{handled:!0,result:(t>=0&&n>=0?e:e.getCell(0,0)).getBoundingRect(e.getLastCell().getOffsetRange(t,n))})},t.Range_GetRowsAbove=function(t,n){return p()?{handled:!1,result:null}:(null==n&&(n=1),t._ensureInteger(n,"RowsAbove"),0===n&&OfficeExtension.Utility.throwError(e.ErrorCodes.invalidArgument,"count","RowsAbove"),{handled:!0,result:t._getAdjacentRange("getRowsAbove",n,t.getRow(0),-1,0)})},t.Range_GetRowsBelow=function(t,n){return p()?{handled:!1,result:null}:(null==n&&(n=1),t._ensureInteger(n,"RowsAbove"),0===n&&OfficeExtension.Utility.throwError(e.ErrorCodes.invalidArgument,"count","RowsAbove"),{handled:!0,result:this._getAdjacentRange("getRowsBelow",n,t.getLastRow(),1,0)})},t.Range_Formulas_Set=function(e,t){return e.m_formulas=t,u(e,"Formulas",t)?{handled:!0}:{handled:!1}},t.Range_FormulasLocal_Set=function(e,t){return e.m_formulasLocal=t,u(e,"FormulasLocal",t)?{handled:!0}:{handled:!1}},t.Range_FormulasR1C1_Set=function(e,t){return e.m_formulasR1C1=t,u(e,"FormulasR1C1",t)?{handled:!0}:{handled:!1}},t.Range_NumberFormat_Set=function(e,t){return e.m_numberFormat=t,u(e,"NumberFormat",t,V(2,"ExcelApiUndo","1.7"))?{handled:!0}:{handled:!1}},t.Range_Values_Set=function(e,t){return e.m_values=t,u(e,"Values",t,V(2,"ExcelApiUndo","1.7"))?{handled:!0}:{handled:!1}}}(b=e._CC||(e._CC={}));var ge="RangeAreas",_e=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"RangeAreas"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["_ReferenceId","address","addressLocal","areaCount","cellCount","isEntireColumn","isEntireRow","style"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["_ReferenceId","Address","AddressLocal","AreaCount","CellCount","IsEntireColumn","IsEntireRow","Style"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!1,!1,!1,!1,!1,!1,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["areas","conditionalFormats","format","dataValidation","worksheet"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"areas",{get:function(){return this._Are||(this._Are=S(e.RangeCollection,this,"Areas",!0,4)),this._Are},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"conditionalFormats",{get:function(){return this._Co||(this._Co=S(e.ConditionalFormatCollection,this,"ConditionalFormats",!0,4)),this._Co},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"dataValidation",{get:function(){return this._D||(this._D=S(e.DataValidation,this,"DataValidation",!1,4)),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.RangeFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"worksheet",{get:function(){return this._W||(this._W=S(e.Worksheet,this,"Worksheet",!1,4)),this._W},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"address",{get:function(){return T("address",this._A,ge,this._isNull),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"addressLocal",{get:function(){return T("addressLocal",this._Ad,ge,this._isNull),this._Ad},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"areaCount",{get:function(){return T("areaCount",this._Ar,ge,this._isNull),this._Ar},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"cellCount",{get:function(){return T("cellCount",this._C,ge,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isEntireColumn",{get:function(){return T("isEntireColumn",this.m_isEntireColumn,ge,this._isNull),this.m_isEntireColumn},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isEntireRow",{get:function(){return T("isEntireRow",this.m_isEntireRow,ge,this._isNull),this.m_isEntireRow},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"style",{get:function(){return T("style",this._S,ge,this._isNull),this._S},set:function(e){this._S=e,I(this,"Style",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_ReferenceId",{get:function(){return T("_ReferenceId",this.__R,ge,this._isNull),this.__R},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["style"],["format","dataValidation"],["areas","conditionalFormats","worksheet"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.calculate=function(){E(this,"Calculate",0,[],0,0)},n.prototype.clear=function(e){E(this,"Clear",0,[e],0,0)},n.prototype.convertDataTypeToText=function(){E(this,"ConvertDataTypeToText",0,[],0,0)},n.prototype.convertToLinkedDataType=function(e,t){E(this,"ConvertToLinkedDataType",0,[e,t],0,0)},n.prototype.copyFrom=function(e,t,n,r){E(this,"CopyFrom",0,[e,t,n,r],0,0)},n.prototype.getEntireColumn=function(){return C(e.RangeAreas,this,"GetEntireColumn",1,[],!1,!0,null,4)},n.prototype.getEntireRow=function(){return C(e.RangeAreas,this,"GetEntireRow",1,[],!1,!0,null,4)},n.prototype.getIntersection=function(t){return C(e.RangeAreas,this,"GetIntersection",1,[t],!1,!0,null,4)},n.prototype.getIntersectionOrNullObject=function(t){return C(e.RangeAreas,this,"GetIntersectionOrNullObject",1,[t],!1,!0,null,4)},n.prototype.getOffsetRangeAreas=function(t,n){return C(e.RangeAreas,this,"GetOffsetRangeAreas",1,[t,n],!1,!0,null,4)},n.prototype.getSpecialCells=function(t,n){return C(e.RangeAreas,this,"GetSpecialCells",1,[t,n],!1,!0,null,4)},n.prototype.getSpecialCellsOrNullObject=function(t,n){return C(e.RangeAreas,this,"GetSpecialCellsOrNullObject",1,[t,n],!1,!0,null,4)},n.prototype.getTables=function(t){return C(e.TableScopedCollection,this,"GetTables",1,[t],!0,!1,null,4)},n.prototype.getUsedRangeAreas=function(t){return C(e.RangeAreas,this,"GetUsedRangeAreas",1,[t],!1,!0,null,4)},n.prototype.getUsedRangeAreasOrNullObject=function(t){return C(e.RangeAreas,this,"GetUsedRangeAreasOrNullObject",1,[t],!1,!0,null,4)},n.prototype.setDirty=function(){E(this,"SetDirty",0,[],0,0)},n.prototype._KeepReference=function(){E(this,"_KeepReference",1,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Address)||(this._A=n.Address),N(n.AddressLocal)||(this._Ad=n.AddressLocal),N(n.AreaCount)||(this._Ar=n.AreaCount),N(n.CellCount)||(this._C=n.CellCount),N(n.IsEntireColumn)||(this.m_isEntireColumn=n.IsEntireColumn),N(n.IsEntireRow)||(this.m_isEntireRow=n.IsEntireRow),N(n.Style)||(this._S=n.Style),N(n._ReferenceId)||(this.__R=n._ReferenceId),L(this,n,["areas","Areas","conditionalFormats","ConditionalFormats","dataValidation","DataValidation","format","Format","worksheet","Worksheet"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e._ReferenceId)||(this.__R=e._ReferenceId)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.track=function(){return this.context.trackedObjects.add(this),this},n.prototype.untrack=function(){return this.context.trackedObjects.remove(this),this},n.prototype.toJSON=function(){return k(this,{address:this._A,addressLocal:this._Ad,areaCount:this._Ar,cellCount:this._C,isEntireColumn:this.m_isEntireColumn,isEntireRow:this.m_isEntireRow,style:this._S},{areas:this._Are,conditionalFormats:this._Co,dataValidation:this._D,format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.RangeAreas=_e;var be="WorkbookRangeAreas",Oe=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"WorkbookRangeAreas"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["_ReferenceId","addresses"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["_ReferenceId","Addresses"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["ranges","areas"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"areas",{get:function(){return this._Ar||(this._Ar=S(e.RangeAreasCollection,this,"Areas",!0,4)),this._Ar},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"ranges",{get:function(){return this._R||(this._R=S(e.RangeCollection,this,"Ranges",!0,4)),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"addresses",{get:function(){return T("addresses",this._A,be,this._isNull),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_ReferenceId",{get:function(){return T("_ReferenceId",this.__R,be,this._isNull),this.__R},enumerable:!0,configurable:!0}),n.prototype.getRangeAreasBySheet=function(t){return C(e.RangeAreas,this,"GetRangeAreasBySheet",1,[t],!1,!0,null,4)},n.prototype.getRangeAreasOrNullObjectBySheet=function(t){return C(e.RangeAreas,this,"GetRangeAreasOrNullObjectBySheet",1,[t],!1,!0,null,4)},n.prototype._KeepReference=function(){E(this,"_KeepReference",1,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Addresses)||(this._A=n.Addresses),N(n._ReferenceId)||(this.__R=n._ReferenceId),L(this,n,["areas","Areas","ranges","Ranges"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e._ReferenceId)||(this.__R=e._ReferenceId)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.track=function(){return this.context.trackedObjects.add(this),this},n.prototype.untrack=function(){return this.context.trackedObjects.remove(this),this},n.prototype.toJSON=function(){return k(this,{addresses:this._A},{areas:this._Ar,ranges:this._R})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.WorkbookRangeAreas=Oe;var ve="RangeView",Se=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"RangeView"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["numberFormat","values","text","formulas","formulasLocal","formulasR1C1","valueTypes","rowCount","columnCount","cellAddresses","index"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["NumberFormat","Values","Text","Formulas","FormulasLocal","FormulasR1C1","ValueTypes","RowCount","ColumnCount","CellAddresses","Index"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!1,!0,!0,!0,!1,!1,!1,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["rows"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rows",{get:function(){return this._Ro||(this._Ro=S(e.RangeViewCollection,this,"Rows",!0,4)),this._Ro},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"cellAddresses",{get:function(){return T("cellAddresses",this._C,ve,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"columnCount",{get:function(){return T("columnCount",this._Co,ve,this._isNull),this._Co},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"formulas",{get:function(){return T("formulas",this._F,ve,this._isNull),this._F},set:function(e){this._F=e,I(this,"Formulas",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"formulasLocal",{get:function(){return T("formulasLocal",this._Fo,ve,this._isNull),this._Fo},set:function(e){this._Fo=e,I(this,"FormulasLocal",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"formulasR1C1",{get:function(){return T("formulasR1C1",this._For,ve,this._isNull),this._For},set:function(e){this._For=e,I(this,"FormulasR1C1",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"index",{get:function(){return T("index",this._I,ve,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormat",{get:function(){return T("numberFormat",this._N,ve,this._isNull),this._N},set:function(e){this._N=e,I(this,"NumberFormat",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rowCount",{get:function(){return T("rowCount",this._R,ve,this._isNull),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"text",{get:function(){return T("text",this._T,ve,this._isNull),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"valueTypes",{get:function(){return T("valueTypes",this._Va,ve,this._isNull),this._Va},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"values",{get:function(){return T("values",this._V,ve,this._isNull),this._V},set:function(e){this._V=e,I(this,"Values",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["numberFormat","values","formulas","formulasLocal","formulasR1C1"],[],["rows"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.getRange=function(){return C(e.Range,this,"GetRange",1,[],!1,!0,null,4)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.CellAddresses)||(this._C=n.CellAddresses),N(n.ColumnCount)||(this._Co=n.ColumnCount),N(n.Formulas)||(this._F=n.Formulas),N(n.FormulasLocal)||(this._Fo=n.FormulasLocal),N(n.FormulasR1C1)||(this._For=n.FormulasR1C1),N(n.Index)||(this._I=n.Index),N(n.NumberFormat)||(this._N=n.NumberFormat),N(n.RowCount)||(this._R=n.RowCount),N(n.Text)||(this._T=n.Text),N(n.ValueTypes)||(this._Va=n.ValueTypes),N(n.Values)||(this._V=n.Values),L(this,n,["rows","Rows"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{cellAddresses:this._C,columnCount:this._Co,formulas:this._F,formulasLocal:this._Fo,formulasR1C1:this._For,index:this._I,numberFormat:this._N,rowCount:this._R,text:this._T,values:this._V,valueTypes:this._Va},{rows:this._Ro})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.RangeView=Se;var Ce=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"RangeViewCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"RangeViewCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return w("RangeViewCollection.getCount",v,"1.4",O),E(this,"GetCount",1,[],4,0)},n.prototype.getItemAt=function(t){return C(e.RangeView,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.RangeView,!1,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.RangeView,!1,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.RangeView,!1,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.RangeViewCollection=Ce;var Pe=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"SettingCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"SettingCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n){var r=b.SettingCollection_Add(this,t,n),i=r.handled,o=r.result;return i?o:C(e.Setting,this,"Add",0,[t,n],!1,!0,null,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.Setting,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.Setting,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Setting,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Setting,!0,i,t,n)}))},Object.defineProperty(n.prototype,"onSettingsChanged",{get:function(){var e=this;return this.m_settingsChanged||(this.m_settingsChanged=new OfficeExtension.EventHandlers(this.context,this,"SettingsChanged",{registerFunc:function(t){return e.context.eventRegistration.register(b.office10EventIdSettingsChangedEvent,"",t)},unregisterFunc:function(t){return e.context.eventRegistration.unregister(b.office10EventIdSettingsChangedEvent,"",t)},eventArgsTransformFunc:function(t){var n=b.SettingCollection_SettingsChanged_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_settingsChanged},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Setting,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.SettingCollection=Pe,function(t){t.SettingCollection_Add=function(n,r,i){return i=t._replaceDateWithStringDate(i),{handled:!0,result:C(e.Setting,n,"Add",0,[r,i],!1,!0,null,0)}},t.SettingCollection_SettingsChanged_EventArgsTransform=function(e,t){return{settings:e}}}(b=e._CC||(e._CC={}));var Ae="Setting",Fe=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"Setting"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["key","value","_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Key","Value","_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"key",{get:function(){return T("key",this._K,Ae,this._isNull),this._K},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"value",{get:function(){return T("value",this.m_value,Ae,this._isNull),this.m_value},set:function(e){b.Setting_Value_Set(this,e).handled||(this.m_value=e,I(this,"Value",e,0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_Id",{get:function(){return T("_Id",this.__I,Ae,this._isNull),this.__I},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["value"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype.delete=function(){E(this,"Delete",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),b.Setting_HandleResult(this,n),N(n.Key)||(this._K=n.Key),N(n.Value)||(this.m_value=n.Value),N(n._Id)||(this.__I=n._Id)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleIdResult=function(t){e.prototype._handleIdResult.call(this,t),x(t)||N(t._Id)||(this.__I=t._Id)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{key:this._K,value:this.m_value},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.Setting=Fe,function(e){var t="Date(";function n(e){var n=JSON.stringify(e,(function(e,n){return this[e]instanceof Date?t+this[e].getTime()+")":n}));return e=JSON.parse(n)}e._replaceDateWithStringDate=n,e.Setting_HandleResult=function(e,n){N(n.Value)||(n.Value=function(e){var n=JSON.stringify(e);return JSON.parse(n,(function(e,n){var r;return"string"==typeof n&&n&&n.length>6&&n.slice(0,5)===t&&")"===n.slice(-1)&&(r=new Date(parseInt(n.slice(5,-1))))?r:n}))}(n.Value))},e.Setting_Value_Set=function(e,t){if(!x(t)){e.m_value=t;var r=n(t);return I(e,"Value",r,0),{handled:!0}}return{handled:!1}}}(b=e._CC||(e._CC={}));var De=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"NamedItemCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"NamedItemCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n,r){return w("NamedItemCollection.add",v,"1.4",O),C(e.NamedItem,this,"Add",0,[t,n,r],!1,!0,null,0)},n.prototype.addFormulaLocal=function(t,n,r){return w("NamedItemCollection.addFormulaLocal",v,"1.4",O),C(e.NamedItem,this,"AddFormulaLocal",0,[t,n,r],!1,!1,null,0)},n.prototype.getCount=function(){return w("NamedItemCollection.getCount",v,"1.4",O),E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.NamedItem,this,[t])},n.prototype.getItemOrNullObject=function(t){return w("NamedItemCollection.getItemOrNullObject",v,"1.4",O),C(e.NamedItem,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.NamedItem,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.NamedItem,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.NamedItem,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.NamedItemCollection=De;var Ee=function(){function e(){}return Object.defineProperty(e.prototype,"_ParentObject",{get:function(){return this.m__ParentObject},set:function(e){this.m__ParentObject=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"_eventTargetId",{get:function(){return this._ParentObject?this._ParentObject.id:OfficeExtension.Constants.eventWorkbookId},enumerable:!0,configurable:!0}),e}();e.NamedItemCollectionCustom=Ee,OfficeExtension.Utility.applyMixin(De,Ee);var Re="NamedItem",Ie=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"NamedItem"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["name","type","value","visible","_Id","comment","scope","formula"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name","Type","Value","Visible","_Id","Comment","Scope","Formula"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!1,!1,!0,!1,!0,!1,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["worksheet","worksheetOrNullObject","arrayValues"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"arrayValues",{get:function(){return w("NamedItem.arrayValues",v,"1.7",O),this._A||(this._A=S(e.NamedItemArrayValues,this,"ArrayValues",!1,4)),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"worksheet",{get:function(){return w("NamedItem.worksheet",v,"1.4",O),this._W||(this._W=S(e.Worksheet,this,"Worksheet",!1,4)),this._W},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"worksheetOrNullObject",{get:function(){return w("NamedItem.worksheetOrNullObject",v,"1.4",O),this._Wo||(this._Wo=S(e.Worksheet,this,"WorksheetOrNullObject",!1,4)),this._Wo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"comment",{get:function(){return T("comment",this._C,Re,this._isNull),w("NamedItem.comment",v,"1.4",O),this._C},set:function(e){this._C=e,I(this,"Comment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"formula",{get:function(){return T("formula",this._F,Re,this._isNull),w("NamedItem.formula",v,"1.7",O),this._F},set:function(e){this._F=e,I(this,"Formula",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,Re,this._isNull),this._N},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"scope",{get:function(){return T("scope",this._S,Re,this._isNull),w("NamedItem.scope",v,"1.4",O),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"type",{get:function(){return T("type",this._T,Re,this._isNull),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"value",{get:function(){return T("value",this._V,Re,this._isNull),this._V},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"visible",{get:function(){return T("visible",this._Vi,Re,this._isNull),this._Vi},set:function(e){this._Vi=e,I(this,"Visible",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_Id",{get:function(){return T("_Id",this.__I,Re,this._isNull),this.__I},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["visible","comment","formula"],[],["arrayValues","worksheet","worksheetOrNullObject"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){w("NamedItem.delete",v,"1.4",O),E(this,"Delete",0,[],0,0)},n.prototype.getRange=function(){return C(e.Range,this,"GetRange",1,[],!1,!0,null,4)},n.prototype.getRangeOrNullObject=function(){return w("NamedItem.getRangeOrNullObject",v,"1.4",O),C(e.Range,this,"GetRangeOrNullObject",1,[],!1,!0,null,4)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Comment)||(this._C=n.Comment),N(n.Formula)||(this._F=n.Formula),N(n.Name)||(this._N=n.Name),N(n.Scope)||(this._S=n.Scope),N(n.Type)||(this._T=n.Type),N(n.Value)||(this._V=n.Value),N(n.Visible)||(this._Vi=n.Visible),N(n._Id)||(this.__I=n._Id),L(this,n,["arrayValues","ArrayValues","worksheet","Worksheet","worksheetOrNullObject","WorksheetOrNullObject"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e._Id)||(this.__I=e._Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{comment:this._C,formula:this._F,name:this._N,scope:this._S,type:this._T,value:this._V,visible:this._Vi},{arrayValues:this._A})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.NamedItem=Ie;var xe="NamedItemArrayValues",Ne=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"NamedItemArrayValues"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["values","types"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Values","Types"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"types",{get:function(){return T("types",this._T,xe,this._isNull),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"values",{get:function(){return T("values",this._V,xe,this._isNull),this._V},enumerable:!0,configurable:!0}),t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Types)||(this._T=n.Types),N(n.Values)||(this._V=n.Values)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{types:this._T,values:this._V},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.NamedItemArrayValues=Ne;var Te="Binding",we=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Binding"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","type"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Type"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,Te,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"type",{get:function(){return T("type",this._T,Te,this._isNull),this._T},enumerable:!0,configurable:!0}),n.prototype.delete=function(){w("Binding.delete",v,"1.3",O),E(this,"Delete",0,[],0,0)},n.prototype.getRange=function(){return C(e.Range,this,"GetRange",1,[],!1,!1,null,4)},n.prototype.getTable=function(){return C(e.Table,this,"GetTable",1,[],!1,!1,null,4)},n.prototype.getText=function(){return E(this,"GetText",1,[],4,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Id)||(this._I=n.Id),N(n.Type)||(this._T=n.Type)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},Object.defineProperty(n.prototype,"onDataChanged",{get:function(){var e=this;return w("Binding.onDataChanged",v,"1.3",O),this.m_dataChanged||(this.m_dataChanged=new OfficeExtension.EventHandlers(this.context,this,"DataChanged",{registerFunc:function(t){return e.context.eventRegistration.register(b.office10EventIdBindingDataChangedEvent,e.id,t)},unregisterFunc:function(t){return e.context.eventRegistration.unregister(b.office10EventIdBindingDataChangedEvent,e.id,t)},eventArgsTransformFunc:function(t){var n=b.Binding_DataChanged_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_dataChanged},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onSelectionChanged",{get:function(){var e=this;return w("Binding.onSelectionChanged",v,"1.3",O),this.m_selectionChanged||(this.m_selectionChanged=new OfficeExtension.EventHandlers(this.context,this,"SelectionChanged",{registerFunc:function(t){return e.context.eventRegistration.register(b.office10EventIdBindingSelectionChangedEvent,e.id,t)},unregisterFunc:function(t){return e.context.eventRegistration.unregister(b.office10EventIdBindingSelectionChangedEvent,e.id,t)},eventArgsTransformFunc:function(t){var n=b.Binding_SelectionChanged_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_selectionChanged},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{id:this._I,type:this._T},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Binding=we,function(e){e.Binding_DataChanged_EventArgsTransform=function(e,t){return{binding:e}},e.Binding_SelectionChanged_EventArgsTransform=function(e,t){return{binding:e,columnCount:t.columnCount,rowCount:t.rowCount,startColumn:t.startColumn,startRow:t.startRow}}}(b=e._CC||(e._CC={}));var je="BindingCollection",Me=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"BindingCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,je,this._isNull),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"count",{get:function(){return T("count",this._C,je,this._isNull),this._C},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n,r){return w("BindingCollection.add",v,"1.3",O),C(e.Binding,this,"Add",0,[t,n,r],!1,!0,null,0)},n.prototype.addFromNamedItem=function(t,n,r){return w("BindingCollection.addFromNamedItem",v,"1.3",O),C(e.Binding,this,"AddFromNamedItem",0,[t,n,r],!1,!1,null,0)},n.prototype.addFromSelection=function(t,n){return w("BindingCollection.addFromSelection",v,"1.3",O),C(e.Binding,this,"AddFromSelection",0,[t,n],!1,!1,null,0)},n.prototype.getCount=function(){return w("BindingCollection.getCount",v,"1.4",O),E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.Binding,this,[t])},n.prototype.getItemAt=function(t){return C(e.Binding,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return w("BindingCollection.getItemOrNullObject",v,"1.4",O),C(e.Binding,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),N(r.Count)||(this._C=r.Count),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Binding,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Binding,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{count:this._C},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Binding,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.BindingCollection=Me;var ke="TableCollection",Ue=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TableCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,ke,this._isNull),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"count",{get:function(){return T("count",this._C,ke,this._isNull),this._C},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n){return C(e.Table,this,"Add",0,[t,n],!1,!0,null,0)},n.prototype.getCount=function(){return w("TableCollection.getCount",v,"1.4",O),E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.Table,this,[t])},n.prototype.getItemAt=function(t){return C(e.Table,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return w("TableCollection.getItemOrNullObject",v,"1.4",O),C(e.Table,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._RegisterAddedEvent=function(){w("TableCollection._RegisterAddedEvent",v,"1.9",O),E(this,"_RegisterAddedEvent",0,[],0,0)},n.prototype._RegisterDataChangedEvent=function(){w("TableCollection._RegisterDataChangedEvent",v,"1.7",O),E(this,"_RegisterDataChangedEvent",0,[],0,0)},n.prototype._RegisterDeletedEvent=function(){w("TableCollection._RegisterDeletedEvent",v,"1.9",O),E(this,"_RegisterDeletedEvent",0,[],0,0)},n.prototype._UnregisterAddedEvent=function(){w("TableCollection._UnregisterAddedEvent",v,"1.9",O),E(this,"_UnregisterAddedEvent",0,[],0,0)},n.prototype._UnregisterDataChangedEvent=function(){w("TableCollection._UnregisterDataChangedEvent",v,"1.7",O),E(this,"_UnregisterDataChangedEvent",0,[],0,0)},n.prototype._UnregisterDeletedEvent=function(){w("TableCollection._UnregisterDeletedEvent",v,"1.9",O),E(this,"_UnregisterDeletedEvent",0,[],0,0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),N(r.Count)||(this._C=r.Count),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Table,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Table,!0,i,t,n)}))},Object.defineProperty(n.prototype,"onAdded",{get:function(){var e=this;return w("TableCollection.onAdded",v,"1.9",O),this.m_added||(this.m_added=new OfficeExtension.GenericEventHandlers(this.context,this,"Added",{eventType:102,registerFunc:function(){return e._RegisterAddedEvent()},unregisterFunc:function(){return e._UnregisterAddedEvent()},getTargetIdFunc:function(){return e._eventTargetId},eventArgsTransformFunc:function(e){var t={type:Oo.tableAdded,source:e.source,tableId:e.tableId,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_added},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onChanged",{get:function(){var e=this;return w("TableCollection.onChanged",v,"1.7",O),this.m_changed||(this.m_changed=new OfficeExtension.GenericEventHandlers(this.context,this,"Changed",{eventType:101,registerFunc:function(){return e._RegisterDataChangedEvent()},unregisterFunc:function(){return e._UnregisterDataChangedEvent()},getTargetIdFunc:function(){return e._eventTargetId},eventArgsTransformFunc:function(t){var n=b.TableCollection_Changed_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_changed},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onDeleted",{get:function(){var e=this;return w("TableCollection.onDeleted",v,"1.9",O),this.m_deleted||(this.m_deleted=new OfficeExtension.GenericEventHandlers(this.context,this,"Deleted",{eventType:103,registerFunc:function(){return e._RegisterDeletedEvent()},unregisterFunc:function(){return e._UnregisterDeletedEvent()},getTargetIdFunc:function(){return e._eventTargetId},eventArgsTransformFunc:function(e){var t={type:Oo.tableDeleted,source:e.source,tableId:e.tableId,tableName:e.tableName,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_deleted},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{count:this._C},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Table,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.TableCollection=Ue;var Le=function(){function e(){}return Object.defineProperty(e.prototype,"_ParentObject",{get:function(){return this.m__ParentObject},set:function(e){this.m__ParentObject=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"_eventTargetId",{get:function(){return this._ParentObject?this._ParentObject.id:OfficeExtension.Constants.eventWorkbookId},enumerable:!0,configurable:!0}),e}();e.TableCollectionCustom=Le,OfficeExtension.Utility.applyMixin(Ue,Le),function(t){t.TableCollection_Changed_EventArgsTransform=function(t,n){var r,i=n;return null==i.valueBefore&&null==i.valueAfter||(r={valueBefore:i.valueBefore,valueAfter:i.valueAfter,valueTypeBefore:i.valueTypeBefore,valueTypeAfter:i.valueTypeAfter}),{type:e.EventType.tableChanged,changeType:i.changeType,source:i.source,worksheetId:i.worksheetId,tableId:i.tableId,address:i.address,getRange:function(e){return w("TableChangedEventArgs.getRange",v,"1.8",O),e.workbook._GetRangeForEventByReferenceId(i.referenceId)},getRangeOrNullObject:function(e){return w("TableChangedEventArgs.getRangeOrNullObject",v,"1.8",O),e.workbook._GetRangeOrNullObjectForEventByReferenceId(i.referenceId)},details:r}}}(b=e._CC||(e._CC={}));var Be=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TableScopedCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"TableScopedCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getFirst=function(){return C(e.Table,this,"GetFirst",1,[],!1,!0,null,4)},n.prototype.getItem=function(t){return P(e.Table,this,[t])},n.prototype.getItemOrNullObject=function(t){return w("TableScopedCollection.getItemOrNullObject",v,"1.14",O),C(e.Table,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Table,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Table,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Table,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.TableScopedCollection=Be;var He="Table",We=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Table"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_collectionPropertyPath",{get:function(){return"workbook.tables"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","name","showHeaders","showTotals","style","highlightFirstColumn","highlightLastColumn","showBandedRows","showBandedColumns","showFilterButton","legacyId"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Name","ShowHeaders","ShowTotals","Style","HighlightFirstColumn","HighlightLastColumn","ShowBandedRows","ShowBandedColumns","ShowFilterButton","LegacyId"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0,!0,!0,!0,!0,!0,!0,!0,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["columns","rows","sort","worksheet","autoFilter"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"autoFilter",{get:function(){return w("Table.autoFilter",v,"1.9",O),this._A||(this._A=S(e.AutoFilter,this,"AutoFilter",!1,4)),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"columns",{get:function(){return this._C||(this._C=S(e.TableColumnCollection,this,"Columns",!0,4)),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rows",{get:function(){return this._R||(this._R=S(e.TableRowCollection,this,"Rows",!0,4)),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"sort",{get:function(){return w("Table.sort",v,"1.2",O),this._So||(this._So=S(e.TableSort,this,"Sort",!1,4)),this._So},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"worksheet",{get:function(){return w("Table.worksheet",v,"1.2",O),this._W||(this._W=S(e.Worksheet,this,"Worksheet",!1,4)),this._W},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"highlightFirstColumn",{get:function(){return T("highlightFirstColumn",this._H,He,this._isNull),w("Table.highlightFirstColumn",v,"1.3",O),this._H},set:function(e){this._H=e,I(this,"HighlightFirstColumn",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"highlightLastColumn",{get:function(){return T("highlightLastColumn",this._Hi,He,this._isNull),w("Table.highlightLastColumn",v,"1.3",O),this._Hi},set:function(e){this._Hi=e,I(this,"HighlightLastColumn",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,He,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"legacyId",{get:function(){return T("legacyId",this._L,He,this._isNull),w("Table.legacyId",v,"1.8",O),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,He,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showBandedColumns",{get:function(){return T("showBandedColumns",this._S,He,this._isNull),w("Table.showBandedColumns",v,"1.3",O),this._S},set:function(e){this._S=e,I(this,"ShowBandedColumns",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showBandedRows",{get:function(){return T("showBandedRows",this._Sh,He,this._isNull),w("Table.showBandedRows",v,"1.3",O),this._Sh},set:function(e){this._Sh=e,I(this,"ShowBandedRows",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showFilterButton",{get:function(){return T("showFilterButton",this._Sho,He,this._isNull),w("Table.showFilterButton",v,"1.3",O),this._Sho},set:function(e){this._Sho=e,I(this,"ShowFilterButton",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showHeaders",{get:function(){return T("showHeaders",this._Show,He,this._isNull),this._Show},set:function(e){this._Show=e,I(this,"ShowHeaders",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showTotals",{get:function(){return T("showTotals",this._ShowT,He,this._isNull),this._ShowT},set:function(e){this._ShowT=e,I(this,"ShowTotals",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"style",{get:function(){return T("style",this._St,He,this._isNull),this._St},set:function(e){this._St=e,I(this,"Style",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name","showHeaders","showTotals","style","highlightFirstColumn","highlightLastColumn","showBandedRows","showBandedColumns","showFilterButton"],[],["autoFilter","columns","rows","sort","worksheet"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.clearFilters=function(){w("Table.clearFilters",v,"1.2",O),E(this,"ClearFilters",0,[],0,0)},n.prototype.convertToRange=function(){return w("Table.convertToRange",v,"1.2",O),C(e.Range,this,"ConvertToRange",0,[],!1,!0,null,0)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.getDataBodyRange=function(){return C(e.Range,this,"GetDataBodyRange",1,[],!1,!0,null,4)},n.prototype.getHeaderRowRange=function(){return C(e.Range,this,"GetHeaderRowRange",1,[],!1,!0,null,4)},n.prototype.getRange=function(){return C(e.Range,this,"GetRange",1,[],!1,!0,null,4)},n.prototype.getTotalRowRange=function(){return C(e.Range,this,"GetTotalRowRange",1,[],!1,!0,null,4)},n.prototype.reapplyFilters=function(){w("Table.reapplyFilters",v,"1.2",O),E(this,"ReapplyFilters",0,[],0,0)},n.prototype.resize=function(e){w("Table.resize",v,"1.13",O),E(this,"Resize",0,[e],0,0)},n.prototype._RegisterDataChangedEvent=function(){w("Table._RegisterDataChangedEvent",v,"1.7",O),E(this,"_RegisterDataChangedEvent",0,[],0,0)},n.prototype._RegisterSelectionChangedEvent=function(){w("Table._RegisterSelectionChangedEvent",v,"1.7",O),E(this,"_RegisterSelectionChangedEvent",0,[],0,0)},n.prototype._UnregisterDataChangedEvent=function(){w("Table._UnregisterDataChangedEvent",v,"1.7",O),E(this,"_UnregisterDataChangedEvent",0,[],0,0)},n.prototype._UnregisterSelectionChangedEvent=function(){w("Table._UnregisterSelectionChangedEvent",v,"1.7",O),E(this,"_UnregisterSelectionChangedEvent",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),b.Table_HandleResult(this,n),N(n.HighlightFirstColumn)||(this._H=n.HighlightFirstColumn),N(n.HighlightLastColumn)||(this._Hi=n.HighlightLastColumn),N(n.Id)||(this._I=n.Id),N(n.LegacyId)||(this._L=n.LegacyId),N(n.Name)||(this._N=n.Name),N(n.ShowBandedColumns)||(this._S=n.ShowBandedColumns),N(n.ShowBandedRows)||(this._Sh=n.ShowBandedRows),N(n.ShowFilterButton)||(this._Sho=n.ShowFilterButton),N(n.ShowHeaders)||(this._Show=n.ShowHeaders),N(n.ShowTotals)||(this._ShowT=n.ShowTotals),N(n.Style)||(this._St=n.Style),L(this,n,["autoFilter","AutoFilter","columns","Columns","rows","Rows","sort","Sort","worksheet","Worksheet"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||(b.Table_HandleIdResult(this,e),N(e.Id)||(this._I=e.Id))},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},Object.defineProperty(n.prototype,"onChanged",{get:function(){var e=this;return w("Table.onChanged",v,"1.7",O),this.m_changed||(this.m_changed=new OfficeExtension.GenericEventHandlers(this.context,this,"Changed",{eventType:101,registerFunc:function(){return e._RegisterDataChangedEvent()},unregisterFunc:function(){return e._UnregisterDataChangedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(t){var n=b.Table_Changed_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_changed},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onSelectionChanged",{get:function(){var e=this;return w("Table.onSelectionChanged",v,"1.7",O),this.m_selectionChanged||(this.m_selectionChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"SelectionChanged",{eventType:100,registerFunc:function(){return e._RegisterSelectionChangedEvent()},unregisterFunc:function(){return e._UnregisterSelectionChangedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(t){var n=b.Table_SelectionChanged_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_selectionChanged},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{highlightFirstColumn:this._H,highlightLastColumn:this._Hi,id:this._I,legacyId:this._L,name:this._N,showBandedColumns:this._S,showBandedRows:this._Sh,showFilterButton:this._Sho,showHeaders:this._Show,showTotals:this._ShowT,style:this._St},{autoFilter:this._A,columns:this._C,rows:this._R,sort:this._So})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Table=We,function(t){t.Table_HandleIdResult=function(e,t){N(t.Id)||(t.Id=t.Id.toString())},t.Table_HandleResult=function(e,t){N(t.Id)||(t.Id=t.Id.toString())},t.Table_Changed_EventArgsTransform=function(t,n){var r,i=n;return null==i.valueBefore&&null==i.valueAfter||(r={valueBefore:i.valueBefore,valueAfter:i.valueAfter,valueTypeBefore:i.valueTypeBefore,valueTypeAfter:i.valueTypeAfter}),{type:e.EventType.tableChanged,changeType:i.changeType,source:i.source,worksheetId:i.worksheetId,tableId:i.tableId,address:i.address,getRange:function(e){return w("TableChangedEventArgs.getRange",v,"1.8",O),e.workbook._GetRangeForEventByReferenceId(i.referenceId)},getRangeOrNullObject:function(e){return w("TableChangedEventArgs.getRangeOrNullObject",v,"1.8",O),e.workbook._GetRangeOrNullObjectForEventByReferenceId(i.referenceId)},details:r}},t.Table_SelectionChanged_EventArgsTransform=function(t,n){var r=n,i=!r.address||0===r.address.length;return{type:e.EventType.tableSelectionChanged,isInsideTable:!i,worksheetId:r.worksheetId,tableId:t.id,address:r.address}}}(b=e._CC||(e._CC={}));var Ve="TableColumnCollection",Ge=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TableColumnCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,Ve,this._isNull),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"count",{get:function(){return T("count",this._C,Ve,this._isNull),this._C},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n,r){return C(e.TableColumn,this,"Add",0,[t,n,r],!1,!0,null,0)},n.prototype.getCount=function(){return w("TableColumnCollection.getCount",v,"1.4",O),E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.TableColumn,this,[t])},n.prototype.getItemAt=function(t){return C(e.TableColumn,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return w("TableColumnCollection.getItemOrNullObject",v,"1.4",O),C(e.TableColumn,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),N(r.Count)||(this._C=r.Count),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.TableColumn,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.TableColumn,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{count:this._C},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.TableColumn,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.TableColumnCollection=Ge;var ze="TableColumn",qe=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TableColumn"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","index","values","name"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Index","Values","Name"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!1,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["filter"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"filter",{get:function(){return w("TableColumn.filter",v,"1.2",O),this._F||(this._F=S(e.Filter,this,"Filter",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,ze,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"index",{get:function(){return T("index",this._In,ze,this._isNull),this._In},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,ze,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"values",{get:function(){return T("values",this._V,ze,this._isNull),this._V},set:function(e){this._V=e,I(this,"Values",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["values","name"],[],["filter"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.getDataBodyRange=function(){return C(e.Range,this,"GetDataBodyRange",1,[],!1,!0,null,4)},n.prototype.getHeaderRowRange=function(){return C(e.Range,this,"GetHeaderRowRange",1,[],!1,!0,null,4)},n.prototype.getRange=function(){return C(e.Range,this,"GetRange",1,[],!1,!0,null,4)},n.prototype.getTotalRowRange=function(){return C(e.Range,this,"GetTotalRowRange",1,[],!1,!0,null,4)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Id)||(this._I=n.Id),N(n.Index)||(this._In=n.Index),N(n.Name)||(this._N=n.Name),N(n.Values)||(this._V=n.Values),L(this,n,["filter","Filter"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{id:this._I,index:this._In,name:this._N,values:this._V},{filter:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.TableColumn=qe;var Je="TableRowCollection",Ke=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TableRowCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,Je,this._isNull),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"count",{get:function(){return T("count",this._C,Je,this._isNull),this._C},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n,r){return C(e.TableRow,this,"Add",0,[t,n,r],!1,!0,null,V(2,"ExcelApiUndo","99.0"))},n.prototype.deleteRows=function(e){w("TableRowCollection.deleteRows","ExcelApiOnline","1.1",O),E(this,"DeleteRows",0,[e],0,0)},n.prototype.deleteRowsAt=function(e,t){w("TableRowCollection.deleteRowsAt","ExcelApiOnline","1.1",O),E(this,"DeleteRowsAt",0,[e,t],0,0)},n.prototype.getCount=function(){return w("TableRowCollection.getCount",v,"1.4",O),E(this,"GetCount",1,[],4,0)},n.prototype.getItemAt=function(t){return C(e.TableRow,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),N(r.Count)||(this._C=r.Count),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.TableRow,!1,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.TableRow,!1,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{count:this._C},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.TableRow,!1,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.TableRowCollection=Ke;var Qe="TableRow",Xe=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TableRow"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["index","values"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Index","Values"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"index",{get:function(){return T("index",this._I,Qe,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"values",{get:function(){return T("values",this._V,Qe,this._isNull),this._V},set:function(e){this._V=e,I(this,"Values",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["values"],[],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){E(this,"Delete",0,[],V(2,"ExcelApiUndo","99.0"),0)},n.prototype.getRange=function(){return C(e.Range,this,"GetRange",1,[],!1,!0,null,4)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Index)||(this._I=n.Index),N(n.Values)||(this._V=n.Values)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{index:this._I,values:this._V},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.TableRow=Xe;var Ye="DataValidation",Ze=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"DataValidation"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["type","rule","prompt","errorAlert","ignoreBlanks","valid"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Type","Rule","Prompt","ErrorAlert","IgnoreBlanks","Valid"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0,!0,!0,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"errorAlert",{get:function(){return T("errorAlert",this._E,Ye,this._isNull),this._E},set:function(e){this._E=e,I(this,"ErrorAlert",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"ignoreBlanks",{get:function(){return T("ignoreBlanks",this._I,Ye,this._isNull),this._I},set:function(e){this._I=e,I(this,"IgnoreBlanks",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"prompt",{get:function(){return T("prompt",this._P,Ye,this._isNull),this._P},set:function(e){this._P=e,I(this,"Prompt",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rule",{get:function(){return T("rule",this._R,Ye,this._isNull),this._R},set:function(e){this._R=e,I(this,"Rule",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"type",{get:function(){return T("type",this._T,Ye,this._isNull),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"valid",{get:function(){return T("valid",this._V,Ye,this._isNull),this._V},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["rule","prompt","errorAlert","ignoreBlanks"],[],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.clear=function(){E(this,"Clear",0,[],0,0)},n.prototype.getInvalidCells=function(){return w("DataValidation.getInvalidCells",v,"1.9",O),C(e.RangeAreas,this,"GetInvalidCells",1,[],!1,!0,null,4)},n.prototype.getInvalidCellsOrNullObject=function(){return w("DataValidation.getInvalidCellsOrNullObject",v,"1.9",O),C(e.RangeAreas,this,"GetInvalidCellsOrNullObject",1,[],!1,!0,null,4)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.ErrorAlert)||(this._E=n.ErrorAlert),N(n.IgnoreBlanks)||(this._I=n.IgnoreBlanks),N(n.Prompt)||(this._P=n.Prompt),N(n.Rule)||(this._R=n.Rule),N(n.Type)||(this._T=n.Type),N(n.Valid)||(this._V=n.Valid)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{errorAlert:this._E,ignoreBlanks:this._I,prompt:this._P,rule:this._R,type:this._T,valid:this._V},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.DataValidation=Ze;var $e="RemoveDuplicatesResult",et=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"RemoveDuplicatesResult"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["removed","uniqueRemaining"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Removed","UniqueRemaining"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"removed",{get:function(){return T("removed",this._R,$e,this._isNull),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"uniqueRemaining",{get:function(){return T("uniqueRemaining",this._U,$e,this._isNull),this._U},enumerable:!0,configurable:!0}),t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Removed)||(this._R=n.Removed),N(n.UniqueRemaining)||(this._U=n.UniqueRemaining)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{removed:this._R,uniqueRemaining:this._U},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.RemoveDuplicatesResult=et;var tt="RangeFormat",nt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"RangeFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["wrapText","horizontalAlignment","verticalAlignment","columnWidth","rowHeight","textOrientation","useStandardHeight","useStandardWidth","readingOrder","shrinkToFit","indentLevel","autoIndent"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["WrapText","HorizontalAlignment","VerticalAlignment","ColumnWidth","RowHeight","TextOrientation","UseStandardHeight","UseStandardWidth","ReadingOrder","ShrinkToFit","IndentLevel","AutoIndent"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["fill","font","borders","protection"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"borders",{get:function(){return this._B||(this._B=S(e.RangeBorderCollection,this,"Borders",!0,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.RangeFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._Fo||(this._Fo=S(e.RangeFont,this,"Font",!1,4)),this._Fo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"protection",{get:function(){return w("RangeFormat.protection",v,"1.2",O),this._P||(this._P=S(e.FormatProtection,this,"Protection",!1,4)),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"autoIndent",{get:function(){return T("autoIndent",this._A,tt,this._isNull),w("RangeFormat.autoIndent",v,"1.9",O),this._A},set:function(e){this._A=e,I(this,"AutoIndent",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"columnWidth",{get:function(){return T("columnWidth",this._C,tt,this._isNull),w("RangeFormat.columnWidth",v,"1.2",O),this._C},set:function(e){this._C=e,I(this,"ColumnWidth",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"horizontalAlignment",{get:function(){return T("horizontalAlignment",this._H,tt,this._isNull),this._H},set:function(e){this._H=e,I(this,"HorizontalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"indentLevel",{get:function(){return T("indentLevel",this._I,tt,this._isNull),w("RangeFormat.indentLevel",v,"1.9",O),this._I},set:function(e){this._I=e,I(this,"IndentLevel",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"readingOrder",{get:function(){return T("readingOrder",this._R,tt,this._isNull),w("RangeFormat.readingOrder",v,"1.9",O),this._R},set:function(e){this._R=e,I(this,"ReadingOrder",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rowHeight",{get:function(){return T("rowHeight",this._Ro,tt,this._isNull),w("RangeFormat.rowHeight",v,"1.2",O),this._Ro},set:function(e){this._Ro=e,I(this,"RowHeight",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"shrinkToFit",{get:function(){return T("shrinkToFit",this._S,tt,this._isNull),w("RangeFormat.shrinkToFit",v,"1.9",O),this._S},set:function(e){this._S=e,I(this,"ShrinkToFit",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"textOrientation",{get:function(){return T("textOrientation",this._T,tt,this._isNull),w("RangeFormat.textOrientation",v,"1.7",O),this._T},set:function(e){this._T=e,I(this,"TextOrientation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"useStandardHeight",{get:function(){return T("useStandardHeight",this._U,tt,this._isNull),w("RangeFormat.useStandardHeight",v,"1.7",O),this._U},set:function(e){this._U=e,I(this,"UseStandardHeight",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"useStandardWidth",{get:function(){return T("useStandardWidth",this._Us,tt,this._isNull),w("RangeFormat.useStandardWidth",v,"1.7",O),this._Us},set:function(e){this._Us=e,I(this,"UseStandardWidth",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"verticalAlignment",{get:function(){return T("verticalAlignment",this._V,tt,this._isNull),this._V},set:function(e){this._V=e,I(this,"VerticalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"wrapText",{get:function(){return T("wrapText",this._W,tt,this._isNull),this._W},set:function(e){this._W=e,I(this,"WrapText",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["wrapText","horizontalAlignment","verticalAlignment","columnWidth","rowHeight","textOrientation","useStandardHeight","useStandardWidth","readingOrder","shrinkToFit","indentLevel","autoIndent"],["fill","font","protection"],["borders"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.adjustIndent=function(e){w("RangeFormat.adjustIndent",v,"1.11",O),E(this,"AdjustIndent",0,[e],0,0)},n.prototype.autofitColumns=function(){w("RangeFormat.autofitColumns",v,"1.2",O),E(this,"AutofitColumns",0,[],0,0)},n.prototype.autofitRows=function(){w("RangeFormat.autofitRows",v,"1.2",O),E(this,"AutofitRows",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AutoIndent)||(this._A=n.AutoIndent),N(n.ColumnWidth)||(this._C=n.ColumnWidth),N(n.HorizontalAlignment)||(this._H=n.HorizontalAlignment),N(n.IndentLevel)||(this._I=n.IndentLevel),N(n.ReadingOrder)||(this._R=n.ReadingOrder),N(n.RowHeight)||(this._Ro=n.RowHeight),N(n.ShrinkToFit)||(this._S=n.ShrinkToFit),N(n.TextOrientation)||(this._T=n.TextOrientation),N(n.UseStandardHeight)||(this._U=n.UseStandardHeight),N(n.UseStandardWidth)||(this._Us=n.UseStandardWidth),N(n.VerticalAlignment)||(this._V=n.VerticalAlignment),N(n.WrapText)||(this._W=n.WrapText),L(this,n,["borders","Borders","fill","Fill","font","Font","protection","Protection"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{autoIndent:this._A,columnWidth:this._C,horizontalAlignment:this._H,indentLevel:this._I,readingOrder:this._R,rowHeight:this._Ro,shrinkToFit:this._S,textOrientation:this._T,useStandardHeight:this._U,useStandardWidth:this._Us,verticalAlignment:this._V,wrapText:this._W},{borders:this._B,fill:this._F,font:this._Fo,protection:this._P})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.RangeFormat=nt;var rt="FormatProtection",it=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"FormatProtection"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["locked","formulaHidden"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Locked","FormulaHidden"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"formulaHidden",{get:function(){return T("formulaHidden",this._F,rt,this._isNull),this._F},set:function(e){this._F=e,I(this,"FormulaHidden",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"locked",{get:function(){return T("locked",this._L,rt,this._isNull),this._L},set:function(e){this._L=e,I(this,"Locked",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["locked","formulaHidden"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.FormulaHidden)||(this._F=n.FormulaHidden),N(n.Locked)||(this._L=n.Locked)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{formulaHidden:this._F,locked:this._L},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.FormatProtection=it;var ot="RangeFill",st=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"RangeFill"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["color","tintAndShade","patternTintAndShade","pattern","patternColor"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Color","TintAndShade","PatternTintAndShade","Pattern","PatternColor"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return T("color",this._C,ot,this._isNull),this._C},set:function(e){this._C=e,I(this,"Color",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"pattern",{get:function(){return T("pattern",this._P,ot,this._isNull),w("RangeFill.pattern",v,"1.9",O),this._P},set:function(e){this._P=e,I(this,"Pattern",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"patternColor",{get:function(){return T("patternColor",this._Pa,ot,this._isNull),w("RangeFill.patternColor",v,"1.9",O),this._Pa},set:function(e){this._Pa=e,I(this,"PatternColor",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"patternTintAndShade",{get:function(){return T("patternTintAndShade",this._Pat,ot,this._isNull),w("RangeFill.patternTintAndShade",v,"1.9",O),this._Pat},set:function(e){this._Pat=e,I(this,"PatternTintAndShade",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tintAndShade",{get:function(){return T("tintAndShade",this._T,ot,this._isNull),w("RangeFill.tintAndShade",v,"1.9",O),this._T},set:function(e){this._T=e,I(this,"TintAndShade",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["color","tintAndShade","patternTintAndShade","pattern","patternColor"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype.clear=function(){E(this,"Clear",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Color)||(this._C=n.Color),N(n.Pattern)||(this._P=n.Pattern),N(n.PatternColor)||(this._Pa=n.PatternColor),N(n.PatternTintAndShade)||(this._Pat=n.PatternTintAndShade),N(n.TintAndShade)||(this._T=n.TintAndShade)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{color:this._C,pattern:this._P,patternColor:this._Pa,patternTintAndShade:this._Pat,tintAndShade:this._T},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.RangeFill=st;var at="RangeBorder",lt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"RangeBorder"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["sideIndex","style","weight","color","tintAndShade"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["SideIndex","Style","Weight","Color","TintAndShade"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return T("color",this._C,at,this._isNull),this._C},set:function(e){this._C=e,I(this,"Color",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"sideIndex",{get:function(){return T("sideIndex",this._S,at,this._isNull),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"style",{get:function(){return T("style",this._St,at,this._isNull),this._St},set:function(e){this._St=e,I(this,"Style",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tintAndShade",{get:function(){return T("tintAndShade",this._T,at,this._isNull),w("RangeBorder.tintAndShade",v,"1.9",O),this._T},set:function(e){this._T=e,I(this,"TintAndShade",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"weight",{get:function(){return T("weight",this._W,at,this._isNull),this._W},set:function(e){this._W=e,I(this,"Weight",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["style","weight","color","tintAndShade"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Color)||(this._C=n.Color),N(n.SideIndex)||(this._S=n.SideIndex),N(n.Style)||(this._St=n.Style),N(n.TintAndShade)||(this._T=n.TintAndShade),N(n.Weight)||(this._W=n.Weight)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{color:this._C,sideIndex:this._S,style:this._St,tintAndShade:this._T,weight:this._W},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.RangeBorder=lt;var ut="RangeBorderCollection",ct=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"RangeBorderCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["count","tintAndShade"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Count","TintAndShade"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,ut,this._isNull),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"count",{get:function(){return T("count",this._C,ut,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"tintAndShade",{get:function(){return T("tintAndShade",this._T,ut,this._isNull),w("RangeBorderCollection.tintAndShade",v,"1.9",O),this._T},set:function(e){this._T=e,I(this,"TintAndShade",e,0)},enumerable:!0,configurable:!0}),n.prototype.getItem=function(t){return P(e.RangeBorder,this,[t])},n.prototype.getItemAt=function(t){return C(e.RangeBorder,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),N(r.Count)||(this._C=r.Count),N(r.TintAndShade)||(this._T=r.TintAndShade),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.RangeBorder,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.RangeBorder,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{count:this._C,tintAndShade:this._T},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.RangeBorder,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.RangeBorderCollection=ct;var pt="RangeFont",ft=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"RangeFont"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["name","size","color","italic","bold","underline","strikethrough","subscript","superscript","tintAndShade"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name","Size","Color","Italic","Bold","Underline","Strikethrough","Subscript","Superscript","TintAndShade"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"bold",{get:function(){return T("bold",this._B,pt,this._isNull),this._B},set:function(e){this._B=e,I(this,"Bold",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return T("color",this._C,pt,this._isNull),this._C},set:function(e){this._C=e,I(this,"Color",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"italic",{get:function(){return T("italic",this._I,pt,this._isNull),this._I},set:function(e){this._I=e,I(this,"Italic",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"name",{get:function(){return T("name",this._N,pt,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"size",{get:function(){return T("size",this._S,pt,this._isNull),this._S},set:function(e){this._S=e,I(this,"Size",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strikethrough",{get:function(){return T("strikethrough",this._St,pt,this._isNull),w("RangeFont.strikethrough",v,"1.9",O),this._St},set:function(e){this._St=e,I(this,"Strikethrough",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"subscript",{get:function(){return T("subscript",this._Su,pt,this._isNull),w("RangeFont.subscript",v,"1.9",O),this._Su},set:function(e){this._Su=e,I(this,"Subscript",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"superscript",{get:function(){return T("superscript",this._Sup,pt,this._isNull),w("RangeFont.superscript",v,"1.9",O),this._Sup},set:function(e){this._Sup=e,I(this,"Superscript",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tintAndShade",{get:function(){return T("tintAndShade",this._T,pt,this._isNull),w("RangeFont.tintAndShade",v,"1.9",O),this._T},set:function(e){this._T=e,I(this,"TintAndShade",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"underline",{get:function(){return T("underline",this._U,pt,this._isNull),this._U},set:function(e){this._U=e,I(this,"Underline",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["name","size","color","italic","bold","underline","strikethrough","subscript","superscript","tintAndShade"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Bold)||(this._B=n.Bold),N(n.Color)||(this._C=n.Color),N(n.Italic)||(this._I=n.Italic),N(n.Name)||(this._N=n.Name),N(n.Size)||(this._S=n.Size),N(n.Strikethrough)||(this._St=n.Strikethrough),N(n.Subscript)||(this._Su=n.Subscript),N(n.Superscript)||(this._Sup=n.Superscript),N(n.TintAndShade)||(this._T=n.TintAndShade),N(n.Underline)||(this._U=n.Underline)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{bold:this._B,color:this._C,italic:this._I,name:this._N,size:this._S,strikethrough:this._St,subscript:this._Su,superscript:this._Sup,tintAndShade:this._T,underline:this._U},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.RangeFont=ft;var ht="ChartCollection",dt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,ht,this._isNull),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"count",{get:function(){return T("count",this._C,ht,this._isNull),this._C},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n,r){var i=b.ChartCollection_Add(this,t,n,r),o=i.handled,s=i.result;return o?s:C(e.Chart,this,"Add",0,[t,n,r],!1,!0,null,V(2,"ExcelApiUndo","1.5"))},n.prototype.getCount=function(){return w("ChartCollection.getCount",v,"1.4",O),E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return C(e.Chart,this,"GetItem",1,[t],!1,!1,null,4)},n.prototype.getItemAt=function(t){return C(e.Chart,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return w("ChartCollection.getItemOrNullObject",v,"1.4",O),C(e.Chart,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._GetItem=function(t){return P(e.Chart,this,[t])},n.prototype._RegisterActivatedEvent=function(){w("ChartCollection._RegisterActivatedEvent",v,"1.8",O),E(this,"_RegisterActivatedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._RegisterAddedEvent=function(){w("ChartCollection._RegisterAddedEvent",v,"1.8",O),E(this,"_RegisterAddedEvent",0,[],0,0)},n.prototype._RegisterDeactivatedEvent=function(){w("ChartCollection._RegisterDeactivatedEvent",v,"1.8",O),E(this,"_RegisterDeactivatedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._RegisterDeletedEvent=function(){w("ChartCollection._RegisterDeletedEvent",v,"1.8",O),E(this,"_RegisterDeletedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._UnregisterActivatedEvent=function(){w("ChartCollection._UnregisterActivatedEvent",v,"1.8",O),E(this,"_UnregisterActivatedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._UnregisterAddedEvent=function(){w("ChartCollection._UnregisterAddedEvent",v,"1.8",O),E(this,"_UnregisterAddedEvent",0,[],0,0)},n.prototype._UnregisterDeactivatedEvent=function(){w("ChartCollection._UnregisterDeactivatedEvent",v,"1.8",O),E(this,"_UnregisterDeactivatedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._UnregisterDeletedEvent=function(){w("ChartCollection._UnregisterDeletedEvent",v,"1.8",O),E(this,"_UnregisterDeletedEvent",0,[],V(2,"ExcelApiUndo","1.2"),0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),N(r.Count)||(this._C=r.Count),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Chart,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Chart,!0,i,t,n)}))},Object.defineProperty(n.prototype,"onActivated",{get:function(){var e=this;return w("ChartCollection.onActivated",v,"1.8",O),this.m_activated||(this.m_activated=new OfficeExtension.GenericEventHandlers(this.context,this,"Activated",{eventType:51,registerFunc:function(){return e._RegisterActivatedEvent()},unregisterFunc:function(){return e._UnregisterActivatedEvent()},getTargetIdFunc:function(){return e._ParentObject.id},eventArgsTransformFunc:function(e){var t={type:Oo.chartActivated,chartId:e.chartId,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_activated},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onAdded",{get:function(){var e=this;return w("ChartCollection.onAdded",v,"1.8",O),this.m_added||(this.m_added=new OfficeExtension.GenericEventHandlers(this.context,this,"Added",{eventType:50,registerFunc:function(){return e._RegisterAddedEvent()},unregisterFunc:function(){return e._UnregisterAddedEvent()},getTargetIdFunc:function(){return e._ParentObject.id},eventArgsTransformFunc:function(e){var t={type:Oo.chartAdded,chartId:e.chartId,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_added},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onDeactivated",{get:function(){var e=this;return w("ChartCollection.onDeactivated",v,"1.8",O),this.m_deactivated||(this.m_deactivated=new OfficeExtension.GenericEventHandlers(this.context,this,"Deactivated",{eventType:52,registerFunc:function(){return e._RegisterDeactivatedEvent()},unregisterFunc:function(){return e._UnregisterDeactivatedEvent()},getTargetIdFunc:function(){return e._ParentObject.id},eventArgsTransformFunc:function(e){var t={type:Oo.chartDeactivated,chartId:e.chartId,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_deactivated},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onDeleted",{get:function(){var e=this;return w("ChartCollection.onDeleted",v,"1.8",O),this.m_deleted||(this.m_deleted=new OfficeExtension.GenericEventHandlers(this.context,this,"Deleted",{eventType:53,registerFunc:function(){return e._RegisterDeletedEvent()},unregisterFunc:function(){return e._UnregisterDeletedEvent()},getTargetIdFunc:function(){return e._ParentObject.id},eventArgsTransformFunc:function(e){var t={type:Oo.chartDeleted,chartId:e.chartId,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_deleted},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{count:this._C},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Chart,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.ChartCollection=dt;var yt=function(){function e(){}return Object.defineProperty(e.prototype,"_ParentObject",{get:function(){return this.m__ParentObject},set:function(e){this.m__ParentObject=e},enumerable:!0,configurable:!0}),e}();e.ChartCollectionCustom=yt,OfficeExtension.Utility.applyMixin(dt,yt),function(t){t.ChartCollection_Add=function(t,n,r,i){if(!(r instanceof e.Range))throw OfficeExtension.Utility.createRuntimeError(OfficeExtension.ResourceStrings.invalidArgument,"sourceData","Charts.Add");return{handled:!1,result:null}}}(b=e._CC||(e._CC={}));var mt="Chart",gt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Chart"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["name","top","left","width","height","id","showAllFieldButtons","chartType","showDataLabelsOverMaximum","categoryLabelLevel","style","displayBlanksAs","plotBy","plotVisibleOnly","seriesNameLevel"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name","Top","Left","Width","Height","Id","ShowAllFieldButtons","ChartType","ShowDataLabelsOverMaximum","CategoryLabelLevel","Style","DisplayBlanksAs","PlotBy","PlotVisibleOnly","SeriesNameLevel"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!1,!0,!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["title","dataLabels","legend","series","axes","format","worksheet","plotArea","pivotOptions"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"axes",{get:function(){return this._A||(this._A=S(e.ChartAxes,this,"Axes",!1,4)),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"dataLabels",{get:function(){return this._D||(this._D=S(e.ChartDataLabels,this,"DataLabels",!1,4)),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartAreaFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"legend",{get:function(){return this._Le||(this._Le=S(e.ChartLegend,this,"Legend",!1,4)),this._Le},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"pivotOptions",{get:function(){return w("Chart.pivotOptions",v,"1.9",O),this._P||(this._P=S(e.ChartPivotOptions,this,"PivotOptions",!1,4)),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"plotArea",{get:function(){return w("Chart.plotArea",v,"1.8",O),this._Pl||(this._Pl=S(e.ChartPlotArea,this,"PlotArea",!1,4)),this._Pl},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"series",{get:function(){return this._S||(this._S=S(e.ChartSeriesCollection,this,"Series",!0,4)),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"title",{get:function(){return this._T||(this._T=S(e.ChartTitle,this,"Title",!1,4)),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"worksheet",{get:function(){return w("Chart.worksheet",v,"1.2",O),this._Wo||(this._Wo=S(e.Worksheet,this,"Worksheet",!1,4)),this._Wo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"categoryLabelLevel",{get:function(){return T("categoryLabelLevel",this._C,mt,this._isNull),w("Chart.categoryLabelLevel",v,"1.8",O),this._C},set:function(e){this._C=e,I(this,"CategoryLabelLevel",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"chartType",{get:function(){return T("chartType",this._Ch,mt,this._isNull),w("Chart.chartType",v,"1.7",O),this._Ch},set:function(e){this._Ch=e,I(this,"ChartType",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"displayBlanksAs",{get:function(){return T("displayBlanksAs",this._Di,mt,this._isNull),w("Chart.displayBlanksAs",v,"1.8",O),this._Di},set:function(e){this._Di=e,I(this,"DisplayBlanksAs",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"height",{get:function(){return T("height",this._H,mt,this._isNull),this._H},set:function(e){this._H=e,I(this,"Height",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,mt,this._isNull),w("Chart.id",v,"1.7",O),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"left",{get:function(){return T("left",this._L,mt,this._isNull),this._L},set:function(e){this._L=e,I(this,"Left",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,mt,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"plotBy",{get:function(){return T("plotBy",this._Plo,mt,this._isNull),w("Chart.plotBy",v,"1.8",O),this._Plo},set:function(e){this._Plo=e,I(this,"PlotBy",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"plotVisibleOnly",{get:function(){return T("plotVisibleOnly",this._Plot,mt,this._isNull),w("Chart.plotVisibleOnly",v,"1.8",O),this._Plot},set:function(e){this._Plot=e,I(this,"PlotVisibleOnly",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"seriesNameLevel",{get:function(){return T("seriesNameLevel",this._Se,mt,this._isNull),w("Chart.seriesNameLevel",v,"1.8",O),this._Se},set:function(e){this._Se=e,I(this,"SeriesNameLevel",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showAllFieldButtons",{get:function(){return T("showAllFieldButtons",this._Sh,mt,this._isNull),w("Chart.showAllFieldButtons",v,"1.7",O),this._Sh},set:function(e){this._Sh=e,I(this,"ShowAllFieldButtons",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showDataLabelsOverMaximum",{get:function(){return T("showDataLabelsOverMaximum",this._Sho,mt,this._isNull),w("Chart.showDataLabelsOverMaximum",v,"1.8",O),this._Sho},set:function(e){this._Sho=e,I(this,"ShowDataLabelsOverMaximum",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"style",{get:function(){return T("style",this._St,mt,this._isNull),w("Chart.style",v,"1.8",O),this._St},set:function(e){this._St=e,I(this,"Style",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"top",{get:function(){return T("top",this._To,mt,this._isNull),this._To},set:function(e){this._To=e,I(this,"Top",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"width",{get:function(){return T("width",this._W,mt,this._isNull),this._W},set:function(e){this._W=e,I(this,"Width",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name","top","left","width","height","showAllFieldButtons","chartType","showDataLabelsOverMaximum","categoryLabelLevel","style","displayBlanksAs","plotBy","plotVisibleOnly","seriesNameLevel"],["title","dataLabels","legend","axes","format","plotArea","pivotOptions"],["series","worksheet"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.activate=function(){w("Chart.activate",v,"1.9",O),E(this,"Activate",1,[],0,0)},n.prototype.delete=function(){E(this,"Delete",0,[],V(2,"ExcelApiUndo","1.5"),0)},n.prototype.getDataTable=function(){return w("Chart.getDataTable",v,"1.14",O),C(e.ChartDataTable,this,"GetDataTable",1,[],!1,!1,null,4)},n.prototype.getDataTableOrNullObject=function(){return w("Chart.getDataTableOrNullObject",v,"1.14",O),C(e.ChartDataTable,this,"GetDataTableOrNullObject",1,[],!1,!1,null,4)},n.prototype.getImage=function(e,t,n){return w("Chart.getImage",v,"1.2",O),E(this,"GetImage",1,[e,t,n],4,0)},n.prototype.setData=function(e,t){b.Chart_SetData(this,e,t).handled||E(this,"SetData",0,[e,t],V(2,"ExcelApiUndo","1.5"),0)},n.prototype.setPosition=function(e,t){E(this,"SetPosition",0,[e,t],V(2,"ExcelApiUndo","1.5"),0)},n.prototype._RegisterActivatedEvent=function(){w("Chart._RegisterActivatedEvent",v,"1.8",O),E(this,"_RegisterActivatedEvent",0,[],0,0)},n.prototype._RegisterDeactivatedEvent=function(){w("Chart._RegisterDeactivatedEvent",v,"1.8",O),E(this,"_RegisterDeactivatedEvent",0,[],0,0)},n.prototype._UnregisterActivatedEvent=function(){w("Chart._UnregisterActivatedEvent",v,"1.8",O),E(this,"_UnregisterActivatedEvent",0,[],0,0)},n.prototype._UnregisterDeactivatedEvent=function(){w("Chart._UnregisterDeactivatedEvent",v,"1.8",O),E(this,"_UnregisterDeactivatedEvent",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.CategoryLabelLevel)||(this._C=n.CategoryLabelLevel),N(n.ChartType)||(this._Ch=n.ChartType),N(n.DisplayBlanksAs)||(this._Di=n.DisplayBlanksAs),N(n.Height)||(this._H=n.Height),N(n.Id)||(this._I=n.Id),N(n.Left)||(this._L=n.Left),N(n.Name)||(this._N=n.Name),N(n.PlotBy)||(this._Plo=n.PlotBy),N(n.PlotVisibleOnly)||(this._Plot=n.PlotVisibleOnly),N(n.SeriesNameLevel)||(this._Se=n.SeriesNameLevel),N(n.ShowAllFieldButtons)||(this._Sh=n.ShowAllFieldButtons),N(n.ShowDataLabelsOverMaximum)||(this._Sho=n.ShowDataLabelsOverMaximum),N(n.Style)||(this._St=n.Style),N(n.Top)||(this._To=n.Top),N(n.Width)||(this._W=n.Width),L(this,n,["axes","Axes","dataLabels","DataLabels","format","Format","legend","Legend","pivotOptions","PivotOptions","plotArea","PlotArea","series","Series","title","Title","worksheet","Worksheet"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},Object.defineProperty(n.prototype,"onActivated",{get:function(){var e=this;return w("Chart.onActivated",v,"1.8",O),this.m_activated||(this.m_activated=new OfficeExtension.GenericEventHandlers(this.context,this,"Activated",{eventType:51,registerFunc:function(){return e._RegisterActivatedEvent()},unregisterFunc:function(){return e._UnregisterActivatedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.chartActivated,chartId:e.chartId,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_activated},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onDeactivated",{get:function(){var e=this;return w("Chart.onDeactivated",v,"1.8",O),this.m_deactivated||(this.m_deactivated=new OfficeExtension.GenericEventHandlers(this.context,this,"Deactivated",{eventType:52,registerFunc:function(){return e._RegisterDeactivatedEvent()},unregisterFunc:function(){return e._UnregisterDeactivatedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.chartDeactivated,chartId:e.chartId,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_deactivated},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{categoryLabelLevel:this._C,chartType:this._Ch,displayBlanksAs:this._Di,height:this._H,id:this._I,left:this._L,name:this._N,plotBy:this._Plo,plotVisibleOnly:this._Plot,seriesNameLevel:this._Se,showAllFieldButtons:this._Sh,showDataLabelsOverMaximum:this._Sho,style:this._St,top:this._To,width:this._W},{axes:this._A,dataLabels:this._D,format:this._F,legend:this._Le,pivotOptions:this._P,plotArea:this._Pl,series:this._S,title:this._T})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Chart=gt,function(t){t.Chart_SetData=function(t,n,r){if(!(n instanceof e.Range))throw OfficeExtension.Utility.createRuntimeError(OfficeExtension.ResourceStrings.invalidArgument,"sourceData","Chart.setData");return{handled:!1}}}(b=e._CC||(e._CC={}));var _t="ChartPivotOptions",bt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ChartPivotOptions"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["showAxisFieldButtons","showLegendFieldButtons","showReportFilterFieldButtons","showValueFieldButtons"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["ShowAxisFieldButtons","ShowLegendFieldButtons","ShowReportFilterFieldButtons","ShowValueFieldButtons"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"showAxisFieldButtons",{get:function(){return T("showAxisFieldButtons",this._S,_t,this._isNull),this._S},set:function(e){this._S=e,I(this,"ShowAxisFieldButtons",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"showLegendFieldButtons",{get:function(){return T("showLegendFieldButtons",this._Sh,_t,this._isNull),this._Sh},set:function(e){this._Sh=e,I(this,"ShowLegendFieldButtons",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"showReportFilterFieldButtons",{get:function(){return T("showReportFilterFieldButtons",this._Sho,_t,this._isNull),this._Sho},set:function(e){this._Sho=e,I(this,"ShowReportFilterFieldButtons",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"showValueFieldButtons",{get:function(){return T("showValueFieldButtons",this._Show,_t,this._isNull),this._Show},set:function(e){this._Show=e,I(this,"ShowValueFieldButtons",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["showAxisFieldButtons","showLegendFieldButtons","showReportFilterFieldButtons","showValueFieldButtons"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.ShowAxisFieldButtons)||(this._S=n.ShowAxisFieldButtons),N(n.ShowLegendFieldButtons)||(this._Sh=n.ShowLegendFieldButtons),N(n.ShowReportFilterFieldButtons)||(this._Sho=n.ShowReportFilterFieldButtons),N(n.ShowValueFieldButtons)||(this._Show=n.ShowValueFieldButtons)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{showAxisFieldButtons:this._S,showLegendFieldButtons:this._Sh,showReportFilterFieldButtons:this._Sho,showValueFieldButtons:this._Show},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ChartPivotOptions=bt;var Ot="ChartAreaFormat",vt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartAreaFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["roundedCorners","colorScheme"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["RoundedCorners","ColorScheme"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["fill","font","border"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"border",{get:function(){return w("ChartAreaFormat.border",v,"1.7",O),this._B||(this._B=S(e.ChartBorder,this,"Border",!1,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.ChartFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._Fo||(this._Fo=S(e.ChartFont,this,"Font",!1,4)),this._Fo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"colorScheme",{get:function(){return T("colorScheme",this._C,Ot,this._isNull),w("ChartAreaFormat.colorScheme",v,"1.9",O),this._C},set:function(e){this._C=e,I(this,"ColorScheme",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"roundedCorners",{get:function(){return T("roundedCorners",this._R,Ot,this._isNull),w("ChartAreaFormat.roundedCorners",v,"1.9",O),this._R},set:function(e){this._R=e,I(this,"RoundedCorners",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["roundedCorners","colorScheme"],["font","border"],["fill"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.ColorScheme)||(this._C=n.ColorScheme),N(n.RoundedCorners)||(this._R=n.RoundedCorners),L(this,n,["border","Border","fill","Fill","font","Font"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{colorScheme:this._C,roundedCorners:this._R},{border:this._B,font:this._Fo})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartAreaFormat=vt;var St="ChartSeriesCollection",Ct=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartSeriesCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,St,this._isNull),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"count",{get:function(){return T("count",this._C,St,this._isNull),this._C},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n){return w("ChartSeriesCollection.add",v,"1.7",O),C(e.ChartSeries,this,"Add",0,[t,n],!1,!0,null,V(2,"ExcelApiUndo","1.5"))},n.prototype.getCount=function(){return w("ChartSeriesCollection.getCount",v,"1.4",O),E(this,"GetCount",1,[],4,0)},n.prototype.getItemAt=function(t){return C(e.ChartSeries,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),N(r.Count)||(this._C=r.Count),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.ChartSeries,!1,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.ChartSeries,!1,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{count:this._C},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.ChartSeries,!1,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.ChartSeriesCollection=Ct;var Pt="ChartSeries",At=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartSeries"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["name","chartType","hasDataLabels","filtered","markerSize","markerStyle","showShadow","markerBackgroundColor","markerForegroundColor","smooth","plotOrder","gapWidth","doughnutHoleSize","axisGroup","explosion","firstSliceAngle","invertIfNegative","bubbleScale","secondPlotSize","splitType","splitValue","varyByCategories","showLeaderLines","overlap","gradientStyle","gradientMinimumType","gradientMidpointType","gradientMaximumType","gradientMinimumValue","gradientMidpointValue","gradientMaximumValue","gradientMinimumColor","gradientMidpointColor","gradientMaximumColor","parentLabelStrategy","showConnectorLines","invertColor"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name","ChartType","HasDataLabels","Filtered","MarkerSize","MarkerStyle","ShowShadow","MarkerBackgroundColor","MarkerForegroundColor","Smooth","PlotOrder","GapWidth","DoughnutHoleSize","AxisGroup","Explosion","FirstSliceAngle","InvertIfNegative","BubbleScale","SecondPlotSize","SplitType","SplitValue","VaryByCategories","ShowLeaderLines","Overlap","GradientStyle","GradientMinimumType","GradientMidpointType","GradientMaximumType","GradientMinimumValue","GradientMidpointValue","GradientMaximumValue","GradientMinimumColor","GradientMidpointColor","GradientMaximumColor","ParentLabelStrategy","ShowConnectorLines","InvertColor"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["points","format","trendlines","xErrorBars","yErrorBars","dataLabels","binOptions","mapOptions","boxwhiskerOptions"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"binOptions",{get:function(){return w("ChartSeries.binOptions",v,"1.9",O),this._B||(this._B=S(e.ChartBinOptions,this,"BinOptions",!1,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"boxwhiskerOptions",{get:function(){return w("ChartSeries.boxwhiskerOptions",v,"1.9",O),this._Bo||(this._Bo=S(e.ChartBoxwhiskerOptions,this,"BoxwhiskerOptions",!1,4)),this._Bo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"dataLabels",{get:function(){return w("ChartSeries.dataLabels",v,"1.8",O),this._D||(this._D=S(e.ChartDataLabels,this,"DataLabels",!1,4)),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._Fo||(this._Fo=S(e.ChartSeriesFormat,this,"Format",!1,4)),this._Fo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"mapOptions",{get:function(){return w("ChartSeries.mapOptions",v,"1.9",O),this._M||(this._M=S(e.ChartMapOptions,this,"MapOptions",!1,4)),this._M},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"points",{get:function(){return this._Po||(this._Po=S(e.ChartPointsCollection,this,"Points",!0,4)),this._Po},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"trendlines",{get:function(){return w("ChartSeries.trendlines",v,"1.7",O),this._T||(this._T=S(e.ChartTrendlineCollection,this,"Trendlines",!0,4)),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"xErrorBars",{get:function(){return w("ChartSeries.xErrorBars",v,"1.9",O),this._X||(this._X=S(e.ChartErrorBars,this,"XErrorBars",!1,4)),this._X},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"yErrorBars",{get:function(){return w("ChartSeries.yErrorBars",v,"1.9",O),this._Y||(this._Y=S(e.ChartErrorBars,this,"YErrorBars",!1,4)),this._Y},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"axisGroup",{get:function(){return T("axisGroup",this._A,Pt,this._isNull),w("ChartSeries.axisGroup",v,"1.8",O),this._A},set:function(e){this._A=e,I(this,"AxisGroup",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"bubbleScale",{get:function(){return T("bubbleScale",this._Bu,Pt,this._isNull),w("ChartSeries.bubbleScale",v,"1.9",O),this._Bu},set:function(e){this._Bu=e,I(this,"BubbleScale",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"chartType",{get:function(){return T("chartType",this._C,Pt,this._isNull),w("ChartSeries.chartType",v,"1.7",O),this._C},set:function(e){this._C=e,I(this,"ChartType",e,V(2,"ExcelApiUndo","1.6"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"doughnutHoleSize",{get:function(){return T("doughnutHoleSize",this._Do,Pt,this._isNull),w("ChartSeries.doughnutHoleSize",v,"1.7",O),this._Do},set:function(e){this._Do=e,I(this,"DoughnutHoleSize",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"explosion",{get:function(){return T("explosion",this._E,Pt,this._isNull),w("ChartSeries.explosion",v,"1.8",O),this._E},set:function(e){this._E=e,I(this,"Explosion",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"filtered",{get:function(){return T("filtered",this._F,Pt,this._isNull),w("ChartSeries.filtered",v,"1.7",O),this._F},set:function(e){this._F=e,I(this,"Filtered",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"firstSliceAngle",{get:function(){return T("firstSliceAngle",this._Fi,Pt,this._isNull),w("ChartSeries.firstSliceAngle",v,"1.8",O),this._Fi},set:function(e){this._Fi=e,I(this,"FirstSliceAngle",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"gapWidth",{get:function(){return T("gapWidth",this._G,Pt,this._isNull),w("ChartSeries.gapWidth",v,"1.7",O),this._G},set:function(e){this._G=e,I(this,"GapWidth",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"gradientMaximumColor",{get:function(){return T("gradientMaximumColor",this._Gr,Pt,this._isNull),w("ChartSeries.gradientMaximumColor",v,"1.9",O),this._Gr},set:function(e){this._Gr=e,I(this,"GradientMaximumColor",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"gradientMaximumType",{get:function(){return T("gradientMaximumType",this._Gra,Pt,this._isNull),w("ChartSeries.gradientMaximumType",v,"1.9",O),this._Gra},set:function(e){this._Gra=e,I(this,"GradientMaximumType",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"gradientMaximumValue",{get:function(){return T("gradientMaximumValue",this._Grad,Pt,this._isNull),w("ChartSeries.gradientMaximumValue",v,"1.9",O),this._Grad},set:function(e){this._Grad=e,I(this,"GradientMaximumValue",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"gradientMidpointColor",{get:function(){return T("gradientMidpointColor",this._Gradi,Pt,this._isNull),w("ChartSeries.gradientMidpointColor",v,"1.9",O),this._Gradi},set:function(e){this._Gradi=e,I(this,"GradientMidpointColor",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"gradientMidpointType",{get:function(){return T("gradientMidpointType",this._Gradie,Pt,this._isNull),w("ChartSeries.gradientMidpointType",v,"1.9",O),this._Gradie},set:function(e){this._Gradie=e,I(this,"GradientMidpointType",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"gradientMidpointValue",{get:function(){return T("gradientMidpointValue",this._Gradien,Pt,this._isNull),w("ChartSeries.gradientMidpointValue",v,"1.9",O),this._Gradien},set:function(e){this._Gradien=e,I(this,"GradientMidpointValue",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"gradientMinimumColor",{get:function(){return T("gradientMinimumColor",this._Gradient,Pt,this._isNull),w("ChartSeries.gradientMinimumColor",v,"1.9",O),this._Gradient},set:function(e){this._Gradient=e,I(this,"GradientMinimumColor",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"gradientMinimumType",{get:function(){return T("gradientMinimumType",this._GradientM,Pt,this._isNull),w("ChartSeries.gradientMinimumType",v,"1.9",O),this._GradientM},set:function(e){this._GradientM=e,I(this,"GradientMinimumType",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"gradientMinimumValue",{get:function(){return T("gradientMinimumValue",this._GradientMi,Pt,this._isNull),w("ChartSeries.gradientMinimumValue",v,"1.9",O),this._GradientMi},set:function(e){this._GradientMi=e,I(this,"GradientMinimumValue",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"gradientStyle",{get:function(){return T("gradientStyle",this._GradientS,Pt,this._isNull),w("ChartSeries.gradientStyle",v,"1.9",O),this._GradientS},set:function(e){this._GradientS=e,I(this,"GradientStyle",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"hasDataLabels",{get:function(){return T("hasDataLabels",this._H,Pt,this._isNull),w("ChartSeries.hasDataLabels",v,"1.7",O),this._H},set:function(e){this._H=e,I(this,"HasDataLabels",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"invertColor",{get:function(){return T("invertColor",this._I,Pt,this._isNull),w("ChartSeries.invertColor",v,"1.9",O),this._I},set:function(e){this._I=e,I(this,"InvertColor",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"invertIfNegative",{get:function(){return T("invertIfNegative",this._In,Pt,this._isNull),w("ChartSeries.invertIfNegative",v,"1.8",O),this._In},set:function(e){this._In=e,I(this,"InvertIfNegative",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"markerBackgroundColor",{get:function(){return T("markerBackgroundColor",this._Ma,Pt,this._isNull),w("ChartSeries.markerBackgroundColor",v,"1.7",O),this._Ma},set:function(e){this._Ma=e,I(this,"MarkerBackgroundColor",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"markerForegroundColor",{get:function(){return T("markerForegroundColor",this._Mar,Pt,this._isNull),w("ChartSeries.markerForegroundColor",v,"1.7",O),this._Mar},set:function(e){this._Mar=e,I(this,"MarkerForegroundColor",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"markerSize",{get:function(){return T("markerSize",this._Mark,Pt,this._isNull),w("ChartSeries.markerSize",v,"1.7",O),this._Mark},set:function(e){this._Mark=e,I(this,"MarkerSize",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"markerStyle",{get:function(){return T("markerStyle",this._Marke,Pt,this._isNull),w("ChartSeries.markerStyle",v,"1.7",O),this._Marke},set:function(e){this._Marke=e,I(this,"MarkerStyle",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,Pt,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"overlap",{get:function(){return T("overlap",this._O,Pt,this._isNull),w("ChartSeries.overlap",v,"1.8",O),this._O},set:function(e){this._O=e,I(this,"Overlap",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"parentLabelStrategy",{get:function(){return T("parentLabelStrategy",this._P,Pt,this._isNull),w("ChartSeries.parentLabelStrategy",v,"1.9",O),this._P},set:function(e){this._P=e,I(this,"ParentLabelStrategy",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"plotOrder",{get:function(){return T("plotOrder",this._Pl,Pt,this._isNull),w("ChartSeries.plotOrder",v,"1.7",O),this._Pl},set:function(e){this._Pl=e,I(this,"PlotOrder",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"secondPlotSize",{get:function(){return T("secondPlotSize",this._S,Pt,this._isNull),w("ChartSeries.secondPlotSize",v,"1.8",O),this._S},set:function(e){this._S=e,I(this,"SecondPlotSize",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showConnectorLines",{get:function(){return T("showConnectorLines",this._Sh,Pt,this._isNull),w("ChartSeries.showConnectorLines",v,"1.9",O),this._Sh},set:function(e){this._Sh=e,I(this,"ShowConnectorLines",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showLeaderLines",{get:function(){return T("showLeaderLines",this._Sho,Pt,this._isNull),w("ChartSeries.showLeaderLines",v,"1.9",O),this._Sho},set:function(e){this._Sho=e,I(this,"ShowLeaderLines",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showShadow",{get:function(){return T("showShadow",this._Show,Pt,this._isNull),w("ChartSeries.showShadow",v,"1.7",O),this._Show},set:function(e){this._Show=e,I(this,"ShowShadow",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"smooth",{get:function(){return T("smooth",this._Sm,Pt,this._isNull),w("ChartSeries.smooth",v,"1.7",O),this._Sm},set:function(e){this._Sm=e,I(this,"Smooth",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"splitType",{get:function(){return T("splitType",this._Sp,Pt,this._isNull),w("ChartSeries.splitType",v,"1.8",O),this._Sp},set:function(e){this._Sp=e,I(this,"SplitType",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"splitValue",{get:function(){return T("splitValue",this._Spl,Pt,this._isNull),w("ChartSeries.splitValue",v,"1.9",O),this._Spl},set:function(e){this._Spl=e,I(this,"SplitValue",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"varyByCategories",{get:function(){return T("varyByCategories",this._V,Pt,this._isNull),w("ChartSeries.varyByCategories",v,"1.8",O),this._V},set:function(e){this._V=e,I(this,"VaryByCategories",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name","chartType","hasDataLabels","filtered","markerSize","markerStyle","showShadow","markerBackgroundColor","markerForegroundColor","smooth","plotOrder","gapWidth","doughnutHoleSize","axisGroup","explosion","firstSliceAngle","invertIfNegative","bubbleScale","secondPlotSize","splitType","splitValue","varyByCategories","showLeaderLines","overlap","gradientStyle","gradientMinimumType","gradientMidpointType","gradientMaximumType","gradientMinimumValue","gradientMidpointValue","gradientMaximumValue","gradientMinimumColor","gradientMidpointColor","gradientMaximumColor","parentLabelStrategy","showConnectorLines","invertColor"],["format","xErrorBars","yErrorBars","dataLabels","binOptions","mapOptions","boxwhiskerOptions"],["points","trendlines"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){w("ChartSeries.delete",v,"1.7",O),E(this,"Delete",0,[],V(2,"ExcelApiUndo","1.5"),0)},n.prototype.getDimensionDataSourceString=function(e){return w("ChartSeries.getDimensionDataSourceString",v,"1.15",O),E(this,"GetDimensionDataSourceString",0,[e],0,0)},n.prototype.getDimensionDataSourceType=function(e){return w("ChartSeries.getDimensionDataSourceType",v,"1.15",O),E(this,"GetDimensionDataSourceType",0,[e],0,0)},n.prototype.getDimensionValues=function(e){return w("ChartSeries.getDimensionValues",v,"1.12",O),E(this,"GetDimensionValues",0,[e],V(2,"ExcelApiUndo","1.5"),0)},n.prototype.setBubbleSizes=function(e){w("ChartSeries.setBubbleSizes",v,"1.7",O),E(this,"SetBubbleSizes",0,[e],0,0)},n.prototype.setValues=function(e){w("ChartSeries.setValues",v,"1.7",O),E(this,"SetValues",0,[e],V(2,"ExcelApiUndo","1.5"),0)},n.prototype.setXAxisValues=function(e){w("ChartSeries.setXAxisValues",v,"1.7",O),E(this,"SetXAxisValues",0,[e],V(2,"ExcelApiUndo","1.5"),0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AxisGroup)||(this._A=n.AxisGroup),N(n.BubbleScale)||(this._Bu=n.BubbleScale),N(n.ChartType)||(this._C=n.ChartType),N(n.DoughnutHoleSize)||(this._Do=n.DoughnutHoleSize),N(n.Explosion)||(this._E=n.Explosion),N(n.Filtered)||(this._F=n.Filtered),N(n.FirstSliceAngle)||(this._Fi=n.FirstSliceAngle),N(n.GapWidth)||(this._G=n.GapWidth),N(n.GradientMaximumColor)||(this._Gr=n.GradientMaximumColor),N(n.GradientMaximumType)||(this._Gra=n.GradientMaximumType),N(n.GradientMaximumValue)||(this._Grad=n.GradientMaximumValue),N(n.GradientMidpointColor)||(this._Gradi=n.GradientMidpointColor),N(n.GradientMidpointType)||(this._Gradie=n.GradientMidpointType),N(n.GradientMidpointValue)||(this._Gradien=n.GradientMidpointValue),N(n.GradientMinimumColor)||(this._Gradient=n.GradientMinimumColor),N(n.GradientMinimumType)||(this._GradientM=n.GradientMinimumType),N(n.GradientMinimumValue)||(this._GradientMi=n.GradientMinimumValue),N(n.GradientStyle)||(this._GradientS=n.GradientStyle),N(n.HasDataLabels)||(this._H=n.HasDataLabels),N(n.InvertColor)||(this._I=n.InvertColor),N(n.InvertIfNegative)||(this._In=n.InvertIfNegative),N(n.MarkerBackgroundColor)||(this._Ma=n.MarkerBackgroundColor),N(n.MarkerForegroundColor)||(this._Mar=n.MarkerForegroundColor),N(n.MarkerSize)||(this._Mark=n.MarkerSize),N(n.MarkerStyle)||(this._Marke=n.MarkerStyle),N(n.Name)||(this._N=n.Name),N(n.Overlap)||(this._O=n.Overlap),N(n.ParentLabelStrategy)||(this._P=n.ParentLabelStrategy),N(n.PlotOrder)||(this._Pl=n.PlotOrder),N(n.SecondPlotSize)||(this._S=n.SecondPlotSize),N(n.ShowConnectorLines)||(this._Sh=n.ShowConnectorLines),N(n.ShowLeaderLines)||(this._Sho=n.ShowLeaderLines),N(n.ShowShadow)||(this._Show=n.ShowShadow),N(n.Smooth)||(this._Sm=n.Smooth),N(n.SplitType)||(this._Sp=n.SplitType),N(n.SplitValue)||(this._Spl=n.SplitValue),N(n.VaryByCategories)||(this._V=n.VaryByCategories),L(this,n,["binOptions","BinOptions","boxwhiskerOptions","BoxwhiskerOptions","dataLabels","DataLabels","format","Format","mapOptions","MapOptions","points","Points","trendlines","Trendlines","xErrorBars","XErrorBars","yErrorBars","YErrorBars"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{axisGroup:this._A,bubbleScale:this._Bu,chartType:this._C,doughnutHoleSize:this._Do,explosion:this._E,filtered:this._F,firstSliceAngle:this._Fi,gapWidth:this._G,gradientMaximumColor:this._Gr,gradientMaximumType:this._Gra,gradientMaximumValue:this._Grad,gradientMidpointColor:this._Gradi,gradientMidpointType:this._Gradie,gradientMidpointValue:this._Gradien,gradientMinimumColor:this._Gradient,gradientMinimumType:this._GradientM,gradientMinimumValue:this._GradientMi,gradientStyle:this._GradientS,hasDataLabels:this._H,invertColor:this._I,invertIfNegative:this._In,markerBackgroundColor:this._Ma,markerForegroundColor:this._Mar,markerSize:this._Mark,markerStyle:this._Marke,name:this._N,overlap:this._O,parentLabelStrategy:this._P,plotOrder:this._Pl,secondPlotSize:this._S,showConnectorLines:this._Sh,showLeaderLines:this._Sho,showShadow:this._Show,smooth:this._Sm,splitType:this._Sp,splitValue:this._Spl,varyByCategories:this._V},{binOptions:this._B,boxwhiskerOptions:this._Bo,dataLabels:this._D,format:this._Fo,mapOptions:this._M,points:this._Po,trendlines:this._T,xErrorBars:this._X,yErrorBars:this._Y})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartSeries=At;var Ft=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartSeriesFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["fill","line"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.ChartFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"line",{get:function(){return this._L||(this._L=S(e.ChartLineFormat,this,"Line",!1,4)),this._L},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["line"],["fill"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["fill","Fill","line","Line"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{line:this._L})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartSeriesFormat=Ft;var Dt="ChartPointsCollection",Et=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartPointsCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,Dt,this._isNull),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"count",{get:function(){return T("count",this._C,Dt,this._isNull),this._C},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return w("ChartPointsCollection.getCount",v,"1.4",O),E(this,"GetCount",1,[],4,0)},n.prototype.getItemAt=function(t){return C(e.ChartPoint,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),N(r.Count)||(this._C=r.Count),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.ChartPoint,!1,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.ChartPoint,!1,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{count:this._C},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.ChartPoint,!1,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.ChartPointsCollection=Et;var Rt="ChartPoint",It=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartPoint"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["value","hasDataLabel","markerStyle","markerSize","markerBackgroundColor","markerForegroundColor"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Value","HasDataLabel","MarkerStyle","MarkerSize","MarkerBackgroundColor","MarkerForegroundColor"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format","dataLabel"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"dataLabel",{get:function(){return w("ChartPoint.dataLabel",v,"1.7",O),this._D||(this._D=S(e.ChartDataLabel,this,"DataLabel",!1,4)),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartPointFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"hasDataLabel",{get:function(){return T("hasDataLabel",this._H,Rt,this._isNull),w("ChartPoint.hasDataLabel",v,"1.7",O),this._H},set:function(e){this._H=e,I(this,"HasDataLabel",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"markerBackgroundColor",{get:function(){return T("markerBackgroundColor",this._M,Rt,this._isNull),w("ChartPoint.markerBackgroundColor",v,"1.7",O),this._M},set:function(e){this._M=e,I(this,"MarkerBackgroundColor",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"markerForegroundColor",{get:function(){return T("markerForegroundColor",this._Ma,Rt,this._isNull),w("ChartPoint.markerForegroundColor",v,"1.7",O),this._Ma},set:function(e){this._Ma=e,I(this,"MarkerForegroundColor",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"markerSize",{get:function(){return T("markerSize",this._Mar,Rt,this._isNull),w("ChartPoint.markerSize",v,"1.7",O),this._Mar},set:function(e){this._Mar=e,I(this,"MarkerSize",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"markerStyle",{get:function(){return T("markerStyle",this._Mark,Rt,this._isNull),w("ChartPoint.markerStyle",v,"1.7",O),this._Mark},set:function(e){this._Mark=e,I(this,"MarkerStyle",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"value",{get:function(){return T("value",this._V,Rt,this._isNull),this._V},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["hasDataLabel","markerStyle","markerSize","markerBackgroundColor","markerForegroundColor"],["format","dataLabel"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.HasDataLabel)||(this._H=n.HasDataLabel),N(n.MarkerBackgroundColor)||(this._M=n.MarkerBackgroundColor),N(n.MarkerForegroundColor)||(this._Ma=n.MarkerForegroundColor),N(n.MarkerSize)||(this._Mar=n.MarkerSize),N(n.MarkerStyle)||(this._Mark=n.MarkerStyle),N(n.Value)||(this._V=n.Value),L(this,n,["dataLabel","DataLabel","format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{hasDataLabel:this._H,markerBackgroundColor:this._M,markerForegroundColor:this._Ma,markerSize:this._Mar,markerStyle:this._Mark,value:this._V},{dataLabel:this._D,format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartPoint=It;var xt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartPointFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["fill","border"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"border",{get:function(){return w("ChartPointFormat.border",v,"1.7",O),this._B||(this._B=S(e.ChartBorder,this,"Border",!1,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.ChartFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["border"],["fill"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["border","Border","fill","Fill"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{border:this._B})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartPointFormat=xt;var Nt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartAxes"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["categoryAxis","seriesAxis","valueAxis"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"categoryAxis",{get:function(){return this._C||(this._C=S(e.ChartAxis,this,"CategoryAxis",!1,4)),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"seriesAxis",{get:function(){return this._S||(this._S=S(e.ChartAxis,this,"SeriesAxis",!1,4)),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"valueAxis",{get:function(){return this._V||(this._V=S(e.ChartAxis,this,"ValueAxis",!1,4)),this._V},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["categoryAxis","seriesAxis","valueAxis"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.getItem=function(t,n){return w("ChartAxes.getItem",v,"1.7",O),C(e.ChartAxis,this,"GetItem",1,[t,n],!1,!1,null,4)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["categoryAxis","CategoryAxis","seriesAxis","SeriesAxis","valueAxis","ValueAxis"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{categoryAxis:this._C,seriesAxis:this._S,valueAxis:this._V})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartAxes=Nt;var Tt="ChartAxis",wt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartAxis"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["majorUnit","maximum","minimum","minorUnit","displayUnit","showDisplayUnitLabel","customDisplayUnit","type","minorTimeUnitScale","majorTimeUnitScale","baseTimeUnit","categoryType","axisGroup","scaleType","logBase","left","top","height","width","reversePlotOrder","crosses","crossesAt","visible","isBetweenCategories","majorTickMark","minorTickMark","tickMarkSpacing","tickLabelPosition","tickLabelSpacing","alignment","multiLevel","numberFormat","linkNumberFormat","offset","textOrientation","position","positionAt"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["MajorUnit","Maximum","Minimum","MinorUnit","DisplayUnit","ShowDisplayUnitLabel","CustomDisplayUnit","Type","MinorTimeUnitScale","MajorTimeUnitScale","BaseTimeUnit","CategoryType","AxisGroup","ScaleType","LogBase","Left","Top","Height","Width","ReversePlotOrder","Crosses","CrossesAt","Visible","IsBetweenCategories","MajorTickMark","MinorTickMark","TickMarkSpacing","TickLabelPosition","TickLabelSpacing","Alignment","MultiLevel","NumberFormat","LinkNumberFormat","Offset","TextOrientation","Position","PositionAt"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!1,!1,!0,!0,!0,!0,!1,!0,!0,!1,!1,!1,!1,!0,!0,!1,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["majorGridlines","minorGridlines","title","format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartAxisFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"majorGridlines",{get:function(){return this._M||(this._M=S(e.ChartGridlines,this,"MajorGridlines",!1,4)),this._M},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"minorGridlines",{get:function(){return this._Min||(this._Min=S(e.ChartGridlines,this,"MinorGridlines",!1,4)),this._Min},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"title",{get:function(){return this._Tit||(this._Tit=S(e.ChartAxisTitle,this,"Title",!1,4)),this._Tit},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"alignment",{get:function(){return T("alignment",this._A,Tt,this._isNull),w("ChartAxis.alignment",v,"1.8",O),this._A},set:function(e){this._A=e,I(this,"Alignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"axisGroup",{get:function(){return T("axisGroup",this._Ax,Tt,this._isNull),w("ChartAxis.axisGroup",v,"1.7",O),this._Ax},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"baseTimeUnit",{get:function(){return T("baseTimeUnit",this._B,Tt,this._isNull),w("ChartAxis.baseTimeUnit",v,"1.7",O),this._B},set:function(e){this._B=e,I(this,"BaseTimeUnit",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"categoryType",{get:function(){return T("categoryType",this._C,Tt,this._isNull),w("ChartAxis.categoryType",v,"1.7",O),this._C},set:function(e){this._C=e,I(this,"CategoryType",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"crosses",{get:function(){return T("crosses",this._Cr,Tt,this._isNull),w("ChartAxis.crosses",v,"1.7",O),this._Cr},set:function(e){this._Cr=e,I(this,"Crosses",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"crossesAt",{get:function(){return T("crossesAt",this._Cro,Tt,this._isNull),w("ChartAxis.crossesAt",v,"1.7",O),this._Cro},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"customDisplayUnit",{get:function(){return T("customDisplayUnit",this._Cu,Tt,this._isNull),w("ChartAxis.customDisplayUnit",v,"1.7",O),this._Cu},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"displayUnit",{get:function(){return T("displayUnit",this._D,Tt,this._isNull),w("ChartAxis.displayUnit",v,"1.7",O),this._D},set:function(e){this._D=e,I(this,"DisplayUnit",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"height",{get:function(){return T("height",this._H,Tt,this._isNull),w("ChartAxis.height",v,"1.7",O),this._H},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isBetweenCategories",{get:function(){return T("isBetweenCategories",this._I,Tt,this._isNull),w("ChartAxis.isBetweenCategories",v,"1.8",O),this._I},set:function(e){this._I=e,I(this,"IsBetweenCategories",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"left",{get:function(){return T("left",this._L,Tt,this._isNull),w("ChartAxis.left",v,"1.7",O),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"linkNumberFormat",{get:function(){return T("linkNumberFormat",this._Li,Tt,this._isNull),w("ChartAxis.linkNumberFormat",v,"1.9",O),this._Li},set:function(e){this._Li=e,I(this,"LinkNumberFormat",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"logBase",{get:function(){return T("logBase",this._Lo,Tt,this._isNull),w("ChartAxis.logBase",v,"1.7",O),this._Lo},set:function(e){this._Lo=e,I(this,"LogBase",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"majorTickMark",{get:function(){return T("majorTickMark",this._Ma,Tt,this._isNull),w("ChartAxis.majorTickMark",v,"1.7",O),this._Ma},set:function(e){this._Ma=e,I(this,"MajorTickMark",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"majorTimeUnitScale",{get:function(){return T("majorTimeUnitScale",this._Maj,Tt,this._isNull),w("ChartAxis.majorTimeUnitScale",v,"1.7",O),this._Maj},set:function(e){this._Maj=e,I(this,"MajorTimeUnitScale",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"majorUnit",{get:function(){return T("majorUnit",this._Majo,Tt,this._isNull),this._Majo},set:function(e){this._Majo=e,I(this,"MajorUnit",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"maximum",{get:function(){return T("maximum",this._Max,Tt,this._isNull),this._Max},set:function(e){this._Max=e,I(this,"Maximum",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"minimum",{get:function(){return T("minimum",this._Mi,Tt,this._isNull),this._Mi},set:function(e){this._Mi=e,I(this,"Minimum",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"minorTickMark",{get:function(){return T("minorTickMark",this._Mino,Tt,this._isNull),w("ChartAxis.minorTickMark",v,"1.7",O),this._Mino},set:function(e){this._Mino=e,I(this,"MinorTickMark",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"minorTimeUnitScale",{get:function(){return T("minorTimeUnitScale",this._Minor,Tt,this._isNull),w("ChartAxis.minorTimeUnitScale",v,"1.7",O),this._Minor},set:function(e){this._Minor=e,I(this,"MinorTimeUnitScale",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"minorUnit",{get:function(){return T("minorUnit",this._MinorU,Tt,this._isNull),this._MinorU},set:function(e){this._MinorU=e,I(this,"MinorUnit",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"multiLevel",{get:function(){return T("multiLevel",this._Mu,Tt,this._isNull),w("ChartAxis.multiLevel",v,"1.8",O),this._Mu},set:function(e){this._Mu=e,I(this,"MultiLevel",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormat",{get:function(){return T("numberFormat",this._N,Tt,this._isNull),w("ChartAxis.numberFormat",v,"1.8",O),this._N},set:function(e){this._N=e,I(this,"NumberFormat",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"offset",{get:function(){return T("offset",this._O,Tt,this._isNull),w("ChartAxis.offset",v,"1.8",O),this._O},set:function(e){this._O=e,I(this,"Offset",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"position",{get:function(){return T("position",this._P,Tt,this._isNull),w("ChartAxis.position",v,"1.8",O),this._P},set:function(e){this._P=e,I(this,"Position",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"positionAt",{get:function(){return T("positionAt",this._Po,Tt,this._isNull),w("ChartAxis.positionAt",v,"1.8",O),this._Po},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"reversePlotOrder",{get:function(){return T("reversePlotOrder",this._R,Tt,this._isNull),w("ChartAxis.reversePlotOrder",v,"1.7",O),this._R},set:function(e){this._R=e,I(this,"ReversePlotOrder",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"scaleType",{get:function(){return T("scaleType",this._S,Tt,this._isNull),w("ChartAxis.scaleType",v,"1.7",O),this._S},set:function(e){this._S=e,I(this,"ScaleType",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showDisplayUnitLabel",{get:function(){return T("showDisplayUnitLabel",this._Sh,Tt,this._isNull),w("ChartAxis.showDisplayUnitLabel",v,"1.7",O),this._Sh},set:function(e){this._Sh=e,I(this,"ShowDisplayUnitLabel",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"textOrientation",{get:function(){return T("textOrientation",this._T,Tt,this._isNull),w("ChartAxis.textOrientation",v,"1.8",O),this._T},set:function(e){this._T=e,I(this,"TextOrientation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"tickLabelPosition",{get:function(){return T("tickLabelPosition",this._Ti,Tt,this._isNull),w("ChartAxis.tickLabelPosition",v,"1.7",O),this._Ti},set:function(e){this._Ti=e,I(this,"TickLabelPosition",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"tickLabelSpacing",{get:function(){return T("tickLabelSpacing",this._Tic,Tt,this._isNull),w("ChartAxis.tickLabelSpacing",v,"1.7",O),this._Tic},set:function(e){this._Tic=e,I(this,"TickLabelSpacing",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"tickMarkSpacing",{get:function(){return T("tickMarkSpacing",this._Tick,Tt,this._isNull),w("ChartAxis.tickMarkSpacing",v,"1.7",O),this._Tick},set:function(e){this._Tick=e,I(this,"TickMarkSpacing",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"top",{get:function(){return T("top",this._To,Tt,this._isNull),w("ChartAxis.top",v,"1.7",O),this._To},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"type",{get:function(){return T("type",this._Ty,Tt,this._isNull),w("ChartAxis.type",v,"1.7",O),this._Ty},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"visible",{get:function(){return T("visible",this._V,Tt,this._isNull),w("ChartAxis.visible",v,"1.7",O),this._V},set:function(e){this._V=e,I(this,"Visible",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"width",{get:function(){return T("width",this._W,Tt,this._isNull),w("ChartAxis.width",v,"1.7",O),this._W},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["majorUnit","maximum","minimum","minorUnit","displayUnit","showDisplayUnitLabel","minorTimeUnitScale","majorTimeUnitScale","baseTimeUnit","categoryType","scaleType","logBase","reversePlotOrder","crosses","visible","isBetweenCategories","majorTickMark","minorTickMark","tickMarkSpacing","tickLabelPosition","tickLabelSpacing","alignment","multiLevel","numberFormat","linkNumberFormat","offset","textOrientation","position"],["majorGridlines","minorGridlines","title","format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.setCategoryNames=function(e){w("ChartAxis.setCategoryNames",v,"1.7",O),E(this,"SetCategoryNames",0,[e],0,0)},n.prototype.setCrossesAt=function(e){w("ChartAxis.setCrossesAt",v,"1.7",O),E(this,"SetCrossesAt",0,[e],0,0)},n.prototype.setCustomDisplayUnit=function(e){w("ChartAxis.setCustomDisplayUnit",v,"1.7",O),E(this,"SetCustomDisplayUnit",0,[e],0,0)},n.prototype.setPositionAt=function(e){w("ChartAxis.setPositionAt",v,"1.8",O),E(this,"SetPositionAt",0,[e],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Alignment)||(this._A=n.Alignment),N(n.AxisGroup)||(this._Ax=n.AxisGroup),N(n.BaseTimeUnit)||(this._B=n.BaseTimeUnit),N(n.CategoryType)||(this._C=n.CategoryType),N(n.Crosses)||(this._Cr=n.Crosses),N(n.CrossesAt)||(this._Cro=n.CrossesAt),N(n.CustomDisplayUnit)||(this._Cu=n.CustomDisplayUnit),N(n.DisplayUnit)||(this._D=n.DisplayUnit),N(n.Height)||(this._H=n.Height),N(n.IsBetweenCategories)||(this._I=n.IsBetweenCategories),N(n.Left)||(this._L=n.Left),N(n.LinkNumberFormat)||(this._Li=n.LinkNumberFormat),N(n.LogBase)||(this._Lo=n.LogBase),N(n.MajorTickMark)||(this._Ma=n.MajorTickMark),N(n.MajorTimeUnitScale)||(this._Maj=n.MajorTimeUnitScale),N(n.MajorUnit)||(this._Majo=n.MajorUnit),N(n.Maximum)||(this._Max=n.Maximum),N(n.Minimum)||(this._Mi=n.Minimum),N(n.MinorTickMark)||(this._Mino=n.MinorTickMark),N(n.MinorTimeUnitScale)||(this._Minor=n.MinorTimeUnitScale),N(n.MinorUnit)||(this._MinorU=n.MinorUnit),N(n.MultiLevel)||(this._Mu=n.MultiLevel),N(n.NumberFormat)||(this._N=n.NumberFormat),N(n.Offset)||(this._O=n.Offset),N(n.Position)||(this._P=n.Position),N(n.PositionAt)||(this._Po=n.PositionAt),N(n.ReversePlotOrder)||(this._R=n.ReversePlotOrder),N(n.ScaleType)||(this._S=n.ScaleType),N(n.ShowDisplayUnitLabel)||(this._Sh=n.ShowDisplayUnitLabel),N(n.TextOrientation)||(this._T=n.TextOrientation),N(n.TickLabelPosition)||(this._Ti=n.TickLabelPosition),N(n.TickLabelSpacing)||(this._Tic=n.TickLabelSpacing),N(n.TickMarkSpacing)||(this._Tick=n.TickMarkSpacing),N(n.Top)||(this._To=n.Top),N(n.Type)||(this._Ty=n.Type),N(n.Visible)||(this._V=n.Visible),N(n.Width)||(this._W=n.Width),L(this,n,["format","Format","majorGridlines","MajorGridlines","minorGridlines","MinorGridlines","title","Title"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{alignment:this._A,axisGroup:this._Ax,baseTimeUnit:this._B,categoryType:this._C,crosses:this._Cr,crossesAt:this._Cro,customDisplayUnit:this._Cu,displayUnit:this._D,height:this._H,isBetweenCategories:this._I,left:this._L,linkNumberFormat:this._Li,logBase:this._Lo,majorTickMark:this._Ma,majorTimeUnitScale:this._Maj,majorUnit:this._Majo,maximum:this._Max,minimum:this._Mi,minorTickMark:this._Mino,minorTimeUnitScale:this._Minor,minorUnit:this._MinorU,multiLevel:this._Mu,numberFormat:this._N,offset:this._O,position:this._P,positionAt:this._Po,reversePlotOrder:this._R,scaleType:this._S,showDisplayUnitLabel:this._Sh,textOrientation:this._T,tickLabelPosition:this._Ti,tickLabelSpacing:this._Tic,tickMarkSpacing:this._Tick,top:this._To,type:this._Ty,visible:this._V,width:this._W},{format:this._F,majorGridlines:this._M,minorGridlines:this._Min,title:this._Tit})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartAxis=wt;var jt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartAxisFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["font","line","fill"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return w("ChartAxisFormat.fill",v,"1.8",O),this._F||(this._F=S(e.ChartFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._Fo||(this._Fo=S(e.ChartFont,this,"Font",!1,4)),this._Fo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"line",{get:function(){return this._L||(this._L=S(e.ChartLineFormat,this,"Line",!1,4)),this._L},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["font","line"],["fill"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["fill","Fill","font","Font","line","Line"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{font:this._Fo,line:this._L})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartAxisFormat=jt;var Mt="ChartAxisTitle",kt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartAxisTitle"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["text","visible","textOrientation"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Text","Visible","TextOrientation"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartAxisTitleFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"text",{get:function(){return T("text",this._T,Mt,this._isNull),this._T},set:function(e){this._T=e,I(this,"Text",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"textOrientation",{get:function(){return T("textOrientation",this._Te,Mt,this._isNull),w("ChartAxisTitle.textOrientation",v,"1.12",O),this._Te},set:function(e){this._Te=e,I(this,"TextOrientation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"visible",{get:function(){return T("visible",this._V,Mt,this._isNull),this._V},set:function(e){this._V=e,I(this,"Visible",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["text","visible","textOrientation"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.setFormula=function(e){w("ChartAxisTitle.setFormula",v,"1.8",O),E(this,"SetFormula",0,[e],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Text)||(this._T=n.Text),N(n.TextOrientation)||(this._Te=n.TextOrientation),N(n.Visible)||(this._V=n.Visible),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{text:this._T,textOrientation:this._Te,visible:this._V},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartAxisTitle=kt;var Ut=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartAxisTitleFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["font","fill","border"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"border",{get:function(){return w("ChartAxisTitleFormat.border",v,"1.8",O),this._B||(this._B=S(e.ChartBorder,this,"Border",!1,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return w("ChartAxisTitleFormat.fill",v,"1.8",O),this._F||(this._F=S(e.ChartFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._Fo||(this._Fo=S(e.ChartFont,this,"Font",!1,4)),this._Fo},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["font","border"],["fill"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["border","Border","fill","Fill","font","Font"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{border:this._B,font:this._Fo})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartAxisTitleFormat=Ut;var Lt="ChartDataLabels",Bt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartDataLabels"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["position","showValue","showSeriesName","showCategoryName","showLegendKey","showPercentage","showBubbleSize","separator","numberFormat","linkNumberFormat","textOrientation","autoText","horizontalAlignment","verticalAlignment"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Position","ShowValue","ShowSeriesName","ShowCategoryName","ShowLegendKey","ShowPercentage","ShowBubbleSize","Separator","NumberFormat","LinkNumberFormat","TextOrientation","AutoText","HorizontalAlignment","VerticalAlignment"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartDataLabelFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"autoText",{get:function(){return T("autoText",this._A,Lt,this._isNull),w("ChartDataLabels.autoText",v,"1.8",O),this._A},set:function(e){this._A=e,I(this,"AutoText",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"horizontalAlignment",{get:function(){return T("horizontalAlignment",this._H,Lt,this._isNull),w("ChartDataLabels.horizontalAlignment",v,"1.8",O),this._H},set:function(e){this._H=e,I(this,"HorizontalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"linkNumberFormat",{get:function(){return T("linkNumberFormat",this._L,Lt,this._isNull),w("ChartDataLabels.linkNumberFormat",v,"1.9",O),this._L},set:function(e){this._L=e,I(this,"LinkNumberFormat",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormat",{get:function(){return T("numberFormat",this._N,Lt,this._isNull),w("ChartDataLabels.numberFormat",v,"1.8",O),this._N},set:function(e){this._N=e,I(this,"NumberFormat",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"position",{get:function(){return T("position",this._P,Lt,this._isNull),this._P},set:function(e){this._P=e,I(this,"Position",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"separator",{get:function(){return T("separator",this._S,Lt,this._isNull),this._S},set:function(e){this._S=e,I(this,"Separator",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showBubbleSize",{get:function(){return T("showBubbleSize",this._Sh,Lt,this._isNull),this._Sh},set:function(e){this._Sh=e,I(this,"ShowBubbleSize",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showCategoryName",{get:function(){return T("showCategoryName",this._Sho,Lt,this._isNull),this._Sho},set:function(e){this._Sho=e,I(this,"ShowCategoryName",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showLegendKey",{get:function(){return T("showLegendKey",this._Show,Lt,this._isNull),this._Show},set:function(e){this._Show=e,I(this,"ShowLegendKey",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showPercentage",{get:function(){return T("showPercentage",this._ShowP,Lt,this._isNull),this._ShowP},set:function(e){this._ShowP=e,I(this,"ShowPercentage",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showSeriesName",{get:function(){return T("showSeriesName",this._ShowS,Lt,this._isNull),this._ShowS},set:function(e){this._ShowS=e,I(this,"ShowSeriesName",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showValue",{get:function(){return T("showValue",this._ShowV,Lt,this._isNull),this._ShowV},set:function(e){this._ShowV=e,I(this,"ShowValue",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"textOrientation",{get:function(){return T("textOrientation",this._T,Lt,this._isNull),w("ChartDataLabels.textOrientation",v,"1.8",O),this._T},set:function(e){this._T=e,I(this,"TextOrientation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"verticalAlignment",{get:function(){return T("verticalAlignment",this._V,Lt,this._isNull),w("ChartDataLabels.verticalAlignment",v,"1.8",O),this._V},set:function(e){this._V=e,I(this,"VerticalAlignment",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["position","showValue","showSeriesName","showCategoryName","showLegendKey","showPercentage","showBubbleSize","separator","numberFormat","linkNumberFormat","textOrientation","autoText","horizontalAlignment","verticalAlignment"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AutoText)||(this._A=n.AutoText),N(n.HorizontalAlignment)||(this._H=n.HorizontalAlignment),N(n.LinkNumberFormat)||(this._L=n.LinkNumberFormat),N(n.NumberFormat)||(this._N=n.NumberFormat),N(n.Position)||(this._P=n.Position),N(n.Separator)||(this._S=n.Separator),N(n.ShowBubbleSize)||(this._Sh=n.ShowBubbleSize),N(n.ShowCategoryName)||(this._Sho=n.ShowCategoryName),N(n.ShowLegendKey)||(this._Show=n.ShowLegendKey),N(n.ShowPercentage)||(this._ShowP=n.ShowPercentage),N(n.ShowSeriesName)||(this._ShowS=n.ShowSeriesName),N(n.ShowValue)||(this._ShowV=n.ShowValue),N(n.TextOrientation)||(this._T=n.TextOrientation),N(n.VerticalAlignment)||(this._V=n.VerticalAlignment),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{autoText:this._A,horizontalAlignment:this._H,linkNumberFormat:this._L,numberFormat:this._N,position:this._P,separator:this._S,showBubbleSize:this._Sh,showCategoryName:this._Sho,showLegendKey:this._Show,showPercentage:this._ShowP,showSeriesName:this._ShowS,showValue:this._ShowV,textOrientation:this._T,verticalAlignment:this._V},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartDataLabels=Bt;var Ht="ChartDataLabel",Wt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartDataLabel"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["position","showValue","showSeriesName","showCategoryName","showLegendKey","showPercentage","showBubbleSize","separator","top","left","width","height","formula","textOrientation","horizontalAlignment","verticalAlignment","text","autoText","numberFormat","linkNumberFormat"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Position","ShowValue","ShowSeriesName","ShowCategoryName","ShowLegendKey","ShowPercentage","ShowBubbleSize","Separator","Top","Left","Width","Height","Formula","TextOrientation","HorizontalAlignment","VerticalAlignment","Text","AutoText","NumberFormat","LinkNumberFormat"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!1,!1,!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return w("ChartDataLabel.format",v,"1.8",O),this._F||(this._F=S(e.ChartDataLabelFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"autoText",{get:function(){return T("autoText",this._A,Ht,this._isNull),w("ChartDataLabel.autoText",v,"1.8",O),this._A},set:function(e){this._A=e,I(this,"AutoText",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"formula",{get:function(){return T("formula",this._Fo,Ht,this._isNull),w("ChartDataLabel.formula",v,"1.8",O),this._Fo},set:function(e){this._Fo=e,I(this,"Formula",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"height",{get:function(){return T("height",this._H,Ht,this._isNull),w("ChartDataLabel.height",v,"1.8",O),this._H},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"horizontalAlignment",{get:function(){return T("horizontalAlignment",this._Ho,Ht,this._isNull),w("ChartDataLabel.horizontalAlignment",v,"1.8",O),this._Ho},set:function(e){this._Ho=e,I(this,"HorizontalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"left",{get:function(){return T("left",this._L,Ht,this._isNull),w("ChartDataLabel.left",v,"1.8",O),this._L},set:function(e){this._L=e,I(this,"Left",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"linkNumberFormat",{get:function(){return T("linkNumberFormat",this._Li,Ht,this._isNull),w("ChartDataLabel.linkNumberFormat",v,"1.9",O),this._Li},set:function(e){this._Li=e,I(this,"LinkNumberFormat",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormat",{get:function(){return T("numberFormat",this._N,Ht,this._isNull),w("ChartDataLabel.numberFormat",v,"1.8",O),this._N},set:function(e){this._N=e,I(this,"NumberFormat",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"position",{get:function(){return T("position",this._P,Ht,this._isNull),this._P},set:function(e){this._P=e,I(this,"Position",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"separator",{get:function(){return T("separator",this._S,Ht,this._isNull),this._S},set:function(e){this._S=e,I(this,"Separator",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showBubbleSize",{get:function(){return T("showBubbleSize",this._Sh,Ht,this._isNull),this._Sh},set:function(e){this._Sh=e,I(this,"ShowBubbleSize",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showCategoryName",{get:function(){return T("showCategoryName",this._Sho,Ht,this._isNull),this._Sho},set:function(e){this._Sho=e,I(this,"ShowCategoryName",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showLegendKey",{get:function(){return T("showLegendKey",this._Show,Ht,this._isNull),this._Show},set:function(e){this._Show=e,I(this,"ShowLegendKey",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showPercentage",{get:function(){return T("showPercentage",this._ShowP,Ht,this._isNull),this._ShowP},set:function(e){this._ShowP=e,I(this,"ShowPercentage",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showSeriesName",{get:function(){return T("showSeriesName",this._ShowS,Ht,this._isNull),this._ShowS},set:function(e){this._ShowS=e,I(this,"ShowSeriesName",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showValue",{get:function(){return T("showValue",this._ShowV,Ht,this._isNull),this._ShowV},set:function(e){this._ShowV=e,I(this,"ShowValue",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"text",{get:function(){return T("text",this._T,Ht,this._isNull),w("ChartDataLabel.text",v,"1.8",O),this._T},set:function(e){this._T=e,I(this,"Text",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"textOrientation",{get:function(){return T("textOrientation",this._Te,Ht,this._isNull),w("ChartDataLabel.textOrientation",v,"1.8",O),this._Te},set:function(e){this._Te=e,I(this,"TextOrientation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"top",{get:function(){return T("top",this._To,Ht,this._isNull),w("ChartDataLabel.top",v,"1.8",O),this._To},set:function(e){this._To=e,I(this,"Top",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"verticalAlignment",{get:function(){return T("verticalAlignment",this._V,Ht,this._isNull),w("ChartDataLabel.verticalAlignment",v,"1.8",O),this._V},set:function(e){this._V=e,I(this,"VerticalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"width",{get:function(){return T("width",this._W,Ht,this._isNull),w("ChartDataLabel.width",v,"1.8",O),this._W},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["position","showValue","showSeriesName","showCategoryName","showLegendKey","showPercentage","showBubbleSize","separator","top","left","formula","textOrientation","horizontalAlignment","verticalAlignment","text","autoText","numberFormat","linkNumberFormat"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AutoText)||(this._A=n.AutoText),N(n.Formula)||(this._Fo=n.Formula),N(n.Height)||(this._H=n.Height),N(n.HorizontalAlignment)||(this._Ho=n.HorizontalAlignment),N(n.Left)||(this._L=n.Left),N(n.LinkNumberFormat)||(this._Li=n.LinkNumberFormat),N(n.NumberFormat)||(this._N=n.NumberFormat),N(n.Position)||(this._P=n.Position),N(n.Separator)||(this._S=n.Separator),N(n.ShowBubbleSize)||(this._Sh=n.ShowBubbleSize),N(n.ShowCategoryName)||(this._Sho=n.ShowCategoryName),N(n.ShowLegendKey)||(this._Show=n.ShowLegendKey),N(n.ShowPercentage)||(this._ShowP=n.ShowPercentage),N(n.ShowSeriesName)||(this._ShowS=n.ShowSeriesName),N(n.ShowValue)||(this._ShowV=n.ShowValue),N(n.Text)||(this._T=n.Text),N(n.TextOrientation)||(this._Te=n.TextOrientation),N(n.Top)||(this._To=n.Top),N(n.VerticalAlignment)||(this._V=n.VerticalAlignment),N(n.Width)||(this._W=n.Width),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{autoText:this._A,formula:this._Fo,height:this._H,horizontalAlignment:this._Ho,left:this._L,linkNumberFormat:this._Li,numberFormat:this._N,position:this._P,separator:this._S,showBubbleSize:this._Sh,showCategoryName:this._Sho,showLegendKey:this._Show,showPercentage:this._ShowP,showSeriesName:this._ShowS,showValue:this._ShowV,text:this._T,textOrientation:this._Te,top:this._To,verticalAlignment:this._V,width:this._W},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartDataLabel=Wt;var Vt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartDataLabelFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["font","fill","border"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"border",{get:function(){return w("ChartDataLabelFormat.border",v,"1.8",O),this._B||(this._B=S(e.ChartBorder,this,"Border",!1,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.ChartFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._Fo||(this._Fo=S(e.ChartFont,this,"Font",!1,4)),this._Fo},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["font","border"],["fill"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["border","Border","fill","Fill","font","Font"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{border:this._B,font:this._Fo})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartDataLabelFormat=Vt;var Gt="ChartDataTable",zt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartDataTable"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["visible","showLegendKey","showHorizontalBorder","showVerticalBorder","showOutlineBorder"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Visible","ShowLegendKey","ShowHorizontalBorder","ShowVerticalBorder","ShowOutlineBorder"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartDataTableFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showHorizontalBorder",{get:function(){return T("showHorizontalBorder",this._S,Gt,this._isNull),this._S},set:function(e){this._S=e,I(this,"ShowHorizontalBorder",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showLegendKey",{get:function(){return T("showLegendKey",this._Sh,Gt,this._isNull),this._Sh},set:function(e){this._Sh=e,I(this,"ShowLegendKey",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showOutlineBorder",{get:function(){return T("showOutlineBorder",this._Sho,Gt,this._isNull),this._Sho},set:function(e){this._Sho=e,I(this,"ShowOutlineBorder",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showVerticalBorder",{get:function(){return T("showVerticalBorder",this._Show,Gt,this._isNull),this._Show},set:function(e){this._Show=e,I(this,"ShowVerticalBorder",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"visible",{get:function(){return T("visible",this._V,Gt,this._isNull),this._V},set:function(e){this._V=e,I(this,"Visible",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["visible","showLegendKey","showHorizontalBorder","showVerticalBorder","showOutlineBorder"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.ShowHorizontalBorder)||(this._S=n.ShowHorizontalBorder),N(n.ShowLegendKey)||(this._Sh=n.ShowLegendKey),N(n.ShowOutlineBorder)||(this._Sho=n.ShowOutlineBorder),N(n.ShowVerticalBorder)||(this._Show=n.ShowVerticalBorder),N(n.Visible)||(this._V=n.Visible),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{showHorizontalBorder:this._S,showLegendKey:this._Sh,showOutlineBorder:this._Sho,showVerticalBorder:this._Show,visible:this._V},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartDataTable=zt;var qt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartDataTableFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["fill","font","border"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"border",{get:function(){return this._B||(this._B=S(e.ChartBorder,this,"Border",!1,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.ChartFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._Fo||(this._Fo=S(e.ChartFont,this,"Font",!1,4)),this._Fo},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["font","border"],["fill"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["border","Border","fill","Fill","font","Font"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{border:this._B,font:this._Fo})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartDataTableFormat=qt;var Jt="ChartErrorBars",Kt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartErrorBars"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["endStyleCap","include","type","visible"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["EndStyleCap","Include","Type","Visible"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartErrorBarsFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"endStyleCap",{get:function(){return T("endStyleCap",this._E,Jt,this._isNull),this._E},set:function(e){this._E=e,I(this,"EndStyleCap",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"include",{get:function(){return T("include",this._I,Jt,this._isNull),this._I},set:function(e){this._I=e,I(this,"Include",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"type",{get:function(){return T("type",this._T,Jt,this._isNull),this._T},set:function(e){this._T=e,I(this,"Type",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"visible",{get:function(){return T("visible",this._V,Jt,this._isNull),this._V},set:function(e){this._V=e,I(this,"Visible",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["endStyleCap","include","type","visible"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.EndStyleCap)||(this._E=n.EndStyleCap),N(n.Include)||(this._I=n.Include),N(n.Type)||(this._T=n.Type),N(n.Visible)||(this._V=n.Visible),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{endStyleCap:this._E,include:this._I,type:this._T,visible:this._V},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartErrorBars=Kt;var Qt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartErrorBarsFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["line"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"line",{get:function(){return this._L||(this._L=S(e.ChartLineFormat,this,"Line",!1,4)),this._L},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["line"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["line","Line"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{line:this._L})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartErrorBarsFormat=Qt;var Xt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartGridlines"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["visible"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Visible"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartGridlinesFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"visible",{get:function(){return T("visible",this._V,"ChartGridlines",this._isNull),this._V},set:function(e){this._V=e,I(this,"Visible",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["visible"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Visible)||(this._V=n.Visible),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{visible:this._V},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartGridlines=Xt;var Yt=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartGridlinesFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["line"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"line",{get:function(){return this._L||(this._L=S(e.ChartLineFormat,this,"Line",!1,4)),this._L},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["line"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["line","Line"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{line:this._L})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartGridlinesFormat=Yt;var Zt="ChartLegend",$t=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartLegend"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["visible","position","overlay","left","top","width","height","showShadow"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Visible","Position","Overlay","Left","Top","Width","Height","ShowShadow"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format","legendEntries"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartLegendFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"legendEntries",{get:function(){return w("ChartLegend.legendEntries",v,"1.7",O),this._Le||(this._Le=S(e.ChartLegendEntryCollection,this,"LegendEntries",!0,4)),this._Le},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"height",{get:function(){return T("height",this._H,Zt,this._isNull),w("ChartLegend.height",v,"1.7",O),this._H},set:function(e){this._H=e,I(this,"Height",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"left",{get:function(){return T("left",this._L,Zt,this._isNull),w("ChartLegend.left",v,"1.7",O),this._L},set:function(e){this._L=e,I(this,"Left",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"overlay",{get:function(){return T("overlay",this._O,Zt,this._isNull),this._O},set:function(e){this._O=e,I(this,"Overlay",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"position",{get:function(){return T("position",this._P,Zt,this._isNull),this._P},set:function(e){this._P=e,I(this,"Position",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showShadow",{get:function(){return T("showShadow",this._S,Zt,this._isNull),w("ChartLegend.showShadow",v,"1.7",O),this._S},set:function(e){this._S=e,I(this,"ShowShadow",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"top",{get:function(){return T("top",this._T,Zt,this._isNull),w("ChartLegend.top",v,"1.7",O),this._T},set:function(e){this._T=e,I(this,"Top",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"visible",{get:function(){return T("visible",this._V,Zt,this._isNull),this._V},set:function(e){this._V=e,I(this,"Visible",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"width",{get:function(){return T("width",this._W,Zt,this._isNull),w("ChartLegend.width",v,"1.7",O),this._W},set:function(e){this._W=e,I(this,"Width",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["visible","position","overlay","left","top","width","height","showShadow"],["format"],["legendEntries"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Height)||(this._H=n.Height),N(n.Left)||(this._L=n.Left),N(n.Overlay)||(this._O=n.Overlay),N(n.Position)||(this._P=n.Position),N(n.ShowShadow)||(this._S=n.ShowShadow),N(n.Top)||(this._T=n.Top),N(n.Visible)||(this._V=n.Visible),N(n.Width)||(this._W=n.Width),L(this,n,["format","Format","legendEntries","LegendEntries"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{height:this._H,left:this._L,overlay:this._O,position:this._P,showShadow:this._S,top:this._T,visible:this._V,width:this._W},{format:this._F,legendEntries:this._Le})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartLegend=$t;var en="ChartLegendEntry",tn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ChartLegendEntry"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["visible","left","top","width","height","index"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Visible","Left","Top","Width","Height","Index"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!1,!1,!1,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"height",{get:function(){return T("height",this._H,en,this._isNull),w("ChartLegendEntry.height",v,"1.8",O),this._H},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"index",{get:function(){return T("index",this._I,en,this._isNull),w("ChartLegendEntry.index",v,"1.8",O),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"left",{get:function(){return T("left",this._L,en,this._isNull),w("ChartLegendEntry.left",v,"1.8",O),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"top",{get:function(){return T("top",this._T,en,this._isNull),w("ChartLegendEntry.top",v,"1.8",O),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"visible",{get:function(){return T("visible",this._V,en,this._isNull),this._V},set:function(e){this._V=e,I(this,"Visible",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return T("width",this._W,en,this._isNull),w("ChartLegendEntry.width",v,"1.8",O),this._W},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["visible"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Height)||(this._H=n.Height),N(n.Index)||(this._I=n.Index),N(n.Left)||(this._L=n.Left),N(n.Top)||(this._T=n.Top),N(n.Visible)||(this._V=n.Visible),N(n.Width)||(this._W=n.Width)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{height:this._H,index:this._I,left:this._L,top:this._T,visible:this._V,width:this._W},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ChartLegendEntry=tn;var nn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartLegendEntryCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"ChartLegendEntryCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItemAt=function(t){return C(e.ChartLegendEntry,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.ChartLegendEntry,!1,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.ChartLegendEntry,!1,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.ChartLegendEntry,!1,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.ChartLegendEntryCollection=nn;var rn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartLegendFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["font","fill","border"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"border",{get:function(){return w("ChartLegendFormat.border",v,"1.8",O),this._B||(this._B=S(e.ChartBorder,this,"Border",!1,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.ChartFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._Fo||(this._Fo=S(e.ChartFont,this,"Font",!1,4)),this._Fo},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["font","border"],["fill"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["border","Border","fill","Fill","font","Font"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{border:this._B,font:this._Fo})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartLegendFormat=rn;var on="ChartMapOptions",sn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ChartMapOptions"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["level","labelStrategy","projectionType"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Level","LabelStrategy","ProjectionType"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"labelStrategy",{get:function(){return T("labelStrategy",this._L,on,this._isNull),this._L},set:function(e){this._L=e,I(this,"LabelStrategy",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"level",{get:function(){return T("level",this._Le,on,this._isNull),this._Le},set:function(e){this._Le=e,I(this,"Level",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"projectionType",{get:function(){return T("projectionType",this._P,on,this._isNull),this._P},set:function(e){this._P=e,I(this,"ProjectionType",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["level","labelStrategy","projectionType"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.LabelStrategy)||(this._L=n.LabelStrategy),N(n.Level)||(this._Le=n.Level),N(n.ProjectionType)||(this._P=n.ProjectionType)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{labelStrategy:this._L,level:this._Le,projectionType:this._P},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ChartMapOptions=sn;var an="ChartTitle",ln=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartTitle"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["visible","text","overlay","horizontalAlignment","top","left","width","height","verticalAlignment","textOrientation","position","showShadow"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Visible","Text","Overlay","HorizontalAlignment","Top","Left","Width","Height","VerticalAlignment","TextOrientation","Position","ShowShadow"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!1,!1,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartTitleFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"height",{get:function(){return T("height",this._H,an,this._isNull),w("ChartTitle.height",v,"1.7",O),this._H},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"horizontalAlignment",{get:function(){return T("horizontalAlignment",this._Ho,an,this._isNull),w("ChartTitle.horizontalAlignment",v,"1.7",O),this._Ho},set:function(e){this._Ho=e,I(this,"HorizontalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"left",{get:function(){return T("left",this._L,an,this._isNull),w("ChartTitle.left",v,"1.7",O),this._L},set:function(e){this._L=e,I(this,"Left",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"overlay",{get:function(){return T("overlay",this._O,an,this._isNull),this._O},set:function(e){this._O=e,I(this,"Overlay",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"position",{get:function(){return T("position",this._P,an,this._isNull),w("ChartTitle.position",v,"1.7",O),this._P},set:function(e){this._P=e,I(this,"Position",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showShadow",{get:function(){return T("showShadow",this._S,an,this._isNull),w("ChartTitle.showShadow",v,"1.7",O),this._S},set:function(e){this._S=e,I(this,"ShowShadow",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"text",{get:function(){return T("text",this._T,an,this._isNull),this._T},set:function(e){this._T=e,I(this,"Text",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"textOrientation",{get:function(){return T("textOrientation",this._Te,an,this._isNull),w("ChartTitle.textOrientation",v,"1.7",O),this._Te},set:function(e){this._Te=e,I(this,"TextOrientation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"top",{get:function(){return T("top",this._To,an,this._isNull),w("ChartTitle.top",v,"1.7",O),this._To},set:function(e){this._To=e,I(this,"Top",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"verticalAlignment",{get:function(){return T("verticalAlignment",this._V,an,this._isNull),w("ChartTitle.verticalAlignment",v,"1.7",O),this._V},set:function(e){this._V=e,I(this,"VerticalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"visible",{get:function(){return T("visible",this._Vi,an,this._isNull),this._Vi},set:function(e){this._Vi=e,I(this,"Visible",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"width",{get:function(){return T("width",this._W,an,this._isNull),w("ChartTitle.width",v,"1.7",O),this._W},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["visible","text","overlay","horizontalAlignment","top","left","verticalAlignment","textOrientation","position","showShadow"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.getSubstring=function(t,n){return w("ChartTitle.getSubstring",v,"1.7",O),C(e.ChartFormatString,this,"GetSubstring",1,[t,n],!1,!1,null,4)},n.prototype.setFormula=function(e){w("ChartTitle.setFormula",v,"1.7",O),E(this,"SetFormula",0,[e],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Height)||(this._H=n.Height),N(n.HorizontalAlignment)||(this._Ho=n.HorizontalAlignment),N(n.Left)||(this._L=n.Left),N(n.Overlay)||(this._O=n.Overlay),N(n.Position)||(this._P=n.Position),N(n.ShowShadow)||(this._S=n.ShowShadow),N(n.Text)||(this._T=n.Text),N(n.TextOrientation)||(this._Te=n.TextOrientation),N(n.Top)||(this._To=n.Top),N(n.VerticalAlignment)||(this._V=n.VerticalAlignment),N(n.Visible)||(this._Vi=n.Visible),N(n.Width)||(this._W=n.Width),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{height:this._H,horizontalAlignment:this._Ho,left:this._L,overlay:this._O,position:this._P,showShadow:this._S,text:this._T,textOrientation:this._Te,top:this._To,verticalAlignment:this._V,visible:this._Vi,width:this._W},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartTitle=ln;var un=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartFormatString"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["font"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._F||(this._F=S(e.ChartFont,this,"Font",!1,4)),this._F},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["font"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["font","Font"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{font:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartFormatString=un;var cn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartTitleFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["font","fill","border"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"border",{get:function(){return w("ChartTitleFormat.border",v,"1.7",O),this._B||(this._B=S(e.ChartBorder,this,"Border",!1,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.ChartFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._Fo||(this._Fo=S(e.ChartFont,this,"Font",!1,4)),this._Fo},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["font","border"],["fill"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["border","Border","fill","Fill","font","Font"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{border:this._B,font:this._Fo})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartTitleFormat=cn;var pn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ChartFill"},enumerable:!0,configurable:!0}),t.prototype.clear=function(){E(this,"Clear",0,[],0,0)},t.prototype.setSolidColor=function(e){E(this,"SetSolidColor",0,[e],V(2,"ExcelApiUndo","1.5"),0)},t.prototype._handleResult=function(t){(e.prototype._handleResult.call(this,t),x(t))||U(this,t)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{},{})},t}(OfficeExtension.ClientObject);e.ChartFill=pn;var fn=function(){function e(){}return e.prototype.load=function(e){return j(this,e),this},e}();e.ChartFillCustom=fn,OfficeExtension.Utility.applyMixin(pn,fn);var hn="ChartBorder",dn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ChartBorder"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["color","lineStyle","weight"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Color","LineStyle","Weight"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return T("color",this._C,hn,this._isNull),this._C},set:function(e){this._C=e,I(this,"Color",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lineStyle",{get:function(){return T("lineStyle",this._L,hn,this._isNull),this._L},set:function(e){this._L=e,I(this,"LineStyle",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"weight",{get:function(){return T("weight",this._W,hn,this._isNull),this._W},set:function(e){this._W=e,I(this,"Weight",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["color","lineStyle","weight"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype.clear=function(){w("ChartBorder.clear",v,"1.8",O),E(this,"Clear",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Color)||(this._C=n.Color),N(n.LineStyle)||(this._L=n.LineStyle),N(n.Weight)||(this._W=n.Weight)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{color:this._C,lineStyle:this._L,weight:this._W},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ChartBorder=dn;var yn="ChartBinOptions",mn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ChartBinOptions"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["type","width","count","allowOverflow","allowUnderflow","overflowValue","underflowValue"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Type","Width","Count","AllowOverflow","AllowUnderflow","OverflowValue","UnderflowValue"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"allowOverflow",{get:function(){return T("allowOverflow",this._A,yn,this._isNull),this._A},set:function(e){this._A=e,I(this,"AllowOverflow",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"allowUnderflow",{get:function(){return T("allowUnderflow",this._Al,yn,this._isNull),this._Al},set:function(e){this._Al=e,I(this,"AllowUnderflow",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"count",{get:function(){return T("count",this._C,yn,this._isNull),this._C},set:function(e){this._C=e,I(this,"Count",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"overflowValue",{get:function(){return T("overflowValue",this._O,yn,this._isNull),this._O},set:function(e){this._O=e,I(this,"OverflowValue",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"type",{get:function(){return T("type",this._T,yn,this._isNull),this._T},set:function(e){this._T=e,I(this,"Type",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"underflowValue",{get:function(){return T("underflowValue",this._U,yn,this._isNull),this._U},set:function(e){this._U=e,I(this,"UnderflowValue",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return T("width",this._W,yn,this._isNull),this._W},set:function(e){this._W=e,I(this,"Width",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["type","width","count","allowOverflow","allowUnderflow","overflowValue","underflowValue"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.AllowOverflow)||(this._A=n.AllowOverflow),N(n.AllowUnderflow)||(this._Al=n.AllowUnderflow),N(n.Count)||(this._C=n.Count),N(n.OverflowValue)||(this._O=n.OverflowValue),N(n.Type)||(this._T=n.Type),N(n.UnderflowValue)||(this._U=n.UnderflowValue),N(n.Width)||(this._W=n.Width)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{allowOverflow:this._A,allowUnderflow:this._Al,count:this._C,overflowValue:this._O,type:this._T,underflowValue:this._U,width:this._W},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ChartBinOptions=mn;var gn="ChartBoxwhiskerOptions",_n=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ChartBoxwhiskerOptions"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["showInnerPoints","showOutlierPoints","showMeanMarker","showMeanLine","quartileCalculation"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["ShowInnerPoints","ShowOutlierPoints","ShowMeanMarker","ShowMeanLine","QuartileCalculation"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"quartileCalculation",{get:function(){return T("quartileCalculation",this._Q,gn,this._isNull),this._Q},set:function(e){this._Q=e,I(this,"QuartileCalculation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"showInnerPoints",{get:function(){return T("showInnerPoints",this._S,gn,this._isNull),this._S},set:function(e){this._S=e,I(this,"ShowInnerPoints",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"showMeanLine",{get:function(){return T("showMeanLine",this._Sh,gn,this._isNull),this._Sh},set:function(e){this._Sh=e,I(this,"ShowMeanLine",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"showMeanMarker",{get:function(){return T("showMeanMarker",this._Sho,gn,this._isNull),this._Sho},set:function(e){this._Sho=e,I(this,"ShowMeanMarker",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"showOutlierPoints",{get:function(){return T("showOutlierPoints",this._Show,gn,this._isNull),this._Show},set:function(e){this._Show=e,I(this,"ShowOutlierPoints",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["showInnerPoints","showOutlierPoints","showMeanMarker","showMeanLine","quartileCalculation"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.QuartileCalculation)||(this._Q=n.QuartileCalculation),N(n.ShowInnerPoints)||(this._S=n.ShowInnerPoints),N(n.ShowMeanLine)||(this._Sh=n.ShowMeanLine),N(n.ShowMeanMarker)||(this._Sho=n.ShowMeanMarker),N(n.ShowOutlierPoints)||(this._Show=n.ShowOutlierPoints)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{quartileCalculation:this._Q,showInnerPoints:this._S,showMeanLine:this._Sh,showMeanMarker:this._Sho,showOutlierPoints:this._Show},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ChartBoxwhiskerOptions=_n;var bn="ChartLineFormat",On=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ChartLineFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["color","lineStyle","weight"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Color","LineStyle","Weight"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return T("color",this._C,bn,this._isNull),this._C},set:function(e){this._C=e,I(this,"Color",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lineStyle",{get:function(){return T("lineStyle",this._L,bn,this._isNull),w("ChartLineFormat.lineStyle",v,"1.7",O),this._L},set:function(e){this._L=e,I(this,"LineStyle",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"weight",{get:function(){return T("weight",this._W,bn,this._isNull),w("ChartLineFormat.weight",v,"1.7",O),this._W},set:function(e){this._W=e,I(this,"Weight",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["color","lineStyle","weight"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype.clear=function(){E(this,"Clear",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Color)||(this._C=n.Color),N(n.LineStyle)||(this._L=n.LineStyle),N(n.Weight)||(this._W=n.Weight)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{color:this._C,lineStyle:this._L,weight:this._W},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ChartLineFormat=On;var vn="ChartFont",Sn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ChartFont"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["bold","color","italic","name","size","underline"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Bold","Color","Italic","Name","Size","Underline"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"bold",{get:function(){return T("bold",this._B,vn,this._isNull),this._B},set:function(e){this._B=e,I(this,"Bold",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return T("color",this._C,vn,this._isNull),this._C},set:function(e){this._C=e,I(this,"Color",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"italic",{get:function(){return T("italic",this._I,vn,this._isNull),this._I},set:function(e){this._I=e,I(this,"Italic",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"name",{get:function(){return T("name",this._N,vn,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"size",{get:function(){return T("size",this._S,vn,this._isNull),this._S},set:function(e){this._S=e,I(this,"Size",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"underline",{get:function(){return T("underline",this._U,vn,this._isNull),this._U},set:function(e){this._U=e,I(this,"Underline",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["bold","color","italic","name","size","underline"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Bold)||(this._B=n.Bold),N(n.Color)||(this._C=n.Color),N(n.Italic)||(this._I=n.Italic),N(n.Name)||(this._N=n.Name),N(n.Size)||(this._S=n.Size),N(n.Underline)||(this._U=n.Underline)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{bold:this._B,color:this._C,italic:this._I,name:this._N,size:this._S,underline:this._U},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ChartFont=Sn;var Cn="ChartTrendline",Pn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartTrendline"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["type","polynomialOrder","movingAveragePeriod","_Id","showEquation","showRSquared","forwardPeriod","backwardPeriod","name","intercept"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Type","PolynomialOrder","MovingAveragePeriod","_Id","ShowEquation","ShowRSquared","ForwardPeriod","BackwardPeriod","Name","Intercept"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!1,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format","label"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartTrendlineFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"label",{get:function(){return w("ChartTrendline.label",v,"1.8",O),this._L||(this._L=S(e.ChartTrendlineLabel,this,"Label",!1,4)),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"backwardPeriod",{get:function(){return T("backwardPeriod",this._B,Cn,this._isNull),w("ChartTrendline.backwardPeriod",v,"1.8",O),this._B},set:function(e){this._B=e,I(this,"BackwardPeriod",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"forwardPeriod",{get:function(){return T("forwardPeriod",this._Fo,Cn,this._isNull),w("ChartTrendline.forwardPeriod",v,"1.8",O),this._Fo},set:function(e){this._Fo=e,I(this,"ForwardPeriod",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"intercept",{get:function(){return T("intercept",this._I,Cn,this._isNull),this._I},set:function(e){this._I=e,I(this,"Intercept",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"movingAveragePeriod",{get:function(){return T("movingAveragePeriod",this._M,Cn,this._isNull),this._M},set:function(e){this._M=e,I(this,"MovingAveragePeriod",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,Cn,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"polynomialOrder",{get:function(){return T("polynomialOrder",this._P,Cn,this._isNull),this._P},set:function(e){this._P=e,I(this,"PolynomialOrder",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showEquation",{get:function(){return T("showEquation",this._S,Cn,this._isNull),w("ChartTrendline.showEquation",v,"1.8",O),this._S},set:function(e){this._S=e,I(this,"ShowEquation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showRSquared",{get:function(){return T("showRSquared",this._Sh,Cn,this._isNull),w("ChartTrendline.showRSquared",v,"1.8",O),this._Sh},set:function(e){this._Sh=e,I(this,"ShowRSquared",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"type",{get:function(){return T("type",this._T,Cn,this._isNull),this._T},set:function(e){this._T=e,I(this,"Type",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_Id",{get:function(){return T("_Id",this.__I,Cn,this._isNull),this.__I},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["type","polynomialOrder","movingAveragePeriod","showEquation","showRSquared","forwardPeriod","backwardPeriod","name","intercept"],["format","label"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.BackwardPeriod)||(this._B=n.BackwardPeriod),N(n.ForwardPeriod)||(this._Fo=n.ForwardPeriod),N(n.Intercept)||(this._I=n.Intercept),N(n.MovingAveragePeriod)||(this._M=n.MovingAveragePeriod),N(n.Name)||(this._N=n.Name),N(n.PolynomialOrder)||(this._P=n.PolynomialOrder),N(n.ShowEquation)||(this._S=n.ShowEquation),N(n.ShowRSquared)||(this._Sh=n.ShowRSquared),N(n.Type)||(this._T=n.Type),N(n._Id)||(this.__I=n._Id),L(this,n,["format","Format","label","Label"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e._Id)||(this.__I=e._Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{backwardPeriod:this._B,forwardPeriod:this._Fo,intercept:this._I,movingAveragePeriod:this._M,name:this._N,polynomialOrder:this._P,showEquation:this._S,showRSquared:this._Sh,type:this._T},{format:this._F,label:this._L})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartTrendline=Pn;var An=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartTrendlineCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"ChartTrendlineCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t){return C(e.ChartTrendline,this,"Add",0,[t],!1,!0,null,V(2,"ExcelApiUndo","1.5"))},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.ChartTrendline,this,[t])},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.ChartTrendline,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.ChartTrendline,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.ChartTrendline,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.ChartTrendlineCollection=An;var Fn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartTrendlineFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["line"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"line",{get:function(){return this._L||(this._L=S(e.ChartLineFormat,this,"Line",!1,4)),this._L},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["line"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["line","Line"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{line:this._L})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartTrendlineFormat=Fn;var Dn="ChartTrendlineLabel",En=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartTrendlineLabel"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["top","left","width","height","formula","textOrientation","horizontalAlignment","verticalAlignment","text","autoText","numberFormat","linkNumberFormat"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Top","Left","Width","Height","Formula","TextOrientation","HorizontalAlignment","VerticalAlignment","Text","AutoText","NumberFormat","LinkNumberFormat"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!1,!1,!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartTrendlineLabelFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"autoText",{get:function(){return T("autoText",this._A,Dn,this._isNull),this._A},set:function(e){this._A=e,I(this,"AutoText",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"formula",{get:function(){return T("formula",this._Fo,Dn,this._isNull),this._Fo},set:function(e){this._Fo=e,I(this,"Formula",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"height",{get:function(){return T("height",this._H,Dn,this._isNull),this._H},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"horizontalAlignment",{get:function(){return T("horizontalAlignment",this._Ho,Dn,this._isNull),this._Ho},set:function(e){this._Ho=e,I(this,"HorizontalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"left",{get:function(){return T("left",this._L,Dn,this._isNull),this._L},set:function(e){this._L=e,I(this,"Left",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"linkNumberFormat",{get:function(){return T("linkNumberFormat",this._Li,Dn,this._isNull),w("ChartTrendlineLabel.linkNumberFormat",v,"1.9",O),this._Li},set:function(e){this._Li=e,I(this,"LinkNumberFormat",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormat",{get:function(){return T("numberFormat",this._N,Dn,this._isNull),this._N},set:function(e){this._N=e,I(this,"NumberFormat",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"text",{get:function(){return T("text",this._T,Dn,this._isNull),this._T},set:function(e){this._T=e,I(this,"Text",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"textOrientation",{get:function(){return T("textOrientation",this._Te,Dn,this._isNull),this._Te},set:function(e){this._Te=e,I(this,"TextOrientation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"top",{get:function(){return T("top",this._To,Dn,this._isNull),this._To},set:function(e){this._To=e,I(this,"Top",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"verticalAlignment",{get:function(){return T("verticalAlignment",this._V,Dn,this._isNull),this._V},set:function(e){this._V=e,I(this,"VerticalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"width",{get:function(){return T("width",this._W,Dn,this._isNull),this._W},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["top","left","formula","textOrientation","horizontalAlignment","verticalAlignment","text","autoText","numberFormat","linkNumberFormat"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AutoText)||(this._A=n.AutoText),N(n.Formula)||(this._Fo=n.Formula),N(n.Height)||(this._H=n.Height),N(n.HorizontalAlignment)||(this._Ho=n.HorizontalAlignment),N(n.Left)||(this._L=n.Left),N(n.LinkNumberFormat)||(this._Li=n.LinkNumberFormat),N(n.NumberFormat)||(this._N=n.NumberFormat),N(n.Text)||(this._T=n.Text),N(n.TextOrientation)||(this._Te=n.TextOrientation),N(n.Top)||(this._To=n.Top),N(n.VerticalAlignment)||(this._V=n.VerticalAlignment),N(n.Width)||(this._W=n.Width),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{autoText:this._A,formula:this._Fo,height:this._H,horizontalAlignment:this._Ho,left:this._L,linkNumberFormat:this._Li,numberFormat:this._N,text:this._T,textOrientation:this._Te,top:this._To,verticalAlignment:this._V,width:this._W},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartTrendlineLabel=En;var Rn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartTrendlineLabelFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["fill","border","font"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"border",{get:function(){return this._B||(this._B=S(e.ChartBorder,this,"Border",!1,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.ChartFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._Fo||(this._Fo=S(e.ChartFont,this,"Font",!1,4)),this._Fo},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["border","font"],["fill"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["border","Border","fill","Fill","font","Font"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{border:this._B,font:this._Fo})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartTrendlineLabelFormat=Rn;var In="ChartPlotArea",xn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartPlotArea"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["left","top","width","height","insideLeft","insideTop","insideWidth","insideHeight","position"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Left","Top","Width","Height","InsideLeft","InsideTop","InsideWidth","InsideHeight","Position"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ChartPlotAreaFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"height",{get:function(){return T("height",this._H,In,this._isNull),this._H},set:function(e){this._H=e,I(this,"Height",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"insideHeight",{get:function(){return T("insideHeight",this._I,In,this._isNull),this._I},set:function(e){this._I=e,I(this,"InsideHeight",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"insideLeft",{get:function(){return T("insideLeft",this._In,In,this._isNull),this._In},set:function(e){this._In=e,I(this,"InsideLeft",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"insideTop",{get:function(){return T("insideTop",this._Ins,In,this._isNull),this._Ins},set:function(e){this._Ins=e,I(this,"InsideTop",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"insideWidth",{get:function(){return T("insideWidth",this._Insi,In,this._isNull),this._Insi},set:function(e){this._Insi=e,I(this,"InsideWidth",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"left",{get:function(){return T("left",this._L,In,this._isNull),this._L},set:function(e){this._L=e,I(this,"Left",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"position",{get:function(){return T("position",this._P,In,this._isNull),this._P},set:function(e){this._P=e,I(this,"Position",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"top",{get:function(){return T("top",this._T,In,this._isNull),this._T},set:function(e){this._T=e,I(this,"Top",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"width",{get:function(){return T("width",this._W,In,this._isNull),this._W},set:function(e){this._W=e,I(this,"Width",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["left","top","width","height","insideLeft","insideTop","insideWidth","insideHeight","position"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Height)||(this._H=n.Height),N(n.InsideHeight)||(this._I=n.InsideHeight),N(n.InsideLeft)||(this._In=n.InsideLeft),N(n.InsideTop)||(this._Ins=n.InsideTop),N(n.InsideWidth)||(this._Insi=n.InsideWidth),N(n.Left)||(this._L=n.Left),N(n.Position)||(this._P=n.Position),N(n.Top)||(this._T=n.Top),N(n.Width)||(this._W=n.Width),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{height:this._H,insideHeight:this._I,insideLeft:this._In,insideTop:this._Ins,insideWidth:this._Insi,left:this._L,position:this._P,top:this._T,width:this._W},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartPlotArea=xn;var Nn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ChartPlotAreaFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["border","fill"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"border",{get:function(){return this._B||(this._B=S(e.ChartBorder,this,"Border",!1,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.ChartFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["border"],["fill"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["border","Border","fill","Fill"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{border:this._B})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ChartPlotAreaFormat=Nn;var Tn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"VisualCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"VisualCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n,r){return C(e.Visual,this,"Add",0,[t,n,r],!1,!0,null,2)},n.prototype.bootstrapAgaveVisual=function(){E(this,"BootstrapAgaveVisual",0,[],2,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getDefinitions=function(){return E(this,"GetDefinitions",1,[],4,0)},n.prototype.getPreview=function(e,t,n,r){return E(this,"GetPreview",1,[e,t,n,r],4,0)},n.prototype.getSelectedOrNullObject=function(){return C(e.Visual,this,"GetSelectedOrNullObject",1,[],!1,!1,null,4)},n.prototype._GetItem=function(t){return P(e.Visual,this,[t])},n.prototype._RegisterSelectionChangedEvent=function(){E(this,"_RegisterSelectionChangedEvent",1,[],0,0)},n.prototype._UnregisterSelectionChangedEvent=function(){E(this,"_UnregisterSelectionChangedEvent",1,[],0,0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Visual,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Visual,!0,i,t,n)}))},Object.defineProperty(n.prototype,"onAgaveVisualUpdate",{get:function(){var e=this;return this.m_agaveVisualUpdate||(this.m_agaveVisualUpdate=new OfficeExtension.GenericEventHandlers(this.context,this,"AgaveVisualUpdate",{eventType:150,registerFunc:function(){return OfficeExtension.Utility._createPromiseFromResult(null)},unregisterFunc:function(){return OfficeExtension.Utility._createPromiseFromResult(null)},getTargetIdFunc:function(){return""},eventArgsTransformFunc:function(t){var n=b.VisualCollection_AgaveVisualUpdate_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_agaveVisualUpdate},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onSelectionChanged",{get:function(){var e=this;return this.m_selectionChanged||(this.m_selectionChanged=new OfficeExtension.GenericEventHandlers(this.context,this,"SelectionChanged",{eventType:2e3,registerFunc:function(){return e._RegisterSelectionChangedEvent()},unregisterFunc:function(){return e._UnregisterSelectionChangedEvent()},getTargetIdFunc:function(){return e._ParentObject.id},eventArgsTransformFunc:function(e){var t={type:Oo.visualSelectionChanged,revisionLoad:e.revisionLoad,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_selectionChanged},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Visual,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.VisualCollection=Tn;var wn=function(){function e(){}return Object.defineProperty(e.prototype,"_ParentObject",{get:function(){return this.m__ParentObject},set:function(e){this.m__ParentObject=e},enumerable:!0,configurable:!0}),e}();e.VisualCollectionCustom=wn,OfficeExtension.Utility.applyMixin(Tn,wn),function(e){e.VisualCollection_AgaveVisualUpdate_EventArgsTransform=function(e,t){var n=t;return{type:n.type,payload:n.payload}}}(b=e._CC||(e._CC={}));var jn="Visual",Mn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Visual"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","isSupportedInVisualTaskpane"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","IsSupportedInVisualTaskpane"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["properties"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"properties",{get:function(){return this._P||(this._P=S(e.VisualPropertyCollection,this,"Properties",!0,4)),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,jn,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isSupportedInVisualTaskpane",{get:function(){return T("isSupportedInVisualTaskpane",this._Is,jn,this._isNull),this._Is},enumerable:!0,configurable:!0}),n.prototype.addChildProperty=function(e,t){return w("Visual.addChildProperty","ExcelApiOnline","1.1",O),E(this,"AddChildProperty",0,[e,t],2,0)},n.prototype.changeDataSource=function(e,t){E(this,"ChangeDataSource",0,[e,t],2,0)},n.prototype.delete=function(){E(this,"Delete",0,[],2,0)},n.prototype.deserializeProperties=function(e){E(this,"DeserializeProperties",0,[e],2,0)},n.prototype.getChildProperties=function(t,n){return C(e.VisualPropertyCollection,this,"GetChildProperties",1,[t,n],!0,!1,null,4)},n.prototype.getDataControllerClient=function(){return C(e.DataControllerClient,this,"GetDataControllerClient",1,[],!1,!1,null,4)},n.prototype.getDataSource=function(){return E(this,"GetDataSource",1,[],4,0)},n.prototype.getElementChildProperties=function(t,n,r){return C(e.VisualPropertyCollection,this,"GetElementChildProperties",1,[t,n,r],!0,!1,null,4)},n.prototype.getProperty=function(e){return E(this,"GetProperty",1,[e],4,0)},n.prototype.removeChildProperty=function(e,t){w("Visual.removeChildProperty","ExcelApiOnline","1.1",O),E(this,"RemoveChildProperty",0,[e,t],2,0)},n.prototype.serializeProperties=function(){return E(this,"SerializeProperties",1,[],4,0)},n.prototype.setProperty=function(e,t){E(this,"SetProperty",0,[e,t],2,0)},n.prototype.setPropertyToDefault=function(e){E(this,"SetPropertyToDefault",0,[e],2,0)},n.prototype._RegisterChangeEvent=function(){E(this,"_RegisterChangeEvent",1,[],0,0)},n.prototype._UnregisterChangeEvent=function(){E(this,"_UnregisterChangeEvent",1,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Id)||(this._I=n.Id),N(n.IsSupportedInVisualTaskpane)||(this._Is=n.IsSupportedInVisualTaskpane),L(this,n,["properties","Properties"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},Object.defineProperty(n.prototype,"onChangeNotification",{get:function(){var e=this;return this.m_changeNotification||(this.m_changeNotification=new OfficeExtension.GenericEventHandlers(this.context,this,"ChangeNotification",{eventType:2001,registerFunc:function(){return e._RegisterChangeEvent()},unregisterFunc:function(){return e._UnregisterChangeEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(t){var n=b.Visual_ChangeNotification_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_changeNotification},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{id:this._I,isSupportedInVisualTaskpane:this._Is},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Visual=Mn,function(t){t.Visual_ChangeNotification_EventArgsTransform=function(t,n){var r=n;return{targetId:r.targetId,changeType:r.changeType,payload:r.payload,type:e.EventType.visualChange}}}(b=e._CC||(e._CC={}));var kn="VisualProperty",Un=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"VisualProperty"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["type","value","id","localizedName","options","localizedOptions","hasDefault","isDefault","min","max","stepSize","hideMeButShowChildrenUI","expandableUI","nextPropOnSameLine","showResetUI","size","minSize","maxSize","index","parentName"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Type","Value","Id","LocalizedName","Options","LocalizedOptions","HasDefault","IsDefault","Min","Max","StepSize","HideMeButShowChildrenUI","ExpandableUI","NextPropOnSameLine","ShowResetUI","Size","MinSize","MaxSize","Index","ParentName"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"expandableUI",{get:function(){return T("expandableUI",this._E,kn,this._isNull),this._E},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"hasDefault",{get:function(){return T("hasDefault",this._H,kn,this._isNull),this._H},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"hideMeButShowChildrenUI",{get:function(){return T("hideMeButShowChildrenUI",this._Hi,kn,this._isNull),this._Hi},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"id",{get:function(){return T("id",this._I,kn,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"index",{get:function(){return T("index",this._In,kn,this._isNull),w("VisualProperty.index","ExcelApiOnline","1.1",O),this._In},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isDefault",{get:function(){return T("isDefault",this._Is,kn,this._isNull),this._Is},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"localizedName",{get:function(){return T("localizedName",this._L,kn,this._isNull),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"localizedOptions",{get:function(){return T("localizedOptions",this._Lo,kn,this._isNull),this._Lo},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"max",{get:function(){return T("max",this._M,kn,this._isNull),this._M},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"maxSize",{get:function(){return T("maxSize",this._Ma,kn,this._isNull),w("VisualProperty.maxSize","ExcelApiOnline","1.1",O),this._Ma},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"min",{get:function(){return T("min",this._Mi,kn,this._isNull),this._Mi},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"minSize",{get:function(){return T("minSize",this._Min,kn,this._isNull),w("VisualProperty.minSize","ExcelApiOnline","1.1",O),this._Min},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"nextPropOnSameLine",{get:function(){return T("nextPropOnSameLine",this._N,kn,this._isNull),this._N},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"options",{get:function(){return T("options",this._O,kn,this._isNull),this._O},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"parentName",{get:function(){return T("parentName",this._P,kn,this._isNull),w("VisualProperty.parentName","ExcelApiOnline","1.1",O),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"showResetUI",{get:function(){return T("showResetUI",this._S,kn,this._isNull),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"size",{get:function(){return T("size",this._Si,kn,this._isNull),w("VisualProperty.size","ExcelApiOnline","1.1",O),this._Si},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"stepSize",{get:function(){return T("stepSize",this._St,kn,this._isNull),this._St},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"type",{get:function(){return T("type",this._T,kn,this._isNull),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"value",{get:function(){return T("value",this._V,kn,this._isNull),this._V},enumerable:!0,configurable:!0}),t.prototype.getBoolMetaProperty=function(e){return E(this,"GetBoolMetaProperty",1,[e],4,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.ExpandableUI)||(this._E=n.ExpandableUI),N(n.HasDefault)||(this._H=n.HasDefault),N(n.HideMeButShowChildrenUI)||(this._Hi=n.HideMeButShowChildrenUI),N(n.Id)||(this._I=n.Id),N(n.Index)||(this._In=n.Index),N(n.IsDefault)||(this._Is=n.IsDefault),N(n.LocalizedName)||(this._L=n.LocalizedName),N(n.LocalizedOptions)||(this._Lo=n.LocalizedOptions),N(n.Max)||(this._M=n.Max),N(n.MaxSize)||(this._Ma=n.MaxSize),N(n.Min)||(this._Mi=n.Min),N(n.MinSize)||(this._Min=n.MinSize),N(n.NextPropOnSameLine)||(this._N=n.NextPropOnSameLine),N(n.Options)||(this._O=n.Options),N(n.ParentName)||(this._P=n.ParentName),N(n.ShowResetUI)||(this._S=n.ShowResetUI),N(n.Size)||(this._Si=n.Size),N(n.StepSize)||(this._St=n.StepSize),N(n.Type)||(this._T=n.Type),N(n.Value)||(this._V=n.Value)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleIdResult=function(t){e.prototype._handleIdResult.call(this,t),x(t)||N(t.Id)||(this._I=t.Id)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{expandableUI:this._E,hasDefault:this._H,hideMeButShowChildrenUI:this._Hi,id:this._I,index:this._In,isDefault:this._Is,localizedName:this._L,localizedOptions:this._Lo,max:this._M,maxSize:this._Ma,min:this._Mi,minSize:this._Min,nextPropOnSameLine:this._N,options:this._O,parentName:this._P,showResetUI:this._S,size:this._Si,stepSize:this._St,type:this._T,value:this._V},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.VisualProperty=Un;var Ln=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"VisualPropertyCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"VisualPropertyCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.VisualProperty,this,[t])},n.prototype.getItemAt=function(t){return C(e.VisualProperty,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.VisualProperty,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.VisualProperty,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.VisualProperty,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.VisualPropertyCollection=Ln;var Bn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"DataControllerClient"},enumerable:!0,configurable:!0}),t.prototype.addField=function(e,t,n){E(this,"AddField",0,[e,t,n],2,0)},t.prototype.getAssociatedFields=function(e){return E(this,"GetAssociatedFields",1,[e],4,0)},t.prototype.getAvailableFields=function(e){return E(this,"GetAvailableFields",1,[e],4,0)},t.prototype.getWells=function(){return E(this,"GetWells",1,[],4,0)},t.prototype.moveField=function(e,t,n){E(this,"MoveField",0,[e,t,n],2,0)},t.prototype.removeField=function(e,t){E(this,"RemoveField",0,[e,t],2,0)},t.prototype._handleResult=function(t){(e.prototype._handleResult.call(this,t),x(t))||U(this,t)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{},{})},t}(OfficeExtension.ClientObject);e.DataControllerClient=Bn;var Hn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"RangeSort"},enumerable:!0,configurable:!0}),t.prototype.apply=function(e,t,n,r,i){E(this,"Apply",0,[e,t,n,r,i],0,0)},t.prototype._handleResult=function(t){(e.prototype._handleResult.call(this,t),x(t))||U(this,t)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{},{})},t}(OfficeExtension.ClientObject);e.RangeSort=Hn;var Wn="TableSort",Vn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"TableSort"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["matchCase","method","fields"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["MatchCase","Method","Fields"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fields",{get:function(){return T("fields",this._F,Wn,this._isNull),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"matchCase",{get:function(){return T("matchCase",this._M,Wn,this._isNull),this._M},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"method",{get:function(){return T("method",this._Me,Wn,this._isNull),this._Me},enumerable:!0,configurable:!0}),t.prototype.apply=function(e,t,n){E(this,"Apply",0,[e,t,n],0,0)},t.prototype.clear=function(){E(this,"Clear",0,[],0,0)},t.prototype.reapply=function(){E(this,"Reapply",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Fields)||(this._F=n.Fields),N(n.MatchCase)||(this._M=n.MatchCase),N(n.Method)||(this._Me=n.Method)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{fields:this._F,matchCase:this._M,method:this._Me},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.TableSort=Vn;var Gn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"Filter"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["criteria"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Criteria"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"criteria",{get:function(){return T("criteria",this._C,"Filter",this._isNull),this._C},enumerable:!0,configurable:!0}),t.prototype.apply=function(e){E(this,"Apply",0,[e],0,0)},t.prototype.applyBottomItemsFilter=function(e){E(this,"ApplyBottomItemsFilter",0,[e],0,0)},t.prototype.applyBottomPercentFilter=function(e){E(this,"ApplyBottomPercentFilter",0,[e],0,0)},t.prototype.applyCellColorFilter=function(e){E(this,"ApplyCellColorFilter",0,[e],0,0)},t.prototype.applyCustomFilter=function(e,t,n){E(this,"ApplyCustomFilter",0,[e,t,n],0,0)},t.prototype.applyDynamicFilter=function(e){E(this,"ApplyDynamicFilter",0,[e],0,0)},t.prototype.applyFontColorFilter=function(e){E(this,"ApplyFontColorFilter",0,[e],0,0)},t.prototype.applyIconFilter=function(e){E(this,"ApplyIconFilter",0,[e],0,0)},t.prototype.applyTopItemsFilter=function(e){E(this,"ApplyTopItemsFilter",0,[e],0,0)},t.prototype.applyTopPercentFilter=function(e){E(this,"ApplyTopPercentFilter",0,[e],0,0)},t.prototype.applyValuesFilter=function(e){E(this,"ApplyValuesFilter",0,[e],0,0)},t.prototype.clear=function(){E(this,"Clear",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Criteria)||(this._C=n.Criteria)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{criteria:this._C},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.Filter=Gn;var zn="AutoFilter",qn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"AutoFilter"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["enabled","isDataFiltered","criteria"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Enabled","IsDataFiltered","Criteria"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"criteria",{get:function(){return T("criteria",this._C,zn,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"enabled",{get:function(){return T("enabled",this._E,zn,this._isNull),this._E},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isDataFiltered",{get:function(){return T("isDataFiltered",this._I,zn,this._isNull),this._I},enumerable:!0,configurable:!0}),n.prototype.apply=function(e,t,n){E(this,"Apply",0,[e,t,n],0,0)},n.prototype.clearColumnCriteria=function(e){w("AutoFilter.clearColumnCriteria",v,"1.14",O),E(this,"ClearColumnCriteria",0,[e],0,0)},n.prototype.clearCriteria=function(){E(this,"ClearCriteria",0,[],0,0)},n.prototype.getRange=function(){return C(e.Range,this,"GetRange",1,[],!1,!0,null,4)},n.prototype.getRangeOrNullObject=function(){return C(e.Range,this,"GetRangeOrNullObject",1,[],!1,!0,null,4)},n.prototype.reapply=function(){E(this,"Reapply",0,[],0,0)},n.prototype.remove=function(){E(this,"Remove",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Criteria)||(this._C=n.Criteria),N(n.Enabled)||(this._E=n.Enabled),N(n.IsDataFiltered)||(this._I=n.IsDataFiltered)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{criteria:this._C,enabled:this._E,isDataFiltered:this._I},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.AutoFilter=qn;var Jn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"CultureInfo"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["name"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["numberFormat","datetimeFormat"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"datetimeFormat",{get:function(){return w("CultureInfo.datetimeFormat",v,"1.12",O),this._D||(this._D=S(e.DatetimeFormatInfo,this,"DatetimeFormat",!1,4)),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormat",{get:function(){return this._Nu||(this._Nu=S(e.NumberFormatInfo,this,"NumberFormat",!1,4)),this._Nu},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,"CultureInfo",this._isNull),this._N},enumerable:!0,configurable:!0}),n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Name)||(this._N=n.Name),L(this,n,["datetimeFormat","DatetimeFormat","numberFormat","NumberFormat"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{name:this._N},{datetimeFormat:this._D,numberFormat:this._Nu})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.CultureInfo=Jn;var Kn="NumberFormatInfo",Qn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"NumberFormatInfo"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["numberDecimalSeparator","numberGroupSeparator"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["NumberDecimalSeparator","NumberGroupSeparator"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"numberDecimalSeparator",{get:function(){return T("numberDecimalSeparator",this._N,Kn,this._isNull),this._N},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"numberGroupSeparator",{get:function(){return T("numberGroupSeparator",this._Nu,Kn,this._isNull),this._Nu},enumerable:!0,configurable:!0}),t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.NumberDecimalSeparator)||(this._N=n.NumberDecimalSeparator),N(n.NumberGroupSeparator)||(this._Nu=n.NumberGroupSeparator)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{numberDecimalSeparator:this._N,numberGroupSeparator:this._Nu},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.NumberFormatInfo=Qn;var Xn="DatetimeFormatInfo",Yn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"DatetimeFormatInfo"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["dateSeparator","longDatePattern","shortDatePattern","timeSeparator","longTimePattern"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["DateSeparator","LongDatePattern","ShortDatePattern","TimeSeparator","LongTimePattern"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dateSeparator",{get:function(){return T("dateSeparator",this._D,Xn,this._isNull),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"longDatePattern",{get:function(){return T("longDatePattern",this._L,Xn,this._isNull),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"longTimePattern",{get:function(){return T("longTimePattern",this._Lo,Xn,this._isNull),this._Lo},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"shortDatePattern",{get:function(){return T("shortDatePattern",this._S,Xn,this._isNull),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"timeSeparator",{get:function(){return T("timeSeparator",this._T,Xn,this._isNull),this._T},enumerable:!0,configurable:!0}),t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.DateSeparator)||(this._D=n.DateSeparator),N(n.LongDatePattern)||(this._L=n.LongDatePattern),N(n.LongTimePattern)||(this._Lo=n.LongTimePattern),N(n.ShortDatePattern)||(this._S=n.ShortDatePattern),N(n.TimeSeparator)||(this._T=n.TimeSeparator)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{dateSeparator:this._D,longDatePattern:this._L,longTimePattern:this._Lo,shortDatePattern:this._S,timeSeparator:this._T},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.DatetimeFormatInfo=Yn;var Zn=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"CustomXmlPartScopedCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"CustomXmlPartScopedCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.CustomXmlPart,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.CustomXmlPart,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype.getOnlyItem=function(){return C(e.CustomXmlPart,this,"GetOnlyItem",1,[],!1,!1,null,4)},n.prototype.getOnlyItemOrNullObject=function(){return C(e.CustomXmlPart,this,"GetOnlyItemOrNullObject",1,[],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.CustomXmlPart,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.CustomXmlPart,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.CustomXmlPart,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.CustomXmlPartScopedCollection=Zn;var $n=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"CustomXmlPartCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"CustomXmlPartCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t){return C(e.CustomXmlPart,this,"Add",0,[t],!1,!0,null,0)},n.prototype.getByNamespace=function(t){return C(e.CustomXmlPartScopedCollection,this,"GetByNamespace",1,[t],!0,!1,null,4)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.CustomXmlPart,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.CustomXmlPart,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.CustomXmlPart,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.CustomXmlPart,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.CustomXmlPart,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.CustomXmlPartCollection=$n;var er="CustomXmlPart",tr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"CustomXmlPart"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["id","namespaceUri"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","NamespaceUri"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"id",{get:function(){return T("id",this._I,er,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"namespaceUri",{get:function(){return T("namespaceUri",this._N,er,this._isNull),this._N},enumerable:!0,configurable:!0}),t.prototype.delete=function(){E(this,"Delete",0,[],0,0)},t.prototype.getXml=function(){return E(this,"GetXml",1,[],4,0)},t.prototype.setXml=function(e){E(this,"SetXml",0,[e],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Id)||(this._I=n.Id),N(n.NamespaceUri)||(this._N=n.NamespaceUri)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleIdResult=function(t){e.prototype._handleIdResult.call(this,t),x(t)||N(t.Id)||(this._I=t.Id)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{id:this._I,namespaceUri:this._N},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.CustomXmlPart=tr;var nr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"_V1Api"},enumerable:!0,configurable:!0}),t.prototype.bindingAddColumns=function(e){return E(this,"BindingAddColumns",0,[e],0,0)},t.prototype.bindingAddFromNamedItem=function(e){return E(this,"BindingAddFromNamedItem",1,[e],0,0)},t.prototype.bindingAddFromPrompt=function(e){return E(this,"BindingAddFromPrompt",1,[e],0,0)},t.prototype.bindingAddFromSelection=function(e){return E(this,"BindingAddFromSelection",1,[e],0,0)},t.prototype.bindingAddRows=function(e){return E(this,"BindingAddRows",0,[e],0,0)},t.prototype.bindingClearFormats=function(e){return E(this,"BindingClearFormats",0,[e],0,0)},t.prototype.bindingDeleteAllDataValues=function(e){return E(this,"BindingDeleteAllDataValues",0,[e],0,0)},t.prototype.bindingGetAll=function(){return E(this,"BindingGetAll",1,[],4,0)},t.prototype.bindingGetById=function(e){return E(this,"BindingGetById",1,[e],4,0)},t.prototype.bindingGetData=function(e){return E(this,"BindingGetData",1,[e],4,0)},t.prototype.bindingReleaseById=function(e){return E(this,"BindingReleaseById",1,[e],0,0)},t.prototype.bindingSetData=function(e){return E(this,"BindingSetData",0,[e],0,0)},t.prototype.bindingSetFormats=function(e){return E(this,"BindingSetFormats",0,[e],0,0)},t.prototype.bindingSetTableOptions=function(e){return E(this,"BindingSetTableOptions",0,[e],0,0)},t.prototype.getFilePropertiesAsync=function(){return w("_V1Api.getFilePropertiesAsync",v,"1.6",O),E(this,"GetFilePropertiesAsync",1,[],4,0)},t.prototype.getSelectedData=function(e){return E(this,"GetSelectedData",1,[e],4,0)},t.prototype.gotoById=function(e){return E(this,"GotoById",1,[e],4,0)},t.prototype.setSelectedData=function(e){return E(this,"SetSelectedData",0,[e],0,0)},t.prototype._handleResult=function(t){(e.prototype._handleResult.call(this,t),x(t))||U(this,t)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{},{})},t}(OfficeExtension.ClientObject);e._V1Api=nr;var rr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PivotTableScopedCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"PivotTableScopedCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getFirst=function(){return C(e.PivotTable,this,"GetFirst",1,[],!1,!0,null,4)},n.prototype.getFirstOrNullObject=function(){return w("PivotTableScopedCollection.getFirstOrNullObject",v,"1.15",O),C(e.PivotTable,this,"GetFirstOrNullObject",1,[],!1,!1,null,4)},n.prototype.getItem=function(t){return P(e.PivotTable,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.PivotTable,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.PivotTable,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.PivotTable,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.PivotTable,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.PivotTableScopedCollection=rr;var ir=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PivotTableCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"PivotTableCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n,r){return w("PivotTableCollection.add",v,"1.8",O),C(e.PivotTable,this,"Add",0,[t,n,r],!1,!0,null,V(2,"ExcelApiUndo","1.1"))},n.prototype.getCount=function(){return w("PivotTableCollection.getCount",v,"1.4",O),E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.PivotTable,this,[t])},n.prototype.getItemOrNullObject=function(t){return w("PivotTableCollection.getItemOrNullObject",v,"1.4",O),C(e.PivotTable,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype.refreshAll=function(){E(this,"RefreshAll",0,[],V(2,"ExcelApiUndo","1.1"),0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.PivotTable,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.PivotTable,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.PivotTable,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.PivotTableCollection=ir;var or=function(){function e(){}return Object.defineProperty(e.prototype,"_ParentObject",{get:function(){return this.m__ParentObject},set:function(e){this.m__ParentObject=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"_eventTargetId",{get:function(){return this._ParentObject?this._ParentObject.id:OfficeExtension.Constants.eventWorkbookId},enumerable:!0,configurable:!0}),e}();e.PivotTableCollectionCustom=or,OfficeExtension.Utility.applyMixin(ir,or);var sr="PivotTable",ar=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PivotTable"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["name","id","useCustomSortLists","enableDataValueEditing","refreshOnOpen","allowMultipleFiltersPerField"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name","Id","UseCustomSortLists","EnableDataValueEditing","RefreshOnOpen","AllowMultipleFiltersPerField"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!1,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["worksheet","hierarchies","rowHierarchies","columnHierarchies","dataHierarchies","filterHierarchies","layout"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"columnHierarchies",{get:function(){return w("PivotTable.columnHierarchies",v,"1.8",O),this._C||(this._C=S(e.RowColumnPivotHierarchyCollection,this,"ColumnHierarchies",!0,4)),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"dataHierarchies",{get:function(){return w("PivotTable.dataHierarchies",v,"1.8",O),this._D||(this._D=S(e.DataPivotHierarchyCollection,this,"DataHierarchies",!0,4)),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"filterHierarchies",{get:function(){return w("PivotTable.filterHierarchies",v,"1.8",O),this._F||(this._F=S(e.FilterPivotHierarchyCollection,this,"FilterHierarchies",!0,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"hierarchies",{get:function(){return w("PivotTable.hierarchies",v,"1.8",O),this._H||(this._H=S(e.PivotHierarchyCollection,this,"Hierarchies",!0,4)),this._H},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"layout",{get:function(){return w("PivotTable.layout",v,"1.8",O),this._L||(this._L=S(e.PivotLayout,this,"Layout",!1,4)),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rowHierarchies",{get:function(){return w("PivotTable.rowHierarchies",v,"1.8",O),this._Ro||(this._Ro=S(e.RowColumnPivotHierarchyCollection,this,"RowHierarchies",!0,4)),this._Ro},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"worksheet",{get:function(){return this._W||(this._W=S(e.Worksheet,this,"Worksheet",!1,4)),this._W},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"allowMultipleFiltersPerField",{get:function(){return T("allowMultipleFiltersPerField",this._A,sr,this._isNull),w("PivotTable.allowMultipleFiltersPerField",v,"1.12",O),this._A},set:function(e){this._A=e,I(this,"AllowMultipleFiltersPerField",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"enableDataValueEditing",{get:function(){return T("enableDataValueEditing",this._E,sr,this._isNull),w("PivotTable.enableDataValueEditing",v,"1.9",O),this._E},set:function(e){this._E=e,I(this,"EnableDataValueEditing",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,sr,this._isNull),w("PivotTable.id",v,"1.5",O),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,sr,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"refreshOnOpen",{get:function(){return T("refreshOnOpen",this._R,sr,this._isNull),w("PivotTable.refreshOnOpen",v,"1.13",O),this._R},set:function(e){this._R=e,I(this,"RefreshOnOpen",e,V(2,"ExcelApiUndo","1.2"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"useCustomSortLists",{get:function(){return T("useCustomSortLists",this._U,sr,this._isNull),w("PivotTable.useCustomSortLists",v,"1.9",O),this._U},set:function(e){this._U=e,I(this,"UseCustomSortLists",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name","useCustomSortLists","enableDataValueEditing"],[],["allowMultipleFiltersPerField","columnHierarchies","dataHierarchies","filterHierarchies","hierarchies","layout","refreshOnOpen","rowHierarchies","worksheet"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){w("PivotTable.delete",v,"1.8",O),E(this,"Delete",0,[],V(2,"ExcelApiUndo","1.1"),0)},n.prototype.getDataSourceString=function(){return w("PivotTable.getDataSourceString",v,"1.15",O),E(this,"GetDataSourceString",1,[],4,0)},n.prototype.getDataSourceType=function(){return w("PivotTable.getDataSourceType",v,"1.15",O),E(this,"GetDataSourceType",1,[],4,0)},n.prototype.refresh=function(){E(this,"Refresh",0,[],V(2,"ExcelApiUndo","1.1"),0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AllowMultipleFiltersPerField)||(this._A=n.AllowMultipleFiltersPerField),N(n.EnableDataValueEditing)||(this._E=n.EnableDataValueEditing),N(n.Id)||(this._I=n.Id),N(n.Name)||(this._N=n.Name),N(n.RefreshOnOpen)||(this._R=n.RefreshOnOpen),N(n.UseCustomSortLists)||(this._U=n.UseCustomSortLists),L(this,n,["columnHierarchies","ColumnHierarchies","dataHierarchies","DataHierarchies","filterHierarchies","FilterHierarchies","hierarchies","Hierarchies","layout","Layout","rowHierarchies","RowHierarchies","worksheet","Worksheet"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{enableDataValueEditing:this._E,id:this._I,name:this._N,useCustomSortLists:this._U},{columnHierarchies:this._C,dataHierarchies:this._D,filterHierarchies:this._F,hierarchies:this._H,rowHierarchies:this._Ro})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.PivotTable=ar;var lr="PivotLayout",ur=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PivotLayout"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["showColumnGrandTotals","showRowGrandTotals","enableFieldList","subtotalLocation","layoutType","autoFormat","preserveFormatting","altTextDescription","altTextTitle","emptyCellText","fillEmptyCells","showFieldHeaders"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["ShowColumnGrandTotals","ShowRowGrandTotals","EnableFieldList","SubtotalLocation","LayoutType","AutoFormat","PreserveFormatting","AltTextDescription","AltTextTitle","EmptyCellText","FillEmptyCells","ShowFieldHeaders"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"altTextDescription",{get:function(){return T("altTextDescription",this._A,lr,this._isNull),w("PivotLayout.altTextDescription",v,"1.13",O),this._A},set:function(e){this._A=e,I(this,"AltTextDescription",e,V(2,"ExcelApiUndo","1.2"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"altTextTitle",{get:function(){return T("altTextTitle",this._Al,lr,this._isNull),w("PivotLayout.altTextTitle",v,"1.13",O),this._Al},set:function(e){this._Al=e,I(this,"AltTextTitle",e,V(2,"ExcelApiUndo","1.2"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"autoFormat",{get:function(){return T("autoFormat",this._Au,lr,this._isNull),w("PivotLayout.autoFormat",v,"1.9",O),this._Au},set:function(e){this._Au=e,I(this,"AutoFormat",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"emptyCellText",{get:function(){return T("emptyCellText",this._E,lr,this._isNull),w("PivotLayout.emptyCellText",v,"1.13",O),this._E},set:function(e){this._E=e,I(this,"EmptyCellText",e,V(2,"ExcelApiUndo","1.2"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"enableFieldList",{get:function(){return T("enableFieldList",this._En,lr,this._isNull),w("PivotLayout.enableFieldList",v,"1.10",O),this._En},set:function(e){this._En=e,I(this,"EnableFieldList",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fillEmptyCells",{get:function(){return T("fillEmptyCells",this._F,lr,this._isNull),w("PivotLayout.fillEmptyCells",v,"1.13",O),this._F},set:function(e){this._F=e,I(this,"FillEmptyCells",e,V(2,"ExcelApiUndo","1.2"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"layoutType",{get:function(){return T("layoutType",this._L,lr,this._isNull),this._L},set:function(e){this._L=e,I(this,"LayoutType",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"preserveFormatting",{get:function(){return T("preserveFormatting",this._P,lr,this._isNull),w("PivotLayout.preserveFormatting",v,"1.9",O),this._P},set:function(e){this._P=e,I(this,"PreserveFormatting",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showColumnGrandTotals",{get:function(){return T("showColumnGrandTotals",this._S,lr,this._isNull),this._S},set:function(e){this._S=e,I(this,"ShowColumnGrandTotals",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showFieldHeaders",{get:function(){return T("showFieldHeaders",this._Sh,lr,this._isNull),w("PivotLayout.showFieldHeaders",v,"1.13",O),this._Sh},set:function(e){this._Sh=e,I(this,"ShowFieldHeaders",e,V(2,"ExcelApiUndo","1.2"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showRowGrandTotals",{get:function(){return T("showRowGrandTotals",this._Sho,lr,this._isNull),this._Sho},set:function(e){this._Sho=e,I(this,"ShowRowGrandTotals",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"subtotalLocation",{get:function(){return T("subtotalLocation",this._Su,lr,this._isNull),this._Su},set:function(e){this._Su=e,I(this,"SubtotalLocation",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["showColumnGrandTotals","showRowGrandTotals","enableFieldList","subtotalLocation","layoutType","autoFormat","preserveFormatting"],[],["altTextDescription","altTextTitle","emptyCellText","fillEmptyCells","showFieldHeaders"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.displayBlankLineAfterEachItem=function(e){w("PivotLayout.displayBlankLineAfterEachItem",v,"1.13",O),E(this,"DisplayBlankLineAfterEachItem",0,[e],V(2,"ExcelApiUndo","1.2"),0)},n.prototype.getColumnLabelRange=function(){var t=b.PivotLayout_GetColumnLabelRange(this),n=t.handled,r=t.result;return n?r:C(e.Range,this,"GetColumnLabelRange",1,[],!1,!1,null,0)},n.prototype.getDataBodyRange=function(){var t=b.PivotLayout_GetDataBodyRange(this),n=t.handled,r=t.result;return n?r:C(e.Range,this,"GetDataBodyRange",1,[],!1,!1,null,0)},n.prototype.getDataHierarchy=function(t){var n=b.PivotLayout_GetDataHierarchy(this,t),r=n.handled,i=n.result;return r?i:(w("PivotLayout.getDataHierarchy",v,"1.9",O),C(e.DataPivotHierarchy,this,"GetDataHierarchy",1,[t],!1,!1,null,0))},n.prototype.getFilterAxisRange=function(){var t=b.PivotLayout_GetFilterAxisRange(this),n=t.handled,r=t.result;return n?r:C(e.Range,this,"GetFilterAxisRange",1,[],!1,!1,null,0)},n.prototype.getPivotItems=function(t,n){return w("PivotLayout.getPivotItems",v,"1.9",O),C(e.PivotItemCollection,this,"GetPivotItems",0,[t,n],!0,!1,null,0)},n.prototype.getRange=function(){var t=b.PivotLayout_GetRange(this),n=t.handled,r=t.result;return n?r:C(e.Range,this,"GetRange",1,[],!1,!1,null,0)},n.prototype.getRowLabelRange=function(){var t=b.PivotLayout_GetRowLabelRange(this),n=t.handled,r=t.result;return n?r:C(e.Range,this,"GetRowLabelRange",1,[],!1,!1,null,0)},n.prototype.repeatAllItemLabels=function(e){w("PivotLayout.repeatAllItemLabels",v,"1.13",O),E(this,"RepeatAllItemLabels",0,[e],V(2,"ExcelApiUndo","1.2"),0)},n.prototype.setAutoSortOnCell=function(e,t){w("PivotLayout.setAutoSortOnCell",v,"1.9",O),E(this,"SetAutoSortOnCell",0,[e,t],V(2,"ExcelApiUndo","1.1"),0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AltTextDescription)||(this._A=n.AltTextDescription),N(n.AltTextTitle)||(this._Al=n.AltTextTitle),N(n.AutoFormat)||(this._Au=n.AutoFormat),N(n.EmptyCellText)||(this._E=n.EmptyCellText),N(n.EnableFieldList)||(this._En=n.EnableFieldList),N(n.FillEmptyCells)||(this._F=n.FillEmptyCells),N(n.LayoutType)||(this._L=n.LayoutType),N(n.PreserveFormatting)||(this._P=n.PreserveFormatting),N(n.ShowColumnGrandTotals)||(this._S=n.ShowColumnGrandTotals),N(n.ShowFieldHeaders)||(this._Sh=n.ShowFieldHeaders),N(n.ShowRowGrandTotals)||(this._Sho=n.ShowRowGrandTotals),N(n.SubtotalLocation)||(this._Su=n.SubtotalLocation)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{autoFormat:this._Au,enableFieldList:this._En,layoutType:this._L,preserveFormatting:this._P,showColumnGrandTotals:this._S,showRowGrandTotals:this._Sho,subtotalLocation:this._Su},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.PivotLayout=ur,function(t){t.PivotLayout_GetColumnLabelRange=function(t){return OfficeExtension.Utility.isSetSupported("Pivot","1.5")?{handled:!1,result:void 0}:{handled:!0,result:C(e.Range,t,"GetColumnLabelRange",0,[],!1,!1,null,0)}},t.PivotLayout_GetDataBodyRange=function(t){return OfficeExtension.Utility.isSetSupported("Pivot","1.5")?{handled:!1,result:void 0}:{handled:!0,result:C(e.Range,t,"GetDataBodyRange",0,[],!1,!1,null,0)}},t.PivotLayout_GetDataHierarchy=function(t,n){return OfficeExtension.Utility.isSetSupported("Pivot","1.5")?{handled:!1,result:void 0}:(w("PivotLayout.getDataHierarchy",v,"1.9",O),{handled:!0,result:C(e.DataPivotHierarchy,t,"GetDataHierarchy",0,[n],!1,!1,null,0)})},t.PivotLayout_GetFilterAxisRange=function(t){return OfficeExtension.Utility.isSetSupported("Pivot","1.5")?{handled:!1,result:void 0}:{handled:!0,result:C(e.Range,t,"GetFilterAxisRange",0,[],!1,!1,null,0)}},t.PivotLayout_GetRange=function(t){return OfficeExtension.Utility.isSetSupported("Pivot","1.5")?{handled:!1,result:void 0}:{handled:!0,result:C(e.Range,t,"GetRange",0,[],!1,!1,null,0)}},t.PivotLayout_GetRowLabelRange=function(t){return OfficeExtension.Utility.isSetSupported("Pivot","1.5")?{handled:!1,result:void 0}:{handled:!0,result:C(e.Range,t,"GetRowLabelRange",0,[],!1,!1,null,0)}}}(b=e._CC||(e._CC={}));var cr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PivotHierarchyCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"PivotHierarchyCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.PivotHierarchy,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.PivotHierarchy,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.PivotHierarchy,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.PivotHierarchy,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.PivotHierarchy,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.PivotHierarchyCollection=cr;var pr="PivotHierarchy",fr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PivotHierarchy"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","name"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Name"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["fields"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fields",{get:function(){return this._F||(this._F=S(e.PivotFieldCollection,this,"Fields",!0,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,pr,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,pr,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name"],[],["fields"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Id)||(this._I=n.Id),N(n.Name)||(this._N=n.Name),L(this,n,["fields","Fields"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{id:this._I,name:this._N},{fields:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.PivotHierarchy=fr;var hr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"RowColumnPivotHierarchyCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"RowColumnPivotHierarchyCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t){return C(e.RowColumnPivotHierarchy,this,"Add",0,[t],!1,!0,null,V(2,"ExcelApiUndo","1.1"))},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.RowColumnPivotHierarchy,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.RowColumnPivotHierarchy,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype.remove=function(e){E(this,"Remove",0,[e],V(2,"ExcelApiUndo","1.1"),0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.RowColumnPivotHierarchy,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.RowColumnPivotHierarchy,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.RowColumnPivotHierarchy,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.RowColumnPivotHierarchyCollection=hr;var dr="RowColumnPivotHierarchy",yr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"RowColumnPivotHierarchy"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","name","position"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Name","Position"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["fields"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fields",{get:function(){return this._F||(this._F=S(e.PivotFieldCollection,this,"Fields",!0,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,dr,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,dr,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"position",{get:function(){return T("position",this._P,dr,this._isNull),this._P},set:function(e){this._P=e,I(this,"Position",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name","position"],[],["fields"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.setToDefault=function(){E(this,"SetToDefault",0,[],V(2,"ExcelApiUndo","1.1"),0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Id)||(this._I=n.Id),N(n.Name)||(this._N=n.Name),N(n.Position)||(this._P=n.Position),L(this,n,["fields","Fields"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{id:this._I,name:this._N,position:this._P},{fields:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.RowColumnPivotHierarchy=yr;var mr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"FilterPivotHierarchyCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"FilterPivotHierarchyCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t){return C(e.FilterPivotHierarchy,this,"Add",0,[t],!1,!0,null,V(2,"ExcelApiUndo","1.1"))},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.FilterPivotHierarchy,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.FilterPivotHierarchy,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype.remove=function(e){E(this,"Remove",0,[e],V(2,"ExcelApiUndo","1.1"),0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.FilterPivotHierarchy,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.FilterPivotHierarchy,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.FilterPivotHierarchy,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.FilterPivotHierarchyCollection=mr;var gr="FilterPivotHierarchy",_r=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"FilterPivotHierarchy"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","name","position","enableMultipleFilterItems"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Name","Position","EnableMultipleFilterItems"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["fields"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fields",{get:function(){return this._F||(this._F=S(e.PivotFieldCollection,this,"Fields",!0,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"enableMultipleFilterItems",{get:function(){return T("enableMultipleFilterItems",this._E,gr,this._isNull),this._E},set:function(e){this._E=e,I(this,"EnableMultipleFilterItems",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,gr,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,gr,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"position",{get:function(){return T("position",this._P,gr,this._isNull),this._P},set:function(e){this._P=e,I(this,"Position",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name","position","enableMultipleFilterItems"],[],["fields"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.setToDefault=function(){E(this,"SetToDefault",0,[],V(2,"ExcelApiUndo","1.1"),0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.EnableMultipleFilterItems)||(this._E=n.EnableMultipleFilterItems),N(n.Id)||(this._I=n.Id),N(n.Name)||(this._N=n.Name),N(n.Position)||(this._P=n.Position),L(this,n,["fields","Fields"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{enableMultipleFilterItems:this._E,id:this._I,name:this._N,position:this._P},{fields:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.FilterPivotHierarchy=_r;var br=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"DataPivotHierarchyCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"DataPivotHierarchyCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t){return C(e.DataPivotHierarchy,this,"Add",0,[t],!1,!0,null,V(2,"ExcelApiUndo","1.1"))},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.DataPivotHierarchy,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.DataPivotHierarchy,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype.remove=function(e){E(this,"Remove",0,[e],V(2,"ExcelApiUndo","1.1"),0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.DataPivotHierarchy,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.DataPivotHierarchy,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.DataPivotHierarchy,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.DataPivotHierarchyCollection=br;var Or="DataPivotHierarchy",vr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"DataPivotHierarchy"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","name","position","numberFormat","summarizeBy","showAs"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Name","Position","NumberFormat","SummarizeBy","ShowAs"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["field"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"field",{get:function(){return this._F||(this._F=S(e.PivotField,this,"Field",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,Or,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,Or,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormat",{get:function(){return T("numberFormat",this._Nu,Or,this._isNull),this._Nu},set:function(e){this._Nu=e,I(this,"NumberFormat",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"position",{get:function(){return T("position",this._P,Or,this._isNull),this._P},set:function(e){this._P=e,I(this,"Position",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showAs",{get:function(){return T("showAs",this._S,Or,this._isNull),this._S},set:function(e){this._S=e,I(this,"ShowAs",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"summarizeBy",{get:function(){return T("summarizeBy",this._Su,Or,this._isNull),this._Su},set:function(e){this._Su=e,I(this,"SummarizeBy",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name","position","numberFormat","summarizeBy","showAs"],["field"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.setToDefault=function(){E(this,"SetToDefault",0,[],V(2,"ExcelApiUndo","1.1"),0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Id)||(this._I=n.Id),N(n.Name)||(this._N=n.Name),N(n.NumberFormat)||(this._Nu=n.NumberFormat),N(n.Position)||(this._P=n.Position),N(n.ShowAs)||(this._S=n.ShowAs),N(n.SummarizeBy)||(this._Su=n.SummarizeBy),L(this,n,["field","Field"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{id:this._I,name:this._N,numberFormat:this._Nu,position:this._P,showAs:this._S,summarizeBy:this._Su},{field:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.DataPivotHierarchy=vr;var Sr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PivotFieldCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"PivotFieldCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.PivotField,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.PivotField,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.PivotField,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.PivotField,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.PivotField,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.PivotFieldCollection=Sr;var Cr="PivotField",Pr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PivotField"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","name","subtotals","showAllItems"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Name","Subtotals","ShowAllItems"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["items"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return this._It||(this._It=S(e.PivotItemCollection,this,"Items",!0,4)),this._It},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,Cr,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,Cr,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showAllItems",{get:function(){return T("showAllItems",this._S,Cr,this._isNull),this._S},set:function(e){this._S=e,I(this,"ShowAllItems",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"subtotals",{get:function(){return T("subtotals",this._Su,Cr,this._isNull),this._Su},set:function(e){this._Su=e,I(this,"Subtotals",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name","subtotals","showAllItems"],[],["items"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.applyFilter=function(e){w("PivotField.applyFilter",v,"1.12",O),E(this,"ApplyFilter",0,[e],V(2,"ExcelApiUndo","1.1"),0)},n.prototype.clearAllFilters=function(){w("PivotField.clearAllFilters",v,"1.12",O),E(this,"ClearAllFilters",0,[],V(2,"ExcelApiUndo","1.1"),0)},n.prototype.clearFilter=function(e){w("PivotField.clearFilter",v,"1.12",O),E(this,"ClearFilter",0,[e],V(2,"ExcelApiUndo","1.1"),0)},n.prototype.getFilters=function(){return w("PivotField.getFilters",v,"1.12",O),E(this,"GetFilters",0,[],0,0)},n.prototype.isFiltered=function(e){return w("PivotField.isFiltered",v,"1.12",O),E(this,"IsFiltered",0,[e],0,0)},n.prototype.sortByLabels=function(e){b.PivotField_SortByLabels(this,e).handled||E(this,"SortByLabels",0,[e],V(2,"ExcelApiUndo","1.1"),0)},n.prototype.sortByValues=function(e,t,n){w("PivotField.sortByValues",v,"1.9",O),E(this,"SortByValues",0,[e,t,n],V(2,"ExcelApiUndo","1.1"),0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Id)||(this._I=n.Id),N(n.Name)||(this._N=n.Name),N(n.ShowAllItems)||(this._S=n.ShowAllItems),N(n.Subtotals)||(this._Su=n.Subtotals),L(this,n,["items","Items"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{id:this._I,name:this._N,showAllItems:this._S,subtotals:this._Su},{items:this._It})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.PivotField=Pr,function(e){e.PivotField_SortByLabels=function(e,t){return"string"==typeof t&&(t="ascending"===t.toLowerCase()),E(e,"SortByLabels",0,[t],0,0),{handled:!0}}}(b=e._CC||(e._CC={}));var Ar=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PivotItemCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"PivotItemCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.PivotItem,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.PivotItem,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.PivotItem,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.PivotItem,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.PivotItem,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.PivotItemCollection=Ar;var Fr="PivotItem",Dr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"PivotItem"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["id","name","isExpanded","visible"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Name","IsExpanded","Visible"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"id",{get:function(){return T("id",this._I,Fr,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isExpanded",{get:function(){return T("isExpanded",this._Is,Fr,this._isNull),this._Is},set:function(e){this._Is=e,I(this,"IsExpanded",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"name",{get:function(){return T("name",this._N,Fr,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"visible",{get:function(){return T("visible",this._V,Fr,this._isNull),this._V},set:function(e){this._V=e,I(this,"Visible",e,V(2,"ExcelApiUndo","1.1"))},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["name","isExpanded","visible"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Id)||(this._I=n.Id),N(n.IsExpanded)||(this._Is=n.IsExpanded),N(n.Name)||(this._N=n.Name),N(n.Visible)||(this._V=n.Visible)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleIdResult=function(t){e.prototype._handleIdResult.call(this,t),x(t)||N(t.Id)||(this._I=t.Id)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{id:this._I,isExpanded:this._Is,name:this._N,visible:this._V},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.PivotItem=Dr,function(e){e.invalid="Invalid",e.topItems="TopItems",e.topPercent="TopPercent",e.topSum="TopSum",e.bottomItems="BottomItems",e.bottomPercent="BottomPercent",e.bottomSum="BottomSum"}(e.PivotFilterTopBottomCriterion||(e.PivotFilterTopBottomCriterion={})),function(e){e.ascending="Ascending",e.descending="Descending"}(e.SortBy||(e.SortBy={})),function(e){e.unknown="Unknown",e.automatic="Automatic",e.sum="Sum",e.count="Count",e.average="Average",e.max="Max",e.min="Min",e.product="Product",e.countNumbers="CountNumbers",e.standardDeviation="StandardDeviation",e.standardDeviationP="StandardDeviationP",e.variance="Variance",e.varianceP="VarianceP"}(e.AggregationFunction||(e.AggregationFunction={})),function(e){e.unknown="Unknown",e.none="None",e.percentOfGrandTotal="PercentOfGrandTotal",e.percentOfRowTotal="PercentOfRowTotal",e.percentOfColumnTotal="PercentOfColumnTotal",e.percentOfParentRowTotal="PercentOfParentRowTotal",e.percentOfParentColumnTotal="PercentOfParentColumnTotal",e.percentOfParentTotal="PercentOfParentTotal",e.percentOf="PercentOf",e.runningTotal="RunningTotal",e.percentRunningTotal="PercentRunningTotal",e.differenceFrom="DifferenceFrom",e.percentDifferenceFrom="PercentDifferenceFrom",e.rankAscending="RankAscending",e.rankDecending="RankDecending",e.index="Index"}(e.ShowAsCalculation||(e.ShowAsCalculation={})),function(e){e.unknown="Unknown",e.row="Row",e.column="Column",e.data="Data",e.filter="Filter"}(e.PivotAxis||(e.PivotAxis={}));var Er="WorksheetCustomProperty",Rr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"WorksheetCustomProperty"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["key","value","_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Key","Value","_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"key",{get:function(){return T("key",this._K,Er,this._isNull),this._K},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"value",{get:function(){return T("value",this._V,Er,this._isNull),this._V},set:function(e){this._V=e,I(this,"Value",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_Id",{get:function(){return T("_Id",this.__I,Er,this._isNull),this.__I},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["value"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype.delete=function(){E(this,"Delete",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Key)||(this._K=n.Key),N(n.Value)||(this._V=n.Value),N(n._Id)||(this.__I=n._Id)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleIdResult=function(t){e.prototype._handleIdResult.call(this,t),x(t)||N(t._Id)||(this.__I=t._Id)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{key:this._K,value:this._V},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.WorksheetCustomProperty=Rr;var Ir=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"WorksheetCustomPropertyCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"WorksheetCustomPropertyCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n){return C(e.WorksheetCustomProperty,this,"Add",0,[t,n],!1,!0,null,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.WorksheetCustomProperty,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.WorksheetCustomProperty,this,"GetItemOrNullObject",0,[t],!1,!1,null,0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.WorksheetCustomProperty,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.WorksheetCustomProperty,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.WorksheetCustomProperty,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.WorksheetCustomPropertyCollection=Ir;var xr="DocumentProperties",Nr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"DocumentProperties"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["title","subject","author","keywords","comments","lastAuthor","revisionNumber","creationDate","category","manager","company"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Title","Subject","Author","Keywords","Comments","LastAuthor","RevisionNumber","CreationDate","Category","Manager","Company"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!1,!0,!1,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["custom"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"custom",{get:function(){return this._Cu||(this._Cu=S(e.CustomPropertyCollection,this,"Custom",!0,4)),this._Cu},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"author",{get:function(){return T("author",this._A,xr,this._isNull),this._A},set:function(e){this._A=e,I(this,"Author",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"category",{get:function(){return T("category",this._C,xr,this._isNull),this._C},set:function(e){this._C=e,I(this,"Category",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"comments",{get:function(){return T("comments",this._Co,xr,this._isNull),this._Co},set:function(e){this._Co=e,I(this,"Comments",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"company",{get:function(){return T("company",this._Com,xr,this._isNull),this._Com},set:function(e){this._Com=e,I(this,"Company",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"creationDate",{get:function(){return T("creationDate",this._Cr,xr,this._isNull),this._Cr},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"keywords",{get:function(){return T("keywords",this._K,xr,this._isNull),this._K},set:function(e){this._K=e,I(this,"Keywords",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"lastAuthor",{get:function(){return T("lastAuthor",this._L,xr,this._isNull),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"manager",{get:function(){return T("manager",this._M,xr,this._isNull),this._M},set:function(e){this._M=e,I(this,"Manager",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"revisionNumber",{get:function(){return T("revisionNumber",this._R,xr,this._isNull),this._R},set:function(e){this._R=e,I(this,"RevisionNumber",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"subject",{get:function(){return T("subject",this._S,xr,this._isNull),this._S},set:function(e){this._S=e,I(this,"Subject",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"title",{get:function(){return T("title",this._T,xr,this._isNull),this._T},set:function(e){this._T=e,I(this,"Title",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["title","subject","author","keywords","comments","revisionNumber","category","manager","company"],[],["custom"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Author)||(this._A=n.Author),N(n.Category)||(this._C=n.Category),N(n.Comments)||(this._Co=n.Comments),N(n.Company)||(this._Com=n.Company),N(n.CreationDate)||(this._Cr=B(n.CreationDate)),N(n.Keywords)||(this._K=n.Keywords),N(n.LastAuthor)||(this._L=n.LastAuthor),N(n.Manager)||(this._M=n.Manager),N(n.RevisionNumber)||(this._R=n.RevisionNumber),N(n.Subject)||(this._S=n.Subject),N(n.Title)||(this._T=n.Title),L(this,n,["custom","Custom"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){if(t.prototype._handleRetrieveResult.call(this,e,n),!x(e)){var r=e;N(r.CreationDate)||(r.creationDate=B(r.creationDate)),H(this,e,n)}},n.prototype.toJSON=function(){return k(this,{author:this._A,category:this._C,comments:this._Co,company:this._Com,creationDate:this._Cr,keywords:this._K,lastAuthor:this._L,manager:this._M,revisionNumber:this._R,subject:this._S,title:this._T},{custom:this._Cu})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.DocumentProperties=Nr;var Tr="CustomProperty",wr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"CustomProperty"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["key","value","type"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Key","Value","Type"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"key",{get:function(){return T("key",this._K,Tr,this._isNull),this._K},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"type",{get:function(){return T("type",this._T,Tr,this._isNull),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"value",{get:function(){return T("value",this._V,Tr,this._isNull),this._V},set:function(e){this._V=e,I(this,"Value",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["value"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype.delete=function(){E(this,"Delete",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Key)||(this._K=n.Key),N(n.Type)||(this._T=n.Type),N(n.Value)||(this._V=n.Value)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{key:this._K,type:this._T,value:this._V},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.CustomProperty=wr;var jr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"CustomPropertyCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"CustomPropertyCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n){return C(e.CustomProperty,this,"Add",0,[t,n],!1,!0,null,0)},n.prototype.deleteAll=function(){E(this,"DeleteAll",0,[],0,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.CustomProperty,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.CustomProperty,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.CustomProperty,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.CustomProperty,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.CustomProperty,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.CustomPropertyCollection=jr;var Mr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ConditionalFormatCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"ConditionalFormatCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t){return C(e.ConditionalFormat,this,"Add",0,[t],!1,!0,null,0)},n.prototype.clearAll=function(){E(this,"ClearAll",0,[],0,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.ConditionalFormat,this,[t])},n.prototype.getItemAt=function(t){return C(e.ConditionalFormat,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return w("ConditionalFormatCollection.getItemOrNullObject",v,"1.14",O),C(e.ConditionalFormat,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.ConditionalFormat,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.ConditionalFormat,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.ConditionalFormat,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.ConditionalFormatCollection=Mr;var kr="ConditionalFormat",Ur=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ConditionalFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["stopIfTrue","priority","type","id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["StopIfTrue","Priority","Type","Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["dataBarOrNullObject","dataBar","customOrNullObject","custom","iconSet","iconSetOrNullObject","colorScale","colorScaleOrNullObject","topBottom","topBottomOrNullObject","preset","presetOrNullObject","textComparison","textComparisonOrNullObject","cellValue","cellValueOrNullObject"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"cellValue",{get:function(){return this._C||(this._C=S(e.CellValueConditionalFormat,this,"CellValue",!1,4)),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"cellValueOrNullObject",{get:function(){return this._Ce||(this._Ce=S(e.CellValueConditionalFormat,this,"CellValueOrNullObject",!1,4)),this._Ce},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"colorScale",{get:function(){return this._Co||(this._Co=S(e.ColorScaleConditionalFormat,this,"ColorScale",!1,4)),this._Co},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"colorScaleOrNullObject",{get:function(){return this._Col||(this._Col=S(e.ColorScaleConditionalFormat,this,"ColorScaleOrNullObject",!1,4)),this._Col},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"custom",{get:function(){return this._Cu||(this._Cu=S(e.CustomConditionalFormat,this,"Custom",!1,4)),this._Cu},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"customOrNullObject",{get:function(){return this._Cus||(this._Cus=S(e.CustomConditionalFormat,this,"CustomOrNullObject",!1,4)),this._Cus},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"dataBar",{get:function(){return this._D||(this._D=S(e.DataBarConditionalFormat,this,"DataBar",!1,4)),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"dataBarOrNullObject",{get:function(){return this._Da||(this._Da=S(e.DataBarConditionalFormat,this,"DataBarOrNullObject",!1,4)),this._Da},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"iconSet",{get:function(){return this._I||(this._I=S(e.IconSetConditionalFormat,this,"IconSet",!1,4)),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"iconSetOrNullObject",{get:function(){return this._Ic||(this._Ic=S(e.IconSetConditionalFormat,this,"IconSetOrNullObject",!1,4)),this._Ic},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"preset",{get:function(){return this._P||(this._P=S(e.PresetCriteriaConditionalFormat,this,"Preset",!1,4)),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"presetOrNullObject",{get:function(){return this._Pr||(this._Pr=S(e.PresetCriteriaConditionalFormat,this,"PresetOrNullObject",!1,4)),this._Pr},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"textComparison",{get:function(){return this._T||(this._T=S(e.TextConditionalFormat,this,"TextComparison",!1,4)),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"textComparisonOrNullObject",{get:function(){return this._Te||(this._Te=S(e.TextConditionalFormat,this,"TextComparisonOrNullObject",!1,4)),this._Te},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"topBottom",{get:function(){return this._To||(this._To=S(e.TopBottomConditionalFormat,this,"TopBottom",!1,4)),this._To},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"topBottomOrNullObject",{get:function(){return this._Top||(this._Top=S(e.TopBottomConditionalFormat,this,"TopBottomOrNullObject",!1,4)),this._Top},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._Id0,kr,this._isNull),this._Id0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"priority",{get:function(){return T("priority",this._Pri,kr,this._isNull),this._Pri},set:function(e){this._Pri=e,I(this,"Priority",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"stopIfTrue",{get:function(){return T("stopIfTrue",this._S,kr,this._isNull),this._S},set:function(e){this._S=e,I(this,"StopIfTrue",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"type",{get:function(){return T("type",this._Ty,kr,this._isNull),this._Ty},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["stopIfTrue","priority"],["dataBarOrNullObject","dataBar","customOrNullObject","custom","iconSet","iconSetOrNullObject","colorScale","colorScaleOrNullObject","topBottom","topBottomOrNullObject","preset","presetOrNullObject","textComparison","textComparisonOrNullObject","cellValue","cellValueOrNullObject"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.getRange=function(){return C(e.Range,this,"GetRange",1,[],!1,!0,null,4)},n.prototype.getRangeOrNullObject=function(){return C(e.Range,this,"GetRangeOrNullObject",1,[],!1,!0,null,4)},n.prototype.getRanges=function(){return w("ConditionalFormat.getRanges",v,"1.9",O),C(e.RangeAreas,this,"GetRanges",1,[],!1,!0,null,4)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Id)||(this._Id0=n.Id),N(n.Priority)||(this._Pri=n.Priority),N(n.StopIfTrue)||(this._S=n.StopIfTrue),N(n.Type)||(this._Ty=n.Type),L(this,n,["cellValue","CellValue","cellValueOrNullObject","CellValueOrNullObject","colorScale","ColorScale","colorScaleOrNullObject","ColorScaleOrNullObject","custom","Custom","customOrNullObject","CustomOrNullObject","dataBar","DataBar","dataBarOrNullObject","DataBarOrNullObject","iconSet","IconSet","iconSetOrNullObject","IconSetOrNullObject","preset","Preset","presetOrNullObject","PresetOrNullObject","textComparison","TextComparison","textComparisonOrNullObject","TextComparisonOrNullObject","topBottom","TopBottom","topBottomOrNullObject","TopBottomOrNullObject"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._Id0=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{id:this._Id0,priority:this._Pri,stopIfTrue:this._S,type:this._Ty},{cellValue:this._C,cellValueOrNullObject:this._Ce,colorScale:this._Co,colorScaleOrNullObject:this._Col,custom:this._Cu,customOrNullObject:this._Cus,dataBar:this._D,dataBarOrNullObject:this._Da,iconSet:this._I,iconSetOrNullObject:this._Ic,preset:this._P,presetOrNullObject:this._Pr,textComparison:this._T,textComparisonOrNullObject:this._Te,topBottom:this._To,topBottomOrNullObject:this._Top})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ConditionalFormat=Ur;var Lr="DataBarConditionalFormat",Br=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"DataBarConditionalFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["showDataBarOnly","barDirection","axisFormat","axisColor","lowerBoundRule","upperBoundRule"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["ShowDataBarOnly","BarDirection","AxisFormat","AxisColor","LowerBoundRule","UpperBoundRule"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["positiveFormat","negativeFormat"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"negativeFormat",{get:function(){return this._N||(this._N=S(e.ConditionalDataBarNegativeFormat,this,"NegativeFormat",!1,4)),this._N},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"positiveFormat",{get:function(){return this._P||(this._P=S(e.ConditionalDataBarPositiveFormat,this,"PositiveFormat",!1,4)),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"axisColor",{get:function(){return T("axisColor",this._A,Lr,this._isNull),this._A},set:function(e){this._A=e,I(this,"AxisColor",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"axisFormat",{get:function(){return T("axisFormat",this._Ax,Lr,this._isNull),this._Ax},set:function(e){this._Ax=e,I(this,"AxisFormat",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"barDirection",{get:function(){return T("barDirection",this._B,Lr,this._isNull),this._B},set:function(e){this._B=e,I(this,"BarDirection",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"lowerBoundRule",{get:function(){return T("lowerBoundRule",this._L,Lr,this._isNull),this._L},set:function(e){this._L=e,I(this,"LowerBoundRule",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"showDataBarOnly",{get:function(){return T("showDataBarOnly",this._S,Lr,this._isNull),this._S},set:function(e){this._S=e,I(this,"ShowDataBarOnly",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"upperBoundRule",{get:function(){return T("upperBoundRule",this._U,Lr,this._isNull),this._U},set:function(e){this._U=e,I(this,"UpperBoundRule",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["showDataBarOnly","barDirection","axisFormat","axisColor","lowerBoundRule","upperBoundRule"],["positiveFormat","negativeFormat"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AxisColor)||(this._A=n.AxisColor),N(n.AxisFormat)||(this._Ax=n.AxisFormat),N(n.BarDirection)||(this._B=n.BarDirection),N(n.LowerBoundRule)||(this._L=n.LowerBoundRule),N(n.ShowDataBarOnly)||(this._S=n.ShowDataBarOnly),N(n.UpperBoundRule)||(this._U=n.UpperBoundRule),L(this,n,["negativeFormat","NegativeFormat","positiveFormat","PositiveFormat"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{axisColor:this._A,axisFormat:this._Ax,barDirection:this._B,lowerBoundRule:this._L,showDataBarOnly:this._S,upperBoundRule:this._U},{negativeFormat:this._N,positiveFormat:this._P})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.DataBarConditionalFormat=Br;var Hr="ConditionalDataBarPositiveFormat",Wr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ConditionalDataBarPositiveFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["fillColor","gradientFill","borderColor"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["FillColor","GradientFill","BorderColor"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"borderColor",{get:function(){return T("borderColor",this._B,Hr,this._isNull),this._B},set:function(e){this._B=e,I(this,"BorderColor",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fillColor",{get:function(){return T("fillColor",this._F,Hr,this._isNull),this._F},set:function(e){this._F=e,I(this,"FillColor",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"gradientFill",{get:function(){return T("gradientFill",this._G,Hr,this._isNull),this._G},set:function(e){this._G=e,I(this,"GradientFill",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["fillColor","gradientFill","borderColor"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.BorderColor)||(this._B=n.BorderColor),N(n.FillColor)||(this._F=n.FillColor),N(n.GradientFill)||(this._G=n.GradientFill)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{borderColor:this._B,fillColor:this._F,gradientFill:this._G},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ConditionalDataBarPositiveFormat=Wr;var Vr="ConditionalDataBarNegativeFormat",Gr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ConditionalDataBarNegativeFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["fillColor","matchPositiveFillColor","borderColor","matchPositiveBorderColor"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["FillColor","MatchPositiveFillColor","BorderColor","MatchPositiveBorderColor"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"borderColor",{get:function(){return T("borderColor",this._B,Vr,this._isNull),this._B},set:function(e){this._B=e,I(this,"BorderColor",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fillColor",{get:function(){return T("fillColor",this._F,Vr,this._isNull),this._F},set:function(e){this._F=e,I(this,"FillColor",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"matchPositiveBorderColor",{get:function(){return T("matchPositiveBorderColor",this._M,Vr,this._isNull),this._M},set:function(e){this._M=e,I(this,"MatchPositiveBorderColor",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"matchPositiveFillColor",{get:function(){return T("matchPositiveFillColor",this._Ma,Vr,this._isNull),this._Ma},set:function(e){this._Ma=e,I(this,"MatchPositiveFillColor",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["fillColor","matchPositiveFillColor","borderColor","matchPositiveBorderColor"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.BorderColor)||(this._B=n.BorderColor),N(n.FillColor)||(this._F=n.FillColor),N(n.MatchPositiveBorderColor)||(this._M=n.MatchPositiveBorderColor),N(n.MatchPositiveFillColor)||(this._Ma=n.MatchPositiveFillColor)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{borderColor:this._B,fillColor:this._F,matchPositiveBorderColor:this._M,matchPositiveFillColor:this._Ma},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ConditionalDataBarNegativeFormat=Gr;var zr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"CustomConditionalFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["rule","format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ConditionalRangeFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rule",{get:function(){return this._R||(this._R=S(e.ConditionalFormatRule,this,"Rule",!1,4)),this._R},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,[],["rule","format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),L(this,n,["format","Format","rule","Rule"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{format:this._F,rule:this._R})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.CustomConditionalFormat=zr;var qr="ConditionalFormatRule",Jr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ConditionalFormatRule"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["formula","formulaLocal","formulaR1C1"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Formula","FormulaLocal","FormulaR1C1"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"formula",{get:function(){return T("formula",this._F,qr,this._isNull),this._F},set:function(e){this._F=e,I(this,"Formula",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"formulaLocal",{get:function(){return T("formulaLocal",this._Fo,qr,this._isNull),this._Fo},set:function(e){this._Fo=e,I(this,"FormulaLocal",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"formulaR1C1",{get:function(){return T("formulaR1C1",this._For,qr,this._isNull),this._For},set:function(e){this._For=e,I(this,"FormulaR1C1",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["formula","formulaLocal","formulaR1C1"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Formula)||(this._F=n.Formula),N(n.FormulaLocal)||(this._Fo=n.FormulaLocal),N(n.FormulaR1C1)||(this._For=n.FormulaR1C1)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{formula:this._F,formulaLocal:this._Fo,formulaR1C1:this._For},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ConditionalFormatRule=Jr;var Kr="IconSetConditionalFormat",Qr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"IconSetConditionalFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["reverseIconOrder","showIconOnly","style","criteria"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["ReverseIconOrder","ShowIconOnly","Style","Criteria"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"criteria",{get:function(){return T("criteria",this._C,Kr,this._isNull),this._C},set:function(e){this._C=e,I(this,"Criteria",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"reverseIconOrder",{get:function(){return T("reverseIconOrder",this._R,Kr,this._isNull),this._R},set:function(e){this._R=e,I(this,"ReverseIconOrder",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"showIconOnly",{get:function(){return T("showIconOnly",this._S,Kr,this._isNull),this._S},set:function(e){this._S=e,I(this,"ShowIconOnly",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"style",{get:function(){return T("style",this._St,Kr,this._isNull),this._St},set:function(e){this._St=e,I(this,"Style",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["reverseIconOrder","showIconOnly","style","criteria"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Criteria)||(this._C=n.Criteria),N(n.ReverseIconOrder)||(this._R=n.ReverseIconOrder),N(n.ShowIconOnly)||(this._S=n.ShowIconOnly),N(n.Style)||(this._St=n.Style)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{criteria:this._C,reverseIconOrder:this._R,showIconOnly:this._S,style:this._St},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.IconSetConditionalFormat=Qr;var Xr="ColorScaleConditionalFormat",Yr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ColorScaleConditionalFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["threeColorScale","criteria"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["ThreeColorScale","Criteria"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"criteria",{get:function(){return T("criteria",this._C,Xr,this._isNull),this._C},set:function(e){this._C=e,I(this,"Criteria",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"threeColorScale",{get:function(){return T("threeColorScale",this._T,Xr,this._isNull),this._T},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["criteria"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Criteria)||(this._C=n.Criteria),N(n.ThreeColorScale)||(this._T=n.ThreeColorScale)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{criteria:this._C,threeColorScale:this._T},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ColorScaleConditionalFormat=Yr;var Zr=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TopBottomConditionalFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["rule"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Rule"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ConditionalRangeFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rule",{get:function(){return T("rule",this._R,"TopBottomConditionalFormat",this._isNull),this._R},set:function(e){this._R=e,I(this,"Rule",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["rule"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Rule)||(this._R=n.Rule),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{rule:this._R},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.TopBottomConditionalFormat=Zr;var $r=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PresetCriteriaConditionalFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["rule"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Rule"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ConditionalRangeFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rule",{get:function(){return T("rule",this._R,"PresetCriteriaConditionalFormat",this._isNull),this._R},set:function(e){this._R=e,I(this,"Rule",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["rule"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Rule)||(this._R=n.Rule),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{rule:this._R},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.PresetCriteriaConditionalFormat=$r;var ei=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TextConditionalFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["rule"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Rule"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ConditionalRangeFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rule",{get:function(){return T("rule",this._R,"TextConditionalFormat",this._isNull),this._R},set:function(e){this._R=e,I(this,"Rule",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["rule"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Rule)||(this._R=n.Rule),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{rule:this._R},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.TextConditionalFormat=ei;var ti=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"CellValueConditionalFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["rule"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Rule"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return this._F||(this._F=S(e.ConditionalRangeFormat,this,"Format",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rule",{get:function(){return T("rule",this._R,"CellValueConditionalFormat",this._isNull),this._R},set:function(e){this._R=e,I(this,"Rule",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["rule"],["format"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Rule)||(this._R=n.Rule),L(this,n,["format","Format"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{rule:this._R},{format:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.CellValueConditionalFormat=ti;var ni=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ConditionalRangeFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["numberFormat"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["NumberFormat"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["fill","font","borders"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"borders",{get:function(){return this._B||(this._B=S(e.ConditionalRangeBorderCollection,this,"Borders",!0,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.ConditionalRangeFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._Fo||(this._Fo=S(e.ConditionalRangeFont,this,"Font",!1,4)),this._Fo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormat",{get:function(){return T("numberFormat",this._N,"ConditionalRangeFormat",this._isNull),this._N},set:function(e){this._N=e,I(this,"NumberFormat",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["numberFormat"],["fill","font"],["borders"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.NumberFormat)||(this._N=n.NumberFormat),L(this,n,["borders","Borders","fill","Fill","font","Font"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{numberFormat:this._N},{borders:this._B,fill:this._F,font:this._Fo})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ConditionalRangeFormat=ni;var ri="ConditionalRangeFont",ii=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ConditionalRangeFont"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["color","italic","bold","underline","strikethrough"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Color","Italic","Bold","Underline","Strikethrough"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"bold",{get:function(){return T("bold",this._B,ri,this._isNull),this._B},set:function(e){this._B=e,I(this,"Bold",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return T("color",this._C,ri,this._isNull),this._C},set:function(e){this._C=e,I(this,"Color",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"italic",{get:function(){return T("italic",this._I,ri,this._isNull),this._I},set:function(e){this._I=e,I(this,"Italic",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strikethrough",{get:function(){return T("strikethrough",this._S,ri,this._isNull),this._S},set:function(e){this._S=e,I(this,"Strikethrough",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"underline",{get:function(){return T("underline",this._U,ri,this._isNull),this._U},set:function(e){this._U=e,I(this,"Underline",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["color","italic","bold","underline","strikethrough"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype.clear=function(){E(this,"Clear",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Bold)||(this._B=n.Bold),N(n.Color)||(this._C=n.Color),N(n.Italic)||(this._I=n.Italic),N(n.Strikethrough)||(this._S=n.Strikethrough),N(n.Underline)||(this._U=n.Underline)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{bold:this._B,color:this._C,italic:this._I,strikethrough:this._S,underline:this._U},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ConditionalRangeFont=ii;var oi=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ConditionalRangeFill"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["color"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Color"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return T("color",this._C,"ConditionalRangeFill",this._isNull),this._C},set:function(e){this._C=e,I(this,"Color",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["color"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype.clear=function(){E(this,"Clear",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Color)||(this._C=n.Color)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{color:this._C},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ConditionalRangeFill=oi;var si="ConditionalRangeBorder",ai=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ConditionalRangeBorder"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["sideIndex","style","color"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["SideIndex","Style","Color"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return T("color",this._C,si,this._isNull),this._C},set:function(e){this._C=e,I(this,"Color",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"sideIndex",{get:function(){return T("sideIndex",this._S,si,this._isNull),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"style",{get:function(){return T("style",this._St,si,this._isNull),this._St},set:function(e){this._St=e,I(this,"Style",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["style","color"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Color)||(this._C=n.Color),N(n.SideIndex)||(this._S=n.SideIndex),N(n.Style)||(this._St=n.Style)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{color:this._C,sideIndex:this._S,style:this._St},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ConditionalRangeBorder=ai;var li="ConditionalRangeBorderCollection",ui=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ConditionalRangeBorderCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Count"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["top","bottom","left","right"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"bottom",{get:function(){return this._B||(this._B=S(e.ConditionalRangeBorder,this,"Bottom",!1,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"left",{get:function(){return this._L||(this._L=S(e.ConditionalRangeBorder,this,"Left",!1,4)),this._L},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"right",{get:function(){return this._R||(this._R=S(e.ConditionalRangeBorder,this,"Right",!1,4)),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"top",{get:function(){return this._T||(this._T=S(e.ConditionalRangeBorder,this,"Top",!1,4)),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,li,this._isNull),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"count",{get:function(){return T("count",this._C,li,this._isNull),this._C},enumerable:!0,configurable:!0}),n.prototype.getItem=function(t){return P(e.ConditionalRangeBorder,this,[t])},n.prototype.getItemAt=function(t){return C(e.ConditionalRangeBorder,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),N(r.Count)||(this._C=r.Count),L(this,r,["bottom","Bottom","left","Left","right","Right","top","Top"]),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.ConditionalRangeBorder,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.ConditionalRangeBorder,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{count:this._C},{bottom:this._B,left:this._L,right:this._R,top:this._T},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.ConditionalRangeBorder,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.ConditionalRangeBorderCollection=ui;var ci=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"CustomFunctionManager"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["status"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Status"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"status",{get:function(){return T("status",this._S,"CustomFunctionManager",this._isNull),this._S},enumerable:!0,configurable:!0}),n.prototype.register=function(e,t){E(this,"Register",0,[e,t],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Status)||(this._S=n.Status)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.newObject=function(t){return F(e.CustomFunctionManager,t,"Microsoft.ExcelServices.CustomFunctionManager",!1,4)},n.prototype.toJSON=function(){return k(this,{},{})},n}(OfficeExtension.ClientObject);e.CustomFunctionManager=ci;var pi=function(){function n(){}return n.register=function(n,r){return w("CustomFunctionManager.register","CustomFunctions","1.3",O),t({delayForCellEdit:!0},(function(t){return e.CustomFunctionManager.newObject(t).register(n,r)}))},n.getStatus=function(){return w("CustomFunctionManager.register","CustomFunctions","1.3",O),t({delayForCellEdit:!0},(function(t){var n=e.CustomFunctionManager.newObject(t).load("status");return t.sync().then((function(){return n.status}))}))},n}();e.CustomFunctionManagerCustom=pi,OfficeExtension.Utility.applyMixin(ci,pi),function(e){e.CustomFunctionManager_StaticCustomize=function(e){e.register=pi.register,e.getStatus=pi.getStatus}}(b=e._CC||(e._CC={})),b.CustomFunctionManager_StaticCustomize(ci);var fi="Style",hi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Style"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["builtIn","formulaHidden","horizontalAlignment","includeAlignment","includeBorder","includeFont","includeNumber","includePatterns","includeProtection","indentLevel","locked","name","numberFormat","numberFormatLocal","readingOrder","shrinkToFit","verticalAlignment","wrapText","textOrientation","autoIndent"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["BuiltIn","FormulaHidden","HorizontalAlignment","IncludeAlignment","IncludeBorder","IncludeFont","IncludeNumber","IncludePatterns","IncludeProtection","IndentLevel","Locked","Name","NumberFormat","NumberFormatLocal","ReadingOrder","ShrinkToFit","VerticalAlignment","WrapText","TextOrientation","AutoIndent"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!1,!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["borders","font","fill"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"borders",{get:function(){return this._B||(this._B=S(e.RangeBorderCollection,this,"Borders",!0,4)),this._B},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.RangeFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._Fo||(this._Fo=S(e.RangeFont,this,"Font",!1,4)),this._Fo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"autoIndent",{get:function(){return T("autoIndent",this.m_autoIndent,fi,this._isNull),w("Style.autoIndent",v,"1.8",O),this.m_autoIndent},set:function(e){this.m_autoIndent=e,I(this,"AutoIndent",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"builtIn",{get:function(){return T("builtIn",this._Bu,fi,this._isNull),this._Bu},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"formulaHidden",{get:function(){return T("formulaHidden",this._For,fi,this._isNull),this._For},set:function(e){this._For=e,I(this,"FormulaHidden",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"horizontalAlignment",{get:function(){return T("horizontalAlignment",this._H,fi,this._isNull),this._H},set:function(e){this._H=e,I(this,"HorizontalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"includeAlignment",{get:function(){return T("includeAlignment",this._I,fi,this._isNull),this._I},set:function(e){this._I=e,I(this,"IncludeAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"includeBorder",{get:function(){return T("includeBorder",this._In,fi,this._isNull),this._In},set:function(e){this._In=e,I(this,"IncludeBorder",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"includeFont",{get:function(){return T("includeFont",this._Inc,fi,this._isNull),this._Inc},set:function(e){this._Inc=e,I(this,"IncludeFont",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"includeNumber",{get:function(){return T("includeNumber",this._Incl,fi,this._isNull),this._Incl},set:function(e){this._Incl=e,I(this,"IncludeNumber",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"includePatterns",{get:function(){return T("includePatterns",this._Inclu,fi,this._isNull),this._Inclu},set:function(e){this._Inclu=e,I(this,"IncludePatterns",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"includeProtection",{get:function(){return T("includeProtection",this._Includ,fi,this._isNull),this._Includ},set:function(e){this._Includ=e,I(this,"IncludeProtection",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"indentLevel",{get:function(){return T("indentLevel",this._Ind,fi,this._isNull),this._Ind},set:function(e){this._Ind=e,I(this,"IndentLevel",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"locked",{get:function(){return T("locked",this._L,fi,this._isNull),this._L},set:function(e){this._L=e,I(this,"Locked",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,fi,this._isNull),this._N},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormat",{get:function(){return T("numberFormat",this._Nu,fi,this._isNull),this._Nu},set:function(e){this._Nu=e,I(this,"NumberFormat",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"numberFormatLocal",{get:function(){return T("numberFormatLocal",this._Num,fi,this._isNull),this._Num},set:function(e){this._Num=e,I(this,"NumberFormatLocal",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"readingOrder",{get:function(){return T("readingOrder",this._R,fi,this._isNull),this._R},set:function(e){this._R=e,I(this,"ReadingOrder",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"shrinkToFit",{get:function(){return T("shrinkToFit",this._S,fi,this._isNull),this._S},set:function(e){this._S=e,I(this,"ShrinkToFit",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"textOrientation",{get:function(){return T("textOrientation",this.m_textOrientation,fi,this._isNull),w("Style.textOrientation",v,"1.8",O),this.m_textOrientation},set:function(e){b.Style_TextOrientation_Set(this,e).handled||(this.m_textOrientation=e,I(this,"TextOrientation",e,0))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"verticalAlignment",{get:function(){return T("verticalAlignment",this._V,fi,this._isNull),this._V},set:function(e){this._V=e,I(this,"VerticalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"wrapText",{get:function(){return T("wrapText",this._W,fi,this._isNull),this._W},set:function(e){this._W=e,I(this,"WrapText",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["formulaHidden","horizontalAlignment","includeAlignment","includeBorder","includeFont","includeNumber","includePatterns","includeProtection","indentLevel","locked","numberFormat","numberFormatLocal","readingOrder","shrinkToFit","verticalAlignment","wrapText","textOrientation","autoIndent"],["font","fill"],["borders"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AutoIndent)||(this.m_autoIndent=n.AutoIndent),N(n.BuiltIn)||(this._Bu=n.BuiltIn),N(n.FormulaHidden)||(this._For=n.FormulaHidden),N(n.HorizontalAlignment)||(this._H=n.HorizontalAlignment),N(n.IncludeAlignment)||(this._I=n.IncludeAlignment),N(n.IncludeBorder)||(this._In=n.IncludeBorder),N(n.IncludeFont)||(this._Inc=n.IncludeFont),N(n.IncludeNumber)||(this._Incl=n.IncludeNumber),N(n.IncludePatterns)||(this._Inclu=n.IncludePatterns),N(n.IncludeProtection)||(this._Includ=n.IncludeProtection),N(n.IndentLevel)||(this._Ind=n.IndentLevel),N(n.Locked)||(this._L=n.Locked),N(n.Name)||(this._N=n.Name),N(n.NumberFormat)||(this._Nu=n.NumberFormat),N(n.NumberFormatLocal)||(this._Num=n.NumberFormatLocal),N(n.ReadingOrder)||(this._R=n.ReadingOrder),N(n.ShrinkToFit)||(this._S=n.ShrinkToFit),N(n.TextOrientation)||(this.m_textOrientation=n.TextOrientation),N(n.VerticalAlignment)||(this._V=n.VerticalAlignment),N(n.WrapText)||(this._W=n.WrapText),L(this,n,["borders","Borders","fill","Fill","font","Font"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{autoIndent:this.m_autoIndent,builtIn:this._Bu,formulaHidden:this._For,horizontalAlignment:this._H,includeAlignment:this._I,includeBorder:this._In,includeFont:this._Inc,includeNumber:this._Incl,includePatterns:this._Inclu,includeProtection:this._Includ,indentLevel:this._Ind,locked:this._L,name:this._N,numberFormat:this._Nu,numberFormatLocal:this._Num,readingOrder:this._R,shrinkToFit:this._S,textOrientation:this.m_textOrientation,verticalAlignment:this._V,wrapText:this._W},{borders:this._B,fill:this._F,font:this._Fo})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Style=hi,function(e){e.Style_TextOrientation_Set=function(e,t){return e.m_textOrientation=t,I(e,"Orientation",t,0),{handled:!0}}}(b=e._CC||(e._CC={}));var di=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"StyleCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"StyleCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(e){E(this,"Add",0,[e],0,0)},n.prototype.getCount=function(){return w("StyleCollection.getCount",v,"1.9",O),E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.Style,this,[t])},n.prototype.getItemAt=function(t){return w("StyleCollection.getItemAt",v,"1.9",O),C(e.Style,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return w("StyleCollection.getItemOrNullObject",v,"1.14",O),C(e.Style,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Style,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Style,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Style,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.StyleCollection=di;var yi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TableStyleCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"TableStyleCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n){return C(e.TableStyle,this,"Add",0,[t,n],!1,!0,null,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getDefault=function(){return C(e.TableStyle,this,"GetDefault",0,[],!1,!1,null,0)},n.prototype.getItem=function(t){return P(e.TableStyle,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.TableStyle,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype.setDefault=function(e){E(this,"SetDefault",0,[e],0,0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.TableStyle,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.TableStyle,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.TableStyle,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.TableStyleCollection=yi;var mi="TableStyle",gi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TableStyle"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["name","readOnly","_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name","ReadOnly","_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,mi,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"readOnly",{get:function(){return T("readOnly",this._R,mi,this._isNull),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_Id",{get:function(){return T("_Id",this.__I,mi,this._isNull),this.__I},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name"],[],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.duplicate=function(){return C(e.TableStyle,this,"Duplicate",0,[],!1,!1,null,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Name)||(this._N=n.Name),N(n.ReadOnly)||(this._R=n.ReadOnly),N(n._Id)||(this.__I=n._Id)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e._Id)||(this.__I=e._Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{name:this._N,readOnly:this._R},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.TableStyle=gi;var _i=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PivotTableStyleCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"PivotTableStyleCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n){return C(e.PivotTableStyle,this,"Add",0,[t,n],!1,!0,null,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getDefault=function(){return C(e.PivotTableStyle,this,"GetDefault",0,[],!1,!1,null,0)},n.prototype.getItem=function(t){return P(e.PivotTableStyle,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.PivotTableStyle,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype.setDefault=function(e){E(this,"SetDefault",0,[e],0,0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.PivotTableStyle,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.PivotTableStyle,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.PivotTableStyle,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.PivotTableStyleCollection=_i;var bi="PivotTableStyle",Oi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PivotTableStyle"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["name","readOnly","_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name","ReadOnly","_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,bi,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"readOnly",{get:function(){return T("readOnly",this._R,bi,this._isNull),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_Id",{get:function(){return T("_Id",this.__I,bi,this._isNull),this.__I},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name"],[],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.duplicate=function(){return C(e.PivotTableStyle,this,"Duplicate",0,[],!1,!1,null,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Name)||(this._N=n.Name),N(n.ReadOnly)||(this._R=n.ReadOnly),N(n._Id)||(this.__I=n._Id)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e._Id)||(this.__I=e._Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{name:this._N,readOnly:this._R},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.PivotTableStyle=Oi;var vi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"SlicerStyleCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"SlicerStyleCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n){return C(e.SlicerStyle,this,"Add",0,[t,n],!1,!0,null,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getDefault=function(){return C(e.SlicerStyle,this,"GetDefault",0,[],!1,!1,null,0)},n.prototype.getItem=function(t){return P(e.SlicerStyle,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.SlicerStyle,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype.setDefault=function(e){E(this,"SetDefault",0,[e],0,0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.SlicerStyle,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.SlicerStyle,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.SlicerStyle,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.SlicerStyleCollection=vi;var Si="SlicerStyle",Ci=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"SlicerStyle"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["name","readOnly","_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name","ReadOnly","_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,Si,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"readOnly",{get:function(){return T("readOnly",this._R,Si,this._isNull),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_Id",{get:function(){return T("_Id",this.__I,Si,this._isNull),this.__I},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name"],[],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.duplicate=function(){return C(e.SlicerStyle,this,"Duplicate",0,[],!1,!1,null,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Name)||(this._N=n.Name),N(n.ReadOnly)||(this._R=n.ReadOnly),N(n._Id)||(this.__I=n._Id)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e._Id)||(this.__I=e._Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{name:this._N,readOnly:this._R},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.SlicerStyle=Ci;var Pi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TimelineStyleCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"TimelineStyleCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n){return C(e.TimelineStyle,this,"Add",0,[t,n],!1,!0,null,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getDefault=function(){return C(e.TimelineStyle,this,"GetDefault",0,[],!1,!1,null,0)},n.prototype.getItem=function(t){return P(e.TimelineStyle,this,[t])},n.prototype.getItemOrNullObject=function(t){return C(e.TimelineStyle,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype.setDefault=function(e){E(this,"SetDefault",0,[e],0,0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.TimelineStyle,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.TimelineStyle,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.TimelineStyle,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.TimelineStyleCollection=Pi;var Ai="TimelineStyle",Fi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TimelineStyle"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["name","readOnly","_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name","ReadOnly","_Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,Ai,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"readOnly",{get:function(){return T("readOnly",this._R,Ai,this._isNull),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_Id",{get:function(){return T("_Id",this.__I,Ai,this._isNull),this.__I},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name"],[],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.duplicate=function(){return C(e.TimelineStyle,this,"Duplicate",0,[],!1,!1,null,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Name)||(this._N=n.Name),N(n.ReadOnly)||(this._R=n.ReadOnly),N(n._Id)||(this.__I=n._Id)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e._Id)||(this.__I=e._Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{name:this._N,readOnly:this._R},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.TimelineStyle=Fi;var Di=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"InternalTest"},enumerable:!0,configurable:!0}),n.prototype.delay=function(e){return E(this,"Delay",0,[e],0,0)},n.prototype.enterCellEdit=function(e){w("InternalTest.enterCellEdit",v,"1.9",O),E(this,"EnterCellEdit",0,[e],0,0)},n.prototype.exitCellEdit=function(){w("InternalTest.exitCellEdit",v,"1.9",O),E(this,"ExitCellEdit",0,[],0,0)},n.prototype.firstPartyMethod=function(){w("InternalTest.firstPartyMethod",v,"1.7",O),E(this,"FirstPartyMethod",1,[],5,0)},n.prototype.installCustomFunctionsFromCache=function(){w("InternalTest.installCustomFunctionsFromCache",v,"1.9",O),E(this,"InstallCustomFunctionsFromCache",0,[],0,0)},n.prototype.noPermissionMethod=function(e){return w("InternalTest.noPermissionMethod",v,"1.9",O),E(this,"NoPermissionMethod",0,[e],0,0)},n.prototype.recalc=function(e,t){w("InternalTest.recalc",v,"1.9",O),E(this,"Recalc",0,[e,t],0,0)},n.prototype.recalcBySolutionId=function(e){w("InternalTest.recalcBySolutionId",v,"1.9",O),E(this,"RecalcBySolutionId",0,[e],0,0)},n.prototype.safeForCellEditModeMethod=function(e){return w("InternalTest.safeForCellEditModeMethod",v,"1.9",O),E(this,"SafeForCellEditModeMethod",0,[e],0,0)},n.prototype.triggerMessage=function(e,t,n,r){w("InternalTest.triggerMessage",v,"1.7",O),E(this,"TriggerMessage",0,[e,t,n,r],0,0)},n.prototype.triggerPostProcess=function(){w("InternalTest.triggerPostProcess",v,"1.7",O),E(this,"TriggerPostProcess",0,[],0,0)},n.prototype.triggerTestEvent=function(e,t){w("InternalTest.triggerTestEvent",v,"1.7",O),E(this,"TriggerTestEvent",0,[e,t],0,0)},n.prototype.triggerTestEventWithFilter=function(e,t,n){w("InternalTest.triggerTestEventWithFilter",v,"1.7",O),E(this,"TriggerTestEventWithFilter",0,[e,t,n],0,0)},n.prototype.unregisterAllCustomFunctionExecutionEvents=function(){w("InternalTest.unregisterAllCustomFunctionExecutionEvents","CustomFunctions","1.1",O),E(this,"UnregisterAllCustomFunctionExecutionEvents",0,[],0,0)},n.prototype.verifyCustomFunctionListExist=function(){return w("InternalTest.verifyCustomFunctionListExist",v,"1.9",O),E(this,"VerifyCustomFunctionListExist",0,[],0,0)},n.prototype._RegisterCustomFunctionExecutionBeginEvent=function(){w("InternalTest._RegisterCustomFunctionExecutionBeginEvent","CustomFunctions","1.1",O),E(this,"_RegisterCustomFunctionExecutionBeginEvent",0,[],0,0)},n.prototype._RegisterCustomFunctionExecutionEndEvent=function(){w("InternalTest._RegisterCustomFunctionExecutionEndEvent","CustomFunctions","1.1",O),E(this,"_RegisterCustomFunctionExecutionEndEvent",0,[],0,0)},n.prototype._RegisterTest1Event=function(){w("InternalTest._RegisterTest1Event",v,"1.7",O),E(this,"_RegisterTest1Event",0,[],0,0)},n.prototype._RegisterTestEvent=function(){w("InternalTest._RegisterTestEvent",v,"1.7",O),E(this,"_RegisterTestEvent",0,[],0,0)},n.prototype._UnregisterCustomFunctionExecutionBeginEvent=function(){w("InternalTest._UnregisterCustomFunctionExecutionBeginEvent","CustomFunctions","1.1",O),E(this,"_UnregisterCustomFunctionExecutionBeginEvent",0,[],0,0)},n.prototype._UnregisterCustomFunctionExecutionEndEvent=function(){w("InternalTest._UnregisterCustomFunctionExecutionEndEvent","CustomFunctions","1.1",O),E(this,"_UnregisterCustomFunctionExecutionEndEvent",0,[],0,0)},n.prototype._UnregisterTest1Event=function(){w("InternalTest._UnregisterTest1Event",v,"1.7",O),E(this,"_UnregisterTest1Event",0,[],0,0)},n.prototype._UnregisterTestEvent=function(){w("InternalTest._UnregisterTestEvent",v,"1.7",O),E(this,"_UnregisterTestEvent",0,[],0,0)},n.prototype._handleResult=function(e){(t.prototype._handleResult.call(this,e),x(e))||U(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.newObject=function(t){return F(e.InternalTest,t,"Microsoft.ExcelServices.InternalTest",!1,4)},Object.defineProperty(n.prototype,"onCustomFunctionExecutionBeginEvent",{get:function(){var e=this;return w("InternalTest.onCustomFunctionExecutionBeginEvent","CustomFunctions","1.1",O),this.m_customFunctionExecutionBeginEvent||(this.m_customFunctionExecutionBeginEvent=new OfficeExtension.GenericEventHandlers(this.context,this,"CustomFunctionExecutionBeginEvent",{eventType:200,registerFunc:function(){return e._RegisterCustomFunctionExecutionBeginEvent()},unregisterFunc:function(){return e._UnregisterCustomFunctionExecutionBeginEvent()},getTargetIdFunc:function(){return""},eventArgsTransformFunc:function(e){var t={higherTicks:e.higherTicks,lowerTicks:e.lowerTicks};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_customFunctionExecutionBeginEvent},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onCustomFunctionExecutionEndEvent",{get:function(){var e=this;return w("InternalTest.onCustomFunctionExecutionEndEvent","CustomFunctions","1.1",O),this.m_customFunctionExecutionEndEvent||(this.m_customFunctionExecutionEndEvent=new OfficeExtension.GenericEventHandlers(this.context,this,"CustomFunctionExecutionEndEvent",{eventType:201,registerFunc:function(){return e._RegisterCustomFunctionExecutionEndEvent()},unregisterFunc:function(){return e._UnregisterCustomFunctionExecutionEndEvent()},getTargetIdFunc:function(){return""},eventArgsTransformFunc:function(e){var t={higherTicks:e.higherTicks,lowerTicks:e.lowerTicks};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_customFunctionExecutionEndEvent},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onTest1Event",{get:function(){var e=this;return w("InternalTest.onTest1Event",v,"1.7",O),this.m_test1Event||(this.m_test1Event=new OfficeExtension.GenericEventHandlers(this.context,this,"Test1Event",{eventType:2,registerFunc:function(){return e._RegisterTest1Event()},unregisterFunc:function(){return e._UnregisterTest1Event()},getTargetIdFunc:function(){return""},eventArgsTransformFunc:function(t){var n=b.InternalTest_Test1Event_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_test1Event},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onTestEvent",{get:function(){var e=this;return w("InternalTest.onTestEvent",v,"1.7",O),this.m_testEvent||(this.m_testEvent=new OfficeExtension.GenericEventHandlers(this.context,this,"TestEvent",{eventType:1,registerFunc:function(){return e._RegisterTestEvent()},unregisterFunc:function(){return e._UnregisterTestEvent()},getTargetIdFunc:function(){return""},eventArgsTransformFunc:function(t){var n=b.InternalTest_TestEvent_EventArgsTransform(e,t);return OfficeExtension.Utility._createPromiseFromResult(n)}})),this.m_testEvent},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{},{})},n}(OfficeExtension.ClientObject);e.InternalTest=Di,function(e){e.InternalTest_Test1Event_EventArgsTransform=function(e,t){var n=t;return{prop1:n.prop1,worksheet:e.context.workbook.worksheets.getItem(n.worksheetId)}},e.InternalTest_TestEvent_EventArgsTransform=function(e,t){var n=t;return{prop1:n.prop1,worksheet:e.context.workbook.worksheets.getItem(n.worksheetId)}}}(b=e._CC||(e._CC={}));var Ei="PageLayout",Ri=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PageLayout"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["orientation","paperSize","blackAndWhite","printErrors","zoom","centerHorizontally","centerVertically","printHeadings","printGridlines","leftMargin","rightMargin","topMargin","bottomMargin","headerMargin","footerMargin","printComments","draftMode","firstPageNumber","printOrder"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Orientation","PaperSize","BlackAndWhite","PrintErrors","Zoom","CenterHorizontally","CenterVertically","PrintHeadings","PrintGridlines","LeftMargin","RightMargin","TopMargin","BottomMargin","HeaderMargin","FooterMargin","PrintComments","DraftMode","FirstPageNumber","PrintOrder"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["headersFooters"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"headersFooters",{get:function(){return this._He||(this._He=S(e.HeaderFooterGroup,this,"HeadersFooters",!1,4)),this._He},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"blackAndWhite",{get:function(){return T("blackAndWhite",this._B,Ei,this._isNull),this._B},set:function(e){this._B=e,I(this,"BlackAndWhite",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"bottomMargin",{get:function(){return T("bottomMargin",this._Bo,Ei,this._isNull),this._Bo},set:function(e){this._Bo=e,I(this,"BottomMargin",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"centerHorizontally",{get:function(){return T("centerHorizontally",this._C,Ei,this._isNull),this._C},set:function(e){this._C=e,I(this,"CenterHorizontally",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"centerVertically",{get:function(){return T("centerVertically",this._Ce,Ei,this._isNull),this._Ce},set:function(e){this._Ce=e,I(this,"CenterVertically",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"draftMode",{get:function(){return T("draftMode",this._D,Ei,this._isNull),this._D},set:function(e){this._D=e,I(this,"DraftMode",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"firstPageNumber",{get:function(){return T("firstPageNumber",this._F,Ei,this._isNull),this._F},set:function(e){this._F=e,I(this,"FirstPageNumber",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"footerMargin",{get:function(){return T("footerMargin",this._Fo,Ei,this._isNull),this._Fo},set:function(e){this._Fo=e,I(this,"FooterMargin",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"headerMargin",{get:function(){return T("headerMargin",this._H,Ei,this._isNull),this._H},set:function(e){this._H=e,I(this,"HeaderMargin",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"leftMargin",{get:function(){return T("leftMargin",this._L,Ei,this._isNull),this._L},set:function(e){this._L=e,I(this,"LeftMargin",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"orientation",{get:function(){return T("orientation",this._O,Ei,this._isNull),this._O},set:function(e){this._O=e,I(this,"Orientation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"paperSize",{get:function(){return T("paperSize",this._P,Ei,this._isNull),this._P},set:function(e){this._P=e,I(this,"PaperSize",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"printComments",{get:function(){return T("printComments",this._Pr,Ei,this._isNull),this._Pr},set:function(e){this._Pr=e,I(this,"PrintComments",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"printErrors",{get:function(){return T("printErrors",this._Pri,Ei,this._isNull),this._Pri},set:function(e){this._Pri=e,I(this,"PrintErrors",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"printGridlines",{get:function(){return T("printGridlines",this._Prin,Ei,this._isNull),this._Prin},set:function(e){this._Prin=e,I(this,"PrintGridlines",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"printHeadings",{get:function(){return T("printHeadings",this._Print,Ei,this._isNull),this._Print},set:function(e){this._Print=e,I(this,"PrintHeadings",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"printOrder",{get:function(){return T("printOrder",this._PrintO,Ei,this._isNull),this._PrintO},set:function(e){this._PrintO=e,I(this,"PrintOrder",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rightMargin",{get:function(){return T("rightMargin",this._R,Ei,this._isNull),this._R},set:function(e){this._R=e,I(this,"RightMargin",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"topMargin",{get:function(){return T("topMargin",this._T,Ei,this._isNull),this._T},set:function(e){this._T=e,I(this,"TopMargin",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"zoom",{get:function(){return T("zoom",this._Z,Ei,this._isNull),this._Z},set:function(e){this._Z=e,I(this,"Zoom",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["orientation","paperSize","blackAndWhite","printErrors","zoom","centerHorizontally","centerVertically","printHeadings","printGridlines","leftMargin","rightMargin","topMargin","bottomMargin","headerMargin","footerMargin","printComments","draftMode","firstPageNumber","printOrder"],["headersFooters"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.getPrintArea=function(){return C(e.RangeAreas,this,"GetPrintArea",1,[],!1,!0,null,4)},n.prototype.getPrintAreaOrNullObject=function(){return C(e.RangeAreas,this,"GetPrintAreaOrNullObject",1,[],!1,!0,null,4)},n.prototype.getPrintTitleColumns=function(){return C(e.Range,this,"GetPrintTitleColumns",1,[],!1,!0,null,4)},n.prototype.getPrintTitleColumnsOrNullObject=function(){return C(e.Range,this,"GetPrintTitleColumnsOrNullObject",1,[],!1,!0,null,4)},n.prototype.getPrintTitleRows=function(){return C(e.Range,this,"GetPrintTitleRows",1,[],!1,!0,null,4)},n.prototype.getPrintTitleRowsOrNullObject=function(){return C(e.Range,this,"GetPrintTitleRowsOrNullObject",1,[],!1,!0,null,4)},n.prototype.setPrintArea=function(e){E(this,"SetPrintArea",0,[e],0,0)},n.prototype.setPrintMargins=function(e,t){E(this,"SetPrintMargins",0,[e,t],0,0)},n.prototype.setPrintTitleColumns=function(e){E(this,"SetPrintTitleColumns",0,[e],0,0)},n.prototype.setPrintTitleRows=function(e){E(this,"SetPrintTitleRows",0,[e],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.BlackAndWhite)||(this._B=n.BlackAndWhite),N(n.BottomMargin)||(this._Bo=n.BottomMargin),N(n.CenterHorizontally)||(this._C=n.CenterHorizontally),N(n.CenterVertically)||(this._Ce=n.CenterVertically),N(n.DraftMode)||(this._D=n.DraftMode),N(n.FirstPageNumber)||(this._F=n.FirstPageNumber),N(n.FooterMargin)||(this._Fo=n.FooterMargin),N(n.HeaderMargin)||(this._H=n.HeaderMargin),N(n.LeftMargin)||(this._L=n.LeftMargin),N(n.Orientation)||(this._O=n.Orientation),N(n.PaperSize)||(this._P=n.PaperSize),N(n.PrintComments)||(this._Pr=n.PrintComments),N(n.PrintErrors)||(this._Pri=n.PrintErrors),N(n.PrintGridlines)||(this._Prin=n.PrintGridlines),N(n.PrintHeadings)||(this._Print=n.PrintHeadings),N(n.PrintOrder)||(this._PrintO=n.PrintOrder),N(n.RightMargin)||(this._R=n.RightMargin),N(n.TopMargin)||(this._T=n.TopMargin),N(n.Zoom)||(this._Z=n.Zoom),L(this,n,["headersFooters","HeadersFooters"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{blackAndWhite:this._B,bottomMargin:this._Bo,centerHorizontally:this._C,centerVertically:this._Ce,draftMode:this._D,firstPageNumber:this._F,footerMargin:this._Fo,headerMargin:this._H,leftMargin:this._L,orientation:this._O,paperSize:this._P,printComments:this._Pr,printErrors:this._Pri,printGridlines:this._Prin,printHeadings:this._Print,printOrder:this._PrintO,rightMargin:this._R,topMargin:this._T,zoom:this._Z},{headersFooters:this._He})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.PageLayout=Ri;var Ii="HeaderFooter",xi=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"HeaderFooter"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["leftHeader","centerHeader","rightHeader","leftFooter","centerFooter","rightFooter"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["LeftHeader","CenterHeader","RightHeader","LeftFooter","CenterFooter","RightFooter"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"centerFooter",{get:function(){return T("centerFooter",this._C,Ii,this._isNull),this._C},set:function(e){this._C=e,I(this,"CenterFooter",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"centerHeader",{get:function(){return T("centerHeader",this._Ce,Ii,this._isNull),this._Ce},set:function(e){this._Ce=e,I(this,"CenterHeader",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"leftFooter",{get:function(){return T("leftFooter",this._L,Ii,this._isNull),this._L},set:function(e){this._L=e,I(this,"LeftFooter",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"leftHeader",{get:function(){return T("leftHeader",this._Le,Ii,this._isNull),this._Le},set:function(e){this._Le=e,I(this,"LeftHeader",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"rightFooter",{get:function(){return T("rightFooter",this._R,Ii,this._isNull),this._R},set:function(e){this._R=e,I(this,"RightFooter",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"rightHeader",{get:function(){return T("rightHeader",this._Ri,Ii,this._isNull),this._Ri},set:function(e){this._Ri=e,I(this,"RightHeader",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["leftHeader","centerHeader","rightHeader","leftFooter","centerFooter","rightFooter"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.CenterFooter)||(this._C=n.CenterFooter),N(n.CenterHeader)||(this._Ce=n.CenterHeader),N(n.LeftFooter)||(this._L=n.LeftFooter),N(n.LeftHeader)||(this._Le=n.LeftHeader),N(n.RightFooter)||(this._R=n.RightFooter),N(n.RightHeader)||(this._Ri=n.RightHeader)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{centerFooter:this._C,centerHeader:this._Ce,leftFooter:this._L,leftHeader:this._Le,rightFooter:this._R,rightHeader:this._Ri},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.HeaderFooter=xi;var Ni="HeaderFooterGroup",Ti=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"HeaderFooterGroup"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["state","useSheetMargins","useSheetScale"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["State","UseSheetMargins","UseSheetScale"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["defaultForAllPages","firstPage","evenPages","oddPages"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"defaultForAllPages",{get:function(){return this._D||(this._D=S(e.HeaderFooter,this,"DefaultForAllPages",!1,4)),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"evenPages",{get:function(){return this._E||(this._E=S(e.HeaderFooter,this,"EvenPages",!1,4)),this._E},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"firstPage",{get:function(){return this._F||(this._F=S(e.HeaderFooter,this,"FirstPage",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"oddPages",{get:function(){return this._O||(this._O=S(e.HeaderFooter,this,"OddPages",!1,4)),this._O},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"state",{get:function(){return T("state",this._S,Ni,this._isNull),this._S},set:function(e){this._S=e,I(this,"State",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"useSheetMargins",{get:function(){return T("useSheetMargins",this._U,Ni,this._isNull),this._U},set:function(e){this._U=e,I(this,"UseSheetMargins",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"useSheetScale",{get:function(){return T("useSheetScale",this._Us,Ni,this._isNull),this._Us},set:function(e){this._Us=e,I(this,"UseSheetScale",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["state","useSheetMargins","useSheetScale"],["defaultForAllPages","firstPage","evenPages","oddPages"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.State)||(this._S=n.State),N(n.UseSheetMargins)||(this._U=n.UseSheetMargins),N(n.UseSheetScale)||(this._Us=n.UseSheetScale),L(this,n,["defaultForAllPages","DefaultForAllPages","evenPages","EvenPages","firstPage","FirstPage","oddPages","OddPages"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{state:this._S,useSheetMargins:this._U,useSheetScale:this._Us},{defaultForAllPages:this._D,evenPages:this._E,firstPage:this._F,oddPages:this._O})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.HeaderFooterGroup=Ti;var wi="PageBreak",ji=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PageBreak"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["_Id","columnIndex","rowIndex"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["_Id","ColumnIndex","RowIndex"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"columnIndex",{get:function(){return T("columnIndex",this._C,wi,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rowIndex",{get:function(){return T("rowIndex",this._R,wi,this._isNull),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_Id",{get:function(){return T("_Id",this.__I,wi,this._isNull),this.__I},enumerable:!0,configurable:!0}),n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.getCellAfterBreak=function(){return C(e.Range,this,"GetCellAfterBreak",1,[],!1,!0,null,4)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.ColumnIndex)||(this._C=n.ColumnIndex),N(n.RowIndex)||(this._R=n.RowIndex),N(n._Id)||(this.__I=n._Id)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e._Id)||(this.__I=e._Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{columnIndex:this._C,rowIndex:this._R},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.PageBreak=ji;var Mi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"PageBreakCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"PageBreakCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t){return C(e.PageBreak,this,"Add",0,[t],!1,!0,null,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.PageBreak,this,[t])},n.prototype.removePageBreaks=function(){E(this,"RemovePageBreaks",0,[],0,0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.PageBreak,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.PageBreak,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.PageBreak,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.PageBreakCollection=Mi;var ki=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"DataConnectionCollection"},enumerable:!0,configurable:!0}),t.prototype.refreshAll=function(){E(this,"RefreshAll",0,[],0,0)},t.prototype._handleResult=function(t){(e.prototype._handleResult.call(this,t),x(t))||U(this,t)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{},{})},t}(OfficeExtension.ClientObject);e.DataConnectionCollection=ki;var Ui=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"RangeCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"RangeCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItemAt=function(t){return C(e.Range,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Range,!1,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Range,!1,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Range,!1,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.RangeCollection=Ui;var Li=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"RangeAreasCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"RangeAreasCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItemAt=function(t){return C(e.RangeAreas,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.RangeAreas,!1,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.RangeAreas,!1,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.RangeAreas,!1,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.RangeAreasCollection=Li;var Bi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"CommentCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"CommentCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n,r){return C(e.Comment,this,"Add",0,[t,n,r],!1,!0,null,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.Comment,this,[t])},n.prototype.getItemAt=function(t){return C(e.Comment,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype.getItemByCell=function(t){return C(e.Comment,this,"GetItemByCell",1,[t],!1,!1,null,4)},n.prototype.getItemByReplyId=function(t){return C(e.Comment,this,"GetItemByReplyId",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return w("CommentCollection.getItemOrNullObject",v,"1.14",O),C(e.Comment,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._RegisterAddedEvent=function(){w("CommentCollection._RegisterAddedEvent",v,"1.12",O),E(this,"_RegisterAddedEvent",1,[],0,0)},n.prototype._RegisterChangedEvent=function(){w("CommentCollection._RegisterChangedEvent",v,"1.12",O),E(this,"_RegisterChangedEvent",1,[],0,0)},n.prototype._RegisterDeletedEvent=function(){w("CommentCollection._RegisterDeletedEvent",v,"1.12",O),E(this,"_RegisterDeletedEvent",1,[],0,0)},n.prototype._UnregisterAddedEvent=function(){w("CommentCollection._UnregisterAddedEvent",v,"1.12",O),E(this,"_UnregisterAddedEvent",1,[],0,0)},n.prototype._UnregisterChangedEvent=function(){w("CommentCollection._UnregisterChangedEvent",v,"1.12",O),E(this,"_UnregisterChangedEvent",1,[],0,0)},n.prototype._UnregisterDeletedEvent=function(){w("CommentCollection._UnregisterDeletedEvent",v,"1.12",O),E(this,"_UnregisterDeletedEvent",1,[],0,0)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Comment,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Comment,!0,i,t,n)}))},Object.defineProperty(n.prototype,"onAdded",{get:function(){var e=this;return w("CommentCollection.onAdded",v,"1.12",O),this.m_added||(this.m_added=new OfficeExtension.GenericEventHandlers(this.context,this,"Added",{eventType:250,registerFunc:function(){return e._RegisterAddedEvent()},unregisterFunc:function(){return e._UnregisterAddedEvent()},getTargetIdFunc:function(){return e._eventTargetId},eventArgsTransformFunc:function(e){var t={type:Oo.commentAdded,commentDetails:e.commentDetails,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_added},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onChanged",{get:function(){var e=this;return w("CommentCollection.onChanged",v,"1.12",O),this.m_changed||(this.m_changed=new OfficeExtension.GenericEventHandlers(this.context,this,"Changed",{eventType:252,registerFunc:function(){return e._RegisterChangedEvent()},unregisterFunc:function(){return e._UnregisterChangedEvent()},getTargetIdFunc:function(){return e._eventTargetId},eventArgsTransformFunc:function(e){var t={type:Oo.commentChanged,changeType:e.changeType,commentDetails:e.commentDetails,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_changed},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onDeleted",{get:function(){var e=this;return w("CommentCollection.onDeleted",v,"1.12",O),this.m_deleted||(this.m_deleted=new OfficeExtension.GenericEventHandlers(this.context,this,"Deleted",{eventType:251,registerFunc:function(){return e._RegisterDeletedEvent()},unregisterFunc:function(){return e._UnregisterDeletedEvent()},getTargetIdFunc:function(){return e._eventTargetId},eventArgsTransformFunc:function(e){var t={type:Oo.commentDeleted,commentDetails:e.commentDetails,source:e.source,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_deleted},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Comment,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.CommentCollection=Bi;var Hi=function(){function e(){}return Object.defineProperty(e.prototype,"_ParentObject",{get:function(){return this.m__ParentObject},set:function(e){this.m__ParentObject=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"_eventTargetId",{get:function(){return this._ParentObject?this._ParentObject.id:OfficeExtension.Constants.eventWorkbookId},enumerable:!0,configurable:!0}),e}();e.CommentCollectionCustom=Hi,OfficeExtension.Utility.applyMixin(Bi,Hi);var Wi="Comment",Vi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Comment"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","content","authorName","authorEmail","creationDate","resolved","richContent","mentions","contentType"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Content","AuthorName","AuthorEmail","CreationDate","Resolved","RichContent","Mentions","ContentType"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!1,!1,!1,!0,!1,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["replies"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"replies",{get:function(){return this._R||(this._R=S(e.CommentReplyCollection,this,"Replies",!0,4)),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"authorEmail",{get:function(){return T("authorEmail",this._A,Wi,this._isNull),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"authorName",{get:function(){return T("authorName",this._Au,Wi,this._isNull),this._Au},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"content",{get:function(){return T("content",this._C,Wi,this._isNull),this._C},set:function(e){this._C=e,I(this,"Content",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"contentType",{get:function(){return T("contentType",this._Co,Wi,this._isNull),w("Comment.contentType",v,"1.12",O),this._Co},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"creationDate",{get:function(){return T("creationDate",this._Cr,Wi,this._isNull),this._Cr},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,Wi,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"mentions",{get:function(){return T("mentions",this._M,Wi,this._isNull),w("Comment.mentions",v,"1.11",O),this._M},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"resolved",{get:function(){return T("resolved",this._Re,Wi,this._isNull),w("Comment.resolved",v,"1.11",O),this._Re},set:function(e){this._Re=e,I(this,"Resolved",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"richContent",{get:function(){return T("richContent",this._Ri,Wi,this._isNull),w("Comment.richContent",v,"1.11",O),this._Ri},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["content","resolved"],[],["replies"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.getLocation=function(){return C(e.Range,this,"GetLocation",1,[],!1,!0,null,4)},n.prototype.updateMentions=function(e){w("Comment.updateMentions",v,"1.11",O),E(this,"UpdateMentions",0,[e],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AuthorEmail)||(this._A=n.AuthorEmail),N(n.AuthorName)||(this._Au=n.AuthorName),N(n.Content)||(this._C=n.Content),N(n.ContentType)||(this._Co=n.ContentType),N(n.CreationDate)||(this._Cr=B(n.CreationDate)),N(n.Id)||(this._I=n.Id),N(n.Mentions)||(this._M=n.Mentions),N(n.Resolved)||(this._Re=n.Resolved),N(n.RichContent)||(this._Ri=n.RichContent),L(this,n,["replies","Replies"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){if(t.prototype._handleRetrieveResult.call(this,e,n),!x(e)){var r=e;N(r.CreationDate)||(r.creationDate=B(r.creationDate)),H(this,e,n)}},n.prototype.toJSON=function(){return k(this,{authorEmail:this._A,authorName:this._Au,content:this._C,contentType:this._Co,creationDate:this._Cr,id:this._I,mentions:this._M,resolved:this._Re,richContent:this._Ri},{replies:this._R})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Comment=Vi;var Gi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"CommentReplyCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"CommentReplyCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n){return C(e.CommentReply,this,"Add",0,[t,n],!1,!0,null,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.CommentReply,this,[t])},n.prototype.getItemAt=function(t){return C(e.CommentReply,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return w("CommentReplyCollection.getItemOrNullObject",v,"1.14",O),C(e.CommentReply,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.CommentReply,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.CommentReply,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.CommentReply,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.CommentReplyCollection=Gi;var zi="CommentReply",qi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"CommentReply"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","content","authorName","authorEmail","creationDate","resolved","richContent","mentions","contentType"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Content","AuthorName","AuthorEmail","CreationDate","Resolved","RichContent","Mentions","ContentType"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!1,!1,!1,!1,!1,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"authorEmail",{get:function(){return T("authorEmail",this._A,zi,this._isNull),this._A},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"authorName",{get:function(){return T("authorName",this._Au,zi,this._isNull),this._Au},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"content",{get:function(){return T("content",this._C,zi,this._isNull),this._C},set:function(e){this._C=e,I(this,"Content",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"contentType",{get:function(){return T("contentType",this._Co,zi,this._isNull),w("CommentReply.contentType",v,"1.12",O),this._Co},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"creationDate",{get:function(){return T("creationDate",this._Cr,zi,this._isNull),this._Cr},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,zi,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"mentions",{get:function(){return T("mentions",this._M,zi,this._isNull),w("CommentReply.mentions",v,"1.11",O),this._M},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"resolved",{get:function(){return T("resolved",this._R,zi,this._isNull),w("CommentReply.resolved",v,"1.11",O),this._R},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"richContent",{get:function(){return T("richContent",this._Ri,zi,this._isNull),w("CommentReply.richContent",v,"1.11",O),this._Ri},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["content"],[],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.getLocation=function(){return C(e.Range,this,"GetLocation",1,[],!1,!0,null,4)},n.prototype.getParentComment=function(){return C(e.Comment,this,"GetParentComment",0,[],!1,!1,null,0)},n.prototype.updateMentions=function(e){w("CommentReply.updateMentions",v,"1.11",O),E(this,"UpdateMentions",0,[e],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AuthorEmail)||(this._A=n.AuthorEmail),N(n.AuthorName)||(this._Au=n.AuthorName),N(n.Content)||(this._C=n.Content),N(n.ContentType)||(this._Co=n.ContentType),N(n.CreationDate)||(this._Cr=B(n.CreationDate)),N(n.Id)||(this._I=n.Id),N(n.Mentions)||(this._M=n.Mentions),N(n.Resolved)||(this._R=n.Resolved),N(n.RichContent)||(this._Ri=n.RichContent)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){if(t.prototype._handleRetrieveResult.call(this,e,n),!x(e)){var r=e;N(r.CreationDate)||(r.creationDate=B(r.creationDate)),H(this,e,n)}},n.prototype.toJSON=function(){return k(this,{authorEmail:this._A,authorName:this._Au,content:this._C,contentType:this._Co,creationDate:this._Cr,id:this._I,mentions:this._M,resolved:this._R,richContent:this._Ri},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.CommentReply=qi;var Ji=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ShapeCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"ShapeCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.addGeometricShape=function(t){return C(e.Shape,this,"AddGeometricShape",0,[t],!1,!1,null,0)},n.prototype.addGroup=function(t){return C(e.Shape,this,"AddGroup",0,[t],!1,!1,null,0)},n.prototype.addImage=function(t){return C(e.Shape,this,"AddImage",0,[t],!1,!1,null,0)},n.prototype.addLine=function(t,n,r,i,o){return C(e.Shape,this,"AddLine",0,[t,n,r,i,o],!1,!1,null,0)},n.prototype.addTextBox=function(t){return C(e.Shape,this,"AddTextBox",0,[t],!1,!1,null,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return C(e.Shape,this,"GetItem",1,[t],!1,!1,null,4)},n.prototype.getItemAt=function(t){return C(e.Shape,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return w("ShapeCollection.getItemOrNullObject",v,"1.14",O),C(e.Shape,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._GetItem=function(t){return P(e.Shape,this,[t])},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Shape,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Shape,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Shape,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.ShapeCollection=Ji;var Ki=function(){function e(){}return Object.defineProperty(e.prototype,"_ParentObject",{get:function(){return this.m__ParentObject},set:function(e){this.m__ParentObject=e},enumerable:!0,configurable:!0}),e}();e.ShapeCollectionCustom=Ki,OfficeExtension.Utility.applyMixin(Ji,Ki);var Qi="Shape",Xi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Shape"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","name","left","top","width","height","rotation","zOrderPosition","altTextTitle","altTextDescription","type","lockAspectRatio","placement","geometricShapeType","visible","level","connectionSiteCount","displayName"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Name","Left","Top","Width","Height","Rotation","ZOrderPosition","AltTextTitle","AltTextDescription","Type","LockAspectRatio","Placement","GeometricShapeType","Visible","Level","ConnectionSiteCount","DisplayName"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0,!0,!0,!0,!0,!1,!0,!0,!1,!0,!0,!0,!0,!1,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["geometricShape","image","textFrame","fill","group","parentGroup","line","lineFormat"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"fill",{get:function(){return this._F||(this._F=S(e.ShapeFill,this,"Fill",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"geometricShape",{get:function(){return this._G||(this._G=S(e.GeometricShape,this,"GeometricShape",!1,4)),this._G},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"group",{get:function(){return this._Gr||(this._Gr=S(e.ShapeGroup,this,"Group",!1,4)),this._Gr},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"image",{get:function(){return this._Im||(this._Im=S(e.Image,this,"Image",!1,4)),this._Im},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"line",{get:function(){return this._Li||(this._Li=S(e.Line,this,"Line",!1,4)),this._Li},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"lineFormat",{get:function(){return this._Lin||(this._Lin=S(e.ShapeLineFormat,this,"LineFormat",!1,4)),this._Lin},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"parentGroup",{get:function(){return this._P||(this._P=S(e.Shape,this,"ParentGroup",!1,4)),this._P},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"textFrame",{get:function(){return this._T||(this._T=S(e.TextFrame,this,"TextFrame",!1,4)),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"altTextDescription",{get:function(){return T("altTextDescription",this._A,Qi,this._isNull),this._A},set:function(e){this._A=e,I(this,"AltTextDescription",e,V(2,"ExcelApiUndo","1.5"))},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"altTextTitle",{get:function(){return T("altTextTitle",this._Al,Qi,this._isNull),this._Al},set:function(e){this._Al=e,I(this,"AltTextTitle",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"connectionSiteCount",{get:function(){return T("connectionSiteCount",this._C,Qi,this._isNull),this._C},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"displayName",{get:function(){return T("displayName",this._D,Qi,this._isNull),w("Shape.displayName",v,"1.15",O),this._D},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"geometricShapeType",{get:function(){return T("geometricShapeType",this._Ge,Qi,this._isNull),this._Ge},set:function(e){this._Ge=e,I(this,"GeometricShapeType",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"height",{get:function(){return T("height",this._H,Qi,this._isNull),this._H},set:function(e){this._H=e,I(this,"Height",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,Qi,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"left",{get:function(){return T("left",this._L,Qi,this._isNull),this._L},set:function(e){this._L=e,I(this,"Left",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"level",{get:function(){return T("level",this._Le,Qi,this._isNull),this._Le},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"lockAspectRatio",{get:function(){return T("lockAspectRatio",this._Lo,Qi,this._isNull),this._Lo},set:function(e){this._Lo=e,I(this,"LockAspectRatio",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,Qi,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"placement",{get:function(){return T("placement",this._Pl,Qi,this._isNull),w("Shape.placement",v,"1.10",O),this._Pl},set:function(e){this._Pl=e,I(this,"Placement",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rotation",{get:function(){return T("rotation",this._R,Qi,this._isNull),this._R},set:function(e){this._R=e,I(this,"Rotation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"top",{get:function(){return T("top",this._To,Qi,this._isNull),this._To},set:function(e){this._To=e,I(this,"Top",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"type",{get:function(){return T("type",this._Ty,Qi,this._isNull),this._Ty},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"visible",{get:function(){return T("visible",this._V,Qi,this._isNull),this._V},set:function(e){this._V=e,I(this,"Visible",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"width",{get:function(){return T("width",this._W,Qi,this._isNull),this._W},set:function(e){this._W=e,I(this,"Width",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"zOrderPosition",{get:function(){return T("zOrderPosition",this._Z,Qi,this._isNull),this._Z},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name","left","top","width","height","rotation","altTextTitle","altTextDescription","lockAspectRatio","placement","geometricShapeType","visible"],["fill","lineFormat"],["geometricShape","group","image","line","parentGroup","textFrame"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.copyTo=function(t){return w("Shape.copyTo",v,"1.10",O),C(e.Shape,this,"CopyTo",0,[t],!1,!0,"_GetShapeById",0)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.getAsImage=function(e){return E(this,"GetAsImage",0,[e],0,0)},n.prototype.incrementLeft=function(e){E(this,"IncrementLeft",0,[e],0,0)},n.prototype.incrementRotation=function(e){E(this,"IncrementRotation",0,[e],0,0)},n.prototype.incrementTop=function(e){E(this,"IncrementTop",0,[e],0,0)},n.prototype.scaleHeight=function(e,t,n){E(this,"ScaleHeight",0,[e,t,n],0,0)},n.prototype.scaleWidth=function(e,t,n){E(this,"ScaleWidth",0,[e,t,n],0,0)},n.prototype.setZOrder=function(e){E(this,"SetZOrder",0,[e],0,0)},n.prototype._GetShapeById=function(t){return w("Shape._GetShapeById",v,"1.10",O),C(e.Shape,this,"_GetShapeById",1,[t],!1,!1,null,4)},n.prototype._RegisterActivatedEvent=function(){E(this,"_RegisterActivatedEvent",0,[],0,0)},n.prototype._RegisterDeactivatedEvent=function(){E(this,"_RegisterDeactivatedEvent",0,[],0,0)},n.prototype._UnregisterActivatedEvent=function(){E(this,"_UnregisterActivatedEvent",0,[],0,0)},n.prototype._UnregisterDeactivatedEvent=function(){E(this,"_UnregisterDeactivatedEvent",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AltTextDescription)||(this._A=n.AltTextDescription),N(n.AltTextTitle)||(this._Al=n.AltTextTitle),N(n.ConnectionSiteCount)||(this._C=n.ConnectionSiteCount),N(n.DisplayName)||(this._D=n.DisplayName),N(n.GeometricShapeType)||(this._Ge=n.GeometricShapeType),N(n.Height)||(this._H=n.Height),N(n.Id)||(this._I=n.Id),N(n.Left)||(this._L=n.Left),N(n.Level)||(this._Le=n.Level),N(n.LockAspectRatio)||(this._Lo=n.LockAspectRatio),N(n.Name)||(this._N=n.Name),N(n.Placement)||(this._Pl=n.Placement),N(n.Rotation)||(this._R=n.Rotation),N(n.Top)||(this._To=n.Top),N(n.Type)||(this._Ty=n.Type),N(n.Visible)||(this._V=n.Visible),N(n.Width)||(this._W=n.Width),N(n.ZOrderPosition)||(this._Z=n.ZOrderPosition),L(this,n,["fill","Fill","geometricShape","GeometricShape","group","Group","image","Image","line","Line","lineFormat","LineFormat","parentGroup","ParentGroup","textFrame","TextFrame"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},Object.defineProperty(n.prototype,"onActivated",{get:function(){var e=this;return this.m_activated||(this.m_activated=new OfficeExtension.GenericEventHandlers(this.context,this,"Activated",{eventType:2101,registerFunc:function(){return e._RegisterActivatedEvent()},unregisterFunc:function(){return e._UnregisterActivatedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.shapeActivated,shapeId:e.shapeId,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_activated},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"onDeactivated",{get:function(){var e=this;return this.m_deactivated||(this.m_deactivated=new OfficeExtension.GenericEventHandlers(this.context,this,"Deactivated",{eventType:2102,registerFunc:function(){return e._RegisterDeactivatedEvent()},unregisterFunc:function(){return e._UnregisterDeactivatedEvent()},getTargetIdFunc:function(){return e.id},eventArgsTransformFunc:function(e){var t={type:Oo.shapeDeactivated,shapeId:e.shapeId,worksheetId:e.worksheetId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_deactivated},enumerable:!0,configurable:!0}),n.prototype.toJSON=function(){return k(this,{altTextDescription:this._A,altTextTitle:this._Al,connectionSiteCount:this._C,displayName:this._D,geometricShapeType:this._Ge,height:this._H,id:this._I,left:this._L,level:this._Le,lockAspectRatio:this._Lo,name:this._N,placement:this._Pl,rotation:this._R,top:this._To,type:this._Ty,visible:this._V,width:this._W,zOrderPosition:this._Z},{fill:this._F,lineFormat:this._Lin})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Shape=Xi;var Yi=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"GeometricShape"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["shape"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"shape",{get:function(){return this._S||(this._S=S(e.Shape,this,"Shape",!1,4)),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,"GeometricShape",this._isNull),this._I},enumerable:!0,configurable:!0}),n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Id)||(this._I=n.Id),L(this,n,["shape","Shape"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{id:this._I},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.GeometricShape=Yi;var Zi="Image",$i=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Image"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","format"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["shape"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"shape",{get:function(){return this._S||(this._S=S(e.Shape,this,"Shape",!1,4)),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,Zi,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"format",{get:function(){return T("format",this._f,Zi,this._isNull),this._f},enumerable:!0,configurable:!0}),n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Id)||(this._I=n.Id),N(n.format)||(this._f=n.format),L(this,n,["shape","Shape"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{format:this._f,id:this._I},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Image=$i;var eo=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"ShapeGroup"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["shapes","shape"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"shape",{get:function(){return this._S||(this._S=S(e.Shape,this,"Shape",!1,4)),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"shapes",{get:function(){return this._Sh||(this._Sh=S(e.GroupShapeCollection,this,"Shapes",!0,4)),this._Sh},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,"ShapeGroup",this._isNull),this._I},enumerable:!0,configurable:!0}),n.prototype.ungroup=function(){E(this,"Ungroup",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Id)||(this._I=n.Id),L(this,n,["shape","Shape","shapes","Shapes"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{id:this._I},{shapes:this._Sh})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.ShapeGroup=eo;var to=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"GroupShapeCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"GroupShapeCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return C(e.Shape,this,"GetItem",1,[t],!1,!1,null,4)},n.prototype.getItemAt=function(t){return C(e.Shape,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return w("GroupShapeCollection.getItemOrNullObject",v,"1.14",O),C(e.Shape,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._GetItem=function(t){return P(e.Shape,this,[t])},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Shape,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Shape,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Shape,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.GroupShapeCollection=to;var no="Line",ro=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Line"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","connectorType","beginArrowheadLength","beginArrowheadStyle","beginArrowheadWidth","endArrowheadLength","endArrowheadStyle","endArrowheadWidth","isBeginConnected","beginConnectedSite","isEndConnected","endConnectedSite"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","connectorType","BeginArrowheadLength","BeginArrowheadStyle","BeginArrowheadWidth","EndArrowheadLength","EndArrowheadStyle","EndArrowheadWidth","IsBeginConnected","BeginConnectedSite","IsEndConnected","EndConnectedSite"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0,!0,!0,!0,!0,!0,!1,!1,!1,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["shape","beginConnectedShape","endConnectedShape"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"beginConnectedShape",{get:function(){return this._Begi||(this._Begi=S(e.Shape,this,"BeginConnectedShape",!1,4)),this._Begi},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"endConnectedShape",{get:function(){return this._EndC||(this._EndC=S(e.Shape,this,"EndConnectedShape",!1,4)),this._EndC},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"shape",{get:function(){return this._S||(this._S=S(e.Shape,this,"Shape",!1,4)),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"beginArrowheadLength",{get:function(){return T("beginArrowheadLength",this._B,no,this._isNull),this._B},set:function(e){this._B=e,I(this,"BeginArrowheadLength",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"beginArrowheadStyle",{get:function(){return T("beginArrowheadStyle",this._Be,no,this._isNull),this._Be},set:function(e){this._Be=e,I(this,"BeginArrowheadStyle",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"beginArrowheadWidth",{get:function(){return T("beginArrowheadWidth",this._Beg,no,this._isNull),this._Beg},set:function(e){this._Beg=e,I(this,"BeginArrowheadWidth",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"beginConnectedSite",{get:function(){return T("beginConnectedSite",this._Begin,no,this._isNull),this._Begin},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"endArrowheadLength",{get:function(){return T("endArrowheadLength",this._E,no,this._isNull),this._E},set:function(e){this._E=e,I(this,"EndArrowheadLength",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"endArrowheadStyle",{get:function(){return T("endArrowheadStyle",this._En,no,this._isNull),this._En},set:function(e){this._En=e,I(this,"EndArrowheadStyle",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"endArrowheadWidth",{get:function(){return T("endArrowheadWidth",this._End,no,this._isNull),this._End},set:function(e){this._End=e,I(this,"EndArrowheadWidth",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"endConnectedSite",{get:function(){return T("endConnectedSite",this._EndCo,no,this._isNull),this._EndCo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,no,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isBeginConnected",{get:function(){return T("isBeginConnected",this._Is,no,this._isNull),this._Is},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isEndConnected",{get:function(){return T("isEndConnected",this._IsE,no,this._isNull),this._IsE},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"connectorType",{get:function(){return T("connectorType",this._c,no,this._isNull),this._c},set:function(e){this._c=e,I(this,"connectorType",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["connectorType","beginArrowheadLength","beginArrowheadStyle","beginArrowheadWidth","endArrowheadLength","endArrowheadStyle","endArrowheadWidth"],[],["beginConnectedShape","endConnectedShape","shape"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.connectBeginShape=function(e,t){E(this,"ConnectBeginShape",0,[e,t],0,0)},n.prototype.connectEndShape=function(e,t){E(this,"ConnectEndShape",0,[e,t],0,0)},n.prototype.disconnectBeginShape=function(){E(this,"DisconnectBeginShape",0,[],0,0)},n.prototype.disconnectEndShape=function(){E(this,"DisconnectEndShape",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.BeginArrowheadLength)||(this._B=n.BeginArrowheadLength),N(n.BeginArrowheadStyle)||(this._Be=n.BeginArrowheadStyle),N(n.BeginArrowheadWidth)||(this._Beg=n.BeginArrowheadWidth),N(n.BeginConnectedSite)||(this._Begin=n.BeginConnectedSite),N(n.EndArrowheadLength)||(this._E=n.EndArrowheadLength),N(n.EndArrowheadStyle)||(this._En=n.EndArrowheadStyle),N(n.EndArrowheadWidth)||(this._End=n.EndArrowheadWidth),N(n.EndConnectedSite)||(this._EndCo=n.EndConnectedSite),N(n.Id)||(this._I=n.Id),N(n.IsBeginConnected)||(this._Is=n.IsBeginConnected),N(n.IsEndConnected)||(this._IsE=n.IsEndConnected),N(n.connectorType)||(this._c=n.connectorType),L(this,n,["beginConnectedShape","BeginConnectedShape","endConnectedShape","EndConnectedShape","shape","Shape"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{beginArrowheadLength:this._B,beginArrowheadStyle:this._Be,beginArrowheadWidth:this._Beg,beginConnectedSite:this._Begin,connectorType:this._c,endArrowheadLength:this._E,endArrowheadStyle:this._En,endArrowheadWidth:this._End,endConnectedSite:this._EndCo,id:this._I,isBeginConnected:this._Is,isEndConnected:this._IsE},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Line=ro;var io="ShapeFill",oo=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ShapeFill"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["foregroundColor","type","transparency"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["ForegroundColor","Type","Transparency"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!1,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"foregroundColor",{get:function(){return T("foregroundColor",this._F,io,this._isNull),this._F},set:function(e){this._F=e,I(this,"ForegroundColor",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"transparency",{get:function(){return T("transparency",this._T,io,this._isNull),this._T},set:function(e){this._T=e,I(this,"Transparency",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"type",{get:function(){return T("type",this._Ty,io,this._isNull),this._Ty},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["foregroundColor","transparency"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype.clear=function(){E(this,"Clear",0,[],0,0)},t.prototype.setSolidColor=function(e){E(this,"SetSolidColor",0,[e],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.ForegroundColor)||(this._F=n.ForegroundColor),N(n.Transparency)||(this._T=n.Transparency),N(n.Type)||(this._Ty=n.Type)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{foregroundColor:this._F,transparency:this._T,type:this._Ty},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ShapeFill=oo;var so="ShapeLineFormat",ao=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ShapeLineFormat"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["visible","color","style","weight","dashStyle","transparency"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Visible","Color","Style","Weight","DashStyle","Transparency"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return T("color",this._C,so,this._isNull),this._C},set:function(e){this._C=e,I(this,"Color",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dashStyle",{get:function(){return T("dashStyle",this._D,so,this._isNull),this._D},set:function(e){this._D=e,I(this,"DashStyle",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"style",{get:function(){return T("style",this._S,so,this._isNull),this._S},set:function(e){this._S=e,I(this,"Style",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"transparency",{get:function(){return T("transparency",this._T,so,this._isNull),this._T},set:function(e){this._T=e,I(this,"Transparency",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"visible",{get:function(){return T("visible",this._V,so,this._isNull),this._V},set:function(e){this._V=e,I(this,"Visible",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"weight",{get:function(){return T("weight",this._W,so,this._isNull),this._W},set:function(e){this._W=e,I(this,"Weight",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["visible","color","style","weight","dashStyle","transparency"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Color)||(this._C=n.Color),N(n.DashStyle)||(this._D=n.DashStyle),N(n.Style)||(this._S=n.Style),N(n.Transparency)||(this._T=n.Transparency),N(n.Visible)||(this._V=n.Visible),N(n.Weight)||(this._W=n.Weight)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{color:this._C,dashStyle:this._D,style:this._S,transparency:this._T,visible:this._V,weight:this._W},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ShapeLineFormat=ao;var lo="TextFrame",uo=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TextFrame"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["leftMargin","rightMargin","topMargin","bottomMargin","horizontalAlignment","horizontalOverflow","verticalAlignment","verticalOverflow","orientation","readingOrder","hasText","autoSizeSetting"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["LeftMargin","RightMargin","TopMargin","BottomMargin","HorizontalAlignment","HorizontalOverflow","VerticalAlignment","VerticalOverflow","Orientation","ReadingOrder","HasText","AutoSizeSetting"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0,!0,!0,!0,!0,!1,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["textRange"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"textRange",{get:function(){return this._T||(this._T=S(e.TextRange,this,"TextRange",!1,4)),this._T},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"autoSizeSetting",{get:function(){return T("autoSizeSetting",this._A,lo,this._isNull),this._A},set:function(e){this._A=e,I(this,"AutoSizeSetting",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"bottomMargin",{get:function(){return T("bottomMargin",this._B,lo,this._isNull),this._B},set:function(e){this._B=e,I(this,"BottomMargin",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"hasText",{get:function(){return T("hasText",this._H,lo,this._isNull),this._H},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"horizontalAlignment",{get:function(){return T("horizontalAlignment",this._Ho,lo,this._isNull),this._Ho},set:function(e){this._Ho=e,I(this,"HorizontalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"horizontalOverflow",{get:function(){return T("horizontalOverflow",this._Hor,lo,this._isNull),this._Hor},set:function(e){this._Hor=e,I(this,"HorizontalOverflow",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"leftMargin",{get:function(){return T("leftMargin",this._L,lo,this._isNull),this._L},set:function(e){this._L=e,I(this,"LeftMargin",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"orientation",{get:function(){return T("orientation",this._O,lo,this._isNull),this._O},set:function(e){this._O=e,I(this,"Orientation",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"readingOrder",{get:function(){return T("readingOrder",this._R,lo,this._isNull),this._R},set:function(e){this._R=e,I(this,"ReadingOrder",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"rightMargin",{get:function(){return T("rightMargin",this._Ri,lo,this._isNull),this._Ri},set:function(e){this._Ri=e,I(this,"RightMargin",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"topMargin",{get:function(){return T("topMargin",this._To,lo,this._isNull),this._To},set:function(e){this._To=e,I(this,"TopMargin",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"verticalAlignment",{get:function(){return T("verticalAlignment",this._V,lo,this._isNull),this._V},set:function(e){this._V=e,I(this,"VerticalAlignment",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"verticalOverflow",{get:function(){return T("verticalOverflow",this._Ve,lo,this._isNull),this._Ve},set:function(e){this._Ve=e,I(this,"VerticalOverflow",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["leftMargin","rightMargin","topMargin","bottomMargin","horizontalAlignment","horizontalOverflow","verticalAlignment","verticalOverflow","orientation","readingOrder","autoSizeSetting"],[],["textRange"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.deleteText=function(){E(this,"DeleteText",0,[],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.AutoSizeSetting)||(this._A=n.AutoSizeSetting),N(n.BottomMargin)||(this._B=n.BottomMargin),N(n.HasText)||(this._H=n.HasText),N(n.HorizontalAlignment)||(this._Ho=n.HorizontalAlignment),N(n.HorizontalOverflow)||(this._Hor=n.HorizontalOverflow),N(n.LeftMargin)||(this._L=n.LeftMargin),N(n.Orientation)||(this._O=n.Orientation),N(n.ReadingOrder)||(this._R=n.ReadingOrder),N(n.RightMargin)||(this._Ri=n.RightMargin),N(n.TopMargin)||(this._To=n.TopMargin),N(n.VerticalAlignment)||(this._V=n.VerticalAlignment),N(n.VerticalOverflow)||(this._Ve=n.VerticalOverflow),L(this,n,["textRange","TextRange"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{autoSizeSetting:this._A,bottomMargin:this._B,hasText:this._H,horizontalAlignment:this._Ho,horizontalOverflow:this._Hor,leftMargin:this._L,orientation:this._O,readingOrder:this._R,rightMargin:this._Ri,topMargin:this._To,verticalAlignment:this._V,verticalOverflow:this._Ve},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.TextFrame=uo;var co=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"TextRange"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["text"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Text"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["font"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"font",{get:function(){return this._F||(this._F=S(e.ShapeFont,this,"Font",!1,4)),this._F},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"text",{get:function(){return T("text",this._T,"TextRange",this._isNull),this._T},set:function(e){this._T=e,I(this,"Text",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["text"],["font"],[])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.getSubstring=function(t,n){return C(e.TextRange,this,"GetSubstring",0,[t,n],!1,!1,null,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Text)||(this._T=n.Text),L(this,n,["font","Font"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{text:this._T},{font:this._F})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.TextRange=co;var po="ShapeFont",fo=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"ShapeFont"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["size","name","color","bold","italic","underline"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Size","Name","Color","Bold","Italic","Underline"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0,!0,!0,!0,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"bold",{get:function(){return T("bold",this._B,po,this._isNull),this._B},set:function(e){this._B=e,I(this,"Bold",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return T("color",this._C,po,this._isNull),this._C},set:function(e){this._C=e,I(this,"Color",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"italic",{get:function(){return T("italic",this._I,po,this._isNull),this._I},set:function(e){this._I=e,I(this,"Italic",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"name",{get:function(){return T("name",this._N,po,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"size",{get:function(){return T("size",this._S,po,this._isNull),this._S},set:function(e){this._S=e,I(this,"Size",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"underline",{get:function(){return T("underline",this._U,po,this._isNull),this._U},set:function(e){this._U=e,I(this,"Underline",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["size","name","color","bold","italic","underline"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Bold)||(this._B=n.Bold),N(n.Color)||(this._C=n.Color),N(n.Italic)||(this._I=n.Italic),N(n.Name)||(this._N=n.Name),N(n.Size)||(this._S=n.Size),N(n.Underline)||(this._U=n.Underline)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{bold:this._B,color:this._C,italic:this._I,name:this._N,size:this._S,underline:this._U},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.ShapeFont=fo;var ho="Slicer",yo=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"Slicer"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["id","name","caption","left","top","width","height","isFilterCleared","style","sortBy"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Id","Name","Caption","Left","Top","Width","Height","IsFilterCleared","Style","SortBy"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!0,!0,!0,!0,!0,!0,!1,!0,!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_navigationPropertyNames",{get:function(){return["slicerItems","worksheet"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"slicerItems",{get:function(){return this._S||(this._S=S(e.SlicerItemCollection,this,"SlicerItems",!0,4)),this._S},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"worksheet",{get:function(){return this._Wo||(this._Wo=S(e.Worksheet,this,"Worksheet",!1,4)),this._Wo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"caption",{get:function(){return T("caption",this._C,ho,this._isNull),this._C},set:function(e){this._C=e,I(this,"Caption",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"height",{get:function(){return T("height",this._H,ho,this._isNull),this._H},set:function(e){this._H=e,I(this,"Height",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"id",{get:function(){return T("id",this._I,ho,this._isNull),this._I},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isFilterCleared",{get:function(){return T("isFilterCleared",this._Is,ho,this._isNull),this._Is},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"left",{get:function(){return T("left",this._L,ho,this._isNull),this._L},set:function(e){this._L=e,I(this,"Left",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,ho,this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"sortBy",{get:function(){return T("sortBy",this._So,ho,this._isNull),this._So},set:function(e){this._So=e,I(this,"SortBy",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"style",{get:function(){return T("style",this._St,ho,this._isNull),this._St},set:function(e){this._St=e,I(this,"Style",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"top",{get:function(){return T("top",this._T,ho,this._isNull),this._T},set:function(e){this._T=e,I(this,"Top",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"width",{get:function(){return T("width",this._W,ho,this._isNull),this._W},set:function(e){this._W=e,I(this,"Width",e,0)},enumerable:!0,configurable:!0}),n.prototype.set=function(e,t){this._recursivelySet(e,t,["name","caption","left","top","width","height","style","sortBy"],["worksheet"],["slicerItems"])},n.prototype.update=function(e){this._recursivelyUpdate(e)},n.prototype.clearFilters=function(){E(this,"ClearFilters",0,[],0,0)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.getSelectedItems=function(){return E(this,"GetSelectedItems",0,[],0,0)},n.prototype.selectItems=function(e){E(this,"SelectItems",0,[e],0,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Caption)||(this._C=n.Caption),N(n.Height)||(this._H=n.Height),N(n.Id)||(this._I=n.Id),N(n.IsFilterCleared)||(this._Is=n.IsFilterCleared),N(n.Left)||(this._L=n.Left),N(n.Name)||(this._N=n.Name),N(n.SortBy)||(this._So=n.SortBy),N(n.Style)||(this._St=n.Style),N(n.Top)||(this._T=n.Top),N(n.Width)||(this._W=n.Width),L(this,n,["slicerItems","SlicerItems","worksheet","Worksheet"])}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleIdResult=function(e){t.prototype._handleIdResult.call(this,e),x(e)||N(e.Id)||(this._I=e.Id)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{caption:this._C,height:this._H,id:this._I,isFilterCleared:this._Is,left:this._L,name:this._N,sortBy:this._So,style:this._St,top:this._T,width:this._W},{slicerItems:this._S,worksheet:this._Wo})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.Slicer=yo;var mo=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"SlicerCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"SlicerCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t,n,r){return C(e.Slicer,this,"Add",0,[t,n,r],!1,!0,null,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.Slicer,this,[t])},n.prototype.getItemAt=function(t){return C(e.Slicer,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return C(e.Slicer,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.Slicer,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.Slicer,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.Slicer,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.SlicerCollection=mo;var go="SlicerItem",_o=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"SlicerItem"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["key","name","isSelected","hasData"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Key","Name","IsSelected","HasData"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!1,!1,!0,!1]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"hasData",{get:function(){return T("hasData",this._H,go,this._isNull),this._H},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isSelected",{get:function(){return T("isSelected",this._I,go,this._isNull),this._I},set:function(e){this._I=e,I(this,"IsSelected",e,0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"key",{get:function(){return T("key",this._K,go,this._isNull),this._K},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"name",{get:function(){return T("name",this._N,go,this._isNull),this._N},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["isSelected"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.HasData)||(this._H=n.HasData),N(n.IsSelected)||(this._I=n.IsSelected),N(n.Key)||(this._K=n.Key),N(n.Name)||(this._N=n.Name)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{hasData:this._H,isSelected:this._I,key:this._K,name:this._N},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.SlicerItem=_o;var bo=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"SlicerItemCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"SlicerItemCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.SlicerItem,this,[t])},n.prototype.getItemAt=function(t){return C(e.SlicerItem,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype.getItemOrNullObject=function(t){return C(e.SlicerItem,this,"GetItemOrNullObject",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.SlicerItem,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.SlicerItem,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.SlicerItem,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.SlicerItemCollection=bo;var Oo,vo=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"Ribbon"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["activeTab"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["ActiveTab"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"activeTab",{get:function(){return T("activeTab",this._A,"Ribbon",this._isNull),this._A},set:function(e){this._A=e,I(this,"ActiveTab",e,0)},enumerable:!0,configurable:!0}),t.prototype.set=function(e,t){this._recursivelySet(e,t,["activeTab"],[],[])},t.prototype.update=function(e){this._recursivelyUpdate(e)},t.prototype.executeCommand=function(e,t){E(this,"ExecuteCommand",0,[e,t],0,0)},t.prototype.showTeachingCallout=function(e,t,n){E(this,"ShowTeachingCallout",0,[e,t,n],0,0)},t.prototype._RegisterCommandExecutedEvent=function(){E(this,"_RegisterCommandExecutedEvent",0,[],0,0)},t.prototype._UnregisterCommandExecutedEvent=function(){E(this,"_UnregisterCommandExecutedEvent",0,[],0,0)},t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.ActiveTab)||(this._A=n.ActiveTab)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},Object.defineProperty(t.prototype,"onCommandExecuted",{get:function(){var e=this;return this.m_commandExecuted||(this.m_commandExecuted=new OfficeExtension.GenericEventHandlers(this.context,this,"CommandExecuted",{eventType:2300,registerFunc:function(){return e._RegisterCommandExecutedEvent()},unregisterFunc:function(){return e._UnregisterCommandExecutedEvent()},getTargetIdFunc:function(){return OfficeExtension.Constants.eventWorkbookId},eventArgsTransformFunc:function(e){var t={type:Oo.ribbonCommandExecuted,buttonId:e.buttonId};return OfficeExtension.Utility._createPromiseFromResult(t)}})),this.m_commandExecuted},enumerable:!0,configurable:!0}),t.prototype.toJSON=function(){return k(this,{activeTab:this._A},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.Ribbon=vo,function(e){e.invalid="Invalid",e.category="Category",e.value="Value",e.series="Series"}(e.AxisType||(e.AxisType={})),function(e){e.primary="Primary",e.secondary="Secondary"}(e.AxisGroup||(e.AxisGroup={})),function(e){e.linear="Linear",e.logarithmic="Logarithmic"}(e.AxisScaleType||(e.AxisScaleType={})),function(e){e.automatic="Automatic",e.maximum="Maximum",e.minimum="Minimum",e.custom="Custom"}(e.AxisCrosses||(e.AxisCrosses={})),function(e){e.none="None",e.cross="Cross",e.inside="Inside",e.outside="Outside"}(e.AxisTickMark||(e.AxisTickMark={})),function(e){e.nextToAxis="NextToAxis",e.high="High",e.low="Low",e.none="None"}(e.AxisTickLabelPosition||(e.AxisTickLabelPosition={})),function(e){e.linear="Linear",e.exponential="Exponential",e.logarithmic="Logarithmic",e.movingAverage="MovingAverage",e.polynomial="Polynomial",e.power="Power"}(e.TrendlineType||(e.TrendlineType={})),function(e){e.invalid="Invalid",e.category="Category",e.value="Value",e.series="Series"}(e.ChartAxisType||(e.ChartAxisType={})),function(e){e.primary="Primary",e.secondary="Secondary"}(e.ChartAxisGroup||(e.ChartAxisGroup={})),function(e){e.linear="Linear",e.logarithmic="Logarithmic"}(e.ChartAxisScaleType||(e.ChartAxisScaleType={})),function(e){e.automatic="Automatic",e.maximum="Maximum",e.minimum="Minimum",e.custom="Custom"}(e.ChartAxisPosition||(e.ChartAxisPosition={})),function(e){e.none="None",e.cross="Cross",e.inside="Inside",e.outside="Outside"}(e.ChartAxisTickMark||(e.ChartAxisTickMark={})),function(e){e.done="Done",e.calculating="Calculating",e.pending="Pending"}(e.CalculationState||(e.CalculationState={})),function(e){e.nextToAxis="NextToAxis",e.high="High",e.low="Low",e.none="None"}(e.ChartAxisTickLabelPosition||(e.ChartAxisTickLabelPosition={})),function(e){e.none="None",e.hundreds="Hundreds",e.thousands="Thousands",e.tenThousands="TenThousands",e.hundredThousands="HundredThousands",e.millions="Millions",e.tenMillions="TenMillions",e.hundredMillions="HundredMillions",e.billions="Billions",e.trillions="Trillions",e.custom="Custom"}(e.ChartAxisDisplayUnit||(e.ChartAxisDisplayUnit={})),function(e){e.days="Days",e.months="Months",e.years="Years"}(e.ChartAxisTimeUnit||(e.ChartAxisTimeUnit={})),function(e){e.inclusive="Inclusive",e.exclusive="Exclusive"}(e.ChartBoxQuartileCalculation||(e.ChartBoxQuartileCalculation={})),function(e){e.automatic="Automatic",e.textAxis="TextAxis",e.dateAxis="DateAxis"}(e.ChartAxisCategoryType||(e.ChartAxisCategoryType={})),function(e){e.category="Category",e.auto="Auto",e.binWidth="BinWidth",e.binCount="BinCount"}(e.ChartBinType||(e.ChartBinType={})),function(e){e.none="None",e.continuous="Continuous",e.dash="Dash",e.dashDot="DashDot",e.dashDotDot="DashDotDot",e.dot="Dot",e.grey25="Grey25",e.grey50="Grey50",e.grey75="Grey75",e.automatic="Automatic",e.roundDot="RoundDot"}(e.ChartLineStyle||(e.ChartLineStyle={})),function(e){e.invalid="Invalid",e.none="None",e.center="Center",e.insideEnd="InsideEnd",e.insideBase="InsideBase",e.outsideEnd="OutsideEnd",e.left="Left",e.right="Right",e.top="Top",e.bottom="Bottom",e.bestFit="BestFit",e.callout="Callout"}(e.ChartDataLabelPosition||(e.ChartDataLabelPosition={})),function(e){e.both="Both",e.minusValues="MinusValues",e.plusValues="PlusValues"}(e.ChartErrorBarsInclude||(e.ChartErrorBarsInclude={})),function(e){e.fixedValue="FixedValue",e.percent="Percent",e.stDev="StDev",e.stError="StError",e.custom="Custom"}(e.ChartErrorBarsType||(e.ChartErrorBarsType={})),function(e){e.automatic="Automatic",e.dataOnly="DataOnly",e.city="City",e.county="County",e.state="State",e.country="Country",e.continent="Continent",e.world="World"}(e.ChartMapAreaLevel||(e.ChartMapAreaLevel={})),function(e){e.twoPhaseColor="TwoPhaseColor",e.threePhaseColor="ThreePhaseColor"}(e.ChartGradientStyle||(e.ChartGradientStyle={})),function(e){e.extremeValue="ExtremeValue",e.number="Number",e.percent="Percent"}(e.ChartGradientStyleType||(e.ChartGradientStyleType={})),function(e){e.automatic="Automatic",e.top="Top",e.bottom="Bottom",e.left="Left",e.right="Right"}(e.ChartTitlePosition||(e.ChartTitlePosition={})),function(e){e.invalid="Invalid",e.top="Top",e.bottom="Bottom",e.left="Left",e.right="Right",e.corner="Corner",e.custom="Custom"}(e.ChartLegendPosition||(e.ChartLegendPosition={})),function(e){e.invalid="Invalid",e.automatic="Automatic",e.none="None",e.square="Square",e.diamond="Diamond",e.triangle="Triangle",e.x="X",e.star="Star",e.dot="Dot",e.dash="Dash",e.circle="Circle",e.plus="Plus",e.picture="Picture"}(e.ChartMarkerStyle||(e.ChartMarkerStyle={})),function(e){e.automatic="Automatic",e.custom="Custom"}(e.ChartPlotAreaPosition||(e.ChartPlotAreaPosition={})),function(e){e.none="None",e.bestFit="BestFit",e.showAll="ShowAll"}(e.ChartMapLabelStrategy||(e.ChartMapLabelStrategy={})),function(e){e.automatic="Automatic",e.mercator="Mercator",e.miller="Miller",e.robinson="Robinson",e.albers="Albers"}(e.ChartMapProjectionType||(e.ChartMapProjectionType={})),function(e){e.none="None",e.banner="Banner",e.overlapping="Overlapping"}(e.ChartParentLabelStrategy||(e.ChartParentLabelStrategy={})),function(e){e.auto="Auto",e.columns="Columns",e.rows="Rows"}(e.ChartSeriesBy||(e.ChartSeriesBy={})),function(e){e.localRange="LocalRange",e.externalRange="ExternalRange",e.list="List",e.unknown="Unknown"}(e.ChartDataSourceType||(e.ChartDataSourceType={})),function(e){e.center="Center",e.left="Left",e.right="Right",e.justify="Justify",e.distributed="Distributed"}(e.ChartTextHorizontalAlignment||(e.ChartTextHorizontalAlignment={})),function(e){e.center="Center",e.bottom="Bottom",e.top="Top",e.justify="Justify",e.distributed="Distributed"}(e.ChartTextVerticalAlignment||(e.ChartTextVerticalAlignment={})),function(e){e.center="Center",e.left="Left",e.right="Right"}(e.ChartTickLabelAlignment||(e.ChartTickLabelAlignment={})),function(e){e.invalid="Invalid",e.columnClustered="ColumnClustered",e.columnStacked="ColumnStacked",e.columnStacked100="ColumnStacked100",e._3DColumnClustered="3DColumnClustered",e._3DColumnStacked="3DColumnStacked",e._3DColumnStacked100="3DColumnStacked100",e.barClustered="BarClustered",e.barStacked="BarStacked",e.barStacked100="BarStacked100",e._3DBarClustered="3DBarClustered",e._3DBarStacked="3DBarStacked",e._3DBarStacked100="3DBarStacked100",e.lineStacked="LineStacked",e.lineStacked100="LineStacked100",e.lineMarkers="LineMarkers",e.lineMarkersStacked="LineMarkersStacked",e.lineMarkersStacked100="LineMarkersStacked100",e.pieOfPie="PieOfPie",e.pieExploded="PieExploded",e._3DPieExploded="3DPieExploded",e.barOfPie="BarOfPie",e.xyscatterSmooth="XYScatterSmooth",e.xyscatterSmoothNoMarkers="XYScatterSmoothNoMarkers",e.xyscatterLines="XYScatterLines",e.xyscatterLinesNoMarkers="XYScatterLinesNoMarkers",e.areaStacked="AreaStacked",e.areaStacked100="AreaStacked100",e._3DAreaStacked="3DAreaStacked",e._3DAreaStacked100="3DAreaStacked100",e.doughnutExploded="DoughnutExploded",e.radarMarkers="RadarMarkers",e.radarFilled="RadarFilled",e.surface="Surface",e.surfaceWireframe="SurfaceWireframe",e.surfaceTopView="SurfaceTopView",e.surfaceTopViewWireframe="SurfaceTopViewWireframe",e.bubble="Bubble",e.bubble3DEffect="Bubble3DEffect",e.stockHLC="StockHLC",e.stockOHLC="StockOHLC",e.stockVHLC="StockVHLC",e.stockVOHLC="StockVOHLC",e.cylinderColClustered="CylinderColClustered",e.cylinderColStacked="CylinderColStacked",e.cylinderColStacked100="CylinderColStacked100",e.cylinderBarClustered="CylinderBarClustered",e.cylinderBarStacked="CylinderBarStacked",e.cylinderBarStacked100="CylinderBarStacked100",e.cylinderCol="CylinderCol",e.coneColClustered="ConeColClustered",e.coneColStacked="ConeColStacked",e.coneColStacked100="ConeColStacked100",e.coneBarClustered="ConeBarClustered",e.coneBarStacked="ConeBarStacked",e.coneBarStacked100="ConeBarStacked100",e.coneCol="ConeCol",e.pyramidColClustered="PyramidColClustered",e.pyramidColStacked="PyramidColStacked",e.pyramidColStacked100="PyramidColStacked100",e.pyramidBarClustered="PyramidBarClustered",e.pyramidBarStacked="PyramidBarStacked",e.pyramidBarStacked100="PyramidBarStacked100",e.pyramidCol="PyramidCol",e._3DColumn="3DColumn",e.line="Line",e._3DLine="3DLine",e._3DPie="3DPie",e.pie="Pie",e.xyscatter="XYScatter",e._3DArea="3DArea",e.area="Area",e.doughnut="Doughnut",e.radar="Radar",e.histogram="Histogram",e.boxwhisker="Boxwhisker",e.pareto="Pareto",e.regionMap="RegionMap",e.treemap="Treemap",e.waterfall="Waterfall",e.sunburst="Sunburst",e.funnel="Funnel"}(e.ChartType||(e.ChartType={})),function(e){e.none="None",e.single="Single"}(e.ChartUnderlineStyle||(e.ChartUnderlineStyle={})),function(e){e.notPlotted="NotPlotted",e.zero="Zero",e.interplotted="Interplotted"}(e.ChartDisplayBlanksAs||(e.ChartDisplayBlanksAs={})),function(e){e.rows="Rows",e.columns="Columns"}(e.ChartPlotBy||(e.ChartPlotBy={})),function(e){e.splitByPosition="SplitByPosition",e.splitByValue="SplitByValue",e.splitByPercentValue="SplitByPercentValue",e.splitByCustomSplit="SplitByCustomSplit"}(e.ChartSplitType||(e.ChartSplitType={})),function(e){e.colorfulPalette1="ColorfulPalette1",e.colorfulPalette2="ColorfulPalette2",e.colorfulPalette3="ColorfulPalette3",e.colorfulPalette4="ColorfulPalette4",e.monochromaticPalette1="MonochromaticPalette1",e.monochromaticPalette2="MonochromaticPalette2",e.monochromaticPalette3="MonochromaticPalette3",e.monochromaticPalette4="MonochromaticPalette4",e.monochromaticPalette5="MonochromaticPalette5",e.monochromaticPalette6="MonochromaticPalette6",e.monochromaticPalette7="MonochromaticPalette7",e.monochromaticPalette8="MonochromaticPalette8",e.monochromaticPalette9="MonochromaticPalette9",e.monochromaticPalette10="MonochromaticPalette10",e.monochromaticPalette11="MonochromaticPalette11",e.monochromaticPalette12="MonochromaticPalette12",e.monochromaticPalette13="MonochromaticPalette13"}(e.ChartColorScheme||(e.ChartColorScheme={})),function(e){e.linear="Linear",e.exponential="Exponential",e.logarithmic="Logarithmic",e.movingAverage="MovingAverage",e.polynomial="Polynomial",e.power="Power"}(e.ChartTrendlineType||(e.ChartTrendlineType={})),function(e){e.bringToFront="BringToFront",e.bringForward="BringForward",e.sendToBack="SendToBack",e.sendBackward="SendBackward"}(e.ShapeZOrder||(e.ShapeZOrder={})),function(e){e.unsupported="Unsupported",e.image="Image",e.geometricShape="GeometricShape",e.group="Group",e.line="Line"}(e.ShapeType||(e.ShapeType={})),function(e){e.currentSize="CurrentSize",e.originalSize="OriginalSize"}(e.ShapeScaleType||(e.ShapeScaleType={})),function(e){e.scaleFromTopLeft="ScaleFromTopLeft",e.scaleFromMiddle="ScaleFromMiddle",e.scaleFromBottomRight="ScaleFromBottomRight"}(e.ShapeScaleFrom||(e.ShapeScaleFrom={})),function(e){e.noFill="NoFill",e.solid="Solid",e.gradient="Gradient",e.pattern="Pattern",e.pictureAndTexture="PictureAndTexture",e.mixed="Mixed"}(e.ShapeFillType||(e.ShapeFillType={})),function(e){e.none="None",e.single="Single",e.double="Double",e.heavy="Heavy",e.dotted="Dotted",e.dottedHeavy="DottedHeavy",e.dash="Dash",e.dashHeavy="DashHeavy",e.dashLong="DashLong",e.dashLongHeavy="DashLongHeavy",e.dotDash="DotDash",e.dotDashHeavy="DotDashHeavy",e.dotDotDash="DotDotDash",e.dotDotDashHeavy="DotDotDashHeavy",e.wavy="Wavy",e.wavyHeavy="WavyHeavy",e.wavyDouble="WavyDouble"}(e.ShapeFontUnderlineStyle||(e.ShapeFontUnderlineStyle={})),function(e){e.unknown="UNKNOWN",e.bmp="BMP",e.jpeg="JPEG",e.gif="GIF",e.png="PNG",e.svg="SVG"}(e.PictureFormat||(e.PictureFormat={})),function(e){e.single="Single",e.thickBetweenThin="ThickBetweenThin",e.thickThin="ThickThin",e.thinThick="ThinThick",e.thinThin="ThinThin"}(e.ShapeLineStyle||(e.ShapeLineStyle={})),function(e){e.dash="Dash",e.dashDot="DashDot",e.dashDotDot="DashDotDot",e.longDash="LongDash",e.longDashDot="LongDashDot",e.roundDot="RoundDot",e.solid="Solid",e.squareDot="SquareDot",e.longDashDotDot="LongDashDotDot",e.systemDash="SystemDash",e.systemDot="SystemDot",e.systemDashDot="SystemDashDot"}(e.ShapeLineDashStyle||(e.ShapeLineDashStyle={})),function(e){e.short="Short",e.medium="Medium",e.long="Long"}(e.ArrowheadLength||(e.ArrowheadLength={})),function(e){e.none="None",e.triangle="Triangle",e.stealth="Stealth",e.diamond="Diamond",e.oval="Oval",e.open="Open"}(e.ArrowheadStyle||(e.ArrowheadStyle={})),function(e){e.narrow="Narrow",e.medium="Medium",e.wide="Wide"}(e.ArrowheadWidth||(e.ArrowheadWidth={})),function(e){e.range="Range",e.table="Table",e.text="Text"}(e.BindingType||(e.BindingType={})),function(e){e.edgeTop="EdgeTop",e.edgeBottom="EdgeBottom",e.edgeLeft="EdgeLeft",e.edgeRight="EdgeRight",e.insideVertical="InsideVertical",e.insideHorizontal="InsideHorizontal",e.diagonalDown="DiagonalDown",e.diagonalUp="DiagonalUp"}(e.BorderIndex||(e.BorderIndex={})),function(e){e.none="None",e.continuous="Continuous",e.dash="Dash",e.dashDot="DashDot",e.dashDotDot="DashDotDot",e.dot="Dot",e.double="Double",e.slantDashDot="SlantDashDot"}(e.BorderLineStyle||(e.BorderLineStyle={})),function(e){e.hairline="Hairline",e.thin="Thin",e.medium="Medium",e.thick="Thick"}(e.BorderWeight||(e.BorderWeight={})),function(e){e.automatic="Automatic",e.automaticExceptTables="AutomaticExceptTables",e.manual="Manual"}(e.CalculationMode||(e.CalculationMode={})),function(e){e.recalculate="Recalculate",e.full="Full",e.fullRebuild="FullRebuild"}(e.CalculationType||(e.CalculationType={})),function(e){e.all="All",e.formats="Formats",e.contents="Contents",e.hyperlinks="Hyperlinks",e.removeHyperlinks="RemoveHyperlinks"}(e.ClearApplyTo||(e.ClearApplyTo={})),function(e){e.column="Column",e.bar="Bar",e.line="Line",e.area="Area",e.pie="Pie",e.donut="Donut",e.scatter="Scatter",e.bubble="Bubble",e.statistical="Statistical",e.stock="Stock",e.combo="Combo",e.hierarchy="Hierarchy",e.surface="Surface",e.map="Map",e.funnel="Funnel",e.radar="Radar",e.waterfall="Waterfall",e.threeD="ThreeD",e.other="Other"}(e.VisualCategory||(e.VisualCategory={})),function(e){e.object="Object",e.collection="Collection",e.string="String",e.double="Double",e.int="Int",e.bool="Bool",e.enum="Enum",e.color="Color"}(e.VisualPropertyType||(e.VisualPropertyType={})),function(e){e.dataChange="DataChange",e.propertyChange="PropertyChange",e.genericChange="GenericChange",e.selectionChange="SelectionChange"}(e.VisualChangeType||(e.VisualChangeType={})),function(e){e.writeOnly="WriteOnly",e.readOnly="ReadOnly",e.hideEntireSubtreeUI="HideEntireSubtreeUI",e.hideMeButShowChildrenUI="HideMeButShowChildrenUI",e.expandableUI="ExpandableUI",e.nextPropOnSameLine="NextPropOnSameLine",e.hideLabel="HideLabel",e.showResetUI="ShowResetUI",e.hasOwnExpandableSection="HasOwnExpandableSection",e.nextPropOnSameLineFOTP="NextPropOnSameLineFOTP",e.showResetUIFOTP="ShowResetUIFOTP",e.untransferable="Untransferable"}(e.BoolMetaPropertyType||(e.BoolMetaPropertyType={})),function(e){e.automatic="Automatic",e.none="None",e.cellMidPoint="CellMidPoint"}(e.ConditionalDataBarAxisFormat||(e.ConditionalDataBarAxisFormat={})),function(e){e.context="Context",e.leftToRight="LeftToRight",e.rightToLeft="RightToLeft"}(e.ConditionalDataBarDirection||(e.ConditionalDataBarDirection={})),function(e){e.top="Top",e.bottom="Bottom"}(e.ConditionalFormatDirection||(e.ConditionalFormatDirection={})),function(e){e.custom="Custom",e.dataBar="DataBar",e.colorScale="ColorScale",e.iconSet="IconSet",e.topBottom="TopBottom",e.presetCriteria="PresetCriteria",e.containsText="ContainsText",e.cellValue="CellValue"}(e.ConditionalFormatType||(e.ConditionalFormatType={})),function(e){e.invalid="Invalid",e.automatic="Automatic",e.lowestValue="LowestValue",e.highestValue="HighestValue",e.number="Number",e.percent="Percent",e.formula="Formula",e.percentile="Percentile"}(e.ConditionalFormatRuleType||(e.ConditionalFormatRuleType={})),function(e){e.invalid="Invalid",e.number="Number",e.percent="Percent",e.formula="Formula",e.percentile="Percentile"}(e.ConditionalFormatIconRuleType||(e.ConditionalFormatIconRuleType={})),function(e){e.invalid="Invalid",e.lowestValue="LowestValue",e.highestValue="HighestValue",e.number="Number",e.percent="Percent",e.formula="Formula",e.percentile="Percentile"}(e.ConditionalFormatColorCriterionType||(e.ConditionalFormatColorCriterionType={})),function(e){e.invalid="Invalid",e.topItems="TopItems",e.topPercent="TopPercent",e.bottomItems="BottomItems",e.bottomPercent="BottomPercent"}(e.ConditionalTopBottomCriterionType||(e.ConditionalTopBottomCriterionType={})),function(e){e.invalid="Invalid",e.blanks="Blanks",e.nonBlanks="NonBlanks",e.errors="Errors",e.nonErrors="NonErrors",e.yesterday="Yesterday",e.today="Today",e.tomorrow="Tomorrow",e.lastSevenDays="LastSevenDays",e.lastWeek="LastWeek",e.thisWeek="ThisWeek",e.nextWeek="NextWeek",e.lastMonth="LastMonth",e.thisMonth="ThisMonth",e.nextMonth="NextMonth",e.aboveAverage="AboveAverage",e.belowAverage="BelowAverage",e.equalOrAboveAverage="EqualOrAboveAverage",e.equalOrBelowAverage="EqualOrBelowAverage",e.oneStdDevAboveAverage="OneStdDevAboveAverage",e.oneStdDevBelowAverage="OneStdDevBelowAverage",e.twoStdDevAboveAverage="TwoStdDevAboveAverage",e.twoStdDevBelowAverage="TwoStdDevBelowAverage",e.threeStdDevAboveAverage="ThreeStdDevAboveAverage",e.threeStdDevBelowAverage="ThreeStdDevBelowAverage",e.uniqueValues="UniqueValues",e.duplicateValues="DuplicateValues"}(e.ConditionalFormatPresetCriterion||(e.ConditionalFormatPresetCriterion={})),function(e){e.invalid="Invalid",e.contains="Contains",e.notContains="NotContains",e.beginsWith="BeginsWith",e.endsWith="EndsWith"}(e.ConditionalTextOperator||(e.ConditionalTextOperator={})),function(e){e.invalid="Invalid",e.between="Between",e.notBetween="NotBetween",e.equalTo="EqualTo",e.notEqualTo="NotEqualTo",e.greaterThan="GreaterThan",e.lessThan="LessThan",e.greaterThanOrEqual="GreaterThanOrEqual",e.lessThanOrEqual="LessThanOrEqual"}(e.ConditionalCellValueOperator||(e.ConditionalCellValueOperator={})),function(e){e.invalid="Invalid",e.greaterThan="GreaterThan",e.greaterThanOrEqual="GreaterThanOrEqual"}(e.ConditionalIconCriterionOperator||(e.ConditionalIconCriterionOperator={})),function(e){e.edgeTop="EdgeTop",e.edgeBottom="EdgeBottom",e.edgeLeft="EdgeLeft",e.edgeRight="EdgeRight"}(e.ConditionalRangeBorderIndex||(e.ConditionalRangeBorderIndex={})),function(e){e.none="None",e.continuous="Continuous",e.dash="Dash",e.dashDot="DashDot",e.dashDotDot="DashDotDot",e.dot="Dot"}(e.ConditionalRangeBorderLineStyle||(e.ConditionalRangeBorderLineStyle={})),function(e){e.none="None",e.single="Single",e.double="Double"}(e.ConditionalRangeFontUnderlineStyle||(e.ConditionalRangeFontUnderlineStyle={})),function(e){e.invalid="Invalid",e.script="Script",e.webService="WebService"}(e.CustomFunctionType||(e.CustomFunctionType={})),function(e){e.invalid="Invalid",e.openApi="OpenApi"}(e.CustomFunctionMetadataFormat||(e.CustomFunctionMetadataFormat={})),function(e){e.none="None",e.wholeNumber="WholeNumber",e.decimal="Decimal",e.list="List",e.date="Date",e.time="Time",e.textLength="TextLength",e.custom="Custom",e.inconsistent="Inconsistent",e.mixedCriteria="MixedCriteria"}(e.DataValidationType||(e.DataValidationType={})),function(e){e.between="Between",e.notBetween="NotBetween",e.equalTo="EqualTo",e.notEqualTo="NotEqualTo",e.greaterThan="GreaterThan",e.lessThan="LessThan",e.greaterThanOrEqualTo="GreaterThanOrEqualTo",e.lessThanOrEqualTo="LessThanOrEqualTo"}(e.DataValidationOperator||(e.DataValidationOperator={})),function(e){e.stop="Stop",e.warning="Warning",e.information="Information"}(e.DataValidationAlertStyle||(e.DataValidationAlertStyle={})),function(e){e.up="Up",e.left="Left"}(e.DeleteShiftDirection||(e.DeleteShiftDirection={})),function(e){e.unknown="Unknown",e.aboveAverage="AboveAverage",e.allDatesInPeriodApril="AllDatesInPeriodApril",e.allDatesInPeriodAugust="AllDatesInPeriodAugust",e.allDatesInPeriodDecember="AllDatesInPeriodDecember",e.allDatesInPeriodFebruray="AllDatesInPeriodFebruray",e.allDatesInPeriodJanuary="AllDatesInPeriodJanuary",e.allDatesInPeriodJuly="AllDatesInPeriodJuly",e.allDatesInPeriodJune="AllDatesInPeriodJune",e.allDatesInPeriodMarch="AllDatesInPeriodMarch",e.allDatesInPeriodMay="AllDatesInPeriodMay",e.allDatesInPeriodNovember="AllDatesInPeriodNovember",e.allDatesInPeriodOctober="AllDatesInPeriodOctober",e.allDatesInPeriodQuarter1="AllDatesInPeriodQuarter1",e.allDatesInPeriodQuarter2="AllDatesInPeriodQuarter2",e.allDatesInPeriodQuarter3="AllDatesInPeriodQuarter3",e.allDatesInPeriodQuarter4="AllDatesInPeriodQuarter4",e.allDatesInPeriodSeptember="AllDatesInPeriodSeptember",e.belowAverage="BelowAverage",e.lastMonth="LastMonth",e.lastQuarter="LastQuarter",e.lastWeek="LastWeek",e.lastYear="LastYear",e.nextMonth="NextMonth",e.nextQuarter="NextQuarter",e.nextWeek="NextWeek",e.nextYear="NextYear",e.thisMonth="ThisMonth",e.thisQuarter="ThisQuarter",e.thisWeek="ThisWeek",e.thisYear="ThisYear",e.today="Today",e.tomorrow="Tomorrow",e.yearToDate="YearToDate",e.yesterday="Yesterday"}(e.DynamicFilterCriteria||(e.DynamicFilterCriteria={})),function(e){e.year="Year",e.month="Month",e.day="Day",e.hour="Hour",e.minute="Minute",e.second="Second"}(e.FilterDatetimeSpecificity||(e.FilterDatetimeSpecificity={})),function(e){e.bottomItems="BottomItems",e.bottomPercent="BottomPercent",e.cellColor="CellColor",e.dynamic="Dynamic",e.fontColor="FontColor",e.values="Values",e.topItems="TopItems",e.topPercent="TopPercent",e.icon="Icon",e.custom="Custom"}(e.FilterOn||(e.FilterOn={})),function(e){e.and="And",e.or="Or"}(e.FilterOperator||(e.FilterOperator={})),function(e){e.general="General",e.left="Left",e.center="Center",e.right="Right",e.fill="Fill",e.justify="Justify",e.centerAcrossSelection="CenterAcrossSelection",e.distributed="Distributed"}(e.HorizontalAlignment||(e.HorizontalAlignment={})),function(e){e.invalid="Invalid",e.threeArrows="ThreeArrows",e.threeArrowsGray="ThreeArrowsGray",e.threeFlags="ThreeFlags",e.threeTrafficLights1="ThreeTrafficLights1",e.threeTrafficLights2="ThreeTrafficLights2",e.threeSigns="ThreeSigns",e.threeSymbols="ThreeSymbols",e.threeSymbols2="ThreeSymbols2",e.fourArrows="FourArrows",e.fourArrowsGray="FourArrowsGray",e.fourRedToBlack="FourRedToBlack",e.fourRating="FourRating",e.fourTrafficLights="FourTrafficLights",e.fiveArrows="FiveArrows",e.fiveArrowsGray="FiveArrowsGray",e.fiveRating="FiveRating",e.fiveQuarters="FiveQuarters",e.threeStars="ThreeStars",e.threeTriangles="ThreeTriangles",e.fiveBoxes="FiveBoxes"}(e.IconSet||(e.IconSet={})),function(e){e.fit="Fit",e.fitAndCenter="FitAndCenter",e.fill="Fill"}(e.ImageFittingMode||(e.ImageFittingMode={})),function(e){e.down="Down",e.right="Right"}(e.InsertShiftDirection||(e.InsertShiftDirection={})),function(e){e.worksheet="Worksheet",e.workbook="Workbook"}(e.NamedItemScope||(e.NamedItemScope={})),function(e){e.string="String",e.integer="Integer",e.double="Double",e.boolean="Boolean",e.range="Range",e.error="Error",e.array="Array"}(e.NamedItemType||(e.NamedItemType={})),function(e){e.none="None",e.single="Single",e.double="Double",e.singleAccountant="SingleAccountant",e.doubleAccountant="DoubleAccountant"}(e.RangeUnderlineStyle||(e.RangeUnderlineStyle={})),function(e){e.visible="Visible",e.hidden="Hidden",e.veryHidden="VeryHidden"}(e.SheetVisibility||(e.SheetVisibility={})),function(e){e.unknown="Unknown",e.thisLocalAddin="ThisLocalAddin"}(e.EventTriggerSource||(e.EventTriggerSource={})),function(e){e.unknown="Unknown",e.empty="Empty",e.string="String",e.integer="Integer",e.double="Double",e.boolean="Boolean",e.error="Error",e.richValue="RichValue"}(e.RangeValueType||(e.RangeValueType={})),function(e){e.left="Left",e.right="Right",e.up="Up",e.down="Down"}(e.KeyboardDirection||(e.KeyboardDirection={})),function(e){e.forward="Forward",e.backwards="Backwards"}(e.SearchDirection||(e.SearchDirection={})),function(e){e.rows="Rows",e.columns="Columns"}(e.SortOrientation||(e.SortOrientation={})),function(e){e.value="Value",e.cellColor="CellColor",e.fontColor="FontColor",e.icon="Icon"}(e.SortOn||(e.SortOn={})),function(e){e.normal="Normal",e.textAsNumber="TextAsNumber"}(e.SortDataOption||(e.SortDataOption={})),function(e){e.pinYin="PinYin",e.strokeCount="StrokeCount"}(e.SortMethod||(e.SortMethod={})),function(e){e.top="Top",e.center="Center",e.bottom="Bottom",e.justify="Justify",e.distributed="Distributed"}(e.VerticalAlignment||(e.VerticalAlignment={})),function(e){e.none="None",e.shiftCellLeft="ShiftCellLeft",e.shiftCellUp="ShiftCellUp",e.shiftCellRight="ShiftCellRight",e.shiftCellDown="ShiftCellDown"}(e.InsertDeleteCellsShiftDirection||(e.InsertDeleteCellsShiftDirection={})),function(e){e.number="Number",e.boolean="Boolean",e.date="Date",e.string="String",e.float="Float"}(e.DocumentPropertyType||(e.DocumentPropertyType={})),function(e){e.local="Local",e.remote="Remote"}(e.EventSource||(e.EventSource={})),function(e){e.unknown="Unknown",e.rangeEdited="RangeEdited",e.rowInserted="RowInserted",e.rowDeleted="RowDeleted",e.columnInserted="ColumnInserted",e.columnDeleted="ColumnDeleted",e.cellInserted="CellInserted",e.cellDeleted="CellDeleted"}(e.DataChangeType||(e.DataChangeType={})),function(e){e.unhidden="Unhidden",e.hidden="Hidden"}(e.RowHiddenChangeType||(e.RowHiddenChangeType={})),function(e){e.commentEdited="CommentEdited",e.commentResolved="CommentResolved",e.commentReopened="CommentReopened",e.replyAdded="ReplyAdded",e.replyDeleted="ReplyDeleted",e.replyEdited="ReplyEdited"}(e.CommentChangeType||(e.CommentChangeType={})),function(e){e.worksheetChanged="WorksheetChanged",e.worksheetSelectionChanged="WorksheetSelectionChanged",e.worksheetAdded="WorksheetAdded",e.worksheetActivated="WorksheetActivated",e.worksheetDeactivated="WorksheetDeactivated",e.tableChanged="TableChanged",e.tableSelectionChanged="TableSelectionChanged",e.worksheetDeleted="WorksheetDeleted",e.chartAdded="ChartAdded",e.chartActivated="ChartActivated",e.chartDeactivated="ChartDeactivated",e.chartDeleted="ChartDeleted",e.worksheetCalculated="WorksheetCalculated",e.visualSelectionChanged="VisualSelectionChanged",e.agaveVisualUpdate="AgaveVisualUpdate",e.tableAdded="TableAdded",e.tableDeleted="TableDeleted",e.tableFiltered="TableFiltered",e.worksheetFiltered="WorksheetFiltered",e.shapeActivated="ShapeActivated",e.shapeDeactivated="ShapeDeactivated",e.visualChange="VisualChange",e.workbookAutoSaveSettingChanged="WorkbookAutoSaveSettingChanged",e.worksheetFormatChanged="WorksheetFormatChanged",e.wacoperationEvent="WACOperationEvent",e.ribbonCommandExecuted="RibbonCommandExecuted",e.worksheetRowSorted="WorksheetRowSorted",e.worksheetColumnSorted="WorksheetColumnSorted",e.worksheetSingleClicked="WorksheetSingleClicked",e.worksheetRowHiddenChanged="WorksheetRowHiddenChanged",e.recordingStateChangedEvent="RecordingStateChangedEvent",e.commentAdded="CommentAdded",e.commentDeleted="CommentDeleted",e.commentChanged="CommentChanged",e.linkedDataTypeRefreshRequestCompleted="LinkedDataTypeRefreshRequestCompleted",e.linkedDataTypeRefreshModeChanged="LinkedDataTypeRefreshModeChanged",e.linkedDataTypeLinkedDataTypeAdded="LinkedDataTypeLinkedDataTypeAdded",e.worksheetFormulaChanged="WorksheetFormulaChanged",e.workbookActivated="WorkbookActivated",e.linkedWorkbookWorkbookLinksChanged="LinkedWorkbookWorkbookLinksChanged",e.linkedWorkbookRefreshCompleted="LinkedWorkbookRefreshCompleted",e.worksheetProtectionChanged="WorksheetProtectionChanged",e.worksheetNameChanged="WorksheetNameChanged",e.worksheetVisibilityChanged="WorksheetVisibilityChanged",e.worksheetMoved="WorksheetMoved",e.lineageActivityUpdateAvailable="LineageActivityUpdateAvailable",e.lineageActivityFilterUpdated="LineageActivityFilterUpdated",e.lineageActivityLoadCompleted="LineageActivityLoadCompleted",e.shapeAdded="ShapeAdded",e.shapeDeleted="ShapeDeleted",e.pivotTableAdded="PivotTableAdded",e.pivotTableDeleted="PivotTableDeleted",e.namedItemAdded="NamedItemAdded",e.namedItemDeleted="NamedItemDeleted",e.worksheetNavigationObjectChanged="WorksheetNavigationObjectChanged",e.workbookNavigationObjectChanged="WorkbookNavigationObjectChanged",e.worksheetRowHeightChanged="WorksheetRowHeightChanged",e.workbookFormulaReferenceStyleChanged="WorkbookFormulaReferenceStyleChanged",e.guidedReapplyManagerUpdateAvailable="GuidedReapplyManagerUpdateAvailable"}(Oo=e.EventType||(e.EventType={})),function(e){e.title="Title",e.subject="Subject",e.author="Author",e.keywords="Keywords",e.comments="Comments",e.template="Template",e.lastAuth="LastAuth",e.revision="Revision",e.appName="AppName",e.lastPrint="LastPrint",e.creation="Creation",e.lastSave="LastSave",e.category="Category",e.format="Format",e.manager="Manager",e.company="Company"}(e.DocumentPropertyItem||(e.DocumentPropertyItem={})),function(e){e.atTop="AtTop",e.atBottom="AtBottom",e.off="Off"}(e.SubtotalLocationType||(e.SubtotalLocationType={})),function(e){e.compact="Compact",e.tabular="Tabular",e.outline="Outline"}(e.PivotLayoutType||(e.PivotLayoutType={})),function(e){e.normal="Normal",e.unlocked="Unlocked",e.none="None"}(e.ProtectionSelectionMode||(e.ProtectionSelectionMode={})),function(e){e.portrait="Portrait",e.landscape="Landscape"}(e.PageOrientation||(e.PageOrientation={})),function(e){e.letter="Letter",e.letterSmall="LetterSmall",e.tabloid="Tabloid",e.ledger="Ledger",e.legal="Legal",e.statement="Statement",e.executive="Executive",e.a3="A3",e.a4="A4",e.a4Small="A4Small",e.a5="A5",e.b4="B4",e.b5="B5",e.folio="Folio",e.quatro="Quatro",e.paper10x14="Paper10x14",e.paper11x17="Paper11x17",e.note="Note",e.envelope9="Envelope9",e.envelope10="Envelope10",e.envelope11="Envelope11",e.envelope12="Envelope12",e.envelope14="Envelope14",e.csheet="Csheet",e.dsheet="Dsheet",e.esheet="Esheet",e.envelopeDL="EnvelopeDL",e.envelopeC5="EnvelopeC5",e.envelopeC3="EnvelopeC3",e.envelopeC4="EnvelopeC4",e.envelopeC6="EnvelopeC6",e.envelopeC65="EnvelopeC65",e.envelopeB4="EnvelopeB4",e.envelopeB5="EnvelopeB5",e.envelopeB6="EnvelopeB6",e.envelopeItaly="EnvelopeItaly",e.envelopeMonarch="EnvelopeMonarch",e.envelopePersonal="EnvelopePersonal",e.fanfoldUS="FanfoldUS",e.fanfoldStdGerman="FanfoldStdGerman",e.fanfoldLegalGerman="FanfoldLegalGerman"}(e.PaperType||(e.PaperType={})),function(e){e.context="Context",e.leftToRight="LeftToRight",e.rightToLeft="RightToLeft"}(e.ReadingOrder||(e.ReadingOrder={})),function(e){e.normal="Normal",e.comma="Comma",e.currency="Currency",e.percent="Percent",e.wholeComma="WholeComma",e.wholeDollar="WholeDollar",e.hlink="Hlink",e.hlinkTrav="HlinkTrav",e.note="Note",e.warningText="WarningText",e.emphasis1="Emphasis1",e.emphasis2="Emphasis2",e.emphasis3="Emphasis3",e.sheetTitle="SheetTitle",e.heading1="Heading1",e.heading2="Heading2",e.heading3="Heading3",e.heading4="Heading4",e.input="Input",e.output="Output",e.calculation="Calculation",e.checkCell="CheckCell",e.linkedCell="LinkedCell",e.total="Total",e.good="Good",e.bad="Bad",e.neutral="Neutral",e.accent1="Accent1",e.accent1_20="Accent1_20",e.accent1_40="Accent1_40",e.accent1_60="Accent1_60",e.accent2="Accent2",e.accent2_20="Accent2_20",e.accent2_40="Accent2_40",e.accent2_60="Accent2_60",e.accent3="Accent3",e.accent3_20="Accent3_20",e.accent3_40="Accent3_40",e.accent3_60="Accent3_60",e.accent4="Accent4",e.accent4_20="Accent4_20",e.accent4_40="Accent4_40",e.accent4_60="Accent4_60",e.accent5="Accent5",e.accent5_20="Accent5_20",e.accent5_40="Accent5_40",e.accent5_60="Accent5_60",e.accent6="Accent6",e.accent6_20="Accent6_20",e.accent6_40="Accent6_40",e.accent6_60="Accent6_60",e.explanatoryText="ExplanatoryText"}(e.BuiltInStyle||(e.BuiltInStyle={})),function(e){e.asDisplayed="AsDisplayed",e.blank="Blank",e.dash="Dash",e.notAvailable="NotAvailable"}(e.PrintErrorType||(e.PrintErrorType={})),function(e){e.none="None",e.before="Before",e.after="After",e.beginning="Beginning",e.end="End"}(e.WorksheetPositionType||(e.WorksheetPositionType={})),function(e){e.noComments="NoComments",e.endSheet="EndSheet",e.inPlace="InPlace"}(e.PrintComments||(e.PrintComments={})),function(e){e.downThenOver="DownThenOver",e.overThenDown="OverThenDown"}(e.PrintOrder||(e.PrintOrder={})),function(e){e.points="Points",e.inches="Inches",e.centimeters="Centimeters"}(e.PrintMarginUnit||(e.PrintMarginUnit={})),function(e){e.default="Default",e.firstAndDefault="FirstAndDefault",e.oddAndEven="OddAndEven",e.firstOddAndEven="FirstOddAndEven"}(e.HeaderFooterState||(e.HeaderFooterState={})),function(e){e.fillDefault="FillDefault",e.fillCopy="FillCopy",e.fillSeries="FillSeries",e.fillFormats="FillFormats",e.fillValues="FillValues",e.fillDays="FillDays",e.fillWeekdays="FillWeekdays",e.fillMonths="FillMonths",e.fillYears="FillYears",e.linearTrend="LinearTrend",e.growthTrend="GrowthTrend",e.flashFill="FlashFill"}(e.AutoFillType||(e.AutoFillType={})),function(e){e.byRows="ByRows",e.byColumns="ByColumns"}(e.GroupOption||(e.GroupOption={})),function(e){e.all="All",e.formulas="Formulas",e.values="Values",e.formats="Formats",e.link="Link"}(e.RangeCopyType||(e.RangeCopyType={})),function(e){e.none="None",e.validLinkedData="ValidLinkedData",e.disambiguationNeeded="DisambiguationNeeded",e.brokenLinkedData="BrokenLinkedData",e.fetchingData="FetchingData"}(e.LinkedDataTypeState||(e.LinkedDataTypeState={})),function(e){e.lineInverse="LineInverse",e.triangle="Triangle",e.rightTriangle="RightTriangle",e.rectangle="Rectangle",e.diamond="Diamond",e.parallelogram="Parallelogram",e.trapezoid="Trapezoid",e.nonIsoscelesTrapezoid="NonIsoscelesTrapezoid",e.pentagon="Pentagon",e.hexagon="Hexagon",e.heptagon="Heptagon",e.octagon="Octagon",e.decagon="Decagon",e.dodecagon="Dodecagon",e.star4="Star4",e.star5="Star5",e.star6="Star6",e.star7="Star7",e.star8="Star8",e.star10="Star10",e.star12="Star12",e.star16="Star16",e.star24="Star24",e.star32="Star32",e.roundRectangle="RoundRectangle",e.round1Rectangle="Round1Rectangle",e.round2SameRectangle="Round2SameRectangle",e.round2DiagonalRectangle="Round2DiagonalRectangle",e.snipRoundRectangle="SnipRoundRectangle",e.snip1Rectangle="Snip1Rectangle",e.snip2SameRectangle="Snip2SameRectangle",e.snip2DiagonalRectangle="Snip2DiagonalRectangle",e.plaque="Plaque",e.ellipse="Ellipse",e.teardrop="Teardrop",e.homePlate="HomePlate",e.chevron="Chevron",e.pieWedge="PieWedge",e.pie="Pie",e.blockArc="BlockArc",e.donut="Donut",e.noSmoking="NoSmoking",e.rightArrow="RightArrow",e.leftArrow="LeftArrow",e.upArrow="UpArrow",e.downArrow="DownArrow",e.stripedRightArrow="StripedRightArrow",e.notchedRightArrow="NotchedRightArrow",e.bentUpArrow="BentUpArrow",e.leftRightArrow="LeftRightArrow",e.upDownArrow="UpDownArrow",e.leftUpArrow="LeftUpArrow",e.leftRightUpArrow="LeftRightUpArrow",e.quadArrow="QuadArrow",e.leftArrowCallout="LeftArrowCallout",e.rightArrowCallout="RightArrowCallout",e.upArrowCallout="UpArrowCallout",e.downArrowCallout="DownArrowCallout",e.leftRightArrowCallout="LeftRightArrowCallout",e.upDownArrowCallout="UpDownArrowCallout",e.quadArrowCallout="QuadArrowCallout",e.bentArrow="BentArrow",e.uturnArrow="UturnArrow",e.circularArrow="CircularArrow",e.leftCircularArrow="LeftCircularArrow",e.leftRightCircularArrow="LeftRightCircularArrow",e.curvedRightArrow="CurvedRightArrow",e.curvedLeftArrow="CurvedLeftArrow",e.curvedUpArrow="CurvedUpArrow",e.curvedDownArrow="CurvedDownArrow",e.swooshArrow="SwooshArrow",e.cube="Cube",e.can="Can",e.lightningBolt="LightningBolt",e.heart="Heart",e.sun="Sun",e.moon="Moon",e.smileyFace="SmileyFace",e.irregularSeal1="IrregularSeal1",e.irregularSeal2="IrregularSeal2",e.foldedCorner="FoldedCorner",e.bevel="Bevel",e.frame="Frame",e.halfFrame="HalfFrame",e.corner="Corner",e.diagonalStripe="DiagonalStripe",e.chord="Chord",e.arc="Arc",e.leftBracket="LeftBracket",e.rightBracket="RightBracket",e.leftBrace="LeftBrace",e.rightBrace="RightBrace",e.bracketPair="BracketPair",e.bracePair="BracePair",e.callout1="Callout1",e.callout2="Callout2",e.callout3="Callout3",e.accentCallout1="AccentCallout1",e.accentCallout2="AccentCallout2",e.accentCallout3="AccentCallout3",e.borderCallout1="BorderCallout1",e.borderCallout2="BorderCallout2",e.borderCallout3="BorderCallout3",e.accentBorderCallout1="AccentBorderCallout1",e.accentBorderCallout2="AccentBorderCallout2",e.accentBorderCallout3="AccentBorderCallout3",e.wedgeRectCallout="WedgeRectCallout",e.wedgeRRectCallout="WedgeRRectCallout",e.wedgeEllipseCallout="WedgeEllipseCallout",e.cloudCallout="CloudCallout",e.cloud="Cloud",e.ribbon="Ribbon",e.ribbon2="Ribbon2",e.ellipseRibbon="EllipseRibbon",e.ellipseRibbon2="EllipseRibbon2",e.leftRightRibbon="LeftRightRibbon",e.verticalScroll="VerticalScroll",e.horizontalScroll="HorizontalScroll",e.wave="Wave",e.doubleWave="DoubleWave",e.plus="Plus",e.flowChartProcess="FlowChartProcess",e.flowChartDecision="FlowChartDecision",e.flowChartInputOutput="FlowChartInputOutput",e.flowChartPredefinedProcess="FlowChartPredefinedProcess",e.flowChartInternalStorage="FlowChartInternalStorage",e.flowChartDocument="FlowChartDocument",e.flowChartMultidocument="FlowChartMultidocument",e.flowChartTerminator="FlowChartTerminator",e.flowChartPreparation="FlowChartPreparation",e.flowChartManualInput="FlowChartManualInput",e.flowChartManualOperation="FlowChartManualOperation",e.flowChartConnector="FlowChartConnector",e.flowChartPunchedCard="FlowChartPunchedCard",e.flowChartPunchedTape="FlowChartPunchedTape",e.flowChartSummingJunction="FlowChartSummingJunction",e.flowChartOr="FlowChartOr",e.flowChartCollate="FlowChartCollate",e.flowChartSort="FlowChartSort",e.flowChartExtract="FlowChartExtract",e.flowChartMerge="FlowChartMerge",e.flowChartOfflineStorage="FlowChartOfflineStorage",e.flowChartOnlineStorage="FlowChartOnlineStorage",e.flowChartMagneticTape="FlowChartMagneticTape",e.flowChartMagneticDisk="FlowChartMagneticDisk",e.flowChartMagneticDrum="FlowChartMagneticDrum",e.flowChartDisplay="FlowChartDisplay",e.flowChartDelay="FlowChartDelay",e.flowChartAlternateProcess="FlowChartAlternateProcess",e.flowChartOffpageConnector="FlowChartOffpageConnector",e.actionButtonBlank="ActionButtonBlank",e.actionButtonHome="ActionButtonHome",e.actionButtonHelp="ActionButtonHelp",e.actionButtonInformation="ActionButtonInformation",e.actionButtonForwardNext="ActionButtonForwardNext",e.actionButtonBackPrevious="ActionButtonBackPrevious",e.actionButtonEnd="ActionButtonEnd",e.actionButtonBeginning="ActionButtonBeginning",e.actionButtonReturn="ActionButtonReturn",e.actionButtonDocument="ActionButtonDocument",e.actionButtonSound="ActionButtonSound",e.actionButtonMovie="ActionButtonMovie",e.gear6="Gear6",e.gear9="Gear9",e.funnel="Funnel",e.mathPlus="MathPlus",e.mathMinus="MathMinus",e.mathMultiply="MathMultiply",e.mathDivide="MathDivide",e.mathEqual="MathEqual",e.mathNotEqual="MathNotEqual",e.cornerTabs="CornerTabs",e.squareTabs="SquareTabs",e.plaqueTabs="PlaqueTabs",e.chartX="ChartX",e.chartStar="ChartStar",e.chartPlus="ChartPlus"}(e.GeometricShapeType||(e.GeometricShapeType={})),function(e){e.straight="Straight",e.elbow="Elbow",e.curve="Curve"}(e.ConnectorType||(e.ConnectorType={})),function(e){e.plain="Plain",e.mention="Mention"}(e.ContentType||(e.ContentType={})),function(e){e.conditionalFormats="ConditionalFormats",e.dataValidations="DataValidations",e.blanks="Blanks",e.constants="Constants",e.formulas="Formulas",e.sameConditionalFormat="SameConditionalFormat",e.sameDataValidation="SameDataValidation",e.visible="Visible"}(e.SpecialCellType||(e.SpecialCellType={})),function(e){e.all="All",e.errors="Errors",e.errorsLogical="ErrorsLogical",e.errorsNumbers="ErrorsNumbers",e.errorsText="ErrorsText",e.errorsLogicalNumber="ErrorsLogicalNumber",e.errorsLogicalText="ErrorsLogicalText",e.errorsNumberText="ErrorsNumberText",e.logical="Logical",e.logicalNumbers="LogicalNumbers",e.logicalText="LogicalText",e.logicalNumbersText="LogicalNumbersText",e.numbers="Numbers",e.numbersText="NumbersText",e.text="Text"}(e.SpecialCellValueType||(e.SpecialCellValueType={})),function(e){e.twoCell="TwoCell",e.oneCell="OneCell",e.absolute="Absolute"}(e.Placement||(e.Placement={})),function(e){e.none="None",e.solid="Solid",e.gray50="Gray50",e.gray75="Gray75",e.gray25="Gray25",e.horizontal="Horizontal",e.vertical="Vertical",e.down="Down",e.up="Up",e.checker="Checker",e.semiGray75="SemiGray75",e.lightHorizontal="LightHorizontal",e.lightVertical="LightVertical",e.lightDown="LightDown",e.lightUp="LightUp",e.grid="Grid",e.crissCross="CrissCross",e.gray16="Gray16",e.gray8="Gray8",e.linearGradient="LinearGradient",e.rectangularGradient="RectangularGradient"}(e.FillPattern||(e.FillPattern={})),function(e){e.left="Left",e.center="Center",e.right="Right",e.justify="Justify",e.justifyLow="JustifyLow",e.distributed="Distributed",e.thaiDistributed="ThaiDistributed"}(e.ShapeTextHorizontalAlignment||(e.ShapeTextHorizontalAlignment={})),function(e){e.top="Top",e.middle="Middle",e.bottom="Bottom",e.justified="Justified",e.distributed="Distributed"}(e.ShapeTextVerticalAlignment||(e.ShapeTextVerticalAlignment={})),function(e){e.overflow="Overflow",e.ellipsis="Ellipsis",e.clip="Clip"}(e.ShapeTextVerticalOverflow||(e.ShapeTextVerticalOverflow={})),function(e){e.overflow="Overflow",e.clip="Clip"}(e.ShapeTextHorizontalOverflow||(e.ShapeTextHorizontalOverflow={})),function(e){e.leftToRight="LeftToRight",e.rightToLeft="RightToLeft"}(e.ShapeTextReadingOrder||(e.ShapeTextReadingOrder={})),function(e){e.horizontal="Horizontal",e.vertical="Vertical",e.vertical270="Vertical270",e.wordArtVertical="WordArtVertical",e.eastAsianVertical="EastAsianVertical",e.mongolianVertical="MongolianVertical",e.wordArtVerticalRTL="WordArtVerticalRTL"}(e.ShapeTextOrientation||(e.ShapeTextOrientation={})),function(e){e.autoSizeNone="AutoSizeNone",e.autoSizeTextToFitShape="AutoSizeTextToFitShape",e.autoSizeShapeToFitText="AutoSizeShapeToFitText",e.autoSizeMixed="AutoSizeMixed"}(e.ShapeAutoSize||(e.ShapeAutoSize={})),function(e){e.save="Save",e.skipSave="SkipSave"}(e.CloseBehavior||(e.CloseBehavior={})),function(e){e.save="Save",e.prompt="Prompt"}(e.SaveBehavior||(e.SaveBehavior={})),function(e){e.dataSourceOrder="DataSourceOrder",e.ascending="Ascending",e.descending="Descending"}(e.SlicerSortType||(e.SlicerSortType={})),function(e){e.others="Others",e.home="Home",e.insert="Insert",e.draw="Draw",e.pageLayout="PageLayout",e.formulas="Formulas",e.data="Data",e.review="Review",e.view="View",e.developer="Developer",e.addIns="AddIns",e.help="Help"}(e.RibbonTab||(e.RibbonTab={})),function(e){e.general="General",e.number="Number",e.currency="Currency",e.accounting="Accounting",e.date="Date",e.time="Time",e.percentage="Percentage",e.fraction="Fraction",e.scientific="Scientific",e.text="Text",e.special="Special",e.custom="Custom"}(e.NumberFormatCategory||(e.NumberFormatCategory={}));var So=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"NamedSheetView"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyNames",{get:function(){return["name"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Name"]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_scalarPropertyUpdateable",{get:function(){return[!0]},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"name",{get:function(){return T("name",this._N,"NamedSheetView",this._isNull),this._N},set:function(e){this._N=e,I(this,"Name",e,0)},enumerable:!0,configurable:!0}),n.prototype.activate=function(){E(this,"Activate",0,[],0,0)},n.prototype.delete=function(){E(this,"Delete",0,[],0,0)},n.prototype.duplicate=function(t){return C(e.NamedSheetView,this,"Duplicate",0,[t],!1,!1,null,0)},n.prototype._handleResult=function(e){if(t.prototype._handleResult.call(this,e),!x(e)){var n=e;U(this,n),N(n.Name)||(this._N=n.Name)}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(e,n){t.prototype._handleRetrieveResult.call(this,e,n),H(this,e,n)},n.prototype.toJSON=function(){return k(this,{},{})},n.prototype.setMockData=function(e){W(this,e)},n.prototype.ensureUnchanged=function(e){R(this,e)},n}(OfficeExtension.ClientObject);e.NamedSheetView=So;var Co=function(t){function n(){return null!==t&&t.apply(this,arguments)||this}return __extends(n,t),Object.defineProperty(n.prototype,"_className",{get:function(){return"NamedSheetViewCollection"},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"_isCollection",{get:function(){return!0},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"items",{get:function(){return T("items",this.m__items,"NamedSheetViewCollection",this._isNull),this.m__items},enumerable:!0,configurable:!0}),n.prototype.add=function(t){return C(e.NamedSheetView,this,"Add",0,[t],!1,!0,null,0)},n.prototype.enterTemporary=function(){return C(e.NamedSheetView,this,"EnterTemporary",0,[],!1,!1,null,0)},n.prototype.exit=function(){E(this,"Exit",0,[],0,0)},n.prototype.getActive=function(){return C(e.NamedSheetView,this,"GetActive",0,[],!1,!1,null,0)},n.prototype.getCount=function(){return E(this,"GetCount",1,[],4,0)},n.prototype.getItem=function(t){return P(e.NamedSheetView,this,[t])},n.prototype.getItemAt=function(t){return C(e.NamedSheetView,this,"GetItemAt",1,[t],!1,!1,null,4)},n.prototype._handleResult=function(n){if(t.prototype._handleResult.call(this,n),!x(n)){var r=n;if(U(this,r),!x(r[OfficeExtension.Constants.items])){this.m__items=[];for(var i=r[OfficeExtension.Constants.items],o=0;o<i.length;o++){var s=D(e.NamedSheetView,!0,this,i[o],o);s._handleResult(i[o]),this.m__items.push(s)}}}},n.prototype.load=function(e){return j(this,e)},n.prototype.retrieve=function(e){return M(this,e)},n.prototype._handleRetrieveResult=function(n,r){var i=this;t.prototype._handleRetrieveResult.call(this,n,r),H(this,n,r,(function(t,n){return D(e.NamedSheetView,!0,i,t,n)}))},n.prototype.toJSON=function(){return k(this,{},{},this.m__items)},n.prototype.setMockData=function(t){var n=this;W(this,t,(function(t,r){return D(e.NamedSheetView,!0,n,t,r)}),(function(e){return n.m__items=e}))},n}(OfficeExtension.ClientObject);e.NamedSheetViewCollection=Co;var Po="FunctionResult",Ao=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"FunctionResult<T>"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyNames",{get:function(){return["error","value"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_scalarPropertyOriginalNames",{get:function(){return["Error","Value"]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"error",{get:function(){return T("error",this._E,Po,this._isNull),this._E},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"value",{get:function(){return T("value",this._V,Po,this._isNull),this._V},enumerable:!0,configurable:!0}),t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!x(t)){var n=t;U(this,n),N(n.Error)||(this._E=n.Error),N(n.Value)||(this._V=n.Value)}},t.prototype.load=function(e){return j(this,e)},t.prototype.retrieve=function(e){return M(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{error:this._E,value:this._V},{})},t.prototype.setMockData=function(e){W(this,e)},t.prototype.ensureUnchanged=function(e){R(this,e)},t}(OfficeExtension.ClientObject);e.FunctionResult=Ao;var Fo,Do=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return __extends(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"Functions"},enumerable:!0,configurable:!0}),t.prototype.abs=function(e){return C(Ao,this,"Abs",0,[e],!1,!0,null,0)},t.prototype.accrInt=function(e,t,n,r,i,o,s,a){return C(Ao,this,"AccrInt",0,[e,t,n,r,i,o,s,a],!1,!0,null,0)},t.prototype.accrIntM=function(e,t,n,r,i){return C(Ao,this,"AccrIntM",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.acos=function(e){return C(Ao,this,"Acos",0,[e],!1,!0,null,0)},t.prototype.acosh=function(e){return C(Ao,this,"Acosh",0,[e],!1,!0,null,0)},t.prototype.acot=function(e){return C(Ao,this,"Acot",0,[e],!1,!0,null,0)},t.prototype.acoth=function(e){return C(Ao,this,"Acoth",0,[e],!1,!0,null,0)},t.prototype.amorDegrc=function(e,t,n,r,i,o,s){return C(Ao,this,"AmorDegrc",0,[e,t,n,r,i,o,s],!1,!0,null,0)},t.prototype.amorLinc=function(e,t,n,r,i,o,s){return C(Ao,this,"AmorLinc",0,[e,t,n,r,i,o,s],!1,!0,null,0)},t.prototype.and=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"And",0,[e],!1,!0,null,0)},t.prototype.arabic=function(e){return C(Ao,this,"Arabic",0,[e],!1,!0,null,0)},t.prototype.areas=function(e){return C(Ao,this,"Areas",0,[e],!1,!0,null,0)},t.prototype.asc=function(e){return C(Ao,this,"Asc",0,[e],!1,!0,null,0)},t.prototype.asin=function(e){return C(Ao,this,"Asin",0,[e],!1,!0,null,0)},t.prototype.asinh=function(e){return C(Ao,this,"Asinh",0,[e],!1,!0,null,0)},t.prototype.atan=function(e){return C(Ao,this,"Atan",0,[e],!1,!0,null,0)},t.prototype.atan2=function(e,t){return C(Ao,this,"Atan2",0,[e,t],!1,!0,null,0)},t.prototype.atanh=function(e){return C(Ao,this,"Atanh",0,[e],!1,!0,null,0)},t.prototype.aveDev=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"AveDev",0,[e],!1,!0,null,0)},t.prototype.average=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Average",0,[e],!1,!0,null,0)},t.prototype.averageA=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"AverageA",0,[e],!1,!0,null,0)},t.prototype.averageIf=function(e,t,n){return C(Ao,this,"AverageIf",0,[e,t,n],!1,!0,null,0)},t.prototype.averageIfs=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];return C(Ao,this,"AverageIfs",0,[e,t],!1,!0,null,0)},t.prototype.bahtText=function(e){return C(Ao,this,"BahtText",0,[e],!1,!0,null,0)},t.prototype.base=function(e,t,n){return C(Ao,this,"Base",0,[e,t,n],!1,!0,null,0)},t.prototype.besselI=function(e,t){return C(Ao,this,"BesselI",0,[e,t],!1,!0,null,0)},t.prototype.besselJ=function(e,t){return C(Ao,this,"BesselJ",0,[e,t],!1,!0,null,0)},t.prototype.besselK=function(e,t){return C(Ao,this,"BesselK",0,[e,t],!1,!0,null,0)},t.prototype.besselY=function(e,t){return C(Ao,this,"BesselY",0,[e,t],!1,!0,null,0)},t.prototype.beta_Dist=function(e,t,n,r,i,o){return C(Ao,this,"Beta_Dist",0,[e,t,n,r,i,o],!1,!0,null,0)},t.prototype.beta_Inv=function(e,t,n,r,i){return C(Ao,this,"Beta_Inv",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.bin2Dec=function(e){return C(Ao,this,"Bin2Dec",0,[e],!1,!0,null,0)},t.prototype.bin2Hex=function(e,t){return C(Ao,this,"Bin2Hex",0,[e,t],!1,!0,null,0)},t.prototype.bin2Oct=function(e,t){return C(Ao,this,"Bin2Oct",0,[e,t],!1,!0,null,0)},t.prototype.binom_Dist=function(e,t,n,r){return C(Ao,this,"Binom_Dist",0,[e,t,n,r],!1,!0,null,0)},t.prototype.binom_Dist_Range=function(e,t,n,r){return C(Ao,this,"Binom_Dist_Range",0,[e,t,n,r],!1,!0,null,0)},t.prototype.binom_Inv=function(e,t,n){return C(Ao,this,"Binom_Inv",0,[e,t,n],!1,!0,null,0)},t.prototype.bitand=function(e,t){return C(Ao,this,"Bitand",0,[e,t],!1,!0,null,0)},t.prototype.bitlshift=function(e,t){return C(Ao,this,"Bitlshift",0,[e,t],!1,!0,null,0)},t.prototype.bitor=function(e,t){return C(Ao,this,"Bitor",0,[e,t],!1,!0,null,0)},t.prototype.bitrshift=function(e,t){return C(Ao,this,"Bitrshift",0,[e,t],!1,!0,null,0)},t.prototype.bitxor=function(e,t){return C(Ao,this,"Bitxor",0,[e,t],!1,!0,null,0)},t.prototype.ceiling_Math=function(e,t,n){return C(Ao,this,"Ceiling_Math",0,[e,t,n],!1,!0,null,0)},t.prototype.ceiling_Precise=function(e,t){return C(Ao,this,"Ceiling_Precise",0,[e,t],!1,!0,null,0)},t.prototype.char=function(e){return C(Ao,this,"Char",0,[e],!1,!0,null,0)},t.prototype.chiSq_Dist=function(e,t,n){return C(Ao,this,"ChiSq_Dist",0,[e,t,n],!1,!0,null,0)},t.prototype.chiSq_Dist_RT=function(e,t){return C(Ao,this,"ChiSq_Dist_RT",0,[e,t],!1,!0,null,0)},t.prototype.chiSq_Inv=function(e,t){return C(Ao,this,"ChiSq_Inv",0,[e,t],!1,!0,null,0)},t.prototype.chiSq_Inv_RT=function(e,t){return C(Ao,this,"ChiSq_Inv_RT",0,[e,t],!1,!0,null,0)},t.prototype.choose=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];return C(Ao,this,"Choose",0,[e,t],!1,!0,null,0)},t.prototype.clean=function(e){return C(Ao,this,"Clean",0,[e],!1,!0,null,0)},t.prototype.code=function(e){return C(Ao,this,"Code",0,[e],!1,!0,null,0)},t.prototype.columns=function(e){return C(Ao,this,"Columns",0,[e],!1,!0,null,0)},t.prototype.combin=function(e,t){return C(Ao,this,"Combin",0,[e,t],!1,!0,null,0)},t.prototype.combina=function(e,t){return C(Ao,this,"Combina",0,[e,t],!1,!0,null,0)},t.prototype.complex=function(e,t,n){return C(Ao,this,"Complex",0,[e,t,n],!1,!0,null,0)},t.prototype.concatenate=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Concatenate",0,[e],!1,!0,null,0)},t.prototype.confidence_Norm=function(e,t,n){return C(Ao,this,"Confidence_Norm",0,[e,t,n],!1,!0,null,0)},t.prototype.confidence_T=function(e,t,n){return C(Ao,this,"Confidence_T",0,[e,t,n],!1,!0,null,0)},t.prototype.convert=function(e,t,n){return C(Ao,this,"Convert",0,[e,t,n],!1,!0,null,0)},t.prototype.cos=function(e){return C(Ao,this,"Cos",0,[e],!1,!0,null,0)},t.prototype.cosh=function(e){return C(Ao,this,"Cosh",0,[e],!1,!0,null,0)},t.prototype.cot=function(e){return C(Ao,this,"Cot",0,[e],!1,!0,null,0)},t.prototype.coth=function(e){return C(Ao,this,"Coth",0,[e],!1,!0,null,0)},t.prototype.count=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Count",0,[e],!1,!0,null,0)},t.prototype.countA=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"CountA",0,[e],!1,!0,null,0)},t.prototype.countBlank=function(e){return C(Ao,this,"CountBlank",0,[e],!1,!0,null,0)},t.prototype.countIf=function(e,t){return C(Ao,this,"CountIf",0,[e,t],!1,!0,null,0)},t.prototype.countIfs=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"CountIfs",0,[e],!1,!0,null,0)},t.prototype.coupDayBs=function(e,t,n,r){return C(Ao,this,"CoupDayBs",0,[e,t,n,r],!1,!0,null,0)},t.prototype.coupDays=function(e,t,n,r){return C(Ao,this,"CoupDays",0,[e,t,n,r],!1,!0,null,0)},t.prototype.coupDaysNc=function(e,t,n,r){return C(Ao,this,"CoupDaysNc",0,[e,t,n,r],!1,!0,null,0)},t.prototype.coupNcd=function(e,t,n,r){return C(Ao,this,"CoupNcd",0,[e,t,n,r],!1,!0,null,0)},t.prototype.coupNum=function(e,t,n,r){return C(Ao,this,"CoupNum",0,[e,t,n,r],!1,!0,null,0)},t.prototype.coupPcd=function(e,t,n,r){return C(Ao,this,"CoupPcd",0,[e,t,n,r],!1,!0,null,0)},t.prototype.csc=function(e){return C(Ao,this,"Csc",0,[e],!1,!0,null,0)},t.prototype.csch=function(e){return C(Ao,this,"Csch",0,[e],!1,!0,null,0)},t.prototype.cumIPmt=function(e,t,n,r,i,o){return C(Ao,this,"CumIPmt",0,[e,t,n,r,i,o],!1,!0,null,0)},t.prototype.cumPrinc=function(e,t,n,r,i,o){return C(Ao,this,"CumPrinc",0,[e,t,n,r,i,o],!1,!0,null,0)},t.prototype.daverage=function(e,t,n){return C(Ao,this,"DAverage",0,[e,t,n],!1,!0,null,0)},t.prototype.dcount=function(e,t,n){return C(Ao,this,"DCount",0,[e,t,n],!1,!0,null,0)},t.prototype.dcountA=function(e,t,n){return C(Ao,this,"DCountA",0,[e,t,n],!1,!0,null,0)},t.prototype.dget=function(e,t,n){return C(Ao,this,"DGet",0,[e,t,n],!1,!0,null,0)},t.prototype.dmax=function(e,t,n){return C(Ao,this,"DMax",0,[e,t,n],!1,!0,null,0)},t.prototype.dmin=function(e,t,n){return C(Ao,this,"DMin",0,[e,t,n],!1,!0,null,0)},t.prototype.dproduct=function(e,t,n){return C(Ao,this,"DProduct",0,[e,t,n],!1,!0,null,0)},t.prototype.dstDev=function(e,t,n){return C(Ao,this,"DStDev",0,[e,t,n],!1,!0,null,0)},t.prototype.dstDevP=function(e,t,n){return C(Ao,this,"DStDevP",0,[e,t,n],!1,!0,null,0)},t.prototype.dsum=function(e,t,n){return C(Ao,this,"DSum",0,[e,t,n],!1,!0,null,0)},t.prototype.dvar=function(e,t,n){return C(Ao,this,"DVar",0,[e,t,n],!1,!0,null,0)},t.prototype.dvarP=function(e,t,n){return C(Ao,this,"DVarP",0,[e,t,n],!1,!0,null,0)},t.prototype.date=function(e,t,n){return C(Ao,this,"Date",0,[e,t,n],!1,!0,null,0)},t.prototype.datevalue=function(e){return C(Ao,this,"Datevalue",0,[e],!1,!0,null,0)},t.prototype.day=function(e){return C(Ao,this,"Day",0,[e],!1,!0,null,0)},t.prototype.days=function(e,t){return C(Ao,this,"Days",0,[e,t],!1,!0,null,0)},t.prototype.days360=function(e,t,n){return C(Ao,this,"Days360",0,[e,t,n],!1,!0,null,0)},t.prototype.db=function(e,t,n,r,i){return C(Ao,this,"Db",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.dbcs=function(e){return C(Ao,this,"Dbcs",0,[e],!1,!0,null,0)},t.prototype.ddb=function(e,t,n,r,i){return C(Ao,this,"Ddb",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.dec2Bin=function(e,t){return C(Ao,this,"Dec2Bin",0,[e,t],!1,!0,null,0)},t.prototype.dec2Hex=function(e,t){return C(Ao,this,"Dec2Hex",0,[e,t],!1,!0,null,0)},t.prototype.dec2Oct=function(e,t){return C(Ao,this,"Dec2Oct",0,[e,t],!1,!0,null,0)},t.prototype.decimal=function(e,t){return C(Ao,this,"Decimal",0,[e,t],!1,!0,null,0)},t.prototype.degrees=function(e){return C(Ao,this,"Degrees",0,[e],!1,!0,null,0)},t.prototype.delta=function(e,t){return C(Ao,this,"Delta",0,[e,t],!1,!0,null,0)},t.prototype.devSq=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"DevSq",0,[e],!1,!0,null,0)},t.prototype.disc=function(e,t,n,r,i){return C(Ao,this,"Disc",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.dollar=function(e,t){return C(Ao,this,"Dollar",0,[e,t],!1,!0,null,0)},t.prototype.dollarDe=function(e,t){return C(Ao,this,"DollarDe",0,[e,t],!1,!0,null,0)},t.prototype.dollarFr=function(e,t){return C(Ao,this,"DollarFr",0,[e,t],!1,!0,null,0)},t.prototype.duration=function(e,t,n,r,i,o){return C(Ao,this,"Duration",0,[e,t,n,r,i,o],!1,!0,null,0)},t.prototype.ecma_Ceiling=function(e,t){return C(Ao,this,"ECMA_Ceiling",0,[e,t],!1,!0,null,0)},t.prototype.edate=function(e,t){return C(Ao,this,"EDate",0,[e,t],!1,!0,null,0)},t.prototype.effect=function(e,t){return C(Ao,this,"Effect",0,[e,t],!1,!0,null,0)},t.prototype.eoMonth=function(e,t){return C(Ao,this,"EoMonth",0,[e,t],!1,!0,null,0)},t.prototype.erf=function(e,t){return C(Ao,this,"Erf",0,[e,t],!1,!0,null,0)},t.prototype.erfC=function(e){return C(Ao,this,"ErfC",0,[e],!1,!0,null,0)},t.prototype.erfC_Precise=function(e){return C(Ao,this,"ErfC_Precise",0,[e],!1,!0,null,0)},t.prototype.erf_Precise=function(e){return C(Ao,this,"Erf_Precise",0,[e],!1,!0,null,0)},t.prototype.error_Type=function(e){return C(Ao,this,"Error_Type",0,[e],!1,!0,null,0)},t.prototype.even=function(e){return C(Ao,this,"Even",0,[e],!1,!0,null,0)},t.prototype.exact=function(e,t){return C(Ao,this,"Exact",0,[e,t],!1,!0,null,0)},t.prototype.exp=function(e){return C(Ao,this,"Exp",0,[e],!1,!0,null,0)},t.prototype.expon_Dist=function(e,t,n){return C(Ao,this,"Expon_Dist",0,[e,t,n],!1,!0,null,0)},t.prototype.fvschedule=function(e,t){return C(Ao,this,"FVSchedule",0,[e,t],!1,!0,null,0)},t.prototype.f_Dist=function(e,t,n,r){return C(Ao,this,"F_Dist",0,[e,t,n,r],!1,!0,null,0)},t.prototype.f_Dist_RT=function(e,t,n){return C(Ao,this,"F_Dist_RT",0,[e,t,n],!1,!0,null,0)},t.prototype.f_Inv=function(e,t,n){return C(Ao,this,"F_Inv",0,[e,t,n],!1,!0,null,0)},t.prototype.f_Inv_RT=function(e,t,n){return C(Ao,this,"F_Inv_RT",0,[e,t,n],!1,!0,null,0)},t.prototype.fact=function(e){return C(Ao,this,"Fact",0,[e],!1,!0,null,0)},t.prototype.factDouble=function(e){return C(Ao,this,"FactDouble",0,[e],!1,!0,null,0)},t.prototype.false=function(){return C(Ao,this,"False",0,[],!1,!0,null,0)},t.prototype.find=function(e,t,n){return C(Ao,this,"Find",0,[e,t,n],!1,!0,null,0)},t.prototype.findB=function(e,t,n){return C(Ao,this,"FindB",0,[e,t,n],!1,!0,null,0)},t.prototype.fisher=function(e){return C(Ao,this,"Fisher",0,[e],!1,!0,null,0)},t.prototype.fisherInv=function(e){return C(Ao,this,"FisherInv",0,[e],!1,!0,null,0)},t.prototype.fixed=function(e,t,n){return C(Ao,this,"Fixed",0,[e,t,n],!1,!0,null,0)},t.prototype.floor_Math=function(e,t,n){return C(Ao,this,"Floor_Math",0,[e,t,n],!1,!0,null,0)},t.prototype.floor_Precise=function(e,t){return C(Ao,this,"Floor_Precise",0,[e,t],!1,!0,null,0)},t.prototype.fv=function(e,t,n,r,i){return C(Ao,this,"Fv",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.gamma=function(e){return C(Ao,this,"Gamma",0,[e],!1,!0,null,0)},t.prototype.gammaLn=function(e){return C(Ao,this,"GammaLn",0,[e],!1,!0,null,0)},t.prototype.gammaLn_Precise=function(e){return C(Ao,this,"GammaLn_Precise",0,[e],!1,!0,null,0)},t.prototype.gamma_Dist=function(e,t,n,r){return C(Ao,this,"Gamma_Dist",0,[e,t,n,r],!1,!0,null,0)},t.prototype.gamma_Inv=function(e,t,n){return C(Ao,this,"Gamma_Inv",0,[e,t,n],!1,!0,null,0)},t.prototype.gauss=function(e){return C(Ao,this,"Gauss",0,[e],!1,!0,null,0)},t.prototype.gcd=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Gcd",0,[e],!1,!0,null,0)},t.prototype.geStep=function(e,t){return C(Ao,this,"GeStep",0,[e,t],!1,!0,null,0)},t.prototype.geoMean=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"GeoMean",0,[e],!1,!0,null,0)},t.prototype.hlookup=function(e,t,n,r){return C(Ao,this,"HLookup",0,[e,t,n,r],!1,!0,null,0)},t.prototype.harMean=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"HarMean",0,[e],!1,!0,null,0)},t.prototype.hex2Bin=function(e,t){return C(Ao,this,"Hex2Bin",0,[e,t],!1,!0,null,0)},t.prototype.hex2Dec=function(e){return C(Ao,this,"Hex2Dec",0,[e],!1,!0,null,0)},t.prototype.hex2Oct=function(e,t){return C(Ao,this,"Hex2Oct",0,[e,t],!1,!0,null,0)},t.prototype.hour=function(e){return C(Ao,this,"Hour",0,[e],!1,!0,null,0)},t.prototype.hypGeom_Dist=function(e,t,n,r,i){return C(Ao,this,"HypGeom_Dist",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.hyperlink=function(e,t){return C(Ao,this,"Hyperlink",0,[e,t],!1,!0,null,0)},t.prototype.iso_Ceiling=function(e,t){return C(Ao,this,"ISO_Ceiling",0,[e,t],!1,!0,null,0)},t.prototype.if=function(e,t,n){return C(Ao,this,"If",0,[e,t,n],!1,!0,null,0)},t.prototype.imAbs=function(e){return C(Ao,this,"ImAbs",0,[e],!1,!0,null,0)},t.prototype.imArgument=function(e){return C(Ao,this,"ImArgument",0,[e],!1,!0,null,0)},t.prototype.imConjugate=function(e){return C(Ao,this,"ImConjugate",0,[e],!1,!0,null,0)},t.prototype.imCos=function(e){return C(Ao,this,"ImCos",0,[e],!1,!0,null,0)},t.prototype.imCosh=function(e){return C(Ao,this,"ImCosh",0,[e],!1,!0,null,0)},t.prototype.imCot=function(e){return C(Ao,this,"ImCot",0,[e],!1,!0,null,0)},t.prototype.imCsc=function(e){return C(Ao,this,"ImCsc",0,[e],!1,!0,null,0)},t.prototype.imCsch=function(e){return C(Ao,this,"ImCsch",0,[e],!1,!0,null,0)},t.prototype.imDiv=function(e,t){return C(Ao,this,"ImDiv",0,[e,t],!1,!0,null,0)},t.prototype.imExp=function(e){return C(Ao,this,"ImExp",0,[e],!1,!0,null,0)},t.prototype.imLn=function(e){return C(Ao,this,"ImLn",0,[e],!1,!0,null,0)},t.prototype.imLog10=function(e){return C(Ao,this,"ImLog10",0,[e],!1,!0,null,0)},t.prototype.imLog2=function(e){return C(Ao,this,"ImLog2",0,[e],!1,!0,null,0)},t.prototype.imPower=function(e,t){return C(Ao,this,"ImPower",0,[e,t],!1,!0,null,0)},t.prototype.imProduct=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"ImProduct",0,[e],!1,!0,null,0)},t.prototype.imReal=function(e){return C(Ao,this,"ImReal",0,[e],!1,!0,null,0)},t.prototype.imSec=function(e){return C(Ao,this,"ImSec",0,[e],!1,!0,null,0)},t.prototype.imSech=function(e){return C(Ao,this,"ImSech",0,[e],!1,!0,null,0)},t.prototype.imSin=function(e){return C(Ao,this,"ImSin",0,[e],!1,!0,null,0)},t.prototype.imSinh=function(e){return C(Ao,this,"ImSinh",0,[e],!1,!0,null,0)},t.prototype.imSqrt=function(e){return C(Ao,this,"ImSqrt",0,[e],!1,!0,null,0)},t.prototype.imSub=function(e,t){return C(Ao,this,"ImSub",0,[e,t],!1,!0,null,0)},t.prototype.imSum=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"ImSum",0,[e],!1,!0,null,0)},t.prototype.imTan=function(e){return C(Ao,this,"ImTan",0,[e],!1,!0,null,0)},t.prototype.imaginary=function(e){return C(Ao,this,"Imaginary",0,[e],!1,!0,null,0)},t.prototype.int=function(e){return C(Ao,this,"Int",0,[e],!1,!0,null,0)},t.prototype.intRate=function(e,t,n,r,i){return C(Ao,this,"IntRate",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.ipmt=function(e,t,n,r,i,o){return C(Ao,this,"Ipmt",0,[e,t,n,r,i,o],!1,!0,null,0)},t.prototype.irr=function(e,t){return C(Ao,this,"Irr",0,[e,t],!1,!0,null,0)},t.prototype.isErr=function(e){return C(Ao,this,"IsErr",0,[e],!1,!0,null,0)},t.prototype.isError=function(e){return C(Ao,this,"IsError",0,[e],!1,!0,null,0)},t.prototype.isEven=function(e){return C(Ao,this,"IsEven",0,[e],!1,!0,null,0)},t.prototype.isFormula=function(e){return C(Ao,this,"IsFormula",0,[e],!1,!0,null,0)},t.prototype.isLogical=function(e){return C(Ao,this,"IsLogical",0,[e],!1,!0,null,0)},t.prototype.isNA=function(e){return C(Ao,this,"IsNA",0,[e],!1,!0,null,0)},t.prototype.isNonText=function(e){return C(Ao,this,"IsNonText",0,[e],!1,!0,null,0)},t.prototype.isNumber=function(e){return C(Ao,this,"IsNumber",0,[e],!1,!0,null,0)},t.prototype.isOdd=function(e){return C(Ao,this,"IsOdd",0,[e],!1,!0,null,0)},t.prototype.isText=function(e){return C(Ao,this,"IsText",0,[e],!1,!0,null,0)},t.prototype.isoWeekNum=function(e){return C(Ao,this,"IsoWeekNum",0,[e],!1,!0,null,0)},t.prototype.ispmt=function(e,t,n,r){return C(Ao,this,"Ispmt",0,[e,t,n,r],!1,!0,null,0)},t.prototype.isref=function(e){return C(Ao,this,"Isref",0,[e],!1,!0,null,0)},t.prototype.kurt=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Kurt",0,[e],!1,!0,null,0)},t.prototype.large=function(e,t){return C(Ao,this,"Large",0,[e,t],!1,!0,null,0)},t.prototype.lcm=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Lcm",0,[e],!1,!0,null,0)},t.prototype.left=function(e,t){return C(Ao,this,"Left",0,[e,t],!1,!0,null,0)},t.prototype.leftb=function(e,t){return C(Ao,this,"Leftb",0,[e,t],!1,!0,null,0)},t.prototype.len=function(e){return C(Ao,this,"Len",0,[e],!1,!0,null,0)},t.prototype.lenb=function(e){return C(Ao,this,"Lenb",0,[e],!1,!0,null,0)},t.prototype.ln=function(e){return C(Ao,this,"Ln",0,[e],!1,!0,null,0)},t.prototype.log=function(e,t){return C(Ao,this,"Log",0,[e,t],!1,!0,null,0)},t.prototype.log10=function(e){return C(Ao,this,"Log10",0,[e],!1,!0,null,0)},t.prototype.logNorm_Dist=function(e,t,n,r){return C(Ao,this,"LogNorm_Dist",0,[e,t,n,r],!1,!0,null,0)},t.prototype.logNorm_Inv=function(e,t,n){return C(Ao,this,"LogNorm_Inv",0,[e,t,n],!1,!0,null,0)},t.prototype.lookup=function(e,t,n){return C(Ao,this,"Lookup",0,[e,t,n],!1,!0,null,0)},t.prototype.lower=function(e){return C(Ao,this,"Lower",0,[e],!1,!0,null,0)},t.prototype.mduration=function(e,t,n,r,i,o){return C(Ao,this,"MDuration",0,[e,t,n,r,i,o],!1,!0,null,0)},t.prototype.mirr=function(e,t,n){return C(Ao,this,"MIrr",0,[e,t,n],!1,!0,null,0)},t.prototype.mround=function(e,t){return C(Ao,this,"MRound",0,[e,t],!1,!0,null,0)},t.prototype.match=function(e,t,n){return C(Ao,this,"Match",0,[e,t,n],!1,!0,null,0)},t.prototype.max=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Max",0,[e],!1,!0,null,0)},t.prototype.maxA=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"MaxA",0,[e],!1,!0,null,0)},t.prototype.median=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Median",0,[e],!1,!0,null,0)},t.prototype.mid=function(e,t,n){return C(Ao,this,"Mid",0,[e,t,n],!1,!0,null,0)},t.prototype.midb=function(e,t,n){return C(Ao,this,"Midb",0,[e,t,n],!1,!0,null,0)},t.prototype.min=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Min",0,[e],!1,!0,null,0)},t.prototype.minA=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"MinA",0,[e],!1,!0,null,0)},t.prototype.minute=function(e){return C(Ao,this,"Minute",0,[e],!1,!0,null,0)},t.prototype.mod=function(e,t){return C(Ao,this,"Mod",0,[e,t],!1,!0,null,0)},t.prototype.month=function(e){return C(Ao,this,"Month",0,[e],!1,!0,null,0)},t.prototype.multiNomial=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"MultiNomial",0,[e],!1,!0,null,0)},t.prototype.n=function(e){return C(Ao,this,"N",0,[e],!1,!0,null,0)},t.prototype.nper=function(e,t,n,r,i){return C(Ao,this,"NPer",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.na=function(){return C(Ao,this,"Na",0,[],!1,!0,null,0)},t.prototype.negBinom_Dist=function(e,t,n,r){return C(Ao,this,"NegBinom_Dist",0,[e,t,n,r],!1,!0,null,0)},t.prototype.networkDays=function(e,t,n){return C(Ao,this,"NetworkDays",0,[e,t,n],!1,!0,null,0)},t.prototype.networkDays_Intl=function(e,t,n,r){return C(Ao,this,"NetworkDays_Intl",0,[e,t,n,r],!1,!0,null,0)},t.prototype.nominal=function(e,t){return C(Ao,this,"Nominal",0,[e,t],!1,!0,null,0)},t.prototype.norm_Dist=function(e,t,n,r){return C(Ao,this,"Norm_Dist",0,[e,t,n,r],!1,!0,null,0)},t.prototype.norm_Inv=function(e,t,n){return C(Ao,this,"Norm_Inv",0,[e,t,n],!1,!0,null,0)},t.prototype.norm_S_Dist=function(e,t){return C(Ao,this,"Norm_S_Dist",0,[e,t],!1,!0,null,0)},t.prototype.norm_S_Inv=function(e){return C(Ao,this,"Norm_S_Inv",0,[e],!1,!0,null,0)},t.prototype.not=function(e){return C(Ao,this,"Not",0,[e],!1,!0,null,0)},t.prototype.now=function(){return C(Ao,this,"Now",0,[],!1,!0,null,0)},t.prototype.npv=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];return C(Ao,this,"Npv",0,[e,t],!1,!0,null,0)},t.prototype.numberValue=function(e,t,n){return C(Ao,this,"NumberValue",0,[e,t,n],!1,!0,null,0)},t.prototype.oct2Bin=function(e,t){return C(Ao,this,"Oct2Bin",0,[e,t],!1,!0,null,0)},t.prototype.oct2Dec=function(e){return C(Ao,this,"Oct2Dec",0,[e],!1,!0,null,0)},t.prototype.oct2Hex=function(e,t){return C(Ao,this,"Oct2Hex",0,[e,t],!1,!0,null,0)},t.prototype.odd=function(e){return C(Ao,this,"Odd",0,[e],!1,!0,null,0)},t.prototype.oddFPrice=function(e,t,n,r,i,o,s,a,l){return C(Ao,this,"OddFPrice",0,[e,t,n,r,i,o,s,a,l],!1,!0,null,0)},t.prototype.oddFYield=function(e,t,n,r,i,o,s,a,l){return C(Ao,this,"OddFYield",0,[e,t,n,r,i,o,s,a,l],!1,!0,null,0)},t.prototype.oddLPrice=function(e,t,n,r,i,o,s,a){return C(Ao,this,"OddLPrice",0,[e,t,n,r,i,o,s,a],!1,!0,null,0)},t.prototype.oddLYield=function(e,t,n,r,i,o,s,a){return C(Ao,this,"OddLYield",0,[e,t,n,r,i,o,s,a],!1,!0,null,0)},t.prototype.or=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Or",0,[e],!1,!0,null,0)},t.prototype.pduration=function(e,t,n){return C(Ao,this,"PDuration",0,[e,t,n],!1,!0,null,0)},t.prototype.percentRank_Exc=function(e,t,n){return C(Ao,this,"PercentRank_Exc",0,[e,t,n],!1,!0,null,0)},t.prototype.percentRank_Inc=function(e,t,n){return C(Ao,this,"PercentRank_Inc",0,[e,t,n],!1,!0,null,0)},t.prototype.percentile_Exc=function(e,t){return C(Ao,this,"Percentile_Exc",0,[e,t],!1,!0,null,0)},t.prototype.percentile_Inc=function(e,t){return C(Ao,this,"Percentile_Inc",0,[e,t],!1,!0,null,0)},t.prototype.permut=function(e,t){return C(Ao,this,"Permut",0,[e,t],!1,!0,null,0)},t.prototype.permutationa=function(e,t){return C(Ao,this,"Permutationa",0,[e,t],!1,!0,null,0)},t.prototype.phi=function(e){return C(Ao,this,"Phi",0,[e],!1,!0,null,0)},t.prototype.pi=function(){return C(Ao,this,"Pi",0,[],!1,!0,null,0)},t.prototype.pmt=function(e,t,n,r,i){return C(Ao,this,"Pmt",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.poisson_Dist=function(e,t,n){return C(Ao,this,"Poisson_Dist",0,[e,t,n],!1,!0,null,0)},t.prototype.power=function(e,t){return C(Ao,this,"Power",0,[e,t],!1,!0,null,0)},t.prototype.ppmt=function(e,t,n,r,i,o){return C(Ao,this,"Ppmt",0,[e,t,n,r,i,o],!1,!0,null,0)},t.prototype.price=function(e,t,n,r,i,o,s){return C(Ao,this,"Price",0,[e,t,n,r,i,o,s],!1,!0,null,0)},t.prototype.priceDisc=function(e,t,n,r,i){return C(Ao,this,"PriceDisc",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.priceMat=function(e,t,n,r,i,o){return C(Ao,this,"PriceMat",0,[e,t,n,r,i,o],!1,!0,null,0)},t.prototype.product=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Product",0,[e],!1,!0,null,0)},t.prototype.proper=function(e){return C(Ao,this,"Proper",0,[e],!1,!0,null,0)},t.prototype.pv=function(e,t,n,r,i){return C(Ao,this,"Pv",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.quartile_Exc=function(e,t){return C(Ao,this,"Quartile_Exc",0,[e,t],!1,!0,null,0)},t.prototype.quartile_Inc=function(e,t){return C(Ao,this,"Quartile_Inc",0,[e,t],!1,!0,null,0)},t.prototype.quotient=function(e,t){return C(Ao,this,"Quotient",0,[e,t],!1,!0,null,0)},t.prototype.radians=function(e){return C(Ao,this,"Radians",0,[e],!1,!0,null,0)},t.prototype.rand=function(){return C(Ao,this,"Rand",0,[],!1,!0,null,0)},t.prototype.randBetween=function(e,t){return C(Ao,this,"RandBetween",0,[e,t],!1,!0,null,0)},t.prototype.rank_Avg=function(e,t,n){return C(Ao,this,"Rank_Avg",0,[e,t,n],!1,!0,null,0)},t.prototype.rank_Eq=function(e,t,n){return C(Ao,this,"Rank_Eq",0,[e,t,n],!1,!0,null,0)},t.prototype.rate=function(e,t,n,r,i,o){return C(Ao,this,"Rate",0,[e,t,n,r,i,o],!1,!0,null,0)},t.prototype.received=function(e,t,n,r,i){return C(Ao,this,"Received",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.replace=function(e,t,n,r){return C(Ao,this,"Replace",0,[e,t,n,r],!1,!0,null,0)},t.prototype.replaceB=function(e,t,n,r){return C(Ao,this,"ReplaceB",0,[e,t,n,r],!1,!0,null,0)},t.prototype.rept=function(e,t){return C(Ao,this,"Rept",0,[e,t],!1,!0,null,0)},t.prototype.right=function(e,t){return C(Ao,this,"Right",0,[e,t],!1,!0,null,0)},t.prototype.rightb=function(e,t){return C(Ao,this,"Rightb",0,[e,t],!1,!0,null,0)},t.prototype.roman=function(e,t){return C(Ao,this,"Roman",0,[e,t],!1,!0,null,0)},t.prototype.round=function(e,t){return C(Ao,this,"Round",0,[e,t],!1,!0,null,0)},t.prototype.roundDown=function(e,t){return C(Ao,this,"RoundDown",0,[e,t],!1,!0,null,0)},t.prototype.roundUp=function(e,t){return C(Ao,this,"RoundUp",0,[e,t],!1,!0,null,0)},t.prototype.rows=function(e){return C(Ao,this,"Rows",0,[e],!1,!0,null,0)},t.prototype.rri=function(e,t,n){return C(Ao,this,"Rri",0,[e,t,n],!1,!0,null,0)},t.prototype.sec=function(e){return C(Ao,this,"Sec",0,[e],!1,!0,null,0)},t.prototype.sech=function(e){return C(Ao,this,"Sech",0,[e],!1,!0,null,0)},t.prototype.second=function(e){return C(Ao,this,"Second",0,[e],!1,!0,null,0)},t.prototype.seriesSum=function(e,t,n,r){return C(Ao,this,"SeriesSum",0,[e,t,n,r],!1,!0,null,0)},t.prototype.sheet=function(e){return C(Ao,this,"Sheet",0,[e],!1,!0,null,0)},t.prototype.sheets=function(e){return C(Ao,this,"Sheets",0,[e],!1,!0,null,0)},t.prototype.sign=function(e){return C(Ao,this,"Sign",0,[e],!1,!0,null,0)},t.prototype.sin=function(e){return C(Ao,this,"Sin",0,[e],!1,!0,null,0)},t.prototype.sinh=function(e){return C(Ao,this,"Sinh",0,[e],!1,!0,null,0)},t.prototype.skew=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Skew",0,[e],!1,!0,null,0)},t.prototype.skew_p=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Skew_p",0,[e],!1,!0,null,0)},t.prototype.sln=function(e,t,n){return C(Ao,this,"Sln",0,[e,t,n],!1,!0,null,0)},t.prototype.small=function(e,t){return C(Ao,this,"Small",0,[e,t],!1,!0,null,0)},t.prototype.sqrt=function(e){return C(Ao,this,"Sqrt",0,[e],!1,!0,null,0)},t.prototype.sqrtPi=function(e){return C(Ao,this,"SqrtPi",0,[e],!1,!0,null,0)},t.prototype.stDevA=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"StDevA",0,[e],!1,!0,null,0)},t.prototype.stDevPA=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"StDevPA",0,[e],!1,!0,null,0)},t.prototype.stDev_P=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"StDev_P",0,[e],!1,!0,null,0)},t.prototype.stDev_S=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"StDev_S",0,[e],!1,!0,null,0)},t.prototype.standardize=function(e,t,n){return C(Ao,this,"Standardize",0,[e,t,n],!1,!0,null,0)},t.prototype.substitute=function(e,t,n,r){return C(Ao,this,"Substitute",0,[e,t,n,r],!1,!0,null,0)},t.prototype.subtotal=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];return C(Ao,this,"Subtotal",0,[e,t],!1,!0,null,0)},t.prototype.sum=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Sum",0,[e],!1,!0,null,0)},t.prototype.sumIf=function(e,t,n){return C(Ao,this,"SumIf",0,[e,t,n],!1,!0,null,0)},t.prototype.sumIfs=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];return C(Ao,this,"SumIfs",0,[e,t],!1,!0,null,0)},t.prototype.sumSq=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"SumSq",0,[e],!1,!0,null,0)},t.prototype.syd=function(e,t,n,r){return C(Ao,this,"Syd",0,[e,t,n,r],!1,!0,null,0)},t.prototype.t=function(e){return C(Ao,this,"T",0,[e],!1,!0,null,0)},t.prototype.tbillEq=function(e,t,n){return C(Ao,this,"TBillEq",0,[e,t,n],!1,!0,null,0)},t.prototype.tbillPrice=function(e,t,n){return C(Ao,this,"TBillPrice",0,[e,t,n],!1,!0,null,0)},t.prototype.tbillYield=function(e,t,n){return C(Ao,this,"TBillYield",0,[e,t,n],!1,!0,null,0)},t.prototype.t_Dist=function(e,t,n){return C(Ao,this,"T_Dist",0,[e,t,n],!1,!0,null,0)},t.prototype.t_Dist_2T=function(e,t){return C(Ao,this,"T_Dist_2T",0,[e,t],!1,!0,null,0)},t.prototype.t_Dist_RT=function(e,t){return C(Ao,this,"T_Dist_RT",0,[e,t],!1,!0,null,0)},t.prototype.t_Inv=function(e,t){return C(Ao,this,"T_Inv",0,[e,t],!1,!0,null,0)},t.prototype.t_Inv_2T=function(e,t){return C(Ao,this,"T_Inv_2T",0,[e,t],!1,!0,null,0)},t.prototype.tan=function(e){return C(Ao,this,"Tan",0,[e],!1,!0,null,0)},t.prototype.tanh=function(e){return C(Ao,this,"Tanh",0,[e],!1,!0,null,0)},t.prototype.text=function(e,t){return C(Ao,this,"Text",0,[e,t],!1,!0,null,0)},t.prototype.time=function(e,t,n){return C(Ao,this,"Time",0,[e,t,n],!1,!0,null,0)},t.prototype.timevalue=function(e){return C(Ao,this,"Timevalue",0,[e],!1,!0,null,0)},t.prototype.today=function(){return C(Ao,this,"Today",0,[],!1,!0,null,0)},t.prototype.trim=function(e){return C(Ao,this,"Trim",0,[e],!1,!0,null,0)},t.prototype.trimMean=function(e,t){return C(Ao,this,"TrimMean",0,[e,t],!1,!0,null,0)},t.prototype.true=function(){return C(Ao,this,"True",0,[],!1,!0,null,0)},t.prototype.trunc=function(e,t){return C(Ao,this,"Trunc",0,[e,t],!1,!0,null,0)},t.prototype.type=function(e){return C(Ao,this,"Type",0,[e],!1,!0,null,0)},t.prototype.usdollar=function(e,t){return C(Ao,this,"USDollar",0,[e,t],!1,!0,null,0)},t.prototype.unichar=function(e){return C(Ao,this,"Unichar",0,[e],!1,!0,null,0)},t.prototype.unicode=function(e){return C(Ao,this,"Unicode",0,[e],!1,!0,null,0)},t.prototype.upper=function(e){return C(Ao,this,"Upper",0,[e],!1,!0,null,0)},t.prototype.vlookup=function(e,t,n,r){return C(Ao,this,"VLookup",0,[e,t,n,r],!1,!0,null,0)},t.prototype.value=function(e){return C(Ao,this,"Value",0,[e],!1,!0,null,0)},t.prototype.varA=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"VarA",0,[e],!1,!0,null,0)},t.prototype.varPA=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"VarPA",0,[e],!1,!0,null,0)},t.prototype.var_P=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Var_P",0,[e],!1,!0,null,0)},t.prototype.var_S=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Var_S",0,[e],!1,!0,null,0)},t.prototype.vdb=function(e,t,n,r,i,o,s){return C(Ao,this,"Vdb",0,[e,t,n,r,i,o,s],!1,!0,null,0)},t.prototype.weekNum=function(e,t){return C(Ao,this,"WeekNum",0,[e,t],!1,!0,null,0)},t.prototype.weekday=function(e,t){return C(Ao,this,"Weekday",0,[e,t],!1,!0,null,0)},t.prototype.weibull_Dist=function(e,t,n,r){return C(Ao,this,"Weibull_Dist",0,[e,t,n,r],!1,!0,null,0)},t.prototype.workDay=function(e,t,n){return C(Ao,this,"WorkDay",0,[e,t,n],!1,!0,null,0)},t.prototype.workDay_Intl=function(e,t,n,r){return C(Ao,this,"WorkDay_Intl",0,[e,t,n,r],!1,!0,null,0)},t.prototype.xirr=function(e,t,n){return C(Ao,this,"Xirr",0,[e,t,n],!1,!0,null,0)},t.prototype.xnpv=function(e,t,n){return C(Ao,this,"Xnpv",0,[e,t,n],!1,!0,null,0)},t.prototype.xor=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return C(Ao,this,"Xor",0,[e],!1,!0,null,0)},t.prototype.year=function(e){return C(Ao,this,"Year",0,[e],!1,!0,null,0)},t.prototype.yearFrac=function(e,t,n){return C(Ao,this,"YearFrac",0,[e,t,n],!1,!0,null,0)},t.prototype.yield=function(e,t,n,r,i,o,s){return C(Ao,this,"Yield",0,[e,t,n,r,i,o,s],!1,!0,null,0)},t.prototype.yieldDisc=function(e,t,n,r,i){return C(Ao,this,"YieldDisc",0,[e,t,n,r,i],!1,!0,null,0)},t.prototype.yieldMat=function(e,t,n,r,i,o){return C(Ao,this,"YieldMat",0,[e,t,n,r,i,o],!1,!0,null,0)},t.prototype.z_Test=function(e,t,n){return C(Ao,this,"Z_Test",0,[e,t,n],!1,!0,null,0)},t.prototype._handleResult=function(t){(e.prototype._handleResult.call(this,t),x(t))||U(this,t)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),H(this,t,n)},t.prototype.toJSON=function(){return k(this,{},{})},t}(OfficeExtension.ClientObject);e.Functions=Do,function(e){e.accessDenied="AccessDenied",e.apiNotFound="ApiNotFound",e.conflict="Conflict",e.emptyChartSeries="EmptyChartSeries",e.filteredRangeConflict="FilteredRangeConflict",e.formulaLengthExceedsLimit="FormulaLengthExceedsLimit",e.generalException="GeneralException",e.inactiveWorkbook="InactiveWorkbook",e.insertDeleteConflict="InsertDeleteConflict",e.invalidArgument="InvalidArgument",e.invalidBinding="InvalidBinding",e.invalidOperation="InvalidOperation",e.invalidReference="InvalidReference",e.invalidSelection="InvalidSelection",e.itemAlreadyExists="ItemAlreadyExists",e.itemNotFound="ItemNotFound",e.mergedRangeConflict="MergedRangeConflict",e.nonBlankCellOffSheet="NonBlankCellOffSheet",e.notImplemented="NotImplemented",e.openWorkbookLinksBlocked="OpenWorkbookLinksBlocked",e.operationCellsExceedLimit="OperationCellsExceedLimit",e.pivotTableRangeConflict="PivotTableRangeConflict",e.rangeExceedsLimit="RangeExceedsLimit",e.refreshWorkbookLinksBlocked="RefreshWorkbookLinksBlocked",e.requestAborted="RequestAborted",e.responsePayloadSizeLimitExceeded="ResponsePayloadSizeLimitExceeded",e.unsupportedFeature="UnsupportedFeature",e.unsupportedOperation="UnsupportedOperation",e.unsupportedSheet="UnsupportedSheet",e.invalidOperationInCellEditMode="InvalidOperationInCellEditMode"}(Fo=e.ErrorCodes||(e.ErrorCodes={})),e.Interfaces||(e.Interfaces={})}(Excel||(Excel={}));var _EndExcel="_EndExcel";if("undefined"!=typeof window&&window.OSF&&window.OSF._OfficeAppFactory&&window.OSF._OfficeAppFactory.getHostInfo&&window.OSF._OfficeAppFactory.getHostInfo()){var hostPlatform=window.OSF._OfficeAppFactory.getHostInfo().hostPlatform;"web"===hostPlatform&&(OfficeExtension._internalConfig.enablePreviewExecution=!0)}function exposeOfficeRuntimeThroughOfficeNamespace(e,t){var n,r;void 0===e&&"undefined"!=typeof window&&(e=null===window||void 0===window?void 0:window.OfficeRuntime),void 0===e&&(e={}),void 0!==t&&(t.storage=t.storage||(null==e?void 0:e.storage),t.auth=t.auth||(null==e?void 0:e.auth),t.getAccessToken=t.getAccessToken||(null===(n=null==e?void 0:e.auth)||void 0===n?void 0:n.getAccessToken),t.addin=t.addin||(null==e?void 0:e.addin),t.isSetSupported=t.isSetSupported||(null===(r=null==e?void 0:e.apiInformation)||void 0===r?void 0:r.isSetSupported),t.license=t.license||(null==e?void 0:e.license),t.message=t.message||(null==e?void 0:e.message))}OSFAriaLogger.AriaLogger.EnableSendingTelemetryWithOTel=!0,OSFAriaLogger.AriaLogger.EnableSendingTelemetryWithLegacyAria=!1,function(e){var t={};function n(r){if(t[r])return t[r].exports;var i=t[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}n.m=e,n.c=t,n.d=function(e,t,r){n.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:r})},n.r=function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},n.t=function(e,t){if(1&t&&(e=n(e)),8&t)return e;if(4&t&&"object"==typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var i in e)n.d(r,i,function(t){return e[t]}.bind(null,i));return r},n.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="",n(n.s=1)}([function(e,t){e.exports=OfficeExtensionBatch},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),n(2),"undefined"==typeof CustomFunctionMappings&&(window.CustomFunctionMappings={}),"undefined"==typeof Promise&&(window.Promise=Office.Promise),window.OfficeExtensionBatch=window.OfficeExtensionBatch||window.OfficeExtension;var r,i=n(3);r=!1,void 0!==OSF&&OSF._OfficeAppFactory&&OSF._OfficeAppFactory.getHostInfo&&0!=(2&(OSF._OfficeAppFactory&&OSF._OfficeAppFactory.getHostInfo().flags))&&(r=!0),(0,i.default)(r)},function(e,t){!function(e){"use strict";if(!e.fetch){var t="URLSearchParams"in e,n="Symbol"in e&&"iterator"in Symbol,r="FileReader"in e&&"Blob"in e&&function(){try{return new Blob,!0}catch(e){return!1}}(),i="FormData"in e,o="ArrayBuffer"in e;if(o)var s=["[object Int8Array]","[object Uint8Array]","[object Uint8ClampedArray]","[object Int16Array]","[object Uint16Array]","[object Int32Array]","[object Uint32Array]","[object Float32Array]","[object Float64Array]"],a=function(e){return e&&DataView.prototype.isPrototypeOf(e)},l=ArrayBuffer.isView||function(e){return e&&s.indexOf(Object.prototype.toString.call(e))>-1};d.prototype.append=function(e,t){e=p(e),t=f(t);var n=this.map[e];this.map[e]=n?n+","+t:t},d.prototype.delete=function(e){delete this.map[p(e)]},d.prototype.get=function(e){return e=p(e),this.has(e)?this.map[e]:null},d.prototype.has=function(e){return this.map.hasOwnProperty(p(e))},d.prototype.set=function(e,t){this.map[p(e)]=f(t)},d.prototype.forEach=function(e,t){for(var n in this.map)this.map.hasOwnProperty(n)&&e.call(t,this.map[n],n,this)},d.prototype.keys=function(){var e=[];return this.forEach((function(t,n){e.push(n)})),h(e)},d.prototype.values=function(){var e=[];return this.forEach((function(t){e.push(t)})),h(e)},d.prototype.entries=function(){var e=[];return this.forEach((function(t,n){e.push([n,t])})),h(e)},n&&(d.prototype[Symbol.iterator]=d.prototype.entries);var u=["DELETE","GET","HEAD","OPTIONS","POST","PUT"];O.prototype.clone=function(){return new O(this,{body:this._bodyInit})},b.call(O.prototype),b.call(S.prototype),S.prototype.clone=function(){return new S(this._bodyInit,{status:this.status,statusText:this.statusText,headers:new d(this.headers),url:this.url})},S.error=function(){var e=new S(null,{status:0,statusText:""});return e.type="error",e};var c=[301,302,303,307,308];S.redirect=function(e,t){if(-1===c.indexOf(t))throw new RangeError("Invalid status code");return new S(null,{status:t,headers:{location:e}})},e.Headers=d,e.Request=O,e.Response=S,e.fetch=function(e,t){return new Promise((function(n,i){var o=new O(e,t),s=new XMLHttpRequest;s.onload=function(){var e,t,r={status:s.status,statusText:s.statusText,headers:(e=s.getAllResponseHeaders()||"",t=new d,e.split(/\r?\n/).forEach((function(e){var n=e.split(":"),r=n.shift().trim();if(r){var i=n.join(":").trim();t.append(r,i)}})),t)};r.url="responseURL"in s?s.responseURL:r.headers.get("X-Request-URL");var i="response"in s?s.response:s.responseText;n(new S(i,r))},s.onerror=function(){i(new TypeError("Network request failed"))},s.ontimeout=function(){i(new TypeError("Network request failed"))},s.open(o.method,o.url,!0),"include"===o.credentials&&(s.withCredentials=!0),"responseType"in s&&r&&(s.responseType="blob"),o.headers.forEach((function(e,t){s.setRequestHeader(t,e)})),s.send(void 0===o._bodyInit?null:o._bodyInit)}))},e.fetch.polyfill=!0}function p(e){if("string"!=typeof e&&(e=String(e)),/[^a-z0-9\-#$%&'*+.\^_`|~]/i.test(e))throw new TypeError("Invalid character in header field name");return e.toLowerCase()}function f(e){return"string"!=typeof e&&(e=String(e)),e}function h(e){var t={next:function(){var t=e.shift();return{done:void 0===t,value:t}}};return n&&(t[Symbol.iterator]=function(){return t}),t}function d(e){this.map={},e instanceof d?e.forEach((function(e,t){this.append(t,e)}),this):Array.isArray(e)?e.forEach((function(e){this.append(e[0],e[1])}),this):e&&Object.getOwnPropertyNames(e).forEach((function(t){this.append(t,e[t])}),this)}function y(e){if(e.bodyUsed)return Promise.reject(new TypeError("Already read"));e.bodyUsed=!0}function m(e){return new Promise((function(t,n){e.onload=function(){t(e.result)},e.onerror=function(){n(e.error)}}))}function g(e){var t=new FileReader,n=m(t);return t.readAsArrayBuffer(e),n}function _(e){if(e.slice)return e.slice(0);var t=new Uint8Array(e.byteLength);return t.set(new Uint8Array(e)),t.buffer}function b(){return this.bodyUsed=!1,this._initBody=function(e){if(this._bodyInit=e,e)if("string"==typeof e)this._bodyText=e;else if(r&&Blob.prototype.isPrototypeOf(e))this._bodyBlob=e;else if(i&&FormData.prototype.isPrototypeOf(e))this._bodyFormData=e;else if(t&&URLSearchParams.prototype.isPrototypeOf(e))this._bodyText=e.toString();else if(o&&r&&a(e))this._bodyArrayBuffer=_(e.buffer),this._bodyInit=new Blob([this._bodyArrayBuffer]);else{if(!o||!ArrayBuffer.prototype.isPrototypeOf(e)&&!l(e))throw new Error("unsupported BodyInit type");this._bodyArrayBuffer=_(e)}else this._bodyText="";this.headers.get("content-type")||("string"==typeof e?this.headers.set("content-type","text/plain;charset=UTF-8"):this._bodyBlob&&this._bodyBlob.type?this.headers.set("content-type",this._bodyBlob.type):t&&URLSearchParams.prototype.isPrototypeOf(e)&&this.headers.set("content-type","application/x-www-form-urlencoded;charset=UTF-8"))},r&&(this.blob=function(){var e=y(this);if(e)return e;if(this._bodyBlob)return Promise.resolve(this._bodyBlob);if(this._bodyArrayBuffer)return Promise.resolve(new Blob([this._bodyArrayBuffer]));if(this._bodyFormData)throw new Error("could not read FormData body as blob");return Promise.resolve(new Blob([this._bodyText]))},this.arrayBuffer=function(){return this._bodyArrayBuffer?y(this)||Promise.resolve(this._bodyArrayBuffer):this.blob().then(g)}),this.text=function(){var e,t,n,r=y(this);if(r)return r;if(this._bodyBlob)return e=this._bodyBlob,n=m(t=new FileReader),t.readAsText(e),n;if(this._bodyArrayBuffer)return Promise.resolve(function(e){for(var t=new Uint8Array(e),n=new Array(t.length),r=0;r<t.length;r++)n[r]=String.fromCharCode(t[r]);return n.join("")}(this._bodyArrayBuffer));if(this._bodyFormData)throw new Error("could not read FormData body as text");return Promise.resolve(this._bodyText)},i&&(this.formData=function(){return this.text().then(v)}),this.json=function(){return this.text().then(JSON.parse)},this}function O(e,t){var n,r,i=(t=t||{}).body;if(e instanceof O){if(e.bodyUsed)throw new TypeError("Already read");this.url=e.url,this.credentials=e.credentials,t.headers||(this.headers=new d(e.headers)),this.method=e.method,this.mode=e.mode,i||null==e._bodyInit||(i=e._bodyInit,e.bodyUsed=!0)}else this.url=String(e);if(this.credentials=t.credentials||this.credentials||"omit",!t.headers&&this.headers||(this.headers=new d(t.headers)),this.method=(r=(n=t.method||this.method||"GET").toUpperCase(),u.indexOf(r)>-1?r:n),this.mode=t.mode||this.mode||null,this.referrer=null,("GET"===this.method||"HEAD"===this.method)&&i)throw new TypeError("Body not allowed for GET or HEAD requests");this._initBody(i)}function v(e){var t=new FormData;return e.trim().split("&").forEach((function(e){if(e){var n=e.split("="),r=n.shift().replace(/\+/g," "),i=n.join("=").replace(/\+/g," ");t.append(decodeURIComponent(r),decodeURIComponent(i))}})),t}function S(e,t){t||(t={}),this.type="default",this.status="status"in t?t.status:200,this.ok=this.status>=200&&this.status<300,this.statusText="statusText"in t?t.statusText:"OK",this.headers=new d(t.headers),this.url=t.url||"",this._initBody(e)}}("undefined"!=typeof self?self:this)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4);t.default=function(e){function t(){Office.onReady((function(e){e.host===Office.HostType.Excel?function e(){CustomFunctionMappings&&CustomFunctionMappings.__delay__?setTimeout(e,50):r.CustomFunctions.initialize()}():console.warn("Warning: Expected to be loaded inside of an Excel add-in.")}))}window.CustomFunctions=window.CustomFunctions||{},window.CustomFunctions.setCustomFunctionInvoker=r.setCustomFunctionInvoker,window.CustomFunctions.Error=r.CustomFunctionError,window.CustomFunctions.ErrorCode=r.ErrorCode,window.CustomFunctions.FormattedNumber=r.CustomFunctionFormattedNumber,window.CustomFunctions.Entity=r.CustomFunctionEntity,window.CustomFunctions.WebImage=r.CustomFunctionWebImage,window.CustomFunctions.Attribution=r.CustomFunctionAttribution,window.CustomFunctions.Provider=r.CustomFunctionProvider,r.setCustomFunctionAssociation(window.CustomFunctions._association),e&&("loading"===document.readyState?document.addEventListener("DOMContentLoaded",t):t())}},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(e[n]=t[n])})(e,t)},function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Class extends value "+String(t)+" is not a constructor or null");function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)});Object.defineProperty(t,"__esModule",{value:!0}),t.CustomFunctionsContainer=t.CustomFunctions=t.setCustomFunctionInvoker=t.setCustomFunctionAssociation=t.customFunctionProxy=t.CustomFunctionProxy=t.CreateCustomFunctionInstance=t.CustomFunctionProvider=t.CustomFunctionAttribution=t.CustomFunctionWebImage=t.CustomFunctionFormattedNumber=t.CustomFunctionEntity=t.CustomFunctionError=t.ErrorCode=t.InvocationContext=t.Script=void 0;var o=n(0),s=n(0),a=o.BatchApiHelper.createPropertyObject,l=(o.BatchApiHelper.createMethodObject,o.BatchApiHelper.createIndexerObject,o.BatchApiHelper.createRootServiceObject),u=o.BatchApiHelper.createTopLevelServiceObject,c=(o.BatchApiHelper.createChildItemObject,o.BatchApiHelper.invokeMethod),p=(o.BatchApiHelper.invokeEnsureUnchanged,o.BatchApiHelper.invokeSetProperty,o.Utility.isNullOrUndefined),f=(o.Utility.isUndefined,o.Utility.throwIfNotLoaded,o.Utility.throwIfApiNotSupported),h=o.Utility.load,d=(o.Utility.retrieve,o.Utility.toJson),y=o.Utility.fixObjectPathIfNecessary,m=o.Utility._handleNavigationPropertyResults,g=(o.Utility.adjustToDateTime,o.Utility.processRetrieveResult),_=(o.Utility.setMockData,o.CommonUtility.calculateApiFlags,function(e){function t(t){var n=e.call(this,t)||this;return n.m_customFunctions=x.newObject(n),n.m_container=l(T,n),n._rootObject=n.m_container,n._rootObjectPropertyName="customFunctionsContainer",n._requestFlagModifier=2176,n}return i(t,e),Object.defineProperty(t.prototype,"customFunctions",{get:function(){return this.m_customFunctions},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"customFunctionsContainer",{get:function(){return this.m_container},enumerable:!1,configurable:!0}),t.prototype._processOfficeJsErrorResponse=function(e,t){5004===e&&(t.ErrorCode=N.invalidOperationInCellEditMode,t.ErrorMessage=o.Utility._getResourceString(o.ResourceStrings.invalidOperationInCellEditMode))},t}(o.ClientRequestContext));t.Script={_CustomFunctionMetadata:{}};var b,O=function(){function e(e,t,n,r,i){this._functionName=e,p(t)||(this._address=t),p(n)||(this._parameterAddresses=n),this.setResult=r,this.setError=i}return Object.defineProperty(e.prototype,"onCanceled",{get:function(){if(!p(this._onCanceled)&&"function"==typeof this._onCanceled)return this._onCanceled},set:function(e){this._onCanceled=e},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"functionName",{get:function(){return this._functionName},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"address",{get:function(){return this._address},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"parameterAddresses",{get:function(){return this._parameterAddresses},enumerable:!1,configurable:!0}),e}();t.InvocationContext=O,function(e){e.Info="Medium",e.Error="Unexpected",e.Verbose="Verbose"}(b||(b={}));var v,S=function(e,t){this.Severity=e,this.Message=t},C=function(){function e(){}return e.logEvent=function(t,n,r){if(e.s_shouldLog||o.CoreUtility._logEnabled){var i=t.Severity+" "+t.Message+n;if(r&&(i=i+" "+r),o.Utility.log(i),e.s_shouldLog)switch(t.Severity){case b.Verbose:null!==console.log&&console.log(i);break;case b.Info:null!==console.info&&console.info(i);break;case b.Error:null!==console.error&&console.error(i)}}},e.shouldLog=function(){try{return!p(console)&&!p(window)&&window.name&&"string"==typeof window.name&&JSON.parse(window.name)[e.CustomFunctionLoggingFlag]}catch(e){return o.Utility.log(JSON.stringify(e)),!1}},e.CustomFunctionLoggingFlag="CustomFunctionsRuntimeLogging",e.s_shouldLog=e.shouldLog(),e}();!function(e){e.invalidValue="#VALUE!",e.notAvailable="#N/A",e.divisionByZero="#DIV/0!",e.invalidNumber="#NUM!",e.nullReference="#NULL!",e.invalidName="#NAME?",e.invalidReference="#REF!"}(v=t.ErrorCode||(t.ErrorCode={}));var P=function(e){function t(n,r,i){var o=e.call(this,r||"")||this;return o.type=t.valueType,o.basicType=t.valueType,Object.setPrototypeOf(o,t.prototype),o.code=n||v.invalidValue,o.basicValue=n||v.invalidValue,o.errorSubType=i,o.stack=void 0,o}return i(t,e),t.prototype.toJSON=function(){return{valueType:"CustomFunctionError",code:this.code,type:this.type,basicValue:this.basicValue,basicType:this.type,errorSubType:this.errorSubType,message:this._getMessage(),errorType:this._getErrorType()}},t.prototype._getErrorType=function(){switch(this.basicValue){case v.invalidValue:return"Value";case v.divisionByZero:return"Div0";case v.notAvailable:return"NotAvailable";case v.invalidNumber:return"Num";case v.invalidName:return"Name";case v.invalidReference:return"Ref";case v.nullReference:return"Null";default:return}},t.prototype._getMessage=function(){switch(this.basicValue){case v.invalidValue:case v.notAvailable:return""==this.message?void 0:this.message;case v.divisionByZero:case v.invalidNumber:case v.invalidName:case v.invalidReference:case v.nullReference:default:return}},t.valueType="Error",t}(Error);t.CustomFunctionError=P;var A=function(){function e(t,n){this.type=e.valueType,this.basicValue="#VALUE!",this.basicType="Error",Object.setPrototypeOf(this,e.prototype),this.text=t,this.properties=n}return e.prototype.toJSON=function(){return{type:this.type,text:this.text,basicValue:this.basicValue,basicType:this.basicType,properties:this.properties}},e.valueType="Entity",e}();t.CustomFunctionEntity=A;var F=function(){function e(t,n){this.type=e.valueType,this.basicValue=t,this.numberFormat=n}return e.prototype.toJSON=function(){return{type:this.type,basicValue:this.basicValue,numberFormat:this.numberFormat}},e.valueType="FormattedNumber",e}();t.CustomFunctionFormattedNumber=F;var D=function(){function e(t,n,r,i,o){this.type=e.valueType,this.basicValue="#VALUE!",this.basicType="Error",this.address=t,null!=i&&(this.attribution=i),null!=o&&(this.provider=o),null!=n&&(this.altText=n),null!=r&&(this.relatedImagesAddress=r)}return e.prototype.toJSON=function(){return{type:this.type,basicValue:this.basicValue,basicType:this.basicType,address:this.address,altText:this.altText,relatedImagesAddress:this.relatedImagesAddress,attribution:null==this.attribution?void 0:this.attribution.map((function(e){return e.toJSON()})),provider:null==this.provider?void 0:this.provider.toJSON()}},e.valueType="WebImage",e}();t.CustomFunctionWebImage=D;var E=function(){function e(e,t,n,r){null!=e&&(this.licenseAddress=e),null!=t&&(this.licenseText=t),null!=n&&(this.sourceAddress=n),null!=r&&(this.sourceText=r)}return e.prototype.toJSON=function(){return{licenseAddress:this.licenseAddress,licenseText:this.licenseText,sourceAddress:this.sourceAddress,sourceText:this.sourceText}},e}();t.CustomFunctionAttribution=E;var R=function(){function e(e,t,n){this.description=e,null!=t&&(this.logoSourceAddress=t),null!=n&&(this.logoTargetAddress=n)}return e.prototype.toJSON=function(){return{description:this.description,logoSourceAddress:this.logoSourceAddress,logoTargetAddress:this.logoTargetAddress}},e}();t.CustomFunctionProvider=R,t.CreateCustomFunctionInstance=function e(t){if(t instanceof Array)return t.map((function(t){return e(t)}));if(!(t instanceof Object))return t;var n=JSON.parse(t.value);switch(n.type){case A.valueType:return new A(n.text,n.properties);case F.valueType:return new F(n.primitive,n.numberFormat);case P.valueType:return new P(n.primitive,void 0,n.errorSubCode);case D.valueType:return new D(n.address,n.altText,n.relatedImagesAddress,n.attribution,n.provider);case"Double":case"String":case"Boolean":return n.primitive;default:throw o.Utility.createRuntimeError(N.generalException,"message","CustomFunctionProxy.createRichDataFromJsonString")}};var I=function(){function e(){this._whenInit=void 0,this._isInit=!1,this._setResultsDelayMillis=50,this._setResultsOverdueDelayMillis=2e3,this._maxContextSyncExecutionDurationMills=15e3,this._minContextSyncIntervalMills=500,this._smallerMinContextSyncIntervalMills=200,this._maxContextSyncIntervalMills=2e3,this._setResultsLifeMillis=6e4,this._ensureInitRetryDelayMillis=500,this._resultEntryBuffer={},this._resultEntryRetryBuffer={},this._retryBufferBodySize=0,this._isSetResultsTaskScheduled=!1,this._setResultsTaskOverdueTime=0,this._inProgressContextSyncExpectedFinishTime=0,this._batchQuotaMillis=1e3,this._invocationContextMap={},this._maxResultSize=2e6}return e.prototype._initSettings=function(){if("object"==typeof t.Script&&"object"==typeof t.Script._CustomFunctionSettings){if("number"==typeof t.Script._CustomFunctionSettings.setResultsDelayMillis){var e=t.Script._CustomFunctionSettings.setResultsDelayMillis;e=Math.max(0,e),e=Math.min(1e3,e),this._setResultsDelayMillis=e}if("number"==typeof t.Script._CustomFunctionSettings.ensureInitRetryDelayMillis){var n=t.Script._CustomFunctionSettings.ensureInitRetryDelayMillis;n=Math.max(0,n),n=Math.min(2e3,n),this._ensureInitRetryDelayMillis=n}if("number"==typeof t.Script._CustomFunctionSettings.setResultsLifeMillis){var r=t.Script._CustomFunctionSettings.setResultsLifeMillis;r=Math.max(0,r),r=Math.min(6e5,r),this._setResultsLifeMillis=r}if("number"==typeof t.Script._CustomFunctionSettings.batchQuotaMillis){var i=t.Script._CustomFunctionSettings.batchQuotaMillis;i=Math.max(0,i),i=Math.min(1e3,i),this._batchQuotaMillis=i}}},e.prototype.ensureInit=function(e){var t=this;return this._initSettings(),void 0===this._whenInit&&(this._whenInit=o.Utility._createPromiseFromResult(null).then((function(){if(!t._isInit)return e.eventRegistration.register(5,"",t._handleMessage.bind(t))})).then((function(){t._isInit=!0}))),this._isInit||e._pendingRequest._addPreSyncPromise(this._whenInit),this._whenInit},e.prototype.setCustomFunctionInvoker=function(e){"object"==typeof CustomFunctionMappings&&delete CustomFunctionMappings.__delay__,this._invoker=e},e.prototype.setCustomFunctionAssociation=function(e){var t=this;this._customFunctionMappingsUpperCase=void 0,this._association=e,this._association&&this._association.onchange((function(){t._customFunctionMappingsUpperCase=void 0}))},e.prototype._initFromHostBridge=function(e){var t=this;this._initSettings(),e.addHostMessageHandler((function(e){3===e.type?t._handleMessage(e.message):4===e.type&&t._handleSettings(e.message)})),this._isInit=!0,this._whenInit=o.CoreUtility.Promise.resolve()},e.prototype._handleSettings=function(e){o.Utility.log("CustomFunctionProxy._handleSettings:"+JSON.stringify(e)),e&&"object"==typeof e&&(C.s_shouldLog=e[C.CustomFunctionLoggingFlag])},e.prototype._handleMessage=function(t){try{o.Utility.log("CustomFunctionProxy._handleMessage"),o.Utility.checkArgumentNull(t,"args");for(var n=t.entries,r=[],i=[],s=[],a=0;a<n.length;a++)1===n[a].messageCategory&&("string"==typeof n[a].message&&(n[a].message=JSON.parse(n[a].message)),1e3===n[a].messageType?r.push(n[a]):1001===n[a].messageType?i.push(n[a]):1002===n[a].messageType?s.push(n[a]):o.Utility.log("CustomFunctionProxy._handleMessage unknown message type "+n[a].messageType));if(s.length>0&&this._handleMetadataEntries(s),r.length>0){var l=this._batchInvocationEntries(r);l.length>0&&this._invokeRemainingBatchEntries(l,0)}i.length>0&&this._handleCancellationEntries(i)}catch(t){throw e._tryLog(t),t}return o.Utility._createPromiseFromResult(null)},e.toLogMessage=function(e){var t="Unknown Error";if(e)try{e.toString&&(t=e.toString()),t=t+" "+JSON.stringify(e)}catch(e){t="Unexpected Error"}return t},e._tryLog=function(t){var n=e.toLogMessage(t);o.Utility.log(n)},e.prototype._handleMetadataEntries=function(e){for(var n=0;n<e.length;n++){var r=e[n].message;if(p(r))throw o.Utility.createRuntimeError(N.generalException,"message","CustomFunctionProxy._handleMetadataEntries");t.Script._CustomFunctionMetadata[r.functionName]={options:{stream:r.isStream,cancelable:r.isCancelable}}}},e.prototype._handleCancellationEntries=function(t){for(var n=0;n<t.length;n++){var r=t[n].message;if(p(r))throw o.Utility.createRuntimeError(N.generalException,"message","CustomFunctionProxy._handleCancellationEntries");var i=r.invocationId,s=this._invocationContextMap[i];p(s)||(delete this._invocationContextMap[i],C.logEvent(e.CustomFunctionCancellation,s.functionName),p(s.onCanceled)||s.onCanceled())}},e.prototype._transferCustomFunctionError=function(e){var t=this;return e.map((function(e,n){return null===e?e:null!=e.code&&"CustomFunctionError"==e.type?new P(e.code):e instanceof Array?t._transferCustomFunctionError(e):e}))},e.prototype._transferCustomFunctionRichData=function(e){var t=this;return e.map((function(e,n){return e instanceof Array?t._transferCustomFunctionRichData(e):e instanceof Object?JSON.parse(e.value):e}))},e.prototype._batchInvocationEntries=function(n){for(var r=this,i=[],s=function(s){var l,u=n[s].message;if(Array.isArray(u)?(l={invocationId:u[0],functionName:u[1],parameterValues:u[2],address:u[3],flags:u[4],parameterAddresses:null},p(u[5])||(l.parameterAddresses=u[5])):l=u,p(l))throw o.Utility.createRuntimeError(N.generalException,"message","CustomFunctionProxy._batchInvocationEntries");if(p(l.invocationId)||l.invocationId<0)throw o.Utility.createRuntimeError(N.generalException,"invocationId","CustomFunctionProxy._batchInvocationEntries");if(p(l.functionName))throw o.Utility.createRuntimeError(N.generalException,"functionName","CustomFunctionProxy._batchInvocationEntries");var c=null,f=!1,h=!1,d=!1;if("number"==typeof l.flags)f=0!=(1&l.flags),h=0!=(2&l.flags),d=0!=(4&l.flags),0!=(8&l.flags)&&(l.parameterValues=a._transferCustomFunctionRichData(l.parameterValues)),d&&(l.parameterValues=a._transferCustomFunctionError(l.parameterValues));else{var y=t.Script._CustomFunctionMetadata[l.functionName];if(p(y))return C.logEvent(e.CustomFunctionExecutionNotFoundLog,l.functionName),o.Utility.isSetSupported("CustomFunctions","1.8")?a._setError(l.invocationId,null,9):a._setError(l.invocationId,"N/A",1),"continue";f=y.options.cancelable,h=y.options.stream}if(a._invoker&&!a._customFunctionMappingsContains(l.functionName))return a._invokeFunctionUsingInvoker(l),"continue";try{c=a._getFunction(l.functionName)}catch(t){return C.logEvent(e.CustomFunctionExecutionNotFoundLog,l.functionName),o.Utility.isSetSupported("CustomFunctions","1.8")?a._setError(l.invocationId,t,10):a._setError(l.invocationId,t,1),"continue"}var m=void 0;if(h||f){var g=void 0,_=void 0;h&&(g=function(t){r._invocationContextMap[l.invocationId]?r._setResult(l.invocationId,t):C.logEvent(e.CustomFunctionAlreadyCancelled,l.functionName)},_=function(t){r._invocationContextMap[l.invocationId]?r._setError(l.invocationId,t.message,r._getCustomFunctionResultErrorCodeFromErrorCode(t.code)):C.logEvent(e.CustomFunctionAlreadyCancelled,l.functionName)}),m=new O(l.functionName,l.address,l.parameterAddresses,g,_),a._invocationContextMap[l.invocationId]=m}else m=new O(l.functionName,l.address,l.parameterAddresses);l.parameterValues.push(m),i.push({call:c,isBatching:!1,isStreaming:h,invocationIds:[l.invocationId],parameterValueSets:[l.parameterValues],functionName:l.functionName})},a=this,l=0;l<n.length;l++)s(l);return i},e.prototype._invokeFunctionUsingInvoker=function(e){var t=this,n=0!=(1&e.flags),r=0!=(2&e.flags),i=e.invocationId,o=void 0,s=void 0;if(r)o=function(e){t._invocationContextMap[i]&&t._setResult(i,e)},s=function(e){t._invocationContextMap[i]&&t._setError(i,e.message,t._getCustomFunctionResultErrorCodeFromErrorCode(e.code))};else{var a=!1;o=function(e){a||t._setResult(i,e),a=!0},s=function(e){a||t._setError(i,e.message,t._getCustomFunctionResultErrorCodeFromErrorCode(e.code)),a=!0}}var l=new O(e.functionName,e.address,e.parameterAddresses,o,s);(r||n)&&(this._invocationContextMap[i]=l),this._invoker.invoke(e.functionName,e.parameterValues,l)},e.prototype._ensureCustomFunctionMappingsUpperCase=function(){if(p(this._customFunctionMappingsUpperCase)){if(this._customFunctionMappingsUpperCase={},"object"==typeof CustomFunctionMappings)for(var t in o.CoreUtility.log("CustomFunctionMappings.Keys="+JSON.stringify(Object.keys(CustomFunctionMappings))),CustomFunctionMappings)this._customFunctionMappingsUpperCase[t.toUpperCase()]&&C.logEvent(e.CustomFunctionDuplicatedName,t),this._customFunctionMappingsUpperCase[t.toUpperCase()]=CustomFunctionMappings[t];if(this._association)for(var t in o.CoreUtility.log("CustomFunctionAssociateMappings.Keys="+JSON.stringify(Object.keys(this._association.mappings))),this._association.mappings)this._customFunctionMappingsUpperCase[t.toUpperCase()]&&C.logEvent(e.CustomFunctionDuplicatedName,t),this._customFunctionMappingsUpperCase[t.toUpperCase()]=this._association.mappings[t]}},e.prototype._customFunctionMappingsContains=function(e){this._ensureCustomFunctionMappingsUpperCase();var t=e.toUpperCase();if(!p(this._customFunctionMappingsUpperCase[t]))return!0;if("undefined"!=typeof window){for(var n=window,r=e.split("."),i=0;i<r.length-1;i++)if(n=n[r[i]],p(n)||"object"!=typeof n)return!1;if("function"==typeof n[r[r.length-1]])return!0}return!1},e.prototype._getCustomFunctionMappings=function(e){this._ensureCustomFunctionMappingsUpperCase();var t=e.toUpperCase();if(!p(this._customFunctionMappingsUpperCase[t])){if("function"==typeof this._customFunctionMappingsUpperCase[t])return this._customFunctionMappingsUpperCase[t];throw o.Utility.createRuntimeError(N.invalidOperation,o.Utility._getResourceString(o.ResourceStrings.customFunctionInvalidFunctionMapping,e),"CustomFunctionProxy._getCustomFunctionMappings")}},e.prototype._getFunction=function(e){return this._getCustomFunctionMappings(e)},e.prototype._invokeRemainingBatchEntries=function(e,t){o.Utility.log("CustomFunctionProxy._invokeRemainingBatchEntries");for(var n=Date.now(),r=t;r<e.length;r++){if(!(Date.now()-n<this._batchQuotaMillis)){o.Utility.log("setTimeout(CustomFunctionProxy._invokeRemainingBatchEntries)"),setTimeout(this._invokeRemainingBatchEntries.bind(this),0,e,r);break}this._invokeFunctionAndSetResult(e[r])}},e.prototype._invokeFunctionAndSetResult=function(t){var n,r=this;C.logEvent(e.CustomFunctionExecutionStartLog,t.functionName);var i=Date.now();try{n=t.isBatching?t.call.call(null,t.parameterValueSets):[t.call.apply(null,t.parameterValueSets[0])]}catch(n){var s=[];s.push(Date.now()-i);for(var a=0;a<t.invocationIds.length;a++)n instanceof P?this._setErrorWithOutboundData(t.invocationIds[a],n.message,this._getCustomFunctionResultErrorCodeFromErrorCode(n.code),s):this._setErrorWithOutboundData(t.invocationIds[a],n,2,s);return void C.logEvent(e.CustomFunctionExecutionExceptionThrownLog,t.functionName,e.toLogMessage(n))}var l=[];if(x.isFeatureEnabled(3)||l.push(Date.now()-i),t.isStreaming);else if(n.length===t.parameterValueSets.length){var u=function(o){p(n[o])||"object"!=typeof n[o]||"function"!=typeof n[o].then?(x.isFeatureEnabled(3)&&l.push(Date.now()-i),C.logEvent(e.CustomFunctionExecutionFinishLog,t.functionName),c._setResultWithOutboundData(t.invocationIds[o],n[o],l)):n[o].then((function(n){x.isFeatureEnabled(3)&&l.push(Date.now()-i),C.logEvent(e.CustomFunctionExecutionFinishLog,t.functionName),r._setResultWithOutboundData(t.invocationIds[o],n,l)}),(function(n){x.isFeatureEnabled(3)&&l.push(Date.now()-i),C.logEvent(e.CustomFunctionExecutionRejectedPromoseLog,t.functionName,e.toLogMessage(n)),n instanceof P?r._setErrorWithOutboundData(t.invocationIds[o],n,r._getCustomFunctionResultErrorCodeFromErrorCode(n.code),l):r._setErrorWithOutboundData(t.invocationIds[o],n,3,l)}))},c=this;for(a=0;a<n.length;a++)u(a)}else for(x.isFeatureEnabled(3)&&l.push(Date.now()-i),C.logEvent(e.CustomFunctionExecutionBatchMismatchLog,t.functionName),a=0;a<t.invocationIds.length;a++)this._setErrorWithOutboundData(t.invocationIds[a],o.Utility._getResourceString(o.ResourceStrings.customFunctionUnexpectedNumberOfEntriesInResultBatch),4,l)},e.prototype._setResult=function(e,t){var n=[];n.push(0),this._setResultWithOutboundData(e,t,n)},e.prototype._setResultWithOutboundData=function(t,n,r){var i={id:t,value:n,outboundData:r};"number"==typeof n?isNaN(n)?(i.failed=!0,i.value="NaN"):isFinite(n)||(i.failed=!0,i.value="Infinity",i.errorCode=6):n instanceof P||n instanceof A||n instanceof F||n instanceof D?(i.failed=!1,i.value=n.toJSON()):n instanceof Error&&(i.failed=!0,i.value=e.toLogMessage(n),i.errorCode=0);var o=Date.now();this._resultEntryBuffer[t]={timeCreated:o,result:i},this._ensureSetResultsTaskIsScheduled(o)},e.prototype._setError=function(e,t,n){var r=[];r.push(0),this._setErrorWithOutboundData(e,t,n,r)},e.prototype._setErrorWithOutboundData=function(e,t,n,r){var i="";p(t)||(t instanceof P&&!p(t.message)?i=t.message:"string"==typeof t&&(i=t));var o={id:e,failed:!0,value:i,errorCode:n,outboundData:r},s=Date.now();this._resultEntryBuffer[e]={timeCreated:s,result:o},this._ensureSetResultsTaskIsScheduled(s)},e.prototype._getCustomFunctionResultErrorCodeFromErrorCode=function(e){var t;switch(e){case v.notAvailable:t=1;break;case v.divisionByZero:t=5;break;case v.invalidValue:t=7;break;case v.invalidNumber:t=6;break;case v.nullReference:t=8;break;default:t=7}return t},e.prototype._ensureSetResultsTaskIsScheduled=function(e){if(this._setResultsTaskOverdueTime>0&&e>this._setResultsTaskOverdueTime)return o.Utility.log("SetResultsTask overdue"),void this._executeSetResultsTask();this._isSetResultsTaskScheduled||(o.Utility.log("setTimeout(CustomFunctionProxy._executeSetResultsTask)"),setTimeout(this._executeSetResultsTask.bind(this),this._setResultsDelayMillis),this._isSetResultsTaskScheduled=!0,this._setResultsTaskOverdueTime=e+this._setResultsDelayMillis+this._setResultsOverdueDelayMillis)},e.prototype._convertCustomFunctionInvocationResultToArray=function(e){var t=[];return t.push(e.id),t.push(!e.failed),o.CoreUtility.isUndefined(e.value)?t.push(null):t.push(e.value),e.failed&&(o.CoreUtility.isUndefined(e.errorCode)?t.push(0):t.push(e.errorCode)),o.Utility.isSetSupported("CustomFunctions","1.9")&&t.push(e.outboundData),t},e.prototype._executeSetResultsTask=function(){var e=this;o.Utility.log("CustomFunctionProxy._executeSetResultsTask");var t=Date.now();if(this._inProgressContextSyncExpectedFinishTime>0&&this._inProgressContextSyncExpectedFinishTime>t)return o.Utility.log("context.sync() is in progress. setTimeout(CustomFunctionProxy._executeSetResultsTask)"),setTimeout(this._executeSetResultsTask.bind(this),this._setResultsDelayMillis),void(this._setResultsTaskOverdueTime=t+this._setResultsDelayMillis+this._setResultsOverdueDelayMillis);this._isSetResultsTaskScheduled=!1,this._setResultsTaskOverdueTime=0;var n={},r=!1;x.isFeatureEnabled(1)&&Object.keys(this._resultEntryRetryBuffer).length>0?(n=this._resultEntryRetryBuffer,this._resultEntryRetryBuffer={},this._retryBufferBodySize=2,r=!0):(n=this._resultEntryBuffer,this._resultEntryBuffer={});var i=o.Utility.isSetSupported("CustomFunctions","1.7"),s=[];for(var a in n)i?s.push(this._convertCustomFunctionInvocationResultToArray(n[a].result)):s.push(n[a].result);if(0!==s.length){var l=new _;i?l.customFunctions.setInvocationArrayResults(s):l.customFunctions.setInvocationResults(s);var u=Date.now();this._inProgressContextSyncExpectedFinishTime=u+this._maxContextSyncExecutionDurationMills,l.sync().then((function(t){var n=Date.now();e._clearInProgressContextSyncExpectedFinishTimeAfterMinInterval(n-u),r&&e._ensureSetResultsTaskIsScheduled(n)}),(function(t){var r=Date.now();e._clearInProgressContextSyncExpectedFinishTimeAfterMinInterval(r-u),x.isFeatureEnabled(1)&&t.code==o.ErrorCodes.requestPayloadSizeLimitExceeded?e._restoreAndSliceResultEntries(r,n):e._restoreResultEntries(r,n),e._ensureSetResultsTaskIsScheduled(r)}))}},e.prototype._restoreResultEntries=function(e,t){for(var n in t){var r=t[n];e-r.timeCreated<=this._setResultsLifeMillis&&(this._resultEntryBuffer[n]||(this._resultEntryBuffer[n]=r))}},e.prototype._restoreAndSliceResultEntries=function(e,t){for(var n in t){var r=t[n],i=void 0;if(e-r.timeCreated<=this._setResultsLifeMillis&&!this._resultEntryBuffer[n]){try{i=JSON.stringify(this._convertCustomFunctionInvocationResultToArray(r.result)).length+1}catch(e){this._resultEntryBuffer[n]=r,o.Utility.log("Result can't pass in JSON.stringify");continue}this._retryBufferBodySize+i<this._maxResultSize?(this._resultEntryRetryBuffer[n]=r,this._retryBufferBodySize+=i):this._resultEntryBuffer[n]=r}}},e.prototype._clearInProgressContextSyncExpectedFinishTimeAfterMinInterval=function(e){var t=this,n=x.isFeatureEnabled(2)?this._smallerMinContextSyncIntervalMills:this._minContextSyncIntervalMills,r=Math.max(n,2*e),i=x.isFeatureEnabled(4)?Math.min(this._maxContextSyncIntervalMills,r):r;o.Utility.log("setTimeout(clearInProgressContestSyncExpectedFinishedTime,"+r+")"),setTimeout((function(){o.Utility.log("clearInProgressContestSyncExpectedFinishedTime"),t._inProgressContextSyncExpectedFinishTime=0}),i)},e.CustomFunctionExecutionStartLog=new S(b.Verbose,"CustomFunctions [Execution] [Begin] Function="),e.CustomFunctionExecutionFailureLog=new S(b.Error,"CustomFunctions [Execution] [End] [Failure] Function="),e.CustomFunctionExecutionRejectedPromoseLog=new S(b.Error,"CustomFunctions [Execution] [End] [Failure] [RejectedPromise] Function="),e.CustomFunctionExecutionExceptionThrownLog=new S(b.Error,"CustomFunctions [Execution] [End] [Failure] [ExceptionThrown] Function="),e.CustomFunctionExecutionBatchMismatchLog=new S(b.Error,"CustomFunctions [Execution] [End] [Failure] [BatchMismatch] Function="),e.CustomFunctionExecutionFinishLog=new S(b.Info,"CustomFunctions [Execution] [End] [Success] Function="),e.CustomFunctionExecutionNotFoundLog=new S(b.Error,"CustomFunctions [Execution] [NotFound] Function="),e.CustomFunctionCancellation=new S(b.Info,"CustomFunctions [Cancellation] Function="),e.CustomFunctionAlreadyCancelled=new S(b.Info,"CustomFunctions [AlreadyCancelled] Function="),e.CustomFunctionDuplicatedName=new S(b.Error,"CustomFunctions [DuplicatedName] Function="),e.CustomFunctionInvalidArg=new S(b.Error,"CustomFunctions [InvalidArg] Name="),e}();t.CustomFunctionProxy=I,t.customFunctionProxy=new I,t.setCustomFunctionAssociation=t.customFunctionProxy.setCustomFunctionAssociation.bind(t.customFunctionProxy),t.setCustomFunctionInvoker=t.customFunctionProxy.setCustomFunctionInvoker.bind(t.customFunctionProxy),s.HostBridge.onInited((function(e){t.customFunctionProxy._initFromHostBridge(e)}));var x=function(e){function n(){return null!==e&&e.apply(this,arguments)||this}return i(n,e),Object.defineProperty(n.prototype,"_className",{get:function(){return"CustomFunctions"},enumerable:!1,configurable:!0}),n.isFeatureEnabled=function(e){return this._enabledFeature instanceof Array&&this._enabledFeature.indexOf(e)>=0},n.initialize=function(){var e=this,r=new _;return t.customFunctionProxy.ensureInit(r).then((function(){var i;return r.customFunctions._SetOsfControlContainerReadyForCustomFunctions(),o.Utility.isSetSupported("CustomFunctions","1.9")&&(i=r.customFunctions._GetEnabledFeatures()),o.Utility.log("OsfControl activation lifecycle: Set OsfControlContainer ready for CustomFunctions"),r._customData="SetOsfControlContainerReadyForCustomFunctions",r.sync().then((function(){p(i)||(e._enabledFeature=i.value)})).catch((function(e){!function(e,r){var i=e instanceof o.Error&&e.code===N.invalidOperationInCellEditMode;if(o.CoreUtility.log("Error on starting custom functions: "+e),i){o.CoreUtility.log("Was in cell-edit mode, will try again");var s=t.customFunctionProxy._ensureInitRetryDelayMillis;return new o.CoreUtility.Promise((function(e){return setTimeout(e,s)})).then((function(){return n.initialize()}))}throw e}(e)}))}))},n.prototype.setInvocationArrayResults=function(e){f("CustomFunctions.setInvocationArrayResults","CustomFunctions","1.4","Excel"),c(this,"SetInvocationArrayResults",0,[e],2,0)},n.prototype.setInvocationResults=function(e){c(this,"SetInvocationResults",0,[e],2,0)},n.prototype._GetEnabledFeatures=function(){return f("CustomFunctions._GetEnabledFeatures","CustomFunctions","1.4","Excel"),c(this,"_GetEnabledFeatures",1,[],4,0)},n.prototype._SetInvocationError=function(e,t){c(this,"_SetInvocationError",0,[e,t],2,0)},n.prototype._SetInvocationResult=function(e,t){c(this,"_SetInvocationResult",0,[e,t],2,0)},n.prototype._SetOsfControlContainerReadyForCustomFunctions=function(){c(this,"_SetOsfControlContainerReadyForCustomFunctions",0,[],10,0)},n.prototype._handleResult=function(t){e.prototype._handleResult.call(this,t),p(t)||y(this,t)},n.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),g(this,t,n)},n.newObject=function(e){return u(n,e,"Microsoft.ExcelServices.CustomFunctions",!1,4)},n.prototype.toJSON=function(){return d(this,{},{})},n}(o.ClientObject);t.CustomFunctions=x;var N,T=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return i(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"CustomFunctionsContainer"},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"_navigationPropertyNames",{get:function(){return["customFunctions"]},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"customFunctions",{get:function(){return f("CustomFunctionsContainer.customFunctions","CustomFunctions","1.2","Excel"),this._C||(this._C=a(x,this,"CustomFunctions",!1,4)),this._C},enumerable:!1,configurable:!0}),t.prototype._handleResult=function(t){if(e.prototype._handleResult.call(this,t),!p(t)){var n=t;y(this,n),m(this,n,["customFunctions","CustomFunctions"])}},t.prototype.load=function(e){return h(this,e)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),g(this,t,n)},t.prototype.toJSON=function(){return d(this,{},{})},t}(o.ClientObject);t.CustomFunctionsContainer=T,function(e){e.generalException="GeneralException",e.invalidOperation="InvalidOperation",e.invalidOperationInCellEditMode="InvalidOperationInCellEditMode"}(N||(N={}))}]),window.OfficeExtensionBatch=window.OfficeExtension,function(e){var t={};function n(r){if(t[r])return t[r].exports;var i=t[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}n.m=e,n.c=t,n.d=function(e,t,r){n.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:r})},n.r=function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},n.t=function(e,t){if(1&t&&(e=n(e)),8&t)return e;if(4&t&&"object"==typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var i in e)n.d(r,i,function(t){return e[t]}.bind(null,i));return r},n.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="",n(n.s=1)}([function(e,t){e.exports=OfficeExtensionBatch},function(e,t,n){"use strict";var r=this&&this.__assign||function(){return(r=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};Object.defineProperty(t,"__esModule",{value:!0});var i=n(2),o=n(3),s=n(5),a=n(6),l=n(7),u=n(8),c=n(9),p=n(10),f=n(13),h=n(14);window._useSDXSandbox?window._OfficeRuntimeNative=r(r({},window.OfficeRuntime),{displayWebDialog:a.displayWebDialog,AsyncStorage:l.AsyncStorage,storage:l.storage,experimentation:c.experimentation,apiInformation:o.apiInformation,message:p.Message.instance,ui:f.ui,auth:h.Auth.instance}):window._OfficeRuntimeNative=r(r({},window.OfficeRuntime),{displayWebDialog:a.displayWebDialog,AsyncStorage:s,storage:u.storage,experimentation:c.experimentation,apiInformation:i.apiInformation,message:p.Message.instance,ui:f.ui,auth:h.Auth.instance}),Object.freeze(window._OfficeRuntimeNative)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.apiInformation=void 0,t.apiInformation={isSetSupported:function(e,t){try{return!!(Office&&Office.context&&Office.context.requirements)&&Office.context.requirements.isSetSupported(e,t)}catch(e){return!1}}}},function(e,t,n){"use strict";(function(e){function n(e){var t,n=e.split("."),r=0,i=0;if(n.length<2&&isNaN(Number(e)))throw"version format incorrect";if(t=Number(n[0]),n.length>=2&&(r=Number(n[1])),n.length>=3&&(i=Number(n[2])),isNaN(t)||isNaN(r)||isNaN(i))throw"version format incorrect";return{major:t,minor:r,patch:i}}Object.defineProperty(t,"__esModule",{value:!0}),t.apiInformation=void 0,t.apiInformation={isSetSupported:function(t,r){if("string"!=typeof t)return!1;if(null==r&&(r="0.0.0"),void 0===e.__apiSets)return!1;var i=e.__apiSets,o=Object.keys(i).map((function(e){return e.toLowerCase()})).indexOf(t.toLowerCase());if(o>-1){var s=i[Object.keys(i)[o]];try{var a,l=n(r);if((a=Array.isArray(s)?function(e,t){for(var r=0;r<e.length;r++){var i=n(e[r]);if(i.major==t)return i}return{major:0,minor:0,patch:0}}(s,l.major):n(s)).major>0){if(a.major>l.major)return!0;if(a.major==l.major&&a.minor>l.minor)return!0;if(a.major==l.major&&a.minor==l.minor&&a.patch>=l.patch)return!0}}catch(e){return!1}}return!1}}}).call(this,n(4))},function(e,t){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(e){"object"==typeof window&&(n=window)}e.exports=n},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.multiGet=t.multiRemove=t.multiSet=t.getAllKeys=t.clear=t.removeItem=t.setItem=t.getItem=void 0;var r="_Office_AsyncStorage_";function i(){window.localStorage.setItem("_Office_AsyncStorage_|_unusedKey_",null),window.localStorage.removeItem("_Office_AsyncStorage_|_unusedKey_")}function o(e,t){return void 0===t&&(t=function(){}),new Promise((function(n,r){try{i(),e(),t(null),n()}catch(e){t(e),r(e)}}))}function s(e,t){return void 0===t&&(t=function(){}),new Promise((function(n,r){try{i();var o=e();t(null,o),n(o)}catch(e){t(e,null),r(e)}}))}function a(e,t,n){return void 0===n&&(n=function(){}),new Promise((function(r,o){var s=[];try{i()}catch(e){s.push(e)}e.forEach((function(e){try{t(e)}catch(e){s.push(e)}})),n(s),s.length>0?o(s):r()}))}t.getItem=function(e,t){return s((function(){return window.localStorage.getItem(r+e)}),t)},t.setItem=function(e,t,n){return o((function(){return window.localStorage.setItem(r+e,t)}),n)},t.removeItem=function(e,t){return o((function(){return window.localStorage.removeItem(r+e)}),t)},t.clear=function(e){return o((function(){Object.keys(window.localStorage).filter((function(e){return 0===e.indexOf(r)})).forEach((function(e){return window.localStorage.removeItem(e)}))}),e)},t.getAllKeys=function(e){return s((function(){return Object.keys(window.localStorage).filter((function(e){return 0===e.indexOf(r)})).map((function(e){return e.substr(r.length)}))}),e)},t.multiSet=function(e,t){return a(e,(function(e){var t=e[0],n=e[1];return window.localStorage.setItem(r+t,n)}),t)},t.multiRemove=function(e,t){return a(e,(function(e){return window.localStorage.removeItem(r+e)}),t)},t.multiGet=function(e,t){return new Promise((function(n,i){t||(t=function(){});var o=[],s=e.map((function(e){try{return[e,window.localStorage.getItem(r+e)]}catch(e){o.push(e)}})).filter((function(e){return e}));o.length>0?(t(o,s),i(o)):(t(null,s),n(s))}))}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.displayWebDialog=t.Dialog=void 0;var r=n(0),i=function(){function e(e){this._dialog=e}return e.prototype.close=function(){return this._dialog.close(),r.CoreUtility.Promise.resolve()},e}();t.Dialog=i,t.displayWebDialog=function(e,t){return new r.CoreUtility.Promise((function(n,o){if(t.width&&t.height&&(!c(t.width)||!c(t.height)))throw new r.Error({code:"InvalidArgument",message:'Dimensions must be "number%" or number.'});var s,a={width:t.width?parseInt(t.width,10):50,height:t.height?parseInt(t.height,10):50,displayInIframe:t.displayInIFrame||!1};function l(e){t.onMessage&&t.onMessage(e.message,s,e.origin)}function u(e){12006===e.error?t.onClose&&t.onClose():t.onRuntimeError&&t.onRuntimeError(new r.Error(p(e.error)),s)}function c(e){return/^(\-|\+)?([0-9]+)%?$/.test(e)}function p(e){var t,n=((t={})[12002]={code:"InvalidUrl",message:"Cannot load URL, no such page or bad URL syntax."},t[12003]={code:"InvalidUrl",message:"HTTPS is required."},t[12004]={code:"Untrusted",message:"Domain is not trusted."},t[12005]={code:"InvalidUrl",message:"HTTPS is required."},t[12007]={code:"FailedToOpen",message:"Another dialog is already opened."},t);return n[e]?n[e]:{code:"Unknown",message:"An unknown error has occured"}}Office.context.ui.displayDialogAsync(e,a,(function(e){"failed"===e.status?o(new r.Error(p(e.error.code))):((s=e.value).addEventHandler(Office.EventType.DialogMessageReceived,l),s.addEventHandler(Office.EventType.DialogEventReceived,u),n(new i(s)))}))}))}},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)});Object.defineProperty(t,"__esModule",{value:!0}),t.ErrorCodes=t.PersistentKvStorageService=t.storage=t.AsyncStorage=void 0;var o=n(0),s=(o.BatchApiHelper.createPropertyObject,o.BatchApiHelper.createMethodObject,o.BatchApiHelper.createIndexerObject,o.BatchApiHelper.createRootServiceObject,o.BatchApiHelper.createTopLevelServiceObject),a=(o.BatchApiHelper.createChildItemObject,o.BatchApiHelper.invokeMethod),l=(o.BatchApiHelper.invokeEnsureUnchanged,o.BatchApiHelper.invokeSetProperty,o.Utility.isNullOrUndefined),u=(o.Utility.isUndefined,o.Utility.throwIfNotLoaded,o.Utility.throwIfApiNotSupported,o.Utility.load,o.Utility.retrieve,o.Utility.toJson),c=o.Utility.fixObjectPathIfNecessary,p=(o.Utility._handleNavigationPropertyResults,o.Utility.adjustToDateTime,o.Utility.processRetrieveResult);function f(e,t){return new o.CoreUtility.Promise((function(n,r){var i=d.getInstance(),o=i.setCallBack((function(e,i){if(i)r(i);else{var o=t(e);n(o)}}));i.ctx.sync().then((function(){var t=i.getPersistentKvStorageService();return e(t,o),i.ctx.sync()})).catch((function(e){r(e)}))}))}function h(e,t,n){return new o.CoreUtility.Promise((function(r,i){var o=d.getInstance(),s=o.setCallBack((function(e,o){if(o)return n&&n(o),void i(o);var s=t(e);n&&n(null,s),r(s)}));o.ctx.sync().then((function(){var t=o.getPersistentKvStorageService();return e(t,s),o.ctx.sync()})).catch((function(e){i(e)}))}))}o.Utility.setMockData,o.CommonUtility.calculateApiFlags,t.AsyncStorage={getItem:function(e,t){return h((function(t,n){return t.multiGet(n,JSON.stringify([e]))}),(function(e){var t=JSON.parse(e);return t&&t[0]&&t[0][1]?t[0][1]:null}),t)},setItem:function(e,t,n){return h((function(n,r){return n.multiSet(r,JSON.stringify([[e,t]]))}),(function(){return null}),n)},removeItem:function(e,t){return h((function(t,n){return t.multiRemove(n,JSON.stringify([e]))}),(function(){return null}),t)},multiGet:function(e,t){return h((function(t,n){return t.multiGet(n,JSON.stringify(e))}),(function(t){var n=JSON.parse(t),r={};return n&&n.forEach((function(e){var t=e[0],n=e[1];return r[t]=n,n})),e.map((function(e){return[e,r[e]?r[e]:null]}))}),t)},multiSet:function(e,t){return h((function(t,n){return t.multiSet(n,JSON.stringify(e))}),(function(){return null}),t)},multiRemove:function(e,t){return h((function(t,n){return t.multiRemove(n,JSON.stringify(e))}),(function(){return null}),t)},getAllKeys:function(e){return h((function(e,t){return e.getAllKeys(t)}),(function(e){return JSON.parse(e)}),e)},clear:function(e){return h((function(e,t){return e.clear(t)}),(function(){return null}),e)}},t.storage={getItem:function(e){return f((function(t,n){return t.multiGet(n,JSON.stringify([e]))}),(function(e){var t=JSON.parse(e);return t&&t[0]&&t[0][1]?t[0][1]:null}))},setItem:function(e,t){return f((function(n,r){return n.multiSet(r,JSON.stringify([[e,t]]))}),(function(){return null}))},removeItem:function(e){return f((function(t,n){return t.multiRemove(n,JSON.stringify([e]))}),(function(){return null}))},getItems:function(e){return f((function(t,n){return t.multiGet(n,JSON.stringify(e))}),(function(t){var n=JSON.parse(t),r={};return e.forEach((function(e){r[e]=null})),n&&n.forEach((function(e){var t=e[0],n=e[1];return r[t]=n,n})),r}))},setItems:function(e){var t=[];for(var n in e)e.hasOwnProperty(n)&&t.push([n,e[n]]);return f((function(e,n){return e.multiSet(n,JSON.stringify(t))}),(function(){return null}))},removeItems:function(e){return f((function(t,n){return t.multiRemove(n,JSON.stringify(e))}),(function(){return null}))},getKeys:function(){return f((function(e,t){return e.getAllKeys(t)}),(function(e){return JSON.parse(e)}))}};var d=function(){function e(){var e=this;this._invokeId=0,this._callDict={},this.ctx=new o.ClientRequestContext,this._perkvstorService=y.newObject(this.ctx),this._eventResult=this._perkvstorService.onPersistentStorageMessage.add((function(t){o.Utility.log("persistentKvStoragegMessageHandler:"+JSON.stringify(t));var n=e._callDict[t.invokeId];n&&(n(t.message,t.error),delete e._callDict[t.invokeId])}))}return e.getInstance=function(){return void 0===e.instance?e.instance=new e:e.instance._perkvstorService=y.newObject(e.instance.ctx),e.instance},e.prototype.getPersistentKvStorageService=function(){return this._perkvstorService},e.prototype.getCallBack=function(e){return this._callDict[e]},e.prototype.setCallBack=function(e){var t=this._invokeId;return this._callDict[this._invokeId++]=e,t},e}(),y=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return i(t,e),Object.defineProperty(t.prototype,"_className",{get:function(){return"PersistentKvStorageService"},enumerable:!1,configurable:!0}),t.prototype.clear=function(e){a(this,"Clear",1,[e],4,0)},t.prototype.getAllKeys=function(e){a(this,"GetAllKeys",1,[e],4,0)},t.prototype.multiGet=function(e,t){a(this,"MultiGet",1,[e,t],4,0)},t.prototype.multiRemove=function(e,t){a(this,"MultiRemove",1,[e,t],4,0)},t.prototype.multiSet=function(e,t){a(this,"MultiSet",1,[e,t],4,0)},t.prototype._handleResult=function(t){e.prototype._handleResult.call(this,t),l(t)||c(this,t)},t.prototype._handleRetrieveResult=function(t,n){e.prototype._handleRetrieveResult.call(this,t,n),p(this,t,n)},t.newObject=function(e){return s(t,e,"Microsoft.PersistentKvStorage.PersistentKvStorageService",!1,4)},Object.defineProperty(t.prototype,"onPersistentStorageMessage",{get:function(){return this.m_persistentStorageMessage||(this.m_persistentStorageMessage=new o.GenericEventHandlers(this.context,this,"PersistentStorageMessage",{eventType:65537,registerFunc:function(){},unregisterFunc:function(){},getTargetIdFunc:function(){return null},eventArgsTransformFunc:function(e){var t,n,r,i;try{var s=JSON.parse(e.message),a=parseInt(s.errorCode),l=0!=a?new o.Error((n=a,(r={})[16389]={code:"GenericException",message:"Unknown error."},r[65535]={code:"Unexcepted",message:"Catastrophic failure."},r[14]={code:"OutOfMemory",message:"Ran out of memory."},r[87]={code:"InvalidArg",message:"One or more arguments are invalid."},r[16385]={code:"NotImplemented",message:"Not implemented."},r[6]={code:"BadHandle",message:"File Handle is not Set."},r[5]={code:"AccessDenied",message:"Can't read the Storage File."},(i=r)[n]?i[n]:{code:"Unknown",message:"An unknown error has occured"})):null;t={invokeId:s.invokeId,message:s.message,error:l}}catch(e){t={invokeId:-1,message:e.message,error:new o.Error({code:"GenericException",message:"Unknown error"})}}return o.Utility._createPromiseFromResult(t)}})),this.m_persistentStorageMessage},enumerable:!1,configurable:!0}),t.prototype.toJSON=function(){return u(this,{},{})},t}(o.ClientObject);t.PersistentKvStorageService=y,(t.ErrorCodes||(t.ErrorCodes={})).generalException="GeneralException"},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.storage=void 0;var r="_OfficeRuntime_Storage_";function i(){window.localStorage.setItem("_OfficeRuntime_Storage_|_unusedKey_",null),window.localStorage.removeItem("_OfficeRuntime_Storage_|_unusedKey_")}function o(e){return new Promise((function(t,n){try{i(),e(),t()}catch(e){n(e)}}))}function s(e){return new Promise((function(t,n){try{i(),t(e())}catch(e){n(e)}}))}function a(e,t){return new Promise((function(n,r){var o=[];try{i()}catch(e){o.push(e)}for(var s in e)if(e.hasOwnProperty(s)||Array.isArray(e))try{Array.isArray(e)?t(e[s]):t(s)}catch(e){o.push(e)}o.length>0?r(new Error("Unknown error.")):n()}))}t.storage={getItem:function(e){return s((function(){return window.localStorage.getItem(r+e)}))},setItem:function(e,t){return o((function(){return window.localStorage.setItem(r+e,t)}))},removeItem:function(e){return o((function(){return window.localStorage.removeItem(r+e)}))},getItems:function(e){return new Promise((function(t,n){var o={},s=[];try{i()}catch(e){n(e)}e.forEach((function(e){try{var t=window.localStorage.getItem(r+e);o[e]=t||null}catch(e){s.push(e)}})),s.length>0?n(new Error("Unknown error.")):t(o)}))},setItems:function(e){return a(e,(function(t){return window.localStorage.setItem(r+t,e[t])}))},removeItems:function(e){return a(e,(function(e){window.localStorage.removeItem(r+e)}))},getKeys:function(){return s((function(){return Object.keys(window.localStorage).filter((function(e){return 0===e.indexOf(r)})).map((function(e){return e.substr(r.length)}))}))}}},function(e,t,n){"use strict";var r=this&&this.__awaiter||function(e,t,n,r){return new(n||(n=Promise))((function(i,o){function s(e){try{l(r.next(e))}catch(e){o(e)}}function a(e){try{l(r.throw(e))}catch(e){o(e)}}function l(e){var t;e.done?i(e.value):(t=e.value,t instanceof n?t:new n((function(e){e(t)}))).then(s,a)}l((r=r.apply(e,t||[])).next())}))},i=this&&this.__generator||function(e,t){var n,r,i,o,s={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return o={next:a(0),throw:a(1),return:a(2)},"function"==typeof Symbol&&(o[Symbol.iterator]=function(){return this}),o;function a(o){return function(a){return function(o){if(n)throw new TypeError("Generator is already executing.");for(;s;)try{if(n=1,r&&(i=2&o[0]?r.return:o[0]?r.throw||((i=r.return)&&i.call(r),0):r.next)&&!(i=i.call(r,o[1])).done)return i;switch(r=0,i&&(o=[2&o[0],i.value]),o[0]){case 0:case 1:i=o;break;case 4:return s.label++,{value:o[1],done:!1};case 5:s.label++,r=o[1],o=[0];continue;case 7:o=s.ops.pop(),s.trys.pop();continue;default:if(!((i=(i=s.trys).length>0&&i[i.length-1])||6!==o[0]&&2!==o[0])){s=0;continue}if(3===o[0]&&(!i||o[1]>i[0]&&o[1]<i[3])){s.label=o[1];break}if(6===o[0]&&s.label<i[1]){s.label=i[1],i=o;break}if(i&&s.label<i[2]){s.label=i[2],s.ops.push(o);break}i[2]&&s.ops.pop(),s.trys.pop();continue}o=t.call(e,s)}catch(e){o=[6,e],r=0}finally{n=i=0}if(5&o[0])throw o[1];return{value:o[0]?o[1]:void 0,done:!0}}([o,a])}}};Object.defineProperty(t,"__esModule",{value:!0}),t.experimentation=void 0;var o=function(){function e(){}return e.prototype.getBooleanFeatureGate=function(e,t){try{var n=Microsoft.Office.WebExtension.FeatureGates[e];return"true"===n.toString().toLowerCase()||!0===n}catch(e){return t}},e.prototype.getIntFeatureGate=function(e,t){try{var n=parseInt(Microsoft.Office.WebExtension.FeatureGates[e]);return isNaN(n)?t:n}catch(e){return t}},e.prototype.getStringFeatureGate=function(e,t){try{var n=Microsoft.Office.WebExtension.FeatureGates[e];return null==n?t:n}catch(e){return t}},e.prototype.getChangeGate=function(e){return this.getBooleanFeatureGate(e,!0)},e.prototype.getBooleanFeatureGateAsync=function(e,t){return r(this,void 0,void 0,(function(){var n,r;return i(this,(function(i){switch(i.label){case 0:return i.trys.push([0,2,,3]),n=this.getRequestContext(),r=n.flighting.getFeature(e,"Boolean",t),n.load(r),[4,n.sync()];case 1:return i.sent(),[2,r.value];case 2:return i.sent(),[2,Promise.resolve(t)];case 3:return[2]}}))}))},e.prototype.getStringFeatureGateAsync=function(e,t){return r(this,void 0,void 0,(function(){var n,r;return i(this,(function(i){switch(i.label){case 0:return i.trys.push([0,2,,3]),n=this.getRequestContext(),r=n.flighting.getFeature(e,"String",t),n.load(r),[4,n.sync()];case 1:return i.sent(),[2,r.value];case 2:return i.sent(),[2,Promise.resolve(t)];case 3:return[2]}}))}))},e.prototype.getIntFeatureGateAsync=function(e,t){return r(this,void 0,void 0,(function(){var n,r;return i(this,(function(i){switch(i.label){case 0:return i.trys.push([0,2,,3]),n=this.getRequestContext(),r=n.flighting.getFeature(e,"Integer",t),n.load(r),[4,n.sync()];case 1:return i.sent(),[2,r.value];case 2:return i.sent(),[2,Promise.resolve(t)];case 3:return[2]}}))}))},e.prototype.getRequestContext=function(){var e;if(Office.context.platform!==Office.PlatformType.OfficeOnline){switch(Office.context.host){case Office.HostType.Excel:e=new Excel.RequestContext;break;case Office.HostType.OneNote:e=new OneNote.RequestContext;break;case Office.HostType.PowerPoint:e=new PowerPoint.RequestContext;break;case Office.HostType.Word:e=new Word.RequestContext;break;default:throw new Error("Unknown Application "+Office.context.host)}return e}},e}();t.experimentation=new o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Message=void 0;var r=n(11);Object.defineProperty(t,"Message",{enumerable:!0,get:function(){return r.Message}})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Message=void 0;var r=n(12),i=function(){function e(){this.listeners={}}return e.prototype.on=function(e,t){return this.add(e,t),new Promise((function(e){e()}))},e.prototype.off=function(e,t){return this.remove(e,t),new Promise((function(e){e()}))},e.prototype.emit=function(e,t){return this.send(e,t),new Promise((function(e){e()}))},Object.defineProperty(e,"instance",{get:function(){return e.singleton||(e.singleton=new e),this.singleton},enumerable:!1,configurable:!0}),e.prototype.setupReceive=function(){Office&&Office.context&&Office.context.messaging&&!Office.context.messaging.onMessage&&(Office.context.messaging.onMessage=this.receiveMessage.bind(this))},e.prototype.add=function(e,t){this.listeners.hasOwnProperty(e)||(this.listeners[e]=new r.ListenerManager,this.setupReceive()),this.listeners[e].add(t)},e.prototype.remove=function(e,t){this.listeners.hasOwnProperty(e)&&(t?this.listeners[e].remove(t):delete this.listeners[e])},e.prototype.send=function(e,t){var n={eventName:e,message:t};Office&&Office.context&&Office.context.messaging&&Office.context.messaging.sendMessage&&Office.context.messaging.sendMessage(n)},e.prototype.receiveMessage=function(e){var t=e.eventName,n=e.message;this.listeners.hasOwnProperty(t)&&this.listeners[t].call(n)},e}();t.Message=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.ListenerManager=void 0;var r=function(){function e(){this.listeners=[]}return e.prototype.add=function(e){this.listeners.push(e)},e.prototype.remove=function(e){var t=this.listeners.lastIndexOf(e);-1!==t&&this.listeners.splice(t,1)},e.prototype.call=function(e){this.listeners.forEach((function(t){return t(e)}))},e}();t.ListenerManager=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Ribbon=t.ui=void 0,t.ui={getRibbon:function(){return new Promise((function(e,t){e(new r)}))}};var r=function(){function e(){this.requestContext=new OfficeCore.RequestContext,OSF.WebAuth&&"web"==OSF._OfficeAppFactory.getHostInfo().hostPlatform&&(this.requestContext._customData="WacPartition")}return e.prototype.requestUpdate=function(e){var t=this.requestContext.ribbon;return e.tabs.filter((function(e){return!!e.id})).forEach((function(e){var n=t.getTab(e.id);void 0!==e.visible&&null!==e.visible&&n.setVisibility(e.visible),e.controls.filter((function(e){return!!e.id})).forEach((function(e){var n=t.getButton(e.id);void 0!==e.enabled&&null!==e.enabled&&(n.enabled=e.enabled)}))})),this.requestContext.sync()},e}();t.Ribbon=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Auth=void 0;var r=n(0),i=function(){function e(){}return e.prototype.getAccessToken=function(e){return new r.CoreUtility.Promise((function(t,n){try{Office.context.auth.getAccessTokenAsync(e||{},(function(e){"succeeded"===e.status?t(e.value):n(e.error)}))}catch(e){n(e)}}))},Object.defineProperty(e,"instance",{get:function(){return e.singleton||(e.singleton=new e),e.singleton},enumerable:!1,configurable:!0}),e}();t.Auth=i}]),window.OfficeRuntime=window._OfficeRuntimeNative,exposeOfficeRuntimeThroughOfficeNamespace("undefined"!=typeof OfficeRuntime&&OfficeRuntime||void 0,void 0!==Office&&Office||void 0),"undefined"!=typeof OSFPerformance&&(OSFPerformance.hostInitializationEnd=OSFPerformance.now(),OSFPerformance.totalJSHeapSize=OSFPerformance.getTotalJSHeapSize(),OSFPerformance.usedJSHeapSize=OSFPerformance.getUsedJSHeapSize(),OSFPerformance.jsHeapSizeLimit=OSFPerformance.getJSHeapSizeLimit()),OfficeExtension.Utility._doApiNotSupportedCheck=!0;