!function(e,t){"object"==typeof exports&&"object"==typeof module?module.exports=t():"function"==typeof define&&define.amd?define("BrowserAuth",[],t):"object"==typeof exports?exports.BrowserAuth=t():e.BrowserAuth=t()}(window,function(){return function(e){var t={};function r(n){if(t[n])return t[n].exports;var o=t[n]={i:n,l:!1,exports:{}};return e[n].call(o.exports,o,o.exports,r),o.l=!0,o.exports}return r.m=e,r.c=t,r.d=function(e,t,n){r.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:n})},r.r=function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},r.t=function(e,t){if(1&t&&(e=r(e)),8&t)return e;if(4&t&&"object"==typeof e&&e&&e.__esModule)return e;var n=Object.create(null);if(r.r(n),Object.defineProperty(n,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var o in e)r.d(n,o,function(t){return e[t]}.bind(null,o));return n},r.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return r.d(t,"a",t),t},r.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},r.p="",r(r.s=0)}({"./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js":function(e,t,r){"use strict";r.r(t),r.d(t,"__assign",function(){return i}),r.d(t,"__awaiter",function(){return s}),r.d(t,"__extends",function(){return o}),r.d(t,"__generator",function(){return a}),r.d(t,"__read",function(){return c}),r.d(t,"__spread",function(){return u});
/*! @azure/msal-browser v2.19.0 2021-11-02 */
/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
var n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&(e[r]=t[r])})(e,t)};function o(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}var i=function(){return(i=Object.assign||function(e){for(var t,r=1,n=arguments.length;r<n;r++)for(var o in t=arguments[r])Object.prototype.hasOwnProperty.call(t,o)&&(e[o]=t[o]);return e}).apply(this,arguments)};function s(e,t,r,n){return new(r||(r=Promise))(function(o,i){function s(e){try{c(n.next(e))}catch(e){i(e)}}function a(e){try{c(n.throw(e))}catch(e){i(e)}}function c(e){var t;e.done?o(e.value):(t=e.value,t instanceof r?t:new r(function(e){e(t)})).then(s,a)}c((n=n.apply(e,t||[])).next())})}function a(e,t){var r,n,o,i,s={label:0,sent:function(){if(1&o[0])throw o[1];return o[1]},trys:[],ops:[]};return i={next:a(0),throw:a(1),return:a(2)},"function"==typeof Symbol&&(i[Symbol.iterator]=function(){return this}),i;function a(i){return function(a){return function(i){if(r)throw new TypeError("Generator is already executing.");for(;s;)try{if(r=1,n&&(o=2&i[0]?n.return:i[0]?n.throw||((o=n.return)&&o.call(n),0):n.next)&&!(o=o.call(n,i[1])).done)return o;switch(n=0,o&&(i=[2&i[0],o.value]),i[0]){case 0:case 1:o=i;break;case 4:return s.label++,{value:i[1],done:!1};case 5:s.label++,n=i[1],i=[0];continue;case 7:i=s.ops.pop(),s.trys.pop();continue;default:if(!(o=(o=s.trys).length>0&&o[o.length-1])&&(6===i[0]||2===i[0])){s=0;continue}if(3===i[0]&&(!o||i[1]>o[0]&&i[1]<o[3])){s.label=i[1];break}if(6===i[0]&&s.label<o[1]){s.label=o[1],o=i;break}if(o&&s.label<o[2]){s.label=o[2],s.ops.push(i);break}o[2]&&s.ops.pop(),s.trys.pop();continue}i=t.call(e,s)}catch(e){i=[6,e],n=0}finally{r=o=0}if(5&i[0])throw i[1];return{value:i[0]?i[1]:void 0,done:!0}}([i,a])}}}function c(e,t){var r="function"==typeof Symbol&&e[Symbol.iterator];if(!r)return e;var n,o,i=r.call(e),s=[];try{for(;(void 0===t||t-- >0)&&!(n=i.next()).done;)s.push(n.value)}catch(e){o={error:e}}finally{try{n&&!n.done&&(r=i.return)&&r.call(i)}finally{if(o)throw o.error}}return s}function u(){for(var e=[],t=0;t<arguments.length;t++)e=e.concat(c(arguments[t]));return e}},"./node_modules/@azure/msal-browser/dist/app/ClientApplication.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ClientApplication",function(){return E});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-browser/dist/crypto/CryptoOps.js"),i=r("./node_modules/@azure/msal-common/dist/index.js"),s=r("./node_modules/@azure/msal-browser/dist/cache/BrowserCacheManager.js"),a=r("./node_modules/@azure/msal-browser/dist/config/Configuration.js"),c=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),u=r("./node_modules/@azure/msal-browser/dist/utils/BrowserUtils.js"),d=r("./node_modules/@azure/msal-browser/dist/packageMetadata.js"),l=r("./node_modules/@azure/msal-browser/dist/event/EventType.js"),h=r("./node_modules/@azure/msal-browser/dist/error/BrowserConfigurationAuthError.js"),p=r("./node_modules/@azure/msal-browser/dist/event/EventHandler.js"),m=r("./node_modules/@azure/msal-browser/dist/interaction_client/PopupClient.js"),g=r("./node_modules/@azure/msal-browser/dist/interaction_client/RedirectClient.js"),f=r("./node_modules/@azure/msal-browser/dist/interaction_client/SilentIframeClient.js"),y=r("./node_modules/@azure/msal-browser/dist/interaction_client/SilentRefreshClient.js"),C=r("./node_modules/@azure/msal-browser/dist/cache/TokenCache.js"),E=function(){function e(e){this.isBrowserEnvironment="undefined"!=typeof window,this.config=Object(a.buildConfiguration)(e,this.isBrowserEnvironment),this.logger=new i.Logger(this.config.system.loggerOptions,d.name,d.version),this.networkClient=this.config.system.networkClient,this.navigationClient=this.config.system.navigationClient,this.redirectResponse=new Map,this.browserCrypto=this.isBrowserEnvironment?new o.CryptoOps(this.logger):i.DEFAULT_CRYPTO_IMPLEMENTATION,this.eventHandler=new p.EventHandler(this.logger,this.browserCrypto),this.browserStorage=this.isBrowserEnvironment?new s.BrowserCacheManager(this.config.auth.clientId,this.config.cache,this.browserCrypto,this.logger):Object(s.DEFAULT_BROWSER_CACHE_MANAGER)(this.config.auth.clientId,this.logger),this.tokenCache=new C.TokenCache(this.config,this.browserStorage,this.logger,this.browserCrypto)}return e.prototype.handleRedirectPromise=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,o,s,a,u=this;return Object(n.__generator)(this,function(n){return this.logger.verbose("handleRedirectPromise called"),t=this.getAllAccounts(),this.isBrowserEnvironment?(r=e||i.Constants.EMPTY_STRING,void 0===(o=this.redirectResponse.get(r))?(this.eventHandler.emitEvent(l.EventType.HANDLE_REDIRECT_START,c.InteractionType.Redirect),this.logger.verbose("handleRedirectPromise has been called for the first time, storing the promise"),s=this.browserStorage.getTemporaryCache(c.TemporaryCacheKeys.CORRELATION_ID,!0)||"",a=new g.RedirectClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,s),o=a.handleRedirectPromise(e).then(function(e){e&&(t.length<u.getAllAccounts().length?(u.eventHandler.emitEvent(l.EventType.LOGIN_SUCCESS,c.InteractionType.Redirect,e),u.logger.verbose("handleRedirectResponse returned result, login success")):(u.eventHandler.emitEvent(l.EventType.ACQUIRE_TOKEN_SUCCESS,c.InteractionType.Redirect,e),u.logger.verbose("handleRedirectResponse returned result, acquire token success")));return u.eventHandler.emitEvent(l.EventType.HANDLE_REDIRECT_END,c.InteractionType.Redirect),e}).catch(function(e){throw t.length>0?u.eventHandler.emitEvent(l.EventType.ACQUIRE_TOKEN_FAILURE,c.InteractionType.Redirect,null,e):u.eventHandler.emitEvent(l.EventType.LOGIN_FAILURE,c.InteractionType.Redirect,null,e),u.eventHandler.emitEvent(l.EventType.HANDLE_REDIRECT_END,c.InteractionType.Redirect),e}),this.redirectResponse.set(r,o)):this.logger.verbose("handleRedirectPromise has been called previously, returning the result from the first call"),[2,o]):(this.logger.verbose("handleRedirectPromise returns null, not browser environment"),[2,null])})})},e.prototype.acquireTokenRedirect=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r=this;return Object(n.__generator)(this,function(n){return this.preflightBrowserEnvironmentCheck(c.InteractionType.Redirect),this.logger.verbose("acquireTokenRedirect called"),(t=this.getAllAccounts().length>0)?this.eventHandler.emitEvent(l.EventType.ACQUIRE_TOKEN_START,c.InteractionType.Redirect,e):this.eventHandler.emitEvent(l.EventType.LOGIN_START,c.InteractionType.Redirect,e),[2,new g.RedirectClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,e.correlationId).acquireToken(e).catch(function(e){throw t?r.eventHandler.emitEvent(l.EventType.ACQUIRE_TOKEN_FAILURE,c.InteractionType.Redirect,null,e):r.eventHandler.emitEvent(l.EventType.LOGIN_FAILURE,c.InteractionType.Redirect,null,e),e})]})})},e.prototype.acquireTokenPopup=function(e){var t=this;try{this.preflightBrowserEnvironmentCheck(c.InteractionType.Popup),this.logger.verbose("acquireTokenPopup called",e.correlationId)}catch(e){return Promise.reject(e)}var r=this.getAllAccounts();return r.length>0?this.eventHandler.emitEvent(l.EventType.ACQUIRE_TOKEN_START,c.InteractionType.Popup,e):this.eventHandler.emitEvent(l.EventType.LOGIN_START,c.InteractionType.Popup,e),new m.PopupClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,e.correlationId).acquireToken(e).then(function(e){return r.length<t.getAllAccounts().length?t.eventHandler.emitEvent(l.EventType.LOGIN_SUCCESS,c.InteractionType.Popup,e):t.eventHandler.emitEvent(l.EventType.ACQUIRE_TOKEN_SUCCESS,c.InteractionType.Popup,e),e}).catch(function(e){return r.length>0?t.eventHandler.emitEvent(l.EventType.ACQUIRE_TOKEN_FAILURE,c.InteractionType.Popup,null,e):t.eventHandler.emitEvent(l.EventType.LOGIN_FAILURE,c.InteractionType.Popup,null,e),Promise.reject(e)})},e.prototype.ssoSilent=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:this.preflightBrowserEnvironmentCheck(c.InteractionType.Silent),this.logger.verbose("ssoSilent called",e.correlationId),this.eventHandler.emitEvent(l.EventType.SSO_SILENT_START,c.InteractionType.Silent,e),n.label=1;case 1:return n.trys.push([1,3,,4]),[4,new f.SilentIframeClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,c.ApiId.ssoSilent,e.correlationId).acquireToken(e)];case 2:return t=n.sent(),this.eventHandler.emitEvent(l.EventType.SSO_SILENT_SUCCESS,c.InteractionType.Silent,t),[2,t];case 3:throw r=n.sent(),this.eventHandler.emitEvent(l.EventType.SSO_SILENT_FAILURE,c.InteractionType.Silent,null,r),r;case 4:return[2]}})})},e.prototype.acquireTokenByRefreshToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t=this;return Object(n.__generator)(this,function(r){return this.eventHandler.emitEvent(l.EventType.ACQUIRE_TOKEN_NETWORK_START,c.InteractionType.Silent,e),u.BrowserUtils.blockReloadInHiddenIframes(),[2,new y.SilentRefreshClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,e.correlationId).acquireToken(e).catch(function(r){var n=r instanceof i.ServerError,o=r instanceof i.InteractionRequiredAuthError,s=r.errorCode===c.BrowserConstants.INVALID_GRANT_ERROR;if(n&&s&&!o)return t.logger.verbose("Refresh token expired or invalid, attempting acquire token by iframe",e.correlationId),new f.SilentIframeClient(t.config,t.browserStorage,t.browserCrypto,t.logger,t.eventHandler,t.navigationClient,c.ApiId.acquireTokenSilent_authCode,e.correlationId).acquireToken(e);throw r})]})})},e.prototype.logout=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(t){return this.logger.warning("logout API is deprecated and will be removed in msal-browser v3.0.0. Use logoutRedirect instead."),[2,this.logoutRedirect(e)]})})},e.prototype.logoutRedirect=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(t){return this.preflightBrowserEnvironmentCheck(c.InteractionType.Redirect),[2,new g.RedirectClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,null==e?void 0:e.correlationId).logout(e)]})})},e.prototype.logoutPopup=function(e){try{return this.preflightBrowserEnvironmentCheck(c.InteractionType.Popup),new m.PopupClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,null==e?void 0:e.correlationId).logout(e)}catch(e){return Promise.reject(e)}},e.prototype.getAllAccounts=function(){return this.logger.verbose("getAllAccounts called"),this.isBrowserEnvironment?this.browserStorage.getAllAccounts():[]},e.prototype.getAccountByUsername=function(e){var t=this.getAllAccounts();return!i.StringUtils.isEmpty(e)&&t&&t.length?(this.logger.verbose("Account matching username found, returning"),this.logger.verbosePii("Returning signed-in accounts matching username: "+e),t.filter(function(t){return t.username.toLowerCase()===e.toLowerCase()})[0]||null):(this.logger.verbose("getAccountByUsername: No matching account found, returning null"),null)},e.prototype.getAccountByHomeId=function(e){var t=this.getAllAccounts();return!i.StringUtils.isEmpty(e)&&t&&t.length?(this.logger.verbose("Account matching homeAccountId found, returning"),this.logger.verbosePii("Returning signed-in accounts matching homeAccountId: "+e),t.filter(function(t){return t.homeAccountId===e})[0]||null):(this.logger.verbose("getAccountByHomeId: No matching account found, returning null"),null)},e.prototype.getAccountByLocalId=function(e){var t=this.getAllAccounts();return!i.StringUtils.isEmpty(e)&&t&&t.length?(this.logger.verbose("Account matching localAccountId found, returning"),this.logger.verbosePii("Returning signed-in accounts matching localAccountId: "+e),t.filter(function(t){return t.localAccountId===e})[0]||null):(this.logger.verbose("getAccountByLocalId: No matching account found, returning null"),null)},e.prototype.setActiveAccount=function(e){this.browserStorage.setActiveAccount(e)},e.prototype.getActiveAccount=function(){return this.browserStorage.getActiveAccount()},e.prototype.preflightBrowserEnvironmentCheck=function(e){if(this.logger.verbose("preflightBrowserEnvironmentCheck started"),u.BrowserUtils.blockNonBrowserEnvironment(this.isBrowserEnvironment),u.BrowserUtils.blockRedirectInIframe(e,this.config.system.allowRedirectInIframe),u.BrowserUtils.blockReloadInHiddenIframes(),u.BrowserUtils.blockAcquireTokenInPopups(),e===c.InteractionType.Redirect&&this.config.cache.cacheLocation===c.BrowserCacheLocation.MemoryStorage&&!this.config.cache.storeAuthStateInCookie)throw h.BrowserConfigurationAuthError.createInMemoryRedirectUnavailableError()},e.prototype.addEventCallback=function(e){return this.eventHandler.addEventCallback(e)},e.prototype.removeEventCallback=function(e){this.eventHandler.removeEventCallback(e)},e.prototype.enableAccountStorageEvents=function(){this.eventHandler.enableAccountStorageEvents()},e.prototype.disableAccountStorageEvents=function(){this.eventHandler.disableAccountStorageEvents()},e.prototype.getTokenCache=function(){return this.tokenCache},e.prototype.getLogger=function(){return this.logger},e.prototype.setLogger=function(e){this.logger=e},e.prototype.initializeWrapperLibrary=function(e,t){this.browserStorage.setWrapperMetadata(e,t)},e.prototype.setNavigationClient=function(e){this.navigationClient=e},e.prototype.getConfiguration=function(){return this.config},e}()},"./node_modules/@azure/msal-browser/dist/app/IPublicClientApplication.js":function(e,t,r){"use strict";r.r(t),r.d(t,"stubbedPublicClientApplication",function(){return o});var n=r("./node_modules/@azure/msal-browser/dist/error/BrowserConfigurationAuthError.js"),o={acquireTokenPopup:function(){return Promise.reject(n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError())},acquireTokenRedirect:function(){return Promise.reject(n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError())},acquireTokenSilent:function(){return Promise.reject(n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError())},getAllAccounts:function(){return[]},getAccountByHomeId:function(){return null},getAccountByUsername:function(){return null},getAccountByLocalId:function(){return null},handleRedirectPromise:function(){return Promise.reject(n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError())},loginPopup:function(){return Promise.reject(n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError())},loginRedirect:function(){return Promise.reject(n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError())},logout:function(){return Promise.reject(n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError())},logoutRedirect:function(){return Promise.reject(n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError())},logoutPopup:function(){return Promise.reject(n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError())},ssoSilent:function(){return Promise.reject(n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError())},addEventCallback:function(){return null},removeEventCallback:function(){},enableAccountStorageEvents:function(){},disableAccountStorageEvents:function(){},getTokenCache:function(){throw n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError()},getLogger:function(){throw n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError()},setLogger:function(){},setActiveAccount:function(){},getActiveAccount:function(){return null},initializeWrapperLibrary:function(){},setNavigationClient:function(){},getConfiguration:function(){throw n.BrowserConfigurationAuthError.createStubPcaInstanceCalledError()}};
/*! @azure/msal-browser v2.19.0 2021-11-02 */},"./node_modules/@azure/msal-browser/dist/app/PublicClientApplication.js":function(e,t,r){"use strict";r.r(t),r.d(t,"PublicClientApplication",function(){return u});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),i=r("./node_modules/@azure/msal-browser/dist/app/ClientApplication.js"),s=r("./node_modules/@azure/msal-browser/dist/event/EventType.js"),a=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),c=r("./node_modules/@azure/msal-browser/dist/interaction_client/SilentCacheClient.js"),u=function(e){function t(t){var r=e.call(this,t)||this;return r.activeSilentTokenRequests=new Map,r}return Object(n.__extends)(t,e),t.prototype.loginRedirect=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(t){return this.logger.verbose("loginRedirect called"),[2,this.acquireTokenRedirect(e||o.DEFAULT_REQUEST)]})})},t.prototype.loginPopup=function(e){return this.logger.verbose("loginPopup called"),this.acquireTokenPopup(e||o.DEFAULT_REQUEST)},t.prototype.acquireTokenSilent=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,i,s,c,u=this;return Object(n.__generator)(this,function(n){if(this.preflightBrowserEnvironmentCheck(o.InteractionType.Silent),this.logger.verbose("acquireTokenSilent called",e.correlationId),!(t=e.account||this.getActiveAccount()))throw a.BrowserAuthError.createNoAccountError();return r={clientId:this.config.auth.clientId,authority:e.authority||"",scopes:e.scopes,homeAccountIdentifier:t.homeAccountId,authenticationScheme:e.authenticationScheme,resourceRequestMethod:e.resourceRequestMethod,resourceRequestUri:e.resourceRequestUri,shrClaims:e.shrClaims,sshJwk:e.sshJwk,sshKid:e.sshKid},i=JSON.stringify(r),void 0===(s=this.activeSilentTokenRequests.get(i))?(this.logger.verbose("acquireTokenSilent called for the first time, storing active request",e.correlationId),c=this.acquireTokenSilentAsync(e,t).then(function(e){return u.activeSilentTokenRequests.delete(i),e}).catch(function(e){throw u.activeSilentTokenRequests.delete(i),e}),this.activeSilentTokenRequests.set(i,c),[2,c]):(this.logger.verbose("acquireTokenSilent has been called previously, returning the result from the first call",e.correlationId),[2,s])})})},t.prototype.acquireTokenSilentAsync=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,i,a=this;return Object(n.__generator)(this,function(u){return r=new c.SilentCacheClient(this.config,this.browserStorage,this.browserCrypto,this.logger,this.eventHandler,this.navigationClient,e.correlationId),i=r.initializeSilentRequest(e,t),this.eventHandler.emitEvent(s.EventType.ACQUIRE_TOKEN_START,o.InteractionType.Silent,e),[2,r.acquireToken(i).catch(function(){return Object(n.__awaiter)(a,void 0,void 0,function(){var e,t;return Object(n.__generator)(this,function(r){switch(r.label){case 0:return r.trys.push([0,2,,3]),[4,this.acquireTokenByRefreshToken(i)];case 1:return e=r.sent(),this.eventHandler.emitEvent(s.EventType.ACQUIRE_TOKEN_SUCCESS,o.InteractionType.Silent,e),[2,e];case 2:throw t=r.sent(),this.eventHandler.emitEvent(s.EventType.ACQUIRE_TOKEN_FAILURE,o.InteractionType.Silent,null,t),t;case 3:return[2]}})})})]})})},t}(i.ClientApplication)},"./node_modules/@azure/msal-browser/dist/cache/AsyncMemoryStorage.js":function(e,t,r){"use strict";r.r(t),r.d(t,"AsyncMemoryStorage",function(){return a});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-browser/dist/cache/DatabaseStorage.js"),i=r("./node_modules/@azure/msal-browser/dist/cache/MemoryStorage.js"),s=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),a=function(){function e(e){this.inMemoryCache=new i.MemoryStorage,this.indexedDBCache=new o.DatabaseStorage,this.logger=e}return e.prototype.handleDatabaseAccessError=function(e){e instanceof s.BrowserAuthError&&e.errorCode===s.BrowserAuthErrorMessage.databaseUnavailable.code&&this.logger.error("Could not access persistent storage. This may be caused by browser privacy features which block persistent storage in third-party contexts.")},e.prototype.getItem=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:if(t=this.inMemoryCache.getItem(e))return[3,4];n.label=1;case 1:return n.trys.push([1,3,,4]),this.logger.verbose("Queried item not found in in-memory cache, now querying persistent storage."),[4,this.indexedDBCache.getItem(e)];case 2:return[2,n.sent()];case 3:return r=n.sent(),this.handleDatabaseAccessError(r),[3,4];case 4:return[2,t]}})})},e.prototype.setItem=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:this.inMemoryCache.setItem(e,t),n.label=1;case 1:return n.trys.push([1,3,,4]),[4,this.indexedDBCache.setItem(e,t)];case 2:return n.sent(),[3,4];case 3:return r=n.sent(),this.handleDatabaseAccessError(r),[3,4];case 4:return[2]}})})},e.prototype.removeItem=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t;return Object(n.__generator)(this,function(r){switch(r.label){case 0:this.inMemoryCache.removeItem(e),r.label=1;case 1:return r.trys.push([1,3,,4]),[4,this.indexedDBCache.removeItem(e)];case 2:return r.sent(),[3,4];case 3:return t=r.sent(),this.handleDatabaseAccessError(t),[3,4];case 4:return[2]}})})},e.prototype.getKeys=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){var e,t;return Object(n.__generator)(this,function(r){switch(r.label){case 0:if(0!==(e=this.inMemoryCache.getKeys()).length)return[3,4];r.label=1;case 1:return r.trys.push([1,3,,4]),this.logger.verbose("In-memory cache is empty, now querying persistent storage."),[4,this.indexedDBCache.getKeys()];case 2:return[2,r.sent()];case 3:return t=r.sent(),this.handleDatabaseAccessError(t),[3,4];case 4:return[2,e]}})})},e.prototype.containsKey=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:if(t=this.inMemoryCache.containsKey(e))return[3,4];n.label=1;case 1:return n.trys.push([1,3,,4]),this.logger.verbose("Key not found in in-memory cache, now querying persistent storage."),[4,this.indexedDBCache.containsKey(e)];case 2:return[2,n.sent()];case 3:return r=n.sent(),this.handleDatabaseAccessError(r),[3,4];case 4:return[2,t]}})})},e.prototype.clear=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){var e;return Object(n.__generator)(this,function(t){switch(t.label){case 0:this.inMemoryCache.clear(),t.label=1;case 1:return t.trys.push([1,3,,4]),[4,this.indexedDBCache.deleteDatabase()];case 2:return t.sent(),[3,4];case 3:return e=t.sent(),this.handleDatabaseAccessError(e),[3,4];case 4:return[2]}})})},e}()},"./node_modules/@azure/msal-browser/dist/cache/BrowserCacheManager.js":function(e,t,r){"use strict";r.r(t),r.d(t,"BrowserCacheManager",function(){return d}),r.d(t,"DEFAULT_BROWSER_CACHE_MANAGER",function(){return l});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),s=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),a=r("./node_modules/@azure/msal-browser/dist/cache/BrowserStorage.js"),c=r("./node_modules/@azure/msal-browser/dist/cache/MemoryStorage.js"),u=r("./node_modules/@azure/msal-browser/dist/utils/BrowserProtocolUtils.js"),d=function(e){function t(t,r,n,o){var i=e.call(this,t,n)||this;return i.COOKIE_LIFE_MULTIPLIER=864e5,i.cacheConfig=r,i.logger=o,i.internalStorage=new c.MemoryStorage,i.browserStorage=i.setupBrowserStorage(i.cacheConfig.cacheLocation),i.temporaryCacheStorage=i.setupTemporaryCacheStorage(i.cacheConfig.cacheLocation),i.migrateCacheEntries(),i}return Object(n.__extends)(t,e),t.prototype.setupBrowserStorage=function(e){switch(e){case s.BrowserCacheLocation.LocalStorage:case s.BrowserCacheLocation.SessionStorage:try{return new a.BrowserStorage(e)}catch(e){this.logger.verbose(e);break}}return this.cacheConfig.cacheLocation=s.BrowserCacheLocation.MemoryStorage,new c.MemoryStorage},t.prototype.setupTemporaryCacheStorage=function(e){switch(e){case s.BrowserCacheLocation.LocalStorage:case s.BrowserCacheLocation.SessionStorage:try{return new a.BrowserStorage(s.BrowserCacheLocation.SessionStorage)}catch(e){return this.logger.verbose(e),this.internalStorage}case s.BrowserCacheLocation.MemoryStorage:default:return this.internalStorage}},t.prototype.migrateCacheEntries=function(){var e=this,t=o.Constants.CACHE_PREFIX+"."+o.PersistentCacheKeys.ID_TOKEN,r=o.Constants.CACHE_PREFIX+"."+o.PersistentCacheKeys.CLIENT_INFO,n=o.Constants.CACHE_PREFIX+"."+o.PersistentCacheKeys.ERROR,i=o.Constants.CACHE_PREFIX+"."+o.PersistentCacheKeys.ERROR_DESC,s=[this.browserStorage.getItem(t),this.browserStorage.getItem(r),this.browserStorage.getItem(n),this.browserStorage.getItem(i)];[o.PersistentCacheKeys.ID_TOKEN,o.PersistentCacheKeys.CLIENT_INFO,o.PersistentCacheKeys.ERROR,o.PersistentCacheKeys.ERROR_DESC].forEach(function(t,r){return e.migrateCacheEntry(t,s[r])})},t.prototype.migrateCacheEntry=function(e,t){t&&this.setTemporaryCache(e,t,!0)},t.prototype.validateAndParseJson=function(e){try{var t=JSON.parse(e);return t&&"object"==typeof t?t:null}catch(e){return null}},t.prototype.getItem=function(e){return this.browserStorage.getItem(e)},t.prototype.setItem=function(e,t){this.browserStorage.setItem(e,t)},t.prototype.getAccount=function(e){var t=this.getItem(e);if(!t)return null;var r=this.validateAndParseJson(t);return r&&o.AccountEntity.isAccountEntity(r)?o.CacheManager.toObject(new o.AccountEntity,r):null},t.prototype.setAccount=function(e){this.logger.trace("BrowserCacheManager.setAccount called");var t=e.generateAccountKey();this.setItem(t,JSON.stringify(e))},t.prototype.getIdTokenCredential=function(e){var t=this.getItem(e);if(!t)return this.logger.trace("BrowserCacheManager.getIdTokenCredential: called, no cache hit"),null;var r=this.validateAndParseJson(t);return r&&o.IdTokenEntity.isIdTokenEntity(r)?(this.logger.trace("BrowserCacheManager.getIdTokenCredential: cache hit"),o.CacheManager.toObject(new o.IdTokenEntity,r)):(this.logger.trace("BrowserCacheManager.getIdTokenCredential: called, no cache hit"),null)},t.prototype.setIdTokenCredential=function(e){this.logger.trace("BrowserCacheManager.setIdTokenCredential called");var t=e.generateCredentialKey();this.setItem(t,JSON.stringify(e))},t.prototype.getAccessTokenCredential=function(e){var t=this.getItem(e);if(!t)return this.logger.trace("BrowserCacheManager.getAccessTokenCredential: called, no cache hit"),null;var r=this.validateAndParseJson(t);return r&&o.AccessTokenEntity.isAccessTokenEntity(r)?(this.logger.trace("BrowserCacheManager.getAccessTokenCredential: cache hit"),o.CacheManager.toObject(new o.AccessTokenEntity,r)):(this.logger.trace("BrowserCacheManager.getAccessTokenCredential: called, no cache hit"),null)},t.prototype.setAccessTokenCredential=function(e){this.logger.trace("BrowserCacheManager.setAccessTokenCredential called");var t=e.generateCredentialKey();this.setItem(t,JSON.stringify(e))},t.prototype.getRefreshTokenCredential=function(e){var t=this.getItem(e);if(!t)return this.logger.trace("BrowserCacheManager.getRefreshTokenCredential: called, no cache hit"),null;var r=this.validateAndParseJson(t);return r&&o.RefreshTokenEntity.isRefreshTokenEntity(r)?(this.logger.trace("BrowserCacheManager.getRefreshTokenCredential: cache hit"),o.CacheManager.toObject(new o.RefreshTokenEntity,r)):(this.logger.trace("BrowserCacheManager.getRefreshTokenCredential: called, no cache hit"),null)},t.prototype.setRefreshTokenCredential=function(e){this.logger.trace("BrowserCacheManager.setRefreshTokenCredential called");var t=e.generateCredentialKey();this.setItem(t,JSON.stringify(e))},t.prototype.getAppMetadata=function(e){var t=this.getItem(e);if(!t)return this.logger.trace("BrowserCacheManager.getAppMetadata: called, no cache hit"),null;var r=this.validateAndParseJson(t);return r&&o.AppMetadataEntity.isAppMetadataEntity(e,r)?(this.logger.trace("BrowserCacheManager.getAppMetadata: cache hit"),o.CacheManager.toObject(new o.AppMetadataEntity,r)):(this.logger.trace("BrowserCacheManager.getAppMetadata: called, no cache hit"),null)},t.prototype.setAppMetadata=function(e){this.logger.trace("BrowserCacheManager.setAppMetadata called");var t=e.generateAppMetadataKey();this.setItem(t,JSON.stringify(e))},t.prototype.getServerTelemetry=function(e){var t=this.getItem(e);if(!t)return this.logger.trace("BrowserCacheManager.getServerTelemetry: called, no cache hit"),null;var r=this.validateAndParseJson(t);return r&&o.ServerTelemetryEntity.isServerTelemetryEntity(e,r)?(this.logger.trace("BrowserCacheManager.getServerTelemetry: cache hit"),o.CacheManager.toObject(new o.ServerTelemetryEntity,r)):(this.logger.trace("BrowserCacheManager.getServerTelemetry: called, no cache hit"),null)},t.prototype.setServerTelemetry=function(e,t){this.logger.trace("BrowserCacheManager.setServerTelemetry called"),this.setItem(e,JSON.stringify(t))},t.prototype.getAuthorityMetadata=function(e){var t=this.internalStorage.getItem(e);if(!t)return this.logger.trace("BrowserCacheManager.getAuthorityMetadata: called, no cache hit"),null;var r=this.validateAndParseJson(t);return r&&o.AuthorityMetadataEntity.isAuthorityMetadataEntity(e,r)?(this.logger.trace("BrowserCacheManager.getAuthorityMetadata: cache hit"),o.CacheManager.toObject(new o.AuthorityMetadataEntity,r)):null},t.prototype.getAuthorityMetadataKeys=function(){var e=this;return this.internalStorage.getKeys().filter(function(t){return e.isAuthorityMetadata(t)})},t.prototype.setWrapperMetadata=function(e,t){this.internalStorage.setItem(s.InMemoryCacheKeys.WRAPPER_SKU,e),this.internalStorage.setItem(s.InMemoryCacheKeys.WRAPPER_VER,t)},t.prototype.getWrapperMetadata=function(){return[this.internalStorage.getItem(s.InMemoryCacheKeys.WRAPPER_SKU)||"",this.internalStorage.getItem(s.InMemoryCacheKeys.WRAPPER_VER)||""]},t.prototype.setAuthorityMetadata=function(e,t){this.logger.trace("BrowserCacheManager.setAuthorityMetadata called"),this.internalStorage.setItem(e,JSON.stringify(t))},t.prototype.getActiveAccount=function(){var e=this.generateCacheKey(o.PersistentCacheKeys.ACTIVE_ACCOUNT),t=this.browserStorage.getItem(e);return t&&this.getAccountInfoByFilter({localAccountId:t})[0]||null},t.prototype.setActiveAccount=function(e){var t=this.generateCacheKey(o.PersistentCacheKeys.ACTIVE_ACCOUNT);e?(this.logger.verbose("setActiveAccount: Active account set"),this.browserStorage.setItem(t,e.localAccountId)):(this.logger.verbose("setActiveAccount: No account passed, active account not set"),this.browserStorage.removeItem(t))},t.prototype.getAccountInfoByFilter=function(e){return this.getAllAccounts().filter(function(t){return(!e.username||e.username.toLowerCase()===t.username.toLowerCase())&&((!e.homeAccountId||e.homeAccountId===t.homeAccountId)&&((!e.localAccountId||e.localAccountId===t.localAccountId)&&((!e.tenantId||e.tenantId===t.tenantId)&&(!e.environment||e.environment===t.environment))))})},t.prototype.getThrottlingCache=function(e){var t=this.getItem(e);if(!t)return this.logger.trace("BrowserCacheManager.getThrottlingCache: called, no cache hit"),null;var r=this.validateAndParseJson(t);return r&&o.ThrottlingEntity.isThrottlingEntity(e,r)?(this.logger.trace("BrowserCacheManager.getThrottlingCache: cache hit"),o.CacheManager.toObject(new o.ThrottlingEntity,r)):(this.logger.trace("BrowserCacheManager.getThrottlingCache: called, no cache hit"),null)},t.prototype.setThrottlingCache=function(e,t){this.logger.trace("BrowserCacheManager.setThrottlingCache called"),this.setItem(e,JSON.stringify(t))},t.prototype.getTemporaryCache=function(e,t){var r=t?this.generateCacheKey(e):e;if(this.cacheConfig.storeAuthStateInCookie){var n=this.getItemCookie(r);if(n)return this.logger.trace("BrowserCacheManager.getTemporaryCache: storeAuthStateInCookies set to true, retrieving from cookies"),n}var o=this.temporaryCacheStorage.getItem(r);if(!o){if(this.cacheConfig.cacheLocation===s.BrowserCacheLocation.LocalStorage){var i=this.browserStorage.getItem(r);if(i)return this.logger.trace("BrowserCacheManager.getTemporaryCache: Temporary cache item found in local storage"),i}return this.logger.trace("BrowserCacheManager.getTemporaryCache: No cache item found in local storage"),null}return this.logger.trace("BrowserCacheManager.getTemporaryCache: Temporary cache item returned"),o},t.prototype.setTemporaryCache=function(e,t,r){var n=r?this.generateCacheKey(e):e;this.temporaryCacheStorage.setItem(n,t),this.cacheConfig.storeAuthStateInCookie&&(this.logger.trace("BrowserCacheManager.setTemporaryCache: storeAuthStateInCookie set to true, setting item cookie"),this.setItemCookie(n,t))},t.prototype.removeItem=function(e){return this.browserStorage.removeItem(e),this.temporaryCacheStorage.removeItem(e),this.cacheConfig.storeAuthStateInCookie&&(this.logger.trace("BrowserCacheManager.removeItem: storeAuthStateInCookie is true, clearing item cookie"),this.clearItemCookie(e)),!0},t.prototype.containsKey=function(e){return this.browserStorage.containsKey(e)||this.temporaryCacheStorage.containsKey(e)},t.prototype.getKeys=function(){return Object(n.__spread)(this.browserStorage.getKeys(),this.temporaryCacheStorage.getKeys())},t.prototype.clear=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){var e=this;return Object(n.__generator)(this,function(t){switch(t.label){case 0:return[4,this.removeAllAccounts()];case 1:return t.sent(),this.removeAppMetadata(),this.getKeys().forEach(function(t){!e.browserStorage.containsKey(t)&&!e.temporaryCacheStorage.containsKey(t)||-1===t.indexOf(o.Constants.CACHE_PREFIX)&&-1===t.indexOf(e.clientId)||e.removeItem(t)}),this.internalStorage.clear(),[2]}})})},t.prototype.setItemCookie=function(e,t,r){var n=encodeURIComponent(e)+"="+encodeURIComponent(t)+";path=/;";r&&(n+="expires="+this.getCookieExpirationTime(r)+";");this.cacheConfig.secureCookies&&(n+="Secure;"),document.cookie=n},t.prototype.getItemCookie=function(e){for(var t=encodeURIComponent(e)+"=",r=document.cookie.split(";"),n=0;n<r.length;n++){for(var o=r[n];" "===o.charAt(0);)o=o.substring(1);if(0===o.indexOf(t))return decodeURIComponent(o.substring(t.length,o.length))}return""},t.prototype.clearMsalCookies=function(){var e=this,t=o.Constants.CACHE_PREFIX+"."+this.clientId;document.cookie.split(";").forEach(function(r){for(;" "===r.charAt(0);)r=r.substring(1);if(0===r.indexOf(t)){var n=r.split("=")[0];e.clearItemCookie(n)}})},t.prototype.clearItemCookie=function(e){this.setItemCookie(e,"",-1)},t.prototype.getCookieExpirationTime=function(e){var t=new Date;return new Date(t.getTime()+e*this.COOKIE_LIFE_MULTIPLIER).toUTCString()},t.prototype.getCache=function(){return this.browserStorage},t.prototype.setCache=function(){},t.prototype.generateCacheKey=function(e){return this.validateAndParseJson(e)?JSON.stringify(e):o.StringUtils.startsWith(e,o.Constants.CACHE_PREFIX)||o.StringUtils.startsWith(e,o.PersistentCacheKeys.ADAL_ID_TOKEN)?e:o.Constants.CACHE_PREFIX+"."+this.clientId+"."+e},t.prototype.generateAuthorityKey=function(e){var t=o.ProtocolUtils.parseRequestState(this.cryptoImpl,e).libraryState.id;return this.generateCacheKey(s.TemporaryCacheKeys.AUTHORITY+"."+t)},t.prototype.generateNonceKey=function(e){var t=o.ProtocolUtils.parseRequestState(this.cryptoImpl,e).libraryState.id;return this.generateCacheKey(s.TemporaryCacheKeys.NONCE_IDTOKEN+"."+t)},t.prototype.generateStateKey=function(e){var t=o.ProtocolUtils.parseRequestState(this.cryptoImpl,e).libraryState.id;return this.generateCacheKey(s.TemporaryCacheKeys.REQUEST_STATE+"."+t)},t.prototype.getCachedAuthority=function(e){var t=this.generateStateKey(e),r=this.getTemporaryCache(t);if(!r)return null;var n=this.generateAuthorityKey(r);return this.getTemporaryCache(n)},t.prototype.updateCacheEntries=function(e,t,r,n,i){this.logger.trace("BrowserCacheManager.updateCacheEntries called");var a=this.generateStateKey(e);this.setTemporaryCache(a,e,!1);var c=this.generateNonceKey(e);this.setTemporaryCache(c,t,!1);var u=this.generateAuthorityKey(e);if(this.setTemporaryCache(u,r,!1),i){var d={credential:i.homeAccountId,type:o.CcsCredentialType.HOME_ACCOUNT_ID};this.setTemporaryCache(s.TemporaryCacheKeys.CCS_CREDENTIAL,JSON.stringify(d),!0)}else if(!o.StringUtils.isEmpty(n)){d={credential:n,type:o.CcsCredentialType.UPN};this.setTemporaryCache(s.TemporaryCacheKeys.CCS_CREDENTIAL,JSON.stringify(d),!0)}},t.prototype.resetRequestCache=function(e){var t=this;this.logger.trace("BrowserCacheManager.resetRequestCache called"),o.StringUtils.isEmpty(e)||this.getKeys().forEach(function(r){-1!==r.indexOf(e)&&t.removeItem(r)}),e&&(this.removeItem(this.generateStateKey(e)),this.removeItem(this.generateNonceKey(e)),this.removeItem(this.generateAuthorityKey(e))),this.removeItem(this.generateCacheKey(s.TemporaryCacheKeys.REQUEST_PARAMS)),this.removeItem(this.generateCacheKey(s.TemporaryCacheKeys.ORIGIN_URI)),this.removeItem(this.generateCacheKey(s.TemporaryCacheKeys.URL_HASH)),this.removeItem(this.generateCacheKey(s.TemporaryCacheKeys.CORRELATION_ID)),this.removeItem(this.generateCacheKey(s.TemporaryCacheKeys.CCS_CREDENTIAL)),this.setInteractionInProgress(!1)},t.prototype.cleanRequestByState=function(e){if(this.logger.trace("BrowserCacheManager.cleanRequestByState called"),e){var t=this.generateStateKey(e),r=this.temporaryCacheStorage.getItem(t);this.logger.infoPii("BrowserCacheManager.cleanRequestByState: Removing temporary cache items for state: "+r),this.resetRequestCache(r||"")}this.clearMsalCookies()},t.prototype.cleanRequestByInteractionType=function(e){var t=this;this.logger.trace("BrowserCacheManager.cleanRequestByInteractionType called"),this.getKeys().forEach(function(r){if(-1!==r.indexOf(s.TemporaryCacheKeys.REQUEST_STATE)){var n=t.temporaryCacheStorage.getItem(r);if(n){var o=u.BrowserProtocolUtils.extractBrowserRequestState(t.cryptoImpl,n);o&&o.interactionType===e&&(t.logger.infoPii("BrowserCacheManager.cleanRequestByInteractionType: Removing temporary cache items for state: "+n),t.resetRequestCache(n))}}}),this.clearMsalCookies()},t.prototype.cacheCodeRequest=function(e,t){this.logger.trace("BrowserCacheManager.cacheCodeRequest called");var r=t.base64Encode(JSON.stringify(e));this.setTemporaryCache(s.TemporaryCacheKeys.REQUEST_PARAMS,r,!0)},t.prototype.getCachedRequest=function(e,t){this.logger.trace("BrowserCacheManager.getCachedRequest called");var r=this.getTemporaryCache(s.TemporaryCacheKeys.REQUEST_PARAMS,!0);if(!r)throw i.BrowserAuthError.createNoTokenRequestCacheError();var n=this.validateAndParseJson(t.base64Decode(r));if(!n)throw i.BrowserAuthError.createUnableToParseTokenRequestCacheError();if(this.removeItem(this.generateCacheKey(s.TemporaryCacheKeys.REQUEST_PARAMS)),o.StringUtils.isEmpty(n.authority)){var a=this.generateAuthorityKey(e),c=this.getTemporaryCache(a);if(!c)throw i.BrowserAuthError.createNoCachedAuthorityError();n.authority=c}return n},t.prototype.isInteractionInProgress=function(e){var t=this.getInteractionInProgress();return e?t===this.clientId:!!t},t.prototype.getInteractionInProgress=function(){var e=o.Constants.CACHE_PREFIX+"."+s.TemporaryCacheKeys.INTERACTION_STATUS_KEY;return this.getTemporaryCache(e,!1)},t.prototype.setInteractionInProgress=function(e){var t=this.getInteractionInProgress(),r=o.Constants.CACHE_PREFIX+"."+s.TemporaryCacheKeys.INTERACTION_STATUS_KEY;e&&!t?this.setTemporaryCache(r,this.clientId,!1):e||t!==this.clientId||this.removeItem(r)},t.prototype.getLegacyLoginHint=function(){var e=this.getTemporaryCache(o.PersistentCacheKeys.ADAL_ID_TOKEN);e&&(this.browserStorage.removeItem(o.PersistentCacheKeys.ADAL_ID_TOKEN),this.logger.verbose("Cached ADAL id token retrieved."));var t=this.getTemporaryCache(o.PersistentCacheKeys.ID_TOKEN,!0);t&&(this.removeItem(this.generateCacheKey(o.PersistentCacheKeys.ID_TOKEN)),this.logger.verbose("Cached MSAL.js v1 id token retrieved"));var r=t||e;if(r){var n=new o.IdToken(r,this.cryptoImpl);if(n.claims&&n.claims.preferred_username)return this.logger.verbose("No SSO params used and ADAL/MSAL v1 token retrieved, setting ADAL/MSAL v1 preferred_username as loginHint"),n.claims.preferred_username;if(n.claims&&n.claims.upn)return this.logger.verbose("No SSO params used and ADAL/MSAL v1 token retrieved, setting ADAL/MSAL v1 upn as loginHint"),n.claims.upn;this.logger.verbose("No SSO params used and ADAL/MSAL v1 token retrieved, however, no account hint claim found. Enable preferred_username or upn id token claim to get SSO.")}return null},t}(o.CacheManager),l=function(e,t){var r={cacheLocation:s.BrowserCacheLocation.MemoryStorage,storeAuthStateInCookie:!1,secureCookies:!1};return new d(e,r,o.DEFAULT_CRYPTO_IMPLEMENTATION,t)}},"./node_modules/@azure/msal-browser/dist/cache/BrowserStorage.js":function(e,t,r){"use strict";r.r(t),r.d(t,"BrowserStorage",function(){return i});var n=r("./node_modules/@azure/msal-browser/dist/error/BrowserConfigurationAuthError.js"),o=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),i=function(){function e(e){this.validateWindowStorage(e),this.windowStorage=window[e]}return e.prototype.validateWindowStorage=function(e){if(e!==o.BrowserCacheLocation.LocalStorage&&e!==o.BrowserCacheLocation.SessionStorage)throw n.BrowserConfigurationAuthError.createStorageNotSupportedError(e);if(!!!window[e])throw n.BrowserConfigurationAuthError.createStorageNotSupportedError(e)},e.prototype.getItem=function(e){return this.windowStorage.getItem(e)},e.prototype.setItem=function(e,t){this.windowStorage.setItem(e,t)},e.prototype.removeItem=function(e){this.windowStorage.removeItem(e)},e.prototype.getKeys=function(){return Object.keys(this.windowStorage)},e.prototype.containsKey=function(e){return this.windowStorage.hasOwnProperty(e)},e}()},"./node_modules/@azure/msal-browser/dist/cache/DatabaseStorage.js":function(e,t,r){"use strict";r.r(t),r.d(t,"DatabaseStorage",function(){return s});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),i=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),s=function(){function e(){this.dbName=i.DB_NAME,this.version=i.DB_VERSION,this.tableName=i.DB_TABLE_NAME,this.dbOpen=!1}return e.prototype.open=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){var e=this;return Object(n.__generator)(this,function(t){return[2,new Promise(function(t,r){var n=window.indexedDB.open(e.dbName,e.version);n.addEventListener("upgradeneeded",function(t){t.target.result.createObjectStore(e.tableName)}),n.addEventListener("success",function(r){var n=r;e.db=n.target.result,e.dbOpen=!0,t()}),n.addEventListener("error",function(){return r(o.BrowserAuthError.createDatabaseUnavailableError())})})]})})},e.prototype.validateDbIsOpen=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(e){switch(e.label){case 0:return this.dbOpen?[3,2]:[4,this.open()];case 1:return[2,e.sent()];case 2:return[2]}})})},e.prototype.getItem=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t=this;return Object(n.__generator)(this,function(r){switch(r.label){case 0:return[4,this.validateDbIsOpen()];case 1:return r.sent(),[2,new Promise(function(r,n){if(!t.db)return n(o.BrowserAuthError.createDatabaseNotOpenError());var i=t.db.transaction([t.tableName],"readonly").objectStore(t.tableName).get(e);i.addEventListener("success",function(e){r(e.target.result)}),i.addEventListener("error",function(e){return n(e)})})]}})})},e.prototype.setItem=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r=this;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return[4,this.validateDbIsOpen()];case 1:return n.sent(),[2,new Promise(function(n,i){if(!r.db)return i(o.BrowserAuthError.createDatabaseNotOpenError());var s=r.db.transaction([r.tableName],"readwrite").objectStore(r.tableName).put(t,e);s.addEventListener("success",function(){return n()}),s.addEventListener("error",function(e){return i(e)})})]}})})},e.prototype.removeItem=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t=this;return Object(n.__generator)(this,function(r){switch(r.label){case 0:return[4,this.validateDbIsOpen()];case 1:return r.sent(),[2,new Promise(function(r,n){if(!t.db)return n(o.BrowserAuthError.createDatabaseNotOpenError());var i=t.db.transaction([t.tableName],"readwrite").objectStore(t.tableName).delete(e);i.addEventListener("success",function(){return r()}),i.addEventListener("error",function(e){return n(e)})})]}})})},e.prototype.getKeys=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){var e=this;return Object(n.__generator)(this,function(t){switch(t.label){case 0:return[4,this.validateDbIsOpen()];case 1:return t.sent(),[2,new Promise(function(t,r){if(!e.db)return r(o.BrowserAuthError.createDatabaseNotOpenError());var n=e.db.transaction([e.tableName],"readonly").objectStore(e.tableName).getAllKeys();n.addEventListener("success",function(e){t(e.target.result)}),n.addEventListener("error",function(e){return r(e)})})]}})})},e.prototype.containsKey=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t=this;return Object(n.__generator)(this,function(r){switch(r.label){case 0:return[4,this.validateDbIsOpen()];case 1:return r.sent(),[2,new Promise(function(r,n){if(!t.db)return n(o.BrowserAuthError.createDatabaseNotOpenError());var i=t.db.transaction([t.tableName],"readonly").objectStore(t.tableName).count(e);i.addEventListener("success",function(e){r(1===e.target.result)}),i.addEventListener("error",function(e){return n(e)})})]}})})},e.prototype.deleteDatabase=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(e){return[2,new Promise(function(e,t){var r=window.indexedDB.deleteDatabase(i.DB_NAME);r.addEventListener("success",function(){return e(!0)}),r.addEventListener("error",function(){return t(!1)})})]})})},e}()},"./node_modules/@azure/msal-browser/dist/cache/MemoryStorage.js":function(e,t,r){"use strict";r.r(t),r.d(t,"MemoryStorage",function(){return n});
/*! @azure/msal-browser v2.19.0 2021-11-02 */
var n=function(){function e(){this.cache=new Map}return e.prototype.getItem=function(e){return this.cache.get(e)||null},e.prototype.setItem=function(e,t){this.cache.set(e,t)},e.prototype.removeItem=function(e){this.cache.delete(e)},e.prototype.getKeys=function(){var e=[];return this.cache.forEach(function(t,r){e.push(r)}),e},e.prototype.containsKey=function(e){return this.cache.has(e)},e.prototype.clear=function(){this.cache.clear()},e}()},"./node_modules/@azure/msal-browser/dist/cache/TokenCache.js":function(e,t,r){"use strict";r.r(t),r.d(t,"TokenCache",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/index.js"),o=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),i=function(){function e(e,t,r,n){this.isBrowserEnvironment="undefined"!=typeof window,this.config=e,this.storage=t,this.logger=r,this.cryptoObj=n}return e.prototype.loadExternalTokens=function(e,t,r){if(this.logger.info("TokenCache - loadExternalTokens called"),!t.id_token)throw o.BrowserAuthError.createUnableToLoadTokenError("Please ensure server response includes id token.");if(e.account)this.loadIdToken(t.id_token,e.account.homeAccountId,e.account.environment,e.account.tenantId,r),this.loadAccessToken(e,t,e.account.homeAccountId,e.account.environment,e.account.tenantId,r);else{if(!e.authority)throw o.BrowserAuthError.createUnableToLoadTokenError("Please provide a request with an account or a request with authority.");var i={protocolMode:this.config.auth.protocolMode,knownAuthorities:this.config.auth.knownAuthorities,cloudDiscoveryMetadata:this.config.auth.cloudDiscoveryMetadata,authorityMetadata:this.config.auth.authorityMetadata},s=new n.Authority(e.authority,this.config.system.networkClient,this.storage,i);if(r.clientInfo)this.logger.trace("TokenCache - homeAccountId from options"),this.loadIdToken(t.id_token,r.clientInfo,s.hostnameAndPort,s.tenant,r),this.loadAccessToken(e,t,r.clientInfo,s.hostnameAndPort,s.tenant,r);else{if(!t.client_info)throw o.BrowserAuthError.createUnableToLoadTokenError("Please provide clientInfo in the response or options.");this.logger.trace("TokenCache - homeAccountId from response"),this.loadIdToken(t.id_token,t.client_info,s.hostnameAndPort,s.tenant,r),this.loadAccessToken(e,t,t.client_info,s.hostnameAndPort,s.tenant,r)}}},e.prototype.loadIdToken=function(e,t,r,i,s){var a=n.IdTokenEntity.createIdTokenEntity(t,r,e,this.config.auth.clientId,i),c=new n.AuthToken(e,this.cryptoObj),u=s.clientInfo?n.AccountEntity.createAccount(s.clientInfo,t,c,void 0,void 0,void 0,void 0,r):n.AccountEntity.createGenericAccount(t,c,void 0,void 0,void 0,void 0,r);if(!this.isBrowserEnvironment)throw o.BrowserAuthError.createUnableToLoadTokenError("loadExternalTokens is designed to work in browser environments only.");this.logger.verbose("TokenCache - loading id token"),this.storage.setAccount(u),this.storage.setIdTokenCredential(a)},e.prototype.loadAccessToken=function(e,t,r,i,s,a){if(t.access_token){if(!t.expires_in)throw o.BrowserAuthError.createUnableToLoadTokenError("Please ensure server response includes expires_in value.");if(!a.extendedExpiresOn)throw o.BrowserAuthError.createUnableToLoadTokenError("Please provide an extendedExpiresOn value in the options.");var c=new n.ScopeSet(e.scopes).printScopes(),u=t.expires_in,d=a.extendedExpiresOn,l=n.AccessTokenEntity.createAccessTokenEntity(r,i,t.access_token,this.config.auth.clientId,s,c,u,d,this.cryptoObj);if(!this.isBrowserEnvironment)throw o.BrowserAuthError.createUnableToLoadTokenError("loadExternalTokens is designed to work in browser environments only.");this.logger.verbose("TokenCache - loading access token"),this.storage.setAccessTokenCredential(l)}else this.logger.verbose("TokenCache - No access token provided for caching")},e}()},"./node_modules/@azure/msal-browser/dist/config/Configuration.js":function(e,t,r){"use strict";r.r(t),r.d(t,"DEFAULT_IFRAME_TIMEOUT_MS",function(){return u}),r.d(t,"DEFAULT_POPUP_TIMEOUT_MS",function(){return c}),r.d(t,"DEFAULT_REDIRECT_TIMEOUT_MS",function(){return d}),r.d(t,"buildConfiguration",function(){return l});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i=r("./node_modules/@azure/msal-browser/dist/utils/BrowserUtils.js"),s=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),a=r("./node_modules/@azure/msal-browser/dist/navigation/NavigationClient.js"),c=6e4,u=6e3,d=3e4;function l(e,t){var r=e.auth,l=e.cache,h=e.system,p={clientId:"",authority:""+o.Constants.DEFAULT_AUTHORITY,knownAuthorities:[],cloudDiscoveryMetadata:"",authorityMetadata:"",redirectUri:"",postLogoutRedirectUri:"",navigateToLoginRequestUrl:!0,clientCapabilities:[],protocolMode:o.ProtocolMode.AAD},m={cacheLocation:s.BrowserCacheLocation.SessionStorage,storeAuthStateInCookie:!1,secureCookies:!1},g={loggerCallback:function(){},logLevel:o.LogLevel.Info,piiLoggingEnabled:!1},f=Object(n.__assign)(Object(n.__assign)({},o.DEFAULT_SYSTEM_OPTIONS),{loggerOptions:g,networkClient:t?i.BrowserUtils.getBrowserNetworkClient():o.StubbedNetworkModule,navigationClient:new a.NavigationClient,loadFrameTimeout:0,windowHashTimeout:h&&h.loadFrameTimeout||c,iframeHashTimeout:h&&h.loadFrameTimeout||u,navigateFrameWait:t&&i.BrowserUtils.detectIEOrEdge()?500:0,redirectNavigationTimeout:d,asyncPopups:!1,allowRedirectInIframe:!1});return{auth:Object(n.__assign)(Object(n.__assign)({},p),r),cache:Object(n.__assign)(Object(n.__assign)({},m),l),system:Object(n.__assign)(Object(n.__assign)({},f),h)}}},"./node_modules/@azure/msal-browser/dist/crypto/BrowserCrypto.js":function(e,t,r){"use strict";r.r(t),r.d(t,"BrowserCrypto",function(){return l});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-browser/dist/utils/BrowserStringUtils.js"),i=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),s=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),a="RSASSA-PKCS1-v1_5",c="SHA-256",u=2048,d=new Uint8Array([1,0,1]),l=function(){function e(e){if(this.logger=e,!this.hasCryptoAPI())throw i.BrowserAuthError.createCryptoNotAvailableError("Browser crypto or msCrypto object not available.");this._keygenAlgorithmOptions={name:a,hash:c,modulusLength:u,publicExponent:d}}return e.prototype.sha256Digest=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t;return Object(n.__generator)(this,function(r){return t=o.BrowserStringUtils.stringToUtf8Arr(e),[2,this.hasIECrypto()?this.getMSCryptoDigest(c,t):this.getSubtleCryptoDigest(c,t)]})})},e.prototype.getRandomValues=function(e){var t=window.msCrypto||window.crypto;if(!t.getRandomValues)throw i.BrowserAuthError.createCryptoNotAvailableError("getRandomValues does not exist.");t.getRandomValues(e)},e.prototype.generateKeyPair=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(r){return[2,this.hasIECrypto()?this.msCryptoGenerateKey(e,t):window.crypto.subtle.generateKey(this._keygenAlgorithmOptions,e,t)]})})},e.prototype.exportJwk=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(t){return[2,this.hasIECrypto()?this.msCryptoExportJwk(e):window.crypto.subtle.exportKey(s.KEY_FORMAT_JWK,e)]})})},e.prototype.importJwk=function(t,r,i){return Object(n.__awaiter)(this,void 0,void 0,function(){var a,c;return Object(n.__generator)(this,function(n){return a=e.getJwkString(t),c=o.BrowserStringUtils.stringToArrayBuffer(a),[2,this.hasIECrypto()?this.msCryptoImportKey(c,r,i):window.crypto.subtle.importKey(s.KEY_FORMAT_JWK,t,this._keygenAlgorithmOptions,r,i)]})})},e.prototype.sign=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(r){return[2,this.hasIECrypto()?this.msCryptoSign(e,t):window.crypto.subtle.sign(this._keygenAlgorithmOptions,e,t)]})})},e.prototype.hasCryptoAPI=function(){return this.hasIECrypto()||this.hasBrowserCrypto()},e.prototype.hasIECrypto=function(){return"msCrypto"in window},e.prototype.hasBrowserCrypto=function(){return"crypto"in window},e.prototype.getSubtleCryptoDigest=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(r){return[2,window.crypto.subtle.digest(e,t)]})})},e.prototype.getMSCryptoDigest=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(r){return[2,new Promise(function(r,n){var o=window.msCrypto.subtle.digest(e,t.buffer);o.addEventListener("complete",function(e){r(e.target.result)}),o.addEventListener("error",function(e){n(e)})})]})})},e.prototype.msCryptoGenerateKey=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r=this;return Object(n.__generator)(this,function(n){return[2,new Promise(function(n,o){var i=window.msCrypto.subtle.generateKey(r._keygenAlgorithmOptions,e,t);i.addEventListener("complete",function(e){n(e.target.result)}),i.addEventListener("error",function(e){o(e)})})]})})},e.prototype.msCryptoExportJwk=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(t){return[2,new Promise(function(t,r){var n=window.msCrypto.subtle.exportKey(s.KEY_FORMAT_JWK,e);n.addEventListener("complete",function(e){var n=e.target.result,i=o.BrowserStringUtils.utf8ArrToString(new Uint8Array(n)).replace(/\r/g,"").replace(/\n/g,"").replace(/\t/g,"").split(" ").join("").replace("\0","");try{t(JSON.parse(i))}catch(e){r(e)}}),n.addEventListener("error",function(e){r(e)})})]})})},e.prototype.msCryptoImportKey=function(e,t,r){return Object(n.__awaiter)(this,void 0,void 0,function(){var o=this;return Object(n.__generator)(this,function(n){return[2,new Promise(function(n,i){var a=window.msCrypto.subtle.importKey(s.KEY_FORMAT_JWK,e,o._keygenAlgorithmOptions,t,r);a.addEventListener("complete",function(e){n(e.target.result)}),a.addEventListener("error",function(e){i(e)})})]})})},e.prototype.msCryptoSign=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r=this;return Object(n.__generator)(this,function(n){return[2,new Promise(function(n,o){var i=window.msCrypto.subtle.sign(r._keygenAlgorithmOptions,e,t);i.addEventListener("complete",function(e){n(e.target.result)}),i.addEventListener("error",function(e){o(e)})})]})})},e.getJwkString=function(e){return JSON.stringify(e,Object.keys(e).sort())},e}()},"./node_modules/@azure/msal-browser/dist/crypto/CryptoOps.js":function(e,t,r){"use strict";r.r(t),r.d(t,"CryptoOps",function(){return p});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-browser/dist/crypto/GuidGenerator.js"),i=r("./node_modules/@azure/msal-browser/dist/encode/Base64Encode.js"),s=r("./node_modules/@azure/msal-browser/dist/encode/Base64Decode.js"),a=r("./node_modules/@azure/msal-browser/dist/crypto/PkceGenerator.js"),c=r("./node_modules/@azure/msal-browser/dist/crypto/BrowserCrypto.js"),u=r("./node_modules/@azure/msal-browser/dist/utils/BrowserStringUtils.js"),d=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),l=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),h=r("./node_modules/@azure/msal-browser/dist/cache/AsyncMemoryStorage.js"),p=function(){function e(e){this.logger=e,this.browserCrypto=new c.BrowserCrypto(this.logger),this.b64Encode=new i.Base64Encode,this.b64Decode=new s.Base64Decode,this.guidGenerator=new o.GuidGenerator(this.browserCrypto),this.pkceGenerator=new a.PkceGenerator(this.browserCrypto),this.cache={asymmetricKeys:new h.AsyncMemoryStorage(this.logger),symmetricKeys:new h.AsyncMemoryStorage(this.logger)}}return e.prototype.createNewGuid=function(){return this.guidGenerator.generateGuid()},e.prototype.base64Encode=function(e){return this.b64Encode.encode(e)},e.prototype.base64Decode=function(e){return this.b64Decode.decode(e)},e.prototype.generatePkceCodes=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(e){return[2,this.pkceGenerator.generateCodes()]})})},e.prototype.getPublicKeyThumbprint=function(t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,o,i,s,a,u,d,l;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return[4,this.browserCrypto.generateKeyPair(e.EXTRACTABLE,e.POP_KEY_USAGES)];case 1:return r=n.sent(),[4,this.browserCrypto.exportJwk(r.publicKey)];case 2:return o=n.sent(),i={e:o.e,kty:o.kty,n:o.n},s=c.BrowserCrypto.getJwkString(i),[4,this.browserCrypto.sha256Digest(s)];case 3:return a=n.sent(),u=this.b64Encode.urlEncodeArr(new Uint8Array(a)),[4,this.browserCrypto.exportJwk(r.privateKey)];case 4:return d=n.sent(),[4,this.browserCrypto.importJwk(d,!1,["sign"])];case 5:return l=n.sent(),[4,this.cache.asymmetricKeys.setItem(u,{privateKey:l,publicKey:r.publicKey,requestMethod:t.resourceRequestMethod,requestUri:t.resourceRequestUri})];case 6:return n.sent(),[2,u]}})})},e.prototype.removeTokenBindingKey=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(t){switch(t.label){case 0:return[4,this.cache.asymmetricKeys.removeItem(e)];case 1:return t.sent(),[4,this.cache.asymmetricKeys.containsKey(e)];case 2:return[2,!t.sent()]}})})},e.prototype.clearKeystore=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){var e,t,r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return e=Object.keys(this.cache),(t=this.cache[e[0]])?[4,t.deleteDatabase()]:[3,2];case 1:return r=n.sent(),[3,3];case 2:r=!1,n.label=3;case 3:return[2,r]}})})},e.prototype.signJwt=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,o,i,s,a,h,p,m,g,f;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return[4,this.cache.asymmetricKeys.getItem(t)];case 1:if(!(r=n.sent()))throw l.BrowserAuthError.createSigningKeyNotFoundInStorageError(t);return[4,this.browserCrypto.exportJwk(r.publicKey)];case 2:return o=n.sent(),i=c.BrowserCrypto.getJwkString(o),s={alg:o.alg,type:d.KEY_FORMAT_JWK},a=this.b64Encode.urlEncode(JSON.stringify(s)),e.cnf={jwk:JSON.parse(i)},h=this.b64Encode.urlEncode(JSON.stringify(e)),p=a+"."+h,m=u.BrowserStringUtils.stringToArrayBuffer(p),[4,this.browserCrypto.sign(r.privateKey,m)];case 3:return g=n.sent(),f=this.b64Encode.urlEncodeArr(new Uint8Array(g)),[2,p+"."+f]}})})},e.POP_KEY_USAGES=["sign","verify"],e.EXTRACTABLE=!0,e}()},"./node_modules/@azure/msal-browser/dist/crypto/GuidGenerator.js":function(e,t,r){"use strict";r.r(t),r.d(t,"GuidGenerator",function(){return o});var n=r("./node_modules/@azure/msal-browser/dist/utils/MathUtils.js"),o=function(){function e(e){this.cryptoObj=e}return e.prototype.generateGuid=function(){try{var e=new Uint8Array(16);return this.cryptoObj.getRandomValues(e),e[6]|=64,e[6]&=79,e[8]|=128,e[8]&=191,n.MathUtils.decimalToHex(e[0])+n.MathUtils.decimalToHex(e[1])+n.MathUtils.decimalToHex(e[2])+n.MathUtils.decimalToHex(e[3])+"-"+n.MathUtils.decimalToHex(e[4])+n.MathUtils.decimalToHex(e[5])+"-"+n.MathUtils.decimalToHex(e[6])+n.MathUtils.decimalToHex(e[7])+"-"+n.MathUtils.decimalToHex(e[8])+n.MathUtils.decimalToHex(e[9])+"-"+n.MathUtils.decimalToHex(e[10])+n.MathUtils.decimalToHex(e[11])+n.MathUtils.decimalToHex(e[12])+n.MathUtils.decimalToHex(e[13])+n.MathUtils.decimalToHex(e[14])+n.MathUtils.decimalToHex(e[15])}catch(e){for(var t="xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx",r="0123456789abcdef",o=0,i="",s=0;s<36;s++)"-"!==t[s]&&"4"!==t[s]&&(o=16*Math.random()|0),"x"===t[s]?i+=r[o]:"y"===t[s]?(o&=3,i+=r[o|=8]):i+=t[s];return i}},e.isGuid=function(e){return/^[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/i.test(e)},e}();
/*! @azure/msal-browser v2.19.0 2021-11-02 */},"./node_modules/@azure/msal-browser/dist/crypto/PkceGenerator.js":function(e,t,r){"use strict";r.r(t),r.d(t,"PkceGenerator",function(){return s});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),i=r("./node_modules/@azure/msal-browser/dist/encode/Base64Encode.js"),s=function(){function e(e){this.base64Encode=new i.Base64Encode,this.cryptoObj=e}return e.prototype.generateCodes=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){var e,t;return Object(n.__generator)(this,function(r){switch(r.label){case 0:return e=this.generateCodeVerifier(),[4,this.generateCodeChallengeFromVerifier(e)];case 1:return t=r.sent(),[2,{verifier:e,challenge:t}]}})})},e.prototype.generateCodeVerifier=function(){try{var e=new Uint8Array(32);return this.cryptoObj.getRandomValues(e),this.base64Encode.urlEncodeArr(e)}catch(e){throw o.BrowserAuthError.createPkceNotGeneratedError(e)}},e.prototype.generateCodeChallengeFromVerifier=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return n.trys.push([0,2,,3]),[4,this.cryptoObj.sha256Digest(e)];case 1:return t=n.sent(),[2,this.base64Encode.urlEncodeArr(new Uint8Array(t))];case 2:throw r=n.sent(),o.BrowserAuthError.createPkceNotGeneratedError(r);case 3:return[2]}})})},e}()},"./node_modules/@azure/msal-browser/dist/crypto/SignedHttpRequest.js":function(e,t,r){"use strict";r.r(t),r.d(t,"SignedHttpRequest",function(){return a});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-browser/dist/crypto/CryptoOps.js"),i=r("./node_modules/@azure/msal-common/dist/index.js"),s=r("./node_modules/@azure/msal-browser/dist/packageMetadata.js"),a=function(){function e(e,t){var r=t&&t.loggerOptions||{};this.logger=new i.Logger(r,s.name,s.version),this.cryptoOps=new o.CryptoOps(this.logger),this.popTokenGenerator=new i.PopTokenGenerator(this.cryptoOps),this.shrParameters=e}return e.prototype.generatePublicKeyThumbprint=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(e){switch(e.label){case 0:return[4,this.popTokenGenerator.generateKid(this.shrParameters)];case 1:return[2,e.sent().kid]}})})},e.prototype.signRequest=function(e,t,r){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(n){return[2,this.popTokenGenerator.signPayload(e,t,this.shrParameters,r)]})})},e.prototype.removeKeys=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(t){switch(t.label){case 0:return[4,this.cryptoOps.removeTokenBindingKey(e)];case 1:return[2,t.sent()]}})})},e}()},"./node_modules/@azure/msal-browser/dist/encode/Base64Decode.js":function(e,t,r){"use strict";r.r(t),r.d(t,"Base64Decode",function(){return o});var n=r("./node_modules/@azure/msal-browser/dist/utils/BrowserStringUtils.js"),o=function(){function e(){}return e.prototype.decode=function(e){var t=e.replace(/-/g,"+").replace(/_/g,"/");switch(t.length%4){case 0:break;case 2:t+="==";break;case 3:t+="=";break;default:throw new Error("Invalid base64 string")}var r=this.base64DecToArr(t);return n.BrowserStringUtils.utf8ArrToString(r)},e.prototype.base64DecToArr=function(e,t){for(var r=e.replace(/[^A-Za-z0-9\+\/]/g,""),n=r.length,o=t?Math.ceil((3*n+1>>>2)/t)*t:3*n+1>>>2,i=new Uint8Array(o),s=void 0,a=void 0,c=0,u=0,d=0;d<n;d++)if(a=3&d,c|=this.b64ToUint6(r.charCodeAt(d))<<18-6*a,3===a||n-d==1){for(s=0;s<3&&u<o;s++,u++)i[u]=c>>>(16>>>s&24)&255;c=0}return i},e.prototype.b64ToUint6=function(e){return e>64&&e<91?e-65:e>96&&e<123?e-71:e>47&&e<58?e+4:43===e?62:47===e?63:0},e}();
/*! @azure/msal-browser v2.19.0 2021-11-02 */},"./node_modules/@azure/msal-browser/dist/encode/Base64Encode.js":function(e,t,r){"use strict";r.r(t),r.d(t,"Base64Encode",function(){return o});var n=r("./node_modules/@azure/msal-browser/dist/utils/BrowserStringUtils.js"),o=function(){function e(){}return e.prototype.urlEncode=function(e){return encodeURIComponent(this.encode(e).replace(/=/g,"").replace(/\+/g,"-").replace(/\//g,"_"))},e.prototype.urlEncodeArr=function(e){return this.base64EncArr(e).replace(/=/g,"").replace(/\+/g,"-").replace(/\//g,"_")},e.prototype.encode=function(e){var t=n.BrowserStringUtils.stringToUtf8Arr(e);return this.base64EncArr(t)},e.prototype.base64EncArr=function(e){for(var t=(3-e.length%3)%3,r="",n=void 0,o=e.length,i=0,s=0;s<o;s++)n=s%3,i|=e[s]<<(16>>>n&24),2!==n&&e.length-s!=1||(r+=String.fromCharCode(this.uint6ToB64(i>>>18&63),this.uint6ToB64(i>>>12&63),this.uint6ToB64(i>>>6&63),this.uint6ToB64(63&i)),i=0);return 0===t?r:r.substring(0,r.length-t)+(1===t?"=":"==")},e.prototype.uint6ToB64=function(e){return e<26?e+65:e<52?e+71:e<62?e-4:62===e?43:63===e?47:65},e}();
/*! @azure/msal-browser v2.19.0 2021-11-02 */},"./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js":function(e,t,r){"use strict";r.r(t),r.d(t,"BrowserAuthError",function(){return s}),r.d(t,"BrowserAuthErrorMessage",function(){return i});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i={pkceNotGenerated:{code:"pkce_not_created",desc:"The PKCE code challenge and verifier could not be generated."},cryptoDoesNotExist:{code:"crypto_nonexistent",desc:"The crypto object or function is not available."},httpMethodNotImplementedError:{code:"http_method_not_implemented",desc:"The HTTP method given has not been implemented in this library."},emptyNavigateUriError:{code:"empty_navigate_uri",desc:"Navigation URI is empty. Please check stack trace for more info."},hashEmptyError:{code:"hash_empty_error",desc:"Hash value cannot be processed because it is empty. Please verify that your redirectUri is not clearing the hash."},hashDoesNotContainStateError:{code:"no_state_in_hash",desc:"Hash does not contain state. Please verify that the request originated from msal."},hashDoesNotContainKnownPropertiesError:{code:"hash_does_not_contain_known_properties",desc:"Hash does not contain known properites. Please verify that your redirectUri is not changing the hash."},unableToParseStateError:{code:"unable_to_parse_state",desc:"Unable to parse state. Please verify that the request originated from msal."},stateInteractionTypeMismatchError:{code:"state_interaction_type_mismatch",desc:"Hash contains state but the interaction type does not match the caller."},interactionInProgress:{code:"interaction_in_progress",desc:"Interaction is currently in progress. Please ensure that this interaction has been completed before calling an interactive API.  For more visit: aka.ms/msaljs/browser-errors."},popupWindowError:{code:"popup_window_error",desc:"Error opening popup window. This can happen if you are using IE or if popups are blocked in the browser."},emptyWindowError:{code:"empty_window_error",desc:"window.open returned null or undefined window object."},userCancelledError:{code:"user_cancelled",desc:"User cancelled the flow."},monitorPopupTimeoutError:{code:"monitor_window_timeout",desc:"Token acquisition in popup failed due to timeout. For more visit: aka.ms/msaljs/browser-errors."},monitorIframeTimeoutError:{code:"monitor_window_timeout",desc:"Token acquisition in iframe failed due to timeout. For more visit: aka.ms/msaljs/browser-errors."},redirectInIframeError:{code:"redirect_in_iframe",desc:"Code flow is not supported inside an iframe. Please ensure you are using MSAL.js in a top frame of the window if using the redirect APIs, or use the popup APIs."},blockTokenRequestsInHiddenIframeError:{code:"block_iframe_reload",desc:"Request was blocked inside an iframe because MSAL detected an authentication response. For more visit: aka.ms/msaljs/browser-errors"},blockAcquireTokenInPopupsError:{code:"block_nested_popups",desc:"Request was blocked inside a popup because MSAL detected it was running in a popup."},iframeClosedPrematurelyError:{code:"iframe_closed_prematurely",desc:"The iframe being monitored was closed prematurely."},silentLogoutUnsupportedError:{code:"silent_logout_unsupported",desc:"Silent logout not supported. Please call logoutRedirect or logoutPopup instead."},noAccountError:{code:"no_account_error",desc:"No account object provided to acquireTokenSilent and no active account has been set. Please call setActiveAccount or provide an account on the request."},silentPromptValueError:{code:"silent_prompt_value_error",desc:"The value given for the prompt value is not valid for silent requests - must be set to 'none'."},noTokenRequestCacheError:{code:"no_token_request_cache_error",desc:"No token request found in cache."},unableToParseTokenRequestCacheError:{code:"unable_to_parse_token_request_cache_error",desc:"The cached token request could not be parsed."},noCachedAuthorityError:{code:"no_cached_authority_error",desc:"No cached authority found."},authRequestNotSet:{code:"auth_request_not_set_error",desc:"Auth Request not set. Please ensure initiateAuthRequest was called from the InteractionHandler"},invalidCacheType:{code:"invalid_cache_type",desc:"Invalid cache type"},notInBrowserEnvironment:{code:"non_browser_environment",desc:"Login and token requests are not supported in non-browser environments."},databaseNotOpen:{code:"database_not_open",desc:"Database is not open!"},noNetworkConnectivity:{code:"no_network_connectivity",desc:"No network connectivity. Check your internet connection."},postRequestFailed:{code:"post_request_failed",desc:"Network request failed: If the browser threw a CORS error, check that the redirectUri is registered in the Azure App Portal as type 'SPA'"},getRequestFailed:{code:"get_request_failed",desc:"Network request failed. Please check the network trace to determine root cause."},failedToParseNetworkResponse:{code:"failed_to_parse_response",desc:"Failed to parse network response. Check network trace."},unableToLoadTokenError:{code:"unable_to_load_token",desc:"Error loading token to cache."},signingKeyNotFoundInStorage:{code:"crypto_key_not_found",desc:"Cryptographic Key or Keypair not found in browser storage."},databaseUnavailable:{code:"database_unavailable",desc:"IndexedDB, which is required for persistent cryptographic key storage, is unavailable. This may be caused by browser privacy features which block persistent storage in third-party contexts."}},s=function(e){function t(r,n){var o=e.call(this,r,n)||this;return Object.setPrototypeOf(o,t.prototype),o.name="BrowserAuthError",o}return Object(n.__extends)(t,e),t.createPkceNotGeneratedError=function(e){return new t(i.pkceNotGenerated.code,i.pkceNotGenerated.desc+" Detail:"+e)},t.createCryptoNotAvailableError=function(e){return new t(i.cryptoDoesNotExist.code,i.cryptoDoesNotExist.desc+" Detail:"+e)},t.createHttpMethodNotImplementedError=function(e){return new t(i.httpMethodNotImplementedError.code,i.httpMethodNotImplementedError.desc+" Given Method: "+e)},t.createEmptyNavigationUriError=function(){return new t(i.emptyNavigateUriError.code,i.emptyNavigateUriError.desc)},t.createEmptyHashError=function(e){return new t(i.hashEmptyError.code,i.hashEmptyError.desc+" Given Url: "+e)},t.createHashDoesNotContainStateError=function(){return new t(i.hashDoesNotContainStateError.code,i.hashDoesNotContainStateError.desc)},t.createHashDoesNotContainKnownPropertiesError=function(){return new t(i.hashDoesNotContainKnownPropertiesError.code,i.hashDoesNotContainKnownPropertiesError.desc)},t.createUnableToParseStateError=function(){return new t(i.unableToParseStateError.code,i.unableToParseStateError.desc)},t.createStateInteractionTypeMismatchError=function(){return new t(i.stateInteractionTypeMismatchError.code,i.stateInteractionTypeMismatchError.desc)},t.createInteractionInProgressError=function(){return new t(i.interactionInProgress.code,i.interactionInProgress.desc)},t.createPopupWindowError=function(e){var r=i.popupWindowError.desc;return r=o.StringUtils.isEmpty(e)?r:r+" Details: "+e,new t(i.popupWindowError.code,r)},t.createEmptyWindowCreatedError=function(){return new t(i.emptyWindowError.code,i.emptyWindowError.desc)},t.createUserCancelledError=function(){return new t(i.userCancelledError.code,i.userCancelledError.desc)},t.createMonitorPopupTimeoutError=function(){return new t(i.monitorPopupTimeoutError.code,i.monitorPopupTimeoutError.desc)},t.createMonitorIframeTimeoutError=function(){return new t(i.monitorIframeTimeoutError.code,i.monitorIframeTimeoutError.desc)},t.createRedirectInIframeError=function(e){return new t(i.redirectInIframeError.code,i.redirectInIframeError.desc+" (window.parent !== window) => "+e)},t.createBlockReloadInHiddenIframeError=function(){return new t(i.blockTokenRequestsInHiddenIframeError.code,i.blockTokenRequestsInHiddenIframeError.desc)},t.createBlockAcquireTokenInPopupsError=function(){return new t(i.blockAcquireTokenInPopupsError.code,i.blockAcquireTokenInPopupsError.desc)},t.createIframeClosedPrematurelyError=function(){return new t(i.iframeClosedPrematurelyError.code,i.iframeClosedPrematurelyError.desc)},t.createSilentLogoutUnsupportedError=function(){return new t(i.silentLogoutUnsupportedError.code,i.silentLogoutUnsupportedError.desc)},t.createNoAccountError=function(){return new t(i.noAccountError.code,i.noAccountError.desc)},t.createSilentPromptValueError=function(e){return new t(i.silentPromptValueError.code,i.silentPromptValueError.desc+" Given value: "+e)},t.createUnableToParseTokenRequestCacheError=function(){return new t(i.unableToParseTokenRequestCacheError.code,i.unableToParseTokenRequestCacheError.desc)},t.createNoTokenRequestCacheError=function(){return new t(i.noTokenRequestCacheError.code,i.noTokenRequestCacheError.desc)},t.createAuthRequestNotSetError=function(){return new t(i.authRequestNotSet.code,i.authRequestNotSet.desc)},t.createNoCachedAuthorityError=function(){return new t(i.noCachedAuthorityError.code,i.noCachedAuthorityError.desc)},t.createInvalidCacheTypeError=function(){return new t(i.invalidCacheType.code,""+i.invalidCacheType.desc)},t.createNonBrowserEnvironmentError=function(){return new t(i.notInBrowserEnvironment.code,i.notInBrowserEnvironment.desc)},t.createDatabaseNotOpenError=function(){return new t(i.databaseNotOpen.code,i.databaseNotOpen.desc)},t.createNoNetworkConnectivityError=function(){return new t(i.noNetworkConnectivity.code,i.noNetworkConnectivity.desc)},t.createPostRequestFailedError=function(e,r){return new t(i.postRequestFailed.code,i.postRequestFailed.desc+" | Network client threw: "+e+" | Attempted to reach: "+r.split("?")[0])},t.createGetRequestFailedError=function(e,r){return new t(i.getRequestFailed.code,i.getRequestFailed.desc+" | Network client threw: "+e+" | Attempted to reach: "+r.split("?")[0])},t.createFailedToParseNetworkResponseError=function(e){return new t(i.failedToParseNetworkResponse.code,i.failedToParseNetworkResponse.desc+" | Attempted to reach: "+e.split("?")[0])},t.createUnableToLoadTokenError=function(e){return new t(i.unableToLoadTokenError.code,i.unableToLoadTokenError.desc+" | "+e)},t.createSigningKeyNotFoundInStorageError=function(e){return new t(i.signingKeyNotFoundInStorage.code,i.signingKeyNotFoundInStorage.desc+" | No match found for KeyId: "+e)},t.createDatabaseUnavailableError=function(){return new t(i.databaseUnavailable.code,i.databaseUnavailable.desc)},t}(o.AuthError)},"./node_modules/@azure/msal-browser/dist/error/BrowserConfigurationAuthError.js":function(e,t,r){"use strict";r.r(t),r.d(t,"BrowserConfigurationAuthError",function(){return s}),r.d(t,"BrowserConfigurationAuthErrorMessage",function(){return i});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i={redirectUriNotSet:{code:"redirect_uri_empty",desc:"A redirect URI is required for all calls, and none has been set."},postLogoutUriNotSet:{code:"post_logout_uri_empty",desc:"A post logout redirect has not been set."},storageNotSupportedError:{code:"storage_not_supported",desc:"Given storage configuration option was not supported."},noRedirectCallbacksSet:{code:"no_redirect_callbacks",desc:"No redirect callbacks have been set. Please call setRedirectCallbacks() with the appropriate function arguments before continuing. More information is available here: https://github.com/AzureAD/microsoft-authentication-library-for-js/wiki/MSAL-basics."},invalidCallbackObject:{code:"invalid_callback_object",desc:"The object passed for the callback was invalid. More information is available here: https://github.com/AzureAD/microsoft-authentication-library-for-js/wiki/MSAL-basics."},stubPcaInstanceCalled:{code:"stubbed_public_client_application_called",desc:"Stub instance of Public Client Application was called. If using msal-react, please ensure context is not used without a provider. For more visit: aka.ms/msaljs/browser-errors"},inMemRedirectUnavailable:{code:"in_mem_redirect_unavailable",desc:"Redirect cannot be supported. In-memory storage was selected and storeAuthStateInCookie=false, which would cause the library to be unable to handle the incoming hash. If you would like to use the redirect API, please use session/localStorage or set storeAuthStateInCookie=true."}},s=function(e){function t(r,n){var o=e.call(this,r,n)||this;return o.name="BrowserConfigurationAuthError",Object.setPrototypeOf(o,t.prototype),o}return Object(n.__extends)(t,e),t.createRedirectUriEmptyError=function(){return new t(i.redirectUriNotSet.code,i.redirectUriNotSet.desc)},t.createPostLogoutRedirectUriEmptyError=function(){return new t(i.postLogoutUriNotSet.code,i.postLogoutUriNotSet.desc)},t.createStorageNotSupportedError=function(e){return new t(i.storageNotSupportedError.code,i.storageNotSupportedError.desc+" Given Location: "+e)},t.createRedirectCallbacksNotSetError=function(){return new t(i.noRedirectCallbacksSet.code,i.noRedirectCallbacksSet.desc)},t.createStubPcaInstanceCalledError=function(){return new t(i.stubPcaInstanceCalled.code,i.stubPcaInstanceCalled.desc)},t.createInMemoryRedirectUnavailableError=function(){return new t(i.inMemRedirectUnavailable.code,i.inMemRedirectUnavailable.desc)},t}(o.AuthError)},"./node_modules/@azure/msal-browser/dist/event/EventHandler.js":function(e,t,r){"use strict";r.r(t),r.d(t,"EventHandler",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/index.js"),o=r("./node_modules/@azure/msal-browser/dist/event/EventType.js"),i=function(){function e(e,t){this.eventCallbacks=new Map,this.logger=e,this.browserCrypto=t,this.listeningToStorageEvents=!1,this.handleAccountCacheChange=this.handleAccountCacheChange.bind(this)}return e.prototype.addEventCallback=function(e){if("undefined"!=typeof window){var t=this.browserCrypto.createNewGuid();return this.eventCallbacks.set(t,e),this.logger.verbose("Event callback registered with id: "+t),t}return null},e.prototype.removeEventCallback=function(e){this.eventCallbacks.delete(e),this.logger.verbose("Event callback "+e+" removed.")},e.prototype.enableAccountStorageEvents=function(){"undefined"!=typeof window&&(this.listeningToStorageEvents?this.logger.verbose("Account storage listener already registered."):(this.logger.verbose("Adding account storage listener."),this.listeningToStorageEvents=!0,window.addEventListener("storage",this.handleAccountCacheChange)))},e.prototype.disableAccountStorageEvents=function(){"undefined"!=typeof window&&(this.listeningToStorageEvents?(this.logger.verbose("Removing account storage listener."),window.removeEventListener("storage",this.handleAccountCacheChange),this.listeningToStorageEvents=!1):this.logger.verbose("No account storage listener registered."))},e.prototype.emitEvent=function(e,t,r,n){var o=this;if("undefined"!=typeof window){var i={eventType:e,interactionType:t||null,payload:r||null,error:n||null,timestamp:Date.now()};this.logger.info("Emitting event: "+e),this.eventCallbacks.forEach(function(t,r){o.logger.verbose("Emitting event to callback "+r+": "+e),t.apply(null,[i])})}},e.prototype.handleAccountCacheChange=function(e){try{var t=e.newValue||e.oldValue;if(!t)return;var r=JSON.parse(t);if("object"!=typeof r||!n.AccountEntity.isAccountEntity(r))return;var i=n.CacheManager.toObject(new n.AccountEntity,r).getAccountInfo();!e.oldValue&&e.newValue?(this.logger.info("Account was added to cache in a different window"),this.emitEvent(o.EventType.ACCOUNT_ADDED,void 0,i)):!e.newValue&&e.oldValue&&(this.logger.info("Account was removed from cache in a different window"),this.emitEvent(o.EventType.ACCOUNT_REMOVED,void 0,i))}catch(e){return}},e}()},"./node_modules/@azure/msal-browser/dist/event/EventMessage.js":function(e,t,r){"use strict";r.r(t),r.d(t,"EventMessageUtils",function(){return i});var n=r("./node_modules/@azure/msal-browser/dist/event/EventType.js"),o=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),i=function(){function e(){}return e.getInteractionStatusFromEvent=function(e,t){switch(e.eventType){case n.EventType.LOGIN_START:return o.InteractionStatus.Login;case n.EventType.SSO_SILENT_START:return o.InteractionStatus.SsoSilent;case n.EventType.ACQUIRE_TOKEN_START:if(e.interactionType===o.InteractionType.Redirect||e.interactionType===o.InteractionType.Popup)return o.InteractionStatus.AcquireToken;break;case n.EventType.HANDLE_REDIRECT_START:return o.InteractionStatus.HandleRedirect;case n.EventType.LOGOUT_START:return o.InteractionStatus.Logout;case n.EventType.SSO_SILENT_SUCCESS:case n.EventType.SSO_SILENT_FAILURE:if(t&&t!==o.InteractionStatus.SsoSilent)break;return o.InteractionStatus.None;case n.EventType.LOGOUT_END:if(t&&t!==o.InteractionStatus.Logout)break;return o.InteractionStatus.None;case n.EventType.HANDLE_REDIRECT_END:if(t&&t!==o.InteractionStatus.HandleRedirect)break;return o.InteractionStatus.None;case n.EventType.LOGIN_SUCCESS:case n.EventType.LOGIN_FAILURE:case n.EventType.ACQUIRE_TOKEN_SUCCESS:case n.EventType.ACQUIRE_TOKEN_FAILURE:if(e.interactionType===o.InteractionType.Redirect||e.interactionType===o.InteractionType.Popup){if(t&&t!==o.InteractionStatus.Login&&t!==o.InteractionStatus.AcquireToken)break;return o.InteractionStatus.None}}return null},e}()},"./node_modules/@azure/msal-browser/dist/event/EventType.js":function(e,t,r){"use strict";
/*! @azure/msal-browser v2.19.0 2021-11-02 */
var n;r.r(t),r.d(t,"EventType",function(){return n}),function(e){e.ACCOUNT_ADDED="msal:accountAdded",e.ACCOUNT_REMOVED="msal:accountRemoved",e.LOGIN_START="msal:loginStart",e.LOGIN_SUCCESS="msal:loginSuccess",e.LOGIN_FAILURE="msal:loginFailure",e.ACQUIRE_TOKEN_START="msal:acquireTokenStart",e.ACQUIRE_TOKEN_SUCCESS="msal:acquireTokenSuccess",e.ACQUIRE_TOKEN_FAILURE="msal:acquireTokenFailure",e.ACQUIRE_TOKEN_NETWORK_START="msal:acquireTokenFromNetworkStart",e.SSO_SILENT_START="msal:ssoSilentStart",e.SSO_SILENT_SUCCESS="msal:ssoSilentSuccess",e.SSO_SILENT_FAILURE="msal:ssoSilentFailure",e.HANDLE_REDIRECT_START="msal:handleRedirectStart",e.HANDLE_REDIRECT_END="msal:handleRedirectEnd",e.POPUP_OPENED="msal:popupOpened",e.LOGOUT_START="msal:logoutStart",e.LOGOUT_SUCCESS="msal:logoutSuccess",e.LOGOUT_FAILURE="msal:logoutFailure",e.LOGOUT_END="msal:logoutEnd"}(n||(n={}))},"./node_modules/@azure/msal-browser/dist/index.js":function(e,t,r){"use strict";r.r(t);var n=r("./node_modules/@azure/msal-browser/dist/app/PublicClientApplication.js");r.d(t,"PublicClientApplication",function(){return n.PublicClientApplication});var o=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js");r.d(t,"ApiId",function(){return o.ApiId}),r.d(t,"BrowserCacheLocation",function(){return o.BrowserCacheLocation}),r.d(t,"InteractionStatus",function(){return o.InteractionStatus}),r.d(t,"InteractionType",function(){return o.InteractionType}),r.d(t,"WrapperSKU",function(){return o.WrapperSKU});var i=r("./node_modules/@azure/msal-browser/dist/utils/BrowserUtils.js");r.d(t,"BrowserUtils",function(){return i.BrowserUtils});var s=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js");r.d(t,"BrowserAuthError",function(){return s.BrowserAuthError}),r.d(t,"BrowserAuthErrorMessage",function(){return s.BrowserAuthErrorMessage});var a=r("./node_modules/@azure/msal-browser/dist/error/BrowserConfigurationAuthError.js");r.d(t,"BrowserConfigurationAuthError",function(){return a.BrowserConfigurationAuthError}),r.d(t,"BrowserConfigurationAuthErrorMessage",function(){return a.BrowserConfigurationAuthErrorMessage});var c=r("./node_modules/@azure/msal-browser/dist/app/IPublicClientApplication.js");r.d(t,"stubbedPublicClientApplication",function(){return c.stubbedPublicClientApplication});var u=r("./node_modules/@azure/msal-browser/dist/navigation/NavigationClient.js");r.d(t,"NavigationClient",function(){return u.NavigationClient});var d=r("./node_modules/@azure/msal-browser/dist/event/EventMessage.js");r.d(t,"EventMessageUtils",function(){return d.EventMessageUtils});var l=r("./node_modules/@azure/msal-browser/dist/event/EventType.js");r.d(t,"EventType",function(){return l.EventType});var h=r("./node_modules/@azure/msal-browser/dist/crypto/SignedHttpRequest.js");r.d(t,"SignedHttpRequest",function(){return h.SignedHttpRequest});var p=r("./node_modules/@azure/msal-common/dist/index.js");r.d(t,"AccountEntity",function(){return p.AccountEntity}),r.d(t,"AuthError",function(){return p.AuthError}),r.d(t,"AuthErrorMessage",function(){return p.AuthErrorMessage}),r.d(t,"AuthenticationHeaderParser",function(){return p.AuthenticationHeaderParser}),r.d(t,"AuthenticationScheme",function(){return p.AuthenticationScheme}),r.d(t,"ClientAuthError",function(){return p.ClientAuthError}),r.d(t,"ClientAuthErrorMessage",function(){return p.ClientAuthErrorMessage}),r.d(t,"ClientConfigurationError",function(){return p.ClientConfigurationError}),r.d(t,"ClientConfigurationErrorMessage",function(){return p.ClientConfigurationErrorMessage}),r.d(t,"InteractionRequiredAuthError",function(){return p.InteractionRequiredAuthError}),r.d(t,"InteractionRequiredAuthErrorMessage",function(){return p.InteractionRequiredAuthErrorMessage}),r.d(t,"LogLevel",function(){return p.LogLevel}),r.d(t,"Logger",function(){return p.Logger}),r.d(t,"ProtocolMode",function(){return p.ProtocolMode}),r.d(t,"ServerError",function(){return p.ServerError}),r.d(t,"StringUtils",function(){return p.StringUtils}),r.d(t,"UrlString",function(){return p.UrlString});var m=r("./node_modules/@azure/msal-browser/dist/packageMetadata.js");r.d(t,"version",function(){return m.version})},"./node_modules/@azure/msal-browser/dist/interaction_client/BaseInteractionClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"BaseInteractionClient",function(){return c});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i=r("./node_modules/@azure/msal-browser/dist/packageMetadata.js"),s=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),a=r("./node_modules/@azure/msal-browser/dist/utils/BrowserUtils.js"),c=function(){function e(e,t,r,n,o,a){this.config=e,this.browserStorage=t,this.browserCrypto=r,this.networkClient=this.config.system.networkClient,this.eventHandler=o,this.correlationId=a||this.browserCrypto.createNewGuid(),this.logger=n.clone(s.BrowserConstants.MSAL_SKU,i.version,this.correlationId)}return e.prototype.clearCacheOnLogout=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(t){switch(t.label){case 0:if(!e)return[3,5];o.AccountEntity.accountInfoIsEqual(e,this.browserStorage.getActiveAccount(),!1)&&(this.logger.verbose("Setting active account to null"),this.browserStorage.setActiveAccount(null)),t.label=1;case 1:return t.trys.push([1,3,,4]),[4,this.browserStorage.removeAccount(o.AccountEntity.generateAccountCacheKey(e))];case 2:return t.sent(),this.logger.verbose("Cleared cache items belonging to the account provided in the logout request."),[3,4];case 3:return t.sent(),this.logger.error("Account provided in logout request was not found. Local cache unchanged."),[3,4];case 4:return[3,9];case 5:return t.trys.push([5,8,,9]),[4,this.browserStorage.clear()];case 6:return t.sent(),[4,this.browserCrypto.clearKeystore()];case 7:return t.sent(),this.logger.verbose("No account provided in logout request, clearing all cache items."),[3,9];case 8:return t.sent(),this.logger.error("Attempted to clear all MSAL cache items and failed. Local cache unchanged."),[3,9];case 9:return[2]}})})},e.prototype.initializeBaseRequest=function(e){this.logger.verbose("Initializing BaseAuthRequest");var t=e.authority||this.config.auth.authority,r=Object(n.__spread)(e&&e.scopes||[]);if(e.authenticationScheme){if(e.authenticationScheme===o.AuthenticationScheme.SSH){if(!e.sshJwk)throw o.ClientConfigurationError.createMissingSshJwkError();if(!e.sshKid)throw o.ClientConfigurationError.createMissingSshKidError()}this.logger.verbose('Authentication Scheme set to "'+e.authenticationScheme+'" as configured in Auth request')}else e.authenticationScheme=o.AuthenticationScheme.BEARER,this.logger.verbose('Authentication Scheme wasn\'t explicitly set in request, defaulting to "Bearer" request');return Object(n.__assign)(Object(n.__assign)({},e),{correlationId:this.correlationId,authority:t,scopes:r})},e.prototype.getRedirectUri=function(e){this.logger.verbose("getRedirectUri called");var t=e||this.config.auth.redirectUri||a.BrowserUtils.getCurrentUri();return o.UrlString.getAbsoluteUrl(t,a.BrowserUtils.getCurrentUri())},e.prototype.initializeServerTelemetryManager=function(e,t){this.logger.verbose("initializeServerTelemetryManager called");var r={clientId:this.config.auth.clientId,correlationId:this.correlationId,apiId:e,forceRefresh:t||!1,wrapperSKU:this.browserStorage.getWrapperMetadata()[0],wrapperVer:this.browserStorage.getWrapperMetadata()[1]};return new o.ServerTelemetryManager(r,this.browserStorage)},e}()},"./node_modules/@azure/msal-browser/dist/interaction_client/PopupClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"PopupClient",function(){return l});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i=r("./node_modules/@azure/msal-browser/dist/interaction_client/StandardInteractionClient.js"),s=r("./node_modules/@azure/msal-browser/dist/utils/PopupUtils.js"),a=r("./node_modules/@azure/msal-browser/dist/event/EventType.js"),c=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),u=r("./node_modules/@azure/msal-browser/dist/interaction_handler/PopupHandler.js"),d=r("./node_modules/@azure/msal-browser/dist/utils/BrowserUtils.js"),l=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(n.__extends)(t,e),t.prototype.acquireToken=function(e){try{var t=this.preflightInteractiveRequest(e,c.InteractionType.Popup),r=s.PopupUtils.generatePopupName(this.config.auth.clientId,t),n=e.popupWindowAttributes||{};if(this.config.system.asyncPopups)return this.logger.verbose("asyncPopups set to true, acquiring token"),this.acquireTokenPopupAsync(t,r,n);this.logger.verbose("asyncPopup set to false, opening popup before acquiring token");var o=s.PopupUtils.openSizedPopup("about:blank",r,n,this.logger);return this.acquireTokenPopupAsync(t,r,n,o)}catch(e){return Promise.reject(e)}},t.prototype.logout=function(e){try{this.logger.verbose("logoutPopup called");var t=this.initializeLogoutRequest(e),r=s.PopupUtils.generateLogoutPopupName(this.config.auth.clientId,t),n=e&&e.authority,o=e&&e.mainWindowRedirectUri,i=(null==e?void 0:e.popupWindowAttributes)||{};if(this.config.system.asyncPopups)return this.logger.verbose("asyncPopups set to true"),this.logoutPopupAsync(t,r,i,n,void 0,o);this.logger.verbose("asyncPopup set to false, opening popup");var a=s.PopupUtils.openSizedPopup("about:blank",r,i,this.logger);return this.logoutPopupAsync(t,r,i,n,a,o)}catch(e){return Promise.reject(e)}},t.prototype.acquireTokenPopupAsync=function(e,t,r,i){return Object(n.__awaiter)(this,void 0,void 0,function(){var s,d,l,h,p,m,g,f,y,C;return Object(n.__generator)(this,function(n){switch(n.label){case 0:this.logger.verbose("acquireTokenPopupAsync called"),s=this.initializeServerTelemetryManager(c.ApiId.acquireTokenPopup),n.label=1;case 1:return n.trys.push([1,7,,8]),[4,this.initializeAuthorizationCodeRequest(e)];case 2:return d=n.sent(),[4,this.createAuthCodeClient(s,e.authority)];case 3:return l=n.sent(),this.logger.verbose("Auth code client created"),[4,l.getAuthCodeUrl(e)];case 4:return h=n.sent(),p=new u.PopupHandler(l,this.browserStorage,d,this.logger),m={popup:i,popupName:t,popupWindowAttributes:r},g=p.initiateAuthRequest(h,m),this.eventHandler.emitEvent(a.EventType.POPUP_OPENED,c.InteractionType.Popup,{popupWindow:g},null),[4,p.monitorPopupForHash(g)];case 5:return f=n.sent(),y=this.validateAndExtractStateFromHash(f,c.InteractionType.Popup,e.correlationId),o.ThrottlingUtils.removeThrottle(this.browserStorage,this.config.auth.clientId,d),[4,p.handleCodeResponse(f,y,l.authority,this.networkClient)];case 6:return[2,n.sent()];case 7:throw C=n.sent(),i&&i.close(),C instanceof o.AuthError&&C.setCorrelationId(this.correlationId),s.cacheFailedRequest(C),this.browserStorage.cleanRequestByState(e.state),C;case 8:return[2]}})})},t.prototype.logoutPopupAsync=function(e,t,r,i,u,l){return Object(n.__awaiter)(this,void 0,void 0,function(){var h,p,m,g,f,y,C,E,_;return Object(n.__generator)(this,function(n){switch(n.label){case 0:this.logger.verbose("logoutPopupAsync called"),this.eventHandler.emitEvent(a.EventType.LOGOUT_START,c.InteractionType.Popup,e),h=this.initializeServerTelemetryManager(c.ApiId.logoutPopup),n.label=1;case 1:return n.trys.push([1,8,,9]),[4,this.clearCacheOnLogout(e.account)];case 2:return n.sent(),this.browserStorage.setInteractionInProgress(!0),[4,this.createAuthCodeClient(h,i)];case 3:p=n.sent(),this.logger.verbose("Auth code client created"),m=p.getLogoutUri(e),this.eventHandler.emitEvent(a.EventType.LOGOUT_SUCCESS,c.InteractionType.Popup,e),g=new s.PopupUtils(this.browserStorage,this.logger),f=g.openPopup(m,{popupName:t,popupWindowAttributes:r,popup:u}),this.eventHandler.emitEvent(a.EventType.POPUP_OPENED,c.InteractionType.Popup,{popupWindow:f},null),n.label=4;case 4:return n.trys.push([4,6,,7]),[4,g.monitorPopupForSameOrigin(f)];case 5:return n.sent(),this.logger.verbose("Popup successfully redirected to postLogoutRedirectUri"),[3,7];case 6:return y=n.sent(),this.logger.verbose("Error occurred while monitoring popup for same origin. Session on server may remain active. Error: "+y),[3,7];case 7:return g.cleanPopup(f),l?(C={apiId:c.ApiId.logoutPopup,timeout:this.config.system.redirectNavigationTimeout,noHistory:!1},E=o.UrlString.getAbsoluteUrl(l,d.BrowserUtils.getCurrentUri()),this.logger.verbose("Redirecting main window to url specified in the request"),this.logger.verbosePii("Redirecing main window to: "+E),this.navigationClient.navigateInternal(E,C)):this.logger.verbose("No main window navigation requested"),[3,9];case 8:throw _=n.sent(),u&&u.close(),_ instanceof o.AuthError&&_.setCorrelationId(this.correlationId),this.browserStorage.setInteractionInProgress(!1),this.eventHandler.emitEvent(a.EventType.LOGOUT_FAILURE,c.InteractionType.Popup,null,_),this.eventHandler.emitEvent(a.EventType.LOGOUT_END,c.InteractionType.Popup),h.cacheFailedRequest(_),_;case 9:return this.eventHandler.emitEvent(a.EventType.LOGOUT_END,c.InteractionType.Popup),[2]}})})},t}(i.StandardInteractionClient)},"./node_modules/@azure/msal-browser/dist/interaction_client/RedirectClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"RedirectClient",function(){return l});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i=r("./node_modules/@azure/msal-browser/dist/interaction_client/StandardInteractionClient.js"),s=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),a=r("./node_modules/@azure/msal-browser/dist/interaction_handler/RedirectHandler.js"),c=r("./node_modules/@azure/msal-browser/dist/utils/BrowserUtils.js"),u=r("./node_modules/@azure/msal-browser/dist/event/EventType.js"),d=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),l=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(n.__extends)(t,e),t.prototype.acquireToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,i,c,u,d,l,h;return Object(n.__generator)(this,function(n){switch(n.label){case 0:t=this.preflightInteractiveRequest(e,s.InteractionType.Redirect),r=this.initializeServerTelemetryManager(s.ApiId.acquireTokenRedirect),n.label=1;case 1:return n.trys.push([1,6,,7]),[4,this.initializeAuthorizationCodeRequest(t)];case 2:return i=n.sent(),[4,this.createAuthCodeClient(r,t.authority)];case 3:return c=n.sent(),this.logger.verbose("Auth code client created"),u=new a.RedirectHandler(c,this.browserStorage,i,this.logger,this.browserCrypto),[4,c.getAuthCodeUrl(t)];case 4:return d=n.sent(),l=this.getRedirectStartPage(e.redirectStartPage),this.logger.verbosePii("Redirect start page: "+l),[4,u.initiateAuthRequest(d,{navigationClient:this.navigationClient,redirectTimeout:this.config.system.redirectNavigationTimeout,redirectStartPage:l,onRedirectNavigate:e.onRedirectNavigate})];case 5:return[2,n.sent()];case 6:throw(h=n.sent())instanceof o.AuthError&&h.setCorrelationId(this.correlationId),r.cacheFailedRequest(h),this.browserStorage.cleanRequestByState(t.state),h;case 7:return[2]}})})},t.prototype.handleRedirectPromise=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,i,a,u,d,l,h,p,m,g;return Object(n.__generator)(this,function(n){switch(n.label){case 0:t=this.initializeServerTelemetryManager(s.ApiId.handleRedirectPromise),n.label=1;case 1:if(n.trys.push([1,10,,11]),!this.browserStorage.isInteractionInProgress(!0))return this.logger.info("handleRedirectPromise called but there is no interaction in progress, returning null."),[2,null];if(!(r=this.getRedirectResponseHash(e||window.location.hash)))return this.logger.info("handleRedirectPromise did not detect a response hash as a result of a redirect. Cleaning temporary cache."),this.browserStorage.cleanRequestByInteractionType(s.InteractionType.Redirect),[2,null];i=void 0;try{i=this.validateAndExtractStateFromHash(r,s.InteractionType.Redirect),c.BrowserUtils.clearHash(window),this.logger.verbose("State extracted from hash")}catch(e){return this.logger.info("handleRedirectPromise was unable to extract state due to: "+e),this.browserStorage.cleanRequestByInteractionType(s.InteractionType.Redirect),[2,null]}return a=this.browserStorage.getTemporaryCache(s.TemporaryCacheKeys.ORIGIN_URI,!0)||"",u=o.UrlString.removeHashFromUrl(a),d=o.UrlString.removeHashFromUrl(window.location.href),u===d&&this.config.auth.navigateToLoginRequestUrl?(this.logger.verbose("Current page is loginRequestUrl, handling hash"),[4,this.handleHash(r,i,t)]):[3,3];case 2:return l=n.sent(),a.indexOf("#")>-1&&c.BrowserUtils.replaceHash(a),[2,l];case 3:return this.config.auth.navigateToLoginRequestUrl?[3,4]:(this.logger.verbose("NavigateToLoginRequestUrl set to false, handling hash"),[2,this.handleHash(r,i,t)]);case 4:return c.BrowserUtils.isInIframe()&&!this.config.system.allowRedirectInIframe?[3,9]:(this.browserStorage.setTemporaryCache(s.TemporaryCacheKeys.URL_HASH,r,!0),h={apiId:s.ApiId.handleRedirectPromise,timeout:this.config.system.redirectNavigationTimeout,noHistory:!0},p=!0,a&&"null"!==a?[3,6]:(m=c.BrowserUtils.getHomepage(),this.browserStorage.setTemporaryCache(s.TemporaryCacheKeys.ORIGIN_URI,m,!0),this.logger.warning("Unable to get valid login request url from cache, redirecting to home page"),[4,this.navigationClient.navigateInternal(m,h)]));case 5:return p=n.sent(),[3,8];case 6:return this.logger.verbose("Navigating to loginRequestUrl: "+a),[4,this.navigationClient.navigateInternal(a,h)];case 7:p=n.sent(),n.label=8;case 8:if(!p)return[2,this.handleHash(r,i,t)];n.label=9;case 9:return[2,null];case 10:throw(g=n.sent())instanceof o.AuthError&&g.setCorrelationId(this.correlationId),t.cacheFailedRequest(g),this.browserStorage.cleanRequestByInteractionType(s.InteractionType.Redirect),g;case 11:return[2]}})})},t.prototype.getRedirectResponseHash=function(e){this.logger.verbose("getRedirectResponseHash called");var t=o.UrlString.hashContainsKnownProperties(e),r=this.browserStorage.getTemporaryCache(s.TemporaryCacheKeys.URL_HASH,!0);return this.browserStorage.removeItem(this.browserStorage.generateCacheKey(s.TemporaryCacheKeys.URL_HASH)),t?(this.logger.verbose("Hash contains known properties, returning response hash"),e):(this.logger.verbose("Hash does not contain known properties, returning cached hash"),r)},t.prototype.handleHash=function(e,t,r){return Object(n.__awaiter)(this,void 0,void 0,function(){var o,i,s;return Object(n.__generator)(this,function(n){switch(n.label){case 0:if(o=this.browserStorage.getCachedRequest(t,this.browserCrypto),this.logger.verbose("handleHash called, retrieved cached request"),!(i=this.browserStorage.getCachedAuthority(t)))throw d.BrowserAuthError.createNoCachedAuthorityError();return[4,this.createAuthCodeClient(r,i)];case 1:return s=n.sent(),this.logger.verbose("Auth code client created"),[4,new a.RedirectHandler(s,this.browserStorage,o,this.logger,this.browserCrypto).handleCodeResponse(e,t,s.authority,this.networkClient,this.config.auth.clientId)];case 2:return[2,n.sent()]}})})},t.prototype.logout=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,i,a,c,d;return Object(n.__generator)(this,function(n){switch(n.label){case 0:this.logger.verbose("logoutRedirect called"),t=this.initializeLogoutRequest(e),r=this.initializeServerTelemetryManager(s.ApiId.logout),n.label=1;case 1:return n.trys.push([1,10,,11]),this.eventHandler.emitEvent(u.EventType.LOGOUT_START,s.InteractionType.Redirect,e),[4,this.clearCacheOnLogout(t.account)];case 2:return n.sent(),i={apiId:s.ApiId.logout,timeout:this.config.system.redirectNavigationTimeout,noHistory:!1},[4,this.createAuthCodeClient(r,e&&e.authority)];case 3:return a=n.sent(),this.logger.verbose("Auth code client created"),c=a.getLogoutUri(t),this.eventHandler.emitEvent(u.EventType.LOGOUT_SUCCESS,s.InteractionType.Redirect,t),e&&"function"==typeof e.onRedirectNavigate?!1===e.onRedirectNavigate(c)?[3,5]:(this.logger.verbose("Logout onRedirectNavigate did not return false, navigating"),[4,this.navigationClient.navigateExternal(c,i)]):[3,7];case 4:return n.sent(),[2];case 5:this.logger.verbose("Logout onRedirectNavigate returned false, stopping navigation"),n.label=6;case 6:return[3,9];case 7:return[4,this.navigationClient.navigateExternal(c,i)];case 8:return n.sent(),[2];case 9:return[3,11];case 10:throw(d=n.sent())instanceof o.AuthError&&d.setCorrelationId(this.correlationId),r.cacheFailedRequest(d),this.eventHandler.emitEvent(u.EventType.LOGOUT_FAILURE,s.InteractionType.Redirect,null,d),this.eventHandler.emitEvent(u.EventType.LOGOUT_END,s.InteractionType.Redirect),d;case 11:return this.eventHandler.emitEvent(u.EventType.LOGOUT_END,s.InteractionType.Redirect),[2]}})})},t.prototype.getRedirectStartPage=function(e){var t=e||window.location.href;return o.UrlString.getAbsoluteUrl(t,c.BrowserUtils.getCurrentUri())},t}(i.StandardInteractionClient)},"./node_modules/@azure/msal-browser/dist/interaction_client/SilentCacheClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"SilentCacheClient",function(){return u});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-browser/dist/interaction_client/StandardInteractionClient.js"),i=r("./node_modules/@azure/msal-common/dist/index.js"),s=r("./node_modules/@azure/msal-browser/dist/event/EventType.js"),a=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),c=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),u=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(n.__extends)(t,e),t.prototype.acquireToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,o,i;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return t=this.initializeServerTelemetryManager(a.ApiId.acquireTokenSilent_silentFlow),[4,this.createSilentFlowClient(t,e.authority)];case 1:r=n.sent(),this.logger.verbose("Silent auth client created"),n.label=2;case 2:return n.trys.push([2,4,,5]),[4,r.acquireCachedToken(e)];case 3:return o=n.sent(),this.eventHandler.emitEvent(s.EventType.ACQUIRE_TOKEN_SUCCESS,a.InteractionType.Silent,o),[2,o];case 4:throw(i=n.sent())instanceof c.BrowserAuthError&&i.errorCode===c.BrowserAuthErrorMessage.signingKeyNotFoundInStorage.code&&this.logger.verbose("Signing keypair for bound access token not found. Refreshing bound access token and generating a new crypto keypair."),i;case 5:return[2]}})})},t.prototype.logout=function(){return Promise.reject(c.BrowserAuthError.createSilentLogoutUnsupportedError())},t.prototype.createSilentFlowClient=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return[4,this.getClientConfiguration(e,t)];case 1:return r=n.sent(),[2,new i.SilentFlowClient(r)]}})})},t.prototype.initializeSilentRequest=function(e,t){return Object(n.__assign)(Object(n.__assign)(Object(n.__assign)({},e),this.initializeBaseRequest(e)),{account:t,forceRefresh:e.forceRefresh||!1})},t}(o.StandardInteractionClient)},"./node_modules/@azure/msal-browser/dist/interaction_client/SilentIframeClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"SilentIframeClient",function(){return u});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i=r("./node_modules/@azure/msal-browser/dist/interaction_client/StandardInteractionClient.js"),s=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),a=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),c=r("./node_modules/@azure/msal-browser/dist/interaction_handler/SilentHandler.js"),u=function(e){function t(t,r,n,o,i,s,a,c){var u=e.call(this,t,r,n,o,i,s,c)||this;return u.apiId=a,u}return Object(n.__extends)(t,e),t.prototype.acquireToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,i,c,u,d;return Object(n.__generator)(this,function(l){switch(l.label){case 0:if(this.logger.verbose("acquireTokenByIframe called"),o.StringUtils.isEmpty(e.loginHint)&&o.StringUtils.isEmpty(e.sid)&&(!e.account||o.StringUtils.isEmpty(e.account.username))&&this.logger.warning("No user hint provided. The authorization server may need more information to complete this request."),e.prompt&&e.prompt!==o.PromptValue.NONE)throw s.BrowserAuthError.createSilentPromptValueError(e.prompt);t=this.initializeAuthorizationRequest(Object(n.__assign)(Object(n.__assign)({},e),{prompt:o.PromptValue.NONE}),a.InteractionType.Silent),r=this.initializeServerTelemetryManager(this.apiId),l.label=1;case 1:return l.trys.push([1,6,,7]),[4,this.initializeAuthorizationCodeRequest(t)];case 2:return i=l.sent(),[4,this.createAuthCodeClient(r,t.authority)];case 3:return c=l.sent(),this.logger.verbose("Auth code client created"),[4,c.getAuthCodeUrl(t)];case 4:return u=l.sent(),[4,this.silentTokenHelper(u,i,c,this.logger)];case 5:return[2,l.sent()];case 6:throw(d=l.sent())instanceof o.AuthError&&d.setCorrelationId(this.correlationId),r.cacheFailedRequest(d),this.browserStorage.cleanRequestByState(t.state),d;case 7:return[2]}})})},t.prototype.logout=function(){return Promise.reject(s.BrowserAuthError.createSilentLogoutUnsupportedError())},t.prototype.silentTokenHelper=function(e,t,r,o){return Object(n.__awaiter)(this,void 0,void 0,function(){var i,s,u,d;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return[4,(i=new c.SilentHandler(r,this.browserStorage,t,o,this.config.system.navigateFrameWait)).initiateAuthRequest(e)];case 1:return s=n.sent(),[4,i.monitorIframeForHash(s,this.config.system.iframeHashTimeout)];case 2:return u=n.sent(),d=this.validateAndExtractStateFromHash(u,a.InteractionType.Silent,t.correlationId),[2,i.handleCodeResponse(u,d,r.authority,this.networkClient)]}})})},t}(i.StandardInteractionClient)},"./node_modules/@azure/msal-browser/dist/interaction_client/SilentRefreshClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"SilentRefreshClient",function(){return c});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-browser/dist/interaction_client/StandardInteractionClient.js"),i=r("./node_modules/@azure/msal-common/dist/index.js"),s=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),a=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),c=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(n.__extends)(t,e),t.prototype.acquireToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,o,a=this;return Object(n.__generator)(this,function(c){switch(c.label){case 0:return t=Object(n.__assign)(Object(n.__assign)({},e),this.initializeBaseRequest(e)),r=this.initializeServerTelemetryManager(s.ApiId.acquireTokenSilent_silentFlow),[4,this.createRefreshTokenClient(r,t.authority)];case 1:return o=c.sent(),this.logger.verbose("Refresh token client created"),[2,o.acquireTokenByRefreshToken(t).catch(function(e){throw e instanceof i.AuthError&&e.setCorrelationId(a.correlationId),r.cacheFailedRequest(e),e})]}})})},t.prototype.logout=function(){return Promise.reject(a.BrowserAuthError.createSilentLogoutUnsupportedError())},t.prototype.createRefreshTokenClient=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return[4,this.getClientConfiguration(e,t)];case 1:return r=n.sent(),[2,new i.RefreshTokenClient(r)]}})})},t}(o.StandardInteractionClient)},"./node_modules/@azure/msal-browser/dist/interaction_client/StandardInteractionClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"StandardInteractionClient",function(){return l});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i=r("./node_modules/@azure/msal-browser/dist/interaction_client/BaseInteractionClient.js"),s=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),a=r("./node_modules/@azure/msal-browser/dist/packageMetadata.js"),c=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),u=r("./node_modules/@azure/msal-browser/dist/utils/BrowserProtocolUtils.js"),d=r("./node_modules/@azure/msal-browser/dist/utils/BrowserUtils.js"),l=function(e){function t(t,r,n,o,i,s,a){var c=e.call(this,t,r,n,o,i,a)||this;return c.navigationClient=s,c}return Object(n.__extends)(t,e),t.prototype.initializeAuthorizationCodeRequest=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r;return Object(n.__generator)(this,function(i){switch(i.label){case 0:return this.logger.verbose("initializeAuthorizationRequest called",e.correlationId),[4,this.browserCrypto.generatePkceCodes()];case 1:return t=i.sent(),r=Object(n.__assign)(Object(n.__assign)({},e),{redirectUri:e.redirectUri,code:"",codeVerifier:t.verifier}),e.codeChallenge=t.challenge,e.codeChallengeMethod=o.Constants.S256_CODE_CHALLENGE_METHOD,[2,r]}})})},t.prototype.initializeLogoutRequest=function(e){if(this.logger.verbose("initializeLogoutRequest called",null==e?void 0:e.correlationId),this.browserStorage.isInteractionInProgress())throw c.BrowserAuthError.createInteractionInProgressError();var t=Object(n.__assign)({correlationId:this.browserCrypto.createNewGuid()},e);return e&&null===e.postLogoutRedirectUri?this.logger.verbose("postLogoutRedirectUri passed as null, not setting post logout redirect uri",t.correlationId):e&&e.postLogoutRedirectUri?(this.logger.verbose("Setting postLogoutRedirectUri to uri set on logout request",t.correlationId),t.postLogoutRedirectUri=o.UrlString.getAbsoluteUrl(e.postLogoutRedirectUri,d.BrowserUtils.getCurrentUri())):null===this.config.auth.postLogoutRedirectUri?this.logger.verbose("postLogoutRedirectUri configured as null and no uri set on request, not passing post logout redirect",t.correlationId):this.config.auth.postLogoutRedirectUri?(this.logger.verbose("Setting postLogoutRedirectUri to configured uri",t.correlationId),t.postLogoutRedirectUri=o.UrlString.getAbsoluteUrl(this.config.auth.postLogoutRedirectUri,d.BrowserUtils.getCurrentUri())):(this.logger.verbose("Setting postLogoutRedirectUri to current page",t.correlationId),t.postLogoutRedirectUri=o.UrlString.getAbsoluteUrl(d.BrowserUtils.getCurrentUri(),d.BrowserUtils.getCurrentUri())),t},t.prototype.createAuthCodeClient=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return[4,this.getClientConfiguration(e,t)];case 1:return r=n.sent(),[2,new o.AuthorizationCodeClient(r)]}})})},t.prototype.getClientConfiguration=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return this.logger.verbose("getClientConfiguration called"),[4,this.getDiscoveredAuthority(t)];case 1:return r=n.sent(),[2,{authOptions:{clientId:this.config.auth.clientId,authority:r,clientCapabilities:this.config.auth.clientCapabilities},systemOptions:{tokenRenewalOffsetSeconds:this.config.system.tokenRenewalOffsetSeconds,preventCorsPreflight:!0},loggerOptions:{loggerCallback:this.config.system.loggerOptions.loggerCallback,piiLoggingEnabled:this.config.system.loggerOptions.piiLoggingEnabled,logLevel:this.config.system.loggerOptions.logLevel,correlationId:this.correlationId},cryptoInterface:this.browserCrypto,networkInterface:this.networkClient,storageInterface:this.browserStorage,serverTelemetryManager:e,libraryInfo:{sku:s.BrowserConstants.MSAL_SKU,version:a.version,cpu:"",os:""}}]}})})},t.prototype.validateAndExtractStateFromHash=function(e,t,r){this.logger.verbose("validateAndExtractStateFromHash called",r);var n=o.UrlString.getDeserializedHash(e);if(!n.state)throw c.BrowserAuthError.createHashDoesNotContainStateError();var i=u.BrowserProtocolUtils.extractBrowserRequestState(this.browserCrypto,n.state);if(!i)throw c.BrowserAuthError.createUnableToParseStateError();if(i.interactionType!==t)throw c.BrowserAuthError.createStateInteractionTypeMismatchError();return this.logger.verbose("Returning state from hash",r),n.state},t.prototype.getDiscoveredAuthority=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t;return Object(n.__generator)(this,function(r){switch(r.label){case 0:return this.logger.verbose("getDiscoveredAuthority called"),t={protocolMode:this.config.auth.protocolMode,knownAuthorities:this.config.auth.knownAuthorities,cloudDiscoveryMetadata:this.config.auth.cloudDiscoveryMetadata,authorityMetadata:this.config.auth.authorityMetadata},e?(this.logger.verbose("Creating discovered authority with request authority"),[4,o.AuthorityFactory.createDiscoveredInstance(e,this.config.system.networkClient,this.browserStorage,t)]):[3,2];case 1:return[2,r.sent()];case 2:return this.logger.verbose("Creating discovered authority with configured authority"),[4,o.AuthorityFactory.createDiscoveredInstance(this.config.auth.authority,this.config.system.networkClient,this.browserStorage,t)];case 3:return[2,r.sent()]}})})},t.prototype.preflightInteractiveRequest=function(e,t){if(this.logger.verbose("preflightInteractiveRequest called, validating app environment",null==e?void 0:e.correlationId),d.BrowserUtils.blockReloadInHiddenIframes(),this.browserStorage.isInteractionInProgress(!1))throw c.BrowserAuthError.createInteractionInProgressError();return this.initializeAuthorizationRequest(e,t)},t.prototype.initializeAuthorizationRequest=function(e,t){this.logger.verbose("initializeAuthorizationRequest called");var r=this.getRedirectUri(e.redirectUri),i={interactionType:t},s=o.ProtocolUtils.setRequestState(this.browserCrypto,e&&e.state||"",i),a=Object(n.__assign)(Object(n.__assign)({},this.initializeBaseRequest(e)),{redirectUri:r,state:s,nonce:e.nonce||this.browserCrypto.createNewGuid(),responseMode:o.ResponseMode.FRAGMENT}),c=e.account||this.browserStorage.getActiveAccount();if(c&&(this.logger.verbose("Setting validated request account"),this.logger.verbosePii("Setting validated request account: "+c),a.account=c),o.StringUtils.isEmpty(a.loginHint)&&!c){var u=this.browserStorage.getLegacyLoginHint();u&&(a.loginHint=u)}return this.browserStorage.updateCacheEntries(a.state,a.nonce,a.authority,a.loginHint||"",a.account||null),a},t}(i.BaseInteractionClient)},"./node_modules/@azure/msal-browser/dist/interaction_handler/InteractionHandler.js":function(e,t,r){"use strict";r.r(t),r.d(t,"InteractionHandler",function(){return a});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),s=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),a=function(){function e(e,t,r,n){this.authModule=e,this.browserStorage=t,this.authCodeRequest=r,this.browserRequestLogger=n}return e.prototype.handleCodeResponse=function(e,t,r,s){return Object(n.__awaiter)(this,void 0,void 0,function(){var a,c,u,d,l,h,p;return Object(n.__generator)(this,function(n){switch(n.label){case 0:if(this.browserRequestLogger.verbose("InteractionHandler.handleCodeResponse called"),o.StringUtils.isEmpty(e))throw i.BrowserAuthError.createEmptyHashError(e);if(a=this.browserStorage.generateStateKey(t),!(c=this.browserStorage.getTemporaryCache(a)))throw o.ClientAuthError.createStateNotFoundError("Cached State");return u=this.authModule.handleFragmentResponse(e,c),d=this.browserStorage.generateNonceKey(c),l=this.browserStorage.getTemporaryCache(d),this.authCodeRequest.code=u.code,u.cloud_instance_host_name?[4,this.updateTokenEndpointAuthority(u.cloud_instance_host_name,r,s)]:[3,2];case 1:n.sent(),n.label=2;case 2:return u.nonce=l||void 0,u.state=c,u.client_info?this.authCodeRequest.clientInfo=u.client_info:(h=this.checkCcsCredentials())&&(this.authCodeRequest.ccsCredential=h),[4,this.authModule.acquireToken(this.authCodeRequest,u)];case 3:return p=n.sent(),this.browserStorage.cleanRequestByState(t),[2,p]}})})},e.prototype.updateTokenEndpointAuthority=function(e,t,r){return Object(n.__awaiter)(this,void 0,void 0,function(){var i,s;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return i="https://"+e+"/"+t.tenant+"/",[4,o.AuthorityFactory.createDiscoveredInstance(i,r,this.browserStorage,t.options)];case 1:return s=n.sent(),this.authModule.updateAuthority(s),[2]}})})},e.prototype.checkCcsCredentials=function(){var e=this.browserStorage.getTemporaryCache(s.TemporaryCacheKeys.CCS_CREDENTIAL,!0);if(e)try{return JSON.parse(e)}catch(t){this.authModule.logger.error("Cache credential could not be parsed"),this.authModule.logger.errorPii("Cache credential could not be parsed: "+e)}return null},e}()},"./node_modules/@azure/msal-browser/dist/interaction_handler/PopupHandler.js":function(e,t,r){"use strict";r.r(t),r.d(t,"PopupHandler",function(){return u});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i=r("./node_modules/@azure/msal-browser/dist/interaction_handler/InteractionHandler.js"),s=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),a=r("./node_modules/@azure/msal-browser/dist/utils/PopupUtils.js"),c=r("./node_modules/@azure/msal-browser/dist/utils/BrowserUtils.js"),u=function(e){function t(t,r,n,o){var i=e.call(this,t,r,n,o)||this;return i.popupUtils=new a.PopupUtils(r,o),i}return Object(n.__extends)(t,e),t.prototype.initiateAuthRequest=function(e,t){if(o.StringUtils.isEmpty(e))throw this.browserRequestLogger.error("Navigate url is empty"),s.BrowserAuthError.createEmptyNavigationUriError();return this.browserStorage.setInteractionInProgress(!0),this.browserRequestLogger.infoPii("Navigate to: "+e),this.popupUtils.openPopup(e,t)},t.prototype.monitorPopupForHash=function(e){var t=this;return this.popupUtils.monitorPopupForSameOrigin(e).then(function(){var r=e.location.hash;if(c.BrowserUtils.clearHash(e),t.popupUtils.cleanPopup(e),!r)throw s.BrowserAuthError.createEmptyHashError(e.location.href);if(o.UrlString.hashContainsKnownProperties(r))return r;throw s.BrowserAuthError.createHashDoesNotContainKnownPropertiesError()})},t}(i.InteractionHandler)},"./node_modules/@azure/msal-browser/dist/interaction_handler/RedirectHandler.js":function(e,t,r){"use strict";r.r(t),r.d(t,"RedirectHandler",function(){return a});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),s=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),a=function(e){function t(t,r,n,o,i){var s=e.call(this,t,r,n,o)||this;return s.browserCrypto=i,s}return Object(n.__extends)(t,e),t.prototype.initiateAuthRequest=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return this.browserRequestLogger.verbose("RedirectHandler.initiateAuthRequest called"),o.StringUtils.isEmpty(e)?[3,7]:(t.redirectStartPage&&(this.browserRequestLogger.verbose("RedirectHandler.initiateAuthRequest: redirectStartPage set, caching start page"),this.browserStorage.setTemporaryCache(s.TemporaryCacheKeys.ORIGIN_URI,t.redirectStartPage,!0)),this.browserStorage.setInteractionInProgress(!0),this.browserStorage.setTemporaryCache(s.TemporaryCacheKeys.CORRELATION_ID,this.authCodeRequest.correlationId,!0),this.browserStorage.cacheCodeRequest(this.authCodeRequest,this.browserCrypto),this.browserRequestLogger.infoPii("RedirectHandler.initiateAuthRequest: Navigate to: "+e),r={apiId:s.ApiId.acquireTokenRedirect,timeout:t.redirectTimeout,noHistory:!1},"function"!=typeof t.onRedirectNavigate?[3,4]:(this.browserRequestLogger.verbose("RedirectHandler.initiateAuthRequest: Invoking onRedirectNavigate callback"),!1===t.onRedirectNavigate(e)?[3,2]:(this.browserRequestLogger.verbose("RedirectHandler.initiateAuthRequest: onRedirectNavigate did not return false, navigating"),[4,t.navigationClient.navigateExternal(e,r)])));case 1:return n.sent(),[2];case 2:return this.browserRequestLogger.verbose("RedirectHandler.initiateAuthRequest: onRedirectNavigate returned false, stopping navigation"),[2];case 3:return[3,6];case 4:return this.browserRequestLogger.verbose("RedirectHandler.initiateAuthRequest: Navigating window to navigate url"),[4,t.navigationClient.navigateExternal(e,r)];case 5:return n.sent(),[2];case 6:return[3,8];case 7:throw this.browserRequestLogger.info("RedirectHandler.initiateAuthRequest: Navigate url is empty"),i.BrowserAuthError.createEmptyNavigationUriError();case 8:return[2]}})})},t.prototype.handleCodeResponse=function(e,t,r,s,a){return Object(n.__awaiter)(this,void 0,void 0,function(){var c,u,d,l,h,p,m;return Object(n.__generator)(this,function(n){switch(n.label){case 0:if(this.browserRequestLogger.verbose("RedirectHandler.handleCodeResponse called"),o.StringUtils.isEmpty(e))throw i.BrowserAuthError.createEmptyHashError(e);if(this.browserStorage.setInteractionInProgress(!1),c=this.browserStorage.generateStateKey(t),!(u=this.browserStorage.getTemporaryCache(c)))throw o.ClientAuthError.createStateNotFoundError("Cached State");return d=this.authModule.handleFragmentResponse(e,u),l=this.browserStorage.generateNonceKey(u),h=this.browserStorage.getTemporaryCache(l),this.authCodeRequest.code=d.code,d.cloud_instance_host_name?[4,this.updateTokenEndpointAuthority(d.cloud_instance_host_name,r,s)]:[3,2];case 1:n.sent(),n.label=2;case 2:return d.nonce=h||void 0,d.state=u,d.client_info?this.authCodeRequest.clientInfo=d.client_info:(p=this.checkCcsCredentials())&&(this.authCodeRequest.ccsCredential=p),a&&o.ThrottlingUtils.removeThrottle(this.browserStorage,a,this.authCodeRequest),[4,this.authModule.acquireToken(this.authCodeRequest,d)];case 3:return m=n.sent(),this.browserStorage.cleanRequestByState(t),[2,m]}})})},t}(r("./node_modules/@azure/msal-browser/dist/interaction_handler/InteractionHandler.js").InteractionHandler)},"./node_modules/@azure/msal-browser/dist/interaction_handler/SilentHandler.js":function(e,t,r){"use strict";r.r(t),r.d(t,"SilentHandler",function(){return u});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/index.js"),i=r("./node_modules/@azure/msal-browser/dist/interaction_handler/InteractionHandler.js"),s=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),a=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),c=r("./node_modules/@azure/msal-browser/dist/config/Configuration.js"),u=function(e){function t(t,r,n,o,i){var s=e.call(this,t,r,n,o)||this;return s.navigateFrameWait=i,s}return Object(n.__extends)(t,e),t.prototype.initiateAuthRequest=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t;return Object(n.__generator)(this,function(r){switch(r.label){case 0:if(o.StringUtils.isEmpty(e))throw this.browserRequestLogger.info("Navigate url is empty"),a.BrowserAuthError.createEmptyNavigationUriError();return this.navigateFrameWait?[4,this.loadFrame(e)]:[3,2];case 1:return t=r.sent(),[3,3];case 2:t=this.loadFrameSync(e),r.label=3;case 3:return[2,t]}})})},t.prototype.monitorIframeForHash=function(e,t){var r=this;return new Promise(function(n,i){t<c.DEFAULT_IFRAME_TIMEOUT_MS&&r.browserRequestLogger.warning("system.loadFrameTimeout or system.iframeHashTimeout set to lower ("+t+"ms) than the default ("+c.DEFAULT_IFRAME_TIMEOUT_MS+"ms). This may result in timeouts.");var u=window.performance.now()+t,d=setInterval(function(){if(window.performance.now()>u)return r.removeHiddenIframe(e),clearInterval(d),void i(a.BrowserAuthError.createMonitorIframeTimeoutError());var t=o.Constants.EMPTY_STRING,s=e.contentWindow;try{t=s?s.location.href:o.Constants.EMPTY_STRING}catch(e){}if(!o.StringUtils.isEmpty(t)){var c=s?s.location.hash:o.Constants.EMPTY_STRING;return o.UrlString.hashContainsKnownProperties(c)?(r.removeHiddenIframe(e),clearInterval(d),void n(c)):void 0}},s.BrowserConstants.POLL_INTERVAL_MS)})},t.prototype.loadFrame=function(e){var t=this;return new Promise(function(r,n){var o=t.createHiddenIframe();setTimeout(function(){o?(o.src=e,r(o)):n("Unable to load iframe")},t.navigateFrameWait)})},t.prototype.loadFrameSync=function(e){var t=this.createHiddenIframe();return t.src=e,t},t.prototype.createHiddenIframe=function(){var e=document.createElement("iframe");return e.style.visibility="hidden",e.style.position="absolute",e.style.width=e.style.height="0",e.style.border="0",e.setAttribute("sandbox","allow-scripts allow-same-origin allow-forms"),document.getElementsByTagName("body")[0].appendChild(e),e},t.prototype.removeHiddenIframe=function(e){document.body===e.parentNode&&document.body.removeChild(e)},t}(i.InteractionHandler)},"./node_modules/@azure/msal-browser/dist/navigation/NavigationClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"NavigationClient",function(){return n});
/*! @azure/msal-browser v2.19.0 2021-11-02 */
var n=function(){function e(){}return e.prototype.navigateInternal=function(t,r){return e.defaultNavigateWindow(t,r)},e.prototype.navigateExternal=function(t,r){return e.defaultNavigateWindow(t,r)},e.defaultNavigateWindow=function(e,t){return t.noHistory?window.location.replace(e):window.location.assign(e),new Promise(function(e){setTimeout(function(){e(!0)},t.timeout)})},e}()},"./node_modules/@azure/msal-browser/dist/network/FetchClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"FetchClient",function(){return s});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),i=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),s=function(){function e(){}return e.prototype.sendGetRequestAsync=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,s,a;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return n.trys.push([0,2,,3]),[4,fetch(e,{method:i.HTTP_REQUEST_TYPE.GET,headers:this.getFetchHeaders(t)})];case 1:return r=n.sent(),[3,3];case 2:throw s=n.sent(),window.navigator.onLine?o.BrowserAuthError.createGetRequestFailedError(s,e):o.BrowserAuthError.createNoNetworkConnectivityError();case 3:return n.trys.push([3,5,,6]),a={headers:this.getHeaderDict(r.headers)},[4,r.json()];case 4:return[2,(a.body=n.sent(),a.status=r.status,a)];case 5:throw n.sent(),o.BrowserAuthError.createFailedToParseNetworkResponseError(e);case 6:return[2]}})})},e.prototype.sendPostRequestAsync=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,s,a,c;return Object(n.__generator)(this,function(n){switch(n.label){case 0:r=t&&t.body||"",n.label=1;case 1:return n.trys.push([1,3,,4]),[4,fetch(e,{method:i.HTTP_REQUEST_TYPE.POST,headers:this.getFetchHeaders(t),body:r})];case 2:return s=n.sent(),[3,4];case 3:throw a=n.sent(),window.navigator.onLine?o.BrowserAuthError.createPostRequestFailedError(a,e):o.BrowserAuthError.createNoNetworkConnectivityError();case 4:return n.trys.push([4,6,,7]),c={headers:this.getHeaderDict(s.headers)},[4,s.json()];case 5:return[2,(c.body=n.sent(),c.status=s.status,c)];case 6:throw n.sent(),o.BrowserAuthError.createFailedToParseNetworkResponseError(e);case 7:return[2]}})})},e.prototype.getFetchHeaders=function(e){var t=new Headers;if(!e||!e.headers)return t;var r=e.headers;return Object.keys(r).forEach(function(e){t.append(e,r[e])}),t},e.prototype.getHeaderDict=function(e){var t={};return e.forEach(function(e,r){t[r]=e}),t},e}()},"./node_modules/@azure/msal-browser/dist/network/XhrClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"XhrClient",function(){return s});var n=r("./node_modules/@azure/msal-browser/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),i=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),s=function(){function e(){}return e.prototype.sendGetRequestAsync=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(r){return[2,this.sendRequestAsync(e,i.HTTP_REQUEST_TYPE.GET,t)]})})},e.prototype.sendPostRequestAsync=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(r){return[2,this.sendRequestAsync(e,i.HTTP_REQUEST_TYPE.POST,t)]})})},e.prototype.sendRequestAsync=function(e,t,r){var n=this;return new Promise(function(s,a){var c=new XMLHttpRequest;if(c.open(t,e,!0),n.setXhrHeaders(c,r),c.onload=function(){(c.status<200||c.status>=300)&&(t===i.HTTP_REQUEST_TYPE.POST?a(o.BrowserAuthError.createPostRequestFailedError("Failed with status "+c.status,e)):a(o.BrowserAuthError.createGetRequestFailedError("Failed with status "+c.status,e)));try{var r=JSON.parse(c.responseText),u={headers:n.getHeaderDict(c),body:r,status:c.status};s(u)}catch(t){a(o.BrowserAuthError.createFailedToParseNetworkResponseError(e))}},c.onerror=function(){window.navigator.onLine?t===i.HTTP_REQUEST_TYPE.POST?a(o.BrowserAuthError.createPostRequestFailedError("Failed with status "+c.status,e)):a(o.BrowserAuthError.createGetRequestFailedError("Failed with status "+c.status,e)):a(o.BrowserAuthError.createNoNetworkConnectivityError())},t===i.HTTP_REQUEST_TYPE.POST&&r&&r.body)c.send(r.body);else{if(t!==i.HTTP_REQUEST_TYPE.GET)throw o.BrowserAuthError.createHttpMethodNotImplementedError(t);c.send()}})},e.prototype.setXhrHeaders=function(e,t){if(t&&t.headers){var r=t.headers;Object.keys(r).forEach(function(t){e.setRequestHeader(t,r[t])})}},e.prototype.getHeaderDict=function(e){var t=e.getAllResponseHeaders().trim().split(/[\r\n]+/),r={};return t.forEach(function(e){var t=e.split(": "),n=t.shift(),o=t.join(": ");n&&o&&(r[n]=o)}),r},e}()},"./node_modules/@azure/msal-browser/dist/packageMetadata.js":function(e,t,r){"use strict";r.r(t),r.d(t,"name",function(){return n}),r.d(t,"version",function(){return o});
/*! @azure/msal-browser v2.19.0 2021-11-02 */
var n="@azure/msal-browser",o="2.19.0"},"./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ApiId",function(){return a}),r.d(t,"BrowserCacheLocation",function(){return n}),r.d(t,"BrowserConstants",function(){return l}),r.d(t,"DB_NAME",function(){return g}),r.d(t,"DB_TABLE_NAME",function(){return y}),r.d(t,"DB_VERSION",function(){return f}),r.d(t,"DEFAULT_REQUEST",function(){return p}),r.d(t,"HTTP_REQUEST_TYPE",function(){return o}),r.d(t,"InMemoryCacheKeys",function(){return s}),r.d(t,"InteractionStatus",function(){return u}),r.d(t,"InteractionType",function(){return c}),r.d(t,"KEY_FORMAT_JWK",function(){return m}),r.d(t,"TemporaryCacheKeys",function(){return i}),r.d(t,"WrapperSKU",function(){return h});var n,o,i,s,a,c,u,d=r("./node_modules/@azure/msal-common/dist/index.js"),l={INTERACTION_IN_PROGRESS_VALUE:"interaction_in_progress",INVALID_GRANT_ERROR:"invalid_grant",POPUP_WIDTH:483,POPUP_HEIGHT:600,POPUP_NAME_PREFIX:"msal",POLL_INTERVAL_MS:50,MSAL_SKU:"msal.js.browser"};
/*! @azure/msal-browser v2.19.0 2021-11-02 */!function(e){e.LocalStorage="localStorage",e.SessionStorage="sessionStorage",e.MemoryStorage="memoryStorage"}(n||(n={})),function(e){e.GET="GET",e.POST="POST"}(o||(o={})),function(e){e.AUTHORITY="authority",e.ACQUIRE_TOKEN_ACCOUNT="acquireToken.account",e.SESSION_STATE="session.state",e.REQUEST_STATE="request.state",e.NONCE_IDTOKEN="nonce.id_token",e.ORIGIN_URI="request.origin",e.RENEW_STATUS="token.renew.status",e.URL_HASH="urlHash",e.REQUEST_PARAMS="request.params",e.SCOPES="scopes",e.INTERACTION_STATUS_KEY="interaction.status",e.CCS_CREDENTIAL="ccs.credential",e.CORRELATION_ID="request.correlationId"}(i||(i={})),function(e){e.WRAPPER_SKU="wrapper.sku",e.WRAPPER_VER="wrapper.version"}(s||(s={})),function(e){e[e.acquireTokenRedirect=861]="acquireTokenRedirect",e[e.acquireTokenPopup=862]="acquireTokenPopup",e[e.ssoSilent=863]="ssoSilent",e[e.acquireTokenSilent_authCode=864]="acquireTokenSilent_authCode",e[e.handleRedirectPromise=865]="handleRedirectPromise",e[e.acquireTokenSilent_silentFlow=61]="acquireTokenSilent_silentFlow",e[e.logout=961]="logout",e[e.logoutPopup=962]="logoutPopup"}(a||(a={})),function(e){e.Redirect="redirect",e.Popup="popup",e.Silent="silent"}(c||(c={})),function(e){e.Startup="startup",e.Login="login",e.Logout="logout",e.AcquireToken="acquireToken",e.SsoSilent="ssoSilent",e.HandleRedirect="handleRedirect",e.None="none"}(u||(u={}));var h,p={scopes:d.OIDC_DEFAULT_SCOPES},m="jwk";!function(e){e.React="@azure/msal-react",e.Angular="@azure/msal-angular"}(h||(h={}));var g="msal.db",f=1,y=g+".keys"},"./node_modules/@azure/msal-browser/dist/utils/BrowserProtocolUtils.js":function(e,t,r){"use strict";r.r(t),r.d(t,"BrowserProtocolUtils",function(){return o});var n=r("./node_modules/@azure/msal-common/dist/index.js"),o=function(){function e(){}return e.extractBrowserRequestState=function(e,t){if(n.StringUtils.isEmpty(t))return null;try{return n.ProtocolUtils.parseRequestState(e,t).libraryState.meta}catch(e){throw n.ClientAuthError.createInvalidStateError(t,e)}},e.parseServerResponseFromHash=function(e){if(!e)return{};var t=new n.UrlString(e);return n.UrlString.getDeserializedHash(t.getHash())},e}();
/*! @azure/msal-browser v2.19.0 2021-11-02 */},"./node_modules/@azure/msal-browser/dist/utils/BrowserStringUtils.js":function(e,t,r){"use strict";r.r(t),r.d(t,"BrowserStringUtils",function(){return n});
/*! @azure/msal-browser v2.19.0 2021-11-02 */
var n=function(){function e(){}return e.stringToUtf8Arr=function(e){for(var t,r=0,n=e.length,o=0;o<n;o++)r+=(t=e.charCodeAt(o))<128?1:t<2048?2:t<65536?3:t<2097152?4:t<67108864?5:6;for(var i=new Uint8Array(r),s=0,a=0;s<r;a++)(t=e.charCodeAt(a))<128?i[s++]=t:t<2048?(i[s++]=192+(t>>>6),i[s++]=128+(63&t)):t<65536?(i[s++]=224+(t>>>12),i[s++]=128+(t>>>6&63),i[s++]=128+(63&t)):t<2097152?(i[s++]=240+(t>>>18),i[s++]=128+(t>>>12&63),i[s++]=128+(t>>>6&63),i[s++]=128+(63&t)):t<67108864?(i[s++]=248+(t>>>24),i[s++]=128+(t>>>18&63),i[s++]=128+(t>>>12&63),i[s++]=128+(t>>>6&63),i[s++]=128+(63&t)):(i[s++]=252+(t>>>30),i[s++]=128+(t>>>24&63),i[s++]=128+(t>>>18&63),i[s++]=128+(t>>>12&63),i[s++]=128+(t>>>6&63),i[s++]=128+(63&t));return i},e.stringToArrayBuffer=function(e){for(var t=new ArrayBuffer(e.length),r=new Uint8Array(t),n=0;n<e.length;n++)r[n]=e.charCodeAt(n);return t},e.utf8ArrToString=function(e){for(var t="",r=void 0,n=e.length,o=0;o<n;o++)r=e[o],t+=String.fromCharCode(r>251&&r<254&&o+5<n?1073741824*(r-252)+(e[++o]-128<<24)+(e[++o]-128<<18)+(e[++o]-128<<12)+(e[++o]-128<<6)+e[++o]-128:r>247&&r<252&&o+4<n?(r-248<<24)+(e[++o]-128<<18)+(e[++o]-128<<12)+(e[++o]-128<<6)+e[++o]-128:r>239&&r<248&&o+3<n?(r-240<<18)+(e[++o]-128<<12)+(e[++o]-128<<6)+e[++o]-128:r>223&&r<240&&o+2<n?(r-224<<12)+(e[++o]-128<<6)+e[++o]-128:r>191&&r<224&&o+1<n?(r-192<<6)+e[++o]-128:r);return t},e}()},"./node_modules/@azure/msal-browser/dist/utils/BrowserUtils.js":function(e,t,r){"use strict";r.r(t),r.d(t,"BrowserUtils",function(){return c});var n=r("./node_modules/@azure/msal-common/dist/index.js"),o=r("./node_modules/@azure/msal-browser/dist/network/FetchClient.js"),i=r("./node_modules/@azure/msal-browser/dist/network/XhrClient.js"),s=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),a=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),c=function(){function e(){}return e.clearHash=function(e){e.location.hash=n.Constants.EMPTY_STRING,"function"==typeof e.history.replaceState&&e.history.replaceState(null,n.Constants.EMPTY_STRING,""+e.location.origin+e.location.pathname+e.location.search)},e.replaceHash=function(e){var t=e.split("#");t.shift(),window.location.hash=t.length>0?t.join("#"):""},e.isInIframe=function(){return window.parent!==window},e.isInPopup=function(){return"undefined"!=typeof window&&!!window.opener&&window.opener!==window&&"string"==typeof window.name&&0===window.name.indexOf(a.BrowserConstants.POPUP_NAME_PREFIX+".")},e.getCurrentUri=function(){return window.location.href.split("?")[0].split("#")[0]},e.getHomepage=function(){var e=new n.UrlString(window.location.href).getUrlComponents();return e.Protocol+"//"+e.HostNameAndPort+"/"},e.getBrowserNetworkClient=function(){return window.fetch&&window.Headers?new o.FetchClient:new i.XhrClient},e.blockReloadInHiddenIframes=function(){if(n.UrlString.hashContainsKnownProperties(window.location.hash)&&e.isInIframe())throw s.BrowserAuthError.createBlockReloadInHiddenIframeError()},e.blockRedirectInIframe=function(t,r){var n=e.isInIframe();if(t===a.InteractionType.Redirect&&n&&!r)throw s.BrowserAuthError.createRedirectInIframeError(n)},e.blockAcquireTokenInPopups=function(){if(e.isInPopup())throw s.BrowserAuthError.createBlockAcquireTokenInPopupsError()},e.blockNonBrowserEnvironment=function(e){if(!e)throw s.BrowserAuthError.createNonBrowserEnvironmentError()},e.detectIEOrEdge=function(){var e=window.navigator.userAgent,t=e.indexOf("MSIE "),r=e.indexOf("Trident/"),n=e.indexOf("Edge/");return t>0||r>0||n>0},e}()},"./node_modules/@azure/msal-browser/dist/utils/MathUtils.js":function(e,t,r){"use strict";r.r(t),r.d(t,"MathUtils",function(){return n});
/*! @azure/msal-browser v2.19.0 2021-11-02 */
var n=function(){function e(){}return e.decimalToHex=function(e){for(var t=e.toString(16);t.length<2;)t="0"+t;return t},e}()},"./node_modules/@azure/msal-browser/dist/utils/PopupUtils.js":function(e,t,r){"use strict";r.r(t),r.d(t,"PopupUtils",function(){return s});var n=r("./node_modules/@azure/msal-common/dist/index.js"),o=r("./node_modules/@azure/msal-browser/dist/error/BrowserAuthError.js"),i=r("./node_modules/@azure/msal-browser/dist/utils/BrowserConstants.js"),s=function(){function e(e,t){this.browserStorage=e,this.logger=t,this.unloadWindow=this.unloadWindow.bind(this)}return e.prototype.openPopup=function(t,r){try{var n=void 0;if(r.popup?(n=r.popup,this.logger.verbosePii("Navigating popup window to: "+t),n.location.assign(t)):void 0===r.popup&&(this.logger.verbosePii("Opening popup window to: "+t),n=e.openSizedPopup(t,r.popupName,r.popupWindowAttributes,this.logger)),!n)throw o.BrowserAuthError.createEmptyWindowCreatedError();return n.focus&&n.focus(),this.currentWindow=n,window.addEventListener("beforeunload",this.unloadWindow),n}catch(e){throw this.logger.error("error opening popup "+e.message),this.browserStorage.setInteractionInProgress(!1),o.BrowserAuthError.createPopupWindowError(e.toString())}},e.openSizedPopup=function(e,t,r,n){var o,s,a,c,u=window.screenLeft?window.screenLeft:window.screenX,d=window.screenTop?window.screenTop:window.screenY,l=window.innerWidth||document.documentElement.clientWidth||document.body.clientWidth,h=window.innerHeight||document.documentElement.clientHeight||document.body.clientHeight,p=null===(o=r.popupSize)||void 0===o?void 0:o.width,m=null===(s=r.popupSize)||void 0===s?void 0:s.height,g=null===(a=r.popupPosition)||void 0===a?void 0:a.top,f=null===(c=r.popupPosition)||void 0===c?void 0:c.left;return(!p||p<0||p>l)&&(n.verbose("Default popup window width used. Window width not configured or invalid."),p=i.BrowserConstants.POPUP_WIDTH),(!m||m<0||m>h)&&(n.verbose("Default popup window height used. Window height not configured or invalid."),m=i.BrowserConstants.POPUP_HEIGHT),(!g||g<0||g>h)&&(n.verbose("Default popup window top position used. Window top not configured or invalid."),g=Math.max(0,h/2-i.BrowserConstants.POPUP_HEIGHT/2+d)),(!f||f<0||f>l)&&(n.verbose("Default popup window left position used. Window left not configured or invalid."),f=Math.max(0,l/2-i.BrowserConstants.POPUP_WIDTH/2+u)),window.open(e,t,"width="+p+", height="+m+", top="+g+", left="+f+", scrollbars=yes")},e.prototype.unloadWindow=function(e){this.browserStorage.cleanRequestByInteractionType(i.InteractionType.Popup),this.currentWindow&&this.currentWindow.close(),e.preventDefault()},e.prototype.cleanPopup=function(e){e&&e.close(),window.removeEventListener("beforeunload",this.unloadWindow),this.browserStorage.setInteractionInProgress(!1)},e.prototype.monitorPopupForSameOrigin=function(e){var t=this;return new Promise(function(r,s){var a=setInterval(function(){if(e.closed)return t.cleanPopup(),clearInterval(a),void s(o.BrowserAuthError.createUserCancelledError());var i=n.Constants.EMPTY_STRING;try{i=e.location.href}catch(e){}n.StringUtils.isEmpty(i)||"about:blank"===i||(clearInterval(a),r())},i.BrowserConstants.POLL_INTERVAL_MS)})},e.generatePopupName=function(e,t){return i.BrowserConstants.POPUP_NAME_PREFIX+"."+e+"."+t.scopes.join("-")+"."+t.authority+"."+t.correlationId},e.generateLogoutPopupName=function(e,t){var r=t.account&&t.account.homeAccountId;return i.BrowserConstants.POPUP_NAME_PREFIX+"."+e+"."+r+"."+t.correlationId},e}()},"./node_modules/@azure/msal-common/dist/_virtual/_tslib.js":function(e,t,r){"use strict";r.r(t),r.d(t,"__assign",function(){return i}),r.d(t,"__awaiter",function(){return s}),r.d(t,"__extends",function(){return o}),r.d(t,"__generator",function(){return a}),r.d(t,"__spreadArrays",function(){return c});
/*! @azure/msal-common v5.1.0 2021-11-02 */
/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
var n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&(e[r]=t[r])})(e,t)};function o(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}var i=function(){return(i=Object.assign||function(e){for(var t,r=1,n=arguments.length;r<n;r++)for(var o in t=arguments[r])Object.prototype.hasOwnProperty.call(t,o)&&(e[o]=t[o]);return e}).apply(this,arguments)};function s(e,t,r,n){return new(r||(r=Promise))(function(o,i){function s(e){try{c(n.next(e))}catch(e){i(e)}}function a(e){try{c(n.throw(e))}catch(e){i(e)}}function c(e){var t;e.done?o(e.value):(t=e.value,t instanceof r?t:new r(function(e){e(t)})).then(s,a)}c((n=n.apply(e,t||[])).next())})}function a(e,t){var r,n,o,i,s={label:0,sent:function(){if(1&o[0])throw o[1];return o[1]},trys:[],ops:[]};return i={next:a(0),throw:a(1),return:a(2)},"function"==typeof Symbol&&(i[Symbol.iterator]=function(){return this}),i;function a(i){return function(a){return function(i){if(r)throw new TypeError("Generator is already executing.");for(;s;)try{if(r=1,n&&(o=2&i[0]?n.return:i[0]?n.throw||((o=n.return)&&o.call(n),0):n.next)&&!(o=o.call(n,i[1])).done)return o;switch(n=0,o&&(i=[2&i[0],o.value]),i[0]){case 0:case 1:o=i;break;case 4:return s.label++,{value:i[1],done:!1};case 5:s.label++,n=i[1],i=[0];continue;case 7:i=s.ops.pop(),s.trys.pop();continue;default:if(!(o=(o=s.trys).length>0&&o[o.length-1])&&(6===i[0]||2===i[0])){s=0;continue}if(3===i[0]&&(!o||i[1]>o[0]&&i[1]<o[3])){s.label=i[1];break}if(6===i[0]&&s.label<o[1]){s.label=o[1],o=i;break}if(o&&s.label<o[2]){s.label=o[2],s.ops.push(i);break}o[2]&&s.ops.pop(),s.trys.pop();continue}i=t.call(e,s)}catch(e){i=[6,e],n=0}finally{r=o=0}if(5&i[0])throw i[1];return{value:i[0]?i[1]:void 0,done:!0}}([i,a])}}}function c(){for(var e=0,t=0,r=arguments.length;t<r;t++)e+=arguments[t].length;var n=Array(e),o=0;for(t=0;t<r;t++)for(var i=arguments[t],s=0,a=i.length;s<a;s++,o++)n[o]=i[s];return n}},"./node_modules/@azure/msal-common/dist/account/AuthToken.js":function(e,t,r){"use strict";r.r(t),r.d(t,"AuthToken",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),o=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),i=function(){function e(t,r){if(o.StringUtils.isEmpty(t))throw n.ClientAuthError.createTokenNullOrEmptyError(t);this.rawToken=t,this.claims=e.extractTokenClaims(t,r)}return e.extractTokenClaims=function(e,t){var r=o.StringUtils.decodeAuthToken(e);try{var i=r.JWSPayload,s=t.base64Decode(i);return JSON.parse(s)}catch(e){throw n.ClientAuthError.createTokenParsingError(e)}},e}()},"./node_modules/@azure/msal-common/dist/account/CcsCredential.js":function(e,t,r){"use strict";
/*! @azure/msal-common v5.1.0 2021-11-02 */
var n;r.r(t),r.d(t,"CcsCredentialType",function(){return n}),function(e){e.HOME_ACCOUNT_ID="home_account_id",e.UPN="UPN"}(n||(n={}))},"./node_modules/@azure/msal-common/dist/account/ClientInfo.js":function(e,t,r){"use strict";r.r(t),r.d(t,"buildClientInfo",function(){return s}),r.d(t,"buildClientInfoFromHomeAccountId",function(){return a});var n=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),o=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),i=r("./node_modules/@azure/msal-common/dist/utils/Constants.js");
/*! @azure/msal-common v5.1.0 2021-11-02 */
function s(e,t){if(o.StringUtils.isEmpty(e))throw n.ClientAuthError.createClientInfoEmptyError();try{var r=t.base64Decode(e);return JSON.parse(r)}catch(e){throw n.ClientAuthError.createClientInfoDecodingError(e)}}function a(e){if(o.StringUtils.isEmpty(e))throw n.ClientAuthError.createClientInfoDecodingError("Home account ID was empty.");var t=e.split(i.Separators.CLIENT_INFO_SEPARATOR,2);return{uid:t[0],utid:t.length<2?i.Constants.EMPTY_STRING:t[1]}}},"./node_modules/@azure/msal-common/dist/authority/Authority.js":function(e,t,r){"use strict";r.r(t),r.d(t,"Authority",function(){return m});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/authority/AuthorityType.js"),i=r("./node_modules/@azure/msal-common/dist/authority/OpenIdConfigResponse.js"),s=r("./node_modules/@azure/msal-common/dist/url/UrlString.js"),a=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),c=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),u=r("./node_modules/@azure/msal-common/dist/error/ClientConfigurationError.js"),d=r("./node_modules/@azure/msal-common/dist/authority/ProtocolMode.js"),l=r("./node_modules/@azure/msal-common/dist/cache/entities/AuthorityMetadataEntity.js"),h=r("./node_modules/@azure/msal-common/dist/authority/CloudInstanceDiscoveryResponse.js"),p=r("./node_modules/@azure/msal-common/dist/authority/RegionDiscovery.js"),m=function(){function e(e,t,r,n){this.canonicalAuthority=e,this._canonicalAuthority.validateAsUri(),this.networkInterface=t,this.cacheManager=r,this.authorityOptions=n,this.regionDiscovery=new p.RegionDiscovery(t),this.regionDiscoveryMetadata={region_used:void 0,region_source:void 0,region_outcome:void 0}}return Object.defineProperty(e.prototype,"authorityType",{get:function(){var e=this.canonicalAuthorityUrlComponents.PathSegments;return e.length&&e[0].toLowerCase()===c.Constants.ADFS?o.AuthorityType.Adfs:o.AuthorityType.Default},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"protocolMode",{get:function(){return this.authorityOptions.protocolMode},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"options",{get:function(){return this.authorityOptions},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"canonicalAuthority",{get:function(){return this._canonicalAuthority.urlString},set:function(e){this._canonicalAuthority=new s.UrlString(e),this._canonicalAuthority.validateAsUri(),this._canonicalAuthorityUrlComponents=null},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"canonicalAuthorityUrlComponents",{get:function(){return this._canonicalAuthorityUrlComponents||(this._canonicalAuthorityUrlComponents=this._canonicalAuthority.getUrlComponents()),this._canonicalAuthorityUrlComponents},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"hostnameAndPort",{get:function(){return this.canonicalAuthorityUrlComponents.HostNameAndPort.toLowerCase()},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"tenant",{get:function(){return this.canonicalAuthorityUrlComponents.PathSegments[0]},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"authorizationEndpoint",{get:function(){if(this.discoveryComplete()){var e=this.replacePath(this.metadata.authorization_endpoint);return this.replaceTenant(e)}throw a.ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"tokenEndpoint",{get:function(){if(this.discoveryComplete()){var e=this.replacePath(this.metadata.token_endpoint);return this.replaceTenant(e)}throw a.ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"deviceCodeEndpoint",{get:function(){if(this.discoveryComplete()){var e=this.replacePath(this.metadata.token_endpoint.replace("/token","/devicecode"));return this.replaceTenant(e)}throw a.ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"endSessionEndpoint",{get:function(){if(this.discoveryComplete()){if(!this.metadata.end_session_endpoint)throw a.ClientAuthError.createLogoutNotSupportedError();var e=this.replacePath(this.metadata.end_session_endpoint);return this.replaceTenant(e)}throw a.ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"selfSignedJwtAudience",{get:function(){if(this.discoveryComplete()){var e=this.replacePath(this.metadata.issuer);return this.replaceTenant(e)}throw a.ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},enumerable:!1,configurable:!0}),e.prototype.replaceTenant=function(e){return e.replace(/{tenant}|{tenantid}/g,this.tenant)},e.prototype.replacePath=function(e){var t=e,r=new s.UrlString(this.metadata.canonical_authority).getUrlComponents().PathSegments;return this.canonicalAuthorityUrlComponents.PathSegments.forEach(function(e,n){var o=r[n];e!==o&&(t=t.replace("/"+o+"/","/"+e+"/"))}),t},Object.defineProperty(e.prototype,"defaultOpenIdConfigurationEndpoint",{get:function(){return this.authorityType===o.AuthorityType.Adfs||this.protocolMode===d.ProtocolMode.OIDC?this.canonicalAuthority+".well-known/openid-configuration":this.canonicalAuthority+"v2.0/.well-known/openid-configuration"},enumerable:!1,configurable:!0}),e.prototype.discoveryComplete=function(){return!!this.metadata},e.prototype.resolveEndpointsAsync=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){var e,t,r,o;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return(e=this.cacheManager.getAuthorityMetadataByAlias(this.hostnameAndPort))||(e=new l.AuthorityMetadataEntity).updateCanonicalAuthority(this.canonicalAuthority),[4,this.updateCloudDiscoveryMetadata(e)];case 1:return t=n.sent(),this.canonicalAuthority=this.canonicalAuthority.replace(this.hostnameAndPort,e.preferred_network),[4,this.updateEndpointMetadata(e)];case 2:return r=n.sent(),t!==c.AuthorityMetadataSource.CACHE&&r!==c.AuthorityMetadataSource.CACHE&&(e.resetExpiresAt(),e.updateCanonicalAuthority(this.canonicalAuthority)),o=this.cacheManager.generateAuthorityMetadataCacheKey(e.preferred_cache),this.cacheManager.setAuthorityMetadata(o,e),this.metadata=e,[2]}})})},e.prototype.updateEndpointMetadata=function(t){var r;return Object(n.__awaiter)(this,void 0,void 0,function(){var o,i,s;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return(o=this.getEndpointMetadataFromConfig())?(t.updateEndpointMetadata(o,!1),[2,c.AuthorityMetadataSource.CONFIG]):this.isAuthoritySameType(t)&&t.endpointsFromNetwork&&!t.isExpired()?[2,c.AuthorityMetadataSource.CACHE]:[4,this.getEndpointMetadataFromNetwork()];case 1:return(o=n.sent())?(null===(r=this.authorityOptions.azureRegionConfiguration)||void 0===r?void 0:r.azureRegion)?[4,this.regionDiscovery.detectRegion(this.authorityOptions.azureRegionConfiguration.environmentRegion,this.regionDiscoveryMetadata)]:[3,3]:[3,4];case 2:i=n.sent(),s=this.authorityOptions.azureRegionConfiguration.azureRegion===c.Constants.AZURE_REGION_AUTO_DISCOVER_FLAG?i:this.authorityOptions.azureRegionConfiguration.azureRegion,this.authorityOptions.azureRegionConfiguration.azureRegion===c.Constants.AZURE_REGION_AUTO_DISCOVER_FLAG?this.regionDiscoveryMetadata.region_outcome=i?c.RegionDiscoveryOutcomes.AUTO_DETECTION_REQUESTED_SUCCESSFUL:c.RegionDiscoveryOutcomes.AUTO_DETECTION_REQUESTED_FAILED:this.regionDiscoveryMetadata.region_outcome=i?this.authorityOptions.azureRegionConfiguration.azureRegion===i?c.RegionDiscoveryOutcomes.CONFIGURED_MATCHES_DETECTED:c.RegionDiscoveryOutcomes.CONFIGURED_NOT_DETECTED:c.RegionDiscoveryOutcomes.CONFIGURED_NO_AUTO_DETECTION,s&&(this.regionDiscoveryMetadata.region_used=s,o=e.replaceWithRegionalInformation(o,s)),n.label=3;case 3:return t.updateEndpointMetadata(o,!0),[2,c.AuthorityMetadataSource.NETWORK];case 4:throw a.ClientAuthError.createUnableToGetOpenidConfigError(this.defaultOpenIdConfigurationEndpoint)}})})},e.prototype.isAuthoritySameType=function(e){return new s.UrlString(e.canonical_authority).getUrlComponents().PathSegments.length===this.canonicalAuthorityUrlComponents.PathSegments.length},e.prototype.getEndpointMetadataFromConfig=function(){if(this.authorityOptions.authorityMetadata)try{return JSON.parse(this.authorityOptions.authorityMetadata)}catch(e){throw u.ClientConfigurationError.createInvalidAuthorityMetadataError()}return null},e.prototype.getEndpointMetadataFromNetwork=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){var e;return Object(n.__generator)(this,function(t){switch(t.label){case 0:return t.trys.push([0,2,,3]),[4,this.networkInterface.sendGetRequestAsync(this.defaultOpenIdConfigurationEndpoint)];case 1:return e=t.sent(),[2,Object(i.isOpenIdConfigResponse)(e.body)?e.body:null];case 2:return t.sent(),[2,null];case 3:return[2]}})})},e.prototype.updateCloudDiscoveryMetadata=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t;return Object(n.__generator)(this,function(r){switch(r.label){case 0:return(t=this.getCloudDiscoveryMetadataFromConfig())?(e.updateCloudDiscoveryMetadata(t,!1),[2,c.AuthorityMetadataSource.CONFIG]):this.isAuthoritySameType(e)&&e.aliasesFromNetwork&&!e.isExpired()?[2,c.AuthorityMetadataSource.CACHE]:[4,this.getCloudDiscoveryMetadataFromNetwork()];case 1:if(t=r.sent())return e.updateCloudDiscoveryMetadata(t,!0),[2,c.AuthorityMetadataSource.NETWORK];throw u.ClientConfigurationError.createUntrustedAuthorityError()}})})},e.prototype.getCloudDiscoveryMetadataFromConfig=function(){if(this.authorityOptions.cloudDiscoveryMetadata)try{var t=JSON.parse(this.authorityOptions.cloudDiscoveryMetadata),r=e.getCloudDiscoveryMetadataFromNetworkResponse(t.metadata,this.hostnameAndPort);if(r)return r}catch(e){throw u.ClientConfigurationError.createInvalidCloudDiscoveryMetadataError()}return this.isInKnownAuthorities()?e.createCloudDiscoveryMetadataFromHost(this.hostnameAndPort):null},e.prototype.getCloudDiscoveryMetadataFromNetwork=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,o,i;return Object(n.__generator)(this,function(n){switch(n.label){case 0:t=""+c.Constants.AAD_INSTANCE_DISCOVERY_ENDPT+this.canonicalAuthority+"oauth2/v2.0/authorize",r=null,n.label=1;case 1:return n.trys.push([1,3,,4]),[4,this.networkInterface.sendGetRequestAsync(t)];case 2:return o=n.sent(),0===(i=Object(h.isCloudInstanceDiscoveryResponse)(o.body)?o.body.metadata:[]).length?[2,null]:(r=e.getCloudDiscoveryMetadataFromNetworkResponse(i,this.hostnameAndPort),[3,4]);case 3:return n.sent(),[2,null];case 4:return r||(r=e.createCloudDiscoveryMetadataFromHost(this.hostnameAndPort)),[2,r]}})})},e.prototype.isInKnownAuthorities=function(){var e=this;return this.authorityOptions.knownAuthorities.filter(function(t){return s.UrlString.getDomainFromUrl(t).toLowerCase()===e.hostnameAndPort}).length>0},e.createCloudDiscoveryMetadataFromHost=function(e){return{preferred_network:e,preferred_cache:e,aliases:[e]}},e.getCloudDiscoveryMetadataFromNetworkResponse=function(e,t){for(var r=0;r<e.length;r++){var n=e[r];if(n.aliases.indexOf(t)>-1)return n}return null},e.prototype.getPreferredCache=function(){if(this.discoveryComplete())return this.metadata.preferred_cache;throw a.ClientAuthError.createEndpointDiscoveryIncompleteError("Discovery incomplete.")},e.prototype.isAlias=function(e){return this.metadata.aliases.indexOf(e)>-1},e.isPublicCloudAuthority=function(e){return c.Constants.KNOWN_PUBLIC_CLOUDS.indexOf(e)>=0},e.buildRegionalAuthorityString=function(e,t,r){var o=new s.UrlString(e);o.validateAsUri();var i=o.getUrlComponents(),a=t+"."+i.HostNameAndPort;this.isPublicCloudAuthority(i.HostNameAndPort)&&(a=t+"."+c.Constants.REGIONAL_AUTH_PUBLIC_CLOUD_SUFFIX);var u=s.UrlString.constructAuthorityUriFromObject(Object(n.__assign)(Object(n.__assign)({},o.getUrlComponents()),{HostNameAndPort:a})).urlString;return r?u+"?"+r:u},e.replaceWithRegionalInformation=function(t,r){return t.authorization_endpoint=e.buildRegionalAuthorityString(t.authorization_endpoint,r),t.token_endpoint=e.buildRegionalAuthorityString(t.token_endpoint,r,"allowestsrnonmsi=true"),t.end_session_endpoint&&(t.end_session_endpoint=e.buildRegionalAuthorityString(t.end_session_endpoint,r)),t},e}()},"./node_modules/@azure/msal-common/dist/authority/AuthorityFactory.js":function(e,t,r){"use strict";r.r(t),r.d(t,"AuthorityFactory",function(){return c});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/authority/Authority.js"),i=r("./node_modules/@azure/msal-common/dist/error/ClientConfigurationError.js"),s=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),a=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),c=function(){function e(){}return e.createDiscoveredInstance=function(t,r,o,i){return Object(n.__awaiter)(this,void 0,void 0,function(){var s,c;return Object(n.__generator)(this,function(n){switch(n.label){case 0:s=e.createInstance(t,r,o,i),n.label=1;case 1:return n.trys.push([1,3,,4]),[4,s.resolveEndpointsAsync()];case 2:return n.sent(),[2,s];case 3:throw c=n.sent(),a.ClientAuthError.createEndpointDiscoveryIncompleteError(c);case 4:return[2]}})})},e.createInstance=function(e,t,r,n){if(s.StringUtils.isEmpty(e))throw i.ClientConfigurationError.createUrlEmptyError();return new o.Authority(e,t,r,n)},e}()},"./node_modules/@azure/msal-common/dist/authority/AuthorityType.js":function(e,t,r){"use strict";
/*! @azure/msal-common v5.1.0 2021-11-02 */
var n;r.r(t),r.d(t,"AuthorityType",function(){return n}),function(e){e[e.Default=0]="Default",e[e.Adfs=1]="Adfs"}(n||(n={}))},"./node_modules/@azure/msal-common/dist/authority/CloudInstanceDiscoveryResponse.js":function(e,t,r){"use strict";
/*! @azure/msal-common v5.1.0 2021-11-02 */
function n(e){return e.hasOwnProperty("tenant_discovery_endpoint")&&e.hasOwnProperty("metadata")}r.r(t),r.d(t,"isCloudInstanceDiscoveryResponse",function(){return n})},"./node_modules/@azure/msal-common/dist/authority/OpenIdConfigResponse.js":function(e,t,r){"use strict";
/*! @azure/msal-common v5.1.0 2021-11-02 */
function n(e){return e.hasOwnProperty("authorization_endpoint")&&e.hasOwnProperty("token_endpoint")&&e.hasOwnProperty("issuer")}r.r(t),r.d(t,"isOpenIdConfigResponse",function(){return n})},"./node_modules/@azure/msal-common/dist/authority/ProtocolMode.js":function(e,t,r){"use strict";
/*! @azure/msal-common v5.1.0 2021-11-02 */
var n;r.r(t),r.d(t,"ProtocolMode",function(){return n}),function(e){e.AAD="AAD",e.OIDC="OIDC"}(n||(n={}))},"./node_modules/@azure/msal-common/dist/authority/RegionDiscovery.js":function(e,t,r){"use strict";r.r(t),r.d(t,"RegionDiscovery",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),i=function(){function e(e){this.networkInterface=e}return e.prototype.detectRegion=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,i,s,a;return Object(n.__generator)(this,function(n){switch(n.label){case 0:if(r=e)return[3,8];n.label=1;case 1:return n.trys.push([1,6,,7]),[4,this.getRegionFromIMDS(o.Constants.IMDS_VERSION)];case 2:return(i=n.sent()).status===o.ResponseCodes.httpSuccess&&(r=i.body,t.region_source=o.RegionDiscoverySources.IMDS),i.status!==o.ResponseCodes.httpBadRequest?[3,5]:[4,this.getCurrentVersion()];case 3:return(s=n.sent())?[4,this.getRegionFromIMDS(s)]:(t.region_source=o.RegionDiscoverySources.FAILED_AUTO_DETECTION,[2,null]);case 4:(a=n.sent()).status===o.ResponseCodes.httpSuccess&&(r=a.body,t.region_source=o.RegionDiscoverySources.IMDS),n.label=5;case 5:return[3,7];case 6:return n.sent(),t.region_source=o.RegionDiscoverySources.FAILED_AUTO_DETECTION,[2,null];case 7:return[3,9];case 8:t.region_source=o.RegionDiscoverySources.ENVIRONMENT_VARIABLE,n.label=9;case 9:return r||(t.region_source=o.RegionDiscoverySources.FAILED_AUTO_DETECTION),[2,r||null]}})})},e.prototype.getRegionFromIMDS=function(t){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(r){return[2,this.networkInterface.sendGetRequestAsync(o.Constants.IMDS_ENDPOINT+"?api-version="+t+"&format=text",e.IMDS_OPTIONS,o.Constants.IMDS_TIMEOUT)]})})},e.prototype.getCurrentVersion=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){var t;return Object(n.__generator)(this,function(r){switch(r.label){case 0:return r.trys.push([0,2,,3]),[4,this.networkInterface.sendGetRequestAsync(o.Constants.IMDS_ENDPOINT+"?format=json",e.IMDS_OPTIONS)];case 1:return(t=r.sent()).status===o.ResponseCodes.httpBadRequest&&t.body&&t.body["newest-versions"]&&t.body["newest-versions"].length>0?[2,t.body["newest-versions"][0]]:[2,null];case 2:return r.sent(),[2,null];case 3:return[2]}})})},e.IMDS_OPTIONS={headers:{Metadata:"true"}},e}()},"./node_modules/@azure/msal-common/dist/cache/CacheManager.js":function(e,t,r){"use strict";r.r(t),r.d(t,"CacheManager",function(){return l}),r.d(t,"DefaultStorageClass",function(){return h});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),i=r("./node_modules/@azure/msal-common/dist/cache/entities/CredentialEntity.js"),s=r("./node_modules/@azure/msal-common/dist/request/ScopeSet.js"),a=r("./node_modules/@azure/msal-common/dist/cache/entities/AccountEntity.js"),c=r("./node_modules/@azure/msal-common/dist/error/AuthError.js"),u=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),d=r("./node_modules/@azure/msal-common/dist/account/AuthToken.js"),l=function(){function e(e,t){this.clientId=e,this.cryptoImpl=t}return e.prototype.getAllAccounts=function(){var t=this,r=this.getAccountsFilteredBy(),n=Object.keys(r).map(function(e){return r[e]});return n.length<1?[]:n.map(function(r){var n=e.toObject(new a.AccountEntity,r).getAccountInfo(),o=t.readIdTokenFromCache(t.clientId,n);return o&&!n.idTokenClaims&&(n.idTokenClaims=new d.AuthToken(o.secret,t.cryptoImpl).claims),n})},e.prototype.saveCacheRecord=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(t){switch(t.label){case 0:if(!e)throw u.ClientAuthError.createNullOrUndefinedCacheRecord();return e.account&&this.setAccount(e.account),e.idToken&&this.setIdTokenCredential(e.idToken),e.accessToken?[4,this.saveAccessToken(e.accessToken)]:[3,2];case 1:t.sent(),t.label=2;case 2:return e.refreshToken&&this.setRefreshTokenCredential(e.refreshToken),e.appMetadata&&this.setAppMetadata(e.appMetadata),[2]}})})},e.prototype.saveAccessToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,o,i,a=this;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return t=this.getCredentialsFilteredBy({clientId:e.clientId,credentialType:e.credentialType,environment:e.environment,homeAccountId:e.homeAccountId,realm:e.realm,tokenType:e.tokenType}),r=s.ScopeSet.fromString(e.target),(o=Object.keys(t.accessTokens).map(function(e){return t.accessTokens[e]}))?(i=[],o.forEach(function(e){s.ScopeSet.fromString(e.target).intersectingScopeSets(r)&&i.push(a.removeCredential(e))}),[4,Promise.all(i)]):[3,2];case 1:n.sent(),n.label=2;case 2:return this.setAccessTokenCredential(e),[2]}})})},e.prototype.getAccountsFilteredBy=function(e){return this.getAccountsFilteredByInternal(e?e.homeAccountId:"",e?e.environment:"",e?e.realm:"")},e.prototype.getAccountsFilteredByInternal=function(e,t,r){var n=this,o=this.getKeys(),i={};return o.forEach(function(o){var s=n.getAccount(o);s&&(e&&!n.matchHomeAccountId(s,e)||t&&!n.matchEnvironment(s,t)||r&&!n.matchRealm(s,r)||(i[o]=s))}),i},e.prototype.getCredentialsFilteredBy=function(e){return this.getCredentialsFilteredByInternal(e.homeAccountId,e.environment,e.credentialType,e.clientId,e.familyId,e.realm,e.target,e.oboAssertion,e.tokenType,e.keyId)},e.prototype.getCredentialsFilteredByInternal=function(e,t,r,n,s,a,c,u,d,l){var h=this,p=this.getKeys(),m={idTokens:{},accessTokens:{},refreshTokens:{}};return p.forEach(function(p){var g=i.CredentialEntity.getCredentialType(p);if(g!==o.Constants.NOT_DEFINED){var f=h.getSpecificCredential(p,g);if(f&&(!u||h.matchOboAssertion(f,u))&&(!e||h.matchHomeAccountId(f,e))&&(!t||h.matchEnvironment(f,t))&&(!a||h.matchRealm(f,a))&&(!r||h.matchCredentialType(f,r))&&(!n||h.matchClientId(f,n))&&(!s||h.matchFamilyId(f,s))&&(!c||h.matchTarget(f,c))){if(r===o.CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME){if(d&&!h.matchTokenType(f,d))return;switch(d){case o.AuthenticationScheme.POP:if(-1===p.indexOf(o.AuthenticationScheme.POP))return void h.removeItem(p,o.CacheSchemaType.CREDENTIAL);break;case o.AuthenticationScheme.SSH:if(l&&!h.matchKeyId(f,l))return}}switch(g){case o.CredentialType.ID_TOKEN:m.idTokens[p]=f;break;case o.CredentialType.ACCESS_TOKEN:case o.CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME:m.accessTokens[p]=f;break;case o.CredentialType.REFRESH_TOKEN:m.refreshTokens[p]=f}}}}),m},e.prototype.getAppMetadataFilteredBy=function(e){return this.getAppMetadataFilteredByInternal(e.environment,e.clientId)},e.prototype.getAppMetadataFilteredByInternal=function(e,t){var r=this,n=this.getKeys(),o={};return n.forEach(function(n){if(r.isAppMetadata(n)){var i=r.getAppMetadata(n);i&&(e&&!r.matchEnvironment(i,e)||t&&!r.matchClientId(i,t)||(o[n]=i))}}),o},e.prototype.getAuthorityMetadataByAlias=function(e){var t=this,r=this.getAuthorityMetadataKeys(),n=null;return r.forEach(function(r){if(t.isAuthorityMetadata(r)&&-1!==r.indexOf(t.clientId)){var o=t.getAuthorityMetadata(r);o&&-1!==o.aliases.indexOf(e)&&(n=o)}}),n},e.prototype.removeAllAccounts=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){var e,t,r=this;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return e=this.getKeys(),t=[],e.forEach(function(e){r.getAccount(e)&&t.push(r.removeAccount(e))}),[4,Promise.all(t)];case 1:return n.sent(),[2,!0]}})})},e.prototype.removeAccount=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t;return Object(n.__generator)(this,function(r){switch(r.label){case 0:if(!(t=this.getAccount(e)))throw u.ClientAuthError.createNoAccountFoundError();return[4,this.removeAccountContext(t)];case 1:return[2,r.sent()&&this.removeItem(e,o.CacheSchemaType.ACCOUNT)]}})})},e.prototype.removeAccountContext=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,s,a=this;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return t=this.getKeys(),r=e.generateAccountId(),s=[],t.forEach(function(e){var t=i.CredentialEntity.getCredentialType(e);if(t!==o.Constants.NOT_DEFINED){var n=a.getSpecificCredential(e,t);n&&r===n.generateAccountId()&&s.push(a.removeCredential(n))}}),[4,Promise.all(s)];case 1:return n.sent(),[2,!0]}})})},e.prototype.removeCredential=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:if(t=e.generateCredentialKey(),e.credentialType.toLowerCase()!==o.CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME.toLowerCase())return[3,4];if(e.tokenType!==o.AuthenticationScheme.POP)return[3,4];if(!(r=e.keyId))return[3,4];n.label=1;case 1:return n.trys.push([1,3,,4]),[4,this.cryptoImpl.removeTokenBindingKey(r)];case 2:return n.sent(),[3,4];case 3:throw n.sent(),u.ClientAuthError.createBindingKeyNotRemovedError();case 4:return[2,this.removeItem(t,o.CacheSchemaType.CREDENTIAL)]}})})},e.prototype.removeAppMetadata=function(){var e=this;return this.getKeys().forEach(function(t){e.isAppMetadata(t)&&e.removeItem(t,o.CacheSchemaType.APP_METADATA)}),!0},e.prototype.readCacheRecord=function(e,t,r,n,o,i){var s=this.readAccountFromCache(e),a=this.readIdTokenFromCache(t,e),c=this.readAccessTokenFromCache(t,e,r,o,i),u=this.readRefreshTokenFromCache(t,e,!1),l=this.readAppMetadataFromCache(n,t);return s&&a&&(s.idTokenClaims=new d.AuthToken(a.secret,this.cryptoImpl).claims),{account:s,idToken:a,accessToken:c,refreshToken:u,appMetadata:l}},e.prototype.readAccountFromCache=function(e){var t=a.AccountEntity.generateAccountCacheKey(e);return this.getAccount(t)},e.prototype.readIdTokenFromCache=function(e,t){var r={homeAccountId:t.homeAccountId,environment:t.environment,credentialType:o.CredentialType.ID_TOKEN,clientId:e,realm:t.tenantId},n=this.getCredentialsFilteredBy(r),i=Object.keys(n.idTokens).map(function(e){return n.idTokens[e]}),s=i.length;if(s<1)return null;if(s>1)throw u.ClientAuthError.createMultipleMatchingTokensInCacheError();return i[0]},e.prototype.readAccessTokenFromCache=function(e,t,r,n,i){var s=n&&n!==o.AuthenticationScheme.BEARER?o.CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME:o.CredentialType.ACCESS_TOKEN,a={homeAccountId:t.homeAccountId,environment:t.environment,credentialType:s,clientId:e,realm:t.tenantId,target:r.printScopesLowerCase(),tokenType:n,keyId:i},c=this.getCredentialsFilteredBy(a),d=Object.keys(c.accessTokens).map(function(e){return c.accessTokens[e]}),l=d.length;if(l<1)return null;if(l>1)throw u.ClientAuthError.createMultipleMatchingTokensInCacheError();return d[0]},e.prototype.readRefreshTokenFromCache=function(e,t,r){var n=r?o.THE_FAMILY_ID:void 0,i={homeAccountId:t.homeAccountId,environment:t.environment,credentialType:o.CredentialType.REFRESH_TOKEN,clientId:e,familyId:n},s=this.getCredentialsFilteredBy(i),a=Object.keys(s.refreshTokens).map(function(e){return s.refreshTokens[e]});return a.length<1?null:a[0]},e.prototype.readAppMetadataFromCache=function(e,t){var r={environment:e,clientId:t},n=this.getAppMetadataFilteredBy(r),o=Object.keys(n).map(function(e){return n[e]}),i=o.length;if(i<1)return null;if(i>1)throw u.ClientAuthError.createMultipleMatchingAppMetadataInCacheError();return o[0]},e.prototype.isAppMetadataFOCI=function(e,t){var r=this.readAppMetadataFromCache(e,t);return!(!r||r.familyId!==o.THE_FAMILY_ID)},e.prototype.matchHomeAccountId=function(e,t){return!(!e.homeAccountId||t!==e.homeAccountId)},e.prototype.matchOboAssertion=function(e,t){return!(!e.oboAssertion||t!==e.oboAssertion)},e.prototype.matchEnvironment=function(e,t){var r=this.getAuthorityMetadataByAlias(t);return!!(r&&r.aliases.indexOf(e.environment)>-1)},e.prototype.matchCredentialType=function(e,t){return e.credentialType&&t.toLowerCase()===e.credentialType.toLowerCase()},e.prototype.matchClientId=function(e,t){return!(!e.clientId||t!==e.clientId)},e.prototype.matchFamilyId=function(e,t){return!(!e.familyId||t!==e.familyId)},e.prototype.matchRealm=function(e,t){return!(!e.realm||t!==e.realm)},e.prototype.matchTarget=function(e,t){if(e.credentialType!==o.CredentialType.ACCESS_TOKEN&&e.credentialType!==o.CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME||!e.target)return!1;var r=s.ScopeSet.fromString(e.target),n=s.ScopeSet.fromString(t);return n.containsOnlyOIDCScopes()?n.removeScope(o.Constants.OFFLINE_ACCESS_SCOPE):n.removeOIDCScopes(),r.containsScopeSet(n)},e.prototype.matchTokenType=function(e,t){return!(!e.tokenType||e.tokenType!==t)},e.prototype.matchKeyId=function(e,t){return!(!e.keyId||e.keyId!==t)},e.prototype.isAppMetadata=function(e){return-1!==e.indexOf(o.APP_METADATA)},e.prototype.isAuthorityMetadata=function(e){return-1!==e.indexOf(o.AUTHORITY_METADATA_CONSTANTS.CACHE_KEY)},e.prototype.generateAuthorityMetadataCacheKey=function(e){return o.AUTHORITY_METADATA_CONSTANTS.CACHE_KEY+"-"+this.clientId+"-"+e},e.prototype.getSpecificCredential=function(e,t){switch(t){case o.CredentialType.ID_TOKEN:return this.getIdTokenCredential(e);case o.CredentialType.ACCESS_TOKEN:case o.CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME:return this.getAccessTokenCredential(e);case o.CredentialType.REFRESH_TOKEN:return this.getRefreshTokenCredential(e);default:return null}},e.toObject=function(e,t){for(var r in t)e[r]=t[r];return e},e}(),h=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(n.__extends)(t,e),t.prototype.setAccount=function(){throw c.AuthError.createUnexpectedError("Storage interface - setAccount() has not been implemented for the cacheStorage interface.")},t.prototype.getAccount=function(){throw c.AuthError.createUnexpectedError("Storage interface - getAccount() has not been implemented for the cacheStorage interface.")},t.prototype.setIdTokenCredential=function(){throw c.AuthError.createUnexpectedError("Storage interface - setIdTokenCredential() has not been implemented for the cacheStorage interface.")},t.prototype.getIdTokenCredential=function(){throw c.AuthError.createUnexpectedError("Storage interface - getIdTokenCredential() has not been implemented for the cacheStorage interface.")},t.prototype.setAccessTokenCredential=function(){throw c.AuthError.createUnexpectedError("Storage interface - setAccessTokenCredential() has not been implemented for the cacheStorage interface.")},t.prototype.getAccessTokenCredential=function(){throw c.AuthError.createUnexpectedError("Storage interface - getAccessTokenCredential() has not been implemented for the cacheStorage interface.")},t.prototype.setRefreshTokenCredential=function(){throw c.AuthError.createUnexpectedError("Storage interface - setRefreshTokenCredential() has not been implemented for the cacheStorage interface.")},t.prototype.getRefreshTokenCredential=function(){throw c.AuthError.createUnexpectedError("Storage interface - getRefreshTokenCredential() has not been implemented for the cacheStorage interface.")},t.prototype.setAppMetadata=function(){throw c.AuthError.createUnexpectedError("Storage interface - setAppMetadata() has not been implemented for the cacheStorage interface.")},t.prototype.getAppMetadata=function(){throw c.AuthError.createUnexpectedError("Storage interface - getAppMetadata() has not been implemented for the cacheStorage interface.")},t.prototype.setServerTelemetry=function(){throw c.AuthError.createUnexpectedError("Storage interface - setServerTelemetry() has not been implemented for the cacheStorage interface.")},t.prototype.getServerTelemetry=function(){throw c.AuthError.createUnexpectedError("Storage interface - getServerTelemetry() has not been implemented for the cacheStorage interface.")},t.prototype.setAuthorityMetadata=function(){throw c.AuthError.createUnexpectedError("Storage interface - setAuthorityMetadata() has not been implemented for the cacheStorage interface.")},t.prototype.getAuthorityMetadata=function(){throw c.AuthError.createUnexpectedError("Storage interface - getAuthorityMetadata() has not been implemented for the cacheStorage interface.")},t.prototype.getAuthorityMetadataKeys=function(){throw c.AuthError.createUnexpectedError("Storage interface - getAuthorityMetadataKeys() has not been implemented for the cacheStorage interface.")},t.prototype.setThrottlingCache=function(){throw c.AuthError.createUnexpectedError("Storage interface - setThrottlingCache() has not been implemented for the cacheStorage interface.")},t.prototype.getThrottlingCache=function(){throw c.AuthError.createUnexpectedError("Storage interface - getThrottlingCache() has not been implemented for the cacheStorage interface.")},t.prototype.removeItem=function(){throw c.AuthError.createUnexpectedError("Storage interface - removeItem() has not been implemented for the cacheStorage interface.")},t.prototype.containsKey=function(){throw c.AuthError.createUnexpectedError("Storage interface - containsKey() has not been implemented for the cacheStorage interface.")},t.prototype.getKeys=function(){throw c.AuthError.createUnexpectedError("Storage interface - getKeys() has not been implemented for the cacheStorage interface.")},t.prototype.clear=function(){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(e){throw"Storage interface - clear() has not been implemented for the cacheStorage interface.",c.AuthError.createUnexpectedError("Storage interface - clear() has not been implemented for the cacheStorage interface.")})})},t}(l)},"./node_modules/@azure/msal-common/dist/cache/entities/AccessTokenEntity.js":function(e,t,r){"use strict";r.r(t),r.d(t,"AccessTokenEntity",function(){return d});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/cache/entities/CredentialEntity.js"),i=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),s=r("./node_modules/@azure/msal-common/dist/utils/TimeUtils.js"),a=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),c=r("./node_modules/@azure/msal-common/dist/account/AuthToken.js"),u=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),d=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(n.__extends)(t,e),t.createAccessTokenEntity=function(e,r,n,o,d,l,h,p,m,g,f,y,C){var E,_=new t;_.homeAccountId=e,_.credentialType=i.CredentialType.ACCESS_TOKEN,_.secret=n;var v=s.TimeUtils.nowSeconds();if(_.cachedAt=v.toString(),_.expiresOn=h.toString(),_.extendedExpiresOn=p.toString(),g&&(_.refreshOn=g.toString()),_.environment=r,_.clientId=o,_.realm=d,_.target=l,_.oboAssertion=y,_.tokenType=a.StringUtils.isEmpty(f)?i.AuthenticationScheme.BEARER:f,_.tokenType!==i.AuthenticationScheme.BEARER)switch(_.credentialType=i.CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME,_.tokenType){case i.AuthenticationScheme.POP:var T=c.AuthToken.extractTokenClaims(n,m);if(!(null===(E=null==T?void 0:T.cnf)||void 0===E?void 0:E.kid))throw u.ClientAuthError.createTokenClaimsRequiredError();_.keyId=T.cnf.kid;break;case i.AuthenticationScheme.SSH:_.keyId=C}return _},t.isAccessTokenEntity=function(e){return!!e&&(e.hasOwnProperty("homeAccountId")&&e.hasOwnProperty("environment")&&e.hasOwnProperty("credentialType")&&e.hasOwnProperty("realm")&&e.hasOwnProperty("clientId")&&e.hasOwnProperty("secret")&&e.hasOwnProperty("target")&&(e.credentialType===i.CredentialType.ACCESS_TOKEN||e.credentialType===i.CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME))},t}(o.CredentialEntity)},"./node_modules/@azure/msal-common/dist/cache/entities/AccountEntity.js":function(e,t,r){"use strict";r.r(t),r.d(t,"AccountEntity",function(){return c});var n=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),o=r("./node_modules/@azure/msal-common/dist/account/ClientInfo.js"),i=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),s=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),a=r("./node_modules/@azure/msal-common/dist/authority/AuthorityType.js"),c=function(){function e(){}return e.prototype.generateAccountId=function(){return[this.homeAccountId,this.environment].join(n.Separators.CACHE_KEY_SEPARATOR).toLowerCase()},e.prototype.generateAccountKey=function(){return e.generateAccountCacheKey({homeAccountId:this.homeAccountId,environment:this.environment,tenantId:this.realm,username:this.username,localAccountId:this.localAccountId})},e.prototype.generateType=function(){switch(this.authorityType){case n.CacheAccountType.ADFS_ACCOUNT_TYPE:return n.CacheType.ADFS;case n.CacheAccountType.MSAV1_ACCOUNT_TYPE:return n.CacheType.MSA;case n.CacheAccountType.MSSTS_ACCOUNT_TYPE:return n.CacheType.MSSTS;case n.CacheAccountType.GENERIC_ACCOUNT_TYPE:return n.CacheType.GENERIC;default:throw s.ClientAuthError.createUnexpectedAccountTypeError()}},e.prototype.getAccountInfo=function(){return{homeAccountId:this.homeAccountId,environment:this.environment,tenantId:this.realm,username:this.username,localAccountId:this.localAccountId,name:this.name,idTokenClaims:this.idTokenClaims}},e.generateAccountCacheKey=function(e){return[e.homeAccountId,e.environment||"",e.tenantId||""].join(n.Separators.CACHE_KEY_SEPARATOR).toLowerCase()},e.createAccount=function(t,r,o,i,a,c,u,d){var l,h,p,m,g,f,y=new e;y.authorityType=n.CacheAccountType.MSSTS_ACCOUNT_TYPE,y.clientInfo=t,y.homeAccountId=r;var C=d||i&&i.getPreferredCache();if(!C)throw s.ClientAuthError.createInvalidCacheEnvironmentError();return y.environment=C,y.realm=(null===(l=null==o?void 0:o.claims)||void 0===l?void 0:l.tid)||"",y.oboAssertion=a,o&&(y.idTokenClaims=o.claims,y.localAccountId=(null===(h=null==o?void 0:o.claims)||void 0===h?void 0:h.oid)||(null===(p=null==o?void 0:o.claims)||void 0===p?void 0:p.sub)||"",y.username=(null===(m=null==o?void 0:o.claims)||void 0===m?void 0:m.preferred_username)||((null===(g=null==o?void 0:o.claims)||void 0===g?void 0:g.emails)?o.claims.emails[0]:""),y.name=null===(f=null==o?void 0:o.claims)||void 0===f?void 0:f.name),y.cloudGraphHostName=c,y.msGraphHost=u,y},e.createGenericAccount=function(t,r,o,i,c,u,d){var l,h,p,m,g=new e;g.authorityType=o&&o.authorityType===a.AuthorityType.Adfs?n.CacheAccountType.ADFS_ACCOUNT_TYPE:n.CacheAccountType.GENERIC_ACCOUNT_TYPE,g.homeAccountId=t,g.realm="",g.oboAssertion=i;var f=d||o&&o.getPreferredCache();if(!f)throw s.ClientAuthError.createInvalidCacheEnvironmentError();return r&&(g.localAccountId=(null===(l=null==r?void 0:r.claims)||void 0===l?void 0:l.oid)||(null===(h=null==r?void 0:r.claims)||void 0===h?void 0:h.sub)||"",g.username=(null===(p=null==r?void 0:r.claims)||void 0===p?void 0:p.upn)||"",g.name=(null===(m=null==r?void 0:r.claims)||void 0===m?void 0:m.name)||"",g.idTokenClaims=null==r?void 0:r.claims),g.environment=f,g.cloudGraphHostName=c,g.msGraphHost=u,g},e.generateHomeAccountId=function(e,t,r,s,c){var u,d=(null===(u=null==c?void 0:c.claims)||void 0===u?void 0:u.sub)?c.claims.sub:n.Constants.EMPTY_STRING;if(t===a.AuthorityType.Adfs)return d;if(e)try{var l=Object(o.buildClientInfo)(e,s);if(!i.StringUtils.isEmpty(l.uid)&&!i.StringUtils.isEmpty(l.utid))return""+l.uid+n.Separators.CLIENT_INFO_SEPARATOR+l.utid}catch(e){}return r.verbose("No client info in response"),d},e.isAccountEntity=function(e){return!!e&&(e.hasOwnProperty("homeAccountId")&&e.hasOwnProperty("environment")&&e.hasOwnProperty("realm")&&e.hasOwnProperty("localAccountId")&&e.hasOwnProperty("username")&&e.hasOwnProperty("authorityType"))},e.accountInfoIsEqual=function(e,t,r){if(!e||!t)return!1;var n=!0;if(r){var o=e.idTokenClaims||{},i=t.idTokenClaims||{};n=o.iat===i.iat&&o.nonce===i.nonce}return e.homeAccountId===t.homeAccountId&&e.localAccountId===t.localAccountId&&e.username===t.username&&e.tenantId===t.tenantId&&e.environment===t.environment&&n},e}()},"./node_modules/@azure/msal-common/dist/cache/entities/AppMetadataEntity.js":function(e,t,r){"use strict";r.r(t),r.d(t,"AppMetadataEntity",function(){return o});var n=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),o=function(){function e(){}return e.prototype.generateAppMetadataKey=function(){return e.generateAppMetadataCacheKey(this.environment,this.clientId)},e.generateAppMetadataCacheKey=function(e,t){return[n.APP_METADATA,e,t].join(n.Separators.CACHE_KEY_SEPARATOR).toLowerCase()},e.createAppMetadataEntity=function(t,r,n){var o=new e;return o.clientId=t,o.environment=r,n&&(o.familyId=n),o},e.isAppMetadataEntity=function(e,t){return!!t&&(0===e.indexOf(n.APP_METADATA)&&t.hasOwnProperty("clientId")&&t.hasOwnProperty("environment"))},e}();
/*! @azure/msal-common v5.1.0 2021-11-02 */},"./node_modules/@azure/msal-common/dist/cache/entities/AuthorityMetadataEntity.js":function(e,t,r){"use strict";r.r(t),r.d(t,"AuthorityMetadataEntity",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),o=r("./node_modules/@azure/msal-common/dist/utils/TimeUtils.js"),i=function(){function e(){this.expiresAt=o.TimeUtils.nowSeconds()+n.AUTHORITY_METADATA_CONSTANTS.REFRESH_TIME_SECONDS}return e.prototype.updateCloudDiscoveryMetadata=function(e,t){this.aliases=e.aliases,this.preferred_cache=e.preferred_cache,this.preferred_network=e.preferred_network,this.aliasesFromNetwork=t},e.prototype.updateEndpointMetadata=function(e,t){this.authorization_endpoint=e.authorization_endpoint,this.token_endpoint=e.token_endpoint,this.end_session_endpoint=e.end_session_endpoint,this.issuer=e.issuer,this.endpointsFromNetwork=t},e.prototype.updateCanonicalAuthority=function(e){this.canonical_authority=e},e.prototype.resetExpiresAt=function(){this.expiresAt=o.TimeUtils.nowSeconds()+n.AUTHORITY_METADATA_CONSTANTS.REFRESH_TIME_SECONDS},e.prototype.isExpired=function(){return this.expiresAt<=o.TimeUtils.nowSeconds()},e.isAuthorityMetadataEntity=function(e,t){return!!t&&(0===e.indexOf(n.AUTHORITY_METADATA_CONSTANTS.CACHE_KEY)&&t.hasOwnProperty("aliases")&&t.hasOwnProperty("preferred_cache")&&t.hasOwnProperty("preferred_network")&&t.hasOwnProperty("canonical_authority")&&t.hasOwnProperty("authorization_endpoint")&&t.hasOwnProperty("token_endpoint")&&t.hasOwnProperty("issuer")&&t.hasOwnProperty("aliasesFromNetwork")&&t.hasOwnProperty("endpointsFromNetwork")&&t.hasOwnProperty("expiresAt"))},e}()},"./node_modules/@azure/msal-common/dist/cache/entities/CacheRecord.js":function(e,t,r){"use strict";r.r(t),r.d(t,"CacheRecord",function(){return n});
/*! @azure/msal-common v5.1.0 2021-11-02 */
var n=function(){return function(e,t,r,n,o){this.account=e||null,this.idToken=t||null,this.accessToken=r||null,this.refreshToken=n||null,this.appMetadata=o||null}}()},"./node_modules/@azure/msal-common/dist/cache/entities/CredentialEntity.js":function(e,t,r){"use strict";r.r(t),r.d(t,"CredentialEntity",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),o=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),i=function(){function e(){}return e.prototype.generateAccountId=function(){return e.generateAccountIdForCacheKey(this.homeAccountId,this.environment)},e.prototype.generateCredentialId=function(){return e.generateCredentialIdForCacheKey(this.credentialType,this.clientId,this.realm,this.familyId)},e.prototype.generateTarget=function(){return e.generateTargetForCacheKey(this.target)},e.prototype.generateCredentialKey=function(){return e.generateCredentialCacheKey(this.homeAccountId,this.environment,this.credentialType,this.clientId,this.realm,this.target,this.familyId,this.tokenType)},e.prototype.generateType=function(){switch(this.credentialType){case n.CredentialType.ID_TOKEN:return n.CacheType.ID_TOKEN;case n.CredentialType.ACCESS_TOKEN:case n.CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME:return n.CacheType.ACCESS_TOKEN;case n.CredentialType.REFRESH_TOKEN:return n.CacheType.REFRESH_TOKEN;default:throw o.ClientAuthError.createUnexpectedCredentialTypeError()}},e.getCredentialType=function(e){return-1!==e.indexOf(n.CredentialType.ACCESS_TOKEN.toLowerCase())?-1!==e.indexOf(n.CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME.toLowerCase())?n.CredentialType.ACCESS_TOKEN_WITH_AUTH_SCHEME:n.CredentialType.ACCESS_TOKEN:-1!==e.indexOf(n.CredentialType.ID_TOKEN.toLowerCase())?n.CredentialType.ID_TOKEN:-1!==e.indexOf(n.CredentialType.REFRESH_TOKEN.toLowerCase())?n.CredentialType.REFRESH_TOKEN:n.Constants.NOT_DEFINED},e.generateCredentialCacheKey=function(e,t,r,o,i,s,a,c){var u=[this.generateAccountIdForCacheKey(e,t),this.generateCredentialIdForCacheKey(r,o,i,a),this.generateTargetForCacheKey(s)];return c&&c!==n.AuthenticationScheme.BEARER&&u.push(c.toLowerCase()),u.join(n.Separators.CACHE_KEY_SEPARATOR).toLowerCase()},e.generateAccountIdForCacheKey=function(e,t){return[e,t].join(n.Separators.CACHE_KEY_SEPARATOR).toLowerCase()},e.generateCredentialIdForCacheKey=function(e,t,r,o){return[e,e===n.CredentialType.REFRESH_TOKEN&&o||t,r||""].join(n.Separators.CACHE_KEY_SEPARATOR).toLowerCase()},e.generateTargetForCacheKey=function(e){return(e||"").toLowerCase()},e}()},"./node_modules/@azure/msal-common/dist/cache/entities/IdTokenEntity.js":function(e,t,r){"use strict";r.r(t),r.d(t,"IdTokenEntity",function(){return s});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/cache/entities/CredentialEntity.js"),i=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),s=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(n.__extends)(t,e),t.createIdTokenEntity=function(e,r,n,o,s,a){var c=new t;return c.credentialType=i.CredentialType.ID_TOKEN,c.homeAccountId=e,c.environment=r,c.clientId=o,c.secret=n,c.realm=s,c.oboAssertion=a,c},t.isIdTokenEntity=function(e){return!!e&&(e.hasOwnProperty("homeAccountId")&&e.hasOwnProperty("environment")&&e.hasOwnProperty("credentialType")&&e.hasOwnProperty("realm")&&e.hasOwnProperty("clientId")&&e.hasOwnProperty("secret")&&e.credentialType===i.CredentialType.ID_TOKEN)},t}(o.CredentialEntity)},"./node_modules/@azure/msal-common/dist/cache/entities/RefreshTokenEntity.js":function(e,t,r){"use strict";r.r(t),r.d(t,"RefreshTokenEntity",function(){return s});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/cache/entities/CredentialEntity.js"),i=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),s=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(n.__extends)(t,e),t.createRefreshTokenEntity=function(e,r,n,o,s,a){var c=new t;return c.clientId=o,c.credentialType=i.CredentialType.REFRESH_TOKEN,c.environment=r,c.homeAccountId=e,c.secret=n,c.oboAssertion=a,s&&(c.familyId=s),c},t.isRefreshTokenEntity=function(e){return!!e&&(e.hasOwnProperty("homeAccountId")&&e.hasOwnProperty("environment")&&e.hasOwnProperty("credentialType")&&e.hasOwnProperty("clientId")&&e.hasOwnProperty("secret")&&e.credentialType===i.CredentialType.REFRESH_TOKEN)},t}(o.CredentialEntity)},"./node_modules/@azure/msal-common/dist/cache/entities/ServerTelemetryEntity.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ServerTelemetryEntity",function(){return o});var n=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),o=function(){function e(){this.failedRequests=[],this.errors=[],this.cacheHits=0}return e.isServerTelemetryEntity=function(e,t){var r=0===e.indexOf(n.SERVER_TELEM_CONSTANTS.CACHE_KEY),o=!0;return t&&(o=t.hasOwnProperty("failedRequests")&&t.hasOwnProperty("errors")&&t.hasOwnProperty("cacheHits")),r&&o},e}();
/*! @azure/msal-common v5.1.0 2021-11-02 */},"./node_modules/@azure/msal-common/dist/cache/entities/ThrottlingEntity.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ThrottlingEntity",function(){return o});var n=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),o=function(){function e(){}return e.isThrottlingEntity=function(e,t){var r=!1;e&&(r=0===e.indexOf(n.ThrottlingConstants.THROTTLING_PREFIX));var o=!0;return t&&(o=t.hasOwnProperty("throttleTime")),r&&o},e}();
/*! @azure/msal-common v5.1.0 2021-11-02 */},"./node_modules/@azure/msal-common/dist/cache/persistence/TokenCacheContext.js":function(e,t,r){"use strict";r.r(t),r.d(t,"TokenCacheContext",function(){return n});
/*! @azure/msal-common v5.1.0 2021-11-02 */
var n=function(){function e(e,t){this.cache=e,this.hasChanged=t}return Object.defineProperty(e.prototype,"cacheHasChanged",{get:function(){return this.hasChanged},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"tokenCache",{get:function(){return this.cache},enumerable:!1,configurable:!0}),e}()},"./node_modules/@azure/msal-common/dist/client/AuthorizationCodeClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"AuthorizationCodeClient",function(){return f});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/client/BaseClient.js"),i=r("./node_modules/@azure/msal-common/dist/request/RequestParameterBuilder.js"),s=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),a=r("./node_modules/@azure/msal-common/dist/response/ResponseHandler.js"),c=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),u=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),d=r("./node_modules/@azure/msal-common/dist/url/UrlString.js"),l=r("./node_modules/@azure/msal-common/dist/crypto/PopTokenGenerator.js"),h=r("./node_modules/@azure/msal-common/dist/utils/TimeUtils.js"),p=r("./node_modules/@azure/msal-common/dist/account/ClientInfo.js"),m=r("./node_modules/@azure/msal-common/dist/account/CcsCredential.js"),g=r("./node_modules/@azure/msal-common/dist/error/ClientConfigurationError.js"),f=function(e){function t(t){return e.call(this,t)||this}return Object(n.__extends)(t,e),t.prototype.getAuthCodeUrl=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t;return Object(n.__generator)(this,function(r){return t=this.createAuthCodeUrlQueryString(e),[2,d.UrlString.appendQueryString(this.authority.authorizationEndpoint,t)]})})},t.prototype.acquireToken=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,o,i;return Object(n.__generator)(this,function(n){switch(n.label){case 0:if(this.logger.info("in acquireToken call"),!e||c.StringUtils.isEmpty(e.code))throw u.ClientAuthError.createTokenRequestCannotBeMadeError();return r=h.TimeUtils.nowSeconds(),[4,this.executeTokenRequest(this.authority,e)];case 1:return o=n.sent(),(i=new a.ResponseHandler(this.config.authOptions.clientId,this.cacheManager,this.cryptoUtils,this.logger,this.config.serializableCache,this.config.persistencePlugin)).validateTokenResponse(o.body),[4,i.handleServerTokenResponse(o.body,this.authority,r,e,t)];case 2:return[2,n.sent()]}})})},t.prototype.handleFragmentResponse=function(e,t){var r=new a.ResponseHandler(this.config.authOptions.clientId,this.cacheManager,this.cryptoUtils,this.logger,null,null),o=new d.UrlString(e),i=d.UrlString.getDeserializedHash(o.getHash());if(r.validateServerAuthorizationCodeResponse(i,t,this.cryptoUtils),!i.code)throw u.ClientAuthError.createNoAuthCodeInServerResponseError();return Object(n.__assign)(Object(n.__assign)({},i),{code:i.code})},t.prototype.getLogoutUri=function(e){if(!e)throw g.ClientConfigurationError.createEmptyLogoutRequestError();var t=this.createLogoutUrlQueryString(e);return d.UrlString.appendQueryString(this.authority.endSessionEndpoint,t)},t.prototype.executeTokenRequest=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,o,i,a,u,d,l;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return r={clientId:this.config.authOptions.clientId,authority:e.canonicalAuthority,scopes:t.scopes,authenticationScheme:t.authenticationScheme,resourceRequestMethod:t.resourceRequestMethod,resourceRequestUri:t.resourceRequestUri,shrClaims:t.shrClaims,sshJwk:t.sshJwk,sshKid:t.sshKid},[4,this.createTokenRequestBody(t)];case 1:if(o=n.sent(),i=this.createTokenQueryParameters(t),a=void 0,t.clientInfo)try{u=Object(p.buildClientInfo)(t.clientInfo,this.cryptoUtils),a={credential:""+u.uid+s.Separators.CLIENT_INFO_SEPARATOR+u.utid,type:m.CcsCredentialType.HOME_ACCOUNT_ID}}catch(e){this.logger.verbose("Could not parse client info for CCS Header: "+e)}return d=this.createTokenRequestHeaders(a||t.ccsCredential),l=c.StringUtils.isEmpty(i)?e.tokenEndpoint:e.tokenEndpoint+"?"+i,[2,this.executePostToTokenEndpoint(l,o,d,r)]}})})},t.prototype.createTokenQueryParameters=function(e){var t=new i.RequestParameterBuilder;return e.tokenQueryParameters&&t.addExtraQueryParameters(e.tokenQueryParameters),t.createQueryString()},t.prototype.createTokenRequestBody=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,o,a,u,d;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return(t=new i.RequestParameterBuilder).addClientId(this.config.authOptions.clientId),t.addRedirectUri(e.redirectUri),t.addScopes(e.scopes),t.addAuthorizationCode(e.code),t.addLibraryInfo(this.config.libraryInfo),t.addThrottling(),this.serverTelemetryManager&&t.addServerTelemetry(this.serverTelemetryManager),e.codeVerifier&&t.addCodeVerifier(e.codeVerifier),this.config.clientCredentials.clientSecret&&t.addClientSecret(this.config.clientCredentials.clientSecret),this.config.clientCredentials.clientAssertion&&(r=this.config.clientCredentials.clientAssertion,t.addClientAssertion(r.assertion),t.addClientAssertionType(r.assertionType)),t.addGrantType(s.GrantType.AUTHORIZATION_CODE_GRANT),t.addClientInfo(),e.authenticationScheme!==s.AuthenticationScheme.POP?[3,2]:[4,new l.PopTokenGenerator(this.cryptoUtils).generateCnf(e)];case 1:return o=n.sent(),t.addPopToken(o),[3,3];case 2:if(e.authenticationScheme===s.AuthenticationScheme.SSH){if(!e.sshJwk)throw g.ClientConfigurationError.createMissingSshJwkError();t.addSshJwk(e.sshJwk)}n.label=3;case 3:if(a=e.correlationId||this.config.cryptoInterface.createNewGuid(),t.addCorrelationId(a),(!c.StringUtils.isEmptyObj(e.claims)||this.config.authOptions.clientCapabilities&&this.config.authOptions.clientCapabilities.length>0)&&t.addClaims(e.claims,this.config.authOptions.clientCapabilities),u=void 0,e.clientInfo)try{d=Object(p.buildClientInfo)(e.clientInfo,this.cryptoUtils),u={credential:""+d.uid+s.Separators.CLIENT_INFO_SEPARATOR+d.utid,type:m.CcsCredentialType.HOME_ACCOUNT_ID}}catch(e){this.logger.verbose("Could not parse client info for CCS Header: "+e)}else u=e.ccsCredential;if(this.config.systemOptions.preventCorsPreflight&&u)switch(u.type){case m.CcsCredentialType.HOME_ACCOUNT_ID:try{d=Object(p.buildClientInfoFromHomeAccountId)(u.credential),t.addCcsOid(d)}catch(e){this.logger.verbose("Could not parse home account ID for CCS Header: "+e)}break;case m.CcsCredentialType.UPN:t.addCcsUpn(u.credential)}return[2,t.createQueryString()]}})})},t.prototype.createAuthCodeUrlQueryString=function(e){var t=new i.RequestParameterBuilder;t.addClientId(this.config.authOptions.clientId);var r=Object(n.__spreadArrays)(e.scopes||[],e.extraScopesToConsent||[]);t.addScopes(r),t.addRedirectUri(e.redirectUri);var o=e.correlationId||this.config.cryptoInterface.createNewGuid();if(t.addCorrelationId(o),t.addResponseMode(e.responseMode),t.addResponseTypeCode(),t.addLibraryInfo(this.config.libraryInfo),t.addClientInfo(),e.codeChallenge&&e.codeChallengeMethod&&t.addCodeChallengeParams(e.codeChallenge,e.codeChallengeMethod),e.prompt&&t.addPrompt(e.prompt),e.domainHint&&t.addDomainHint(e.domainHint),e.prompt!==s.PromptValue.SELECT_ACCOUNT)if(e.sid&&e.prompt===s.PromptValue.NONE)this.logger.verbose("createAuthCodeUrlQueryString: Prompt is none, adding sid from request"),t.addSid(e.sid);else if(e.account){var a=this.extractAccountSid(e.account);if(a&&e.prompt===s.PromptValue.NONE){this.logger.verbose("createAuthCodeUrlQueryString: Prompt is none, adding sid from account"),t.addSid(a);try{var u=Object(p.buildClientInfoFromHomeAccountId)(e.account.homeAccountId);t.addCcsOid(u)}catch(e){this.logger.verbose("Could not parse home account ID for CCS Header: "+e)}}else if(e.loginHint)this.logger.verbose("createAuthCodeUrlQueryString: Adding login_hint from request"),t.addLoginHint(e.loginHint),t.addCcsUpn(e.loginHint);else if(e.account.username){this.logger.verbose("createAuthCodeUrlQueryString: Adding login_hint from account"),t.addLoginHint(e.account.username);try{u=Object(p.buildClientInfoFromHomeAccountId)(e.account.homeAccountId);t.addCcsOid(u)}catch(e){this.logger.verbose("Could not parse home account ID for CCS Header: "+e)}}}else e.loginHint&&(this.logger.verbose("createAuthCodeUrlQueryString: No account, adding login_hint from request"),t.addLoginHint(e.loginHint),t.addCcsUpn(e.loginHint));else this.logger.verbose("createAuthCodeUrlQueryString: Prompt is select_account, ignoring account hints");return e.nonce&&t.addNonce(e.nonce),e.state&&t.addState(e.state),(!c.StringUtils.isEmpty(e.claims)||this.config.authOptions.clientCapabilities&&this.config.authOptions.clientCapabilities.length>0)&&t.addClaims(e.claims,this.config.authOptions.clientCapabilities),e.extraQueryParameters&&t.addExtraQueryParameters(e.extraQueryParameters),t.createQueryString()},t.prototype.createLogoutUrlQueryString=function(e){var t=new i.RequestParameterBuilder;return e.postLogoutRedirectUri&&t.addPostLogoutRedirectUri(e.postLogoutRedirectUri),e.correlationId&&t.addCorrelationId(e.correlationId),e.idTokenHint&&t.addIdTokenHint(e.idTokenHint),e.state&&t.addState(e.state),e.extraQueryParameters&&t.addExtraQueryParameters(e.extraQueryParameters),t.createQueryString()},t.prototype.extractAccountSid=function(e){return e.idTokenClaims&&e.idTokenClaims.sid||null},t}(o.BaseClient)},"./node_modules/@azure/msal-common/dist/client/BaseClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"BaseClient",function(){return h});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/config/ClientConfiguration.js"),i=r("./node_modules/@azure/msal-common/dist/network/NetworkManager.js"),s=r("./node_modules/@azure/msal-common/dist/logger/Logger.js"),a=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),c=r("./node_modules/@azure/msal-common/dist/packageMetadata.js"),u=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),d=r("./node_modules/@azure/msal-common/dist/account/CcsCredential.js"),l=r("./node_modules/@azure/msal-common/dist/account/ClientInfo.js"),h=function(){function e(e){this.config=Object(o.buildClientConfiguration)(e),this.logger=new s.Logger(this.config.loggerOptions,c.name,c.version),this.cryptoUtils=this.config.cryptoInterface,this.cacheManager=this.config.storageInterface,this.networkClient=this.config.networkInterface,this.networkManager=new i.NetworkManager(this.networkClient,this.cacheManager),this.serverTelemetryManager=this.config.serverTelemetryManager,this.authority=this.config.authOptions.authority}return e.prototype.createTokenRequestHeaders=function(e){var t={};if(t[a.HeaderNames.CONTENT_TYPE]=a.Constants.URL_FORM_CONTENT_TYPE,!this.config.systemOptions.preventCorsPreflight&&e)switch(e.type){case d.CcsCredentialType.HOME_ACCOUNT_ID:try{var r=Object(l.buildClientInfoFromHomeAccountId)(e.credential);t[a.HeaderNames.CCS_HEADER]="Oid:"+r.uid+"@"+r.utid}catch(e){this.logger.verbose("Could not parse home account ID for CCS Header: "+e)}break;case d.CcsCredentialType.UPN:t[a.HeaderNames.CCS_HEADER]="UPN: "+e.credential}return t},e.prototype.executePostToTokenEndpoint=function(e,t,r,o){return Object(n.__awaiter)(this,void 0,void 0,function(){var i;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return[4,this.networkManager.sendPostRequest(o,e,{body:t,headers:r})];case 1:return i=n.sent(),this.config.serverTelemetryManager&&i.status<500&&429!==i.status&&this.config.serverTelemetryManager.clearTelemetryCache(),[2,i]}})})},e.prototype.updateAuthority=function(e){if(!e.discoveryComplete())throw u.ClientAuthError.createEndpointDiscoveryIncompleteError("Updated authority has not completed endpoint discovery.");this.authority=e},e}()},"./node_modules/@azure/msal-common/dist/client/ClientCredentialClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ClientCredentialClient",function(){return h});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/client/BaseClient.js"),i=r("./node_modules/@azure/msal-common/dist/request/RequestParameterBuilder.js"),s=r("./node_modules/@azure/msal-common/dist/request/ScopeSet.js"),a=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),c=r("./node_modules/@azure/msal-common/dist/response/ResponseHandler.js"),u=r("./node_modules/@azure/msal-common/dist/utils/TimeUtils.js"),d=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),l=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),h=function(e){function t(t){return e.call(this,t)||this}return Object(n.__extends)(t,e),t.prototype.acquireToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t;return Object(n.__generator)(this,function(r){switch(r.label){case 0:return this.scopeSet=new s.ScopeSet(e.scopes||[]),e.skipCache?[4,this.executeTokenRequest(e,this.authority)]:[3,2];case 1:return[2,r.sent()];case 2:return[4,this.getCachedAuthenticationResult(e)];case 3:return(t=r.sent())?[2,t]:[3,4];case 4:return[4,this.executeTokenRequest(e,this.authority)];case 5:return[2,r.sent()]}})})},t.prototype.getCachedAuthenticationResult=function(e){var t,r;return Object(n.__awaiter)(this,void 0,void 0,function(){var o;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return(o=this.readAccessTokenFromCache())?u.TimeUtils.isTokenExpired(o.expiresOn,this.config.systemOptions.tokenRenewalOffsetSeconds)?(null===(r=this.serverTelemetryManager)||void 0===r||r.setCacheOutcome(a.CacheOutcome.CACHED_ACCESS_TOKEN_EXPIRED),[2,null]):[4,c.ResponseHandler.generateAuthenticationResult(this.cryptoUtils,this.authority,{account:null,idToken:null,accessToken:o,refreshToken:null,appMetadata:null},!0,e)]:(null===(t=this.serverTelemetryManager)||void 0===t||t.setCacheOutcome(a.CacheOutcome.NO_CACHED_ACCESS_TOKEN),[2,null]);case 1:return[2,n.sent()]}})})},t.prototype.readAccessTokenFromCache=function(){var e={homeAccountId:"",environment:this.authority.canonicalAuthorityUrlComponents.HostNameAndPort,credentialType:a.CredentialType.ACCESS_TOKEN,clientId:this.config.authOptions.clientId,realm:this.authority.tenant,target:this.scopeSet.printScopesLowerCase()},t=this.cacheManager.getCredentialsFilteredBy(e),r=Object.keys(t.accessTokens).map(function(e){return t.accessTokens[e]});if(r.length<1)return null;if(r.length>1)throw l.ClientAuthError.createMultipleMatchingTokensInCacheError();return r[0]},t.prototype.executeTokenRequest=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,o,i,s,a,d;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return r=this.createTokenRequestBody(e),o=this.createTokenRequestHeaders(),i={clientId:this.config.authOptions.clientId,authority:e.authority,scopes:e.scopes,authenticationScheme:e.authenticationScheme,resourceRequestMethod:e.resourceRequestMethod,resourceRequestUri:e.resourceRequestUri,shrClaims:e.shrClaims,sshJwk:e.sshJwk,sshKid:e.sshKid},s=u.TimeUtils.nowSeconds(),[4,this.executePostToTokenEndpoint(t.tokenEndpoint,r,o,i)];case 1:return a=n.sent(),(d=new c.ResponseHandler(this.config.authOptions.clientId,this.cacheManager,this.cryptoUtils,this.logger,this.config.serializableCache,this.config.persistencePlugin)).validateTokenResponse(a.body),[4,d.handleServerTokenResponse(a.body,this.authority,s,e)];case 2:return[2,n.sent()]}})})},t.prototype.createTokenRequestBody=function(e){var t=new i.RequestParameterBuilder;t.addClientId(this.config.authOptions.clientId),t.addScopes(e.scopes,!1),t.addGrantType(a.GrantType.CLIENT_CREDENTIALS_GRANT),t.addLibraryInfo(this.config.libraryInfo),t.addThrottling(),this.serverTelemetryManager&&t.addServerTelemetry(this.serverTelemetryManager);var r=e.correlationId||this.config.cryptoInterface.createNewGuid();if(t.addCorrelationId(r),this.config.clientCredentials.clientSecret&&t.addClientSecret(this.config.clientCredentials.clientSecret),this.config.clientCredentials.clientAssertion){var n=this.config.clientCredentials.clientAssertion;t.addClientAssertion(n.assertion),t.addClientAssertionType(n.assertionType)}return(!d.StringUtils.isEmptyObj(e.claims)||this.config.authOptions.clientCapabilities&&this.config.authOptions.clientCapabilities.length>0)&&t.addClaims(e.claims,this.config.authOptions.clientCapabilities),t.createQueryString()},t}(o.BaseClient)},"./node_modules/@azure/msal-common/dist/client/DeviceCodeClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"DeviceCodeClient",function(){return l});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/client/BaseClient.js"),i=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),s=r("./node_modules/@azure/msal-common/dist/request/RequestParameterBuilder.js"),a=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),c=r("./node_modules/@azure/msal-common/dist/utils/TimeUtils.js"),u=r("./node_modules/@azure/msal-common/dist/response/ResponseHandler.js"),d=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),l=function(e){function t(t){return e.call(this,t)||this}return Object(n.__extends)(t,e),t.prototype.acquireToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,o,i;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return[4,this.getDeviceCode(e)];case 1:return t=n.sent(),e.deviceCodeCallback(t),r=c.TimeUtils.nowSeconds(),[4,this.acquireTokenWithDeviceCode(e,t)];case 2:return o=n.sent(),(i=new u.ResponseHandler(this.config.authOptions.clientId,this.cacheManager,this.cryptoUtils,this.logger,this.config.serializableCache,this.config.persistencePlugin)).validateTokenResponse(o),[4,i.handleServerTokenResponse(o,this.authority,r,e)];case 3:return[2,n.sent()]}})})},t.prototype.getDeviceCode=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,o;return Object(n.__generator)(this,function(n){return t=this.createQueryString(e),r=this.createTokenRequestHeaders(),o={clientId:this.config.authOptions.clientId,authority:e.authority,scopes:e.scopes,authenticationScheme:e.authenticationScheme,resourceRequestMethod:e.resourceRequestMethod,resourceRequestUri:e.resourceRequestUri,shrClaims:e.shrClaims,sshJwk:e.sshJwk,sshKid:e.sshKid},[2,this.executePostRequestToDeviceCodeEndpoint(this.authority.deviceCodeEndpoint,t,r,o)]})})},t.prototype.executePostRequestToDeviceCodeEndpoint=function(e,t,r,o){return Object(n.__awaiter)(this,void 0,void 0,function(){var i,s,a,c,u,d,l;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return[4,this.networkManager.sendPostRequest(o,e,{body:t,headers:r})];case 1:return i=n.sent().body,s=i.user_code,a=i.device_code,c=i.verification_uri,u=i.expires_in,d=i.interval,l=i.message,[2,{userCode:s,deviceCode:a,verificationUri:c,expiresIn:u,interval:d,message:l}]}})})},t.prototype.createQueryString=function(e){var t=new s.RequestParameterBuilder;return t.addScopes(e.scopes),t.addClientId(this.config.authOptions.clientId),(!d.StringUtils.isEmpty(e.claims)||this.config.authOptions.clientCapabilities&&this.config.authOptions.clientCapabilities.length>0)&&t.addClaims(e.claims,this.config.authOptions.clientCapabilities),t.createQueryString()},t.prototype.continuePolling=function(e,t,r){if(r)throw this.logger.error("Token request cancelled by setting DeviceCodeRequest.cancel = true"),i.ClientAuthError.createDeviceCodeCancelledError();if(t&&t<e&&c.TimeUtils.nowSeconds()>t)throw this.logger.error("User defined timeout for device code polling reached. The timeout was set for "+t),i.ClientAuthError.createUserTimeoutReachedError();if(c.TimeUtils.nowSeconds()>e)throw t&&this.logger.verbose("User specified timeout ignored as the device code has expired before the timeout elapsed. The user specified timeout was set for "+t),this.logger.error("Device code expired. Expiration time of device code was "+e),i.ClientAuthError.createDeviceCodeExpiredError();return!0},t.prototype.acquireTokenWithDeviceCode=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,o,s,u,d,l,h;return Object(n.__generator)(this,function(n){switch(n.label){case 0:r=this.createTokenRequestBody(e,t),o=this.createTokenRequestHeaders(),s=e.timeout?c.TimeUtils.nowSeconds()+e.timeout:void 0,u=c.TimeUtils.nowSeconds()+t.expiresIn,d=1e3*t.interval,n.label=1;case 1:return this.continuePolling(u,s,e.cancel)?(l={clientId:this.config.authOptions.clientId,authority:e.authority,scopes:e.scopes,authenticationScheme:e.authenticationScheme,resourceRequestMethod:e.resourceRequestMethod,resourceRequestUri:e.resourceRequestUri,shrClaims:e.shrClaims,sshJwk:e.sshJwk,sshKid:e.sshKid},[4,this.executePostToTokenEndpoint(this.authority.tokenEndpoint,r,o,l)]):[3,6];case 2:return(h=n.sent()).body&&h.body.error===a.Constants.AUTHORIZATION_PENDING?(this.logger.info(h.body.error_description||"Authorization pending. Continue polling."),[4,c.TimeUtils.delay(d)]):[3,4];case 3:return n.sent(),[3,5];case 4:return this.logger.verbose("Authorization completed successfully. Polling stopped."),[2,h.body];case 5:return[3,1];case 6:throw this.logger.error("Polling stopped for unknown reasons."),i.ClientAuthError.createDeviceCodeUnknownError()}})})},t.prototype.createTokenRequestBody=function(e,t){var r=new s.RequestParameterBuilder;r.addScopes(e.scopes),r.addClientId(this.config.authOptions.clientId),r.addGrantType(a.GrantType.DEVICE_CODE_GRANT),r.addDeviceCode(t.deviceCode);var n=e.correlationId||this.config.cryptoInterface.createNewGuid();return r.addCorrelationId(n),r.addClientInfo(),r.addLibraryInfo(this.config.libraryInfo),r.addThrottling(),this.serverTelemetryManager&&r.addServerTelemetry(this.serverTelemetryManager),(!d.StringUtils.isEmptyObj(e.claims)||this.config.authOptions.clientCapabilities&&this.config.authOptions.clientCapabilities.length>0)&&r.addClaims(e.claims,this.config.authOptions.clientCapabilities),r.createQueryString()},t}(o.BaseClient)},"./node_modules/@azure/msal-common/dist/client/OnBehalfOfClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"OnBehalfOfClient",function(){return h});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/client/BaseClient.js"),i=r("./node_modules/@azure/msal-common/dist/request/RequestParameterBuilder.js"),s=r("./node_modules/@azure/msal-common/dist/request/ScopeSet.js"),a=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),c=r("./node_modules/@azure/msal-common/dist/response/ResponseHandler.js"),u=r("./node_modules/@azure/msal-common/dist/utils/TimeUtils.js"),d=r("./node_modules/@azure/msal-common/dist/account/AuthToken.js"),l=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),h=function(e){function t(t){return e.call(this,t)||this}return Object(n.__extends)(t,e),t.prototype.acquireToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t;return Object(n.__generator)(this,function(r){switch(r.label){case 0:return this.scopeSet=new s.ScopeSet(e.scopes||[]),e.skipCache?[4,this.executeTokenRequest(e,this.authority)]:[3,2];case 1:return[2,r.sent()];case 2:return[4,this.getCachedAuthenticationResult(e)];case 3:return(t=r.sent())?[2,t]:[3,4];case 4:return[4,this.executeTokenRequest(e,this.authority)];case 5:return[2,r.sent()]}})})},t.prototype.getCachedAuthenticationResult=function(e){var t;return Object(n.__awaiter)(this,void 0,void 0,function(){var r,o,i,s,l,h;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return!(r=this.readAccessTokenFromCache())||u.TimeUtils.isTokenExpired(r.expiresOn,this.config.systemOptions.tokenRenewalOffsetSeconds)?(null===(t=this.serverTelemetryManager)||void 0===t||t.setCacheOutcome(r?a.CacheOutcome.NO_CACHED_ACCESS_TOKEN:a.CacheOutcome.CACHED_ACCESS_TOKEN_EXPIRED),[2,null]):(o=this.readIdTokenFromCache(e),s=null,o&&(i=new d.AuthToken(o.secret,this.config.cryptoInterface),l=i.claims.oid?i.claims.oid:i.claims.sub,h={homeAccountId:o.homeAccountId,environment:o.environment,tenantId:o.realm,username:a.Constants.EMPTY_STRING,localAccountId:l||""},s=this.readAccountFromCache(h)),[4,c.ResponseHandler.generateAuthenticationResult(this.cryptoUtils,this.authority,{account:s,accessToken:r,idToken:o,refreshToken:null,appMetadata:null},!0,e,i)]);case 1:return[2,n.sent()]}})})},t.prototype.readAccessTokenFromCache=function(){var e={environment:this.authority.canonicalAuthorityUrlComponents.HostNameAndPort,credentialType:a.CredentialType.ACCESS_TOKEN,clientId:this.config.authOptions.clientId,realm:this.authority.tenant,target:this.scopeSet.printScopesLowerCase()},t=this.cacheManager.getCredentialsFilteredBy(e),r=Object.keys(t.accessTokens).map(function(e){return t.accessTokens[e]}),n=r.length;if(n<1)return null;if(n>1)throw l.ClientAuthError.createMultipleMatchingTokensInCacheError();return r[0]},t.prototype.readIdTokenFromCache=function(e){var t={environment:this.authority.canonicalAuthorityUrlComponents.HostNameAndPort,credentialType:a.CredentialType.ID_TOKEN,clientId:this.config.authOptions.clientId,realm:this.authority.tenant,oboAssertion:e.oboAssertion},r=this.cacheManager.getCredentialsFilteredBy(t),n=Object.keys(r.idTokens).map(function(e){return r.idTokens[e]});return n.length<1?null:n[0]},t.prototype.readAccountFromCache=function(e){return this.cacheManager.readAccountFromCache(e)},t.prototype.executeTokenRequest=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,o,i,s,a,d;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return r=this.createTokenRequestBody(e),o=this.createTokenRequestHeaders(),i={clientId:this.config.authOptions.clientId,authority:e.authority,scopes:e.scopes,authenticationScheme:e.authenticationScheme,resourceRequestMethod:e.resourceRequestMethod,resourceRequestUri:e.resourceRequestUri,shrClaims:e.shrClaims,sshJwk:e.sshJwk,sshKid:e.sshKid},s=u.TimeUtils.nowSeconds(),[4,this.executePostToTokenEndpoint(t.tokenEndpoint,r,o,i)];case 1:return a=n.sent(),(d=new c.ResponseHandler(this.config.authOptions.clientId,this.cacheManager,this.cryptoUtils,this.logger,this.config.serializableCache,this.config.persistencePlugin)).validateTokenResponse(a.body),[4,d.handleServerTokenResponse(a.body,this.authority,s,e)];case 2:return[2,n.sent()]}})})},t.prototype.createTokenRequestBody=function(e){var t=new i.RequestParameterBuilder;t.addClientId(this.config.authOptions.clientId),t.addScopes(e.scopes),t.addGrantType(a.GrantType.JWT_BEARER),t.addClientInfo(),t.addLibraryInfo(this.config.libraryInfo),t.addThrottling(),this.serverTelemetryManager&&t.addServerTelemetry(this.serverTelemetryManager);var r=e.correlationId||this.config.cryptoInterface.createNewGuid();if(t.addCorrelationId(r),t.addRequestTokenUse(a.AADServerParamKeys.ON_BEHALF_OF),t.addOboAssertion(e.oboAssertion),this.config.clientCredentials.clientSecret&&t.addClientSecret(this.config.clientCredentials.clientSecret),this.config.clientCredentials.clientAssertion){var n=this.config.clientCredentials.clientAssertion;t.addClientAssertion(n.assertion),t.addClientAssertionType(n.assertionType)}return t.createQueryString()},t}(o.BaseClient)},"./node_modules/@azure/msal-common/dist/client/RefreshTokenClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"RefreshTokenClient",function(){return C});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/client/BaseClient.js"),i=r("./node_modules/@azure/msal-common/dist/request/RequestParameterBuilder.js"),s=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),a=r("./node_modules/@azure/msal-common/dist/response/ResponseHandler.js"),c=r("./node_modules/@azure/msal-common/dist/crypto/PopTokenGenerator.js"),u=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),d=r("./node_modules/@azure/msal-common/dist/error/ClientConfigurationError.js"),l=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),h=r("./node_modules/@azure/msal-common/dist/error/ServerError.js"),p=r("./node_modules/@azure/msal-common/dist/utils/TimeUtils.js"),m=r("./node_modules/@azure/msal-common/dist/url/UrlString.js"),g=r("./node_modules/@azure/msal-common/dist/account/CcsCredential.js"),f=r("./node_modules/@azure/msal-common/dist/account/ClientInfo.js"),y=r("./node_modules/@azure/msal-common/dist/error/InteractionRequiredAuthError.js"),C=function(e){function t(t){return e.call(this,t)||this}return Object(n.__extends)(t,e),t.prototype.acquireToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,o;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return t=p.TimeUtils.nowSeconds(),[4,this.executeTokenRequest(e,this.authority)];case 1:return r=n.sent(),(o=new a.ResponseHandler(this.config.authOptions.clientId,this.cacheManager,this.cryptoUtils,this.logger,this.config.serializableCache,this.config.persistencePlugin)).validateTokenResponse(r.body),[2,o.handleServerTokenResponse(r.body,this.authority,t,e,void 0,void 0,!0)]}})})},t.prototype.acquireTokenByRefreshToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r;return Object(n.__generator)(this,function(n){if(!e)throw d.ClientConfigurationError.createEmptyTokenRequestError();if(!e.account)throw l.ClientAuthError.createNoAccountInSilentRequestError();if(this.cacheManager.isAppMetadataFOCI(e.account.environment,this.config.authOptions.clientId))try{return[2,this.acquireTokenWithCachedRefreshToken(e,!0)]}catch(n){if(t=n instanceof y.InteractionRequiredAuthError&&n.errorCode===y.InteractionRequiredAuthErrorMessage.noTokensFoundError.code,r=n instanceof h.ServerError&&n.errorCode===s.Errors.INVALID_GRANT_ERROR&&n.subError===s.Errors.CLIENT_MISMATCH_ERROR,t||r)return[2,this.acquireTokenWithCachedRefreshToken(e,!1)];throw n}return[2,this.acquireTokenWithCachedRefreshToken(e,!1)]})})},t.prototype.acquireTokenWithCachedRefreshToken=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,o;return Object(n.__generator)(this,function(i){if(!(r=this.cacheManager.readRefreshTokenFromCache(this.config.authOptions.clientId,e.account,t)))throw y.InteractionRequiredAuthError.createNoTokensFoundError();return o=Object(n.__assign)(Object(n.__assign)({},e),{refreshToken:r.secret,authenticationScheme:e.authenticationScheme||s.AuthenticationScheme.BEARER,ccsCredential:{credential:e.account.homeAccountId,type:g.CcsCredentialType.HOME_ACCOUNT_ID}}),[2,this.acquireToken(o)]})})},t.prototype.executeTokenRequest=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,o,i,s,a;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return[4,this.createTokenRequestBody(e)];case 1:return r=n.sent(),o=this.createTokenQueryParameters(e),i=this.createTokenRequestHeaders(e.ccsCredential),s={clientId:this.config.authOptions.clientId,authority:t.canonicalAuthority,scopes:e.scopes,authenticationScheme:e.authenticationScheme,resourceRequestMethod:e.resourceRequestMethod,resourceRequestUri:e.resourceRequestUri,shrClaims:e.shrClaims,sshJwk:e.sshJwk,sshKid:e.sshKid},a=m.UrlString.appendQueryString(t.tokenEndpoint,o),[2,this.executePostToTokenEndpoint(a,r,i,s)]}})})},t.prototype.createTokenQueryParameters=function(e){var t=new i.RequestParameterBuilder;return e.tokenQueryParameters&&t.addExtraQueryParameters(e.tokenQueryParameters),t.createQueryString()},t.prototype.createTokenRequestBody=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,o,a,l;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return(t=new i.RequestParameterBuilder).addClientId(this.config.authOptions.clientId),t.addScopes(e.scopes),t.addGrantType(s.GrantType.REFRESH_TOKEN_GRANT),t.addClientInfo(),t.addLibraryInfo(this.config.libraryInfo),t.addThrottling(),this.serverTelemetryManager&&t.addServerTelemetry(this.serverTelemetryManager),r=e.correlationId||this.config.cryptoInterface.createNewGuid(),t.addCorrelationId(r),t.addRefreshToken(e.refreshToken),this.config.clientCredentials.clientSecret&&t.addClientSecret(this.config.clientCredentials.clientSecret),this.config.clientCredentials.clientAssertion&&(o=this.config.clientCredentials.clientAssertion,t.addClientAssertion(o.assertion),t.addClientAssertionType(o.assertionType)),e.authenticationScheme!==s.AuthenticationScheme.POP?[3,2]:[4,new c.PopTokenGenerator(this.cryptoUtils).generateCnf(e)];case 1:return a=n.sent(),t.addPopToken(a),[3,3];case 2:if(e.authenticationScheme===s.AuthenticationScheme.SSH){if(!e.sshJwk)throw d.ClientConfigurationError.createMissingSshJwkError();t.addSshJwk(e.sshJwk)}n.label=3;case 3:if((!u.StringUtils.isEmptyObj(e.claims)||this.config.authOptions.clientCapabilities&&this.config.authOptions.clientCapabilities.length>0)&&t.addClaims(e.claims,this.config.authOptions.clientCapabilities),this.config.systemOptions.preventCorsPreflight&&e.ccsCredential)switch(e.ccsCredential.type){case g.CcsCredentialType.HOME_ACCOUNT_ID:try{l=Object(f.buildClientInfoFromHomeAccountId)(e.ccsCredential.credential),t.addCcsOid(l)}catch(e){this.logger.verbose("Could not parse home account ID for CCS Header: "+e)}break;case g.CcsCredentialType.UPN:t.addCcsUpn(e.ccsCredential.credential)}return[2,t.createQueryString()]}})})},t}(o.BaseClient)},"./node_modules/@azure/msal-common/dist/client/SilentFlowClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"SilentFlowClient",function(){return m});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/client/BaseClient.js"),i=r("./node_modules/@azure/msal-common/dist/request/ScopeSet.js"),s=r("./node_modules/@azure/msal-common/dist/account/AuthToken.js"),a=r("./node_modules/@azure/msal-common/dist/utils/TimeUtils.js"),c=r("./node_modules/@azure/msal-common/dist/client/RefreshTokenClient.js"),u=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),d=r("./node_modules/@azure/msal-common/dist/error/ClientConfigurationError.js"),l=r("./node_modules/@azure/msal-common/dist/response/ResponseHandler.js"),h=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),p=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),m=function(e){function t(t){return e.call(this,t)||this}return Object(n.__extends)(t,e),t.prototype.acquireToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t;return Object(n.__generator)(this,function(r){switch(r.label){case 0:return r.trys.push([0,2,,3]),[4,this.acquireCachedToken(e)];case 1:return[2,r.sent()];case 2:if((t=r.sent())instanceof u.ClientAuthError&&t.errorCode===u.ClientAuthErrorMessage.tokenRefreshRequired.code)return[2,new c.RefreshTokenClient(this.config).acquireTokenByRefreshToken(e)];throw t;case 3:return[2]}})})},t.prototype.acquireCachedToken=function(e){var t,r,o,s;return Object(n.__awaiter)(this,void 0,void 0,function(){var c,l,m,g;return Object(n.__generator)(this,function(n){switch(n.label){case 0:if(!e)throw d.ClientConfigurationError.createEmptyTokenRequestError();if(e.forceRefresh)throw null===(t=this.serverTelemetryManager)||void 0===t||t.setCacheOutcome(h.CacheOutcome.FORCE_REFRESH),this.logger.info("SilentFlowClient:acquireCachedToken - Skipping cache because forceRefresh is true."),u.ClientAuthError.createRefreshRequiredError();if(!p.StringUtils.isEmptyObj(e.claims))throw this.logger.info("SilentFlowClient:acquireCachedToken - Skipping cache because claims are requested."),u.ClientAuthError.createRefreshRequiredError();if(!e.account)throw u.ClientAuthError.createNoAccountInSilentRequestError();if(c=new i.ScopeSet(e.scopes||[]),l=e.authority||this.authority.getPreferredCache(),m=e.authenticationScheme||h.AuthenticationScheme.BEARER,!(g=this.cacheManager.readCacheRecord(e.account,this.config.authOptions.clientId,c,l,m,e.sshKid)).accessToken)throw null===(r=this.serverTelemetryManager)||void 0===r||r.setCacheOutcome(h.CacheOutcome.NO_CACHED_ACCESS_TOKEN),this.logger.info("SilentFlowClient:acquireCachedToken - No access token found in cache for the given properties."),u.ClientAuthError.createRefreshRequiredError();if(a.TimeUtils.wasClockTurnedBack(g.accessToken.cachedAt)||a.TimeUtils.isTokenExpired(g.accessToken.expiresOn,this.config.systemOptions.tokenRenewalOffsetSeconds))throw null===(o=this.serverTelemetryManager)||void 0===o||o.setCacheOutcome(h.CacheOutcome.CACHED_ACCESS_TOKEN_EXPIRED),this.logger.info("SilentFlowClient:acquireCachedToken - Cached access token is expired or will expire within "+this.config.systemOptions.tokenRenewalOffsetSeconds+" seconds."),u.ClientAuthError.createRefreshRequiredError();if(g.accessToken.refreshOn&&a.TimeUtils.isTokenExpired(g.accessToken.refreshOn,0))throw null===(s=this.serverTelemetryManager)||void 0===s||s.setCacheOutcome(h.CacheOutcome.REFRESH_CACHED_ACCESS_TOKEN),this.logger.info("SilentFlowClient:acquireCachedToken - Cached access token's refreshOn property has been exceeded'."),u.ClientAuthError.createRefreshRequiredError();return this.config.serverTelemetryManager&&this.config.serverTelemetryManager.incrementCacheHits(),[4,this.generateResultFromCacheRecord(g,e)];case 1:return[2,n.sent()]}})})},t.prototype.generateResultFromCacheRecord=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return e.idToken&&(r=new s.AuthToken(e.idToken.secret,this.config.cryptoInterface)),[4,l.ResponseHandler.generateAuthenticationResult(this.cryptoUtils,this.authority,e,!0,t,r)];case 1:return[2,n.sent()]}})})},t}(o.BaseClient)},"./node_modules/@azure/msal-common/dist/client/UsernamePasswordClient.js":function(e,t,r){"use strict";r.r(t),r.d(t,"UsernamePasswordClient",function(){return l});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/client/BaseClient.js"),i=r("./node_modules/@azure/msal-common/dist/response/ResponseHandler.js"),s=r("./node_modules/@azure/msal-common/dist/request/RequestParameterBuilder.js"),a=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),c=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),u=r("./node_modules/@azure/msal-common/dist/utils/TimeUtils.js"),d=r("./node_modules/@azure/msal-common/dist/account/CcsCredential.js"),l=function(e){function t(t){return e.call(this,t)||this}return Object(n.__extends)(t,e),t.prototype.acquireToken=function(e){return Object(n.__awaiter)(this,void 0,void 0,function(){var t,r,o;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return this.logger.info("in acquireToken call"),t=u.TimeUtils.nowSeconds(),[4,this.executeTokenRequest(this.authority,e)];case 1:return r=n.sent(),(o=new i.ResponseHandler(this.config.authOptions.clientId,this.cacheManager,this.cryptoUtils,this.logger,this.config.serializableCache,this.config.persistencePlugin)).validateTokenResponse(r.body),[2,o.handleServerTokenResponse(r.body,this.authority,t,e)]}})})},t.prototype.executeTokenRequest=function(e,t){return Object(n.__awaiter)(this,void 0,void 0,function(){var r,o,i;return Object(n.__generator)(this,function(n){return r={clientId:this.config.authOptions.clientId,authority:e.canonicalAuthority,scopes:t.scopes,authenticationScheme:t.authenticationScheme,resourceRequestMethod:t.resourceRequestMethod,resourceRequestUri:t.resourceRequestUri,shrClaims:t.shrClaims,sshJwk:t.sshJwk,sshKid:t.sshKid},o=this.createTokenRequestBody(t),i=this.createTokenRequestHeaders({credential:t.username,type:d.CcsCredentialType.UPN}),[2,this.executePostToTokenEndpoint(e.tokenEndpoint,o,i,r)]})})},t.prototype.createTokenRequestBody=function(e){var t=new s.RequestParameterBuilder;t.addClientId(this.config.authOptions.clientId),t.addUsername(e.username),t.addPassword(e.password),t.addScopes(e.scopes),t.addGrantType(a.GrantType.RESOURCE_OWNER_PASSWORD_GRANT),t.addClientInfo(),t.addLibraryInfo(this.config.libraryInfo),t.addThrottling(),this.serverTelemetryManager&&t.addServerTelemetry(this.serverTelemetryManager);var r=e.correlationId||this.config.cryptoInterface.createNewGuid();if(t.addCorrelationId(r),this.config.clientCredentials.clientSecret&&t.addClientSecret(this.config.clientCredentials.clientSecret),this.config.clientCredentials.clientAssertion){var n=this.config.clientCredentials.clientAssertion;t.addClientAssertion(n.assertion),t.addClientAssertionType(n.assertionType)}return(!c.StringUtils.isEmptyObj(e.claims)||this.config.authOptions.clientCapabilities&&this.config.authOptions.clientCapabilities.length>0)&&t.addClaims(e.claims,this.config.authOptions.clientCapabilities),this.config.systemOptions.preventCorsPreflight&&e.username&&t.addCcsUpn(e.username),t.createQueryString()},t}(o.BaseClient)},"./node_modules/@azure/msal-common/dist/config/ClientConfiguration.js":function(e,t,r){"use strict";r.r(t),r.d(t,"DEFAULT_SYSTEM_OPTIONS",function(){return d}),r.d(t,"buildClientConfiguration",function(){return g});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/crypto/ICrypto.js"),i=r("./node_modules/@azure/msal-common/dist/error/AuthError.js"),s=r("./node_modules/@azure/msal-common/dist/logger/Logger.js"),a=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),c=r("./node_modules/@azure/msal-common/dist/packageMetadata.js"),u=r("./node_modules/@azure/msal-common/dist/cache/CacheManager.js"),d={tokenRenewalOffsetSeconds:300,preventCorsPreflight:!1},l={loggerCallback:function(){},piiLoggingEnabled:!1,logLevel:s.LogLevel.Info,correlationId:""},h={sendGetRequestAsync:function(){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(e){throw"Network interface - sendGetRequestAsync() has not been implemented",i.AuthError.createUnexpectedError("Network interface - sendGetRequestAsync() has not been implemented")})})},sendPostRequestAsync:function(){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(e){throw"Network interface - sendPostRequestAsync() has not been implemented",i.AuthError.createUnexpectedError("Network interface - sendPostRequestAsync() has not been implemented")})})}},p={sku:a.Constants.SKU,version:c.version,cpu:"",os:""},m={clientSecret:"",clientAssertion:void 0};function g(e){var t,r=e.authOptions,i=e.systemOptions,s=e.loggerOptions,a=e.storageInterface,c=e.networkInterface,g=e.cryptoInterface,f=e.clientCredentials,y=e.libraryInfo,C=e.serverTelemetryManager,E=e.persistencePlugin,_=e.serializableCache,v=Object(n.__assign)(Object(n.__assign)({},l),s);return{authOptions:(t=r,Object(n.__assign)({clientCapabilities:[]},t)),systemOptions:Object(n.__assign)(Object(n.__assign)({},d),i),loggerOptions:v,storageInterface:a||new u.DefaultStorageClass(r.clientId,o.DEFAULT_CRYPTO_IMPLEMENTATION),networkInterface:c||h,cryptoInterface:g||o.DEFAULT_CRYPTO_IMPLEMENTATION,clientCredentials:f||m,libraryInfo:Object(n.__assign)(Object(n.__assign)({},p),y),serverTelemetryManager:C||null,persistencePlugin:E||null,serializableCache:_||null}}},"./node_modules/@azure/msal-common/dist/crypto/ICrypto.js":function(e,t,r){"use strict";r.r(t),r.d(t,"DEFAULT_CRYPTO_IMPLEMENTATION",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/error/AuthError.js"),i={createNewGuid:function(){throw o.AuthError.createUnexpectedError("Crypto interface - createNewGuid() has not been implemented")},base64Decode:function(){throw o.AuthError.createUnexpectedError("Crypto interface - base64Decode() has not been implemented")},base64Encode:function(){throw o.AuthError.createUnexpectedError("Crypto interface - base64Encode() has not been implemented")},generatePkceCodes:function(){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(e){throw"Crypto interface - generatePkceCodes() has not been implemented",o.AuthError.createUnexpectedError("Crypto interface - generatePkceCodes() has not been implemented")})})},getPublicKeyThumbprint:function(){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(e){throw"Crypto interface - getPublicKeyThumbprint() has not been implemented",o.AuthError.createUnexpectedError("Crypto interface - getPublicKeyThumbprint() has not been implemented")})})},removeTokenBindingKey:function(){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(e){throw"Crypto interface - removeTokenBindingKey() has not been implemented",o.AuthError.createUnexpectedError("Crypto interface - removeTokenBindingKey() has not been implemented")})})},clearKeystore:function(){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(e){throw"Crypto interface - clearKeystore() has not been implemented",o.AuthError.createUnexpectedError("Crypto interface - clearKeystore() has not been implemented")})})},signJwt:function(){return Object(n.__awaiter)(this,void 0,void 0,function(){return Object(n.__generator)(this,function(e){throw"Crypto interface - signJwt() has not been implemented",o.AuthError.createUnexpectedError("Crypto interface - signJwt() has not been implemented")})})}}},"./node_modules/@azure/msal-common/dist/crypto/PopTokenGenerator.js":function(e,t,r){"use strict";r.r(t),r.d(t,"PopTokenGenerator",function(){return u});var n,o=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),i=r("./node_modules/@azure/msal-common/dist/account/AuthToken.js"),s=r("./node_modules/@azure/msal-common/dist/utils/TimeUtils.js"),a=r("./node_modules/@azure/msal-common/dist/url/UrlString.js"),c=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js");!function(e){e.SW="sw",e.UHW="uhw"}(n||(n={}));var u=function(){function e(e){this.cryptoUtils=e}return e.prototype.generateCnf=function(e){return Object(o.__awaiter)(this,void 0,void 0,function(){var t;return Object(o.__generator)(this,function(r){switch(r.label){case 0:return[4,this.generateKid(e)];case 1:return t=r.sent(),[2,this.cryptoUtils.base64Encode(JSON.stringify(t))]}})})},e.prototype.generateKid=function(e){return Object(o.__awaiter)(this,void 0,void 0,function(){return Object(o.__generator)(this,function(t){switch(t.label){case 0:return[4,this.cryptoUtils.getPublicKeyThumbprint(e)];case 1:return[2,{kid:t.sent(),xms_ksl:n.SW}]}})})},e.prototype.signPopToken=function(e,t){var r;return Object(o.__awaiter)(this,void 0,void 0,function(){var n;return Object(o.__generator)(this,function(o){if(n=i.AuthToken.extractTokenClaims(e,this.cryptoUtils),!(null===(r=null==n?void 0:n.cnf)||void 0===r?void 0:r.kid))throw c.ClientAuthError.createTokenClaimsRequiredError();return[2,this.signPayload(e,n.cnf.kid,t)]})})},e.prototype.signPayload=function(e,t,r,n){return Object(o.__awaiter)(this,void 0,void 0,function(){var i,c,u,d,l,h;return Object(o.__generator)(this,function(p){switch(p.label){case 0:return i=r.resourceRequestMethod,c=r.resourceRequestUri,u=r.shrClaims,d=r.shrNonce,l=c?new a.UrlString(c):void 0,h=null==l?void 0:l.getUrlComponents(),[4,this.cryptoUtils.signJwt(Object(o.__assign)({at:e,ts:s.TimeUtils.nowSeconds(),m:null==i?void 0:i.toUpperCase(),u:null==h?void 0:h.HostNameAndPort,nonce:d||this.cryptoUtils.createNewGuid(),p:null==h?void 0:h.AbsolutePath,q:(null==h?void 0:h.QueryString)?[[],h.QueryString]:void 0,client_claims:u||void 0},n),t)];case 1:return[2,p.sent()]}})})},e}()},"./node_modules/@azure/msal-common/dist/error/AuthError.js":function(e,t,r){"use strict";r.r(t),r.d(t,"AuthError",function(){return s}),r.d(t,"AuthErrorMessage",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),i={unexpectedError:{code:"unexpected_error",desc:"Unexpected error in authentication."}},s=function(e){function t(r,n,i){var s=this,a=n?r+": "+n:r;return s=e.call(this,a)||this,Object.setPrototypeOf(s,t.prototype),s.errorCode=r||o.Constants.EMPTY_STRING,s.errorMessage=n||"",s.subError=i||"",s.name="AuthError",s}return Object(n.__extends)(t,e),t.prototype.setCorrelationId=function(e){this.correlationId=e},t.createUnexpectedError=function(e){return new t(i.unexpectedError.code,i.unexpectedError.desc+": "+e)},t}(Error)},"./node_modules/@azure/msal-common/dist/error/ClientAuthError.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ClientAuthError",function(){return s}),r.d(t,"ClientAuthErrorMessage",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/error/AuthError.js"),i={clientInfoDecodingError:{code:"client_info_decoding_error",desc:"The client info could not be parsed/decoded correctly. Please review the trace to determine the root cause."},clientInfoEmptyError:{code:"client_info_empty_error",desc:"The client info was empty. Please review the trace to determine the root cause."},tokenParsingError:{code:"token_parsing_error",desc:"Token cannot be parsed. Please review stack trace to determine root cause."},nullOrEmptyToken:{code:"null_or_empty_token",desc:"The token is null or empty. Please review the trace to determine the root cause."},endpointResolutionError:{code:"endpoints_resolution_error",desc:"Error: could not resolve endpoints. Please check network and try again."},networkError:{code:"network_error",desc:"Network request failed. Please check network trace to determine root cause."},unableToGetOpenidConfigError:{code:"openid_config_error",desc:"Could not retrieve endpoints. Check your authority and verify the .well-known/openid-configuration endpoint returns the required endpoints."},hashNotDeserialized:{code:"hash_not_deserialized",desc:"The hash parameters could not be deserialized. Please review the trace to determine the root cause."},blankGuidGenerated:{code:"blank_guid_generated",desc:"The guid generated was blank. Please review the trace to determine the root cause."},invalidStateError:{code:"invalid_state",desc:"State was not the expected format. Please check the logs to determine whether the request was sent using ProtocolUtils.setRequestState()."},stateMismatchError:{code:"state_mismatch",desc:"State mismatch error. Please check your network. Continued requests may cause cache overflow."},stateNotFoundError:{code:"state_not_found",desc:"State not found"},nonceMismatchError:{code:"nonce_mismatch",desc:"Nonce mismatch error. This may be caused by a race condition in concurrent requests."},nonceNotFoundError:{code:"nonce_not_found",desc:"nonce not found"},noTokensFoundError:{code:"no_tokens_found",desc:"No tokens were found for the given scopes, and no authorization code was passed to acquireToken. You must retrieve an authorization code before making a call to acquireToken()."},multipleMatchingTokens:{code:"multiple_matching_tokens",desc:"The cache contains multiple tokens satisfying the requirements. Call AcquireToken again providing more requirements such as authority or account."},multipleMatchingAccounts:{code:"multiple_matching_accounts",desc:"The cache contains multiple accounts satisfying the given parameters. Please pass more info to obtain the correct account"},multipleMatchingAppMetadata:{code:"multiple_matching_appMetadata",desc:"The cache contains multiple appMetadata satisfying the given parameters. Please pass more info to obtain the correct appMetadata"},tokenRequestCannotBeMade:{code:"request_cannot_be_made",desc:"Token request cannot be made without authorization code or refresh token."},appendEmptyScopeError:{code:"cannot_append_empty_scope",desc:"Cannot append null or empty scope to ScopeSet. Please check the stack trace for more info."},removeEmptyScopeError:{code:"cannot_remove_empty_scope",desc:"Cannot remove null or empty scope from ScopeSet. Please check the stack trace for more info."},appendScopeSetError:{code:"cannot_append_scopeset",desc:"Cannot append ScopeSet due to error."},emptyInputScopeSetError:{code:"empty_input_scopeset",desc:"Empty input ScopeSet cannot be processed."},DeviceCodePollingCancelled:{code:"device_code_polling_cancelled",desc:"Caller has cancelled token endpoint polling during device code flow by setting DeviceCodeRequest.cancel = true."},DeviceCodeExpired:{code:"device_code_expired",desc:"Device code is expired."},DeviceCodeUnknownError:{code:"device_code_unknown_error",desc:"Device code stopped polling for unknown reasons."},NoAccountInSilentRequest:{code:"no_account_in_silent_request",desc:"Please pass an account object, silent flow is not supported without account information"},invalidCacheRecord:{code:"invalid_cache_record",desc:"Cache record object was null or undefined."},invalidCacheEnvironment:{code:"invalid_cache_environment",desc:"Invalid environment when attempting to create cache entry"},noAccountFound:{code:"no_account_found",desc:"No account found in cache for given key."},CachePluginError:{code:"no cache plugin set on CacheManager",desc:"ICachePlugin needs to be set before using readFromStorage or writeFromStorage"},noCryptoObj:{code:"no_crypto_object",desc:"No crypto object detected. This is required for the following operation: "},invalidCacheType:{code:"invalid_cache_type",desc:"Invalid cache type"},unexpectedAccountType:{code:"unexpected_account_type",desc:"Unexpected account type."},unexpectedCredentialType:{code:"unexpected_credential_type",desc:"Unexpected credential type."},invalidAssertion:{code:"invalid_assertion",desc:"Client assertion must meet requirements described in https://tools.ietf.org/html/rfc7515"},invalidClientCredential:{code:"invalid_client_credential",desc:"Client credential (secret, certificate, or assertion) must not be empty when creating a confidential client. An application should at most have one credential"},tokenRefreshRequired:{code:"token_refresh_required",desc:"Cannot return token from cache because it must be refreshed. This may be due to one of the following reasons: forceRefresh parameter is set to true, claims have been requested, there is no cached access token or it is expired."},userTimeoutReached:{code:"user_timeout_reached",desc:"User defined timeout for device code polling reached"},tokenClaimsRequired:{code:"token_claims_cnf_required_for_signedjwt",desc:"Cannot generate a POP jwt if the token_claims are not populated"},noAuthorizationCodeFromServer:{code:"authorization_code_missing_from_server_response",desc:"Server response does not contain an authorization code to proceed"},noAzureRegionDetected:{code:"no_azure_region_detected",desc:"No azure region was detected and no fallback was made available"},accessTokenEntityNullError:{code:"access_token_entity_null",desc:"Access token entity is null, please check logs and cache to ensure a valid access token is present."},bindingKeyNotRemovedError:{code:"binding_key_not_removed",desc:"Could not remove the credential's binding key from storage."},logoutNotSupported:{code:"end_session_endpoint_not_supported",desc:"Provided authority does not support logout."}},s=function(e){function t(r,n){var o=e.call(this,r,n)||this;return o.name="ClientAuthError",Object.setPrototypeOf(o,t.prototype),o}return Object(n.__extends)(t,e),t.createClientInfoDecodingError=function(e){return new t(i.clientInfoDecodingError.code,i.clientInfoDecodingError.desc+" Failed with error: "+e)},t.createClientInfoEmptyError=function(){return new t(i.clientInfoEmptyError.code,""+i.clientInfoEmptyError.desc)},t.createTokenParsingError=function(e){return new t(i.tokenParsingError.code,i.tokenParsingError.desc+" Failed with error: "+e)},t.createTokenNullOrEmptyError=function(e){return new t(i.nullOrEmptyToken.code,i.nullOrEmptyToken.desc+" Raw Token Value: "+e)},t.createEndpointDiscoveryIncompleteError=function(e){return new t(i.endpointResolutionError.code,i.endpointResolutionError.desc+" Detail: "+e)},t.createNetworkError=function(e,r){return new t(i.networkError.code,i.networkError.desc+" | Fetch client threw: "+r+" | Attempted to reach: "+e.split("?")[0])},t.createUnableToGetOpenidConfigError=function(e){return new t(i.unableToGetOpenidConfigError.code,i.unableToGetOpenidConfigError.desc+" Attempted to retrieve endpoints from: "+e)},t.createHashNotDeserializedError=function(e){return new t(i.hashNotDeserialized.code,i.hashNotDeserialized.desc+" Given Object: "+e)},t.createInvalidStateError=function(e,r){return new t(i.invalidStateError.code,i.invalidStateError.desc+" Invalid State: "+e+", Root Err: "+r)},t.createStateMismatchError=function(){return new t(i.stateMismatchError.code,i.stateMismatchError.desc)},t.createStateNotFoundError=function(e){return new t(i.stateNotFoundError.code,i.stateNotFoundError.desc+":  "+e)},t.createNonceMismatchError=function(){return new t(i.nonceMismatchError.code,i.nonceMismatchError.desc)},t.createNonceNotFoundError=function(e){return new t(i.nonceNotFoundError.code,i.nonceNotFoundError.desc+":  "+e)},t.createMultipleMatchingTokensInCacheError=function(){return new t(i.multipleMatchingTokens.code,i.multipleMatchingTokens.desc+".")},t.createMultipleMatchingAccountsInCacheError=function(){return new t(i.multipleMatchingAccounts.code,i.multipleMatchingAccounts.desc)},t.createMultipleMatchingAppMetadataInCacheError=function(){return new t(i.multipleMatchingAppMetadata.code,i.multipleMatchingAppMetadata.desc)},t.createTokenRequestCannotBeMadeError=function(){return new t(i.tokenRequestCannotBeMade.code,i.tokenRequestCannotBeMade.desc)},t.createAppendEmptyScopeToSetError=function(e){return new t(i.appendEmptyScopeError.code,i.appendEmptyScopeError.desc+" Given Scope: "+e)},t.createRemoveEmptyScopeFromSetError=function(e){return new t(i.removeEmptyScopeError.code,i.removeEmptyScopeError.desc+" Given Scope: "+e)},t.createAppendScopeSetError=function(e){return new t(i.appendScopeSetError.code,i.appendScopeSetError.desc+" Detail Error: "+e)},t.createEmptyInputScopeSetError=function(){return new t(i.emptyInputScopeSetError.code,""+i.emptyInputScopeSetError.desc)},t.createDeviceCodeCancelledError=function(){return new t(i.DeviceCodePollingCancelled.code,""+i.DeviceCodePollingCancelled.desc)},t.createDeviceCodeExpiredError=function(){return new t(i.DeviceCodeExpired.code,""+i.DeviceCodeExpired.desc)},t.createDeviceCodeUnknownError=function(){return new t(i.DeviceCodeUnknownError.code,""+i.DeviceCodeUnknownError.desc)},t.createNoAccountInSilentRequestError=function(){return new t(i.NoAccountInSilentRequest.code,""+i.NoAccountInSilentRequest.desc)},t.createNullOrUndefinedCacheRecord=function(){return new t(i.invalidCacheRecord.code,i.invalidCacheRecord.desc)},t.createInvalidCacheEnvironmentError=function(){return new t(i.invalidCacheEnvironment.code,i.invalidCacheEnvironment.desc)},t.createNoAccountFoundError=function(){return new t(i.noAccountFound.code,i.noAccountFound.desc)},t.createCachePluginError=function(){return new t(i.CachePluginError.code,""+i.CachePluginError.desc)},t.createNoCryptoObjectError=function(e){return new t(i.noCryptoObj.code,""+i.noCryptoObj.desc+e)},t.createInvalidCacheTypeError=function(){return new t(i.invalidCacheType.code,""+i.invalidCacheType.desc)},t.createUnexpectedAccountTypeError=function(){return new t(i.unexpectedAccountType.code,""+i.unexpectedAccountType.desc)},t.createUnexpectedCredentialTypeError=function(){return new t(i.unexpectedCredentialType.code,""+i.unexpectedCredentialType.desc)},t.createInvalidAssertionError=function(){return new t(i.invalidAssertion.code,""+i.invalidAssertion.desc)},t.createInvalidCredentialError=function(){return new t(i.invalidClientCredential.code,""+i.invalidClientCredential.desc)},t.createRefreshRequiredError=function(){return new t(i.tokenRefreshRequired.code,i.tokenRefreshRequired.desc)},t.createUserTimeoutReachedError=function(){return new t(i.userTimeoutReached.code,i.userTimeoutReached.desc)},t.createTokenClaimsRequiredError=function(){return new t(i.tokenClaimsRequired.code,i.tokenClaimsRequired.desc)},t.createNoAuthCodeInServerResponseError=function(){return new t(i.noAuthorizationCodeFromServer.code,i.noAuthorizationCodeFromServer.desc)},t.createBindingKeyNotRemovedError=function(){return new t(i.bindingKeyNotRemovedError.code,i.bindingKeyNotRemovedError.desc)},t.createLogoutNotSupportedError=function(){return new t(i.logoutNotSupported.code,i.logoutNotSupported.desc)},t}(o.AuthError)},"./node_modules/@azure/msal-common/dist/error/ClientConfigurationError.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ClientConfigurationError",function(){return s}),r.d(t,"ClientConfigurationErrorMessage",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),i={redirectUriNotSet:{code:"redirect_uri_empty",desc:"A redirect URI is required for all calls, and none has been set."},postLogoutUriNotSet:{code:"post_logout_uri_empty",desc:"A post logout redirect has not been set."},claimsRequestParsingError:{code:"claims_request_parsing_error",desc:"Could not parse the given claims request object."},authorityUriInsecure:{code:"authority_uri_insecure",desc:"Authority URIs must use https.  Please see here for valid authority configuration options: https://docs.microsoft.com/en-us/azure/active-directory/develop/msal-js-initializing-client-applications#configuration-options"},urlParseError:{code:"url_parse_error",desc:"URL could not be parsed into appropriate segments."},urlEmptyError:{code:"empty_url_error",desc:"URL was empty or null."},emptyScopesError:{code:"empty_input_scopes_error",desc:"Scopes cannot be passed as null, undefined or empty array because they are required to obtain an access token."},nonArrayScopesError:{code:"nonarray_input_scopes_error",desc:"Scopes cannot be passed as non-array."},clientIdSingleScopeError:{code:"clientid_input_scopes_error",desc:"Client ID can only be provided as a single scope."},invalidPrompt:{code:"invalid_prompt_value",desc:"Supported prompt values are 'login', 'select_account', 'consent', 'create' and 'none'.  Please see here for valid configuration options: https://azuread.github.io/microsoft-authentication-library-for-js/ref/modules/_azure_msal_common.html#commonauthorizationurlrequest"},invalidClaimsRequest:{code:"invalid_claims",desc:"Given claims parameter must be a stringified JSON object."},tokenRequestEmptyError:{code:"token_request_empty",desc:"Token request was empty and not found in cache."},logoutRequestEmptyError:{code:"logout_request_empty",desc:"The logout request was null or undefined."},invalidCodeChallengeMethod:{code:"invalid_code_challenge_method",desc:'code_challenge_method passed is invalid. Valid values are "plain" and "S256".'},invalidCodeChallengeParams:{code:"pkce_params_missing",desc:"Both params: code_challenge and code_challenge_method are to be passed if to be sent in the request"},invalidCloudDiscoveryMetadata:{code:"invalid_cloud_discovery_metadata",desc:"Invalid cloudDiscoveryMetadata provided. Must be a stringified JSON object containing tenant_discovery_endpoint and metadata fields"},invalidAuthorityMetadata:{code:"invalid_authority_metadata",desc:"Invalid authorityMetadata provided. Must by a stringified JSON object containing authorization_endpoint, token_endpoint, issuer fields."},untrustedAuthority:{code:"untrusted_authority",desc:"The provided authority is not a trusted authority. Please include this authority in the knownAuthorities config parameter."},missingSshJwk:{code:"missing_ssh_jwk",desc:"Missing sshJwk in SSH certificate request. A stringified JSON Web Key is required when using the SSH authentication scheme."},missingSshKid:{code:"missing_ssh_kid",desc:"Missing sshKid in SSH certificate request. A string that uniquely identifies the public SSH key is required when using the SSH authentication scheme."},missingNonceAuthenticationHeader:{code:"missing_nonce_authentication_header",desc:"Unable to find an authentication header containing server nonce. Either the Authentication-Info or WWW-Authenticate headers must be present in order to obtain a server nonce."},invalidAuthenticationHeader:{code:"invalid_authentication_header",desc:"Invalid authentication header provided"}},s=function(e){function t(r,n){var o=e.call(this,r,n)||this;return o.name="ClientConfigurationError",Object.setPrototypeOf(o,t.prototype),o}return Object(n.__extends)(t,e),t.createRedirectUriEmptyError=function(){return new t(i.redirectUriNotSet.code,i.redirectUriNotSet.desc)},t.createPostLogoutRedirectUriEmptyError=function(){return new t(i.postLogoutUriNotSet.code,i.postLogoutUriNotSet.desc)},t.createClaimsRequestParsingError=function(e){return new t(i.claimsRequestParsingError.code,i.claimsRequestParsingError.desc+" Given value: "+e)},t.createInsecureAuthorityUriError=function(e){return new t(i.authorityUriInsecure.code,i.authorityUriInsecure.desc+" Given URI: "+e)},t.createUrlParseError=function(e){return new t(i.urlParseError.code,i.urlParseError.desc+" Given Error: "+e)},t.createUrlEmptyError=function(){return new t(i.urlEmptyError.code,i.urlEmptyError.desc)},t.createEmptyScopesArrayError=function(){return new t(i.emptyScopesError.code,""+i.emptyScopesError.desc)},t.createClientIdSingleScopeError=function(e){return new t(i.clientIdSingleScopeError.code,i.clientIdSingleScopeError.desc+" Given Scopes: "+e)},t.createInvalidPromptError=function(e){return new t(i.invalidPrompt.code,i.invalidPrompt.desc+" Given value: "+e)},t.createInvalidClaimsRequestError=function(){return new t(i.invalidClaimsRequest.code,i.invalidClaimsRequest.desc)},t.createEmptyLogoutRequestError=function(){return new t(i.logoutRequestEmptyError.code,i.logoutRequestEmptyError.desc)},t.createEmptyTokenRequestError=function(){return new t(i.tokenRequestEmptyError.code,i.tokenRequestEmptyError.desc)},t.createInvalidCodeChallengeMethodError=function(){return new t(i.invalidCodeChallengeMethod.code,i.invalidCodeChallengeMethod.desc)},t.createInvalidCodeChallengeParamsError=function(){return new t(i.invalidCodeChallengeParams.code,i.invalidCodeChallengeParams.desc)},t.createInvalidCloudDiscoveryMetadataError=function(){return new t(i.invalidCloudDiscoveryMetadata.code,i.invalidCloudDiscoveryMetadata.desc)},t.createInvalidAuthorityMetadataError=function(){return new t(i.invalidAuthorityMetadata.code,i.invalidAuthorityMetadata.desc)},t.createUntrustedAuthorityError=function(){return new t(i.untrustedAuthority.code,i.untrustedAuthority.desc)},t.createMissingSshJwkError=function(){return new t(i.missingSshJwk.code,i.missingSshJwk.desc)},t.createMissingSshKidError=function(){return new t(i.missingSshKid.code,i.missingSshKid.desc)},t.createMissingNonceAuthenticationHeadersError=function(){return new t(i.missingNonceAuthenticationHeader.code,i.missingNonceAuthenticationHeader.desc)},t.createInvalidAuthenticationHeaderError=function(e,r){return new t(i.invalidAuthenticationHeader.code,i.invalidAuthenticationHeader.desc+". Invalid header: "+e+". Details: "+r)},t}(o.ClientAuthError)},"./node_modules/@azure/msal-common/dist/error/InteractionRequiredAuthError.js":function(e,t,r){"use strict";r.r(t),r.d(t,"InteractionRequiredAuthError",function(){return c}),r.d(t,"InteractionRequiredAuthErrorMessage",function(){return a}),r.d(t,"InteractionRequiredAuthSubErrorMessage",function(){return s}),r.d(t,"InteractionRequiredServerErrorMessage",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/error/AuthError.js"),i=["interaction_required","consent_required","login_required"],s=["message_only","additional_action","basic_action","user_password_expired","consent_required"],a={noTokensFoundError:{code:"no_tokens_found",desc:"No refresh token found in the cache. Please sign-in."}},c=function(e){function t(r,n,o){var i=e.call(this,r,n,o)||this;return i.name="InteractionRequiredAuthError",Object.setPrototypeOf(i,t.prototype),i}return Object(n.__extends)(t,e),t.isInteractionRequiredError=function(e,t,r){var n=!!e&&i.indexOf(e)>-1,o=!!r&&s.indexOf(r)>-1,a=!!t&&i.some(function(e){return t.indexOf(e)>-1});return n||a||o},t.createNoTokensFoundError=function(){return new t(a.noTokensFoundError.code,a.noTokensFoundError.desc)},t}(o.AuthError)},"./node_modules/@azure/msal-common/dist/error/ServerError.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ServerError",function(){return o});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=function(e){function t(r,n,o){var i=e.call(this,r,n,o)||this;return i.name="ServerError",Object.setPrototypeOf(i,t.prototype),i}return Object(n.__extends)(t,e),t}(r("./node_modules/@azure/msal-common/dist/error/AuthError.js").AuthError)},"./node_modules/@azure/msal-common/dist/index.js":function(e,t,r){"use strict";r.r(t);var n=r("./node_modules/@azure/msal-common/dist/client/AuthorizationCodeClient.js");r.d(t,"AuthorizationCodeClient",function(){return n.AuthorizationCodeClient});var o=r("./node_modules/@azure/msal-common/dist/client/DeviceCodeClient.js");r.d(t,"DeviceCodeClient",function(){return o.DeviceCodeClient});var i=r("./node_modules/@azure/msal-common/dist/client/RefreshTokenClient.js");r.d(t,"RefreshTokenClient",function(){return i.RefreshTokenClient});var s=r("./node_modules/@azure/msal-common/dist/client/ClientCredentialClient.js");r.d(t,"ClientCredentialClient",function(){return s.ClientCredentialClient});var a=r("./node_modules/@azure/msal-common/dist/client/OnBehalfOfClient.js");r.d(t,"OnBehalfOfClient",function(){return a.OnBehalfOfClient});var c=r("./node_modules/@azure/msal-common/dist/client/SilentFlowClient.js");r.d(t,"SilentFlowClient",function(){return c.SilentFlowClient});var u=r("./node_modules/@azure/msal-common/dist/client/UsernamePasswordClient.js");r.d(t,"UsernamePasswordClient",function(){return u.UsernamePasswordClient});var d=r("./node_modules/@azure/msal-common/dist/config/ClientConfiguration.js");r.d(t,"DEFAULT_SYSTEM_OPTIONS",function(){return d.DEFAULT_SYSTEM_OPTIONS});var l=r("./node_modules/@azure/msal-common/dist/account/AuthToken.js");r.d(t,"AuthToken",function(){return l.AuthToken}),r.d(t,"IdToken",function(){return l.AuthToken});var h=r("./node_modules/@azure/msal-common/dist/account/CcsCredential.js");r.d(t,"CcsCredentialType",function(){return h.CcsCredentialType});var p=r("./node_modules/@azure/msal-common/dist/authority/Authority.js");r.d(t,"Authority",function(){return p.Authority});var m=r("./node_modules/@azure/msal-common/dist/authority/AuthorityFactory.js");r.d(t,"AuthorityFactory",function(){return m.AuthorityFactory});var g=r("./node_modules/@azure/msal-common/dist/authority/AuthorityType.js");r.d(t,"AuthorityType",function(){return g.AuthorityType});var f=r("./node_modules/@azure/msal-common/dist/authority/ProtocolMode.js");r.d(t,"ProtocolMode",function(){return f.ProtocolMode});var y=r("./node_modules/@azure/msal-common/dist/cache/CacheManager.js");r.d(t,"CacheManager",function(){return y.CacheManager}),r.d(t,"DefaultStorageClass",function(){return y.DefaultStorageClass});var C=r("./node_modules/@azure/msal-common/dist/cache/entities/CredentialEntity.js");r.d(t,"CredentialEntity",function(){return C.CredentialEntity});var E=r("./node_modules/@azure/msal-common/dist/cache/entities/AppMetadataEntity.js");r.d(t,"AppMetadataEntity",function(){return E.AppMetadataEntity});var _=r("./node_modules/@azure/msal-common/dist/cache/entities/AccountEntity.js");r.d(t,"AccountEntity",function(){return _.AccountEntity});var v=r("./node_modules/@azure/msal-common/dist/cache/entities/IdTokenEntity.js");r.d(t,"IdTokenEntity",function(){return v.IdTokenEntity});var T=r("./node_modules/@azure/msal-common/dist/cache/entities/AccessTokenEntity.js");r.d(t,"AccessTokenEntity",function(){return T.AccessTokenEntity});var A=r("./node_modules/@azure/msal-common/dist/cache/entities/RefreshTokenEntity.js");r.d(t,"RefreshTokenEntity",function(){return A.RefreshTokenEntity});var w=r("./node_modules/@azure/msal-common/dist/cache/entities/ServerTelemetryEntity.js");r.d(t,"ServerTelemetryEntity",function(){return w.ServerTelemetryEntity});var S=r("./node_modules/@azure/msal-common/dist/cache/entities/AuthorityMetadataEntity.js");r.d(t,"AuthorityMetadataEntity",function(){return S.AuthorityMetadataEntity});var b=r("./node_modules/@azure/msal-common/dist/cache/entities/ThrottlingEntity.js");r.d(t,"ThrottlingEntity",function(){return b.ThrottlingEntity});var I=r("./node_modules/@azure/msal-common/dist/cache/persistence/TokenCacheContext.js");r.d(t,"TokenCacheContext",function(){return I.TokenCacheContext});var R=r("./node_modules/@azure/msal-common/dist/network/INetworkModule.js");r.d(t,"StubbedNetworkModule",function(){return R.StubbedNetworkModule});var O=r("./node_modules/@azure/msal-common/dist/network/NetworkManager.js");r.d(t,"NetworkManager",function(){return O.NetworkManager});var k=r("./node_modules/@azure/msal-common/dist/network/ThrottlingUtils.js");r.d(t,"ThrottlingUtils",function(){return k.ThrottlingUtils});var P=r("./node_modules/@azure/msal-common/dist/url/UrlString.js");r.d(t,"UrlString",function(){return P.UrlString});var U=r("./node_modules/@azure/msal-common/dist/crypto/ICrypto.js");r.d(t,"DEFAULT_CRYPTO_IMPLEMENTATION",function(){return U.DEFAULT_CRYPTO_IMPLEMENTATION});var j=r("./node_modules/@azure/msal-common/dist/request/ScopeSet.js");r.d(t,"ScopeSet",function(){return j.ScopeSet});var N=r("./node_modules/@azure/msal-common/dist/request/AuthenticationHeaderParser.js");r.d(t,"AuthenticationHeaderParser",function(){return N.AuthenticationHeaderParser});var M=r("./node_modules/@azure/msal-common/dist/logger/Logger.js");r.d(t,"LogLevel",function(){return M.LogLevel}),r.d(t,"Logger",function(){return M.Logger});var L=r("./node_modules/@azure/msal-common/dist/error/InteractionRequiredAuthError.js");r.d(t,"InteractionRequiredAuthError",function(){return L.InteractionRequiredAuthError}),r.d(t,"InteractionRequiredAuthErrorMessage",function(){return L.InteractionRequiredAuthErrorMessage});var z=r("./node_modules/@azure/msal-common/dist/error/AuthError.js");r.d(t,"AuthError",function(){return z.AuthError}),r.d(t,"AuthErrorMessage",function(){return z.AuthErrorMessage});var D=r("./node_modules/@azure/msal-common/dist/error/ServerError.js");r.d(t,"ServerError",function(){return D.ServerError});var q=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js");r.d(t,"ClientAuthError",function(){return q.ClientAuthError}),r.d(t,"ClientAuthErrorMessage",function(){return q.ClientAuthErrorMessage});var H=r("./node_modules/@azure/msal-common/dist/error/ClientConfigurationError.js");r.d(t,"ClientConfigurationError",function(){return H.ClientConfigurationError}),r.d(t,"ClientConfigurationErrorMessage",function(){return H.ClientConfigurationErrorMessage});var B=r("./node_modules/@azure/msal-common/dist/utils/Constants.js");r.d(t,"AuthenticationScheme",function(){return B.AuthenticationScheme}),r.d(t,"CacheAccountType",function(){return B.CacheAccountType}),r.d(t,"CacheSchemaType",function(){return B.CacheSchemaType}),r.d(t,"CacheType",function(){return B.CacheType}),r.d(t,"Constants",function(){return B.Constants}),r.d(t,"CredentialType",function(){return B.CredentialType}),r.d(t,"OIDC_DEFAULT_SCOPES",function(){return B.OIDC_DEFAULT_SCOPES}),r.d(t,"PersistentCacheKeys",function(){return B.PersistentCacheKeys}),r.d(t,"PromptValue",function(){return B.PromptValue}),r.d(t,"ResponseMode",function(){return B.ResponseMode});var K=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js");r.d(t,"StringUtils",function(){return K.StringUtils});var F=r("./node_modules/@azure/msal-common/dist/utils/ProtocolUtils.js");r.d(t,"ProtocolUtils",function(){return F.ProtocolUtils});var x=r("./node_modules/@azure/msal-common/dist/utils/TimeUtils.js");r.d(t,"TimeUtils",function(){return x.TimeUtils});var G=r("./node_modules/@azure/msal-common/dist/telemetry/server/ServerTelemetryManager.js");r.d(t,"ServerTelemetryManager",function(){return G.ServerTelemetryManager});var W=r("./node_modules/@azure/msal-common/dist/crypto/PopTokenGenerator.js");r.d(t,"PopTokenGenerator",function(){return W.PopTokenGenerator});var Y=r("./node_modules/@azure/msal-common/dist/packageMetadata.js");r.d(t,"version",function(){return Y.version})},"./node_modules/@azure/msal-common/dist/logger/Logger.js":function(e,t,r){"use strict";r.r(t),r.d(t,"LogLevel",function(){return n}),r.d(t,"Logger",function(){return s});var n,o=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),i=r("./node_modules/@azure/msal-common/dist/utils/Constants.js");!function(e){e[e.Error=0]="Error",e[e.Warning=1]="Warning",e[e.Info=2]="Info",e[e.Verbose=3]="Verbose",e[e.Trace=4]="Trace"}(n||(n={}));var s=function(){function e(e,t,r){this.level=n.Info;this.localCallback=e.loggerCallback||function(){},this.piiLoggingEnabled=e.piiLoggingEnabled||!1,this.level="number"==typeof e.logLevel?e.logLevel:n.Info,this.correlationId=e.correlationId||"",this.packageName=t||i.Constants.EMPTY_STRING,this.packageVersion=r||i.Constants.EMPTY_STRING}return e.prototype.clone=function(t,r,n){return new e({loggerCallback:this.localCallback,piiLoggingEnabled:this.piiLoggingEnabled,logLevel:this.level,correlationId:n||this.correlationId},t,r)},e.prototype.logMessage=function(e,t){if(!(t.logLevel>this.level||!this.piiLoggingEnabled&&t.containsPii)){var r=(new Date).toUTCString(),i=(o.StringUtils.isEmpty(t.correlationId)?o.StringUtils.isEmpty(this.correlationId)?"["+r+"]":"["+r+"] : ["+this.correlationId+"]":"["+r+"] : ["+t.correlationId+"]")+" : "+this.packageName+"@"+this.packageVersion+" : "+n[t.logLevel]+" - "+e;this.executeCallback(t.logLevel,i,t.containsPii||!1)}},e.prototype.executeCallback=function(e,t,r){this.localCallback&&this.localCallback(e,t,r)},e.prototype.error=function(e,t){this.logMessage(e,{logLevel:n.Error,containsPii:!1,correlationId:t||""})},e.prototype.errorPii=function(e,t){this.logMessage(e,{logLevel:n.Error,containsPii:!0,correlationId:t||""})},e.prototype.warning=function(e,t){this.logMessage(e,{logLevel:n.Warning,containsPii:!1,correlationId:t||""})},e.prototype.warningPii=function(e,t){this.logMessage(e,{logLevel:n.Warning,containsPii:!0,correlationId:t||""})},e.prototype.info=function(e,t){this.logMessage(e,{logLevel:n.Info,containsPii:!1,correlationId:t||""})},e.prototype.infoPii=function(e,t){this.logMessage(e,{logLevel:n.Info,containsPii:!0,correlationId:t||""})},e.prototype.verbose=function(e,t){this.logMessage(e,{logLevel:n.Verbose,containsPii:!1,correlationId:t||""})},e.prototype.verbosePii=function(e,t){this.logMessage(e,{logLevel:n.Verbose,containsPii:!0,correlationId:t||""})},e.prototype.trace=function(e,t){this.logMessage(e,{logLevel:n.Trace,containsPii:!1,correlationId:t||""})},e.prototype.tracePii=function(e,t){this.logMessage(e,{logLevel:n.Trace,containsPii:!0,correlationId:t||""})},e.prototype.isPiiLoggingEnabled=function(){return this.piiLoggingEnabled||!1},e}()},"./node_modules/@azure/msal-common/dist/network/INetworkModule.js":function(e,t,r){"use strict";r.r(t),r.d(t,"StubbedNetworkModule",function(){return o});var n=r("./node_modules/@azure/msal-common/dist/error/AuthError.js"),o={sendGetRequestAsync:function(){return Promise.reject(n.AuthError.createUnexpectedError("Network interface - sendGetRequestAsync() has not been implemented for the Network interface."))},sendPostRequestAsync:function(){return Promise.reject(n.AuthError.createUnexpectedError("Network interface - sendPostRequestAsync() has not been implemented for the Network interface."))}};
/*! @azure/msal-common v5.1.0 2021-11-02 */},"./node_modules/@azure/msal-common/dist/network/NetworkManager.js":function(e,t,r){"use strict";r.r(t),r.d(t,"NetworkManager",function(){return a});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/network/ThrottlingUtils.js"),i=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),s=r("./node_modules/@azure/msal-common/dist/error/AuthError.js"),a=function(){function e(e,t){this.networkClient=e,this.cacheManager=t}return e.prototype.sendPostRequest=function(e,t,r){return Object(n.__awaiter)(this,void 0,void 0,function(){var a,c;return Object(n.__generator)(this,function(n){switch(n.label){case 0:o.ThrottlingUtils.preProcess(this.cacheManager,e),n.label=1;case 1:return n.trys.push([1,3,,4]),[4,this.networkClient.sendPostRequestAsync(t,r)];case 2:return a=n.sent(),[3,4];case 3:throw(c=n.sent())instanceof s.AuthError?c:i.ClientAuthError.createNetworkError(t,c);case 4:return o.ThrottlingUtils.postProcess(this.cacheManager,e,a),[2,a]}})})},e}()},"./node_modules/@azure/msal-common/dist/network/ThrottlingUtils.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ThrottlingUtils",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),o=r("./node_modules/@azure/msal-common/dist/error/ServerError.js"),i=function(){function e(){}return e.generateThrottlingStorageKey=function(e){return n.ThrottlingConstants.THROTTLING_PREFIX+"."+JSON.stringify(e)},e.preProcess=function(t,r){var i,s=e.generateThrottlingStorageKey(r),a=t.getThrottlingCache(s);if(a){if(a.throttleTime<Date.now())return void t.removeItem(s,n.CacheSchemaType.THROTTLING);throw new o.ServerError((null===(i=a.errorCodes)||void 0===i?void 0:i.join(" "))||n.Constants.EMPTY_STRING,a.errorMessage,a.subError)}},e.postProcess=function(t,r,o){if(e.checkResponseStatus(o)||e.checkResponseForRetryAfter(o)){var i={throttleTime:e.calculateThrottleTime(parseInt(o.headers[n.HeaderNames.RETRY_AFTER])),error:o.body.error,errorCodes:o.body.error_codes,errorMessage:o.body.error_description,subError:o.body.suberror};t.setThrottlingCache(e.generateThrottlingStorageKey(r),i)}},e.checkResponseStatus=function(e){return 429===e.status||e.status>=500&&e.status<600},e.checkResponseForRetryAfter=function(e){return!!e.headers&&(e.headers.hasOwnProperty(n.HeaderNames.RETRY_AFTER)&&(e.status<200||e.status>=300))},e.calculateThrottleTime=function(e){var t=e<=0?0:e,r=Date.now()/1e3;return Math.floor(1e3*Math.min(r+(t||n.ThrottlingConstants.DEFAULT_THROTTLE_TIME_SECONDS),r+n.ThrottlingConstants.DEFAULT_MAX_THROTTLE_TIME_SECONDS))},e.removeThrottle=function(e,t,r,o){var i={clientId:t,authority:r.authority,scopes:r.scopes,homeAccountIdentifier:o,authenticationScheme:r.authenticationScheme,resourceRequestMethod:r.resourceRequestMethod,resourceRequestUri:r.resourceRequestUri,shrClaims:r.shrClaims,sshJwk:r.sshJwk,sshKid:r.sshKid},s=this.generateThrottlingStorageKey(i);return e.removeItem(s,n.CacheSchemaType.THROTTLING)},e}()},"./node_modules/@azure/msal-common/dist/packageMetadata.js":function(e,t,r){"use strict";r.r(t),r.d(t,"name",function(){return n}),r.d(t,"version",function(){return o});
/*! @azure/msal-common v5.1.0 2021-11-02 */
var n="@azure/msal-common",o="5.1.0"},"./node_modules/@azure/msal-common/dist/request/AuthenticationHeaderParser.js":function(e,t,r){"use strict";r.r(t),r.d(t,"AuthenticationHeaderParser",function(){return i});var n=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),o=r("./node_modules/@azure/msal-common/dist/error/ClientConfigurationError.js"),i=function(){function e(e){this.headers=e}return e.prototype.getShrNonce=function(){var e=this.headers[n.HeaderNames.AuthenticationInfo];if(e){var t=this.parseChallenges(e);if(t.nextnonce)return t.nextnonce;throw o.ClientConfigurationError.createInvalidAuthenticationHeaderError(n.HeaderNames.AuthenticationInfo,"nextnonce challenge is missing.")}var r=this.headers[n.HeaderNames.WWWAuthenticate];if(r){var i=this.parseChallenges(r);if(i.nonce)return i.nonce;throw o.ClientConfigurationError.createInvalidAuthenticationHeaderError(n.HeaderNames.WWWAuthenticate,"nonce challenge is missing.")}throw o.ClientConfigurationError.createMissingNonceAuthenticationHeadersError()},e.prototype.parseChallenges=function(e){var t=e.indexOf(" "),r=e.substr(t+1).split(","),n={};return r.forEach(function(e){var t=e.split("="),r=t[0],o=t[1];n[r]=unescape(o.replace(/['"]+/g,""))}),n},e}()},"./node_modules/@azure/msal-common/dist/request/RequestParameterBuilder.js":function(e,t,r){"use strict";r.r(t),r.d(t,"RequestParameterBuilder",function(){return u});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),i=r("./node_modules/@azure/msal-common/dist/request/ScopeSet.js"),s=r("./node_modules/@azure/msal-common/dist/error/ClientConfigurationError.js"),a=r("./node_modules/@azure/msal-common/dist/request/RequestValidator.js"),c=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),u=function(){function e(){this.parameters=new Map}return e.prototype.addResponseTypeCode=function(){this.parameters.set(o.AADServerParamKeys.RESPONSE_TYPE,encodeURIComponent(o.Constants.CODE_RESPONSE_TYPE))},e.prototype.addResponseMode=function(e){this.parameters.set(o.AADServerParamKeys.RESPONSE_MODE,encodeURIComponent(e||o.ResponseMode.QUERY))},e.prototype.addScopes=function(e,t){void 0===t&&(t=!0);var r=t?Object(n.__spreadArrays)(e||[],o.OIDC_DEFAULT_SCOPES):e||[],s=new i.ScopeSet(r);this.parameters.set(o.AADServerParamKeys.SCOPE,encodeURIComponent(s.printScopes()))},e.prototype.addClientId=function(e){this.parameters.set(o.AADServerParamKeys.CLIENT_ID,encodeURIComponent(e))},e.prototype.addRedirectUri=function(e){a.RequestValidator.validateRedirectUri(e),this.parameters.set(o.AADServerParamKeys.REDIRECT_URI,encodeURIComponent(e))},e.prototype.addPostLogoutRedirectUri=function(e){a.RequestValidator.validateRedirectUri(e),this.parameters.set(o.AADServerParamKeys.POST_LOGOUT_URI,encodeURIComponent(e))},e.prototype.addIdTokenHint=function(e){this.parameters.set(o.AADServerParamKeys.ID_TOKEN_HINT,encodeURIComponent(e))},e.prototype.addDomainHint=function(e){this.parameters.set(o.SSOTypes.DOMAIN_HINT,encodeURIComponent(e))},e.prototype.addLoginHint=function(e){this.parameters.set(o.SSOTypes.LOGIN_HINT,encodeURIComponent(e))},e.prototype.addCcsUpn=function(e){this.parameters.set(o.HeaderNames.CCS_HEADER,encodeURIComponent("UPN:"+e))},e.prototype.addCcsOid=function(e){this.parameters.set(o.HeaderNames.CCS_HEADER,encodeURIComponent("Oid:"+e.uid+"@"+e.utid))},e.prototype.addSid=function(e){this.parameters.set(o.SSOTypes.SID,encodeURIComponent(e))},e.prototype.addClaims=function(e,t){var r=this.addClientCapabilitiesToClaims(e,t);a.RequestValidator.validateClaims(r),this.parameters.set(o.AADServerParamKeys.CLAIMS,encodeURIComponent(r))},e.prototype.addCorrelationId=function(e){this.parameters.set(o.AADServerParamKeys.CLIENT_REQUEST_ID,encodeURIComponent(e))},e.prototype.addLibraryInfo=function(e){this.parameters.set(o.AADServerParamKeys.X_CLIENT_SKU,e.sku),this.parameters.set(o.AADServerParamKeys.X_CLIENT_VER,e.version),this.parameters.set(o.AADServerParamKeys.X_CLIENT_OS,e.os),this.parameters.set(o.AADServerParamKeys.X_CLIENT_CPU,e.cpu)},e.prototype.addPrompt=function(e){a.RequestValidator.validatePrompt(e),this.parameters.set(""+o.AADServerParamKeys.PROMPT,encodeURIComponent(e))},e.prototype.addState=function(e){c.StringUtils.isEmpty(e)||this.parameters.set(o.AADServerParamKeys.STATE,encodeURIComponent(e))},e.prototype.addNonce=function(e){this.parameters.set(o.AADServerParamKeys.NONCE,encodeURIComponent(e))},e.prototype.addCodeChallengeParams=function(e,t){if(a.RequestValidator.validateCodeChallengeParams(e,t),!e||!t)throw s.ClientConfigurationError.createInvalidCodeChallengeParamsError();this.parameters.set(o.AADServerParamKeys.CODE_CHALLENGE,encodeURIComponent(e)),this.parameters.set(o.AADServerParamKeys.CODE_CHALLENGE_METHOD,encodeURIComponent(t))},e.prototype.addAuthorizationCode=function(e){this.parameters.set(o.AADServerParamKeys.CODE,encodeURIComponent(e))},e.prototype.addDeviceCode=function(e){this.parameters.set(o.AADServerParamKeys.DEVICE_CODE,encodeURIComponent(e))},e.prototype.addRefreshToken=function(e){this.parameters.set(o.AADServerParamKeys.REFRESH_TOKEN,encodeURIComponent(e))},e.prototype.addCodeVerifier=function(e){this.parameters.set(o.AADServerParamKeys.CODE_VERIFIER,encodeURIComponent(e))},e.prototype.addClientSecret=function(e){this.parameters.set(o.AADServerParamKeys.CLIENT_SECRET,encodeURIComponent(e))},e.prototype.addClientAssertion=function(e){this.parameters.set(o.AADServerParamKeys.CLIENT_ASSERTION,encodeURIComponent(e))},e.prototype.addClientAssertionType=function(e){this.parameters.set(o.AADServerParamKeys.CLIENT_ASSERTION_TYPE,encodeURIComponent(e))},e.prototype.addOboAssertion=function(e){this.parameters.set(o.AADServerParamKeys.OBO_ASSERTION,encodeURIComponent(e))},e.prototype.addRequestTokenUse=function(e){this.parameters.set(o.AADServerParamKeys.REQUESTED_TOKEN_USE,encodeURIComponent(e))},e.prototype.addGrantType=function(e){this.parameters.set(o.AADServerParamKeys.GRANT_TYPE,encodeURIComponent(e))},e.prototype.addClientInfo=function(){this.parameters.set(o.CLIENT_INFO,"1")},e.prototype.addExtraQueryParameters=function(e){var t=this;a.RequestValidator.sanitizeEQParams(e,this.parameters),Object.keys(e).forEach(function(r){t.parameters.set(r,e[r])})},e.prototype.addClientCapabilitiesToClaims=function(e,t){var r;if(e)try{r=JSON.parse(e)}catch(e){throw s.ClientConfigurationError.createInvalidClaimsRequestError()}else r={};return t&&t.length>0&&(r.hasOwnProperty(o.ClaimsRequestKeys.ACCESS_TOKEN)||(r[o.ClaimsRequestKeys.ACCESS_TOKEN]={}),r[o.ClaimsRequestKeys.ACCESS_TOKEN][o.ClaimsRequestKeys.XMS_CC]={values:t}),JSON.stringify(r)},e.prototype.addUsername=function(e){this.parameters.set(o.PasswordGrantConstants.username,e)},e.prototype.addPassword=function(e){this.parameters.set(o.PasswordGrantConstants.password,e)},e.prototype.addPopToken=function(e){c.StringUtils.isEmpty(e)||(this.parameters.set(o.AADServerParamKeys.TOKEN_TYPE,o.AuthenticationScheme.POP),this.parameters.set(o.AADServerParamKeys.REQ_CNF,encodeURIComponent(e)))},e.prototype.addSshJwk=function(e){c.StringUtils.isEmpty(e)||(this.parameters.set(o.AADServerParamKeys.TOKEN_TYPE,o.AuthenticationScheme.SSH),this.parameters.set(o.AADServerParamKeys.REQ_CNF,encodeURIComponent(e)))},e.prototype.addServerTelemetry=function(e){this.parameters.set(o.AADServerParamKeys.X_CLIENT_CURR_TELEM,e.generateCurrentRequestHeaderValue()),this.parameters.set(o.AADServerParamKeys.X_CLIENT_LAST_TELEM,e.generateLastRequestHeaderValue())},e.prototype.addThrottling=function(){this.parameters.set(o.AADServerParamKeys.X_MS_LIB_CAPABILITY,o.ThrottlingConstants.X_MS_LIB_CAPABILITY_VALUE)},e.prototype.createQueryString=function(){var e=new Array;return this.parameters.forEach(function(t,r){e.push(r+"="+t)}),e.join("&")},e}()},"./node_modules/@azure/msal-common/dist/request/RequestValidator.js":function(e,t,r){"use strict";r.r(t),r.d(t,"RequestValidator",function(){return s});var n=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),o=r("./node_modules/@azure/msal-common/dist/error/ClientConfigurationError.js"),i=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),s=function(){function e(){}return e.validateRedirectUri=function(e){if(n.StringUtils.isEmpty(e))throw o.ClientConfigurationError.createRedirectUriEmptyError()},e.validatePrompt=function(e){var t=[];for(var r in i.PromptValue)t.push(i.PromptValue[r]);if(t.indexOf(e)<0)throw o.ClientConfigurationError.createInvalidPromptError(e)},e.validateClaims=function(e){try{JSON.parse(e)}catch(e){throw o.ClientConfigurationError.createInvalidClaimsRequestError()}},e.validateCodeChallengeParams=function(e,t){if(n.StringUtils.isEmpty(e)||n.StringUtils.isEmpty(t))throw o.ClientConfigurationError.createInvalidCodeChallengeParamsError();this.validateCodeChallengeMethod(t)},e.validateCodeChallengeMethod=function(e){if([i.CodeChallengeMethodValues.PLAIN,i.CodeChallengeMethodValues.S256].indexOf(e)<0)throw o.ClientConfigurationError.createInvalidCodeChallengeMethodError()},e.sanitizeEQParams=function(e,t){return e?(t.forEach(function(t,r){e[r]&&delete e[r]}),e):{}},e}()},"./node_modules/@azure/msal-common/dist/request/ScopeSet.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ScopeSet",function(){return c});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/error/ClientConfigurationError.js"),i=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),s=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),a=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),c=function(){function e(e){var t=this,r=e?i.StringUtils.trimArrayEntries(Object(n.__spreadArrays)(e)):[],o=r?i.StringUtils.removeEmptyStringsFromArray(r):[];this.validateInputScopes(o),this.scopes=new Set,o.forEach(function(e){return t.scopes.add(e)})}return e.fromString=function(t){return new e((t||"").split(" "))},e.prototype.validateInputScopes=function(e){if(!e||e.length<1)throw o.ClientConfigurationError.createEmptyScopesArrayError()},e.prototype.containsScope=function(t){var r=new e(this.printScopesLowerCase().split(" "));return!i.StringUtils.isEmpty(t)&&r.scopes.has(t.toLowerCase())},e.prototype.containsScopeSet=function(e){var t=this;return!(!e||e.scopes.size<=0)&&(this.scopes.size>=e.scopes.size&&e.asArray().every(function(e){return t.containsScope(e)}))},e.prototype.containsOnlyOIDCScopes=function(){var e=this,t=0;return a.OIDC_SCOPES.forEach(function(r){e.containsScope(r)&&(t+=1)}),this.scopes.size===t},e.prototype.appendScope=function(e){i.StringUtils.isEmpty(e)||this.scopes.add(e.trim())},e.prototype.appendScopes=function(e){var t=this;try{e.forEach(function(e){return t.appendScope(e)})}catch(e){throw s.ClientAuthError.createAppendScopeSetError(e)}},e.prototype.removeScope=function(e){if(i.StringUtils.isEmpty(e))throw s.ClientAuthError.createRemoveEmptyScopeFromSetError(e);this.scopes.delete(e.trim())},e.prototype.removeOIDCScopes=function(){var e=this;a.OIDC_SCOPES.forEach(function(t){e.scopes.delete(t)})},e.prototype.unionScopeSets=function(e){if(!e)throw s.ClientAuthError.createEmptyInputScopeSetError();var t=new Set;return e.scopes.forEach(function(e){return t.add(e.toLowerCase())}),this.scopes.forEach(function(e){return t.add(e.toLowerCase())}),t},e.prototype.intersectingScopeSets=function(e){if(!e)throw s.ClientAuthError.createEmptyInputScopeSetError();e.containsOnlyOIDCScopes()||e.removeOIDCScopes();var t=this.unionScopeSets(e),r=e.getScopeCount(),n=this.getScopeCount();return t.size<n+r},e.prototype.getScopeCount=function(){return this.scopes.size},e.prototype.asArray=function(){var e=[];return this.scopes.forEach(function(t){return e.push(t)}),e},e.prototype.printScopes=function(){return this.scopes?this.asArray().join(" "):""},e.prototype.printScopesLowerCase=function(){return this.printScopes().toLowerCase()},e}()},"./node_modules/@azure/msal-common/dist/response/ResponseHandler.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ResponseHandler",function(){return T});var n=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),o=r("./node_modules/@azure/msal-common/dist/account/ClientInfo.js"),i=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),s=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),a=r("./node_modules/@azure/msal-common/dist/error/ServerError.js"),c=r("./node_modules/@azure/msal-common/dist/account/AuthToken.js"),u=r("./node_modules/@azure/msal-common/dist/request/ScopeSet.js"),d=r("./node_modules/@azure/msal-common/dist/cache/entities/AccountEntity.js"),l=r("./node_modules/@azure/msal-common/dist/authority/AuthorityType.js"),h=r("./node_modules/@azure/msal-common/dist/cache/entities/IdTokenEntity.js"),p=r("./node_modules/@azure/msal-common/dist/cache/entities/AccessTokenEntity.js"),m=r("./node_modules/@azure/msal-common/dist/cache/entities/RefreshTokenEntity.js"),g=r("./node_modules/@azure/msal-common/dist/error/InteractionRequiredAuthError.js"),f=r("./node_modules/@azure/msal-common/dist/cache/entities/CacheRecord.js"),y=r("./node_modules/@azure/msal-common/dist/utils/ProtocolUtils.js"),C=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),E=r("./node_modules/@azure/msal-common/dist/crypto/PopTokenGenerator.js"),_=r("./node_modules/@azure/msal-common/dist/cache/entities/AppMetadataEntity.js"),v=r("./node_modules/@azure/msal-common/dist/cache/persistence/TokenCacheContext.js"),T=function(){function e(e,t,r,n,o,i){this.clientId=e,this.cacheStorage=t,this.cryptoObj=r,this.logger=n,this.serializableCache=o,this.persistencePlugin=i}return e.prototype.validateServerAuthorizationCodeResponse=function(e,t,r){if(!e.state||!t)throw e.state?i.ClientAuthError.createStateNotFoundError("Cached State"):i.ClientAuthError.createStateNotFoundError("Server State");if(decodeURIComponent(e.state)!==decodeURIComponent(t))throw i.ClientAuthError.createStateMismatchError();if(e.error||e.error_description||e.suberror){if(g.InteractionRequiredAuthError.isInteractionRequiredError(e.error,e.error_description,e.suberror))throw new g.InteractionRequiredAuthError(e.error||C.Constants.EMPTY_STRING,e.error_description,e.suberror);throw new a.ServerError(e.error||C.Constants.EMPTY_STRING,e.error_description,e.suberror)}e.client_info&&Object(o.buildClientInfo)(e.client_info,r)},e.prototype.validateTokenResponse=function(e){if(e.error||e.error_description||e.suberror){if(g.InteractionRequiredAuthError.isInteractionRequiredError(e.error,e.error_description,e.suberror))throw new g.InteractionRequiredAuthError(e.error,e.error_description,e.suberror);var t=e.error_codes+" - ["+e.timestamp+"]: "+e.error_description+" - Correlation ID: "+e.correlation_id+" - Trace ID: "+e.trace_id;throw new a.ServerError(e.error,t,e.suberror)}},e.prototype.handleServerTokenResponse=function(t,r,o,a,u,l,h){return Object(n.__awaiter)(this,void 0,void 0,function(){var p,m,g,f,E;return Object(n.__generator)(this,function(n){switch(n.label){case 0:if(t.id_token&&(p=new c.AuthToken(t.id_token||C.Constants.EMPTY_STRING,this.cryptoObj),u&&!s.StringUtils.isEmpty(u.nonce)&&p.claims.nonce!==u.nonce))throw i.ClientAuthError.createNonceMismatchError();this.homeAccountIdentifier=d.AccountEntity.generateHomeAccountId(t.client_info||C.Constants.EMPTY_STRING,r.authorityType,this.logger,this.cryptoObj,p),u&&u.state&&(m=y.ProtocolUtils.parseRequestState(this.cryptoObj,u.state)),t.key_id=t.key_id||a.sshKid||void 0,g=this.generateCacheRecord(t,r,o,p,a.scopes,l,u),n.label=1;case 1:return n.trys.push([1,,5,8]),this.persistencePlugin&&this.serializableCache?(this.logger.verbose("Persistence enabled, calling beforeCacheAccess"),f=new v.TokenCacheContext(this.serializableCache,!0),[4,this.persistencePlugin.beforeCacheAccess(f)]):[3,3];case 2:n.sent(),n.label=3;case 3:return!h||!g.account||(E=g.account.generateAccountKey(),this.cacheStorage.getAccount(E))?[4,this.cacheStorage.saveCacheRecord(g)]:(this.logger.warning("Account used to refresh tokens not in persistence, refreshed tokens will not be stored in the cache"),[2,e.generateAuthenticationResult(this.cryptoObj,r,g,!1,a,p,m)]);case 4:return n.sent(),[3,8];case 5:return this.persistencePlugin&&this.serializableCache&&f?(this.logger.verbose("Persistence enabled, calling afterCacheAccess"),[4,this.persistencePlugin.afterCacheAccess(f)]):[3,7];case 6:n.sent(),n.label=7;case 7:return[7];case 8:return[2,e.generateAuthenticationResult(this.cryptoObj,r,g,!1,a,p,m)]}})})},e.prototype.generateCacheRecord=function(e,t,r,n,o,a,c){var d,l,g=t.getPreferredCache();if(s.StringUtils.isEmpty(g))throw i.ClientAuthError.createInvalidCacheEnvironmentError();!s.StringUtils.isEmpty(e.id_token)&&n&&(d=h.IdTokenEntity.createIdTokenEntity(this.homeAccountIdentifier,g,e.id_token||C.Constants.EMPTY_STRING,this.clientId,n.claims.tid||C.Constants.EMPTY_STRING,a),l=this.generateAccountEntity(e,n,t,a,c));var y=null;if(!s.StringUtils.isEmpty(e.access_token)){var E=e.scope?u.ScopeSet.fromString(e.scope):new u.ScopeSet(o||[]),v=("string"==typeof e.expires_in?parseInt(e.expires_in,10):e.expires_in)||0,T=("string"==typeof e.ext_expires_in?parseInt(e.ext_expires_in,10):e.ext_expires_in)||0,A=("string"==typeof e.refresh_in?parseInt(e.refresh_in,10):e.refresh_in)||void 0,w=r+v,S=w+T,b=A&&A>0?r+A:void 0;y=p.AccessTokenEntity.createAccessTokenEntity(this.homeAccountIdentifier,g,e.access_token||C.Constants.EMPTY_STRING,this.clientId,n?n.claims.tid||C.Constants.EMPTY_STRING:t.tenant,E.printScopes(),w,S,this.cryptoObj,b,e.token_type,a,e.key_id)}var I=null;s.StringUtils.isEmpty(e.refresh_token)||(I=m.RefreshTokenEntity.createRefreshTokenEntity(this.homeAccountIdentifier,g,e.refresh_token||C.Constants.EMPTY_STRING,this.clientId,e.foci,a));var R=null;return s.StringUtils.isEmpty(e.foci)||(R=_.AppMetadataEntity.createAppMetadataEntity(this.clientId,g,e.foci)),new f.CacheRecord(l,d,y,I,R)},e.prototype.generateAccountEntity=function(e,t,r,n,o){var a=r.authorityType,c=o?o.cloud_graph_host_name:"",u=o?o.msgraph_host:"";if(a===l.AuthorityType.Adfs)return this.logger.verbose("Authority type is ADFS, creating ADFS account"),d.AccountEntity.createGenericAccount(this.homeAccountIdentifier,t,r,n,c,u);if(s.StringUtils.isEmpty(e.client_info)&&"AAD"===r.protocolMode)throw i.ClientAuthError.createClientInfoEmptyError();return e.client_info?d.AccountEntity.createAccount(e.client_info,this.homeAccountIdentifier,t,r,n,c,u):d.AccountEntity.createGenericAccount(this.homeAccountIdentifier,t,r,n,c,u)},e.generateAuthenticationResult=function(e,t,r,o,i,s,a){var c,d,l;return Object(n.__awaiter)(this,void 0,void 0,function(){var h,p,m,g,f,y,_;return Object(n.__generator)(this,function(n){switch(n.label){case 0:return h="",p=[],m=null,f=C.Constants.EMPTY_STRING,r.accessToken?r.accessToken.tokenType!==C.AuthenticationScheme.POP?[3,2]:[4,new E.PopTokenGenerator(e).signPopToken(r.accessToken.secret,i)]:[3,4];case 1:return h=n.sent(),[3,3];case 2:h=r.accessToken.secret,n.label=3;case 3:p=u.ScopeSet.fromString(r.accessToken.target).asArray(),m=new Date(1e3*Number(r.accessToken.expiresOn)),g=new Date(1e3*Number(r.accessToken.extendedExpiresOn)),n.label=4;case 4:return r.appMetadata&&(f=r.appMetadata.familyId===C.THE_FAMILY_ID?C.THE_FAMILY_ID:C.Constants.EMPTY_STRING),y=(null==s?void 0:s.claims.oid)||(null==s?void 0:s.claims.sub)||C.Constants.EMPTY_STRING,_=(null==s?void 0:s.claims.tid)||C.Constants.EMPTY_STRING,[2,{authority:t.canonicalAuthority,uniqueId:y,tenantId:_,scopes:p,account:r.account?r.account.getAccountInfo():null,idToken:s?s.rawToken:C.Constants.EMPTY_STRING,idTokenClaims:s?s.claims:{},accessToken:h,fromCache:o,expiresOn:m,correlationId:i.correlationId,extExpiresOn:g,familyId:f,tokenType:(null===(c=r.accessToken)||void 0===c?void 0:c.tokenType)||C.Constants.EMPTY_STRING,state:a?a.userRequestState:C.Constants.EMPTY_STRING,cloudGraphHostName:(null===(d=r.account)||void 0===d?void 0:d.cloudGraphHostName)||C.Constants.EMPTY_STRING,msGraphHost:(null===(l=r.account)||void 0===l?void 0:l.msGraphHost)||C.Constants.EMPTY_STRING}]}})})},e}()},"./node_modules/@azure/msal-common/dist/telemetry/server/ServerTelemetryManager.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ServerTelemetryManager",function(){return s});var n=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),o=r("./node_modules/@azure/msal-common/dist/cache/entities/ServerTelemetryEntity.js"),i=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),s=function(){function e(e,t){this.cacheOutcome=n.CacheOutcome.NO_CACHE_HIT,this.cacheManager=t,this.apiId=e.apiId,this.correlationId=e.correlationId,this.wrapperSKU=e.wrapperSKU||n.Constants.EMPTY_STRING,this.wrapperVer=e.wrapperVer||n.Constants.EMPTY_STRING,this.telemetryCacheKey=n.SERVER_TELEM_CONSTANTS.CACHE_KEY+n.Separators.CACHE_KEY_SEPARATOR+e.clientId}return e.prototype.generateCurrentRequestHeaderValue=function(){var e=""+this.apiId+n.SERVER_TELEM_CONSTANTS.VALUE_SEPARATOR+this.cacheOutcome,t=[this.wrapperSKU,this.wrapperVer].join(n.SERVER_TELEM_CONSTANTS.VALUE_SEPARATOR),r=[e,this.getRegionDiscoveryFields()].join(n.SERVER_TELEM_CONSTANTS.VALUE_SEPARATOR);return[n.SERVER_TELEM_CONSTANTS.SCHEMA_VERSION,r,t].join(n.SERVER_TELEM_CONSTANTS.CATEGORY_SEPARATOR)},e.prototype.generateLastRequestHeaderValue=function(){var t=this.getLastRequests(),r=e.maxErrorsToSend(t),o=t.failedRequests.slice(0,2*r).join(n.SERVER_TELEM_CONSTANTS.VALUE_SEPARATOR),i=t.errors.slice(0,r).join(n.SERVER_TELEM_CONSTANTS.VALUE_SEPARATOR),s=t.errors.length,a=[s,r<s?n.SERVER_TELEM_CONSTANTS.OVERFLOW_TRUE:n.SERVER_TELEM_CONSTANTS.OVERFLOW_FALSE].join(n.SERVER_TELEM_CONSTANTS.VALUE_SEPARATOR);return[n.SERVER_TELEM_CONSTANTS.SCHEMA_VERSION,t.cacheHits,o,i,a].join(n.SERVER_TELEM_CONSTANTS.CATEGORY_SEPARATOR)},e.prototype.cacheFailedRequest=function(e){var t=this.getLastRequests();t.errors.length>=n.SERVER_TELEM_CONSTANTS.MAX_CACHED_ERRORS&&(t.failedRequests.shift(),t.failedRequests.shift(),t.errors.shift()),t.failedRequests.push(this.apiId,this.correlationId),i.StringUtils.isEmpty(e.subError)?i.StringUtils.isEmpty(e.errorCode)?e&&e.toString()?t.errors.push(e.toString()):t.errors.push(n.SERVER_TELEM_CONSTANTS.UNKNOWN_ERROR):t.errors.push(e.errorCode):t.errors.push(e.subError),this.cacheManager.setServerTelemetry(this.telemetryCacheKey,t)},e.prototype.incrementCacheHits=function(){var e=this.getLastRequests();return e.cacheHits+=1,this.cacheManager.setServerTelemetry(this.telemetryCacheKey,e),e.cacheHits},e.prototype.getLastRequests=function(){var e=new o.ServerTelemetryEntity;return this.cacheManager.getServerTelemetry(this.telemetryCacheKey)||e},e.prototype.clearTelemetryCache=function(){var t=this.getLastRequests(),r=e.maxErrorsToSend(t);if(r===t.errors.length)this.cacheManager.removeItem(this.telemetryCacheKey);else{var n=new o.ServerTelemetryEntity;n.failedRequests=t.failedRequests.slice(2*r),n.errors=t.errors.slice(r),this.cacheManager.setServerTelemetry(this.telemetryCacheKey,n)}},e.maxErrorsToSend=function(e){var t,r=0,o=0,i=e.errors.length;for(t=0;t<i;t++){var s=e.failedRequests[2*t]||n.Constants.EMPTY_STRING,a=e.failedRequests[2*t+1]||n.Constants.EMPTY_STRING,c=e.errors[t]||n.Constants.EMPTY_STRING;if(!((o+=s.toString().length+a.toString().length+c.length+3)<n.SERVER_TELEM_CONSTANTS.MAX_LAST_HEADER_BYTES))break;r+=1}return r},e.prototype.getRegionDiscoveryFields=function(){var e=[];return e.push(this.regionUsed||""),e.push(this.regionSource||""),e.push(this.regionOutcome||""),e.join(",")},e.prototype.updateRegionDiscoveryMetadata=function(e){this.regionUsed=e.region_used,this.regionSource=e.region_source,this.regionOutcome=e.region_outcome},e.prototype.setCacheOutcome=function(e){this.cacheOutcome=e},e}()},"./node_modules/@azure/msal-common/dist/url/UrlString.js":function(e,t,r){"use strict";r.r(t),r.d(t,"UrlString",function(){return a});var n=r("./node_modules/@azure/msal-common/dist/error/ClientConfigurationError.js"),o=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),i=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),s=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),a=function(){function e(t){if(this._urlString=t,i.StringUtils.isEmpty(this._urlString))throw n.ClientConfigurationError.createUrlEmptyError();i.StringUtils.isEmpty(this.getHash())&&(this._urlString=e.canonicalizeUri(t))}return Object.defineProperty(e.prototype,"urlString",{get:function(){return this._urlString},enumerable:!1,configurable:!0}),e.canonicalizeUri=function(e){if(e){var t=e.toLowerCase();return i.StringUtils.endsWith(t,"?")?t=t.slice(0,-1):i.StringUtils.endsWith(t,"?/")&&(t=t.slice(0,-2)),i.StringUtils.endsWith(t,"/")||(t+="/"),t}return e},e.prototype.validateAsUri=function(){var e;try{e=this.getUrlComponents()}catch(e){throw n.ClientConfigurationError.createUrlParseError(e)}if(!e.HostNameAndPort||!e.PathSegments)throw n.ClientConfigurationError.createUrlParseError("Given url string: "+this.urlString);if(!e.Protocol||"https:"!==e.Protocol.toLowerCase())throw n.ClientConfigurationError.createInsecureAuthorityUriError(this.urlString)},e.appendQueryString=function(e,t){return i.StringUtils.isEmpty(t)?e:e.indexOf("?")<0?e+"?"+t:e+"&"+t},e.removeHashFromUrl=function(t){return e.canonicalizeUri(t.split("#")[0])},e.prototype.replaceTenantPath=function(t){var r=this.getUrlComponents(),n=r.PathSegments;return!t||0===n.length||n[0]!==s.AADAuthorityConstants.COMMON&&n[0]!==s.AADAuthorityConstants.ORGANIZATIONS||(n[0]=t),e.constructAuthorityUriFromObject(r)},e.prototype.getHash=function(){return e.parseHash(this.urlString)},e.prototype.getUrlComponents=function(){var e=RegExp("^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?"),t=this.urlString.match(e);if(!t)throw n.ClientConfigurationError.createUrlParseError("Given url string: "+this.urlString);var r={Protocol:t[1],HostNameAndPort:t[4],AbsolutePath:t[5],QueryString:t[7]},o=r.AbsolutePath.split("/");return o=o.filter(function(e){return e&&e.length>0}),r.PathSegments=o,!i.StringUtils.isEmpty(r.QueryString)&&r.QueryString.endsWith("/")&&(r.QueryString=r.QueryString.substring(0,r.QueryString.length-1)),r},e.getDomainFromUrl=function(e){var t=RegExp("^([^:/?#]+://)?([^/?#]*)"),r=e.match(t);if(!r)throw n.ClientConfigurationError.createUrlParseError("Given url string: "+e);return r[2]},e.getAbsoluteUrl=function(t,r){if(t[0]===s.Constants.FORWARD_SLASH){var n=new e(r).getUrlComponents();return n.Protocol+"//"+n.HostNameAndPort+t}return t},e.parseHash=function(e){var t=e.indexOf("#"),r=e.indexOf("#/");return r>-1?e.substring(r+2):t>-1?e.substring(t+1):""},e.constructAuthorityUriFromObject=function(t){return new e(t.Protocol+"//"+t.HostNameAndPort+"/"+t.PathSegments.join("/"))},e.getDeserializedHash=function(t){if(i.StringUtils.isEmpty(t))return{};var r=e.parseHash(t),n=i.StringUtils.queryStringToObject(i.StringUtils.isEmpty(r)?t:r);if(!n)throw o.ClientAuthError.createHashNotDeserializedError(JSON.stringify(n));return n},e.hashContainsKnownProperties=function(t){if(i.StringUtils.isEmpty(t)||t.indexOf("=")<0)return!1;var r=e.getDeserializedHash(t);return!!(r.code||r.error_description||r.error||r.state)},e}()},"./node_modules/@azure/msal-common/dist/utils/Constants.js":function(e,t,r){"use strict";r.r(t),r.d(t,"AADAuthorityConstants",function(){return i}),r.d(t,"AADServerParamKeys",function(){return s}),r.d(t,"APP_METADATA",function(){return A}),r.d(t,"AUTHORITY_METADATA_CONSTANTS",function(){return b}),r.d(t,"AuthenticationScheme",function(){return I}),r.d(t,"AuthorityMetadataSource",function(){return T}),r.d(t,"CLIENT_INFO",function(){return w}),r.d(t,"CacheAccountType",function(){return f}),r.d(t,"CacheOutcome",function(){return j}),r.d(t,"CacheSchemaType",function(){return E}),r.d(t,"CacheType",function(){return _}),r.d(t,"ClaimsRequestKeys",function(){return a}),r.d(t,"CodeChallengeMethodValues",function(){return v}),r.d(t,"Constants",function(){return u}),r.d(t,"CredentialType",function(){return C}),r.d(t,"Errors",function(){return M}),r.d(t,"GrantType",function(){return g}),r.d(t,"HeaderNames",function(){return n}),r.d(t,"OIDC_DEFAULT_SCOPES",function(){return d}),r.d(t,"OIDC_SCOPES",function(){return l}),r.d(t,"PasswordGrantConstants",function(){return O}),r.d(t,"PersistentCacheKeys",function(){return o}),r.d(t,"PromptValue",function(){return p}),r.d(t,"RegionDiscoveryOutcomes",function(){return U}),r.d(t,"RegionDiscoverySources",function(){return P}),r.d(t,"ResponseCodes",function(){return k}),r.d(t,"ResponseMode",function(){return m}),r.d(t,"SERVER_TELEM_CONSTANTS",function(){return R}),r.d(t,"SSOTypes",function(){return h}),r.d(t,"Separators",function(){return y}),r.d(t,"THE_FAMILY_ID",function(){return S}),r.d(t,"ThrottlingConstants",function(){return N});var n,o,i,s,a,c=r("./node_modules/@azure/msal-common/dist/_virtual/_tslib.js"),u={LIBRARY_NAME:"MSAL.JS",SKU:"msal.js.common",CACHE_PREFIX:"msal",DEFAULT_AUTHORITY:"https://login.microsoftonline.com/common/",DEFAULT_AUTHORITY_HOST:"login.microsoftonline.com",ADFS:"adfs",AAD_INSTANCE_DISCOVERY_ENDPT:"https://login.microsoftonline.com/common/discovery/instance?api-version=1.1&authorization_endpoint=",RESOURCE_DELIM:"|",NO_ACCOUNT:"NO_ACCOUNT",CLAIMS:"claims",CONSUMER_UTID:"9188040d-6c67-4c5b-b112-36a304b66dad",OPENID_SCOPE:"openid",PROFILE_SCOPE:"profile",OFFLINE_ACCESS_SCOPE:"offline_access",EMAIL_SCOPE:"email",CODE_RESPONSE_TYPE:"code",CODE_GRANT_TYPE:"authorization_code",RT_GRANT_TYPE:"refresh_token",FRAGMENT_RESPONSE_MODE:"fragment",S256_CODE_CHALLENGE_METHOD:"S256",URL_FORM_CONTENT_TYPE:"application/x-www-form-urlencoded;charset=utf-8",AUTHORIZATION_PENDING:"authorization_pending",NOT_DEFINED:"not_defined",EMPTY_STRING:"",FORWARD_SLASH:"/",IMDS_ENDPOINT:"http://169.254.169.254/metadata/instance/compute/location",IMDS_VERSION:"2020-06-01",IMDS_TIMEOUT:2e3,AZURE_REGION_AUTO_DISCOVER_FLAG:"TryAutoDetect",REGIONAL_AUTH_PUBLIC_CLOUD_SUFFIX:"login.microsoft.com",KNOWN_PUBLIC_CLOUDS:["login.microsoftonline.com","login.windows.net","login.microsoft.com","sts.windows.net"]},d=[u.OPENID_SCOPE,u.PROFILE_SCOPE,u.OFFLINE_ACCESS_SCOPE],l=Object(c.__spreadArrays)(d,[u.EMAIL_SCOPE]);
/*! @azure/msal-common v5.1.0 2021-11-02 */!function(e){e.CONTENT_TYPE="Content-Type",e.RETRY_AFTER="Retry-After",e.CCS_HEADER="X-AnchorMailbox",e.WWWAuthenticate="WWW-Authenticate",e.AuthenticationInfo="Authentication-Info"}(n||(n={})),function(e){e.ID_TOKEN="idtoken",e.CLIENT_INFO="client.info",e.ADAL_ID_TOKEN="adal.idtoken",e.ERROR="error",e.ERROR_DESC="error.description",e.ACTIVE_ACCOUNT="active-account"}(o||(o={})),function(e){e.COMMON="common",e.ORGANIZATIONS="organizations",e.CONSUMERS="consumers"}(i||(i={})),function(e){e.CLIENT_ID="client_id",e.REDIRECT_URI="redirect_uri",e.RESPONSE_TYPE="response_type",e.RESPONSE_MODE="response_mode",e.GRANT_TYPE="grant_type",e.CLAIMS="claims",e.SCOPE="scope",e.ERROR="error",e.ERROR_DESCRIPTION="error_description",e.ACCESS_TOKEN="access_token",e.ID_TOKEN="id_token",e.REFRESH_TOKEN="refresh_token",e.EXPIRES_IN="expires_in",e.STATE="state",e.NONCE="nonce",e.PROMPT="prompt",e.SESSION_STATE="session_state",e.CLIENT_INFO="client_info",e.CODE="code",e.CODE_CHALLENGE="code_challenge",e.CODE_CHALLENGE_METHOD="code_challenge_method",e.CODE_VERIFIER="code_verifier",e.CLIENT_REQUEST_ID="client-request-id",e.X_CLIENT_SKU="x-client-SKU",e.X_CLIENT_VER="x-client-VER",e.X_CLIENT_OS="x-client-OS",e.X_CLIENT_CPU="x-client-CPU",e.X_CLIENT_CURR_TELEM="x-client-current-telemetry",e.X_CLIENT_LAST_TELEM="x-client-last-telemetry",e.X_MS_LIB_CAPABILITY="x-ms-lib-capability",e.POST_LOGOUT_URI="post_logout_redirect_uri",e.ID_TOKEN_HINT="id_token_hint",e.DEVICE_CODE="device_code",e.CLIENT_SECRET="client_secret",e.CLIENT_ASSERTION="client_assertion",e.CLIENT_ASSERTION_TYPE="client_assertion_type",e.TOKEN_TYPE="token_type",e.REQ_CNF="req_cnf",e.OBO_ASSERTION="assertion",e.REQUESTED_TOKEN_USE="requested_token_use",e.ON_BEHALF_OF="on_behalf_of",e.FOCI="foci",e.CCS_HEADER="X-AnchorMailbox"}(s||(s={})),function(e){e.ACCESS_TOKEN="access_token",e.XMS_CC="xms_cc"}(a||(a={}));var h,p={LOGIN:"login",SELECT_ACCOUNT:"select_account",CONSENT:"consent",NONE:"none",CREATE:"create"};!function(e){e.ACCOUNT="account",e.SID="sid",e.LOGIN_HINT="login_hint",e.ID_TOKEN="id_token",e.DOMAIN_HINT="domain_hint",e.ORGANIZATIONS="organizations",e.CONSUMERS="consumers",e.ACCOUNT_ID="accountIdentifier",e.HOMEACCOUNT_ID="homeAccountIdentifier"}(h||(h={}));var m,g,f,y,C,E,_,v={PLAIN:"plain",S256:"S256"};!function(e){e.QUERY="query",e.FRAGMENT="fragment",e.FORM_POST="form_post"}(m||(m={})),function(e){e.IMPLICIT_GRANT="implicit",e.AUTHORIZATION_CODE_GRANT="authorization_code",e.CLIENT_CREDENTIALS_GRANT="client_credentials",e.RESOURCE_OWNER_PASSWORD_GRANT="password",e.REFRESH_TOKEN_GRANT="refresh_token",e.DEVICE_CODE_GRANT="device_code",e.JWT_BEARER="urn:ietf:params:oauth:grant-type:jwt-bearer"}(g||(g={})),function(e){e.MSSTS_ACCOUNT_TYPE="MSSTS",e.ADFS_ACCOUNT_TYPE="ADFS",e.MSAV1_ACCOUNT_TYPE="MSA",e.GENERIC_ACCOUNT_TYPE="Generic"}(f||(f={})),function(e){e.CACHE_KEY_SEPARATOR="-",e.CLIENT_INFO_SEPARATOR="."}(y||(y={})),function(e){e.ID_TOKEN="IdToken",e.ACCESS_TOKEN="AccessToken",e.ACCESS_TOKEN_WITH_AUTH_SCHEME="AccessToken_With_AuthScheme",e.REFRESH_TOKEN="RefreshToken"}(C||(C={})),function(e){e.ACCOUNT="Account",e.CREDENTIAL="Credential",e.ID_TOKEN="IdToken",e.ACCESS_TOKEN="AccessToken",e.REFRESH_TOKEN="RefreshToken",e.APP_METADATA="AppMetadata",e.TEMPORARY="TempCache",e.TELEMETRY="Telemetry",e.UNDEFINED="Undefined",e.THROTTLING="Throttling"}(E||(E={})),function(e){e[e.ADFS=1001]="ADFS",e[e.MSA=1002]="MSA",e[e.MSSTS=1003]="MSSTS",e[e.GENERIC=1004]="GENERIC",e[e.ACCESS_TOKEN=2001]="ACCESS_TOKEN",e[e.REFRESH_TOKEN=2002]="REFRESH_TOKEN",e[e.ID_TOKEN=2003]="ID_TOKEN",e[e.APP_METADATA=3001]="APP_METADATA",e[e.UNDEFINED=9999]="UNDEFINED"}(_||(_={}));var T,A="appmetadata",w="client_info",S="1",b={CACHE_KEY:"authority-metadata",REFRESH_TIME_SECONDS:86400};!function(e){e.CONFIG="config",e.CACHE="cache",e.NETWORK="network"}(T||(T={}));var I,R={SCHEMA_VERSION:5,MAX_CUR_HEADER_BYTES:80,MAX_LAST_HEADER_BYTES:330,MAX_CACHED_ERRORS:50,CACHE_KEY:"server-telemetry",CATEGORY_SEPARATOR:"|",VALUE_SEPARATOR:",",OVERFLOW_TRUE:"1",OVERFLOW_FALSE:"0",UNKNOWN_ERROR:"unknown_error"};!function(e){e.BEARER="Bearer",e.POP="pop",e.SSH="ssh-cert"}(I||(I={}));var O,k,P,U,j,N={DEFAULT_THROTTLE_TIME_SECONDS:60,DEFAULT_MAX_THROTTLE_TIME_SECONDS:3600,THROTTLING_PREFIX:"throttling",X_MS_LIB_CAPABILITY_VALUE:"retry-after, h429"},M={INVALID_GRANT_ERROR:"invalid_grant",CLIENT_MISMATCH_ERROR:"client_mismatch"};!function(e){e.username="username",e.password="password"}(O||(O={})),function(e){e[e.httpSuccess=200]="httpSuccess",e[e.httpBadRequest=400]="httpBadRequest"}(k||(k={})),function(e){e.FAILED_AUTO_DETECTION="1",e.INTERNAL_CACHE="2",e.ENVIRONMENT_VARIABLE="3",e.IMDS="4"}(P||(P={})),function(e){e.CONFIGURED_MATCHES_DETECTED="1",e.CONFIGURED_NO_AUTO_DETECTION="2",e.CONFIGURED_NOT_DETECTED="3",e.AUTO_DETECTION_REQUESTED_SUCCESSFUL="4",e.AUTO_DETECTION_REQUESTED_FAILED="5"}(U||(U={})),function(e){e.NO_CACHE_HIT="0",e.FORCE_REFRESH="1",e.NO_CACHED_ACCESS_TOKEN="2",e.CACHED_ACCESS_TOKEN_EXPIRED="3",e.REFRESH_CACHED_ACCESS_TOKEN="4"}(j||(j={}))},"./node_modules/@azure/msal-common/dist/utils/ProtocolUtils.js":function(e,t,r){"use strict";r.r(t),r.d(t,"ProtocolUtils",function(){return s});var n=r("./node_modules/@azure/msal-common/dist/utils/StringUtils.js"),o=r("./node_modules/@azure/msal-common/dist/utils/Constants.js"),i=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),s=function(){function e(){}return e.setRequestState=function(t,r,i){var s=e.generateLibraryState(t,i);return n.StringUtils.isEmpty(r)?s:""+s+o.Constants.RESOURCE_DELIM+r},e.generateLibraryState=function(e,t){if(!e)throw i.ClientAuthError.createNoCryptoObjectError("generateLibraryState");var r={id:e.createNewGuid()};t&&(r.meta=t);var n=JSON.stringify(r);return e.base64Encode(n)},e.parseRequestState=function(e,t){if(!e)throw i.ClientAuthError.createNoCryptoObjectError("parseRequestState");if(n.StringUtils.isEmpty(t))throw i.ClientAuthError.createInvalidStateError(t,"Null, undefined or empty state");try{var r=t.split(o.Constants.RESOURCE_DELIM),s=r[0],a=r.length>1?r.slice(1).join(o.Constants.RESOURCE_DELIM):"",c=e.base64Decode(s),u=JSON.parse(c);return{userRequestState:n.StringUtils.isEmpty(a)?"":a,libraryState:u}}catch(e){throw i.ClientAuthError.createInvalidStateError(t,e)}},e}()},"./node_modules/@azure/msal-common/dist/utils/StringUtils.js":function(e,t,r){"use strict";r.r(t),r.d(t,"StringUtils",function(){return o});var n=r("./node_modules/@azure/msal-common/dist/error/ClientAuthError.js"),o=function(){function e(){}return e.decodeAuthToken=function(t){if(e.isEmpty(t))throw n.ClientAuthError.createTokenNullOrEmptyError(t);var r=/^([^\.\s]*)\.([^\.\s]+)\.([^\.\s]*)$/.exec(t);if(!r||r.length<4)throw n.ClientAuthError.createTokenParsingError("Given token is malformed: "+JSON.stringify(t));return{header:r[1],JWSPayload:r[2],JWSSig:r[3]}},e.isEmpty=function(e){return void 0===e||!e||0===e.length},e.isEmptyObj=function(t){if(t&&!e.isEmpty(t))try{var r=JSON.parse(t);return 0===Object.keys(r).length}catch(e){}return!0},e.startsWith=function(e,t){return 0===e.indexOf(t)},e.endsWith=function(e,t){return e.length>=t.length&&e.lastIndexOf(t)===e.length-t.length},e.queryStringToObject=function(e){var t={},r=e.split("&"),n=function(e){return decodeURIComponent(e.replace(/\+/g," "))};return r.forEach(function(e){if(e.trim()){var r=e.split(/=(.+)/g,2),o=r[0],i=r[1];o&&i&&(t[n(o)]=n(i))}}),t},e.trimArrayEntries=function(e){return e.map(function(e){return e.trim()})},e.removeEmptyStringsFromArray=function(t){return t.filter(function(t){return!e.isEmpty(t)})},e.jsonParseHelper=function(e){try{return JSON.parse(e)}catch(e){return null}},e.matchPattern=function(e,t){return new RegExp(e.replace(/\\/g,"\\\\").replace(/\*/g,"[^ ]*").replace(/\?/g,"\\?")).test(t)},e}();
/*! @azure/msal-common v5.1.0 2021-11-02 */},"./node_modules/@azure/msal-common/dist/utils/TimeUtils.js":function(e,t,r){"use strict";r.r(t),r.d(t,"TimeUtils",function(){return n});
/*! @azure/msal-common v5.1.0 2021-11-02 */
var n=function(){function e(){}return e.nowSeconds=function(){return Math.round((new Date).getTime()/1e3)},e.isTokenExpired=function(t,r){var n=Number(t)||0;return e.nowSeconds()+r>n},e.wasClockTurnedBack=function(t){return Number(t)>e.nowSeconds()},e.delay=function(e,t){return new Promise(function(r){return setTimeout(function(){return r(t)},e)})},e}()},"./packages/Microsoft.Office.WebAuth.BrowserAuth/lib/api.js":function(e,t,r){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.logUserAction=function(e,t,r,n,o,i,s){void 0===t&&(t=!0);void 0===r&&(r=c);void 0===n&&(n=c);void 0===o&&(o=c);void 0===i&&(i=0);void 0===s&&(s=[]);d({name:null,actionName:e,commandSurface:o,parentName:r,triggerMethod:n,durationMs:i,succeeded:t,dataFields:s})},t.logActivity=function(e,t,r,n){void 0===t&&(t=!0);void 0===r&&(r=0);void 0===n&&(n=[]);u({name:e,succeeded:t,durationMs:r,dataFields:n})},t.sendTelemetryEvent=function(e){l({kind:"event",event:e,timestamp:(new Date).getTime()})},t.sendActivityEvent=u,t.sendOtelEvent=function(e){l({kind:"otel",event:e})},t.sendUserActionEvent=d,t.addNamespaceMapping=function(e,t){l({kind:"addNamespaceMapping",namespace:e,ariaTenantToken:t})},t.setEnabledState=function(e){(o=e)||(i=[])},t.shutdown=function(){return l({kind:"shutdown"}),i.length+s},t.registerEventHandler=function(e){n=e,i.forEach(function(e){return l(e)}),i=[]};var n,o=!1,i=[],s=0,a=2e4,c="Unknown";function u(e){l({kind:"activity",event:e,timestamp:(new Date).getTime()})}function d(e){l({kind:"action",event:e,timestamp:(new Date).getTime()})}function l(e){o&&(n?n(e):i.length<=a?i.push(e):s+=1)}},"./packages/Microsoft.Office.WebAuth.BrowserAuth/scripts/BrowserAuth.ts":function(e,t,r){"use strict";var n=this&&this.__awaiter||function(e,t,r,n){return new(r||(r=Promise))(function(o,i){function s(e){try{c(n.next(e))}catch(e){i(e)}}function a(e){try{c(n.throw(e))}catch(e){i(e)}}function c(e){e.done?o(e.value):new r(function(t){t(e.value)}).then(s,a)}c((n=n.apply(e,t||[])).next())})},o=this&&this.__generator||function(e,t){var r,n,o,i,s={label:0,sent:function(){if(1&o[0])throw o[1];return o[1]},trys:[],ops:[]};return i={next:a(0),throw:a(1),return:a(2)},"function"==typeof Symbol&&(i[Symbol.iterator]=function(){return this}),i;function a(i){return function(a){return function(i){if(r)throw new TypeError("Generator is already executing.");for(;s;)try{if(r=1,n&&(o=2&i[0]?n.return:i[0]?n.throw||((o=n.return)&&o.call(n),0):n.next)&&!(o=o.call(n,i[1])).done)return o;switch(n=0,o&&(i=[2&i[0],o.value]),i[0]){case 0:case 1:o=i;break;case 4:return s.label++,{value:i[1],done:!1};case 5:s.label++,n=i[1],i=[0];continue;case 7:i=s.ops.pop(),s.trys.pop();continue;default:if(!(o=(o=s.trys).length>0&&o[o.length-1])&&(6===i[0]||2===i[0])){s=0;continue}if(3===i[0]&&(!o||i[1]>o[0]&&i[1]<o[3])){s.label=i[1];break}if(6===i[0]&&s.label<o[1]){s.label=o[1],o=i;break}if(o&&s.label<o[2]){s.label=o[2],s.ops.push(i);break}o[2]&&s.ops.pop(),s.trys.pop();continue}i=t.call(e,s)}catch(e){i=[6,e],n=0}finally{r=o=0}if(5&i[0])throw i[1];return{value:i[0]?i[1]:void 0,done:!0}}([i,a])}}};Object.defineProperty(t,"__esModule",{value:!0});var i,s,a=r("./node_modules/@azure/msal-browser/dist/index.js"),c=r("./packages/Microsoft.Office.WebAuth.BrowserAuth/lib/api.js"),u=r("./packages/Microsoft.Office.WebAuth.BrowserAuth/scripts/Utils/Constants.ts"),d=r("./packages/Microsoft.Office.WebAuth.BrowserAuth/scripts/Utils/ExtractUtils.ts"),l=r("./packages/Microsoft.Office.WebAuth.BrowserAuth/scripts/Utils/LoggingUtils.ts"),h=r("./packages/Microsoft.Office.WebAuth.BrowserAuth/scripts/Utils/TimerUtils.ts");c.addNamespaceMapping("Office.Identity.WebAuth.BrowserAuth","5c65bbc4edbf480d9637ace04d62bd98-12844893-8ab9-4dde-b850-5612cb12e0f2-7822");var p=!1,m=!0;function g(e,t){var r=h.TimerUtils.timer();(s=e).enableConsoleLogging&&(p=s.enableConsoleLogging),s.enableUpnCheck&&(m=s.enableUpnCheck),s.authority&&(s.authority=s.authority.replace(u.Constants.Authority.ProdLegacy,u.Constants.Authority.Prod)),i||window.addEventListener("message",function(e){if(e.origin&&e.origin==location.origin&&e.data&&e.data.iframe&&e.data.request&&e.data.request==u.Constants.PostMessageType.RequestAuthConfig){var t=u.Constants.PostMessageType.iFramePrefix+e.data.iframe;if(null===(r=document.getElementById(t.replace("+"," ")))){t=u.Constants.PostMessageType.iFrameIdTokenPrefix;var r=document.getElementById(t.replace("+"," "))}if(l.LoggingUtils.log("received from "+t,p),null===r)return void l.LoggingUtils.log("targetiFrame is null",p);var n=r.contentWindow;null!=n&&(l.LoggingUtils.log("returning to "+e.data.iframe,p),n.postMessage({request:u.Constants.PostMessageType.ResponseAuthConfig,config:s},location.origin))}},!1);var n,o=s.idp.toLowerCase()===u.Constants.IdentityProvider.Msa.toLowerCase();o?s.authority=u.Constants.Authority.Prod+u.Constants.Authority.MsaSuffix:(s.authority||(s.authority=u.Constants.Authority.Prod+u.Constants.Authority.AadSuffix),s.authority.indexOf(u.Constants.Authority.AadSuffix)<0&&("/"==s.authority.charAt(s.authority.length-1)?s.authority+=u.Constants.Authority.AadSuffix:s.authority+="/"+u.Constants.Authority.AadSuffix)),i=new Array;for(var c=0,d=s.appIds;c<d.length;c++){var g=d[c];if(l.LoggingUtils.log("["+s.idp+"] Load->init: "+g,p),g&&T(g)){var f={applicationId:g,application:new a.PublicClientApplication({auth:{clientId:g,authority:s.authority,redirectUri:s.redirectUri?s.redirectUri.split("?")[0]:location.href.split("?")[0],navigateToLoginRequestUrl:!s.navigateToLoginRequestUrl||s.navigateToLoginRequestUrl},cache:{cacheLocation:"localStorage",storeAuthStateInCookie:!1},system:{loadFrameTimeout:s.timeout?s.timeout:6e3}})};i.push(f),n||(n=g)}}return n?(l.LoggingUtils.log("["+s.idp+"] ssoSilent@Load->init",p),y(n,s.upn,o?u.Constants.Authority.MsaSuffix:u.Constants.Authority.AadSuffix,t).then(function(e){return l.LoggingUtils.log("["+s.idp+"] ssoSilent@Load->success: "+e.idTokenClaims.aud,p),C(!0,t,i,r)}).catch(function(e){return l.LoggingUtils.log("["+s.idp+"] ssoSilent@Load->error: ["+e.errorCode+"]"+e.errorMessage,p),C(!0,t,i,r,e.errorCode,e.errorMessage)})):Promise.reject(C(!1,t,i,r))}function f(e,t,r,i,a,c,g){var f=this;void 0===c&&(c=!1);var C=h.TimerUtils.timer(),A=i||a,S=!1,b=_(t),I={},R=s.idp.toLowerCase()===u.Constants.IdentityProvider.Msa.toLowerCase();if(r&&T(r)||(r=void 0),!e)return I.ErrorCode="missing_target",I.ErrorMessage="The provided target for BrowserAuth.GetToken is null, blank or empty",Promise.reject(E(I,r,t,c,C,void 0));var O=[v(e)];if(!t||!T(t))return I.ErrorCode="invalid_application_ID",I.ErrorMessage="The provided application ID for BrowserAuth.GetToken is null, blank, empty or with invalid format",Promise.reject(E(I,r,t,c,C,O));var k=function(e){return new Promise(function(r,i){return n(f,void 0,void 0,function(){return o(this,function(n){switch(n.label){case 0:return l.LoggingUtils.log("["+t+"] acquireToken->init: popup:["+A+"]",p),I={},[4,(A?b.acquireTokenPopup(e):b.acquireTokenSilent(e)).then(function(e){return I.MsalResult=e,e.accessToken?w(e.idToken)&&(c||R||function(e){if(!m)return l.LoggingUtils.log("Skip Upn check of access token, returning true",p),!0;if(!s||!s.upn)return l.LoggingUtils.log("Upn does not exist in the configuration, returning true",p),!0;var t=d.ExtractUtils.extractToken(e);if(!t||t&&t.upn&&t.upn.toLowerCase()===s.upn.toLowerCase())return!0;return l.LoggingUtils.log("provided Upn does not match Upn extracted from access token",p),!1}(e.accessToken))?(I.Token=e.accessToken,l.LoggingUtils.log("["+t+"] acquireToken->token: "+I.Token,p),r(I)):(I.ErrorCode="upn_mismatch",I.ErrorMessage="upn doesn't match with given upn in config",l.LoggingUtils.log("["+t+"] acquireToken->error: ["+I.ErrorCode+"]"+I.ErrorMessage,p),i(I)):(I.ErrorCode="no_tokens_found",I.ErrorMessage="Access token doesn't exist in auth response",l.LoggingUtils.log("["+t+"] acquireToken->error: ["+I.ErrorCode+"]"+I.ErrorMessage,p),i(I))}).catch(function(n){if(I.MsalResult=n,I.ErrorCode=n.errorCode,I.ErrorMessage=n.errorMessage,l.LoggingUtils.log("["+t+"] acquireToken->error: ["+I.ErrorCode+"]"+I.ErrorMessage,p),A||!S)return i(I);A=!0,l.LoggingUtils.log("["+t+"] acquireToken->retry: popup:["+A+"]",p),k(e).then(function(e){return r(e)}).catch(function(e){return i(e)})})];case 1:return n.sent(),[2]}})})})},P={scopes:O,account:{username:s.upn},loginHint:s.upn,extraQueryParameters:{domain_hint:R?u.Constants.Authority.MsaSuffix:u.Constants.Authority.AadSuffix},correlationId:r,claims:g};return k(P).catch(function(e){return n(f,void 0,void 0,function(){var i,a=this;return o(this,function(d){switch(d.label){case 0:return i=!1,"no_tokens_found"!==e.ErrorCode?[3,2]:(l.LoggingUtils.log("["+t+"] ssoSilent@GetToken->init",p),[4,y(t,s.upn,R?u.Constants.Authority.MsaSuffix:u.Constants.Authority.AadSuffix,r).then(function(){return n(a,void 0,void 0,function(){return o(this,function(e){switch(e.label){case 0:return l.LoggingUtils.log("["+t+"] ssoSilent@GetToken->success",p),[4,k(P).then(function(e){i=!0,I=e}).catch(function(e){I=e})];case 1:return e.sent(),[2]}})})}).catch(function(e){return n(a,void 0,void 0,function(){return o(this,function(r){switch(r.label){case 0:return I.ErrorCode=e.errorCode,I.ErrorMessage=e.errorMessage,l.LoggingUtils.log("["+t+"] ssoSilent@GetToken->error: ["+I.ErrorCode+"]"+I.ErrorMessage,p),"login_required"===e.errorCode&&(e.errorMessage.startsWith("AADSTS50058")||e.errorMessage.startsWith("Silent authentication was denied"))&&s.autoPopup?(S=!0,[4,k(P).then(function(e){i=!0,I=e}).catch(function(e){l.LoggingUtils.log("["+t+"] acquireToken@GetToken->error",p),I=e})]):[3,2];case 1:r.sent(),r.label=2;case 2:return[2]}})})})]);case 1:d.sent(),d.label=2;case 2:return l.LoggingUtils.log("["+t+"] GetToken->isSuccess["+i+"]",p),I=E(I,r,t,c,C,O),[2,i?Promise.resolve(I):Promise.reject(I)]}})})}).then(function(e){return l.LoggingUtils.log("["+t+"] GetToken->success",p),Promise.resolve(E(e,r,t,c,C,O))})}function y(e,t,r,n){return _(e).ssoSilent({loginHint:t,extraQueryParameters:{domain_hint:r},correlationId:n})}function C(e,t,r,n,o,i){if(!s.telemetryInstance&&"undefined"==typeof OTel){var a=[{name:u.Constants.Telemetry.Duration,int64:n.ms},{name:u.Constants.Telemetry.Succeeded,bool:!0},{name:u.Constants.Telemetry.IdentityProvider,string:s.idp.toLowerCase()},{name:u.Constants.Telemetry.CorrelationId,string:t||"unknown"},{name:u.Constants.Telemetry.LoadedApplicationCount,int64:r.length},{name:u.Constants.Telemetry.ErrorCode,string:o||"unknown"},{name:u.Constants.Telemetry.ErrorMessage,string:i||"unknown"}];c.sendTelemetryEvent({name:u.Constants.Telemetry.LoadTelemetryName,dataFields:a})}return{Telemetry:{timeToLoad:n.ms,succeeded:e,idp:s.idp.toLowerCase(),correlationId:t||"",loadedApplicationCount:r.length,errorCode:o||void 0,errorMessage:i||void 0}}}function E(e,t,r,n,o,i){if(!s.telemetryInstance&&"undefined"==typeof OTel){var a=[{name:u.Constants.Telemetry.Duration,int64:o.ms},{name:u.Constants.Telemetry.Succeeded,string:!e.Token},{name:u.Constants.Telemetry.IdentityProvider,string:s.idp.toLowerCase()},{name:u.Constants.Telemetry.ApplicationId,string:r},{name:u.Constants.Telemetry.TokenScope,string:i&&!n?i.toString():"unknown"},{name:u.Constants.Telemetry.CorrelationId,string:t||"unknown"},{name:u.Constants.Telemetry.ErrorCode,string:e.ErrorCode?e.ErrorCode:"unknown"},{name:u.Constants.Telemetry.ErrorMessage,string:e.ErrorMessage&&!n?e.ErrorMessage:"unknown"},{name:u.Constants.Telemetry.ErrorCodeForGetToken,string:e.ErrorCode?e.ErrorCode:"unknown"},{name:u.Constants.Telemetry.ErrorMessageForGetToken,string:e.ErrorMessage&&!n?e.ErrorMessage:"unknown"}];c.sendTelemetryEvent({name:u.Constants.Telemetry.GetTokenTelemetryName,dataFields:a})}return e.Telemetry={timeToGetToken:o.ms,succeeded:!e.Token,idp:s.idp.toLowerCase(),applicationId:r,tokenScope:i&&!n?i.toString():void 0,correlationId:t,errorCode:e.ErrorCode?e.ErrorCode:void 0,errorMessage:e.ErrorMessage&&!n?e.ErrorMessage:void 0,errorCodeForGetToken:e.ErrorCode?e.ErrorCode:void 0,errorMessageForGetToken:e.ErrorMessage&&!n?e.ErrorMessage:void 0,fromCache:!!e.MsalResult&&!!e.MsalResult.fromCache,expiresOn:e.MsalResult&&e.MsalResult.expiresOn?e.MsalResult.expiresOn.getTime()/1e3:void 0},e}function _(e){var t;if(i&&i.some(function(r){return!(!e||!r.applicationId||e.toUpperCase()!==r.applicationId.toUpperCase())&&(t=r.application,!0)}),!t){t=new a.PublicClientApplication({auth:{clientId:e,authority:s.authority,redirectUri:s.redirectUri?s.redirectUri.split("?")[0]:location.href.split("?")[0],navigateToLoginRequestUrl:!s.navigateToLoginRequestUrl||s.navigateToLoginRequestUrl},cache:{cacheLocation:"localStorage",storeAuthStateInCookie:!1},system:{loadFrameTimeout:s.timeout?s.timeout:6e3}});var r={applicationId:e,application:t};i.push(r)}return t}function v(e){return["HTTPS:","API:"].some(function(t){return e.toLocaleUpperCase().startsWith(t)})||T(e)?e+"/.default":e}function T(e){return/^[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/i.test(e)}function A(e,t,r,n,o,i,a){var d=[{name:u.Constants.Telemetry.Duration,int64:n.ms},{name:u.Constants.Telemetry.Succeeded,bool:!i},{name:u.Constants.Telemetry.IdentityProvider,string:s.idp.toLowerCase()},{name:u.Constants.Telemetry.ApplicationId,string:r},{name:u.Constants.Telemetry.TokenScope,string:o?o.toString():"unknown"},{name:u.Constants.Telemetry.CorrelationId,string:t||"unknown"},{name:u.Constants.Telemetry.ErrorCode,string:i||"unknown"},{name:u.Constants.Telemetry.ErrorMessage,string:a||"unknown"},{name:u.Constants.Telemetry.ErrorCodeForCheckUpn,string:i||"unknown"},{name:u.Constants.Telemetry.ErrorMessageForCheckUpn,string:a||"unknown"}];s.telemetryInstance||"undefined"!=typeof OTel||c.sendTelemetryEvent({name:u.Constants.Telemetry.CheckUpnTelemetryName,dataFields:d}),e.Telemetry={timeToCheckUPN:n.ms,succeeded:!i,idp:s.idp.toLowerCase(),applicationId:r,tokenScope:o?o.toString():void 0,correlationId:t,errorCode:i,errorMessage:a,errorCodeForCheckUPN:i,errorMessageForCheckUPN:a}}function w(e){if(!m)return l.LoggingUtils.log("Skip Upn check of id token, returning true",p),!0;if(!s||!s.upn)return l.LoggingUtils.log("Upn does not exist in the configuration, returning true",p),!0;var t=d.ExtractUtils.extractToken(e);return!!(!t||t&&t.preferred_username&&t.preferred_username.toLowerCase()===s.upn.toLowerCase())||(l.LoggingUtils.log("provided Upn does not match Upn extracted from id token",p),!1)}t.Load=g,t.GetToken=f,t.GetTokenOnce=function(e,t,r,n){return void 0===r&&(r=!1),void 0===n&&(n=!1),g({idp:r?u.Constants.IdentityProvider.Msa.toLowerCase():u.Constants.IdentityProvider.Aad.toLowerCase(),appIds:[t],authority:n?u.Constants.Authority.Ppe:u.Constants.Authority.Prod,upn:"",autoPopup:!0,enableConsoleLogging:!0}),f(e,t,void 0,!0)},t.GetApplication=_,t.CheckUpnMatchIdToken=function(e,t){var r=h.TimerUtils.timer();t&&T(t)||(t=void 0);var n={};if(!e||!T(e))return n.ErrorCode="invalid_application_ID",n.ErrorMessage="The provided application ID for BrowserAuth.CheckUpnMatchIdToken is null, blank, empty or with invalid format",A(n,t,e,r,void 0,n.ErrorCode,n.ErrorMessage),Promise.reject(n);var o=_(e),i=[e];return new Promise(function(a,c){l.LoggingUtils.log("Config: "+JSON.stringify(o),p);var u=o.getAccountByUsername(s.upn);l.LoggingUtils.log("application calls acquireTokenSilent",p),o.acquireTokenSilent({scopes:i,account:u,correlationId:t}).then(function(o){if(n.IsUpnMatch=w(o.idToken),n.IsUpnMatch)return A(n,t,e,r,i),a(n)}).catch(function(o){return n.ErrorCode=o.errorCode,n.ErrorMessage=o.errorMessage,l.LoggingUtils.log("acquireToken->error: "+n.ErrorMessage,p),n.IsUpnMatch=!1,A(n,t,e,r,i,o.errorCode,o.errorMessage),c(n)})})},t.GetAuthConfig=function(){return new Promise(function(e,t){new Promise(function(e,t){if(window.location.hash){var r=window.location.hash;r.indexOf("#/")>-1?r=r.substring(r.indexOf("#/")+2):r.indexOf("#")>-1&&(r=r.substring(1));for(var n=r.split("&"),o=0;o<n.length;o++){var i=n[o].split("=");if("state"==decodeURIComponent(i[0])){var s=decodeURIComponent(i[i.length-1]).split("|");2==s.length&&e(s[1])}}}t()}).then(function(t){window.addEventListener("message",function(t){t.origin&&t.origin==location.origin&&t.data&&t.data.config&&t.data.request&&t.data.request==u.Constants.PostMessageType.ResponseAuthConfig&&e(t.data.config)},!1),parent.postMessage({request:u.Constants.PostMessageType.RequestAuthConfig,iframe:t},location.origin)},function(){t({})})})},t.Logout=function(){i.forEach(function(e){l.LoggingUtils.log("application calls logout",p),e.application.logout()}),i=new Array}},"./packages/Microsoft.Office.WebAuth.BrowserAuth/scripts/Utils/Constants.ts":function(e,t,r){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),function(e){var t=function(){function e(){}return e.Aad="aad",e.Msa="msa",e}();e.IdentityProvider=t;var r=function(){function e(){}return e.RequestAuthConfig="RequestAuthConfig",e.ResponseAuthConfig="ResponseAuthConfig",e.iFrameIdTokenPrefix="msalIdTokenFrame",e.iFramePrefix="msalRenewFrame",e}();e.PostMessageType=r;var n=function(){function e(){}return e.Prod="https://login.microsoftonline.com/",e.ProdLegacy="https://login.windows.net/",e.Ppe="https://login.windows-ppe.net/",e.AadSuffix="organizations",e.MsaSuffix="consumers",e}();e.Authority=n;var o=function(){function e(){}return e.OtelInstance="otel",e.CheckUpnTelemetryName="Office.Identity.WebAuth.BrowserAuth.CheckUpn",e.GetTokenTelemetryName="Office.Identity.WebAuth.BrowserAuth.GetToken",e.LoadTelemetryName="Office.Identity.WebAuth.BrowserAuth.Load",e.Duration="Duration",e.Succeeded="Succeeded",e.ApplicationId="ApplicationId",e.CorrelationId="CorrelationId",e.IdentityProvider="IdentityProvider",e.LoadedApplicationCount="LoadedApplicationCount",e.TokenScope="TokenScope",e.ErrorCode="ErrorCode",e.ErrorMessage="ErrorMessage",e.ErrorCodeForLoad="ErrorCodeForLoad",e.ErrorMessageForLoad="ErrorMessageForLoad",e.ErrorCodeForGetToken="ErrorCodeForGetToken",e.ErrorMessageForGetToken="ErrorMessageForGetToken",e.ErrorCodeForCheckUpn="ErrorCodeForCheckUpn",e.ErrorMessageForCheckUpn="ErrorMessageForCheckUpn",e}();e.Telemetry=o}(t.Constants||(t.Constants={}))},"./packages/Microsoft.Office.WebAuth.BrowserAuth/scripts/Utils/ExtractUtils.ts":function(e,t,r){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var n=r("./packages/Microsoft.Office.WebAuth.BrowserAuth/scripts/Utils/LoggingUtils.ts"),o=function(){function e(){}return e.extractToken=function(e){var t=this.decodeJwt(e);if(!t)return null;try{var r=t.JWSPayload,o=this.base64Decode(r);return o?JSON.parse(o):(n.LoggingUtils.log("The returned token could not be base64 url safe decoded."),null)}catch(e){n.LoggingUtils.error("The returned token could not be decoded"+e)}return null},e.base64Decode=function(e){var t=e.replace(/-/g,"+").replace(/_/g,"/");switch(t.length%4){case 0:break;case 2:t+="==";break;case 3:t+="=";break;default:throw new Error("Invalid base64 string")}return decodeURIComponent(atob(t).split("").map(function(e){return"%"+("00"+e.charCodeAt(0).toString(16)).slice(-2)}).join(""))},e.decodeJwt=function(e){if("undefined"===e||!e||0===e.length)return null;var t=/^([^\.\s]*)\.([^\.\s]+)\.([^\.\s]*)$/.exec(e);return!t||t.length<4?(n.LoggingUtils.warn("The returned access_token is not parseable."),null):{header:t[1],JWSPayload:t[2],JWSSig:t[3]}},e}();t.ExtractUtils=o},"./packages/Microsoft.Office.WebAuth.BrowserAuth/scripts/Utils/LoggingUtils.ts":function(e,t,r){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var n=function(){function e(){}return e.log=function(e,t){return!!this.shouldProceed(t)&&(console.log(e),!0)},e.warn=function(e,t){return!!this.shouldProceed(t)&&(console.warn(e),!0)},e.error=function(e,t){return!!this.shouldProceed(t)&&(console.error(e),!0)},e.shouldProceed=function(e){return!(null!=e&&void 0!==e&&!e)},e}();t.LoggingUtils=n},"./packages/Microsoft.Office.WebAuth.BrowserAuth/scripts/Utils/TimerUtils.ts":function(e,t,r){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var n=function(){function e(){}return e.timer=function(){var e=(new Date).getTime();return{get seconds(){return Math.ceil(((new Date).getTime()-e)/1e3)},get ms(){return(new Date).getTime()-e},get formattedSeconds(){return Math.ceil(this.seconds/1e3)+"s"},get formattedMs(){return this.ms+"ms"}}},e}();t.TimerUtils=n},0:function(e,t,r){r("./packages/Microsoft.Office.WebAuth.BrowserAuth/lib/api.js"),e.exports=r("./packages/Microsoft.Office.WebAuth.BrowserAuth/scripts/BrowserAuth.ts")}})});